using System;
using Newtonsoft.Json;
using System.Globalization;
using WingInterfaceLibrary.Enum;
using WingServerCommon.Log;
using WingMongoDBModule.Entities;
using System.Linq;
using System.Collections.Generic;
using System.Net;

namespace WingMongoDBModule.Common
{
    public static class ConvertHelper
    {
        /// <summary>
        /// convert object to Json string
        /// </summary>
        /// <param name="obj">object</param>
        /// <param name="formatting">format</param>
        /// <param name="settings">settings</param>
        /// <returns></returns>
        public static string ToJson(this object obj, Formatting formatting = Formatting.None, JsonSerializerSettings settings = null)
        {
            if (obj == null) return string.Empty;
            try
            {
                return JsonConvert.SerializeObject(obj, formatting, settings);
            }
            catch { return string.Empty; }
        }

        /// <summary>
        /// convert Json string to object
        /// </summary>
        /// <typeparam name="T">object type</typeparam>
        /// <param name="jsonStr">Json string</param>
        /// <param name="settings">settings</param>
        /// <returns></returns>
        public static T JsonToObj<T>(this string jsonStr, JsonSerializerSettings settings = null)
        {
            if (string.IsNullOrWhiteSpace(jsonStr)) return default(T);
            try
            {
                return JsonConvert.DeserializeObject<T>(jsonStr, settings);
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
                return default(T);
            }
        }

        /// <summary>
        /// convert Json string to object
        /// </summary>
        /// <param name="jsonStr">Json string</param>
        /// <param name="type">object type</param>
        /// <param name="settings">settings</param>
        /// <returns></returns>
        public static object JsonToObj(this string jsonStr, Type type, JsonSerializerSettings settings = null)
        {
            if (string.IsNullOrWhiteSpace(jsonStr)) return type.IsValueType ? Activator.CreateInstance(type) : null;
            try
            {
                return JsonConvert.DeserializeObject(jsonStr, type, settings);
            }
            catch { return type.IsValueType ? Activator.CreateInstance(type) : null; }
        }
        /// <summary>
        /// convert sting to int
        /// </summary>
        /// <param name="str">string</param>
        /// <param name="defaultValue">default Value</param>
        /// <returns></returns>
        public static int ToInt(this string str, int defaultValue = 0)
        {
            if (string.IsNullOrWhiteSpace(str)) return defaultValue;
            int.TryParse(str, out defaultValue);
            return defaultValue;
        }

        public static double ToDouble(this string str, double defaultValue = 0)
        {
            if (string.IsNullOrWhiteSpace(str)) return defaultValue;
            double.TryParse(str, out defaultValue);
            return defaultValue;
        }

        /// <summary>
        /// convert sting to float
        /// </summary>
        /// <param name="str"></param>
        /// <param name="defaultValue"></param>
        /// <returns></returns>
        public static float ToFloat(this string str, float defaultValue = 0)
        {
            if (string.IsNullOrWhiteSpace(str)) return defaultValue;
            float.TryParse(str, out defaultValue);
            return defaultValue;
        }

        /// <summary>
        /// convert sting to DateTime
        /// </summary>
        /// <param name="str"></param>
        /// <param name="defaultValue"></param>
        /// <returns></returns>
        public static DateTime ToDateTime(this string str)
        {
            DateTime defaultValue;
            if (string.IsNullOrWhiteSpace(str)) return DateTime.MinValue;
            DateTime.TryParse(str, out defaultValue);
            return defaultValue;
        }

        /// <summary>
        /// convert sting to int
        /// </summary>
        /// <param name="str">string</param>
        /// <param name="defaultValue">default Value</param>
        /// <returns></returns>
        public static long ToLong(this string str, long defaultValue = 0)
        {
            if (string.IsNullOrWhiteSpace(str)) return defaultValue;
            long.TryParse(str, out defaultValue);
            return defaultValue;
        }

        /// <summary>
        /// Convert string to other type object
        /// </summary>
        /// <param name="str">string</param>
        /// <param name="type">type</param>
        /// <returns></returns>
        public static object ToObject(this string str, Type type)
        {
            try
            {
                return Convert.ChangeType(str, type, CultureInfo.InvariantCulture);
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
                return type.IsValueType ? Activator.CreateInstance(type) : null;
            }
        }

        /// <summary>
        /// convert sting to Boolean
        /// </summary>
        /// <param name="str">sting</param>
        /// <returns></returns>
        public static bool ToBool(this string str)
        {
            if (string.IsNullOrWhiteSpace(str)) return false;
            bool.TryParse(str, out bool val);
            return val;
        }

        /// <summary>
        /// 验证是否为空
        /// </summary>
        /// <param name="str">sting</param>
        /// <returns>true</returns>
        public static bool IsEmpty(this string str)
        {
            if (string.IsNullOrWhiteSpace(str))
            {
                return true;
            }
            return false;
        }

        /// <summary>
        /// 时间转长整形的时间戳
        /// </summary>
        /// <param name="time">当前时间</param>
        /// <param name="isUtc">是否utc格式时间戳</param>
        /// <returns>true</returns>
        public static long ToTimestamp(this DateTime time, bool isUtc = false)
        {
            DateTime refTime = TimeZoneInfo.ConvertTimeFromUtc(new DateTime(1970, 1, 1), TimeZoneInfo.Utc);
            var diffSpan = (isUtc ? time : time.ToUniversalTime()) - refTime;
            long unixTime = (long)Math.Round(diffSpan.TotalMilliseconds, MidpointRounding.AwayFromZero);
            return unixTime;
        }

        /// <summary>
        /// url转token
        /// </summary>
        /// <param name="url">url地址</param>
        /// <returns>true</returns>
        public static string ToUrlToken(this string url)
        {
            string fileToken = "";
            if (string.IsNullOrWhiteSpace(url))
            {
                return fileToken;
            }
            var fileName = "";
            if (url.Contains("http://") || url.Contains("https://"))
            {
                fileToken = TransferUrlToToken(url);
                return fileToken;
            }
            else
            {
                fileName = url;
            }
            //处理 传的是token 的情况
            if (string.IsNullOrEmpty(fileName))
            {
                return fileToken;
            }
            if (fileName.Split('.').Length > 1)
            {
                int lastIndex = fileName.LastIndexOf('.');
                var fileExtend = fileName.Substring(lastIndex + 1);
                try
                {
                    var fileTypeEnum = (WingInterfaceLibrary.Enum.UploadFileTypeEnum)Enum.Parse(typeof(WingInterfaceLibrary.Enum.UploadFileTypeEnum), fileExtend.ToUpper());
                    var fileType = Enum.GetName(typeof(WingInterfaceLibrary.Enum.UploadFileTypeEnum), fileTypeEnum.GetHashCode());
                    if (!string.IsNullOrEmpty(fileType))
                    {
                        fileName = fileName.Substring(0, lastIndex);
                    }
                    else
                    {
                        fileName = "";
                    }
                }
                catch (Exception ex)
                {
                    Logger.WriteLineError($"ToUrlToken error {ex}");
                    fileName = "";
                }
            }
            if (fileName.Split('_').Length > 1)
            {
                fileToken = fileName;
            }
            return fileToken;
        }

        /// <summary>
        /// 本地动态地址，由于局域网重启，可能会更换地址
        /// </summary>
        /// <param name="url">url地址</param>
        /// <returns>true</returns>
        public static string ToLocalUrl(this string url)
        {
            if (string.IsNullOrEmpty(url))
            {
                return string.Empty;
            }
            var storageServer = WingServerCommon.Config.ConfigurationManager.GetParammeter<WingServerCommon.Config.Parameters.StringParameter>("Storage", "StorageServer").Value;
            if (storageServer == "VCS")
            {
                //http://192.168.6.103:7315/FlyinsonoStorage/20240801/9E466E9EF4B0434EADF724AEE3541A8D.jpg
                var newUrl = url.Replace("http://", "").Replace("https://", "");
                var urlArray = newUrl.Split('/');
                if (urlArray.Length >= 4)
                {
                    var fileName = GetServerHostUrlAsync() +  urlArray[1] + "/" + urlArray[2] + "/" + urlArray[3];
                    return fileName;
                }
                else
                {
                    return url;
                }
            }
            else
            {
                return url;
            }
        }

        /// <summary>
        /// 获取服务器host地址
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        private static string GetServerHostUrlAsync()
        {
            //"http://*:9303/",
            string _host = WingServerCommon.Config.EnvironmentConfigManager.GetParammeter<WingServerCommon.Config.Parameters.StringParameter>("Storage", "ServerHost").Value;
            var result = "";
            if (!string.IsNullOrEmpty(_host))
            {
                result = _host;
            }
            if (result.Contains('*'))
            {
                string hostName = Dns.GetHostName(); // 获取本机主机名
                IPAddress[] addresses = Dns.GetHostAddresses(hostName); // 获取本机IP地址列表
                foreach (IPAddress address in addresses)
                {
                    if (address.AddressFamily == System.Net.Sockets.AddressFamily.InterNetwork) // 检查是否为IPv4地址
                    {
                        result = result.Replace("*", address.ToString());
                        break; // 找到IPv4地址后退出循环
                    }
                }
                if (string.IsNullOrEmpty(result))
                {
                    foreach (IPAddress address in addresses)
                    {
                        if (address.AddressFamily == System.Net.Sockets.AddressFamily.InterNetworkV6) // 检查是否为IPv4地址
                        {
                            result = result.Replace("*", address.ToString());
                            break; // 找到IPv4地址后退出循环
                        }
                    }
                }
            }
            return result;
        }

        /// <summary>
        /// url转token
        /// </summary>
        /// <param name="url">url地址</param>
        /// <returns>true</returns>
        private static string TransferUrlToToken(string url)
        {
            //需要做处理
            var newUrl = url.Replace("http://", "").Replace("https://", "");
            var urlArray = newUrl.Split('/');
            var storageServer = WingServerCommon.Config.ConfigurationManager.GetParammeter<WingServerCommon.Config.Parameters.StringParameter>("Storage", "StorageServer").Value;
            if (storageServer == "COS")
            {
                if (urlArray.Length >= 2)
                {
                    var storageSettingsParam = WingServerCommon.Config.ConfigurationManager.StorageSettings;
                    var fileName = storageSettingsParam.CDNServerUrl + "/" + urlArray[1];
                    return fileName;
                }
                else
                {
                    return url;
                }
            }
            else
            {
                return url;
            }
        }
        public static bool CanApply(this DevicePatchEntity patch, string terminalOsVersion, string terminalAppVersion)
        {
            var splitString = "; ";
            var osVersions = patch.OsVersion.Split(new[] { splitString }, StringSplitOptions.RemoveEmptyEntries);
            var osPassed = osVersions.Length == 0 ||
                           terminalOsVersion != null && osVersions.Contains(terminalOsVersion);
            if (!osPassed)
            {
                return false;
            }
            if (string.IsNullOrEmpty(terminalAppVersion))
            {
                return string.IsNullOrEmpty(patch.SoftwareVersion);
            }
            var revisionAndBuilds = terminalAppVersion.Split('|');
            var terminalAppRevision = revisionAndBuilds[0].Split('(')[0];
            var patchAppVersions = patch.SoftwareVersion.Split(new[] { splitString }, StringSplitOptions.RemoveEmptyEntries);
            if (patchAppVersions.Length == 0)
            {
                return true;
            }

            if (patchAppVersions.Contains(terminalAppRevision))
            {
                return true;
            }
            return false;
        }

        
        /// <summary>
        /// 获取当前周的第一天
        /// </summary>
        public static DateTime GetCurrentWeek(this DateTime today)
        {
            int dayOfWeek = (int)today.DayOfWeek;
            // 周日是一周的第一天，所以需要减去dayOfWeek - 1
            if (dayOfWeek == 0)
            {
                DateTime startOfWeek = today.AddDays(-6);
                return startOfWeek;
            }
            else
            {
                DateTime startOfWeek = today.AddDays(-dayOfWeek + 1);
                return startOfWeek;
            }
        }

        /// <summary>
        /// 获取当前年的第一天
        /// </summary>
        public static DateTime GetCurrentYear(this DateTime today)
        {
            DateTime firstDayOfYear = new DateTime(today.Year, 1, 1);
            return firstDayOfYear;
        }

        /// <summary>
        /// 获取当前月的第一天和最后一天
        /// </summary>
        public static Tuple<DateTime, DateTime> GetCurrentMonth(this DateTime today)
        {
            int year = today.Date.Year;
            int month = today.Date.Month;
            var firtDayOfMonth = new DateTime(year, month, 1, 0, 0, 0);
            var endDayOfMonth = firtDayOfMonth.AddMonths(1).AddSeconds(-1);
            return Tuple.Create(firtDayOfMonth, endDayOfMonth);
        }

        /// <summary>
        /// 获取当前月的所有自然周和日期
        /// </summary>
        public static Dictionary<int, List<DateTime>> GetCurrentMonthAllWeek(this DateTime today)
        {
            int weekCount = 0;
            DateTime firstDayOfMonth = new DateTime(today.Year, today.Month, 1);
            DateTime lastDayOfMonth = firstDayOfMonth.AddMonths(1).AddDays(-1);

            var diff = 0;
            switch (firstDayOfMonth.DayOfWeek)
            {
                case DayOfWeek.Sunday:
                    diff = 1;
                    break;
                case DayOfWeek.Monday:
                    diff = 0;
                    break;
                case DayOfWeek.Tuesday:
                    diff = 6;
                    break;
                case DayOfWeek.Wednesday:
                    diff = 5;
                    break;
                case DayOfWeek.Thursday:
                    diff = 4;
                    break;
                case DayOfWeek.Friday:
                    diff = 3;
                    break;
                case DayOfWeek.Saturday:
                    diff = 2;
                    break;
                default:
                    break;
            }

            // 计算中间自然周
            for (DateTime day = firstDayOfMonth.AddDays(diff);
                day <= lastDayOfMonth;
                day = day.AddDays(7))
            {
                var nextDay = day.AddDays(6);
                if (nextDay.Month == firstDayOfMonth.Month)
                {
                    weekCount++;
                }
            }
            if ((int)firstDayOfMonth.DayOfWeek != 1)
            {
                weekCount++;
            }
            if ((int)lastDayOfMonth.DayOfWeek != 0)
            {
                weekCount++;
            }
            var dic = new Dictionary<int, List<DateTime>>();
            for (var i = 1; i <= weekCount; i++)
            {
                var tempList = new List<DateTime>();
                for (var j = 0; j < 7; j++)
                {
                    if (i == 1 && j == 0)
                    {
                        //不处理
                    }
                    else
                    {
                        firstDayOfMonth = firstDayOfMonth.AddDays(1);
                    }
                    tempList.Add(firstDayOfMonth);
                    if (((int)firstDayOfMonth.DayOfWeek) == 0 || firstDayOfMonth.Day == lastDayOfMonth.Day)
                    {
                        break;
                    }
                }
                dic.Add(i, tempList);
            }
            return dic;
        }

        /// <summary>
        /// 获取当前年的自然月和日期
        /// </summary>
        public static Dictionary<int, List<DateTime>> GetCurrentYearAllMonth(this DateTime today)
        {
            var dic = new Dictionary<int, List<DateTime>>();
            int year = today.Year;
            var firstDayOfYear = new DateTime(year, 1, 1, 0, 0, 0);
            for (var i = 1; i <= 12; i++)
            {
                var tempList = new List<DateTime>();
                for (var j = 0; j < 32; j++)
                {
                    if (j != 0)
                    {
                        firstDayOfYear = firstDayOfYear.AddDays(1);
                    }
                    if (firstDayOfYear.Month == i)
                    {
                        tempList.Add(firstDayOfYear);
                    }
                    else
                    {
                        break;
                    }
                }
                dic.Add(i, tempList);
            }
            return dic;
        }
   
        /// <summary>
        /// utc转bj时间
        /// </summary>
        public static DateTime ToUTCToBeijing(this DateTime utcTime)
        {
            // 获取中国标准时间 (CST) 的时区信息
            TimeZoneInfo cstZone = TimeZoneInfo.FindSystemTimeZoneById("China Standard Time");
            // 将UTC时间转换为北京时间
            DateTime beijingTime = TimeZoneInfo.ConvertTimeFromUtc(utcTime, cstZone);
            return beijingTime;
        }
    }
}
