using System;
using Newtonsoft.Json;
using WingInterfaceLibrary.Enum;
using WingServerCommon.Log;
using WingMongoDBModule.Entities;
using System.Linq;
using System.Collections.Generic;
using System.Collections.Concurrent;
using System.Net.Http;
using System.Text;
using System.Dynamic;
using System.IO;
using System.Runtime.Serialization.Json;
using System.Threading.Tasks;

namespace WingMongoDBModule.Common
{
    public class ProjectPosterManager
    {
        private ConcurrentDictionary<ProjectPushTypeEnum, ProjectPoster> _posters = new ConcurrentDictionary<ProjectPushTypeEnum, ProjectPoster>();
        protected Func<ProjectPushRecordEntity, string, ConcurrentDictionary<string, object>> OnGetPushDatas;
        protected Func<string, ProjectPushStatusEnum, string, string, bool> OnSavePushResult;

        public ProjectPosterManager(Func<ProjectPushRecordEntity, string, ConcurrentDictionary<string, object>> funGetPushDatas
            , Func<string, ProjectPushStatusEnum, string, string, bool> funcSavePushResult)
        {
            OnGetPushDatas = funGetPushDatas;
            OnSavePushResult = funcSavePushResult;
        }

        public bool Post(ProjectPushRecordEntity pushRecord)
        {
            var pushType = pushRecord.PushType;
            var projectPoster = _posters.GetOrAdd(pushType, s =>
            {
                if (pushType == ProjectPushTypeEnum.YinChuan)
                {
                    return new YinChuanProjectPoster(OnGetPushDatas, OnSavePushResult);
                }
                else
                {
                    return new ProjectPoster(OnGetPushDatas, OnSavePushResult);
                }
            });
            return projectPoster.Post(pushRecord);
        }
    }

    public class ProjectPoster
    {
        protected static HttpClient _httpClient;
        protected static HttpClient _httpDownloadClient;
        private string _tempFolder = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "ProjectPosterTemp");
        private DateTime _lastClearDate;
        protected Func<ProjectPushRecordEntity, string, ConcurrentDictionary<string, object>> OnGetPushDatas;
        protected Func<string, ProjectPushStatusEnum, string, string, bool> OnSavePushResult;

        public ProjectPoster(Func<ProjectPushRecordEntity, string, ConcurrentDictionary<string, object>> funGetPushDatas
            , Func<string, ProjectPushStatusEnum, string, string, bool> funcSavePushResult)
        {
            OnGetPushDatas = funGetPushDatas;
            OnSavePushResult = funcSavePushResult;
            if (!Directory.Exists(_tempFolder))
            {
                Directory.CreateDirectory(_tempFolder);
            }
            DeleteExpiredFiles();
            _httpClient = new HttpClient { Timeout = TimeSpan.FromSeconds(30) };
            _httpDownloadClient = new HttpClient { Timeout = TimeSpan.FromMinutes(10) };
        }

        public virtual bool Post(ProjectPushRecordEntity pushRecord)
        {
            var projectName = pushRecord.ProjectName;
            var timeoutSeconds = pushRecord.TimeoutSeconds;
            Logger.WriteLineInfo($"ProjectPoster {pushRecord.DataType} Post start, projectName:{projectName}, type:{pushRecord.DataType}, dataCode:{pushRecord.DataCode}, name:{pushRecord.DataName}, apiUrl:{pushRecord.ApiUrl}");

            var reqContent = string.Empty;
            try
            {
                var pushDatas = GetPushDatas(pushRecord);
                var request = new HttpRequestMessage(HttpMethod.Post, pushRecord.ApiUrl);

                reqContent = GetPostContent(pushDatas);
                request.Content = new StringContent(reqContent, Encoding.UTF8, "application/json");

                var headerDatas = GetHeaderDatas(pushRecord.PushTemplate);
                foreach (var item in headerDatas)
                {
                    request.Headers.Add(item.Key, item.Value.ToString());
                }

                if (_httpClient == null || _httpClient.Timeout < TimeSpan.FromSeconds(timeoutSeconds))
                {
                    _httpClient = new HttpClient();
                    _httpClient.Timeout = TimeSpan.FromSeconds(timeoutSeconds);
                }

                var postResult = _httpClient.SendAsync(request);
                postResult.Wait();
                var response = postResult.Result;
                var resultstr = response.Content.ReadAsStringAsync();
                resultstr.Wait();
                var resultContent = resultstr.Result;

                if (response.IsSuccessStatusCode)
                {
                    using (var ms = new MemoryStream(Encoding.UTF8.GetBytes(resultContent)))
                    {
                        try
                        {
                            var posterResult = (ProjectPostResult)new DataContractJsonSerializer(typeof(ProjectPostResult)).ReadObject(ms);
                            if (posterResult.code == "0")
                            {
                                OnSavePushResult?.Invoke(pushRecord.Code, ProjectPushStatusEnum.Success, reqContent, resultContent);
                                Logger.WriteLineInfo($"ProjectPoster {pushRecord.DataType} Post success, projectName:{projectName}, type:{pushRecord.DataType}, dataCode:{pushRecord.DataCode}, name:{pushRecord.DataName}");
                                return true;
                            }
                        }
                        catch (Exception)
                        {
                        }
                    }
                }
                OnSavePushResult?.Invoke(pushRecord.Code, ProjectPushStatusEnum.Failed, reqContent, resultContent);
                Logger.WriteLineWarn($"ProjectPoster {pushRecord.DataType} Post failed, projectName:{projectName}, type:{pushRecord.DataType}, dataCode:{pushRecord.DataCode}, name:{pushRecord.DataName}");
            }
            catch (Exception ex)
            {
                OnSavePushResult?.Invoke(pushRecord.Code, ProjectPushStatusEnum.Failed, reqContent, ex.Message);
                Logger.WriteLineWarn($"ProjectPoster {pushRecord.DataType} Post error, projectName:{projectName}, type:{pushRecord.DataType}, dataCode:{pushRecord.DataCode}, name:{pushRecord.DataName}, ex:{ex}");
            }
            return false;
        }

        protected ConcurrentDictionary<string, object> GetPushDatas(ProjectPushRecordEntity pushRecord)
        {
            var pushDatas = new ConcurrentDictionary<string, object>();

            var templateDefaultItems = GetDefaultDatas(pushRecord.PushTemplate);
            foreach (var item in templateDefaultItems)
            {
                pushDatas.AddOrUpdate(item.Key, item.Value, (k, v) => { return item.Value; });
            }
            var language = GetTemplateSetting(pushRecord.PushTemplate, "Language", "en-US");
            var formatterTime = GetTemplateSetting(pushRecord.PushTemplate, "FormatterTime", "yyyy-MM-dd HH:mm:ss");
            var addHours = GetTemplateSetting(pushRecord.PushTemplate, "AddHours", "0");
            var templatePushItems = GetTemplateItems(pushRecord.PushTemplate).Where(x => !x.VinnoField.Contains("Value:"))?.ToList() ?? new List<TemplateItem>();
            if (templatePushItems?.Any() ?? false)
            {
                var realDatas = OnGetPushDatas?.Invoke(pushRecord, language);
                foreach (var item in templatePushItems)
                {
                    if (realDatas.ContainsKey(item.VinnoField))
                    {
                        if (realDatas[item.VinnoField] is DateTime time)
                        {
                            if (!string.IsNullOrWhiteSpace(addHours))
                            {
                                if (int.TryParse(addHours, out int intHours) && intHours != 0)
                                {
                                    time = time.AddHours(intHours);
                                }
                            }
                            pushDatas.AddOrUpdate(item.TemplateField, time.ToString(formatterTime), (k, v) => { return time.ToString(formatterTime); });
                        }
                        else
                        {
                            pushDatas.AddOrUpdate(item.TemplateField, realDatas[item.VinnoField], (k, v) => { return realDatas[item.VinnoField]; });
                        }
                    }
                    else
                    {
                        pushDatas.AddOrUpdate(item.TemplateField, string.Empty, (k, v) => { return string.Empty; });
                    }
                }
            }
            return pushDatas;
        }

        protected string GetPostContent(ConcurrentDictionary<string, object> pushDatas)
        {
            IDictionary<string, object> expandoObjects = new ExpandoObject();
            foreach (var d in pushDatas)
            {
                expandoObjects.Add(d);
            }
            var settings = new JsonSerializerSettings
            {
                DateTimeZoneHandling = DateTimeZoneHandling.Utc,
                DateFormatString = "yyyy-MM-ddTHH:mm:ssZ"
            };
            return JsonConvert.SerializeObject(expandoObjects as ExpandoObject, settings);
        }

        protected List<TemplateItem> GetTemplateItems(string template)
        {
            try
            {
                var templateArray = template?.Split("\n")?.Where(x => x.Contains("="))?.ToList() ?? new List<string>();
                var templateItems = templateArray.Select(line =>
                {
                    var values = line.Split('=');
                    return new TemplateItem
                    {
                        TemplateField = values[0].Trim(),
                        VinnoField = values[1].Trim(),
                    };
                }).ToList();
                return templateItems;
            }
            catch (Exception ex)
            {
                return new List<TemplateItem>();
            }
        }

        protected virtual ConcurrentDictionary<string, object> GetDefaultDatas(string template)
        {
            var defasultDatas = new ConcurrentDictionary<string, object>();

            var templateItems = GetTemplateItems(template);
            var templateDefaultItems = templateItems.Where(x => x.VinnoField.Contains("DefaultValue"))?.Select(x => { x.VinnoField = x.VinnoField.Replace("DefaultValue:", ""); return x; })?.ToList() ?? new List<TemplateItem>();
            foreach (var item in templateDefaultItems)
            {
                defasultDatas.AddOrUpdate(item.TemplateField, item.VinnoField, (k, v) => { return item.VinnoField; });
            }
            return defasultDatas;
        }

        protected virtual ConcurrentDictionary<string, object> GetHeaderDatas(string template)
        {
            var hearderDatas = new ConcurrentDictionary<string, object>();

            var templateItems = GetTemplateItems(template);
            var templateHeaderItems = templateItems.Where(x => x.VinnoField.Contains("HeaderValue"))?.Select(x => { x.VinnoField = x.VinnoField.Replace("HeaderValue:", ""); return x; })?.ToList() ?? new List<TemplateItem>();
            foreach (var item in templateHeaderItems)
            {
                hearderDatas.AddOrUpdate(item.TemplateField, item.VinnoField, (k, v) => { return item.VinnoField; });
            }
            return hearderDatas;
        }

        protected virtual ConcurrentDictionary<string, object> GetSettingDatas(string template)
        {
            var settingDatas = new ConcurrentDictionary<string, object>();

            var templateItems = GetTemplateItems(template);
            var templateSettingItems = templateItems.Where(x => x.VinnoField.Contains("SettingValue"))?.Select(x => { x.VinnoField = x.VinnoField.Replace("SettingValue:", ""); return x; })?.ToList() ?? new List<TemplateItem>();
            foreach (var item in templateSettingItems)
            {
                settingDatas.AddOrUpdate(item.TemplateField, item.VinnoField, (k, v) => { return item.VinnoField; });
            }
            return settingDatas;
        }

        protected string GetTemplateSetting(string template, string key, string defaultValue)
        {
            var templateSettingItems = GetSettingDatas(template);
            if (templateSettingItems.ContainsKey(key))
            {
                return templateSettingItems[key].ToString() ?? string.Empty;
            }
            return defaultValue;
        }

        /// <summary>
        /// 下载文件
        /// </summary>
        /// <param name="url"></param>
        /// <returns></returns>
        protected async Task<byte[]> DownloadAsync(string url, string fileName)
        {
            try
            {
                DeleteExpiredFiles();
                var filePath = Path.Combine(_tempFolder, fileName);
                if (!File.Exists(filePath))
                {
                    HttpResponseMessage response = await _httpDownloadClient.GetAsync(url);
                    await using var fs = new FileStream(filePath, FileMode.Create, FileAccess.Write);
                    await response.Content.CopyToAsync(fs);
                }
                if (File.Exists(filePath))
                {
                    return File.ReadAllBytes(filePath);
                }
            }
            catch (Exception)
            {
            }
            return Array.Empty<byte>();
        }

        private void DeleteExpiredFiles()
        {
            if (_lastClearDate < DateTime.UtcNow.Date)
            {
                _lastClearDate = DateTime.UtcNow.Date;
                Task.Run(() =>
                {
                    //删除过期文件
                    DeleteFilesByMinTime(_tempFolder, DateTime.UtcNow.Date.AddDays(-1));
                });
            }
        }

        public void DeleteFilesByMinTime(string folderPath, DateTime minTime)
        {
            if (Directory.Exists(folderPath))
            {
                // 获取当前文件夹下的所有文件
                string[] filePaths = Directory.GetFiles(folderPath);

                foreach (string filePath in filePaths)
                {
                    FileInfo fileInfo = new FileInfo(filePath);

                    // 如果文件的创建时间早于今天，则删除该文件
                    if (fileInfo.CreationTime < minTime)
                    {
                        try
                        {
                            File.Delete(filePath);
                        }
                        catch (Exception ex)
                        {
                            //Console.WriteLine($"Failed to delete file '{filePath}': {ex.Message}");
                        }
                    }
                }

                // 递归处理子文件夹
                string[] subDirectories = Directory.GetDirectories(folderPath);
                foreach (string subDirectory in subDirectories)
                {
                    DeleteFilesByMinTime(subDirectory, minTime);

                    // 检查子文件夹是否为空
                    if (Directory.GetFiles(subDirectory).Length == 0 && Directory.GetDirectories(subDirectory).Length == 0)
                    {
                        try
                        {
                            if (new DirectoryInfo(subDirectory).CreationTime < minTime)
                            {
                                Directory.Delete(subDirectory);
                            }
                        }
                        catch (Exception ex)
                        {
                            //Console.WriteLine($"Failed to delete folder '{subDirectory}': {ex.Message}");
                        }
                    }
                }
            }
            else
            {
                //Console.WriteLine($"The folder '{folderPath}' does not exist.");
            }
        }
    }

    public class YinChuanProjectPoster : ProjectPoster
    {
        private YinChuanCos _yinChuanCos;
        public YinChuanProjectPoster(Func<ProjectPushRecordEntity, string, ConcurrentDictionary<string, object>> funGetPushDatas
            , Func<string, ProjectPushStatusEnum, string, string, bool> funcSavePushResult) : base(funGetPushDatas, funcSavePushResult)
        {
            _yinChuanCos = new YinChuanCos();
        }

        public override bool Post(ProjectPushRecordEntity pushRecord)
        {
            var projectName = pushRecord.ProjectName;
            var timeoutSeconds = pushRecord.TimeoutSeconds;
            Logger.WriteLineInfo($"YinChuanProjectPoster {pushRecord.DataType} Post start, projectName:{projectName}, type:{pushRecord.DataType}, dataCode:{pushRecord.DataCode}, name:{pushRecord.DataName}, apiUrl:{pushRecord.ApiUrl}");

            var reqContent = string.Empty;
            try
            {
                var pushDatas = GetPushDatas(pushRecord);
                if (pushRecord.DataType == ProjectDataTypeEnum.Remedical)
                {
                    if (pushDatas.ContainsKey("fileaddress"))
                    {
                        var fileToken = pushDatas["fileaddress"]?.ToString();
                        if (!string.IsNullOrWhiteSpace(fileToken))
                        {
                            pushDatas["fileaddress"] = UploadFile(fileToken);
                        }
                    }
                }
                if (pushRecord.DataType == ProjectDataTypeEnum.Consultation)
                {
                    if (pushDatas.ContainsKey("images"))
                    {
                        var sb = new StringBuilder();
                        sb.Append("{");
                        if (pushDatas["images"] != null && pushDatas["images"] is List<string> imageUrls && imageUrls.Any())
                        {
                            var imageInfos = imageUrls.Select(x => UploadFile(x));
                            sb.Append(string.Join(",", imageInfos));
                        }
                        sb.Append("}");
                        pushDatas["images"] = sb.ToString();
                    }
                    else if (pushDatas.ContainsKey("videos"))
                    {
                        var sb = new StringBuilder();
                        sb.Append("{");
                        if (pushDatas["videos"] != null && pushDatas["videos"] is List<string> videoUrls && videoUrls.Any())
                        {
                            var videoInfos = videoUrls.Select(x => UploadFile(x));
                            sb.Append(string.Join(",", videoInfos));
                        }
                        sb.Append("}");
                        pushDatas["videos"] = sb.ToString();
                    }
                }
                if (pushRecord.DataType == ProjectDataTypeEnum.Report)
                {
                    if (pushDatas.ContainsKey("images"))
                    {
                        var sb = new StringBuilder();
                        sb.Append("{");
                        if (pushDatas["images"] != null && pushDatas["images"] is List<string> imageUrls && imageUrls.Any())
                        {
                            var imageInfos = imageUrls.Select(x => ConvertToBase64(x));
                            sb.Append(string.Join(",", imageInfos));
                        }
                        sb.Append("}");
                        pushDatas["images"] = sb.ToString();
                    }
                }
                var request = new HttpRequestMessage(HttpMethod.Post, pushRecord.ApiUrl);

                reqContent = GetPostContent(pushDatas);
                request.Content = new StringContent(reqContent, Encoding.UTF8, "application/json");

                var headerDatas = GetHeaderDatas(pushRecord.PushTemplate);
                foreach (var item in headerDatas)
                {
                    request.Headers.Add(item.Key, item.Value.ToString());
                }

                var buffer = new StringBuilder();
                int i = 0;
                var sortKeys = pushDatas.Keys.ToList();
                sortKeys.Sort();
                foreach (string key in sortKeys)
                {
                    if (i > 0)
                    {
                        buffer.AppendFormat("&{0}={1}", key, pushDatas[key]);
                    }
                    else
                    {
                        buffer.AppendFormat("{0}={1}", key, pushDatas[key]);
                    }
                    i++;
                }
                var signatureKey = GetTemplateSetting(pushRecord.PushTemplate, "SignatureKey", "");
                var buffersign = buffer + "&" + signatureKey;
                var md5 = System.Security.Cryptography.MD5.Create();
                var md5Str = BitConverter.ToString(md5.ComputeHash(Encoding.UTF8.GetBytes(buffersign))).Replace("-", "").ToLower();
                request.Headers.Add("Authorization", md5Str);

                if (_httpClient == null || _httpClient.Timeout < TimeSpan.FromSeconds(timeoutSeconds))
                {
                    _httpClient = new HttpClient();
                    _httpClient.Timeout = TimeSpan.FromSeconds(timeoutSeconds);
                }

                var postResult = _httpClient.SendAsync(request);
                postResult.Wait();
                var response = postResult.Result;
                var resultstr = response.Content.ReadAsStringAsync();
                resultstr.Wait();
                var resultContent = resultstr.Result;

                if (response.IsSuccessStatusCode)
                {
                    OnSavePushResult?.Invoke(pushRecord.Code, ProjectPushStatusEnum.Success, reqContent, resultContent);
                    Logger.WriteLineInfo($"YinChuanProjectPoster {pushRecord.DataType} Post success, projectName:{projectName}, type:{pushRecord.DataType}, dataCode:{pushRecord.DataCode}, name:{pushRecord.DataName}");
                    return true;
                }
                OnSavePushResult?.Invoke(pushRecord.Code, ProjectPushStatusEnum.Failed, reqContent, resultContent);
                Logger.WriteLineWarn($"YinChuanProjectPoster {pushRecord.DataType} Post failed, projectName:{projectName}, type:{pushRecord.DataType}, dataCode:{pushRecord.DataCode}, name:{pushRecord.DataName}");
            }
            catch (Exception ex)
            {
                OnSavePushResult?.Invoke(pushRecord.Code, ProjectPushStatusEnum.Failed, reqContent, ex.Message);
                Logger.WriteLineWarn($"YinChuanProjectPoster {pushRecord.DataType} Post error, projectName:{projectName}, type:{pushRecord.DataType}, dataCode:{pushRecord.DataCode}, name:{pushRecord.DataName}, ex:{ex}");
            }
            return false;
        }

        protected override ConcurrentDictionary<string, object> GetHeaderDatas(string template)
        {
            var hearderDatas = new ConcurrentDictionary<string, object>();

            var templateItems = GetTemplateItems(template);
            var templateHeaderItems = templateItems.Where(x => x.VinnoField.Contains("HeaderValue"))?.Select(x => { x.VinnoField = x.VinnoField.Replace("HeaderValue:", ""); return x; })?.ToList() ?? new List<TemplateItem>();
            foreach (var item in templateHeaderItems)
            {
                hearderDatas.AddOrUpdate(item.TemplateField, item.VinnoField, (k, v) => { return item.VinnoField; });
            }

            if (!hearderDatas.ContainsKey("App-ID"))
            {
                hearderDatas.AddOrUpdate("App-ID", "60fa745e2ea22436c6d430e8", (k, v) => { return "60fa745e2ea22436c6d430e8"; });
            }
            return hearderDatas;
        }

        protected override ConcurrentDictionary<string, object> GetSettingDatas(string template)
        {
            var settingDatas = new ConcurrentDictionary<string, object>();

            var templateItems = GetTemplateItems(template);
            var templateSettingItems = templateItems.Where(x => x.VinnoField.Contains("SettingValue"))?.Select(x => { x.VinnoField = x.VinnoField.Replace("SettingValue:", ""); return x; })?.ToList() ?? new List<TemplateItem>();
            foreach (var item in templateSettingItems)
            {
                settingDatas.AddOrUpdate(item.TemplateField, item.VinnoField, (k, v) => { return item.VinnoField; });
            }

            if (!settingDatas.ContainsKey("SignatureKey"))
            {
                settingDatas.AddOrUpdate("SignatureKey", "8599e9fdfde59fd20e3f1a6381fe6ce2", (k, v) => { return "8599e9fdfde59fd20e3f1a6381fe6ce2"; });
            }
            if (!settingDatas.ContainsKey("Language"))
            {
                settingDatas.AddOrUpdate("Language", "zh-CN", (k, v) => { return "zh-CN"; });
            }
            if (!settingDatas.ContainsKey("FormatterTime"))
            {
                settingDatas.AddOrUpdate("FormatterTime", "yyyy-MM-dd H:mm:ss", (k, v) => { return "yyyy-MM-dd H:mm:ss"; });
            }
            if (!settingDatas.ContainsKey("AddHours"))
            {
                settingDatas.AddOrUpdate("AddHours", "8", (k, v) => { return "8"; });
            }
            return settingDatas;
        }

        private string UploadFile(string fileToken)
        {
            try
            {
                var uri = new Uri(fileToken);
                var fileName = Path.GetFileName(uri.LocalPath);
                var fileBytes = DownloadAsync(fileToken, fileName).Result;
                var uploadResult = _yinChuanCos.UploadFile(fileBytes, fileName);
                Logger.WriteLineInfo($"YinChuanProjectPoster UploadFile success, upload result:{uploadResult}");
                return uploadResult;
            }
            catch (Exception ex)
            {

            }
            return string.Empty;
        }

        private string ConvertToBase64(string fileToken)
        {
            try
            {
                var uri = new Uri(fileToken);
                var fileName = Path.GetFileName(uri.LocalPath);
                var fileBytes = DownloadAsync(fileToken, fileName).Result;
                return Convert.ToBase64String(fileBytes);
            }
            catch (Exception ex)
            {

            }
            return string.Empty;
        }

    }

    public class TemplateItem
    {
        public string TemplateField { get; set; }
        public string VinnoField { get; set; }
    }

    /// <summary>
    /// PosterResult
    /// </summary>
    public class ProjectPostResult
    {
        public string code { get; set; }
        public string msg { get; set; }
    }
}
