﻿using MongoDB.Driver;
using System.Collections.Generic;
using System;
using WingServerCommon.Config;
using WingServerCommon.Config.Parameters;
using WingServerCommon.Log;
using System.Threading.Tasks;

namespace WingMongoDBModule.DBSync
{
    /// <summary>
    /// Used in master server
    /// </summary>
    class MasterOperationLogDbClient
    {
        private MongoClient _mongoClient;
        private IMongoCollection<OperationLog> _logs;
        private long _maxId = 0;
        private long _minId = 0;
        private object _writeLocker = new object();

        private static int _port = ConfigurationManager.GetParammeter<IntParameter>("Database", "Port").Value;

        private static string _server = ConfigurationManager.GetParammeter<StringParameter>("Database", "Server").Value;

        private static int _maxConn = ConfigurationManager.GetParammeter<IntParameter>("Database", "MaxConn").Value;

        private static MasterOperationLogDbClient _instance;
        //private SequenceExecutor<> //TODO refactor , not insert to database immediately

        /// <summary>
        /// The singlton instance
        /// </summary>
        public static MasterOperationLogDbClient Instance
        {
            get
            {
                return _instance ?? (_instance = new MasterOperationLogDbClient(_server, _port, _maxConn));
            }
        }

        private MasterOperationLogDbClient(string host, int port, int maxConnections)
        {
            try
            {
                var mongoSetting = new MongoClientSettings
                {
                    ConnectTimeout = TimeSpan.FromSeconds(5),
                    Server = new MongoServerAddress(host, port),
                    MaxConnectionPoolSize = maxConnections,
                    WaitQueueTimeout = TimeSpan.FromSeconds(5),
                    ServerSelectionTimeout = TimeSpan.FromSeconds(5),
                    SocketTimeout = TimeSpan.FromSeconds(5)
                };
                _mongoClient = new MongoClient(mongoSetting);
                var database = _mongoClient.GetDatabase("WingMasterOpLogDb");
                _logs = database.GetCollection<OperationLog>("OperationLogs");

                var filters = Builders<OperationLog>.Filter.Where(x => true);
                var maxRecord = _logs.Find(filters).SortByDescending(x => x.Id).Limit(1).FirstOrDefault();
                _maxId = maxRecord == null ? 0 : maxRecord.Id;
                var minRecord = _logs.Find(filters).SortBy(x => x.Id).Limit(1).FirstOrDefault();
                _minId = minRecord == null ? 0 : minRecord.Id;
            }
            catch (Exception ex)
            {
                Logger.WriteLineError("Init master op log client error");
            }
        }


        private long GetMinId()
        {
            return _minId;
        }

        private long GetMaxId()
        {
            return ++_maxId;
        }

        /// <summary>
        /// Insert one operation log
        /// </summary>
        /// <param name="operationLog"></param>
        public void Insert(OperationLog operationLog)
        {
            //TODO refactor , not insert to database immediately

            long maxId = 0;
            lock (_writeLocker)
            {
                maxId = GetMaxId();
                operationLog.Id = maxId;
                _logs.InsertOne(operationLog);
            }

            CleanLogs(maxId);
        }


        /// <summary>
        /// Clean logs when rows is full
        /// </summary>
        /// <param name="maxId"></param>
        private async void CleanLogs(long maxId)
        {
            try
            {
                if (maxId > 0)
                {
                    var minId = GetMinId();
                    if (maxId - minId > 10000)
                    {
                        var minIndex = minId + 5000;
                        await _logs.DeleteManyAsync(f => f.Id < minIndex);
                        _minId = minIndex;
                    }
                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"MasterOperationLogDBClient CleanLogs err, {ex}");
            }
        }

        /// <summary>
        /// Get many
        /// </summary>
        /// <param name="cursor"></param>
        /// <param name="sourceUrl">The slave source url</param>
        public async Task<List<OperationLog>> GetManyAsync(long cursor, string sourceUrl)
        {
            var result = await _logs.FindAsync(x => x.Id > cursor && x.SourceUrl != sourceUrl);
            return await result.ToListAsync();
        }

        /// <summary>
        /// Get many by codes
        /// </summary>
        /// <param name="cursor"></param>
        /// <param name="code">The business db code</param>
        public async Task<List<OperationLog>> GetManyByCodesAsync(IList<string> codes)
        {
            if (codes != null && codes.Count > 0)
            {
                var result = await _logs.FindAsync(x => codes.Contains(x.Code));
                return await result.ToListAsync();
            }
            return new List<OperationLog>();
        }
    }
}
