﻿using MongoDB.Bson;
using MongoDB.Driver;
using System.Collections.Generic;
using MongoDB.Bson.Serialization;
using WingInterfaceLibrary.OpLog;
using System;
using WingServerCommon.Log;

namespace WingMongoDBModule.DBSync
{
    /// <summary>
    /// 日志命令行
    /// </summary>
    internal enum MongoDBLogCommand
    {
        DBLog,
        Collection,
        Action,
        Document,
        Filter,
        Update,
        Time
    }

    /// <summary>
    /// 日志信息
    /// </summary>
    internal class MongoDBLog<TDocument> //: IMongoDBLog
    {
        /// <summary>
        /// 数据集名称
        /// </summary>
        public string CollectionName { get; internal set; }

        /// <summary>
        /// 操作类型
        /// </summary>
        public MongoDBActionTypeEnum ActionType { get; internal set; }

        /// <summary>
        /// 文档信息
        /// </summary>
        public IEnumerable<TDocument> Documents { get; internal set; }

        /// <summary>
        /// 文档信息
        /// </summary>
        public FilterDefinition<TDocument> Filter { get; internal set; }

        /// <summary>
        /// 文档信息
        /// </summary>
        public TDocument Document { get; internal set; }

        /// <summary>
        /// 更新信息
        /// </summary>
        public UpdateDefinition<TDocument> Update { get; internal set; }
    }

    /// <summary>
    /// 日志工厂
    /// </summary>
    public class MongoDBLogFactory
    {

        /// <summary>
        /// 创建插入日志
        /// </summary>
        /// <typeparam name="TDocument">文档类型</typeparam>
        /// <param name="document">文档</param>
        /// <param name="async">是否异步</param>
        /// <returns>日志</returns>
        internal static MongoDBLog<TDocument> CreateInsertOne<TDocument>(string collectionName, TDocument document, bool async = false)
        {
            try
            {
                MongoDBActionTypeEnum actionType = MongoDBActionTypeEnum.InsertOne;
                if (async) actionType = MongoDBActionTypeEnum.InsertOneAsync;
                return new MongoDBLog<TDocument>
                {
                    CollectionName = collectionName,
                    ActionType = actionType,
                    Documents = new List<TDocument> { document }
                };
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"{collectionName},CreateInsertOne{ex.ToString()}");
                throw ex;
            }
        }

        /// <summary>
        /// 创建插入日志
        /// </summary>
        /// <typeparam name="TDocument">文档类型</typeparam>
        /// <param name="documents">文档列表</param>
        /// <param name="async">是否异步</param>
        /// <returns>日志</returns>
        internal static MongoDBLog<TDocument> CreateInsertMany<TDocument>(string collectionName, IEnumerable<TDocument> documents, bool async = false)
        {
            MongoDBActionTypeEnum actionType = MongoDBActionTypeEnum.InsertMany;
            if (async) actionType = MongoDBActionTypeEnum.InsertManyAsync;
            return new MongoDBLog<TDocument>
            {
                CollectionName = collectionName,
                ActionType = actionType,
                Documents = documents
            };
        }

        /// <summary>
        /// 创建删除日志
        /// </summary>
        /// <typeparam name="TDocument">文档类型</typeparam>
        /// <param name="filter">过滤条件</param>
        /// <param name="async">是否异步</param>
        /// <returns>日志</returns>
        internal static MongoDBLog<TDocument> CreateDeleteOne<TDocument>(string collectionName, FilterDefinition<TDocument> filter, bool async = false)
        {
            MongoDBActionTypeEnum actionType = MongoDBActionTypeEnum.DeleteOne;
            if (async) actionType = MongoDBActionTypeEnum.DeleteOneAsync;
            return new MongoDBLog<TDocument>
            {
                CollectionName = collectionName,
                ActionType = actionType,
                Filter = filter
            };
        }

        /// <summary>
        /// 创建删除日志
        /// </summary>
        /// <typeparam name="TDocument">文档类型</typeparam>
        /// <param name="filter">过滤条件</param>
        /// <param name="async">是否异步</param>
        /// <returns>日志</returns>
        internal static MongoDBLog<TDocument> CreateDeleteMany<TDocument>(string collectionName, FilterDefinition<TDocument> filter, bool async = false)
        {
            MongoDBActionTypeEnum actionType = MongoDBActionTypeEnum.DeleteMany;
            if (async) actionType = MongoDBActionTypeEnum.DeleteManyAsync;
            return new MongoDBLog<TDocument>
            {
                CollectionName = collectionName,
                ActionType = actionType,
                Filter = filter
            };
        }

        /// <summary>
        /// 创建查找并删除日志
        /// </summary>
        /// <typeparam name="TDocument">文档类型</typeparam>
        /// <param name="filter">过滤条件</param>
        /// <param name="async">是否异步</param>
        /// <returns>日志</returns>
        internal static MongoDBLog<TDocument> CreateFindOneAndDelete<TDocument>(string collectionName, FilterDefinition<TDocument> filter, bool async = false)
        {
            MongoDBActionTypeEnum actionType = MongoDBActionTypeEnum.FindOneAndDelete;
            if (async) actionType = MongoDBActionTypeEnum.FindOneAndDeleteAsync;
            return new MongoDBLog<TDocument>
            {
                CollectionName = collectionName,
                ActionType = actionType,
                Filter = filter
            };
        }

        /// <summary>
        /// 创建替换日志
        /// </summary>
        /// <typeparam name="TDocument">文档类型</typeparam>
        /// <param name="filter">过滤条件</param>
        /// <param name="document">替换文档</param>
        /// <param name="async">是否异步</param>
        /// <returns>日志</returns>
        internal static MongoDBLog<TDocument> CreateReplaceOne<TDocument>(string collectionName, FilterDefinition<TDocument> filter, TDocument document, bool async = false)
        {
            MongoDBActionTypeEnum actionType = MongoDBActionTypeEnum.ReplaceOne;
            if (async) actionType = MongoDBActionTypeEnum.ReplaceOneAsync;
            return new MongoDBLog<TDocument>
            {
                CollectionName = collectionName,
                ActionType = actionType,
                Filter = filter,
                Document = document
            };
        }

        /// <summary>
        /// 创建查找并替换日志
        /// </summary>
        /// <typeparam name="TDocument">文档类型</typeparam>
        /// <param name="filter">过滤条件</param>
        /// <param name="document">替换文档</param>
        /// <param name="async">是否异步</param>
        /// <returns>日志</returns>
        internal static MongoDBLog<TDocument> CreateFindOneAndReplace<TDocument>(string collectionName, FilterDefinition<TDocument> filter, TDocument document, bool async = false)
        {
            MongoDBActionTypeEnum actionType = MongoDBActionTypeEnum.FindOneAndReplace;
            if (async) actionType = MongoDBActionTypeEnum.FindOneAndReplaceAsync;
            return new MongoDBLog<TDocument>
            {
                CollectionName = collectionName,
                ActionType = actionType,
                Filter = filter,
                Document = document
            };
        }

        /// <summary>
        /// 创建查找并更新日志
        /// </summary>
        /// <typeparam name="TDocument">文档类型</typeparam>
        /// <param name="filter">过滤条件</param>
        /// <param name="update">更新信息</param>
        /// <param name="async">是否异步</param>
        /// <returns>日志</returns>
        internal static MongoDBLog<TDocument> CreateFindOneAndUpdate<TDocument>(string collectionName, FilterDefinition<TDocument> filter, UpdateDefinition<TDocument> update, bool async = false)
        {
            MongoDBActionTypeEnum actionType = MongoDBActionTypeEnum.FindOneAndUpdate;
            if (async) actionType = MongoDBActionTypeEnum.FindOneAndUpdateAsync;
            return new MongoDBLog<TDocument>
            {
                CollectionName = collectionName,
                ActionType = actionType,
                Filter = filter,
                Update = update
            };
        }

        /// <summary>
        /// 创建查找并更新日志
        /// </summary>
        /// <typeparam name="TDocument">文档类型</typeparam>
        /// <param name="filter">过滤条件</param>
        /// <param name="update">更新信息</param>
        /// <param name="async">是否异步</param>
        /// <returns>日志</returns>
        internal static MongoDBLog<TDocument> CreateUpdateOne<TDocument>(string collectionName, FilterDefinition<TDocument> filter, UpdateDefinition<TDocument> update, bool async = false)
        {
            MongoDBActionTypeEnum actionType = MongoDBActionTypeEnum.UpdateOne;
            if (async) actionType = MongoDBActionTypeEnum.UpdateOneAsync;
            return new MongoDBLog<TDocument>
            {
                CollectionName = collectionName,
                ActionType = actionType,
                Filter = filter,
                Update = update
            };
        }

        /// <summary>
        /// 创建查找并更新日志
        /// </summary>
        /// <typeparam name="TDocument">文档类型</typeparam>
        /// <param name="filter">过滤条件</param>
        /// <param name="update">更新信息</param>
        /// <param name="async">是否异步</param>
        /// <returns>日志</returns>
        internal static MongoDBLog<TDocument> CreateUpdateMany<TDocument>(string collectionName, FilterDefinition<TDocument> filter, UpdateDefinition<TDocument> update, bool async = false)
        {
            MongoDBActionTypeEnum actionType = MongoDBActionTypeEnum.UpdateMany;
            if (async) actionType = MongoDBActionTypeEnum.UpdateManyAsync;
            return new MongoDBLog<TDocument>
            {
                CollectionName = collectionName,
                ActionType = actionType,
                Filter = filter,
                Update = update
            };
        }
    }

    public static class IMongoDatabaseExtension
    {
        /// <summary>
        /// get mongodb collection proxy object
        /// </summary>
        /// <typeparam name="TDocument">document</typeparam>
        /// <param name="database">mongodb</param>
        /// <param name="name">collection name</param>
        /// <param name="settings">mongodb collection settings</param>
        /// <returns>mongodb collection proxy object</returns>
        public static MongoCollectionProxy<TDocument> GetCollectionProxy<TDocument>(this IMongoDatabase database, string name, MongoCollectionSettings settings = null)
        {
            return new MongoCollectionProxy<TDocument>(database, name, settings);
        }
    }

    /// <summary>
    /// 更新定义扩展类
    /// </summary>
    public static class UpdateDefinitionExtension
    {

        /// <summary>
        /// UpdateDefinition convert to BsonValue
        /// </summary>
        /// <typeparam name="TDocument">document</typeparam>
        /// <param name="update">UpdateDefinition</param>
        /// <returns>BsonValue</returns>
        public static BsonValue ToBsonValue<TDocument>(this UpdateDefinition<TDocument> update)
        {
            if (update == null) return null;
            return update.Render(BsonSerializer.SerializerRegistry.GetSerializer<TDocument>(), BsonSerializer.SerializerRegistry);
        }

    }

    /// <summary>
    /// 过滤定义扩展类
    /// </summary>
    public static class FilterDefinitionExtension
    {

        /// <summary>
        /// FilterDefinition convert to BsonValue
        /// </summary>
        /// <typeparam name="TDocument">document</typeparam>
        /// <param name="filter">UpdateDefinition</param>
        /// <returns>BsonValue</returns>
        public static BsonValue ToBsonValue<TDocument>(this FilterDefinition<TDocument> filter)
        {
            if (filter == null) return null;
            return filter.Render(BsonSerializer.SerializerRegistry.GetSerializer<TDocument>(), BsonSerializer.SerializerRegistry);
        }
    }
}
