﻿using System;
using System.Net.Http;
using System.Threading;
using System.Threading.Tasks;
using WingMongoDBModule.Common;
using WingMongoDBModule.Mongodb.Driver;
using WingServerCommon.Config;
using WingServerCommon.Config.Parameters;
using WingServerCommon.Log;
using WingServerCommon.Utilities.Executors;
using WingInterfaceLibrary.OpLog;
using System.Collections.Generic;
using System.Linq;
using MongoDB.Bson;

namespace WingMongoDBModule.DBSync
{
    internal class MongoDBLogTracker
    {
        /// <summary>
        /// 日志跟踪器
        /// </summary>
        private static MongoDBLogTracker _instance;
        private Func<OperationLog, bool> _sendOplogToMaster;
        private SequenceExecutor<OperationLog> _syncOpLogToMasterSequence = new SequenceExecutor<OperationLog>("SyncOpLogToMaster");
        private static SpinThread _retrySendThread;
        private const int _maxRetryNum = 3;

        public static MongoDBLogTracker Instance
        {
            get { return _instance ?? (_instance = new MongoDBLogTracker()); }
        }

        /// <summary>
        /// Initialize the instance
        /// </summary>
        /// <param name="sendOplogToMaster"></param>
        public async void Initialize(Func<OperationLog, bool> sendOplogToMaster)
        {
            try
            {
                _sendOplogToMaster = sendOplogToMaster;
                if (ConfigurationManager.IsDistributed && !ConfigurationManager.IsMaster)
                {
                    await RetrySendOpLog(); //启动 重发oplog
                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"MongoDBLogTracker Initialize err, {ex}");
            }
        }

        /// <summary>
        /// 记录日志
        /// </summary>
        /// <param name="log">日志信息</param>
        public void Record<TDocument>(MongoDBLog<TDocument> log, bool isSimple = false, string sourceUrl = null)
        {
            var code = string.Empty;
            try
            {
                if (!ConfigurationManager.IsDistributed)
                {
                    /// do nothing for
                    return;
                }
                //将日志放入队列中  //TODO refacor just direct access log object
                //var commands = log.BuildCommands();
                var opLog = new OperationLog()
                {
                    CollectionName = log.CollectionName,
                    ActionType = log.ActionType,
                    SourceUrl = string.IsNullOrWhiteSpace(sourceUrl) ? ConfigurationManager.Host : sourceUrl,
                    IsSimple = isSimple
                };
                if (isSimple)
                {
                    //新DB 
                    if (log.Filter != null)
                    {
                        opLog.FilterContent = log.Filter.ToBsonValue().ToString();
                        opLog.Code = log.Filter.ToBsonValue()["Code"].ToString();
                        //updatemany
                        if (opLog.Code.Contains("$in"))
                        {
                            var list = log.Filter.ToBsonValue()["Code"]["$in"].AsBsonArray.ToList();
                            //var list = log.Filter.ToBsonValue()["Code"].AsBsonArray.ToList();
                            if (list?.Count > 0)
                            {
                                foreach (var item in list)
                                {
                                    opLog.Code = item.ToString() + ",";
                                }
                                opLog.Code = opLog.Code.TrimEnd(',');
                            }
                        }
                    }
                    if (log.Document != null)
                    {
                        var bsonDocument = log.Document.ToBsonDocument();
                        opLog.BsonContent = bsonDocument.ToString();
                        opLog.Code = bsonDocument["Code"].ToString();
                    }
                    if (log.Documents != null && log.Documents.Count() > 0)
                    {
                        var bsonDocument = log.Documents.LastOrDefault().ToBsonDocument();
                        opLog.BsonContent = bsonDocument.ToString();
                        opLog.Code = bsonDocument["Code"].ToString();
                    }
                }
                else
                {
                    var baseFilter = log.Filter.ToBsonValue();
                    if (log.Filter != null)
                    {
                        opLog.FilterContent = baseFilter.ToString();
                        if (baseFilter is BsonDocument bsonDocument)
                        {
                            if (bsonDocument.Contains("$and"))
                            {
                                var andFilter = baseFilter["$and"];
                                if (andFilter != null)
                                {
                                    opLog.Code = andFilter[0]["QueryData.QueryItems"]["$elemMatch"]["Value"].ToString();
                                    //updatemany
                                    if (andFilter[0]["QueryData.QueryItems"]["$elemMatch"]["Key"].ToString() == "Code" && opLog.Code.Contains("$in"))
                                    {
                                        var list = andFilter[0]["QueryData.QueryItems"]["$elemMatch"]["Value"]["$in"].AsBsonArray.ToList();
                                        if (list?.Count > 0)
                                        {
                                            foreach (var item in list)
                                            {
                                                opLog.Code = item.ToString() + ",";
                                            }
                                            opLog.Code = opLog.Code.TrimEnd(',');
                                        }
                                    }
                                }
                            }
                            else
                            {
                                opLog.Code = baseFilter["QueryData.QueryItems"]["$elemMatch"]["Value"].ToString();
                                //updatemany
                                if (baseFilter["QueryData.QueryItems"]["$elemMatch"]["Key"].ToString() == "Code" && opLog.Code.Contains("$in"))
                                {
                                    var list = baseFilter["QueryData.QueryItems"]["$elemMatch"]["Value"]["$in"].AsBsonArray.ToList();
                                    if (list?.Count > 0)
                                    {
                                        foreach (var item in list)
                                        {
                                            opLog.Code = item.ToString() + ",";
                                        }
                                        opLog.Code = opLog.Code.TrimEnd(',');
                                    }
                                }
                            }
                        }
                    }
                    if (log.Document != null)
                    {
                        opLog.BsonContent = log.Document.ToBsonDocument().ToString();
                        opLog.Code = log.Document.ToBsonDocument()["QueryData"]["QueryItems"][0]["Value"].ToString();
                    }
                    if (log.Documents != null && log.Documents.Count() > 0)
                    {
                        opLog.BsonContent = log.Documents.LastOrDefault().ToBsonDocument().ToString();
                        var list = log.Documents.LastOrDefault().ToBsonDocument()["QueryData"]["QueryItems"].AsBsonArray.ToList();
                        if (list?.Count > 0)
                        {
                            foreach (var item in list)
                            {
                                if (item.ToBsonDocument()["Key"] == "Code")
                                {
                                    opLog.Code = item.ToBsonDocument()["Value"].ToString();
                                    break;
                                }
                            }
                        }
                    }
                }

                if (log.Update != null)
                {
                    opLog.BsonContent = log.Update.ToBsonValue().ToString();
                }
                if (string.IsNullOrEmpty(opLog.BsonContent))
                {
                    opLog.BsonContent = log.ToString();
                }
                code = opLog.Code;
                if (ConfigurationManager.IsMaster)
                {
                    MasterOperationLogDbClient.Instance.Insert(opLog);
                }
                else
                {
                    _syncOpLogToMasterSequence.Add(DoSendOplogToMaster, opLog);
                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineError($"{log.CollectionName},{code},Record:{ex.ToString()}");
            }
        }

        bool DoSendOplogToMaster(OperationLog opLog)
        {
            if (_sendOplogToMaster != null)
            {
                for (int i = 1; i <= _maxRetryNum; i++) //retry 3 count, 执行其内部
                {
                    try
                    {
                        var result = _sendOplogToMaster(opLog);
                        if (result)
                        {
                            break;
                        }
                        else
                        {
                            Logger.WriteLineWarn($"SendOplogToMaster failed retry:{i} , CollectionName:{opLog.CollectionName},id:{opLog.Id},oplpg:{ConvertHelper.ToJson(opLog)} ");
                        }
                    }
                    catch (Exception ex)
                    {
                        Logger.WriteLineWarn($"SendOplogToMaster failed retry:{i} , CollectionName:{opLog.CollectionName},id:{opLog.Id}, ex:{ex.ToString()},oplpg:{ConvertHelper.ToJson(opLog)}");
                    }
                    if (i == _maxRetryNum)//执行三次失败记录DB中
                    {
                        SlaveOperationLogDbClient.Instance.Insert(opLog);
                    }
                    else
                    {
                        Thread.Sleep(200);
                    }
                }
            }
            return true;
        }

        bool RetryDoSendOplogToMaster(OperationLog opLog)
        {
            if (_sendOplogToMaster != null)
            {
                try
                {
                    var result = _sendOplogToMaster(opLog);
                    if (result)
                    {
                        SlaveOperationLogDbClient.Instance.DeleteById(opLog.Id);
                    }
                }
                catch (Exception ex)
                {
                    Logger.WriteLineWarn($"SendOplogToMaster failed retry , CollectionName:{opLog.CollectionName},id:{opLog.Id}, ex:{ex.ToString()},oplpg:{ConvertHelper.ToJson(opLog)} ");
                }

            }
            return true;
        }

        //TODO refactor how can i determin which logs should be synchronize again, maybe other server already change the same document
        private async Task RetrySendOpLog()
        {
            try
            {
                var logs = await SlaveOperationLogDbClient.Instance.GetManyAsync();
                foreach (var opLog in logs)
                {
                    _syncOpLogToMasterSequence.Add(RetryDoSendOplogToMaster, opLog);
                }
            }
            catch (Exception mex)
            {
                Logger.WriteLineWarn($"RetrySendOpLog ex:{mex}");
            }
        }
    }
}
