﻿using MongoDB.Driver;
using System.Collections.Generic;
using System;
using WingServerCommon.Config;
using WingServerCommon.Config.Parameters;
using System.Threading.Tasks;
using WingServerCommon.Log;

namespace WingMongoDBModule.DBSync
{
    class OplogDbInfo
    {
        /// <summary>
        /// The unique
        /// </summary>
        public int Id { get; set; }
        /// <summary>
        /// the current  cursor in master op logs which had sychronized in local 
        /// </summary>
        /// <returns></returns>
        public long CurrentMasterCursor { get; set; }

        /// <summary>
        /// Oplog Db version
        /// </summary>
        public string Version { get; set; }
    }
    /// <summary>
    /// Used in master server
    /// </summary>
    class SlaveOperationLogDbClient
    {
        private MongoClient _mongoClient;
        private IMongoCollection<OperationLog> _logs;
        private IMongoCollection<OplogDbInfo> _logDbInfos;
        private long _maxId = 0;
        private long _currentMasterCursor;
        private static SlaveOperationLogDbClient _instance;
        private const int OplogDbInfoId = 1;
        private static int _port = ConfigurationManager.GetParammeter<IntParameter>("Database", "Port").Value;

        private static string _server = ConfigurationManager.GetParammeter<StringParameter>("Database", "Server").Value;

        private static int _maxConn = ConfigurationManager.GetParammeter<IntParameter>("Database", "MaxConn").Value;
        private object _masterCursorLocker = new object();
        /// <summary>
        /// The singlton instance
        /// </summary>
        public static SlaveOperationLogDbClient Instance
        {
            get { return _instance ?? (_instance = new SlaveOperationLogDbClient()); }
        }

        /// <summary>
        /// the current  cursor in master op logs 
        /// </summary>
        /// <returns></returns>
        public long CurrentMasterCursor
        {
            get { return _currentMasterCursor; }
            set
            {
                if (value != _currentMasterCursor)
                {
                    if (value < _currentMasterCursor)
                    {
                        //Ignore 
                        return;
                    }

                    lock (_masterCursorLocker)
                    {
                        _currentMasterCursor = value;
                        OnCurrentMasterCursorChanged();
                    }
                }
            }
        }



        private SlaveOperationLogDbClient()
        {
            var mongoSetting = new MongoClientSettings
            {
                ConnectTimeout = TimeSpan.FromSeconds(5),
                Server = new MongoServerAddress(_server, _port),
                MaxConnectionPoolSize = _maxConn,
                WaitQueueTimeout = TimeSpan.FromSeconds(5),
                ServerSelectionTimeout = TimeSpan.FromSeconds(5),
                SocketTimeout = TimeSpan.FromSeconds(5)
            };
            _mongoClient = new MongoClient(mongoSetting);
            var database = _mongoClient.GetDatabase("WingSlaveOpLogDb");
            _logs = database.GetCollection<OperationLog>("OperationLogs");
            _logDbInfos = database.GetCollection<OplogDbInfo>("LogDbInfos");

            var filters = Builders<OperationLog>.Filter.Where(x => true);
            var maxRecord = _logs.Find(filters).SortByDescending(x => x.Id).Limit(1).FirstOrDefault();
            _maxId = maxRecord == null ? 0 : maxRecord.Id;

            var filters2 = Builders<OplogDbInfo>.Filter.Where(x => true);
            var record = _logDbInfos.Find(filters2).Limit(1).FirstOrDefault();
            if (record == null)
            {
                _logs.Indexes.CreateOne(new CreateIndexModel<OperationLog>(Builders<OperationLog>.IndexKeys.Ascending(t => t.Id)));
            }
            _currentMasterCursor = record == null ? 0 : record.CurrentMasterCursor;
        }

        private long GetMaxId()
        {
            return ++_maxId;
        }


        private void OnCurrentMasterCursorChanged()
        {
            var filters2 = Builders<OplogDbInfo>.Filter.Where(x => true);
            var record = _logDbInfos.Find(filters2).Limit(1).FirstOrDefault();
            if (record == null)
            {
                var newRecord = new OplogDbInfo() { Id = OplogDbInfoId, CurrentMasterCursor = CurrentMasterCursor };
                _logDbInfos.InsertOne(newRecord);
            }
            else
            {
                var update = Builders<OplogDbInfo>.Update.Set(s => s.CurrentMasterCursor, CurrentMasterCursor);
                _logDbInfos.UpdateOne(x => x.CurrentMasterCursor >= 0 && x.Id == OplogDbInfoId, update);
            }
        }


        /// <summary>
        /// Insert one operation log
        /// </summary>
        /// <param name="operationLog"></param>
        public void Insert(OperationLog operationLog)
        {
            operationLog.Id = GetMaxId();
            _logs.InsertOne(operationLog);
        }

        /// <summary>
        /// Insert many operation logs
        /// </summary>
        /// <param name="logs"></param>
        public void InsertMany(IList<OperationLog> logs)
        {
            foreach (var log in logs)
            {
                log.Id = GetMaxId();
            }
            _logs.InsertMany(logs);
        }

        /// <summary>
        /// Get many
        /// </summary>
        /// <param name="cursor"></param>
        public async Task<IList<OperationLog>> GetManyAsync(int limit = 100)
        {
            var logs = await _logs.FindAsync(x => true, new FindOptions<OperationLog> { Limit = limit });
            return await logs.ToListAsync();
        }

        public async Task<bool> DeleteById(long id)
        {
            try
            {
                var deleteResult = await _logs.DeleteOneAsync(d => d.Id == id);
                return deleteResult.DeletedCount > 0;
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"oplog DeleteByIdid:{id}, error:{ex.ToString()}");
                return false;
            }
        }
    }
}
