﻿using System.Runtime.CompilerServices;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq.Expressions;
using MongoDB.Bson;
using MongoDB.Driver;
using WingMongoDBModule.Entity;
using WingServerCommon.Log;

namespace WingMongoDBModule.Mongodb.Driver
{
    public sealed class MongoCustomerFilter<T> : IEnumerable<FilterDefinition<T>>
    {
        /// <summary>
        /// 过滤器创建者
        /// </summary>
        private FilterDefinitionBuilder<T> _filter = Builders<T>.Filter;

        /// <summary>
        /// 过滤器
        /// </summary>
        private FilterDefinition<T> _allFilterDefinition;

        /// <summary>
        /// 过滤器
        /// </summary>
        private List<FilterDefinition<T>> _tempFilterDefinitionList = new();

        /// <summary>
        /// 是否有过滤条件
        /// </summary>
        /// <value></value>
        public bool HasFilter
        {
            get
            {
                return _tempFilterDefinitionList != null || _allFilterDefinition != null;
            }
        }

        private bool AddFilter(FilterDefinition<T> filter)
        {
            try
            {
                _tempFilterDefinitionList.Add(filter);
                return true;
            }
            catch (Exception ex)
            {
                Logger.WriteLineError($"AddFilter error {ex}");
                return false;
            }
        }

        private bool AddRangeFilter(IEnumerable<FilterDefinition<T>> filter)
        {
            try
            {
                _tempFilterDefinitionList.AddRange(filter);
                return true;
            }
            catch (Exception ex)
            {
                Logger.WriteLineError($"AddRangeFilter error {ex}");
                return false;
            }
        }

        /// <summary>
        /// 清除过滤器
        /// </summary>
        public void ClearFilter()
        {
            _tempFilterDefinitionList.Clear();
            _allFilterDefinition = null;
        }

        #region Expression functions

        /// <summary>
        /// 追加Gte过滤器(大于等于)
        /// </summary>
        /// <param name="key">字段名</param>
        /// <param name="value">字段值</param>
        /// <returns></returns>
        public bool AppendGte<TItem>(Expression<Func<T, TItem>> key, TItem value)
        {
            return AddFilter(Builders<T>.Filter.Gte(key, value));
        }

        /// <summary>
        /// 追加Lte过滤器(小于等于)
        /// </summary>
        /// <param name="key">字段名</param>
        /// <param name="value">字段值</param>
        /// <returns></returns>
        public bool AppendLte<TItem>(Expression<Func<T, TItem>> key, TItem value)
        {
            return AddFilter(Builders<T>.Filter.Lte(key, value));
        }

        /// <summary>
        /// 追加Gt过滤器(大于)
        /// </summary>
        /// <param name="key">字段名</param>
        /// <param name="value">字段值</param>
        /// <returns></returns>
        public bool AppendGt<TItem>(Expression<Func<T, TItem>> key, TItem value)
        {
            return AddFilter(Builders<T>.Filter.Gt(key, value));
        }

        /// <summary>
        /// 追加Lt过滤器(小于)
        /// </summary>
        /// <param name="key">字段名</param>
        /// <param name="value">字段值</param>
        /// <returns></returns>
        public bool AppendLt<TItem>(Expression<Func<T, TItem>> key, TItem value)
        {
            return AddFilter(Builders<T>.Filter.Lt(key, value));
        }

        /// <summary>
        /// 追加In过滤器
        /// </summary>
        /// <param name="key">字段名</param>
        /// <param name="list">字段值列表</param>
        /// <returns></returns>
        public bool AppendIn<TItem>(Expression<Func<T, TItem>> key, List<TItem> list)
        {
            return AddFilter(Builders<T>.Filter.In(key, list));
        }

        /// <summary>
        /// 追加NotIn过滤器
        /// </summary>
        /// <param name="key">字段名</param>
        /// <param name="list">字段值列表</param>
        /// <returns></returns>
        public bool AppendNotIn<TItem>(Expression<Func<T, TItem>> key, List<TItem> list)
        {
            return AddFilter(Builders<T>.Filter.Nin<TItem>(key, list));
        }

        /// <summary>
        /// 追加子查询过滤器
        /// </summary>
        /// <param name="field">子查询字段</param>
        /// <param name="subFilter">子过滤器</param>
        /// <typeparam name="TItem">字段类型</typeparam>
        /// <returns></returns>
        public bool SubFilter<TItem>(Expression<Func<T, IEnumerable<TItem>>> field, params FilterDefinition<TItem>[] subFilters)
        {
            List<FilterDefinition<T>> filters = new List<FilterDefinition<T>>();
            foreach (var subFilter in subFilters)
            {
                filters.Add(Builders<T>.Filter.ElemMatch<TItem>(field, subFilter));
            }
            return AddRangeFilter(filters);
        }

        /// <summary>
        /// 追加子查询过滤器
        /// </summary>
        /// <param name="field">子查询字段</param>
        /// <param name="subFilter">子过滤器</param>
        /// <typeparam name="TItem">字段类型</typeparam>
        /// <returns></returns>
        public bool SubFilter<TItem>(Expression<Func<T, IEnumerable<TItem>>> field, Func<MongoCustomerFilter<TItem>, FilterDefinition<TItem>> subFilter)
        {
            List<FilterDefinition<T>> filters = new List<FilterDefinition<T>>();
            var mongoCustomerFilter = new MongoCustomerFilter<TItem>();
            filters.Add(Builders<T>.Filter.ElemMatch<TItem>(field, subFilter.Invoke(mongoCustomerFilter)));
            return AddRangeFilter(filters);
        }

        /// <summary>
        /// 追加子查询
        /// </summary>
        /// <param name="combineType">条件类型</param>
        /// <param name="key">关键字</param>
        /// <param name="value">键值</param>
        /// <param name="kvArray">关键字条件数组</param>
        /// <returns></returns>
        public bool AppendQueryItemEqFilter(MongoCombineTypeEnum combineType, params (string key, string value)[] kvArray)
        {
            if (kvArray.Length > 0)
            {
                List<FilterDefinition<T>> filters = new List<FilterDefinition<T>>();
                var mongoCustomerFilter = new MongoCustomerFilter<MongoDataItem>();
                foreach (var kv in kvArray)
                {
                    var combineSubFilter = new MongoCustomerFilter<MongoDataItem>();
                    combineSubFilter.AppendEq((item => item.Key, kv.key), (item => item.Value, kv.value));
                    mongoCustomerFilter.AddFilter(combineSubFilter.AndCombine());
                }
                return CombineFilter<MongoDataItem>(combineType, mongoCustomerFilter);
            }
            return false;
        }

        /// <summary>
        /// 追加子查询
        /// </summary>
        /// <param name="combineType">条件类型</param>
        /// <param name="key">关键字</param>
        /// <param name="value">键值</param>
        /// <param name="kvArray">关键字条件数组</param>
        /// <returns></returns>
        public bool AppendQueryItemInFilter(MongoCombineTypeEnum combineType, string key, List<string> values)
        {
            if (values.Count > 0)
            {
                List<FilterDefinition<T>> filters = new List<FilterDefinition<T>>();
                var mongoCustomerFilter = new MongoCustomerFilter<MongoDataItem>();
                foreach (var value in values)
                {
                    var combineSubFilter = new MongoCustomerFilter<MongoDataItem>();
                    combineSubFilter.AppendEq((item=>item.Key, key),(item=>item.Value, value));
                    mongoCustomerFilter.AddFilter(combineSubFilter.AndCombine());
                }
                return CombineFilter<MongoDataItem>(combineType, mongoCustomerFilter);
            }
            return false;
        }

        /// <summary>
        /// 追加子查询
        /// </summary>
        /// <param name="combineType">条件类型</param>
        /// <param name="key">关键字</param>
        /// <param name="value">键值</param>
        /// <param name="isFuzzy">是否模糊匹配</param>
        /// <param name="isMatchCase">大小写匹配</param>
        /// <param name="kvArray">关键字条件数组</param>
        /// <returns></returns>
        public bool AppendQueryItemRegexFilter(MongoCombineTypeEnum combineType, params (string key, string value, bool isFuzzy, bool isMatchCase)[] kvArray)
        {
            if (kvArray.Length > 0)
            {
                List<FilterDefinition<T>> filters = new List<FilterDefinition<T>>();
                var mongoCustomerFilter = new MongoCustomerFilter<MongoDataItem>();
                foreach (var kv in kvArray)
                {
                    var combineSubFilter = new MongoCustomerFilter<MongoDataItem>();
                    combineSubFilter.AppendEq((item => item.Key, kv.key));
                    combineSubFilter.AppendRegex(item => item.Value, kv.value, kv.isFuzzy, kv.isMatchCase);
                    mongoCustomerFilter.AddFilter(combineSubFilter.AndCombine());
                }
                return CombineFilter<MongoDataItem>(combineType, mongoCustomerFilter);
            }
            return false;
        }

        /// <summary>
        /// 追加Eq过滤器
        /// </summary>
        /// <param name="list">条件列表，key：字段名，value：字段值</param>
        /// <returns></returns>
        public bool AppendEq<TItem>(params (Expression<Func<T, TItem>> key, TItem value)[] list)
        {
            List<FilterDefinition<T>> filters = new List<FilterDefinition<T>>();
            foreach (var kv in list)
            {
                filters.Add(Builders<T>.Filter.Eq<TItem>(kv.key, kv.value));
            }
            return AddRangeFilter(filters);
        }

        /// <summary>
        /// 追加Not过滤器
        /// </summary>
        /// <param name="list">条件列表，key：字段名，value：字段值</param>
        /// <returns></returns>
        public bool AppendNot<TItem>(params (Expression<Func<T, TItem>> key, TItem value)[] list)
        {
            List<FilterDefinition<T>> filters = new List<FilterDefinition<T>>();
            foreach (var kv in list)
            {
                filters.Add(Builders<T>.Filter.Not(Builders<T>.Filter.Eq<TItem>(kv.key, kv.value)));
            }
            return AddRangeFilter(filters);
        }

        /// <summary>
        /// 追加正则过滤器
        /// </summary>
        /// <param name="key">字段名</param>
        /// <param name="pattern">正则表达式</param>
        /// <param name="isFuzzy">是否模糊查询</param>
        /// <returns></returns>
        public bool AppendRegex(Expression<Func<T, object>> key, string pattern, bool isFuzzy, bool isMatchCase = true)
        {
            BsonRegularExpression regularExp = null;
            pattern = ReplaceSpecialCharacter(pattern);
            if (isMatchCase)
            {
                regularExp = new BsonRegularExpression((isFuzzy ? $".*{pattern}.*" : pattern));
            }
            else
            {
                regularExp = new BsonRegularExpression((isFuzzy ? $".*{pattern}.*" : pattern), "i");
            }
            var tempFilter = Builders<T>.Filter.Regex(key, regularExp);
            return AddFilter(tempFilter);
        }

        /// <summary>
        /// 追加正则反向匹配过滤器
        /// </summary>
        /// <param name="key">字段名</param>
        /// <param name="pattern">正则表达式</param>
        /// <param name="isFuzzy">是否模糊查询</param>
        /// <returns></returns>
        public bool AppendReverseRegex(Expression<Func<T, object>> key, string pattern, bool isFuzzy, bool isMatchCase = true)
        {
            BsonRegularExpression regularExp = null;
            pattern = ReplaceSpecialCharacter(pattern);
            if (isMatchCase)
            {
                regularExp = new BsonRegularExpression((isFuzzy ? $".*{pattern}.*" : pattern));
            }
            else
            {
                regularExp = new BsonRegularExpression((isFuzzy ? $".*{pattern}.*" : pattern), "i");
            }
            var tempFilter = Builders<T>.Filter.Not(Builders<T>.Filter.Regex(key, regularExp));
            return AddFilter(tempFilter);
        }

        /// <summary>
        /// 追加正则过滤器
        /// </summary>
        /// <param name="key">字段名</param>
        /// <param name="pattern">正则表达式</param>
        /// <param name="isFuzzy">是否模糊查询</param>
        /// <returns></returns>
        public bool AppendRegexForKey(string key, string pattern, bool isFuzzy, bool isMatchCase = true)
        {
            BsonRegularExpression regularExp = null;
            pattern = ReplaceSpecialCharacter(pattern);
            if (isMatchCase)
            {
                regularExp = new BsonRegularExpression((isFuzzy ? $".*{pattern}.*" : pattern));
            }
            else
            {
                regularExp = new BsonRegularExpression((isFuzzy ? $".*{pattern}.*" : pattern), "i");
            }
            var tempFilter = Builders<T>.Filter.Regex(key, regularExp);
            return AddFilter(tempFilter);
        }

        /// <summary>
        /// 追加正则反向匹配过滤器
        /// </summary>
        /// <param name="key">字段名</param>
        /// <param name="pattern">正则表达式</param>
        /// <param name="isFuzzy">是否模糊查询</param>
        /// <returns></returns>
        public bool AppendReverseRegexForKey(string key, string pattern, bool isFuzzy, bool isMatchCase = true)
        {
            BsonRegularExpression regularExp = null;
            pattern = ReplaceSpecialCharacter(pattern);
            if (isMatchCase)
            {
                regularExp = new BsonRegularExpression((isFuzzy ? $".*{pattern}.*" : pattern));
            }
            else
            {
                regularExp = new BsonRegularExpression((isFuzzy ? $".*{pattern}.*" : pattern), "i");
            }
            var tempFilter = Builders<T>.Filter.Not(Builders<T>.Filter.Regex(key, regularExp));
            return AddFilter(tempFilter);
        }

        /// <summary>
        /// 追加列表过滤器
        /// </summary>
        /// <param name="key">字段名</param>
        /// <param name="pattern">正则表达式</param>
        /// <returns></returns>
        public bool AppendArrayEq<TItem>(Expression<Func<T, IEnumerable<TItem>>> key, TItem value)
        {
            var tempFilter = Builders<T>.Filter.AnyEq(key, value);
            return AddFilter(tempFilter);
        }

        /// <summary>
        /// 追加非列表过滤器
        /// </summary>
        /// <param name="key">字段名</param>
        /// <param name="pattern">正则表达式</param>
        /// <returns></returns>
        public bool AppendNotArrayEq<TItem>(Expression<Func<T, IEnumerable<TItem>>> key, TItem value)
        {
            var tempFilter = Builders<T>.Filter.Not(Builders<T>.Filter.AnyEq(key, value));
            return AddFilter(tempFilter);
        }

        /// <summary>
        /// 追加AnyIn过滤器，数组字段中有一个在所传列表中的
        /// </summary>
        /// <param name="key">字段名</param>
        /// <param name="list">字段值列表</param>
        /// <returns></returns>
        public bool AppendAnyIn<TItem>(Expression<Func<T, IEnumerable<TItem>>> key, List<TItem> list)
        {
            return AddFilter(Builders<T>.Filter.AnyIn<TItem>(key, list));
        }

        /// <summary>
        /// 追加AnyNotIn过滤器，数组字段中所有都不在所传列表中的
        /// </summary>
        /// <param name="key">字段名</param>
        /// <param name="list">字段值列表</param>
        /// <returns></returns>
        public bool AppendAnyNotIn<TItem>(Expression<Func<T, IEnumerable<TItem>>> key, List<TItem> list)
        {
            return AddFilter(Builders<T>.Filter.AnyNin<TItem>(key, list));
        }

        /// <summary>
        /// 追加SizeGt过滤器(小于)
        /// </summary>
        /// <param name="key">字段名</param>
        /// <param name="size">大小</param>
        /// <returns></returns>
        public bool AppendSizeLt(FieldDefinition<T> key, int size)
        {
            return AddFilter(Builders<T>.Filter.SizeLt(key, size));
        }

        /// <summary>
        /// 追加SizeGt过滤器(小于等于)
        /// </summary>
        /// <param name="key">字段名</param>
        /// <param name="size">大小</param>
        public bool AppendSizeLte(FieldDefinition<T> key, int size)
        {
            return AddFilter(Builders<T>.Filter.SizeLte(key, size));
        }        

        /// <summary>
        /// 追加SizeGt过滤器(大于)
        /// </summary>
        /// <param name="key">字段名</param>
        /// <param name="size">大小</param>
        public bool AppendSizeGt(FieldDefinition<T> key, int size)
        {
            return AddFilter(Builders<T>.Filter.SizeGt(key, size));
        }        

        /// <summary>
        /// 追加SizeGt过滤器(大于等于)
        /// </summary>
        /// <param name="key">字段名</param>
        /// <param name="size">大小</param>
        public bool AppendSizeGte(FieldDefinition<T> key, int size)
        {
            return AddFilter(Builders<T>.Filter.SizeGte(key, size));
        }

        #endregion


        #region String field functions
        /// <summary>
        /// 追加Gte过滤器(大于等于)
        /// </summary>
        /// <param name="key">字段名</param>
        /// <param name="value">字段值</param>
        /// <returns></returns>
        public bool AppendGte<TItem>(string key, TItem value)
        {
            return AddFilter(Builders<T>.Filter.Gte(key, value));
        }

        /// <summary>
        /// 追加Lte过滤器(小于等于)
        /// </summary>
        /// <param name="key">字段名</param>
        /// <param name="value">字段值</param>
        /// <returns></returns>
        public bool AppendLte<TItem>(string key, TItem value)
        {
            return AddFilter(Builders<T>.Filter.Lte(key, value));
        }

        /// <summary>
        /// 追加Gt过滤器(大于)
        /// </summary>
        /// <param name="key">字段名</param>
        /// <param name="value">字段值</param>
        /// <returns></returns>
        public bool AppendGt<TItem>(string key, TItem value)
        {
            return AddFilter(Builders<T>.Filter.Gt(key, value));
        }

        /// <summary>
        /// 追加Lt过滤器(小于)
        /// </summary>
        /// <param name="key">字段名</param>
        /// <param name="value">字段值</param>
        /// <returns></returns>
        public bool AppendLt<TItem>(string key, TItem value)
        {
            return AddFilter(Builders<T>.Filter.Lt(key, value));
        }

        /// <summary>
        /// 追加In过滤器
        /// </summary>
        /// <param name="key">字段名</param>
        /// <param name="list">字段值列表</param>
        /// <returns></returns>
        public bool AppendIn<TItem>(string key, List<TItem> list)
        {
            return AddFilter(Builders<T>.Filter.In(key, list));
        }

        /// <summary>
        /// 追加NotIn过滤器
        /// </summary>
        /// <param name="key">字段名</param>
        /// <param name="list">字段值列表</param>
        /// <returns></returns>
        public bool AppendNotIn<TItem>(string key, List<TItem> list)
        {
            return AddFilter(Builders<T>.Filter.Nin<TItem>(key, list));
        }

        /// <summary>
        /// 追加AnyIn过滤器，数组字段中有一个在所传列表中的
        /// </summary>
        /// <param name="key">字段名</param>
        /// <param name="list">字段值列表</param>
        /// <returns></returns>
        public bool AppendAnyIn<TItem>(string key, List<TItem> list)
        {
            return AddFilter(Builders<T>.Filter.AnyIn<TItem>(key, list));
        }

        /// <summary>
        /// 追加AnyNotIn过滤器，数组字段中所有都不在所传列表中的
        /// </summary>
        /// <param name="key">字段名</param>
        /// <param name="list">字段值列表</param>
        /// <returns></returns>
        public bool AppendAnyNotIn<TItem>(string key, List<TItem> list)
        {
            return AddFilter(Builders<T>.Filter.AnyNin<TItem>(key, list));
        }

        /// <summary>
        /// 追加子查询过滤器
        /// </summary>
        /// <param name="field">子查询字段</param>
        /// <param name="subFilter">子过滤器</param>
        /// <typeparam name="TItem">字段类型</typeparam>
        /// <returns></returns>
        public bool SubFilter<TItem>(string field, params FilterDefinition<TItem>[] subFilters)
        {
            List<FilterDefinition<T>> filters = new List<FilterDefinition<T>>();
            foreach (var subFilter in subFilters)
            {
                filters.Add(Builders<T>.Filter.ElemMatch<TItem>(field, subFilter));
            }
            return AddRangeFilter(filters);
        }

        /// <summary>
        /// 追加And过滤器
        /// </summary>
        /// <param name="list">条件列表，key：字段名，value：字段值</param>
        /// <returns></returns>
        public bool AppendEq<TItem>(params (string key, TItem value)[] list)
        {
            List<FilterDefinition<T>> filters = new List<FilterDefinition<T>>();
            foreach (var kv in list)
            {
                filters.Add(Builders<T>.Filter.Eq<TItem>(kv.key, kv.value));
            }
            return AddRangeFilter(filters);
        }

        /// <summary>
        /// 追加Not过滤器
        /// </summary>
        /// <param name="list">条件列表，key：字段名，value：字段值</param>
        /// <returns></returns>
        public bool AppendNot<TItem>(params (string key, TItem value)[] list)
        {
            List<FilterDefinition<T>> filters = new List<FilterDefinition<T>>();
            foreach (var kv in list)
            {
                filters.Add(Builders<T>.Filter.Not(Builders<T>.Filter.Eq<TItem>(kv.key, kv.value)));
            }
            return AddRangeFilter(filters);
        }

        /// <summary>
        /// 追加正则过滤器
        /// </summary>
        /// <param name="key">字段名</param>
        /// <param name="pattern">正则表达式</param>
        /// <param name="isFuzzy">是否模糊查询</param>
        /// <returns></returns>
        public bool AppendRegex(string key, string pattern, bool isFuzzy, bool isMatchCase = true)
        {
            BsonRegularExpression regularExp = null;
            pattern = ReplaceSpecialCharacter(pattern);
            if (isMatchCase)
            {
                regularExp = new BsonRegularExpression((isFuzzy ? $".*{pattern}.*" : pattern));
            }
            else
            {
                regularExp = new BsonRegularExpression((isFuzzy ? $".*{pattern}.*" : pattern), "i");
            }
            var tempFilter = Builders<T>.Filter.Regex(key, regularExp);
            return AddFilter(tempFilter);
        }

        /// <summary>
        /// 追加正则反向匹配过滤器
        /// </summary>
        /// <param name="key">字段名</param>
        /// <param name="pattern">正则表达式</param>
        /// <param name="isFuzzy">是否模糊查询</param>
        /// <returns></returns>
        public bool AppendReverseRegex(string key, string pattern, bool isFuzzy, bool isMatchCase = true)
        {
            BsonRegularExpression regularExp = null;
            pattern = ReplaceSpecialCharacter(pattern);
            if (isMatchCase)
            {
                regularExp = new BsonRegularExpression((isFuzzy ? $".*{pattern}.*" : pattern));
            }
            else
            {
                regularExp = new BsonRegularExpression((isFuzzy ? $".*{pattern}.*" : pattern), "i");
            }
            var tempFilter = Builders<T>.Filter.Not(Builders<T>.Filter.Regex(key, regularExp));
            return AddFilter(tempFilter);
        }

        /// <summary>
        /// 追加列表过滤器
        /// </summary>
        /// <param name="key">字段名</param>
        /// <param name="pattern">正则表达式</param>
        /// <returns></returns>
        public bool AppendArrayEq<TItem>(string key, TItem value)
        {
            var tempFilter = Builders<T>.Filter.AnyEq(key, value);
            return AddFilter(tempFilter);
        }

        /// <summary>
        /// 追加非列表过滤器
        /// </summary>
        /// <param name="key">字段名</param>
        /// <param name="pattern">正则表达式</param>
        /// <returns></returns>
        public bool AppendNotArrayEq<TItem>(string key, TItem value)
        {
            var tempFilter = Builders<T>.Filter.Not(Builders<T>.Filter.AnyEq(key, value));
            return AddFilter(tempFilter);
        }

        #endregion

        /// <summary>
        /// 交集查询
        /// </summary>
        /// <param name="filters"></param>
        /// <returns></returns>
        public FilterDefinition<T> AndCombine(params FilterDefinition<T>[] filters)
        {
            if ((_tempFilterDefinitionList == null || _tempFilterDefinitionList.Count == 0) && (filters == null || filters.Length == 0))
            {
                if (_allFilterDefinition == null)
                {
                    return Builders<T>.Filter.Empty;
                }
            }
            else
            {
                _tempFilterDefinitionList.AddRange(filters);
                _allFilterDefinition = Builders<T>.Filter.And(_tempFilterDefinitionList.ToArray());
            }
            _tempFilterDefinitionList.Clear();
            return _allFilterDefinition;
        }

        /// <summary>
        /// 并集查询
        /// </summary>
        /// <param name="filters"></param>
        /// <returns></returns>
        public FilterDefinition<T> OrCombine(params FilterDefinition<T>[] filters)
        {
            if ((_tempFilterDefinitionList == null || _tempFilterDefinitionList.Count == 0) && (filters == null || filters.Length == 0))
            {
                if (_allFilterDefinition == null)
                {
                    return Builders<T>.Filter.Empty;
                }
            }
            else
            {
                _tempFilterDefinitionList.AddRange(filters);
                _allFilterDefinition = Builders<T>.Filter.Or(_tempFilterDefinitionList.ToArray());
            }
            _tempFilterDefinitionList.Clear();
            return _allFilterDefinition;
        }

        /// <summary>
        /// 非集查询
        /// </summary>
        /// <param name="filters"></param>
        /// <returns></returns>
        public FilterDefinition<T> NotCombine(params FilterDefinition<T>[] filters)
        {
            if ((_tempFilterDefinitionList == null || _tempFilterDefinitionList.Count == 0) && (filters == null || filters.Length == 0))
            {
                if (_allFilterDefinition == null)
                {
                    return Builders<T>.Filter.Empty;
                }
            }
            else
            {
                _tempFilterDefinitionList.AddRange(filters);
                _allFilterDefinition = Builders<T>.Filter.Not(Builders<T>.Filter.And(_tempFilterDefinitionList.ToArray()));
            }
            _tempFilterDefinitionList.Clear();
            return _allFilterDefinition;
        }

        private bool CombineFilter<TItem>(MongoCombineTypeEnum combineType, MongoCustomerFilter<TItem> mongoCustomerFilter)
        {
            MongoCustomerFilter<T> subCustomer = new MongoCustomerFilter<T>();
            foreach (var filter in mongoCustomerFilter)
            {
                subCustomer.SubFilter<TItem>("QueryData.QueryItems", filter);
            }
            mongoCustomerFilter.ClearFilter();
            bool isSuccess = false;
            switch (combineType)
            {
                case MongoCombineTypeEnum.And:
                    _tempFilterDefinitionList.Add(subCustomer.AndCombine());
                    isSuccess = true;
                    break;
                case MongoCombineTypeEnum.Or:
                    _tempFilterDefinitionList.Add(subCustomer.OrCombine());
                    isSuccess = true;
                    break;
                case MongoCombineTypeEnum.Not:
                    _tempFilterDefinitionList.Add(subCustomer.NotCombine());
                    isSuccess = true;
                    break;
            }
            return isSuccess;
        }

        public IEnumerator<FilterDefinition<T>> GetEnumerator()
        {
            return _tempFilterDefinitionList.GetEnumerator();
        }

        IEnumerator IEnumerable.GetEnumerator()
        {
            return GetEnumerator();
        }

        private string ReplaceSpecialCharacter(string str)
        {
            return str
            .Replace("(", "\\(")
            .Replace(")", "\\)")
            .Replace("+", "\\+")
            .Replace("<", "\\<")
            .Replace(">", "\\>")
            .Replace("[", "\\[")
            .Replace("]", "\\]")
            .Replace("*", "\\*")
            .Replace(".", "\\.");
        }
    }
}