﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq.Expressions;
using MongoDB.Driver;
using WingServerCommon.Log;

namespace WingMongoDBModule.Mongodb.Driver
{
    public sealed class MongoCustomerUpdate<T>
    {
        /// <summary>
        /// 更新器创建者
        /// </summary>
        private UpdateDefinitionBuilder<T> _update = Builders<T>.Update;

        /// <summary>
        /// 更新器
        /// </summary>
        private List<UpdateDefinition<T>> _allUpdateDefinition = new List<UpdateDefinition<T>>();

        /// <summary>
        /// 更新器属性
        /// </summary>
        /// <value></value>
        public UpdateDefinition<T> AllUpdateDefinition
        {
            get
            {
                if (_allUpdateDefinition != null && _allUpdateDefinition.Count > 0)
                {
                    return _update.Combine(_allUpdateDefinition);
                }
                else
                {
                    return _update.Combine();
                }
            }
        }


        /// <summary>
        /// 它接受一个字段名称和一个值并将其添加到要更新的字段列表中。
        /// </summary>
        /// <param name="field">要更新的字段</param>
        /// <param name="TField">要更新的字段的类型。</param>
        /// <returns>
        /// 一个布尔值。
        /// </returns>
        public bool SetField<TField>(Expression<Func<T, TField>> field, TField value)
        {
            try
            {
                _allUpdateDefinition.Add(_update.Set<TField>(field, value));
                return true;
            }
            catch (Exception ex)
            {
                Logger.WriteLineError($"MongoCustomerUpdate {new StackTrace().GetFrame(0)} error {ex}");
                return false;
            }
        }

        /// <summary>
        /// 此函数将新项目添加到文档的子集合中
        /// </summary>
        /// <param name="field">要将值添加到的字段</param>
        /// <param name="TItem">子项的类型</param>
        /// <returns>
        /// 一个布尔值。
        /// </returns>
        public bool AddSubItem<TItem>(Expression<Func<T, IEnumerable<TItem>>> field, TItem value)
        {
            try
            {
                _allUpdateDefinition.Add(_update.AddToSet<TItem>(field, value));
                return true;
            }
            catch (Exception ex)
            {
                Logger.WriteLineError($"MongoCustomerUpdate {new StackTrace().GetFrame(0)} error {ex}");
                return false;
            }
        }


        /// <summary>
        /// 此函数从子项列表中删除一个子项
        /// </summary>
        /// <param name="field">要更新的字段</param>
        /// <param name="TItem">子项的类型</param>
        /// <returns>
        /// 一个布尔值。
        /// </returns>
        public bool RemoveSubItem<TItem>(Expression<Func<T, IEnumerable<TItem>>> field, TItem value)
        {
            try
            {
                _allUpdateDefinition.Add(_update.Pull<TItem>(field, value));
                return true;
            }
            catch (Exception ex)
            {
                Logger.WriteLineError($"MongoCustomerUpdate {new StackTrace().GetFrame(0)} error {ex}");
                return false;
            }
        }

        /// <summary>
        /// 它将过滤器添加到更新定义
        /// </summary>
        /// <param name="field">要更新的字段</param>
        /// <param name="value">要从数组中删除的值。</param>
        /// <returns>
        /// 一个布尔值。
        /// </returns>

        public bool RemoveSubItemFilter<TItem>(Expression<Func<T, IEnumerable<TItem>>> field, FilterDefinition<TItem> value)
        {
            try
            {
                _allUpdateDefinition.Add(_update.PullFilter<TItem>(field, value));
                return true;
            }
            catch (Exception ex)
            {
                Logger.WriteLineError($"MongoCustomerUpdate {new StackTrace().GetFrame(0)} error {ex}");
                return false;
            }
        }


        /// <summary>
        /// 此函数将项目列表添加到文档的子项目
        /// </summary>
        /// <param name="field">要将数组添加到的字段</param>
        /// <param name="values">列出<TItem> 值</param>
        /// <returns>
        /// 一个布尔值。
        /// </returns>
        public bool AddArrayToSubItem<TItem>(Expression<Func<T, IEnumerable<TItem>>> field, List<TItem> values)
        {
            try
            {
                _allUpdateDefinition.Add(_update.AddToSetEach<TItem>(field, values));
                return true;
            }
            catch (Exception ex)
            {
                Logger.WriteLineError($"MongoCustomerUpdate {new StackTrace().GetFrame(0)} error {ex}");
                return false;
            }
        }

        /// <summary>
        /// 此函数将项目列表附加到文档的子项目
        /// </summary>
        /// <param name="field">要将数组添加到的字段</param>
        /// <param name="values">列出<TItem> 值</param>
        /// <returns>
        /// 一个布尔值。
        /// </returns>
        public bool PushArrayToSubItem<TItem>(Expression<Func<T, IEnumerable<TItem>>> field, List<TItem> values)
        {
            try
            {
                _allUpdateDefinition.Add(_update.PushEach<TItem>(field, values));
                return true;
            }
            catch (Exception ex)
            {
                Logger.WriteLineError($"MongoCustomerUpdate {new StackTrace().GetFrame(0)} error {ex}");
                return false;
            }
        }


        /// <summary>
        /// 它从子项中删除一组项。
        /// </summary>
        /// <param name="field">要更新的字段</param>
        /// <param name="values">要从数组中删除的值列表</param>
        /// <returns>
        /// 一个布尔值。
        /// </returns>
        public bool RemoveArrayToSubItem<TItem>(Expression<Func<T, IEnumerable<TItem>>> field, List<TItem> values)
        {
            try
            {
                _allUpdateDefinition.Add(_update.PullAll<TItem>(field, values));
                return true;
            }
            catch (Exception ex)
            {
                Logger.WriteLineError($"MongoCustomerUpdate {new StackTrace().GetFrame(0)} error {ex}");
                return false;
            }
        }


        /// <summary>
        /// 此函数将字段添加到更新定义
        /// </summary>
        /// <param name="field">要更新的字段名称</param>
        /// <param name="TField">您要更新的字段的类型。</param>
        /// <returns>
        /// 一个布尔值。
        /// </returns>
        public bool SetField<TField>(string field, TField value)
        {
            try
            {
                _allUpdateDefinition.Add(_update.Set<TField>(field, value));
                return true;
            }
            catch (Exception ex)
            {
                Logger.WriteLineError($"MongoCustomerUpdate {new StackTrace().GetFrame(0)} error {ex}");
                return false;
            }
        }


        /// <summary>
        /// 此函数将新项目添加到文档中的项目列表中。
        /// </summary>
        /// <param name="field">要将子项添加到的字段的名称</param>
        /// <param name="TItem">子项的类型</param>
        /// <returns>
        /// 一个布尔值。
        /// </returns>
        public bool AddSubItem<TItem>(string field, TItem value)
        {
            try
            {
                _allUpdateDefinition.Add(_update.AddToSet<TItem>(field, value));
                return true;
            }
            catch (Exception ex)
            {
                Logger.WriteLineError($"MongoCustomerUpdate {new StackTrace().GetFrame(0)} error {ex}");
                return false;
            }
        }


        /// <summary>
        /// 此函数从子项列表中删除一个子项
        /// </summary>
        /// <param name="field">文档中的字段名称</param>
        /// <param name="TItem">子项的类型</param>
        /// <returns>
        /// 一个布尔值。
        /// </returns>
        public bool RemoveSubItem<TItem>(string field, TItem value)
        {
            try
            {
                _allUpdateDefinition.Add(_update.Pull<TItem>(field, value));
                return true;
            }
            catch (Exception ex)
            {
                Logger.WriteLineError($"MongoCustomerUpdate {new StackTrace().GetFrame(0)} error {ex}");
                return false;
            }
        }

        /// <summary>
        /// 此函数从文档的子项列表中删除一个子项
        /// </summary>
        /// <param name="field">文档中字段的名称</param>
        /// <param name="value"></param>
        /// <returns>
        /// 一个布尔值。
        /// </returns>
        public bool RemoveSubItemFilter<TItem>(string field, FilterDefinition<TItem> value)
        {
            try
            {
                _allUpdateDefinition.Add(_update.PullFilter<TItem>(field, value));
                return true;
            }
            catch (Exception ex)
            {
                Logger.WriteLineError($"MongoCustomerUpdate {new StackTrace().GetFrame(0)} error {ex}");
                return false;
            }
        }


        /// <summary>
        /// 此函数将项目数组添加到文档中的子项目
        /// </summary>
        /// <param name="field">文档中的字段名称</param>
        /// <param name="values">列表<TItem></param>
        /// <returns>
        /// 一个布尔值。
        /// </returns>
        public bool AddArrayToSubItem<TItem>(string field, List<TItem> values)
        {
            try
            {
                _allUpdateDefinition.Add(_update.AddToSetEach<TItem>(field, values));
                return true;
            }
            catch (Exception ex)
            {
                Logger.WriteLineError($"MongoCustomerUpdate {new StackTrace().GetFrame(0)} error {ex}");
                return false;
            }
        }

        /// <summary>
        /// 此函数将项目数组附加到文档中的子项目
        /// </summary>
        /// <param name="field">文档中的字段名称</param>
        /// <param name="values">列表<TItem></param>
        /// <returns>
        /// 一个布尔值。
        /// </returns>
        public bool PushArrayToSubItem<TItem>(string field, List<TItem> values)
        {
            try
            {
                _allUpdateDefinition.Add(_update.PushEach<TItem>(field, values));
                return true;
            }
            catch (Exception ex)
            {
                Logger.WriteLineError($"MongoCustomerUpdate {new StackTrace().GetFrame(0)} error {ex}");
                return false;
            }
        }


        /// <summary>
        /// 此函数从子项中删除一组项
        /// </summary>
        /// <param name="field">文档中的字段名称</param>
        /// <param name="values">列表<TItem></param>
        /// <returns>
        /// 一个布尔值。
        /// </returns>
        public bool RemoveArrayToSubItem<TItem>(string field, List<TItem> values)
        {
            try
            {
                _allUpdateDefinition.Add(_update.PullAll<TItem>(field, values));
                return true;
            }
            catch (Exception ex)
            {
                Logger.WriteLineError($"MongoCustomerUpdate {new StackTrace().GetFrame(0)} error {ex}");
                return false;
            }
        }

        /// <summary>
        /// 组合更新条件
        /// </summary>
        /// <param name="combineType">组合方式</param>
        /// <param name="filters">过滤条件集合</param>
        public bool CombineUpdate(params UpdateDefinition<T>[] updates)
        {
            bool isSuccess = false;
            try
            {
                _update.Combine(updates);
            }
            catch (Exception ex)
            {
                Logger.WriteLineError($"CombineUpdate error {ex}");
            }
            return isSuccess;
        }

        /// <summary>
        /// 清除更新器
        /// </summary>
        public void ClearUpdate()
        {
            _allUpdateDefinition.Clear();
        }
    }
}