﻿using System;
using System.Collections.Concurrent;
using System.ComponentModel;
using System.IO;
using System.Threading;
using System.Threading.Tasks;
using MongoDB.Bson;
using MongoDB.Bson.Serialization;
using MongoDB.Bson.Serialization.Attributes;
using MongoDB.Driver;
using WingMongoDBModule.DO;
using WingMongoDBModule.Entity;
using WingMongoDBModule.Repositories;
using WingServerCommon.Log;
using WingMongoDBModule.DBSync;
using System.Collections.Generic;
using WingServerCommon.Config;
using WingServerCommon.Config.Parameters;
using WingServerCommon.Interfaces.OpLog;
using System.Linq;
using WingInterfaceLibrary.OpLog;
using System.Reflection;
using WingMongoDBModule.Entities;
using Newtonsoft.Json;

namespace WingMongoDBModule.Mongodb.Driver
{
    /// <summary>
    /// 仓储中心创建者
    /// </summary>
    public class MongoRepertoryCenter
    {
        private ConcurrentDictionary<string, DataTypeItem> _dataTypeDic = new();
        private static MongoRepertoryCenter _instance;
        private MongoClient _single;
        private bool _isDistributed;
        private string _dbName;

        public event EventHandler<OperationLog> ExecuteOpLogEvent;

        public Func<string, Task<bool>> TokenRemove;

        /// <summary>
        /// 单例对象
        /// </summary>
        /// <value></value>
        internal static MongoRepertoryCenter Instance
        {
            get => _instance ?? (_instance = new MongoRepertoryCenter());
        }

        /// <summary>
        /// 创建DB客户端仓储
        /// </summary>
        /// <param name="host">主机ip</param>
        /// <param name="port">端口</param>
        /// <param name="dbName">db名称</param>
        public void Init(string host, int port, string dbName, bool isDistributed = false)
        {
            try
            {
                _dbName = dbName;
                _isDistributed = isDistributed;
                _single = MongoClientFactory.CreateClient(host, port);
            }
            catch (System.Exception ex)
            {
                Console.WriteLine($"MongoRepertoryCenter init error {ex}");
            }
        }


        public IMongoDatabase GetDatabase(string dbName)
        {
            return _single.GetDatabase(dbName);
        }

        private MongoClient CreateClient(string host, int port)
        {
            MongoClientSettings mongoClientSettings = new MongoClientSettings();
            mongoClientSettings.ConnectTimeout = TimeSpan.FromSeconds(60);
            mongoClientSettings.Server = new MongoServerAddress(host, port);
            var client = new MongoClient(mongoClientSettings);
            return client;
        }

        public bool MongodServerIsRunning()
        {
            try
            {
                var collections = GetDatabase(_dbName).RunCommand(new JsonCommand<BsonDocument>("{ listCollections: 1 }"));
                if (collections.ElementCount > 0)
                {
                    return true;
                }
                return false;
            }
            catch
            {
                return false;
            }
        }


        /// <summary>
        /// 用于判定是否初次部署
        /// </summary>
        /// <returns></returns>
        public bool DatabaseExist()
        {
            if (GetDatabase(_dbName) != null)
            {
                try
                {
                    var res = GetDatabase(_dbName).ListCollectionNames().Any();
                    return res;
                }
                catch (Exception ex)
                {
                    Console.WriteLine(ex);
                    Console.WriteLine("ListCollectionNames error , try again!");
                    var res = GetDatabase(_dbName).ListCollectionNames().Any();
                    return res;
                }
            }
            Logger.WriteLineError($"MongoDatabase is null");
            return false;
        }

        /// <summary>
        /// > 从缓存中获取集合，如果不存在则创建并添加到缓存中
        /// </summary>
        /// <returns>
        /// MongoCollectionProxy<MongoTable>
        /// </returns>
        internal IMongoCollection<MongoTable> GetCollection(string typeName)
        {
            IMongoCollection<MongoTable> collection = null;
            if (_isDistributed)
            {
                collection = GetDatabase(_dbName).GetCollectionProxy<MongoTable>(typeName);
            }
            else
            {
                collection = GetDatabase(_dbName).GetCollection<MongoTable>(typeName);
            }
            return collection;
        }

        /// <summary>
        /// > 从缓存中获取集合，如果不存在则创建并添加到缓存中
        /// </summary>
        /// <returns>
        /// MongoCollectionProxy<T>
        /// </returns>
        internal IMongoCollection<T> GetCollection<T>(string typeName)
        {
            IMongoCollection<T> collection = null;
            if (_isDistributed)
            {
                collection = GetDatabase(_dbName).GetCollectionProxy<T>(typeName);
            }
            else
            {
                collection = GetDatabase(_dbName).GetCollection<T>(typeName);
            }
            return collection;
        }


        /// <summary>
        /// > 从数据库中获取操作日志列表
        /// </summary>
        /// <param name="GetOpLogsFormMasterRequest">这是一个包含传递给 GetOpLogsAsync 方法的参数的类。</param>
        /// <returns>
        /// OperationLog 对象的列表。
        /// </returns>
        internal async Task<List<OperationLog>> GetOpLogsAsync(GetOpLogsFormMasterRequest request)
        {
            return await MasterOperationLogDbClient.Instance.GetManyAsync(request.Cursor, request.SourceUrl);
        }

        /// <summary>
        /// 根据业务编码从数据库中获取操作日志列表
        /// </summary>
        /// <param name="request">这是一个包含传递给 GetOpLogsByCodesFormMasterRequest 方法的参数的类。</param>
        /// <returns>
        /// OperationLog 对象的列表。
        /// </returns>
        internal async Task<List<OperationLog>> GetOpLogsByCodesAsync(GetOpLogsByCodesFormMasterRequest request)
        {
            return await MasterOperationLogDbClient.Instance.GetManyByCodesAsync(request.Codes);
        }


        /// <summary>
        /// 该函数用于同步MongoDB数据库的数据
        /// </summary>
        /// <param name="OperationLog">日志的数据结构</param>
        internal async Task<bool> ReceiveBsonLog(OperationLog opLog)
        {
            try
            {
                if (opLog.IsSimple)
                {
                    if (opLog.CollectionName == "Schedules")
                    {
                        return await ReceiveBsonLog<ScheduleEntity>(opLog);
                    }
                    else if (opLog.CollectionName == "GradingProtectionConfigs")
                    {
                        return await ReceiveBsonLog<GradingProtectionConfigEntity>(opLog);
                    }
                    else if (opLog.CollectionName == "ConsultationRecords")
                    {
                        return await ReceiveBsonLog<ConsultationRecordEntity>(opLog);
                    }
                    else if (opLog.CollectionName == "ConsultationEvaluateInfo")
                    {
                        return await ReceiveBsonLog<ConsultationEvaluateEntity>(opLog);
                    }
                    else if (opLog.CollectionName == "FollowUpVisitInfo")
                    {
                        return await ReceiveBsonLog<FollowUpVisitEntity>(opLog);
                    }
                    else if (opLog.CollectionName == "LiveRooms")
                    {
                        return await ReceiveBsonLog<LiveRoomEntity>(opLog);
                    }
                    else if (opLog.CollectionName == "MessageInfos")
                    {
                        return await ReceiveBsonLog<MessageInfoEntity>(opLog);
                    }
                    else if (opLog.CollectionName == "DiagnosisResultInfos")
                    {
                        return await ReceiveBsonLog<DiagnosisResultEntity>(opLog);
                    }
                    else if (opLog.CollectionName == "AnnouncementInfos")
                    {
                        return await ReceiveBsonLog<AnnouncementInfoEntity>(opLog);
                    }
                    else if (opLog.CollectionName == "BrandModelOutputConfigs")
                    {
                        return await ReceiveBsonLog<BrandModelOutputConfig>(opLog);
                    }
                    else if (opLog.CollectionName == "Courses")
                    {
                        return await ReceiveBsonLog<CourseInfoEntity>(opLog);
                    }
                    else if (opLog.CollectionName == "CourseAlbums")
                    {
                        return await ReceiveBsonLog<CourseAlbumEntity>(opLog);
                    }
                    else if (opLog.CollectionName == "CourseLabels")
                    {
                        return await ReceiveBsonLog<CourseLabelEntity>(opLog);
                    }
                    else if (opLog.CollectionName == "DeviceOutputConfigs")
                    {
                        return await ReceiveBsonLog<DeviceOutputConfig>(opLog);
                    }
                    else if (opLog.CollectionName == "DiagnosisResultInfos")
                    {
                        return await ReceiveBsonLog<DiagnosisResultEntity>(opLog);
                    }
                    else if (opLog.CollectionName == "ExaminationPapers")
                    {
                        return await ReceiveBsonLog<ExaminationPaperEntity>(opLog);
                    }
                    else if (opLog.CollectionName == "ExaminationQuestions")
                    {
                        return await ReceiveBsonLog<ExaminationQuestionEntity>(opLog);
                    }
                    else if (opLog.CollectionName == "Orders")
                    {
                        return await ReceiveBsonLog<OrderEntity>(opLog);
                    }
                    else if (opLog.CollectionName == "PaymentRecords")
                    {
                        return await ReceiveBsonLog<PaymentRecordEntity>(opLog);
                    }
                    else if (opLog.CollectionName == "Products")
                    {
                        return await ReceiveBsonLog<ProductEntity>(opLog);
                    }
                    else if (opLog.CollectionName == "RemoteExaminationDatas")
                    {
                        return await ReceiveBsonLog<RemoteExaminationDataEntity>(opLog);
                    }
                    else if (opLog.CollectionName == "StudentExaminations")
                    {
                        return await ReceiveBsonLog<StudentExaminationEntity>(opLog);
                    }
                    else if (opLog.CollectionName == "UserGroups")
                    {
                        return await ReceiveBsonLog<UserGroupEntity>(opLog);
                    }
                    else if (opLog.CollectionName == "Videos")
                    {
                        return await ReceiveBsonLog<VideoInfoEntity>(opLog);
                    }
                    else if (opLog.CollectionName == "LabFolders")
                    {
                        return await ReceiveBsonLog<LabFolderEntity>(opLog);
                    }
                    else if (opLog.CollectionName == "LabUploadRecords")
                    {
                        return await ReceiveBsonLog<LabUploadRecordEntity>(opLog);
                    }
                    else if (opLog.CollectionName == "DevicePatchList")
                    {
                        return await ReceiveBsonLog<DevicePatchEntity>(opLog);
                    }
                    else if (opLog.CollectionName == "DevicePatchVersions")
                    {
                        return await ReceiveBsonLog<DevicePatchVersionEntity>(opLog);
                    }
                    else if (opLog.CollectionName == "DevicePrinters")
                    {
                        return await ReceiveBsonLog<DevicePrinterEntity>(opLog);
                    }
                    else if (opLog.CollectionName == "RemedicalAISelectedInfos")
                    {
                        return await ReceiveBsonLog<RemedicalAISelectedInfoEntity>(opLog);
                    }
                    else if (opLog.CollectionName == "SystemConfigs")
                    {
                        return await ReceiveBsonLog<SystemConfigEntity>(opLog);
                    }
                    else if (opLog.CollectionName == "ChatMessages")
                    {
                        return await ReceiveBsonLog<ChatMessageEntity>(opLog);
                    }
                    else if (opLog.CollectionName == "ReferralDatas")
                    {
                        return await ReceiveBsonLog<ReferralDataEntity>(opLog);
                    }
                    else if (opLog.CollectionName == "MeasureExportRecords")
                    {
                        return await ReceiveBsonLog<MeasureExportRecordEntity>(opLog);
                    }
                    else if (opLog.CollectionName == "Packages")
                    {
                        return await ReceiveBsonLog<PackagesEntity>(opLog);
                    }
                    else if (opLog.CollectionName == "ProjectInfos")
                    {
                        return await ReceiveBsonLog<ProjectInfoEntity>(opLog);
                    }
                    else if (opLog.CollectionName == "ProjectPushRecords")
                    {
                        return await ReceiveBsonLog<ProjectPushRecordEntity>(opLog);
                    }
                    else if (opLog.CollectionName == "CourseBrowsedRecords")
                    {
                        return await ReceiveBsonLog<CourseBrowsedRecordEntity>(opLog);
                    }
                    else if (opLog.CollectionName == "DeviceReports")
                    {
                        return await ReceiveBsonLog<DeviceReportEntity>(opLog);
                    }
                    else if (opLog.CollectionName == "RegisterPersonInfos")
                    {
                        return await ReceiveBsonLog<RegisterPersonInfoEntity>(opLog);
                    }
                    else if (opLog.CollectionName == "HealthExamBookingInfos")
                    {
                        return await ReceiveBsonLog<HealthExamBookingEntity>(opLog);
                    }
                    else if (opLog.CollectionName == "IOTDeviceVersions")
                    {
                        return await ReceiveBsonLog<IOTDeviceVersionEntity>(opLog);
                    }
                    else if (opLog.CollectionName == "UserDeviceExtendInfos")
                    {
                        return await ReceiveBsonLog<UserDeviceExtendInfoEntity>(opLog);
                    }
                    else if (opLog.CollectionName == "CommonDictionarys")
                    {
                        return await ReceiveBsonLog<CommonDictionaryEntity>(opLog);
                    }
                    else if (opLog.CollectionName == "Tasks")
                    {
                        return await ReceiveBsonLog<TaskEntity>(opLog);
                    }
                    else if (opLog.CollectionName == "ResearchProjects")
                    {
                        return await ReceiveBsonLog<ResearchProjectEntity>(opLog);
                    }
                    else if (opLog.CollectionName == "LicenseInfos")
                    {
                        return await ReceiveBsonLog<LicenseInfoEntity>(opLog);
                    }
                    else if (opLog.CollectionName == "DongleInfos")
                    {
                        return await ReceiveBsonLog<DongleInfoEntity>(opLog);
                    }
                    else if (opLog.CollectionName == "ChargeResourcePackages")
                    {
                        return await ReceiveBsonLog<ChargeResourcePackageEntity>(opLog);
                    }
                    else if (opLog.CollectionName == "MultiPartUploadInfos")
                    {
                        return await ReceiveBsonLog<MultiPartUploadInfoEntity>(opLog);
                    }
                    else if (opLog.CollectionName == "UserUrmConfigs")
                    {
                        return await ReceiveBsonLog<UserUrmConfigEntity>(opLog);
                    }

                    return false;
                }
                else
                {
                    return await ReceiveBsonLog<MongoTable>(opLog);

                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineError($"WingMongoDBModule ReceiveBsonLog error {ex.ToString()} CollectionName:{opLog.CollectionName}, ActionType:{opLog.ActionType}, BsonContent:{opLog.BsonContent.Substring(0, opLog.BsonContent.Length > 2000 ? 2000 : opLog.BsonContent.Length)}");
                return false;
            }
        }

        async Task<bool> ReceiveBsonLog<T>(OperationLog opLog)
        {
            try
            {
                //获取数据集引用对象
                var entityType = typeof(T);
                var collection = GetDatabase(_dbName).GetCollection<T>(opLog.CollectionName);
                MongoDBActionTypeEnum operateType = opLog.ActionType;
                switch (operateType)
                {
                    case MongoDBActionTypeEnum.InsertOneAsync:
                        {
                            //获取BsonDocument类型，BsonDocument为数据实体内容
                            if (BsonDocument.TryParse(opLog.BsonContent, out BsonDocument document))
                            {
                                var bsonEntity = BsonSerializer.Deserialize(document, entityType);
                                var mongoTableDocument = (T)bsonEntity;
                                await collection.InsertOneAsync(mongoTableDocument);
                                if (ConfigurationManager.IsMaster)
                                {
                                    MongoDBLogTracker.Instance.Record(MongoDBLogFactory.CreateInsertOne<T>(opLog.CollectionName, mongoTableDocument, true), opLog.IsSimple, opLog.SourceUrl);
                                }
                                ExecuteOpLogEvent?.Invoke(null, opLog);
                                if (!string.IsNullOrWhiteSpace(opLog.Code))
                                {
                                    DatabaseCollection.RaiseCollectionDocumentChanged(new CollectionDocumentChangedArgs() { CollectionName = opLog.CollectionName, DocumentIds = opLog.Code.Split(',') });
                                }
                                if (opLog.CollectionName == "Tokens")
                                {
                                    try
                                    {
                                        await TokenRemove(opLog.Code);
                                    }
                                    catch (Exception ex)
                                    {

                                    }
                                }
                            }
                        }

                        break;
                    case MongoDBActionTypeEnum.UpdateOneAsync:
                    case MongoDBActionTypeEnum.UpdateManyAsync:
                    case MongoDBActionTypeEnum.DeleteOneAsync:
                        BsonDocument.TryParse(opLog.BsonContent, out BsonDocument updateBson);
                        BsonDocument.TryParse(opLog.FilterContent, out BsonDocument updateFilterBson);
                        var filter = Builders<T>.Filter.And(updateFilterBson);
                        var update = Builders<T>.Update.Combine(updateBson);
                        switch (opLog.ActionType)
                        {
                            case MongoDBActionTypeEnum.UpdateOneAsync:
                                await collection.UpdateOneAsync(filter, update);
                                if (ConfigurationManager.IsMaster)
                                {
                                    MongoDBLogTracker.Instance.Record(MongoDBLogFactory.CreateUpdateOne<T>(opLog.CollectionName, filter, update, true), opLog.IsSimple, opLog.SourceUrl);
                                }
                                ExecuteOpLogEvent?.Invoke(null, opLog);
                                if (!string.IsNullOrWhiteSpace(opLog.Code))
                                {
                                    DatabaseCollection.RaiseCollectionDocumentChanged(new CollectionDocumentChangedArgs() { CollectionName = opLog.CollectionName, DocumentIds = opLog.Code.Split(',') });
                                }
                                break;
                            case MongoDBActionTypeEnum.UpdateManyAsync:
                                await collection.UpdateManyAsync(filter, update);
                                if (ConfigurationManager.IsMaster)
                                {
                                    MongoDBLogTracker.Instance.Record(MongoDBLogFactory.CreateUpdateMany<T>(opLog.CollectionName, filter, update, true), opLog.IsSimple, opLog.SourceUrl);
                                }
                                if (!string.IsNullOrWhiteSpace(opLog.Code))
                                {
                                    DatabaseCollection.RaiseCollectionDocumentChanged(new CollectionDocumentChangedArgs() { CollectionName = opLog.CollectionName, DocumentIds = opLog.Code.Split(',') });
                                }
                                break;
                            case MongoDBActionTypeEnum.DeleteOneAsync:
                                await collection.DeleteOneAsync(filter);
                                if (ConfigurationManager.IsMaster)
                                {
                                    MongoDBLogTracker.Instance.Record(MongoDBLogFactory.CreateDeleteOne(opLog.CollectionName, filter, true), opLog.IsSimple, opLog.SourceUrl);
                                }
                                if (!string.IsNullOrWhiteSpace(opLog.Code))
                                {
                                    DatabaseCollection.RaiseCollectionDocumentChanged(new CollectionDocumentChangedArgs() { CollectionName = opLog.CollectionName, DocumentIds = opLog.Code.Split(',') });
                                }
                                break;
                        }
                        break;
                    default:
                        Logger.WriteLineError($"SyncBsonLog not supply this method!{operateType}");
                        break;
                }
                return true;
            }
            catch (Exception ex)
            {
                if (!ex.ToString().Contains("E11000 duplicate key error"))
                {
                    Logger.WriteLineError($"WingMongoDBModule ReceiveBsonLog<T> error {ex.ToString()} CollectionName:{opLog.CollectionName}, ActionType:{opLog.ActionType}, BsonContent:{opLog.BsonContent.Substring(0, opLog.BsonContent.Length > 2000 ? 2000 : opLog.BsonContent.Length)}");
                }
                return false;
            }
        }


        /// <summary>
        /// 用于更新数据库中的数据。
        /// </summary>
        /// <param name="MongoTable">这是我用来在 MongoDB 中存储数据的数据结构。</param>
        /// <param name="MongoTable">这是我用来在 MongoDB 中存储数据的数据结构。</param>
        /// <param name="canNull">是否允许更新空值</param>
        internal void UpgradeData<T>(MongoTable old, MongoTable newData, bool canNull = false) where T : class, IBaseDO
        {
            var type = old.BaseData.GetType();
            _dataTypeDic.TryGetValue(type.Name, out DataTypeItem typeData);
            if (typeData == null)
            {
                var properties = type.GetProperties();
                if (properties != null)
                {
                    typeData = new DataTypeItem
                    {
                        TypeName = type.Name,
                        Properties = properties
                    };
                    _dataTypeDic.TryAdd(type.Name, typeData);
                }
            }
            var props = typeData.Properties;
            old.QueryData = null;
            var uniqueDic = new Dictionary<string, string>();
            foreach (var prop in props)
            {
                var propGetMethod = prop.GetGetMethod();
                var propValue = propGetMethod?.Invoke(old.BaseData, null);
                var propNewValue = propGetMethod?.Invoke(newData.BaseData, null);
                if (propNewValue != null && !propNewValue.Equals(propValue))
                {
                    bool needUpdate = true;
                    if (propNewValue is string propNewValueStr && string.IsNullOrWhiteSpace(propNewValueStr) && !canNull)
                    {
                        needUpdate = false;
                    }
                    if (needUpdate)
                    {
                        propValue = propNewValue;
                        var propSetMethod = prop.GetSetMethod();
                        propSetMethod?.Invoke(old.BaseData, new object[] { propValue });
                    }
                }
                var attributes = prop.GetCustomAttributes(false).ToList();
                if (attributes != null && attributes.Count > 0)
                {
                    var queryItemAttr = attributes.FirstOrDefault(attr => attr is QueryItemAttribute) as QueryItemAttribute;
                    if (queryItemAttr != null)
                    {
                        var queryItemKey = prop.Name;
                        var queryItemValue = propValue?.ToString();
                        if (old.QueryData == null)
                        {
                            old.QueryData = new MongoQuery();
                        }
                        old.QueryData.QueryItems.Add(new MongoDataItem
                        {
                            Key = queryItemKey,
                            Value = queryItemValue
                        });
                    }
                }
            }
        }


        /// <summary>
        /// 用于更新数据库中的数据。
        /// </summary>
        /// <param name="MongoTable">这是我用来在 MongoDB 中存储数据的数据结构。</param>
        /// <param name="MongoTable">这是我用来在 MongoDB 中存储数据的数据结构。</param>
        /// <param name="canNull">是否允许更新空值</param>
        internal void UpgradeData<T>(T old, T newData, bool canNull = false) where T : class, IBaseDO
        {
            var type = old.GetType();
            _dataTypeDic.TryGetValue(type.Name, out DataTypeItem typeData);
            if (typeData == null)
            {
                var properties = type.GetProperties();
                if (properties != null)
                {
                    typeData = new DataTypeItem
                    {
                        TypeName = type.Name,
                        Properties = properties
                    };
                    _dataTypeDic.TryAdd(type.Name, typeData);
                }
            }
            var props = typeData.Properties;

            var uniqueDic = new Dictionary<string, string>();
            foreach (var prop in props)
            {
                var propGetMethod = prop.GetGetMethod();
                var propValue = propGetMethod?.Invoke(old, null);
                var propNewValue = propGetMethod?.Invoke(newData, null);
                if (propNewValue != null && !propNewValue.Equals(propValue))
                {
                    bool needUpdate = true;
                    if (propNewValue is string propNewValueStr && string.IsNullOrWhiteSpace(propNewValueStr) && !canNull)
                    {
                        needUpdate = false;
                    }
                    if (needUpdate)
                    {
                        propValue = propNewValue;
                        var propSetMethod = prop.GetSetMethod();
                        propSetMethod?.Invoke(old, new object[] { propValue });
                    }
                }
            }
        }

        /// <summary>
        /// > 该函数用于获取类的自定义属性，然后获取自定义属性对应的属性值
        /// </summary>
        /// <param name="MongoTable">这是一个包含要插入数据库的数据的类。</param>
        /// <returns>
        /// 字典<string, string>
        /// </returns>
        internal Dictionary<string, string> EachCustomAttributes<T>(MongoTable data) where T : class, IBaseDO
        {
            var type = data.BaseData.GetType();
            _dataTypeDic.TryGetValue(type.Name, out DataTypeItem typeData);
            if (typeData == null)
            {
                var properties = type.GetProperties();
                if (properties != null)
                {
                    typeData = new DataTypeItem
                    {
                        TypeName = type.Name,
                        Properties = properties
                    };
                    _dataTypeDic.TryAdd(type.Name, typeData);
                }
            }
            var props = typeData.Properties;
            var uniqueDic = new Dictionary<string, string>();
            data.QueryData = new MongoQuery();
            data.QueryData.QueryItems = new HashSet<MongoDataItem>();
            foreach (var prop in props)
            {
                var attributes = prop.GetCustomAttributes(false).ToList();
                if (attributes != null && attributes.Count > 0)
                {
                    var queryItemAttr = attributes.FirstOrDefault(attr => attr is QueryItemAttribute) as QueryItemAttribute;
                    if (queryItemAttr != null)
                    {
                        var propGetMethod = prop.GetGetMethod();
                        var propValue = propGetMethod?.Invoke(data.BaseData, null);
                        var queryItemKey = prop.Name;
                        var queryItemValue = propValue?.ToString();
                        data.QueryData.QueryItems.Add(new MongoDataItem
                        {
                            Key = queryItemKey,
                            Value = queryItemValue
                        });
                    }

                    var uniqueItemAttr = attributes.FirstOrDefault(attr => attr is UniqueItemAttribute) as UniqueItemAttribute;
                    if (uniqueItemAttr != null)
                    {
                        var propGetMethod = prop.GetGetMethod();
                        var propValue = propGetMethod?.Invoke(data.BaseData, null);
                        var uniqueKey = prop.Name;
                        var uniqueValue = propValue?.ToString();
                        uniqueDic.Add(uniqueKey, uniqueValue);
                    }
                }
            }
            return uniqueDic;
        }

        /// <summary>
        /// > 该函数用于获取类的自定义属性，然后获取自定义属性对应的属性值
        /// </summary>
        /// <param name="MongoTable">这是一个包含要插入数据库的数据的类。</param>
        /// <returns>
        /// 字典<string, string>
        /// </returns>
        internal Dictionary<string, string> EachCustomAttributes<T>(T data) where T : class, IBaseDO
        {
            var type = data.GetType();
            _dataTypeDic.TryGetValue(type.Name, out DataTypeItem typeData);
            if (typeData == null)
            {
                var properties = type.GetProperties();
                if (properties != null)
                {
                    typeData = new DataTypeItem
                    {
                        TypeName = type.Name,
                        Properties = properties
                    };
                    _dataTypeDic.TryAdd(type.Name, typeData);
                }
            }
            var props = typeData.Properties;
            var uniqueDic = new Dictionary<string, string>();
            foreach (var prop in props)
            {
                var attributes = prop.GetCustomAttributes(false).ToList();
                if (attributes != null && attributes.Count > 0)
                {
                    var uniqueItemAttr = attributes.FirstOrDefault(attr => attr is UniqueItemAttribute) as UniqueItemAttribute;
                    if (uniqueItemAttr != null)
                    {
                        var propGetMethod = prop.GetGetMethod();
                        var propValue = propGetMethod?.Invoke(data, null);
                        if (propValue == null)
                        {
                            continue;
                        }
                        var uniqueKey = prop.Name;
                        var uniqueValue = propValue?.ToString();
                        uniqueDic.Add(uniqueKey, uniqueValue);
                    }
                }
            }
            return uniqueDic;
        }


        /// <summary>
        /// > 该函数用于设置MongoDB表的数据
        /// </summary>
        /// <param name="T">要存储在数据库中的数据类型</param>
        /// <param name="platformDatas">平台的数据，是一个字典，key是平台名称，value是平台的数据。</param>
        /// <param name="extensionData">这是不在数据表中，但存储在 MongoDB 中的数据。</param>
        /// <returns>
        /// 字典和 MongoTable 的元组。
        /// </returns>
        internal (Dictionary<string, string>, MongoTable) SetTableData<T>(T data, Dictionary<string, string> platformDatas, string extensionData) where T : class, IBaseDO
        {
            var type = data.GetType();
            _dataTypeDic.TryGetValue(type.Name, out DataTypeItem typeData);
            if (typeData == null)
            {
                var properties = type.GetProperties();
                if (properties != null)
                {
                    typeData = new DataTypeItem
                    {
                        TypeName = type.Name,
                        Properties = properties
                    };
                    _dataTypeDic.TryAdd(type.Name, typeData);
                }
            }
            if (typeData != null)
            {
                var uniqueItem = typeData.Properties.FirstOrDefault(m => m.Name == "Code" && m.CustomAttributes.Any(f => f.AttributeType.Name == "UniqueItemAttribute"));
                if (uniqueItem != null)
                {
                    var propValue = uniqueItem.GetValue(data);
                    if (propValue == null || (propValue is string propStr && string.IsNullOrWhiteSpace(propStr)))
                    {
                        uniqueItem.SetValue(data, CodeCreator.CreateCode(typeData.TypeName));
                    }
                }
            }
            var table = new MongoTable();
            table.BaseData = data;
            table.ExtendsData = extensionData;
            table.PlatformData = new MongoPlatform();
            table.QueryData = new MongoQuery();
            var uniqueDic = EachCustomAttributes<T>(table);
            foreach (var plat in platformDatas)
            {
                table.PlatformData.PlatformItems.Add(new MongoDataItem { Key = plat.Key, Value = plat.Value });
            }
            return (uniqueDic, table);
        }

        /// <summary>
        /// > 该函数用于设置MongoDB表的数据
        /// </summary>
        /// <param name="T">要存储在数据库中的数据类型</param>
        /// <param name="platformDatas">平台的数据，是一个字典，key是平台名称，value是平台的数据。</param>
        /// <param name="extensionData">这是不在数据表中，但存储在 MongoDB 中的数据。</param>
        /// <returns>
        /// 字典和 MongoTable 的元组。
        /// </returns>
        internal (Dictionary<string, string>, T) SetTableData<T>(T data) where T : class, IBaseDO
        {
            var type = data.GetType();
            _dataTypeDic.TryGetValue(type.Name, out DataTypeItem typeData);
            if (typeData == null)
            {
                var properties = type.GetProperties();
                if (properties != null)
                {
                    typeData = new DataTypeItem
                    {
                        TypeName = type.Name,
                        Properties = properties
                    };
                    _dataTypeDic.TryAdd(type.Name, typeData);
                }
            }
            if (typeData != null)
            {
                var uniqueItem = typeData.Properties.FirstOrDefault(m => m.Name == "Code" && m.CustomAttributes.Any(f => f.AttributeType.Name == "UniqueItemAttribute"));
                if (uniqueItem != null)
                {
                    var propValue = uniqueItem.GetValue(data);
                    if (propValue == null || (propValue is string propStr && string.IsNullOrWhiteSpace(propStr)))
                    {
                        uniqueItem.SetValue(data, CodeCreator.CreateCode(typeData.TypeName));
                    }
                }
            }
            var uniqueDic = EachCustomAttributes<T>(data);
            return (uniqueDic, data);
        }

    }





    public class DataTypeItem
    {
        /// <summary>
        /// 类型名称
        /// </summary>
        /// <value></value>
        public string TypeName { get; set; }

        /// <summary>
        /// 操作属性
        /// </summary>
        /// <value></value>
        public PropertyInfo[] Properties { get; set; }
    }
}
