﻿using System;
using System.Collections.Generic;
using MongoDB.Bson.Serialization.Attributes;
using WingInterfaceLibrary.DTO.Common;
using WingInterfaceLibrary.Enum;
using WingInterfaceLibrary.Enum.CommentEnum;
using WingInterfaceLibrary.Enum.NotificationEnum;
using WingInterfaceLibrary.LiveConsultation;
using WingMongoDBModule.Entities;
using WingMongoDBModule.Mongodb.Driver;
using WingMongoDBModule.DO;

namespace WingMongoDBModule.Entities
{
    /// <summary>
    /// 房间表
    /// </summary>
    public class LiveRoomEntity : BaseEntity
    {
        /// <summary>
        /// 房间编码
        /// </summary>
        /// <value></value>
        [BsonIgnore]
        public string LiveRoomCode
        {
            get
            {
                return Code;
            }
            set
            {
                Code = value;
            }
        }

        /// <summary>
        /// 关联业务编码
        /// </summary>
        [QueryItem]
        public string RelatedCode { get; set; } = string.Empty;

        /// <summary>
        /// 房间名
        /// </summary>
        [QueryItem]
        public string Name { get; set; }

        /// <summary>
        /// 发起人编码
        /// </summary>
        [QueryItem]
        public string InitiatorCode { get; set; } = string.Empty;

        /// <summary>
        /// RTC房间号
        /// </summary>
        public int RtcRoomId { get; set; }

        /// <summary>
        /// SdkAppId
        /// </summary>
        public int SdkAppId { get; set; }

        /// <summary>
        /// 业务类型
        /// </summary>
        [QueryItem]
        public BusinessModuleEnum BusinessModule { get; set; }

        /// <summary>
        /// 已接入设备集合
        /// </summary>
        public List<LiveMemberDO> DeviceInfos { get; set; } = new List<LiveMemberDO>();

        /// <summary>
        /// 已接入客户端用户集合
        /// </summary>
        public List<LiveMemberDO> UserInfos { get; set; } = new List<LiveMemberDO>();

        /// <summary>
        /// 直播观看人集合
        /// </summary>
        public List<LiveViewerDO> ViewerInfos { get; set; } = new List<LiveViewerDO>();

        /// <summary>
        /// The diagnosis room status
        /// </summary>
        /// <value>0</value>
        public LiveRoomStatus Status { get; set; }
    }

    /// <summary>
    /// 直播用户信息
    /// </summary>
    public class LiveMemberDO
    {
        private string _headImageToken;
        /// <summary>
        /// 设备/用户编码
        /// </summary>
        public string Code { get; set; } = string.Empty;
        /// <summary>
        /// 用户类型
        /// </summary>
        public LiveMemberEnum MemberType { get; set; }
        /// <summary>
        /// 用户姓名
        /// </summary>
        public string Name { get; set; } = string.Empty;
        /// <summary>
        /// 头像
        /// </summary>
        /// <value></value>
        public string HeadImageToken 
        {
            get
            {
                return DOShare.ChangeUrl(_headImageToken);
            }
            set
            {
                _headImageToken = value;
            }
        }
        /// <summary>
        /// 状态
        /// </summary>
        public LiveMemberStatus Status { get; set; }
        /// <summary>
        /// 是否静音
        /// </summary>
        public bool MuteOpened { get; set; } = false;
        /// <summary>
        /// 是否开启视频
        /// </summary>
        public bool VideoOpened { get; set; }
        /// <summary>
        /// 实时通道数据或旁路实时通道数据（用于RTC）
        /// </summary>
        public LiveDataDO LiveData { get; set; }
        /// <summary>
        /// 是否正在控制参数
        /// </summary>
        /// <value></value>
        public bool IsControllingParameter { get; set; }
        /// <summary>
        /// 首次加入时间
        /// </summary>
        /// <value></value>
        public DateTime FirstJoinTime { get; set; } = DateTime.UtcNow;
        /// <summary>
        /// 排序等级
        /// </summary>
        /// <value></value>
        public int SortLevel { get; set; }
        /// <summary>
        /// 设备画面信息
        /// </summary>
        /// <typeparam name="VideoDeviceInfoDO"></typeparam>
        /// <returns></returns>
        public List<VideoDeviceInfoDO> VideoDeviceInfos { get; set; } = new List<VideoDeviceInfoDO>();
        /// <summary>
        /// 用户来源
        /// </summary>
        /// <value></value>
        public LoginSource LoginSource { get; set; }

        /// <summary>
        /// 邀请人
        /// </summary>
        /// <value></value>
        public string InviterCode { get; set; }

        /// <summary>
        /// 是否后台管理角色-用于只加入房间 自己无摄像头画面无语音，用于课程助教或者会诊管理员角色
        /// </summary>
        /// <value></value>
        public bool BackgroundRole { get; set; }
    }

    /// <summary>
    /// 直播观看人信息
    /// </summary>
    public class LiveViewerDO
    {

        /// <summary>
        /// 用户编码
        /// </summary>
        public string UserCode { get; set; } = string.Empty;

        /// <summary>
        /// 用户姓名
        /// </summary>
        public string Name { get; set; } = string.Empty;

        /// <summary>
        /// 最近上报的观看时间
        /// </summary>
        public DateTime LastReportTime { get; set; }
    }

    /// <summary>
    /// 设备画面信息
    /// </summary>
    public class VideoDeviceInfoDO
    {
        /// <summary>
        /// 设备Id（比如超声机唯一码，用来表示这是超声机画面）
        /// </summary>
        /// <value></value>
        public string VideoDeviceId { get; set; }

        /// <summary>
        /// 视屏画面设备来源的类型（设备主屏或摄像头）
        /// </summary>
        /// <value></value>
        public VideoDeviceSourceTypeEnum VideoDeviceSourceType { get; set; }

        /// <summary>
        /// 实时通道数据或旁路实时通道数据（用于RTC）
        /// </summary>
        public LiveDataDO LiveData { get; set; }
    }

    /// <summary>
    /// 实时通道数据或旁路实时通道数据（用于RTC）	
    /// </summary>
    public class LiveDataDO
    {
        /// <summary>
        /// The channel image height
        /// </summary>
        /// <value></value>
        public int Height { get; set; }
        /// <summary>
        /// The channel image width
        /// </summary>
        /// <value></value>
        public int Width { get; set; }
        /// <summary>
        /// Rtmp push url
        /// </summary>
        /// <value></value>
        public string RtmpPushUrl { get; set; } = string.Empty;
        /// <summary>
        /// Rtmp pull url
        /// </summary>
        /// <value></value>
        public string RtmpPullUrl { get; set; } = string.Empty;
        /// <summary>
        /// Http pull url
        /// </summary>
        /// <value></value>
        public string HttpPullUrl { get; set; } = string.Empty;
        /// <summary>
        /// Hls pull url for apple platform
        /// </summary>
        /// <value></value>    
        public string HlsPullUrl { get; set; } = string.Empty;
    }
}
