﻿using System.Text.RegularExpressions;
using WingMongoDBModule.Entity;
using System.Collections.Generic;
using System.Threading.Tasks;
using System.ComponentModel;
using MongoDB.Bson;
using MongoDB.Bson.Serialization;
using MongoDB.Bson.Serialization.Attributes;
using MongoDB.Driver;
using System.Linq;
using System;
using WingMongoDBModule.Mongodb.Driver;
using WingMongoDBModule.DO;
using WingInterfaceLibrary.Request;
using System.Threading;
using JsonRpcLite.Services;
using WingMongoDBModule.Request;
using WingInterfaceLibrary.Enum;
using WingServerCommon.Config;
using WingServerCommon.Config.Parameters;
using WingServerCommon.Utilities;
using WingMongoDBModule.DBSync;
using WingServerCommon.Log;

namespace WingMongoDBModule.Repositories
{
    public class BaseDBRepository<T> where T : class, IBaseDO
    {
        private int _performanceThreshold = ConfigurationManager.GetParammeter<IntParameter>("Log", "DBPerformance").Value;
        private bool _isDistributed = ConfigurationManager.GetParammeter<BoolParameter>("General", "IsDistributed").Value;
        private IMongoCollection<MongoTable> _collect;
        public string CollectionName { get; }

        public BaseDBRepository(string typeName, bool registerType = true)
        {
            if (registerType)
            {
                RegisterDBType(typeof(T));
            }
            CollectionName = typeName;
            _collect = MongoRepertoryCenter.Instance.GetCollection(typeName);
        }

        /// <summary>
        /// 创建索引
        /// </summary>
        /// <remarks>仅初次部署和db更新使用</remarks>
        public void CreateIndex()
        {
            var options = new CreateIndexOptions() { Unique = false, Background = true, Name = "QueryItemsIndex" };
            var queryIndex = new IndexKeysDefinitionBuilder<MongoTable>().Descending("QueryData.QueryItems.Key").Descending("QueryData.QueryItems.Value");
            _collect.Indexes.CreateOne(new CreateIndexModel<MongoTable>(queryIndex, options));
        }

        /// <summary>
        /// 注册DB类型
        /// </summary>
        /// <param name="lookupType">注册类型</param>
        private void RegisterDBType(Type lookupType)
        {
            TypeDescriptor.AddAttributes(lookupType, new BsonIgnoreExtraElementsAttribute());
            BsonClassMap.LookupClassMap(lookupType);
        }

        /// <summary>
        /// It updates the data in the database.
        /// </summary>
        /// <param name="key">the name of the field to be updated</param>
        /// <param name="value">the value of the key</param>
        /// <param name="T">The type of the data to be stored in the database.</param>
        /// <param name="extensionData">This is the data that will be stored in the MongoDB database.</param>
        /// <param name="canNull">If the data is null, whether to delete the data</param>
        /// <param name="isWriteOperationLog">add oplog</param>
        public virtual async Task<bool> UpdateOneAsync(string key, string value, T data, string extensionData, bool canNull = false, bool isWriteOperationLog = true, bool onlyNotDeleted = true)
        {
            var table = new MongoTable();
            if (data != null)
            {
                table.BaseData = data;
                if (!string.IsNullOrWhiteSpace(extensionData))
                {
                    table.ExtendsData = extensionData;
                }
                return await Update(key, value, table, canNull, isWriteOperationLog, onlyNotDeleted);
            }
            return false;
        }

        /// <summary>
        /// It updates the data in the database.
        /// </summary>
        /// <param name="key">the name of the field to be updated</param>
        /// <param name="value">the value of the key</param>
        /// <param name="T">The type of the data to be stored in the database.</param>
        /// <param name="extensionData">This is the data that will be stored in the MongoDB database.</param>
        /// <param name="canNull">If the data is null, whether to delete the data</param>
        /// <param name="isWriteOperationLog">add oplog</param>
        public virtual async Task<bool> UpdateOneForSyncDBAsync(string key, string value, T data, string extensionData, bool canNull = false, bool isWriteOperationLog = true)
        {
            if (data != null)
            {
                var (uniqueDic, table) = MongoRepertoryCenter.Instance.SetTableData(data, new Dictionary<string, string>(), extensionData);
                table.BaseData = data;
                if (!string.IsNullOrWhiteSpace(extensionData))
                {
                    table.ExtendsData = extensionData;
                }
                return await UpdateForSync(key, value, table, canNull, isWriteOperationLog);
            }
            return false;
        }

        /// <summary>
        /// A function that updates the data in the database.
        /// </summary>
        /// <param name="key">the key of the data item</param>
        /// <param name="value">the value of the key</param>
        /// <param name="MongoTable">The data type of the data to be updated</param>
        /// <param name="canNull">Whether to allow null values to be updated</param>
        ///  <param name="isWriteOperationLog">add oplog</param>
        /// <returns>
        /// A boolean value.
        /// </returns>
        public async Task<bool> UpdateForSync(string key, string value, MongoTable data, bool canNull = false, bool isWriteOperationLog = true)
        {
            using (new Performance(_performanceThreshold, "Handle UpdateForSync {0} execute", $"{typeof(T).FullName}"))
            {
                var customerFilter = new MongoCustomerFilter<MongoTable>();
                var mongoDataItemFilter = new MongoCustomerFilter<MongoDataItem>();
                mongoDataItemFilter.AppendEq((item => item.Key, key), (item => item.Value, value));
                customerFilter.SubFilter(item => item.QueryData.QueryItems, mongoDataItemFilter.AndCombine());
                mongoDataItemFilter.ClearFilter();
                MongoCustomerUpdate<MongoTable> update = new();
                update.SetField(item => item.QueryData, data.QueryData);
                update.SetField(item => item.BaseData, data.BaseData);
                update.SetField(item => item.ExtendsData, data.ExtendsData);
                if (isWriteOperationLog || !_isDistributed)
                {
                    var result = await _collect.UpdateOneAsync(customerFilter.AndCombine(), update.AllUpdateDefinition);
                    return true;
                }
                else
                {
                    var result = await ((MongoCollectionProxy<MongoTable>)_collect).UpdateOneWithoutWriteLogAsync(customerFilter.AndCombine(), update.AllUpdateDefinition);
                    return true;
                }
            }
        }

        /// <summary>
        /// It updates a single document in the _collection.
        /// </summary>
        /// <param name="filter">The filter to select the document to update.</param>
        /// <param name="update">The update to apply to the document.</param>
        /// <returns>
        /// The number of documents modified.
        /// </returns>
        public async Task<long> UpdateOneAsync(FilterDefinition<MongoTable> filter, UpdateDefinition<MongoTable> update)
        {
            using (new Performance(_performanceThreshold, "Handle UpdateOneAsync {0} execute", $"{typeof(T).FullName}"))
            {
                if (filter != null && update != null)
                {
                    var result = await _collect.UpdateOneAsync(filter, update);
                    return result.ModifiedCount;
                }
                return 0;
            }
        }


        /// <summary>
        /// Update all the documents that match the filter with the update
        /// </summary>
        /// <param name="filter">The filter to select the documents to update.</param>
        /// <param name="update">The update to apply to the matching documents.</param>
        /// <returns>
        /// The number of documents that were modified.
        /// </returns>
        public async Task<long> UpdateAllAsync(FilterDefinition<MongoTable> filter, UpdateDefinition<MongoTable> update)
        {
            using (new Performance(_performanceThreshold, "Handle UpdateAllAsync {0} execute", $"{typeof(T).FullName}"))
            {
                if (filter != null && update != null)
                {
                    var result = await _collect.UpdateManyAsync(filter, update);
                    return result.ModifiedCount;
                }
                return 0;
            }
        }

        /// <summary>
        /// 更新操作
        /// </summary>
        /// <param name="info">包含一对Key/Value更新操作的数据</param>
        /// <returns></returns>
        public virtual async Task<long> UpdateAllAsync(List<(string key, string value, T data, string extensionData)> info)
        {
            var request = info.Select(c => (
                c.key,
                c.value,
                new MongoTable { BaseData = c.data, ExtendsData = c.extensionData }
            )).ToList();
            return await UpdateTableAll(request);
        }

        /// <summary>
        /// > Update the data in the database according to the specified conditions
        /// </summary>
        /// <param name="infos">a list of tuples, each tuple contains a key, a value, and a MongoTable
        /// object.</param>
        /// <returns>
        /// The number of documents that were modified.
        /// </returns>
        public async Task<long> UpdateTableAll(List<(string key, string value, MongoTable data)> infos)
        {
            using (new Performance(_performanceThreshold, "Handle UpdateTableAll {0} execute", $"{typeof(T).FullName}"))
            {
                var customerFilter = new MongoCustomerFilter<MongoTable>();
                var mongoDataItemFilter = new MongoCustomerFilter<MongoDataItem>();
                var updateCount = 0;
                foreach (var info in infos)
                {
                    mongoDataItemFilter.AppendEq((item => item.Key, info.key), (item => item.Value, info.value));
                    customerFilter.SubFilter(item => item.QueryData.QueryItems, mongoDataItemFilter.OrCombine());
                    mongoDataItemFilter.ClearFilter();
                }
                if (infos?.Count > 0)
                {
                    var filterDefinition = customerFilter.AndCombine();
                    var options = new FindOptions<MongoTable, MongoTable>
                    {
                        BatchSize = 1000
                    };
                    using (var cursor = await _collect.FindAsync(filterDefinition, options))
                    {
                        while (await cursor.MoveNextAsync())
                        {
                            var batch = cursor.Current;
                            foreach (var doc in batch)
                            {
                                var first = infos.FirstOrDefault(info => doc.QueryData.QueryItems.Select(f => $"{f.Key}_{f.Value}").Contains($"{info.key}_{info.value}"));
                                if (first.data == null) { continue; }
                                MongoCustomerUpdate<MongoTable> update = new();
                                MongoRepertoryCenter.Instance.UpgradeData<T>(doc, first.data);
                                update.SetField(item => item.QueryData, doc.QueryData);
                                update.SetField(item => item.BaseData, doc.BaseData);
                                update.SetField(item => item.ExtendsData, doc.ExtendsData);
                                MongoCustomerFilter<MongoTable> customerFilterUpdate = new();
                                MongoCustomerFilter<MongoDataItem> subFilter = new();
                                subFilter.AppendEq((item => item.Key, "Code"));
                                subFilter.AppendEq((item => item.Value, doc.BaseData.Code));
                                customerFilterUpdate.SubFilter(item => item.QueryData.QueryItems, subFilter.AndCombine());
                                await _collect.UpdateOneAsync(customerFilterUpdate.AndCombine(), update.AllUpdateDefinition);
                                updateCount++;
                            }
                        }
                    }
                }
                return updateCount;
            }
        }

        /// <summary>
        /// 获取堆栈
        /// </summary>
        /// <returns></returns>
        private string GetStackTraceModelName()
        {
            //当前堆栈信息
            System.Diagnostics.StackTrace st = new System.Diagnostics.StackTrace();
            System.Diagnostics.StackFrame[] sfs = st.GetFrames();
            //过虑的方法名称
            string _fullName = string.Empty, _methodName = string.Empty;
            for (int i = 1; i < sfs.Length; ++i)
            {
                //系统方法及后面的都是系统调用，不获取用户代码调用结束
                if (System.Diagnostics.StackFrame.OFFSET_UNKNOWN == sfs[i].GetILOffset()) break;
                var methedInfo = sfs[i].GetMethod();
                _methodName = methedInfo.ReflectedType.FullName + "." + methedInfo.Name;//方法名称
                _fullName = _methodName + "()\r\n->" + _fullName;
            }
            st = null;
            sfs = null;
            var content = _fullName.TrimEnd('-', '>');
            return content;
        }


        /// <summary>
        /// Find the first table that matches the key and value
        /// </summary>
        /// <param name="key">the key of the data item</param>
        /// <param name="value">the value of the key</param>
        /// <returns>
        /// A MongoTable object
        /// </returns>
        public async Task<T> FindOneAsync(string key, string value, bool onlyNotDeleted = true)
        {
            var logContent = "";
            var fullName = typeof(T).FullName;
            System.Diagnostics.Stopwatch sw = null;
            if (fullName.Contains("AdminInfoDO"))
            {
                logContent = GetStackTraceModelName();
                sw = new System.Diagnostics.Stopwatch();
                sw.Start();
            }
            using (new Performance(_performanceThreshold, "Handle FindOneAsync by key {0} execute " + logContent, $"{fullName}"))
            {
                if (string.IsNullOrWhiteSpace(key))
                {
                    throw new RpcException((int)CustomerRpcCode.ColumnNameIsEmpty, "key can't be null", "key can't be null");
                }
                if (string.IsNullOrWhiteSpace(value))
                {
                    Logger.WriteLineWarn($"{fullName} FindOneAsync failed, value is null");
                    throw new RpcException((int)CustomerRpcCode.ColumnValueIsEmpty, $"{key} can't be null", $"{key} can't be null");
                }
                var customerFilter = new MongoCustomerFilter<MongoTable>();
                var mongoDataItemFilter = new MongoCustomerFilter<MongoDataItem>();
                mongoDataItemFilter.AppendEq((item => item.Key, key), (item => item.Value, value));
                customerFilter.SubFilter(item => item.QueryData.QueryItems, mongoDataItemFilter.AndCombine());
                var response = new List<MongoTable>();
                var projection = new ProjectionDefinitionBuilder<MongoTable>();
                var baseData = projection.Expression(r => (T)r.BaseData);
                FindOptions<MongoTable, T> newOptions;
                newOptions = new FindOptions<MongoTable, T>()
                {
                    Projection = baseData,
                    NoCursorTimeout = true
                };
                if (onlyNotDeleted)
                {
                    MongoCustomerFilter<MongoDataItem> itemFilter = new();
                    itemFilter.AppendEq((item => item.Key, "IsDelete"), (item => item.Value, "False"));
                    customerFilter.SubFilter(item => item.QueryData.QueryItems, itemFilter.AndCombine());
                }
                var cursor = await _collect.FindAsync(customerFilter.AndCombine(), newOptions);
                if (fullName.Contains("AdminInfoDO"))
                {
                    sw.Stop();
                    if (sw.ElapsedMilliseconds > 100)
                    {
                        WingServerCommon.Log.Logger.WriteLineInfo($"FindOneAsync AdminInfoDO by key _collect.FindAsync end, spend " + sw.ElapsedMilliseconds);
                    }
                    sw.Restart();
                }
                var res = await cursor.FirstOrDefaultAsync();
                if (fullName.Contains("AdminInfoDO"))
                {
                    sw.Stop();
                    if (sw.ElapsedMilliseconds > 100)
                    {
                        WingServerCommon.Log.Logger.WriteLineInfo($"FindOneAsync AdminInfoDO by key FirstOrDefaultAsync end, spend " + sw.ElapsedMilliseconds);
                    }
                }
                return res;
            }
        }

        /// <summary>
        /// > Finds the first document in the _collection that matches the filter
        /// </summary>
        /// <param name="filter">The filter is a document that specifies the query criteria.</param>
        /// <param name="options"></param>
        /// <returns>
        /// A cursor.
        /// </returns>
        public async Task<MongoTable> FindTableOneAsync(FilterDefinition<MongoTable> filter, FindOptions<MongoTable> options = null, bool onlyNotDeleted = true)
        {
            using (new Performance(_performanceThreshold, "Handle FindTableOneAsync by filter {0} execute", $"{typeof(T).FullName}"))
            {
                if (onlyNotDeleted)
                {
                    MongoCustomerFilter<MongoTable> customerFilter = new();
                    MongoCustomerFilter<MongoDataItem> itemFilter = new();
                    itemFilter.AppendEq((item => item.Key, "IsDelete"), (item => item.Value, "False"));
                    customerFilter.SubFilter(item => item.QueryData.QueryItems, itemFilter.AndCombine());
                    filter = filter & customerFilter.AndCombine();
                }
                var cursor = await _collect.FindAsync(filter, options);
                return await cursor.FirstOrDefaultAsync();
            }
        }


        /// <summary>
        /// > Finds one record from the database and returns it as a strongly typed object
        /// </summary>
        /// <param name="filter">The filter to find the document.</param>
        /// <param name="options"></param>
        /// <returns>
        /// The result is a MongoTable object.
        /// </returns>
        public virtual async Task<T> FindOneAsync(FilterDefinition<MongoTable> filter, FindOptions<MongoTable> options = null, bool onlyNotDeleted = true)
        {
            using (new Performance(_performanceThreshold, "Handle FindOneAsync by filter {0} execute", $"{typeof(T).FullName}"))
            {
                var projection = new ProjectionDefinitionBuilder<MongoTable>();
                var baseData = projection.Expression(r => (T)r.BaseData);
                FindOptions<MongoTable, T> newOptions;
                if (options != null)
                {
                    newOptions = new FindOptions<MongoTable, T>()
                    {
                        Projection = baseData,
                        BatchSize = options.BatchSize,
                        Sort = options.Sort,
                        Skip = options.Skip,
                        Limit = options.Limit,
                        NoCursorTimeout = options.NoCursorTimeout
                    };
                }
                else
                {
                    newOptions = new FindOptions<MongoTable, T>()
                    {
                        Projection = baseData,
                        NoCursorTimeout = true
                    };
                }
                if (onlyNotDeleted)
                {
                    MongoCustomerFilter<MongoTable> customerFilter = new();
                    MongoCustomerFilter<MongoDataItem> itemFilter = new();
                    itemFilter.AppendEq((item => item.Key, "IsDelete"), (item => item.Value, "False"));
                    customerFilter.SubFilter(item => item.QueryData.QueryItems, itemFilter.AndCombine());
                    filter = filter & customerFilter.AndCombine();
                }
                var cursor = await _collect.FindAsync(filter, newOptions);
                return await cursor.FirstOrDefaultAsync();
            }
        }


        /// <summary>
        /// It counts the number of documents in the _collection.
        /// </summary>
        /// <param name="filter">The filter to apply the count operation to.</param>
        /// <returns>
        /// The number of documents in the _collection.
        /// </returns>
        public async Task<long> CountDocumentsAsync(FilterDefinition<MongoTable> filter = null, bool onlyNotDeleted = true)
        {
            using (new Performance(_performanceThreshold, "Handle CountDocumentsAsync {0} execute", $"{typeof(T).FullName}"))
            {
                var response = new List<MongoTable>();
                if (filter == null)
                {
                    filter = Builders<MongoTable>.Filter.Empty;
                }
                if (onlyNotDeleted)
                {
                    MongoCustomerFilter<MongoTable> customerFilter = new();
                    MongoCustomerFilter<MongoDataItem> itemFilter = new();
                    itemFilter.AppendEq((item => item.Key, "IsDelete"), (item => item.Value, "False"));
                    customerFilter.SubFilter(item => item.QueryData.QueryItems, itemFilter.AndCombine());
                    filter = filter & customerFilter.AndCombine();
                }
                return await _collect.CountAsync(filter);
            }
        }


        /// <summary>
        /// > Find all the documents in the _collection that match the filter, and return them as a list
        /// </summary>
        /// <param name="filter">The filter to apply to the query.</param>
        /// <param name="options"></param>
        /// <param name="CancellationToken">A cancellation token that can be used by other objects or threads to
        /// receive notice of cancellation.</param>
        /// <returns>
        /// A list of MongoTable objects.
        /// </returns>
        public async Task<List<T>> FindAllAsync(FilterDefinition<MongoTable> filter = null, FindOptions<MongoTable> options = null, CancellationToken cancellationToken = default, bool onlyNotDeleted = true)
        {
            using (new Performance(_performanceThreshold, "Handle FindAllAsync {0} execute", $"{typeof(T).FullName}"))
            {
                if (filter == null)
                {
                    filter = Builders<MongoTable>.Filter.Empty;
                }
                if (onlyNotDeleted)
                {
                    MongoCustomerFilter<MongoTable> customerFilter = new();
                    MongoCustomerFilter<MongoDataItem> itemFilter = new();
                    itemFilter.AppendEq((item => item.Key, "IsDelete"), (item => item.Value, "False"));
                    customerFilter.SubFilter(item => item.QueryData.QueryItems, itemFilter.AndCombine());
                    filter = filter & customerFilter.AndCombine();
                }
                var projection = new ProjectionDefinitionBuilder<MongoTable>();
                var baseData = projection.Expression(r => (T)r.BaseData);
                FindOptions<MongoTable, T> newOptions;
                if (options != null)
                {
                    newOptions = new FindOptions<MongoTable, T>()
                    {
                        Projection = baseData,
                        BatchSize = options.BatchSize,
                        Sort = options.Sort,
                        Skip = options.Skip,
                        Limit = options.Limit,
                        NoCursorTimeout = options.NoCursorTimeout
                    };
                }
                else
                {
                    newOptions = new FindOptions<MongoTable, T>()
                    {
                        Projection = baseData,
                        NoCursorTimeout = true
                    };
                }
                var cursor = await _collect.FindAsync(filter, newOptions, cancellationToken);
                return cursor.ToList();
            }
        }


        /// <summary>
        /// > Find all the documents in the _collection that match the filter, and return them as a list
        /// </summary>
        /// <param name="filter">The filter to apply to the query.</param>
        /// <param name="options"></param>
        /// <param name="CancellationToken">A cancellation token that can be used by other objects or threads to
        /// receive notice of cancellation.</param>
        /// <returns>
        /// A list of MongoTable objects.
        /// </returns>
        private async Task<List<MongoTable>> FindAllTableAsync(FilterDefinition<MongoTable> filter = null, FindOptions<MongoTable> options = null, CancellationToken cancellationToken = default, bool onlyNotDeleted = true)
        {
            using (new Performance(_performanceThreshold, "Handle FindAllTableAsync {0} execute", $"{typeof(T).FullName}"))
            {
                if (filter == null)
                {
                    filter = Builders<MongoTable>.Filter.Empty;
                }
                if (onlyNotDeleted)
                {
                    MongoCustomerFilter<MongoTable> customerFilter = new();
                    MongoCustomerFilter<MongoDataItem> itemFilter = new();
                    itemFilter.AppendEq((item => item.Key, "IsDelete"), (item => item.Value, "False"));
                    customerFilter.SubFilter(item => item.QueryData.QueryItems, itemFilter.AndCombine());
                    filter = filter & customerFilter.AndCombine();
                }
                var cursor = await _collect.FindAsync(filter, options, cancellationToken);
                return cursor.ToList();
            }
        }


        /// <summary>
        /// > Find all the tables that have the same key and value in the query data
        /// </summary>
        /// <param name="filter">a dictionary of key-value pairs, where the key is the name of the field, and
        /// the value is a list of values.</param>
        /// <returns>
        /// A list of MongoTable objects.
        /// </returns>
        private async Task<List<MongoTable>> FindTableInAsync(Dictionary<string, List<string>> filter = null)
        {
            var customerFilter = new MongoCustomerFilter<MongoTable>();
            var mongoDataItemFilter = new MongoCustomerFilter<MongoDataItem>();
            foreach (var para in filter)
            {
                mongoDataItemFilter.AppendEq((item => item.Key, para.Key));
                mongoDataItemFilter.AppendIn(item => item.Value, para.Value);
                customerFilter.SubFilter(item => item.QueryData.QueryItems, mongoDataItemFilter.AndCombine());
                mongoDataItemFilter.ClearFilter();
            }
            return await FindAllTableAsync(customerFilter.AndCombine());
        }


        /// <summary>
        /// > This function takes a dictionary of lists of strings, and returns a list of objects
        /// </summary>
        /// <param name="filter">Dictionary<string, List<string>></param>
        /// <returns>
        /// A list of objects of type T.
        /// </returns>
        public virtual async Task<List<T>> FindInAsync(Dictionary<string, List<string>> filter = null)
        {
            var result = await FindTableInAsync(filter);
            return result.Select(f => (T)f.BaseData).ToList();
        }

        /// <summary>
        /// Get the data
        /// </summary>
        /// <param name="filter"></param>
        /// <returns>
        /// Data List
        /// </returns>
        public async Task<List<T>> FindListByAggregateAsync(List<BsonDocument> filter)
        {
            using (new Performance(_performanceThreshold, "Handle FindListByAggregateAsync {0} execute", $"{typeof(T).FullName}"))
            {
                var list = new List<T>();
                try
                {
                    var aggregationResult = await _collect.AggregateAsync<BsonDocument>(filter);
                    var resultDocuments = await aggregationResult.ToListAsync();
                    if (resultDocuments?.Count > 0)
                    {
                        foreach (var item in resultDocuments)
                        {
                            if (item.ElementCount >= 5 && item.Names.Contains("BaseData"))
                            {
                                var entity = BsonSerializer.Deserialize<T>(item["BaseData"].AsBsonDocument);
                                list.Add(entity);
                            }
                        }
                    }
                }
                catch (Exception ex)
                {
                    Logger.WriteLineError($"FindListByAggregateAsync failed, Exception:{ex}");
                }
                return list;
            }
        }

        /// <summary>
        /// > This function takes a dictionary of filter parameters and returns a list of objects of type T
        /// </summary>
        /// <param name="filter">Dictionary<string, string></param>
        /// <returns>
        /// A list of objects of type T.
        /// </returns>
        public async Task<List<T>> FindListAsync(Dictionary<string, string> filter, bool onlyNotDeleted = true)
        {
            using (new Performance(_performanceThreshold, "Handle FindListAsync {0} execute", $"{typeof(T).FullName}"))
            {
                var customerFilter = new MongoCustomerFilter<MongoTable>();
                if (onlyNotDeleted)
                {
                    MongoCustomerFilter<MongoDataItem> itemFilter = new();
                    itemFilter.AppendEq((item => item.Key, "IsDelete"), (item => item.Value, "False"));
                    customerFilter.SubFilter(item => item.QueryData.QueryItems, itemFilter.AndCombine());
                }
                var mongoDataItemFilter = new MongoCustomerFilter<MongoDataItem>();
                foreach (var para in filter)
                {
                    mongoDataItemFilter.AppendEq((item => item.Key, para.Key), (item => item.Value, para.Value));
                    customerFilter.SubFilter(item => item.QueryData.QueryItems, mongoDataItemFilter.AndCombine());
                    mongoDataItemFilter.ClearFilter();
                }
                var projection = new ProjectionDefinitionBuilder<MongoTable>();
                var baseData = projection.Expression(r => (T)r.BaseData);
                FindOptions<MongoTable, T> newOptions;
                newOptions = new FindOptions<MongoTable, T>()
                {
                    Projection = baseData,
                    NoCursorTimeout = true
                };
                var cursor = await _collect.FindAsync(customerFilter.AndCombine(), newOptions);
                return cursor.ToList();
            }
        }


        /// <summary>
        /// Get the data of the current page according to the filter condition, sort condition, current page
        /// number and page size
        /// </summary>
        /// <param name="DBPageRequest"></param>
        /// <returns>
        /// A PageCollection of MongoTable.
        /// </returns>
        public async Task<PageCollection<MongoTable>> GetTablePageAsync(DBPageRequest request, bool onlyNotDeleted = true)
        {
            using (new Performance(_performanceThreshold, "Handle GetTablePageAsync {0} execute", $"{typeof(T).FullName}"))
            {
                if (onlyNotDeleted)
                {
                    MongoCustomerFilter<MongoTable> customerFilter = new();
                    MongoCustomerFilter<MongoDataItem> itemFilter = new();
                    itemFilter.AppendEq((item => item.Key, "IsDelete"), (item => item.Value, "False"));
                    customerFilter.SubFilter(item => item.QueryData.QueryItems, itemFilter.AndCombine());
                    request.Filter = request.Filter & customerFilter.AndCombine();
                }
                var count = await _collect.CountDocumentsAsync(request.Filter);
                if (request.Sort == null)
                {
                    request.Sort = Builders<MongoTable>.Sort.Descending("BaseData.CreateTime");
                }
                var cursor = await _collect.FindAsync(request.Filter, new FindOptions<MongoTable> { NoCursorTimeout = true, Skip = (request.CurrentPage - 1) * request.PageSize, Limit = request.PageSize, Sort = request.Sort });
                var pageCollection = new PageCollection<MongoTable>();
                pageCollection.CurrentPage = request.CurrentPage;
                pageCollection.PageSize = request.PageSize;
                pageCollection.DataCount = (int)count;
                pageCollection.PageData = await cursor.ToListAsync();
                return pageCollection;
            }
        }

        /// <summary>
        /// Get the data of the current page according to the filter condition, sort condition, current page
        /// number and page size
        /// </summary>
        /// <param name="DBPageRequest"></param>
        /// <returns>
        /// A PageCollection of MongoTable.
        /// </returns>
        public async Task<PageCollection<T>> GetAggregateTablePageAsync(DBPageRequest request)
        {
            using (new Performance(_performanceThreshold, "Handle GetAggregateTablePageAsync {0} execute", $"{typeof(T).FullName}"))
            {
                if (request.Pipeline == null)
                {
                    request.Pipeline = new List<BsonDocument>();
                }
                //分页
                var dicSkip = new Dictionary<string, object>();
                dicSkip.Add("$skip", (request.CurrentPage - 1) * request.PageSize);
                var dicLimit = new Dictionary<string, object>();
                dicLimit.Add("$limit", request.PageSize);
                var dicTotal = new Dictionary<string, object>();
                dicTotal.Add("$count", "total");
                var pageBsonDoc = new BsonDocument("$facet", new BsonDocument
                {
                    { "pagination", new BsonArray
                        {
                            new BsonDocument(dicSkip), // 跳过前面的文档
                            new BsonDocument(dicLimit) // 限制返回的文档数量
                        }
                    },
                    { "totalCount", new BsonArray
                        {
                            new BsonDocument(dicTotal) // 计算满足条件的文档总数
                        }
                    }
                });
                request.Pipeline.Add(pageBsonDoc);
                var totalCount = 0;
                var pageList = new List<T>();
                try
                {
                    var aggregationResult = await _collect.AggregateAsync<BsonDocument>(request.Pipeline);
                    var resultDocuments = await aggregationResult.ToListAsync();
                    //抓取数据，总数和分页内容
                    var paginationResults = resultDocuments[0]["pagination"].AsBsonArray;
                    var totalCountResults = resultDocuments[0]["totalCount"].AsBsonArray;
                    if (totalCountResults.Count > 0)
                    {
                        totalCount = totalCountResults.FirstOrDefault()?["total"]?.ToInt32() ?? 0;
                    }
                    if (paginationResults?.Count > 0)
                    {
                        var index = 0;
                        var isExistBaseData = false;
                        foreach (var bsItem in paginationResults.Values)
                        {
                            if (index == 0)
                            {
                                var firstBsonDoc = paginationResults.Values.FirstOrDefault().AsBsonDocument;
                                if (firstBsonDoc.ElementCount >= 5 && firstBsonDoc.Names.Contains("BaseData"))
                                {
                                    isExistBaseData = true;
                                }
                                index++;
                            }
                            T entity = null;
                            if (isExistBaseData)
                            {
                                entity = BsonSerializer.Deserialize<T>(bsItem["BaseData"].AsBsonDocument);
                            }
                            else
                            {
                                entity = BsonSerializer.Deserialize<T>(bsItem.AsBsonDocument);
                            }
                            pageList.Add(entity);
                        }
                    }
                }
                catch (Exception ex)
                {
                    Logger.WriteLineError($"GetAggregateTablePageAsync failed, Exception:{ex}");
                }
                var pageCollection = new PageCollection<T>();
                pageCollection.CurrentPage = request.CurrentPage;
                pageCollection.PageSize = request.PageSize;
                pageCollection.DataCount = totalCount;
                pageCollection.PageData = pageList;
                return pageCollection;
            }
        }


        /// <summary>
        /// > The function is used to query the data in the MongoDB database, and the query conditions are
        /// combined with the OR operator
        /// </summary>
        /// <param name="filter">the key-value pair of the query condition</param>
        /// <returns>
        /// A list of KeyValuePair<string, string>
        /// </returns>
        public async Task<List<KeyValuePair<string, string>>> OrExists(Dictionary<string, string> filter)
        {
            using (new Performance(_performanceThreshold, "Handle OrExists {0} execute", $"{typeof(T).FullName}"))
            {
                List<FilterDefinition<MongoTable>> definitionList = new List<FilterDefinition<MongoTable>>();
                var fieldDefinition = new StringFieldDefinition<MongoTable, MongoQuery>("QueryData");
                var customerFilter = new MongoCustomerFilter<MongoTable>();
                var mongoDataItemFilter = new MongoCustomerFilter<MongoDataItem>();
                foreach (var para in filter)
                {
                    mongoDataItemFilter.AppendEq((item => item.Key, para.Key), (item => item.Value, para.Value));
                    customerFilter.SubFilter(item => item.QueryData.QueryItems, mongoDataItemFilter.OrCombine());
                    mongoDataItemFilter.ClearFilter();
                }
                //未删除
                MongoCustomerFilter<MongoTable> customerFilterDelete = new();
                MongoCustomerFilter<MongoDataItem> itemFilter = new();
                itemFilter.AppendEq((item => item.Key, "IsDelete"), (item => item.Value, "False"));
                customerFilterDelete.SubFilter(item => item.QueryData.QueryItems, itemFilter.AndCombine());
                var finalFilter = customerFilter.AndCombine() & customerFilterDelete.AndCombine();
                var cursor = await _collect.FindAsync(finalFilter);
                await cursor.MoveNextAsync();
                var resultDic = cursor.Current?.FirstOrDefault()?.QueryData?.QueryItems?.Select(f => new KeyValuePair<string, string>(f.Key, f.Value));
                return resultDic?.Intersect(filter).ToList();
            }
        }

        /// <summary>
        /// It inserts a document into a _collection.
        /// </summary>
        /// <param name="T">The type of the data to be inserted</param>
        /// <param name="platformDatas">The data of the platform, which is the data of the platform, which is
        /// the data of the platform, which is the data of the platform, which is the data of the platform,
        /// which is the data of the platform, which is the data of the platform, which is the data of
        /// the</param>
        /// <param name="extensionData">The data that needs to be stored in the MongoDB database, which is a
        /// string type, and the data is stored in the form of a JSON string.</param>
        /// isWriteOperationLog  isWriteOperationLog default true
        public async Task<string> InsertOneAsync(T data, Dictionary<string, string> platformDatas, string extensionData, bool isWriteOperationLog = true)
        {
            using (new Performance(_performanceThreshold, "Handle InsertOneAsync {0} execute", $"{typeof(T).FullName}"))
            {
                var (uniqueDic, table) = MongoRepertoryCenter.Instance.SetTableData(data, platformDatas, extensionData);
                var result = await OrTableExists(uniqueDic);
                if (result?.Count > 0)
                {
                    throw new System.Exception($"{result.FirstOrDefault().Key} is duplicate key,value is {result.FirstOrDefault().Value}");
                }
                if (isWriteOperationLog || !_isDistributed)
                {
                    await _collect.InsertOneAsync(table);
                }
                else
                {
                    await ((MongoCollectionProxy<MongoTable>)_collect).InsertOneWithoutWriteLogAsync(table);
                }
                return data.Code;
            }
        }

        /// <summary>
        /// It inserts a document into a _collection.
        /// </summary>
        /// <param name="T">The type of the data to be inserted</param>
        /// <param name="platformDatas">The data of the platform, which is the data of the platform, which is
        /// the data of the platform, which is the data of the platform, which is the data of the platform,
        /// which is the data of the platform, which is the data of the platform, which is the data of
        /// the</param>
        /// <param name="extensionData">The data that needs to be stored in the MongoDB database, which is a
        /// string type, and the data is stored in the form of a JSON string.</param>
        /// isWriteOperationLog  isWriteOperationLog default true
        public async Task<string> InsertOneForAsyncDBAsync(T data, Dictionary<string, string> platformDatas, string extensionData, bool isWriteOperationLog = true)
        {
            using (new Performance(_performanceThreshold, "Handle InsertOneForAsyncDBAsync {0} execute", $"{typeof(T).FullName}"))
            {
                var (uniqueDic, table) = MongoRepertoryCenter.Instance.SetTableData(data, platformDatas, extensionData);
                if (isWriteOperationLog || !_isDistributed)
                {
                    await _collect.InsertOneAsync(table);
                }
                else
                {
                    await ((MongoCollectionProxy<MongoTable>)_collect).InsertOneWithoutWriteLogAsync(table);
                }
                return data.Code;
            }
        }


        /// <summary>
        /// > Insert a list of data into the database, and if the data already exists, skip it
        /// </summary>
        /// <param name="infos">a list of tuples, each tuple contains a data object, a dictionary of platform
        /// data, and an extension data string.</param>
        /// <param name="isWriteOperationLog">add oplog</param>
        public async Task<long> InsertAllAsync(List<(T data, Dictionary<string, string> platformDatas, string extensionData)> infos, bool isWriteOperationLog = true)
        {
            using (new Performance(_performanceThreshold, "Handle InsertAllAsync {0} execute", $"{typeof(T).FullName}"))
            {
                var writeCount = 0;
                foreach (var info in infos)
                {
                    var (uniqueDic, table) = MongoRepertoryCenter.Instance.SetTableData(info.data, info.platformDatas, info.extensionData);
                    var existsList = await OrTableExists(uniqueDic);
                    if (existsList?.Count > 0)
                    {
                        continue;
                    }
                    if (isWriteOperationLog || !_isDistributed)
                    {
                        await _collect.InsertOneAsync(table);
                    }
                    else
                    {
                        await ((MongoCollectionProxy<MongoTable>)_collect).InsertOneWithoutWriteLogAsync(table);
                    }
                    writeCount++;
                }
                return writeCount;
            }
        }

        /// <summary>
        /// Insert all
        /// </summary>
        /// <param name="infos"></param>
        /// <returns></returns>
        public virtual async Task<bool> InsertAllWithoutOplogLogAsync(List<T> infos)
        {
            using (new Performance(_performanceThreshold, "Handle InsertAllWithoutOplogLogAsync {0} execute", $"{typeof(T).FullName}"))
            {
                var tables = new List<MongoTable>();
                foreach (var info in infos)
                {
                    var (uniqueDic, table) = MongoRepertoryCenter.Instance.SetTableData(info, new Dictionary<string, string>(), "");
                    tables.Add(table);
                }
                if (!_isDistributed)
                {
                    await _collect.InsertManyAsync(tables);
                }
                else
                {
                    await ((MongoCollectionProxy<MongoTable>)_collect).InsertManyWithoutWriteLogAsync(tables);
                }

            }
            return true;
        }

        /// <summary>
        ///  Insert a list of data into the database, and if the data already exists, skip it
        /// </summary>
        /// <param name="infos">a list of tuples, each tuple contains a data object, a dictionary of platform
        /// data, and an extension data string.</param>
        /// <param name="isWriteOperationLog">add oplog</param>
        public async Task<bool> UpdateAllWithoutOplogLogAsync(List<T> infos)
        {
            using (new Performance(_performanceThreshold, "Handle UpdateAllWithoutOplogLogAsync {0} execute", $"{typeof(T).FullName}"))
            {
                var listWrites = new List<WriteModel<MongoTable>>();
                foreach (var info in infos)
                {
                    var (uniqueDic, table) = MongoRepertoryCenter.Instance.SetTableData(info, new Dictionary<string, string>(), "");
                    MongoCustomerFilter<MongoTable> customerFilterUpdate = new();
                    MongoCustomerFilter<MongoDataItem> subFilter = new();
                    subFilter.AppendEq((item => item.Key, "Code"));
                    subFilter.AppendEq((item => item.Value, table.BaseData.Code));
                    customerFilterUpdate.SubFilter(item => item.QueryData.QueryItems, subFilter.AndCombine());
                    MongoCustomerUpdate<MongoTable> update = new();
                    update.SetField(item => item.QueryData, table.QueryData);
                    update.SetField(item => item.BaseData, table.BaseData);
                    listWrites.Add(new UpdateOneModel<MongoTable>(customerFilterUpdate.AndCombine(), update.AllUpdateDefinition));
                }
                if (!_isDistributed)
                {
                    await _collect.BulkWriteAsync(listWrites);
                }
                else
                {
                    await ((MongoCollectionProxy<MongoTable>)_collect).BulkWriteAsync(listWrites);
                }

            }
            return true;
        }

        /// <summary>
        /// > Delete all documents that match the specified conditions
        /// </summary>
        /// <param name="infos">a list of tuples, each tuple contains a key and a value.</param>
        /// <returns>
        /// The number of documents deleted.
        /// </returns>
        public async Task<long> DeleteAllAsync(List<(string key, string val)> infos)
        {
            using (new Performance(_performanceThreshold, "Handle DeleteAllAsync {0} execute", $"{typeof(T).FullName}"))
            {
                var customerFilter = new MongoCustomerFilter<MongoTable>();
                var mongoDataItemFilter = new MongoCustomerFilter<MongoDataItem>();
                var deleteCount = 0;
                foreach (var info in infos)
                {
                    mongoDataItemFilter.AppendEq((item => item.Key, info.key), (item => item.Value, info.val));
                    customerFilter.SubFilter(item => item.QueryData.QueryItems, mongoDataItemFilter.AndCombine());
                    mongoDataItemFilter.AppendEq((item => item.Key, "IsDelete"));
                    MongoCustomerUpdate<MongoTable> update = new();
                    update.SetField("BaseData.IsDelete", true);
                    update.RemoveSubItemFilter(item => item.QueryData.QueryItems, mongoDataItemFilter.AndCombine());
                    //假删除
                    var result = await UpdateOneAsync(customerFilter.AndCombine(), update.AllUpdateDefinition);
                    update.ClearUpdate();
                    update.AddSubItem(item => item.QueryData.QueryItems, new MongoDataItem
                    {
                        Key = "IsDelete",
                        Value = "True"
                    });
                    await UpdateOneAsync(customerFilter.AndCombine(), update.AllUpdateDefinition);
                    //await _collect.DeleteOneAsync(customerFilter.AndCombine());
                    mongoDataItemFilter.ClearFilter();
                    customerFilter.ClearFilter();
                    if (result > 0)
                    {
                        deleteCount++;
                    }
                }
                return deleteCount;
            }
        }


        /// <summary>
        /// Delete the data in the MongoDB database according to the specified key and value
        /// </summary>
        /// <param name="key">The key of the data item</param>
        /// <param name="value">the value of the key</param>
        /// <returns>
        /// The result of the DeleteOneAsync method is a DeleteResult object.
        /// </returns>
        public async Task<bool> DeleteOneAsync(string key, string value, bool onlyNotDeleted = true)
        {
            using (new Performance(_performanceThreshold, "Handle DeleteOneAsync {0} execute", $"{typeof(T).FullName}"))
            {
                var customerFilter = new MongoCustomerFilter<MongoTable>();
                var mongoDataItemFilter = new MongoCustomerFilter<MongoDataItem>();
                mongoDataItemFilter.AppendEq((item => item.Key, key), (item => item.Value, value));
                customerFilter.SubFilter(item => item.QueryData.QueryItems, mongoDataItemFilter.AndCombine());
                if (onlyNotDeleted)
                {
                    MongoCustomerFilter<MongoDataItem> itemFilter = new();
                    itemFilter.AppendEq((item => item.Key, "IsDelete"), (item => item.Value, "False"));
                    customerFilter.SubFilter(item => item.QueryData.QueryItems, itemFilter.AndCombine());
                }
                mongoDataItemFilter.ClearFilter();
                mongoDataItemFilter.AppendEq((item => item.Key, "IsDelete"));
                MongoCustomerUpdate<MongoTable> update = new();
                update.SetField("BaseData.IsDelete", true);
                update.RemoveSubItemFilter(item => item.QueryData.QueryItems, mongoDataItemFilter.AndCombine());
                //假删除
                var result = await UpdateOneAsync(customerFilter.AndCombine(), update.AllUpdateDefinition);
                update.ClearUpdate();
                update.AddSubItem(item => item.QueryData.QueryItems, new MongoDataItem
                {
                    Key = "IsDelete",
                    Value = "True"
                });
                await UpdateOneAsync(customerFilter.AndCombine(), update.AllUpdateDefinition);
                //var result = await _collect.DeleteOneAsync(customerFilter.AndCombine());
                if (result > 0)
                {
                    return true;
                }
                return false;
            }
        }

        /// <summary>
        /// > Delete all documents that match the specified conditions
        /// </summary>
        /// <param name="infos">a list of tuples, each tuple contains a key and a value.</param>
        /// <returns>
        /// The number of documents deleted.
        /// </returns>
        public async Task<long> RealDeleteAllAsync(List<(string key, string val)> infos)
        {
            using (new Performance(_performanceThreshold, "Handle RealDeleteAllAsync {0} execute", $"{typeof(T).FullName}"))
            {
                var customerFilter = new MongoCustomerFilter<MongoTable>();
                var mongoDataItemFilter = new MongoCustomerFilter<MongoDataItem>();
                var deleteCount = 0;
                foreach (var info in infos)
                {
                    mongoDataItemFilter.AppendEq((item => item.Key, info.key), (item => item.Value, info.val));
                    customerFilter.SubFilter(item => item.QueryData.QueryItems, mongoDataItemFilter.AndCombine());
                    await _collect.DeleteOneAsync(customerFilter.AndCombine());
                    mongoDataItemFilter.ClearFilter();
                    customerFilter.ClearFilter();
                    deleteCount++;
                }
                return deleteCount;
            }
        }


        /// <summary>
        /// Delete the data in the MongoDB database according to the specified key and value
        /// </summary>
        /// <param name="key">The key of the data item</param>
        /// <param name="value">the value of the key</param>
        /// <returns>
        /// The result of the DeleteOneAsync method is a DeleteResult object.
        /// </returns>
        public async Task<bool> RealDeleteOneAsync(string key, string value, bool isWriteOperationLog = true)
        {
            using (new Performance(_performanceThreshold, "Handle RealDeleteOneAsync {0} execute", $"{typeof(T).FullName}"))
            {
                var customerFilter = new MongoCustomerFilter<MongoTable>();
                var mongoDataItemFilter = new MongoCustomerFilter<MongoDataItem>();
                mongoDataItemFilter.AppendEq((item => item.Key, key), (item => item.Value, value));
                customerFilter.SubFilter(item => item.QueryData.QueryItems, mongoDataItemFilter.AndCombine());
                mongoDataItemFilter.ClearFilter();

                if (isWriteOperationLog)
                {
                    var result = await _collect.DeleteOneAsync(customerFilter.AndCombine());
                    if (result.DeletedCount > 0)
                    {
                        return true;
                    }
                }
                else
                {
                    var result = await ((MongoCollectionProxy<MongoTable>)_collect).DeleteOneWithoutWriteLogAsync(customerFilter.AndCombine());
                    if (result.DeletedCount > 0)
                    {
                        return true;
                    }
                }
                return false;
            }
        }

        /// <summary>
        /// A function that updates the data in the database.
        /// </summary>
        /// <param name="key">the key of the data item</param>
        /// <param name="value">the value of the key</param>
        /// <param name="MongoTable">The data type of the data to be updated</param>
        /// <param name="canNull">Whether to allow null values to be updated</param>
        ///  <param name="isWriteOperationLog">add oplog</param>
        /// <returns>
        /// A boolean value.
        /// </returns>
        public async Task<bool> Update(string key, string value, MongoTable data, bool canNull = false, bool isWriteOperationLog = true, bool onlyNotDeleted = true)
        {
            using (new Performance(_performanceThreshold, "Handle Update {0} execute", $"{typeof(T).FullName}"))
            {
                var customerFilter = new MongoCustomerFilter<MongoTable>();
                var mongoDataItemFilter = new MongoCustomerFilter<MongoDataItem>();
                mongoDataItemFilter.AppendEq((item => item.Key, key), (item => item.Value, value));
                customerFilter.SubFilter(item => item.QueryData.QueryItems, mongoDataItemFilter.AndCombine());
                mongoDataItemFilter.ClearFilter();
                if (onlyNotDeleted)
                {
                    MongoCustomerFilter<MongoDataItem> itemFilter = new();
                    itemFilter.AppendEq((item => item.Key, "IsDelete"), (item => item.Value, "False"));
                    customerFilter.SubFilter(item => item.QueryData.QueryItems, itemFilter.AndCombine());
                }
                var cursor = await _collect.FindAsync(customerFilter.AndCombine());
                if (await cursor.MoveNextAsync())
                {
                    var first = cursor.Current.FirstOrDefault();
                    MongoRepertoryCenter.Instance.UpgradeData<T>(first, data, canNull);
                    MongoCustomerUpdate<MongoTable> update = new();
                    update.SetField(item => item.QueryData, first.QueryData);
                    update.SetField(item => item.BaseData, first.BaseData);
                    update.SetField(item => item.ExtendsData, first.ExtendsData);
                    if (isWriteOperationLog || !_isDistributed)
                    {
                        var result = await _collect.UpdateOneAsync(customerFilter.AndCombine(), update.AllUpdateDefinition);
                        return true;
                    }
                    else
                    {
                        var result = await ((MongoCollectionProxy<MongoTable>)_collect).UpdateOneWithoutWriteLogAsync(customerFilter.AndCombine(), update.AllUpdateDefinition);
                        return true;
                    }
                }
                return false;
            }
        }


        /// <summary>
        /// > Find the first table that has a data item with a key and value that matches the key and value in
        /// the dictionary
        /// </summary>
        /// <param name="filter">a dictionary of key-value pairs</param>
        /// <returns>
        /// A cursor.
        /// </returns>
        public async Task<MongoTable> FindTableOr(Dictionary<string, string> filter)
        {
            using (new Performance(_performanceThreshold, "Handle FindTableOr {0} execute", $"{typeof(T).FullName}"))
            {
                var customerFilter = new MongoCustomerFilter<MongoTable>();
                var mongoDataItemFilter = new MongoCustomerFilter<MongoDataItem>();
                foreach (var para in filter)
                {
                    mongoDataItemFilter.AppendEq((item => item.Key, para.Key), (item => item.Value, para.Value));
                    customerFilter.SubFilter(item => item.QueryData.QueryItems, mongoDataItemFilter.OrCombine());
                    mongoDataItemFilter.ClearFilter();
                }
                var cursor = await _collect.FindAsync(customerFilter.AndCombine());
                await cursor.MoveNextAsync();
                return cursor.Current?.FirstOrDefault();
            }
        }


        /// <summary>
        /// > This function is used to query the data in the MongoDB database
        /// </summary>
        /// <param name="filter">The key value pair of the query condition</param>
        /// <returns>
        /// A list of KeyValuePair<string, string>
        /// </returns>
        public async Task<List<KeyValuePair<string, string>>> OrTableExists(Dictionary<string, string> filter)
        {
            using (new Performance(_performanceThreshold, "Handle OrTableExists {0} execute", $"{typeof(T).FullName}"))
            {
                List<FilterDefinition<MongoTable>> definitionList = new List<FilterDefinition<MongoTable>>();
                var fieldDefinition = new StringFieldDefinition<MongoTable, MongoQuery>("QueryData");
                var customerFilter = new MongoCustomerFilter<MongoTable>();
                var mongoDataItemFilter = new MongoCustomerFilter<MongoDataItem>();
                foreach (var para in filter)
                {
                    mongoDataItemFilter.AppendEq((item => item.Key, para.Key), (item => item.Value, para.Value));
                    customerFilter.SubFilter(item => item.QueryData.QueryItems, mongoDataItemFilter.OrCombine());
                    mongoDataItemFilter.ClearFilter();
                }
                //未删除
                MongoCustomerFilter<MongoTable> customerFilterDelete = new();
                MongoCustomerFilter<MongoDataItem> itemFilter = new();
                itemFilter.AppendEq((item => item.Key, "IsDelete"), (item => item.Value, "False"));
                customerFilterDelete.SubFilter(item => item.QueryData.QueryItems, itemFilter.AndCombine());
                var finalFilter = customerFilter.AndCombine() & customerFilterDelete.AndCombine();
                var cursor = await _collect.FindAsync(finalFilter);
                await cursor.MoveNextAsync();
                var resultDic = cursor.Current?.FirstOrDefault()?.QueryData?.QueryItems?.Select(f => new KeyValuePair<string, string>(f.Key, f.Value));
                return resultDic?.Intersect(filter).ToList();
            }
        }

        public async Task<bool> Exists(FilterDefinition<MongoTable> filter = null)
        {
            using (new Performance(_performanceThreshold, "Handle Exists {0} execute", $"{typeof(T).FullName}"))
            {
                MongoCustomerFilter<MongoTable> customerFilter = new();
                MongoCustomerFilter<MongoDataItem> itemFilter = new();
                itemFilter.AppendEq((item => item.Key, "IsDelete"), (item => item.Value, "False"));
                customerFilter.SubFilter(item => item.QueryData.QueryItems, itemFilter.AndCombine());
                filter = filter & customerFilter.AndCombine();
                return await _collect.CountAsync(filter) > 0;
            }
        }


        /// <summary>
        /// > Get the page data from the database, and then convert the data to the type of the current
        /// repository
        /// </summary>
        /// <param name="DBPageRequest"></param>
        /// <returns>
        /// A PageCollection of T.
        /// </returns>
        public virtual async Task<PageCollection<T>> GetPages(DBPageRequest request, bool onlyNotDeleted = true)
        {
            var result = await GetTablePageAsync(request, onlyNotDeleted);
            var pageCollection = new PageCollection<T>();
            pageCollection.CurrentPage = result.CurrentPage;
            pageCollection.PageSize = result.PageSize;
            pageCollection.DataCount = result.DataCount;
            pageCollection.PageData = result.PageData.Select(f => (T)f.BaseData).ToList();
            return pageCollection;
        }



        /// <summary>
        /// Get the total number of records that meet the conditions, and then get the data of the current page
        /// </summary>
        /// <param name="filter">The filter condition</param>
        /// <param name="pageIndex">The current page number</param>
        /// <param name="pageSize">The number of records per page</param>
        /// <param name="sort">The sort condition, which is the sort condition of the query.</param>
        /// <returns>
        /// A PageCollection of T.
        /// </returns>
        public async Task<PageCollection<T>> GetPagesByFilter(FilterDefinition<MongoTable> filter, int pageIndex = 1, int pageSize = 10, SortDefinition<MongoTable> sort = null)
        {
            var pages = new PageCollection<T>()
            {
                CurrentPage = pageIndex,
                PageSize = pageSize
            };
            if (filter == null)
            {
                return pages;
            }
            if (sort == null)
            {
                SortDefinitionBuilder<MongoTable> builderSort = Builders<MongoTable>.Sort;
                sort = builderSort.Descending("BaseData.CreateTime");
            }
            pages.DataCount = (int)await CountDocumentsAsync(filter);
            pages.PageData = await FindAllAsync(filter, new FindOptions<MongoTable> { NoCursorTimeout = true, Skip = (pageIndex - 1) * pageSize, Limit = pageSize, Sort = sort });
            return pages;
        }
    }
}
