﻿using System.Threading.Tasks;
using WingMongoDBModule.DO;
using System;
using WingServerCommon.Log;
using WingMongoDBModule.Mongodb.Driver;
using WingMongoDBModule.Entity;
using WingInterfaceLibrary.DB.Request;
using System.Collections.Generic;
using System.Linq;
using JsonRpcLite.Services;
using WingInterfaceLibrary.Enum;

namespace WingMongoDBModule.Repositories
{
    public class OrganizationDBRepository : BaseDBRepository<OrganizationDO>
    {
        public OrganizationDBRepository():base("Organizations"){}

        /// <summary>
        /// > This function returns an OrganizationDO object based on the code passed in
        /// </summary>
        /// <param name="code">The code of the organization</param>
        /// <returns>
        /// The OrganizationDO object.
        /// </returns>
        public async Task<OrganizationDO> GetOrganizationByCodeAsync(string code)
        {
            return await base.FindOneAsync("Code", code);
        }


        /// <summary>
        /// > Update the organization name in the base data and the query data of the table
        /// </summary>
        /// <param name="UpdateOrganizationNameDBRequest">The request object that contains the parameters to be
        /// updated.</param>
        /// <returns>
        /// The return value is a boolean value.
        /// </returns>
        public async Task<bool> UpdateOrganizationName(UpdateOrganizationNameDBRequest request)
        {
            request.OrganizationName = request.OrganizationName.Trim();
            MongoCustomerFilter<MongoTable> customerFilter = new();
            MongoCustomerFilter<MongoDataItem> subFilter = new();
            //查找当前组织
            subFilter.AppendEq((item => item.Key, "Code"));
            subFilter.AppendEq((item => item.Value, request.OrganizationCode));
            customerFilter.SubFilter(item => item.QueryData.QueryItems, subFilter.AndCombine());
            var currentOrg = await FindOneAsync(customerFilter.AndCombine());
            //验证是否有重复名称
            subFilter.AppendEq((item => item.Key, "OrganizationName"));
            subFilter.AppendEq((item => item.Value, request.OrganizationName));
            customerFilter.SubFilter(item => item.QueryData.QueryItems, subFilter.AndCombine());
            subFilter.ClearFilter();
            var isFound = await FindOneAsync(customerFilter.AndCombine());
            if(isFound!=null && isFound.Code != request.OrganizationCode && isFound.ParentCode == currentOrg.ParentCode)
            {
                throw new RpcException((int)CustomerRpcCode.OrganizationNameDuplicate,"OrganizationName Duplicate","OrganizationName Duplicate");
            }

            subFilter.AppendEq((item => item.Key, "Code"));
            subFilter.AppendEq((item => item.Value, request.OrganizationCode));
            customerFilter.SubFilter(item => item.QueryData.QueryItems, subFilter.AndCombine());
            subFilter.ClearFilter();

            MongoCustomerUpdate<MongoTable> update = new();
            subFilter.AppendEq((item => item.Key, nameof(request.OrganizationName)));
            update.SetField("BaseData.OrganizationName", request.OrganizationName);
            update.RemoveSubItemFilter(item => item.QueryData.QueryItems, subFilter.AndCombine());
            await UpdateOneAsync(customerFilter.AndCombine(), update.AllUpdateDefinition);

            update.ClearUpdate();
            update.AddSubItem(item => item.QueryData.QueryItems, new MongoDataItem
            {
                Key = nameof(request.OrganizationName),
                Value = request.OrganizationName
            });
            return await UpdateOneAsync(customerFilter.AndCombine(), update.AllUpdateDefinition) > 0;
        }


        /// <summary>
        /// > Find all organizations where the assigned admins contains any of the admin codes in the list
        /// </summary>
        /// <param name="adminCodes">List<string></param>
        /// <returns>
        /// A list of OrganizationDO objects.
        /// </returns>
        public async Task<List<OrganizationDO>> FindOrganizationsAsync(List<string> adminCodes)
        {
            if (adminCodes != null && adminCodes.Any())
            {
                MongoCustomerFilter<MongoTable> filter = new();
                filter.AppendAnyIn("BaseData.AssignedAdmins", adminCodes);
                return await FindAllAsync(filter.AndCombine());
            }
            return new List<OrganizationDO>();
        }

    }
}
