﻿using WingMongoDBModule.Entity;
using System.Collections.Generic;
using System.Threading.Tasks;
using System.ComponentModel;
using MongoDB.Bson;
using MongoDB.Bson.Serialization;
using MongoDB.Bson.Serialization.Attributes;
using MongoDB.Driver;
using System.Linq;
using System;
using WingMongoDBModule.Mongodb.Driver;
using WingMongoDBModule.DO;
using WingInterfaceLibrary.Request;
using System.Threading;
using JsonRpcLite.Services;
using WingMongoDBModule.Request;
using WingInterfaceLibrary.Enum;
using WingServerCommon.Config;
using WingServerCommon.Config.Parameters;
using WingServerCommon.Utilities;
using WingMongoDBModule.DBSync;
using WingMongoDBModule.Entities;
using System.Linq.Expressions;
using WingServerCommon.Log;

namespace WingMongoDBModule.Repositories
{
    public class SimpleDBRepository<T> where T : BaseEntity
    {
        private int _performanceThreshold = ConfigurationManager.GetParammeter<IntParameter>("Log", "DBPerformance").Value;
        private bool _isDistributed = ConfigurationManager.GetParammeter<BoolParameter>("General", "IsDistributed").Value;
        private IMongoCollection<T> _collect;
        public string CollectionName { get; set; }

        public SimpleDBRepository(string typeName, bool registerType = true)
        {
            if (registerType)
            {
                RegisterDBType(typeof(T));
            }
            CollectionName = typeName;
            _collect = MongoRepertoryCenter.Instance.GetCollection<T>(typeName);
            var mongoCollectionProxy = _collect as MongoCollectionProxy<T>;
            if (mongoCollectionProxy != null)
            {
                mongoCollectionProxy.IsSimple = true;
            }
        }

        /// <summary>
        /// 创建索引
        /// </summary>
        /// <remarks>仅初次部署和db更新使用</remarks>
        public virtual void CreateIndex()
        {

        }

        /// <summary>
        /// 创建索引 IndexModel
        /// </summary>
        /// <param name="createIndexModel"></param>
        protected async Task CreateIndexModelAsync(CreateIndexModel<T> createIndexModel)
        {
            await _collect.Indexes.CreateOneAsync(createIndexModel);
        }

        /// <summary>
        /// 注册DB类型
        /// </summary>
        /// <param name="lookupType">注册类型</param>
        private void RegisterDBType(Type lookupType)
        {
            TypeDescriptor.AddAttributes(lookupType, new BsonIgnoreExtraElementsAttribute());
            BsonClassMap.LookupClassMap(lookupType);
        }

        /// <summary>
        /// It updates the data in the database.
        /// </summary>
        /// <param name="key">the name of the field to be updated</param>
        /// <param name="value">the value of the key</param>
        /// <param name="T">The type of the data to be stored in the database.</param>
        /// <param name="extensionData">This is the data that will be stored in the MongoDB database.</param>
        /// <param name="canNull">If the data is null, whether to delete the data</param>
        /// <param name="isWriteOperationLog">add oplog</param>
        public virtual async Task<bool> UpdateOneAsync(string key, string value, T data, bool canNull = false, bool isWriteOperationLog = true, bool onlyNotDeleted = true)
        {

            if (data != null)
            {
                return await Update(key, value, data, canNull, isWriteOperationLog, onlyNotDeleted);
            }
            return false;
        }

        /// <summary>
        /// It updates the data in the database.
        /// </summary>
        /// <param name="key">the name of the field to be updated</param>
        /// <param name="value">the value of the key</param>
        /// <param name="T">The type of the data to be stored in the database.</param>
        /// <param name="extensionData">This is the data that will be stored in the MongoDB database.</param>
        /// <param name="canNull">If the data is null, whether to delete the data</param>
        /// <param name="isWriteOperationLog">add oplog</param>
        public virtual async Task<bool> UpdateOneForSyncDBAsync(string key, string value, T data, bool canNull = false, bool isWriteOperationLog = true)
        {
            if (data != null)
            {
                return await UpdateForSync(key, value, data, canNull, isWriteOperationLog);
            }
            return false;
        }

        /// <summary>
        /// A function that updates the data in the database.
        /// </summary>
        /// <param name="key">the key of the data item</param>
        /// <param name="value">the value of the key</param>
        /// <param name="MongoTable">The data type of the data to be updated</param>
        /// <param name="canNull">Whether to allow null values to be updated</param>
        ///  <param name="isWriteOperationLog">add oplog</param>
        /// <returns>
        /// A boolean value.
        /// </returns>
        public async Task<bool> UpdateForSync(string key, string value, T data, bool canNull = false, bool isWriteOperationLog = true)
        {
            using (new Performance(_performanceThreshold, "Handle UpdateForSync {0} execute", $"{typeof(T).FullName}"))
            {
                var filter = Builders<T>.Filter.Eq(key, value);
                MongoCustomerUpdate<T> update = new();
                var type = data.GetType();
                var props = type.GetProperties();
                foreach (var item in props)
                {
                    if (item.Name == "Id")
                    {
                        continue;
                    }
                    if (item.GetCustomAttributes(true).Any(x => x is MongoDB.Bson.Serialization.Attributes.BsonIgnoreAttribute))
                    {
                        continue;
                    }
                    var propGetMethod = item.GetGetMethod();
                    var propValue = propGetMethod?.Invoke(data, null);
                    update.SetField(item.Name, propValue);
                }
                if (isWriteOperationLog || !_isDistributed)
                {
                    var result = await _collect.UpdateOneAsync(filter, update.AllUpdateDefinition);
                    return true;
                }
                else
                {
                    var result = await ((MongoCollectionProxy<T>)_collect).UpdateOneWithoutWriteLogAsync(filter, update.AllUpdateDefinition);
                    return true;
                }
            }
        }

        /// <summary>
        /// It updates a single document in the _collection.
        /// </summary>
        /// <param name="filter">The filter to select the document to update.</param>
        /// <param name="update">The update to apply to the document.</param>
        /// <returns>
        /// The number of documents modified.
        /// </returns>
        public async Task<long> UpdateOneAsync(FilterDefinition<T> filter, UpdateDefinition<T> update)
        {
            using (new Performance(_performanceThreshold, "Handle UpdateOneAsync {0} execute", $"{typeof(T).FullName}"))
            {
                if (filter != null && update != null)
                {
                    var result = await _collect.UpdateOneAsync(filter, update);
                    return result.ModifiedCount;
                }
                return 0;
            }
        }


        /// <summary>
        /// Update all the documents that match the filter with the update
        /// </summary>
        /// <param name="filter">The filter to select the documents to update.</param>
        /// <param name="update">The update to apply to the matching documents.</param>
        /// <returns>
        /// The number of documents that were modified.
        /// </returns>
        public async Task<long> UpdateAllAsync(FilterDefinition<T> filter, UpdateDefinition<T> update)
        {
            using (new Performance(_performanceThreshold, "Handle UpdateAllAsync {0} execute", $"{typeof(T).FullName}"))
            {
                if (filter != null && update != null)
                {
                    var result = await _collect.UpdateManyAsync(filter, update);
                    return result.ModifiedCount;
                }
                return 0;
            }
        }



        /// <summary>
        /// 获取堆栈
        /// </summary>
        /// <returns></returns>
        private string GetStackTraceModelName()
        {
            //当前堆栈信息
            System.Diagnostics.StackTrace st = new System.Diagnostics.StackTrace();
            System.Diagnostics.StackFrame[] sfs = st.GetFrames();
            //过虑的方法名称
            string _fullName = string.Empty, _methodName = string.Empty;
            for (int i = 1; i < sfs.Length; ++i)
            {
                //系统方法及后面的都是系统调用，不获取用户代码调用结束
                if (System.Diagnostics.StackFrame.OFFSET_UNKNOWN == sfs[i].GetILOffset()) break;
                var methedInfo = sfs[i].GetMethod();
                _methodName = methedInfo.ReflectedType.FullName + "." + methedInfo.Name;//方法名称
                _fullName = _methodName + "()\r\n->" + _fullName;
            }
            st = null;
            sfs = null;
            var content = _fullName.TrimEnd('-', '>');
            return content;
        }


        /// <summary>
        /// Find the first table that matches the key and value
        /// </summary>
        /// <param name="key">the key of the data item</param>
        /// <param name="value">the value of the key</param>
        /// <returns>
        /// A MongoTable object
        /// </returns>
        public async Task<T> FindOneAsync(string key, string value, bool onlyNotDeleted = true)
        {
            var logContent = "";
            var fullName = typeof(T).FullName;
            using (new Performance(_performanceThreshold, "Handle FindOneAsync by key {0} execute " + logContent, $"{fullName}"))
            {
                if (string.IsNullOrWhiteSpace(key))
                {
                    throw new RpcException((int)CustomerRpcCode.ColumnNameIsEmpty, "key can't be null", "key can't be null");
                }
                if (string.IsNullOrWhiteSpace(value))
                {
                    Logger.WriteLineWarn($"{fullName} FindOneAsync failed, value is null");
                    throw new RpcException((int)CustomerRpcCode.ColumnValueIsEmpty, $"{key} can't be null", $"{key} can't be null");
                }
                var filter = Builders<T>.Filter.Eq(key, value);
                if (onlyNotDeleted)
                {
                    filter = filter & Builders<T>.Filter.Eq("IsDelete", false);
                }
                var cursor = await _collect.FindAsync(filter);
                var res = await cursor.FirstOrDefaultAsync();
                return res;
            }
        }

        /// <summary>
        /// > Finds the first document in the _collection that matches the filter
        /// </summary>
        /// <param name="filter">The filter is a document that specifies the query criteria.</param>
        /// <param name="options"></param>
        /// <returns>
        /// A cursor.
        /// </returns>
        public async Task<T> FindTableOneAsync(FilterDefinition<T> filter, FindOptions<T> options = null, bool onlyNotDeleted = true)
        {
            using (new Performance(_performanceThreshold, "Handle FindTableOneAsync by filter {0} execute", $"{typeof(T).FullName}"))
            {
                if (onlyNotDeleted)
                {
                    filter = filter & Builders<T>.Filter.Eq("IsDelete", false);
                }
                var cursor = await _collect.FindAsync(filter, options);
                return await cursor.FirstOrDefaultAsync();
            }
        }

        /// <summary>
        /// > Finds one record from the database and returns it as a strongly typed object
        /// </summary>
        /// <param name="filter">The filter to find the document.</param>
        /// <param name="options"></param>
        /// <returns>
        /// The result is a MongoTable object.
        /// </returns>
        public virtual async Task<T> FindOneAsync(FilterDefinition<T> filter, FindOptions<T> options = null, bool onlyNotDeleted = true)
        {
            using (new Performance(_performanceThreshold, "Handle FindOneAsync by filter {0} execute", $"{typeof(T).FullName}"))
            {
                if (onlyNotDeleted)
                {
                    filter = filter & Builders<T>.Filter.Eq("IsDelete", false);
                }
                var cursor = await _collect.FindAsync(filter, options);
                return await cursor.FirstOrDefaultAsync();
            }
        }


        /// <summary>
        /// It counts the number of documents in the _collection.
        /// </summary>
        /// <param name="filter">The filter to apply the count operation to.</param>
        /// <returns>
        /// The number of documents in the _collection.
        /// </returns>
        public async Task<long> CountDocumentsAsync(FilterDefinition<T> filter = null, bool onlyNotDeleted = true)
        {
            using (new Performance(_performanceThreshold, "Handle CountDocumentsAsync {0} execute", $"{typeof(T).FullName}"))
            {
                if (filter == null)
                {
                    filter = Builders<T>.Filter.Empty;
                }
                if (onlyNotDeleted)
                {
                    filter = filter & Builders<T>.Filter.Eq("IsDelete", false);
                }
                return await _collect.CountAsync(filter);
            }
        }


        /// <summary>
        /// > Find all the documents in the _collection that match the filter, and return them as a list
        /// </summary>
        /// <param name="filter">The filter to apply to the query.</param>
        /// <param name="options"></param>
        /// <param name="CancellationToken">A cancellation token that can be used by other objects or threads to
        /// receive notice of cancellation.</param>
        /// <returns>
        /// A list of MongoTable objects.
        /// </returns>
        public async Task<List<T>> FindAllAsync(FilterDefinition<T> filter = null, FindOptions<T> options = null, CancellationToken cancellationToken = default, bool onlyNotDeleted = true)
        {
            using (new Performance(_performanceThreshold, "Handle FindAllAsync {0} execute", $"{typeof(T).FullName}"))
            {
                if (filter == null)
                {
                    filter = Builders<T>.Filter.Empty;
                }
                if (onlyNotDeleted)
                {
                    filter = filter & Builders<T>.Filter.Eq("IsDelete", false);
                }
                var cursor = await _collect.FindAsync(filter, options, cancellationToken);
                return cursor.ToList();
            }
        }


        /// <summary>
        /// > Find all the documents in the _collection that match the filter, and return them as a list
        /// </summary>
        /// <param name="filter">The filter to apply to the query.</param>
        /// <param name="options"></param>
        /// <param name="CancellationToken">A cancellation token that can be used by other objects or threads to
        /// receive notice of cancellation.</param>
        /// <returns>
        /// A list of MongoTable objects.
        /// </returns>
        private async Task<List<T>> FindAllTableAsync(FilterDefinition<T> filter = null, FindOptions<T> options = null, CancellationToken cancellationToken = default, bool onlyNotDeleted = true)
        {
            using (new Performance(_performanceThreshold, "Handle FindAllTableAsync {0} execute", $"{typeof(T).FullName}"))
            {
                if (filter == null)
                {
                    filter = Builders<T>.Filter.Empty;
                }
                if (onlyNotDeleted)
                {
                    filter = filter & Builders<T>.Filter.Eq("IsDelete", false);
                }
                var cursor = await _collect.FindAsync(filter, options, cancellationToken);
                return cursor.ToList();
            }
        }


        /// <summary>
        /// > Find all the tables that have the same key and value in the query data
        /// </summary>
        /// <param name="filter">a dictionary of key-value pairs, where the key is the name of the field, and
        /// the value is a list of values.</param>
        /// <returns>
        /// A list of MongoTable objects.
        /// </returns>
        private async Task<List<T>> FindTableInAsync(Dictionary<string, List<string>> filter = null)
        {

            var definitions = new List<FilterDefinition<T>>();
            foreach (var para in filter)
            {
                definitions.Add(Builders<T>.Filter.In(para.Key, para.Value));
            }
            var customerFilter = Builders<T>.Filter.And(definitions);
            return await FindAllTableAsync(customerFilter);
        }


        /// <summary>
        /// > This function takes a dictionary of lists of strings, and returns a list of objects
        /// </summary>
        /// <param name="filter">Dictionary<string, List<string>></param>
        /// <returns>
        /// A list of objects of type T.
        /// </returns>
        public virtual async Task<List<T>> FindInAsync(Dictionary<string, List<string>> filter = null)
        {
            var result = await FindTableInAsync(filter);
            return result;
        }

        /// <summary>
        /// > This function takes a dictionary of filter parameters and returns a list of objects of type T
        /// </summary>
        /// <param name="filter">Dictionary<string, string></param>
        /// <returns>
        /// A list of objects of type T.
        /// </returns>
        public async Task<List<T>> FindListAsync(Dictionary<string, string> filter, bool onlyNotDeleted = true)
        {
            using (new Performance(_performanceThreshold, "Handle FindListAsync {0} execute", $"{typeof(T).FullName}"))
            {
                var definitions = new List<FilterDefinition<T>>();
                foreach (var para in filter)
                {
                    definitions.Add(Builders<T>.Filter.Eq(para.Key, para.Value));
                }
                var customerFilter = Builders<T>.Filter.And(definitions);
                if (onlyNotDeleted)
                {
                    customerFilter = customerFilter & Builders<T>.Filter.Eq("IsDelete", false);
                }
                FindOptions<T> newOptions = new FindOptions<T>()
                {
                    NoCursorTimeout = true
                };
                var cursor = await _collect.FindAsync(customerFilter, newOptions);
                return cursor.ToList();
            }
        }


        /// <summary>
        /// Get the data of the current page according to the filter condition, sort condition, current page
        /// number and page size
        /// </summary>
        /// <param name="DBPageRequest"></param>
        /// <returns>
        /// A PageCollection of MongoTable.
        /// </returns>
        public async Task<PageCollection<T>> GetTablePageAsync(DBPageRequest<T> request, bool onlyNotDeleted = true)
        {
            using (new Performance(_performanceThreshold, "Handle GetTablePageAsync {0} execute", $"{typeof(T).FullName}"))
            {
                if (request.Filter == null)
                {
                    request.Filter = Builders<T>.Filter.Empty;
                }
                if (onlyNotDeleted)
                {
                    request.Filter = request.Filter & Builders<T>.Filter.Eq("IsDelete", false);
                }
                var count = await _collect.CountDocumentsAsync(request.Filter);
                if (request.Sort == null)
                {
                    request.Sort = Builders<T>.Sort.Descending("CreateTime");
                }
                var cursor = await _collect.FindAsync(request.Filter, new FindOptions<T> { NoCursorTimeout = true, Skip = (request.CurrentPage - 1) * request.PageSize, Limit = request.PageSize, Sort = request.Sort });
                var pageCollection = new PageCollection<T>();
                pageCollection.CurrentPage = request.CurrentPage;
                pageCollection.PageSize = request.PageSize;
                pageCollection.DataCount = (int)count;
                pageCollection.PageData = await cursor.ToListAsync();
                return pageCollection;
            }
        }




        /// <summary>
        /// It inserts a document into a _collection.
        /// </summary>
        /// <param name="T">The type of the data to be inserted</param>
        /// <param name="platformDatas">The data of the platform, which is the data of the platform, which is
        /// the data of the platform, which is the data of the platform, which is the data of the platform,
        /// which is the data of the platform, which is the data of the platform, which is the data of
        /// the</param>
        /// <param name="extensionData">The data that needs to be stored in the MongoDB database, which is a
        /// string type, and the data is stored in the form of a JSON string.</param>
        /// isWriteOperationLog  isWriteOperationLog default true
        public async Task<string> InsertOneAsync(T data, bool isWriteOperationLog = true)
        {
            using (new Performance(_performanceThreshold, "Handle InsertOneAsync {0} execute", $"{typeof(T).FullName}"))
            {
                //验重
                var (uniqueDic, table) = MongoRepertoryCenter.Instance.SetTableData(data);
                var result = await OrTableExists(uniqueDic);
                if (result?.Count > 0)
                {
                    throw new System.Exception($"{result.FirstOrDefault().Key} is duplicate key,value is {result.FirstOrDefault().Value}");
                }
                if (isWriteOperationLog || !_isDistributed)
                {
                    await _collect.InsertOneAsync(table);
                }
                else
                {
                    await ((MongoCollectionProxy<T>)_collect).InsertOneWithoutWriteLogAsync(table);
                }
                return data.Code;
            }
        }

        /// <summary>
        /// It inserts a document into a _collection.
        /// </summary>
        /// <param name="T">The type of the data to be inserted</param>
        /// <param name="platformDatas">The data of the platform, which is the data of the platform, which is
        /// the data of the platform, which is the data of the platform, which is the data of the platform,
        /// which is the data of the platform, which is the data of the platform, which is the data of
        /// the</param>
        /// <param name="extensionData">The data that needs to be stored in the MongoDB database, which is a
        /// string type, and the data is stored in the form of a JSON string.</param>
        /// isWriteOperationLog  isWriteOperationLog default true
        public async Task<string> InsertOneForAsyncDBAsync(T data, bool isWriteOperationLog = true)
        {
            using (new Performance(_performanceThreshold, "Handle InsertOneAsync {0} execute", $"{typeof(T).FullName}"))
            {
                //验重
                var (uniqueDic, table) = MongoRepertoryCenter.Instance.SetTableData(data);
                if (isWriteOperationLog || !_isDistributed)
                {
                    await _collect.InsertOneAsync(table);
                }
                else
                {
                    await ((MongoCollectionProxy<T>)_collect).InsertOneWithoutWriteLogAsync(table);
                }
                return data.Code;
            }
        }


        /// <summary>
        /// > Insert a list of data into the database, and if the data already exists, skip it
        /// </summary>
        /// <param name="infos">a list of tuples, each tuple contains a data object, a dictionary of platform
        /// data, and an extension data string.</param>
        /// <param name="isWriteOperationLog">add oplog</param>
        public async Task<long> InsertAllAsync(List<T> infos, bool isWriteOperationLog = true)
        {
            using (new Performance(_performanceThreshold, "Handle InsertAllAsync {0} execute", $"{typeof(T).FullName}"))
            {
                var writeCount = 0;
                foreach (var info in infos)
                {
                    var (uniqueDic, table) = MongoRepertoryCenter.Instance.SetTableData(info);
                    var existsList = await OrTableExists(uniqueDic);
                    if (existsList?.Count > 0)
                    {
                        continue;
                    }
                    if (isWriteOperationLog || !_isDistributed)
                    {
                        await _collect.InsertOneAsync(info);
                    }
                    else
                    {
                        await ((MongoCollectionProxy<T>)_collect).InsertOneWithoutWriteLogAsync(info);
                    }
                    writeCount++;
                }
                return writeCount;
            }
        }

        /// <summary>
        ///  Insert a list of data into the database, and if the data already exists, skip it
        /// </summary>
        /// <param name="infos">a list of tuples, each tuple contains a data object, a dictionary of platform
        /// data, and an extension data string.</param>
        /// <param name="isWriteOperationLog">add oplog</param>
        public async Task<bool> InsertAllWithoutOplogLogAsync(List<T> infos)
        {
            using (new Performance(_performanceThreshold, "Handle InsertAllWithoutOplogLogAsync {0} execute", $"{typeof(T).FullName}"))
            {
                if (!_isDistributed)
                {
                    await _collect.InsertManyAsync(infos);
                }
                else
                {
                    await ((MongoCollectionProxy<T>)_collect).InsertManyWithoutWriteLogAsync(infos);
                }

            }
            return true;
        }

        /// <summary>
        ///  Insert a list of data into the database, and if the data already exists, skip it
        /// </summary>
        /// <param name="infos">a list of tuples, each tuple contains a data object, a dictionary of platform
        /// data, and an extension data string.</param>
        /// <param name="isWriteOperationLog">add oplog</param>
        public async Task<bool> UpdateAllWithoutOplogLogAsync(List<T> infos)
        {
            using (new Performance(_performanceThreshold, "Handle UpdateAllWithoutOplogLogAsync {0} execute", $"{typeof(T).FullName}"))
            {
                var listWrites = new List<WriteModel<T>>();
                foreach (var item in infos)
                {
                    var filter = Builders<T>.Filter.Eq(n => n.Code, item.Code);
                    MongoCustomerUpdate<T> update = new();
                    var type = item.GetType();
                    var props = type.GetProperties();
                    foreach (var prop in props)
                    {
                        if (prop.Name == "Id")
                        {
                            continue;
                        }
                        if (prop.GetCustomAttributes(true).Any(x => x is MongoDB.Bson.Serialization.Attributes.BsonIgnoreAttribute))
                        {
                            continue;
                        }
                        var propGetMethod = prop.GetGetMethod();
                        var propValue = propGetMethod?.Invoke(item, null);
                        update.SetField(prop.Name, propValue);
                    }
                    listWrites.Add(new UpdateOneModel<T>(filter, update.AllUpdateDefinition));
                }
                if (!_isDistributed)
                {
                    await _collect.BulkWriteAsync(listWrites);
                }
                else
                {
                    await ((MongoCollectionProxy<T>)_collect).BulkWriteAsync(listWrites);
                }

            }
            return true;
        }

        /// <summary>
        /// > Delete all documents that match the specified conditions
        /// </summary>
        /// <param name="infos">a list of tuples, each tuple contains a key and a value.</param>
        /// <returns>
        /// The number of documents deleted.
        /// </returns>
        public async Task<long> DeleteAllAsync(List<(string key, string val)> infos)
        {
            using (new Performance(_performanceThreshold, "Handle DeleteAllAsync {0} execute", $"{typeof(T).FullName}"))
            {
                var deleteCount = 0;
                foreach (var info in infos)
                {
                    var customerFilter = Builders<T>.Filter.Eq(info.key, info.val);
                    var update = Builders<T>.Update.Set(x => x.IsDelete, true);
                    //假删除
                    await _collect.UpdateOneAsync(customerFilter, update);
                    //await _collect.DeleteOneAsync(customerFilter);
                    deleteCount++;
                }
                return deleteCount;
            }
        }


        /// <summary>
        /// Delete the data in the MongoDB database according to the specified key and value
        /// </summary>
        /// <param name="key">The key of the data item</param>
        /// <param name="value">the value of the key</param>
        /// <returns>
        /// The result of the DeleteOneAsync method is a DeleteResult object.
        /// </returns>
        public async Task<bool> DeleteOneAsync(string key, string value, bool onlyNotDeleted = true)
        {
            using (new Performance(_performanceThreshold, "Handle DeleteOneAsync {0} execute", $"{typeof(T).FullName}"))
            {
                var customerFilter = Builders<T>.Filter.Eq(key, value);
                if (onlyNotDeleted)
                {
                    customerFilter = customerFilter & Builders<T>.Filter.Eq("IsDelete", false);
                }
                var update = Builders<T>.Update.Set(x => x.IsDelete, true);
                //假删除
                var result = await _collect.UpdateOneAsync(customerFilter, update);
                //var result = await _collect.DeleteOneAsync(customerFilter);
                if (result.ModifiedCount > 0)
                {
                    return true;
                }
                return false;
            }
        }

        /// <summary>
        /// > Delete all documents that match the specified conditions
        /// </summary>
        /// <param name="infos">a list of tuples, each tuple contains a key and a value.</param>
        /// <returns>
        /// The number of documents deleted.
        /// </returns>
        public async Task<long> RealDeleteAllAsync(List<(string key, string val)> infos)
        {
            using (new Performance(_performanceThreshold, "Handle RealDeleteAllAsync {0} execute", $"{typeof(T).FullName}"))
            {
                var deleteCount = 0;
                foreach (var info in infos)
                {
                    var customerFilter = Builders<T>.Filter.Eq(info.key, info.val);
                    await _collect.DeleteOneAsync(customerFilter);
                    deleteCount++;
                }
                return deleteCount;
            }
        }


        /// <summary>
        /// Delete the data in the MongoDB database according to the specified key and value
        /// </summary>
        /// <param name="key">The key of the data item</param>
        /// <param name="value">the value of the key</param>
        /// <returns>
        /// The result of the DeleteOneAsync method is a DeleteResult object.
        /// </returns>
        public async Task<bool> RealDeleteOneAsync(string key, string value)
        {
            using (new Performance(_performanceThreshold, "Handle RealDeleteOneAsync {0} execute", $"{typeof(T).FullName}"))
            {
                var customerFilter = Builders<T>.Filter.Eq(key, value);
                var result = await _collect.DeleteOneAsync(customerFilter);
                if (result.DeletedCount > 0)
                {
                    return true;
                }
                return false;
            }
        }


        /// <summary>
        /// A function that updates the data in the database.
        /// </summary>
        /// <param name="key">the key of the data item</param>
        /// <param name="value">the value of the key</param>
        /// <param name="MongoTable">The data type of the data to be updated</param>
        /// <param name="canNull">Whether to allow null values to be updated</param>
        ///  <param name="isWriteOperationLog">add oplog</param>
        /// <returns>
        /// A boolean value.
        /// </returns>
        public async Task<bool> Update(string key, string value, T data, bool canNull = false, bool isWriteOperationLog = true, bool onlyNotDeleted = true)
        {
            using (new Performance(_performanceThreshold, "Handle Update {0} execute", $"{typeof(T).FullName}"))
            {
                var filter = Builders<T>.Filter.Eq(key, value);
                if (onlyNotDeleted)
                {
                    filter = filter & Builders<T>.Filter.Eq("IsDelete", false);
                }
                var cursor = await _collect.FindAsync(filter);
                if (await cursor.MoveNextAsync())
                {
                    var first = cursor.Current.FirstOrDefault();
                    MongoRepertoryCenter.Instance.UpgradeData<T>(first, data, canNull);
                    MongoCustomerUpdate<T> update = new();

                    var type = first.GetType();
                    var props = type.GetProperties();
                    foreach (var item in props)
                    {
                        if (item.Name == "Id")
                        {
                            continue;
                        }
                        if (item.GetCustomAttributes(true).Any(x => x is MongoDB.Bson.Serialization.Attributes.BsonIgnoreAttribute))
                        {
                            continue;
                        }
                        var propGetMethod = item.GetGetMethod();
                        var propValue = propGetMethod?.Invoke(first, null);
                        update.SetField(item.Name, propValue);
                    }
                    if (isWriteOperationLog || !_isDistributed)
                    {
                        var result = await _collect.UpdateOneAsync(filter, update.AllUpdateDefinition);
                        return true;
                    }
                    else
                    {
                        var result = await ((MongoCollectionProxy<T>)_collect).UpdateOneWithoutWriteLogAsync(filter, update.AllUpdateDefinition);
                        return true;
                    }
                }
                return false;
            }
        }


        /// <summary>
        /// > Find the first table that has a data item with a key and value that matches the key and value in
        /// the dictionary
        /// </summary>
        /// <param name="filter">a dictionary of key-value pairs</param>
        /// <returns>
        /// A cursor.
        /// </returns>
        public async Task<T> FindTableOr(Dictionary<string, string> filter)
        {
            using (new Performance(_performanceThreshold, "Handle FindTableOr {0} execute", $"{typeof(T).FullName}"))
            {
                var definitions = new List<FilterDefinition<T>>();
                foreach (var para in filter)
                {
                    definitions.Add(Builders<T>.Filter.Eq(para.Key, para.Value));
                }
                var customerFilter = Builders<T>.Filter.Or(definitions);
                customerFilter = customerFilter & Builders<T>.Filter.Eq("IsDelete", false);
                var cursor = await _collect.FindAsync(customerFilter);
                await cursor.MoveNextAsync();
                return cursor.Current?.FirstOrDefault();
            }
        }


        /// <summary>
        /// > This function is used to query the data in the MongoDB database
        /// </summary>
        /// <param name="filter">The key value pair of the query condition</param>
        /// <returns>
        /// A list of KeyValuePair<string, string>
        /// </returns>
        public async Task<List<KeyValuePair<string, string>>> OrTableExists(Dictionary<string, string> filter)
        {
            using (new Performance(_performanceThreshold, "Handle OrTableExists {0} execute", $"{typeof(T).FullName}"))
            {
                var result = await FindTableOr(filter);
                var list = new List<KeyValuePair<string, string>>();
                if (result == null)
                {
                    return list;
                }
                var type = result.GetType();
                var props = type.GetProperties();
                foreach (var item in props)
                {
                    var duplicateModel = filter.FirstOrDefault(x => x.Key == item.Name && x.Value == item.GetValue(result).ToString());
                    if (!string.IsNullOrWhiteSpace(duplicateModel.Key))
                    {
                        list.Add(new KeyValuePair<string, string>(duplicateModel.Key, duplicateModel.Value));
                    }
                }
                return list;
            }
        }

        public async Task<bool> Exists(FilterDefinition<T> filter = null)
        {
            using (new Performance(_performanceThreshold, "Handle Exists {0} execute", $"{typeof(T).FullName}"))
            {
                filter = filter & Builders<T>.Filter.Eq("IsDelete", false);
                return await _collect.CountAsync(filter) > 0;
            }
        }


        /// <summary>
        /// > Get the page data from the database, and then convert the data to the type of the current
        /// repository
        /// </summary>
        /// <param name="DBPageRequest"></param>
        /// <returns>
        /// A PageCollection of T.
        /// </returns>
        public virtual async Task<PageCollection<T>> GetPages(DBPageRequest<T> request, bool onlyNotDeleted = true)
        {
            var result = await GetTablePageAsync(request, onlyNotDeleted);
            return result;
        }



        /// <summary>
        /// Get the total number of records that meet the conditions, and then get the data of the current page
        /// </summary>
        /// <param name="filter">The filter condition</param>
        /// <param name="pageIndex">The current page number</param>
        /// <param name="pageSize">The number of records per page</param>
        /// <param name="sort">The sort condition, which is the sort condition of the query.</param>
        /// <returns>
        /// A PageCollection of T.
        /// </returns>
        public async Task<PageCollection<T>> GetPagesByFilter(FilterDefinition<T> filter, int pageIndex = 1, int pageSize = 10, SortDefinition<T> sort = null)
        {
            var pages = new PageCollection<T>()
            {
                CurrentPage = pageIndex,
                PageSize = pageSize
            };
            if (filter == null)
            {
                return pages;
            }
            if (sort == null)
            {
                SortDefinitionBuilder<T> builderSort = Builders<T>.Sort;
                sort = builderSort.Descending("CreateTime");
            }
            pages.DataCount = (int)await CountDocumentsAsync(filter);
            pages.PageData = await FindAllAsync(filter, new FindOptions<T> { NoCursorTimeout = true, Skip = (pageIndex - 1) * pageSize, Limit = pageSize, Sort = sort });
            return pages;
        }
    }
}
