using System.Threading.Tasks;
using WingInterfaceLibrary.Interface.DBInterface;
using WingInterfaceLibrary.DTO.Management;
using WingServerCommon.Service;
using WingInterfaceLibrary.Result;
using WingMongoDBModule.DO;
using WingInterfaceLibrary.DB.Request;
using WingServerCommon.Mapper;
using System.Collections.Generic;
using System.Linq;
using System;
using WingMongoDBModule.Mongodb.Driver;
using WingMongoDBModule.Entity;
using WingInterfaceLibrary.Request.Management;
using MongoDB.Driver;
using WingMongoDBModule.Common;
using WingMongoDBModule.Request;
using WingServerCommon.Utilities;
using WingInterfaceLibrary.DTO.User;
using WingInterfaceLibrary.Result.Management;
using WingInterfaceLibrary.DTO.CMSStatistic;
using WingInterfaceLibrary.Enum;
using WingServerCommon.Interfaces.Cache;
using WingInterfaceLibrary.DTO.GradingProtection;
using WingMongoDBModule.Entities;
using MongoDB.Bson;
using WingServerCommon.Log;
using Newtonsoft.Json;
using System.IO;
using Newtonsoft.Json.Linq;
using MongoDB.Driver.GeoJsonObjectModel;

namespace WingMongoDBModule.Service
{
    public partial class DatabaseService : JsonRpcService, IAdminDBService
    {

        /// <summary>
        /// 通过代码删除管理员
        /// </summary>
        /// <param name="code">管理员的唯一标识符。</param>
        /// <returns>
        /// 一个布尔值。
        /// </returns>
        public async Task<bool> DeleteAdminByCodeAsync(string code)
        {
            using (new Performance(_performanceThreshold, "{0} DeleteAdminByCodeAsync execute", $"{this.GetType().Name}"))
            {
                var entity = await FindAdminByCodeAsync(code);
                if (entity != null && !string.IsNullOrEmpty(entity.AdminCode))
                {
                    if (entity.RoleCodes.Contains("AdminRoleDO01"))
                    {
                        var list = await FindSupperAdminAsync();
                        if (list?.Count > 1)
                        {
                            return await _adminDBRepository.DeleteOneAsync("Code", code);
                        }
                        else
                        {
                            return false;
                        }
                    }
                    else
                    {
                        return await _adminDBRepository.DeleteOneAsync("Code", code);
                    }
                }
                return false;
            }
        }


        /// <summary>
        /// 按照角色代码查找管理员
        /// </summary>
        /// <param name="adminRoleCode">管理员的角色代码。</param>
        /// <returns>
        /// AdminInfoDTO 对象的列表。
        /// </returns>
        public async Task<List<AdminInfoDTO>> FindSupperAdminAsync(string adminRoleCode = "AdminRoleDO01")
        {
            using (new Performance(_performanceThreshold, "{0} FindSupperAdminAsync execute", $"{this.GetType().Name}"))
            {
                MongoCustomerFilter<MongoTable> customerFilter = new();
                MongoCustomerFilter<MongoDataItem> queryItemsFilter = new();
                customerFilter.AppendArrayEq("BaseData.RoleCodes", adminRoleCode);
                var adminList = await _adminDBRepository.FindAllAsync(customerFilter.AndCombine(), new FindOptions<MongoTable> { NoCursorTimeout = true });
                return adminList.MappingTo<List<AdminInfoDTO>>();
            }
        }


        /// <summary>
        /// 通过代码查找管理员
        /// </summary>
        /// <param name="code">管理员代码</param>
        /// <returns>
        /// AdminInfoDTO
        /// </returns>
        public async Task<AdminInfoDTO> FindAdminByCodeAsync(string code)
        {
            using (new Performance(_performanceThreshold, "{0} FindAdminByCodeAsync execute", $"{this.GetType().Name}"))
            {
                var adminDO = await _adminDBRepository.FindOneAsync("Code", code);
                return adminDO.MappingTo<AdminInfoDTO>();
            }
        }


        /// <summary>
        /// > 按名称查找管理员并返回管理员信息
        /// </summary>
        /// <param name="name">管理员姓名</param>
        /// <returns>
        /// AdminInfoDTO
        /// </returns>
        public async Task<AdminInfoDTO> FindAdminByNameAsync(string name)
        {
            using (new Performance(_performanceThreshold, "{0} FindAdminByNameAsync execute", $"{this.GetType().Name}"))
            {
                var adminDO = await _adminDBRepository.FindOneAsync("AdminName", name);
                var info = new AdminInfoDTO();
                try
                {
                    info = adminDO.MappingTo<AdminInfoDTO>();
                    info.LoginLockInfo = adminDO.LoginLockInfo == null ? null : new LoginLockInfoDTO() {
                            LoginDate = adminDO.LoginLockInfo.LoginDate,
                            Times = adminDO.LoginLockInfo.Times
                        };
                    info.ClearLogOperateList = adminDO.ClearLogOperateList == null ? null : adminDO.ClearLogOperateList.Select(c => new ClearLogOperateDTO() {
                            ServerHost = c.ServerHost,
                            LastClearTime = c.LastClearTime,
                            LastClearOperateTime = c.LastClearOperateTime,
                        }).ToList();
                }
                catch (Exception ex)
                {
                    Logger.WriteLineError($"AdminDBService FindAdminByNameAsync Error: {ex}");
                }
                return info;
            }
        }


        /// <summary>
        /// > 查找名称中包含指定字符串的所有管理员
        /// </summary>
        /// <param name="name">要搜索的管理员的名称。</param>
        /// <returns>
        /// AdminInfoDTO 对象的列表。
        /// </returns>
        public async Task<List<AdminInfoDTO>> FindAdminsByNameAsync(string name)
        {
            using (new Performance(_performanceThreshold, "{0} FindAdminsByNameAsync execute", $"{this.GetType().Name}"))
            {
                var source = await _adminDBRepository.FindAdminsByNameAsync(name);
                return source.MappingTo<List<AdminInfoDTO>>();
            }
        }

        /// <summary>
        /// > 将新管理员插入数据库
        /// </summary>
        /// <param name="CreateAdminDBRequest">客户端发送给服务器的请求对象。</param>
        /// <returns>
        /// 新创建的管理员的 ID。
        /// </returns>
        public async Task<string> InsertAdminAsync(CreateAdminDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} InsertAdminAsync execute", $"{this.GetType().Name}"))
            {
                var adminDO = request.Data.MappingTo<AdminInfoDO>();
                if (!adminDO.SecretPassword.StartsWith("Symmetry_"))
                {
                    adminDO.SecretPassword = $"Symmetry_{CryptionHelper.Encrypt(adminDO.SecretPassword)}";
                }
                return await _adminDBRepository.InsertOneAsync(adminDO, request.PlatformDatas, request.ExtensionData);
            }
        }


        /// <summary>
        /// 修改管理员密码。
        /// </summary>
        /// <param name="code">管理员代码</param>
        /// <param name="newPassword">要修改的新密码</param>
        public async Task<bool> ModifyAdminPasswordAsync(string code, AdminInfoDTO entity)
        {
            using (new Performance(_performanceThreshold, "{0} ModifyAdminPasswordAsync execute", $"{this.GetType().Name}"))
            {
                entity.LoginLockInfo = new LoginLockInfoDTO();
                var adminDO = entity.MappingTo<AdminInfoDO>();
                adminDO.PasswordUpdateTime = DateTime.UtcNow;
                if (!string.IsNullOrEmpty(entity.HeadImageToken))
                {
                    adminDO.HeadImageToken = entity.HeadImageToken.ToUrlToken();
                }
                if (!adminDO.SecretPassword.StartsWith("Symmetry_"))
                {
                    adminDO.SecretPassword = $"Symmetry_{CryptionHelper.Encrypt(adminDO.SecretPassword)}";
                }
                return await _adminDBRepository.UpdateOneAsync("Code", code, adminDO, "", true);
            }
        }

        /// <summary>
        /// 通过管理员代码更新管理员信息。
        /// </summary>
        /// <param name="code">管理员的唯一标识符。</param>
        /// <param name="AdminInfoDTO">管理员的数据传输对象。</param>
        /// <param name="extensionData">扩展数据用于存储数据库中没有的数据，数据以字符串的形式存储。</param>
        public async Task<bool> UpdateAdminByCodeAsync(string code, AdminInfoDTO data, string extensionData)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateAdminByCodeAsync execute", $"{this.GetType().Name}"))
            {
                var entity = await FindAdminByCodeAsync(code);
                if (entity == null || string.IsNullOrEmpty(entity.AdminCode))
                {
                    return false;
                }
                if (!string.IsNullOrEmpty(data.FullName))
                {
                    entity.FullName = data.FullName;
                }
                if (!string.IsNullOrEmpty(data.AdminName))
                {
                    entity.AdminName = data.AdminName;
                }
                if (!string.IsNullOrEmpty(data.Email))
                {
                    entity.Email = data.Email;
                }
                if (!string.IsNullOrEmpty(data.Phone))
                {
                    entity.Phone = data.Phone;
                }
                if (data.RoleCodes?.Count > 0)
                {
                    if (entity.RoleCodes.Contains("AdminRoleDO01"))
                    {
                        var list = await FindSupperAdminAsync();
                        if (list == null || list.Count <= 1)
                        {
                            return false;
                        }
                    }
                    entity.RoleCodes = data.RoleCodes;
                    // data.RoleCodes.ForEach(c => 
                    // {
                    //     if (!entity.RoleCodes.Exists(x => x == c))
                    //     {
                    //         entity.RoleCodes.Add(c); 
                    //     }
                    // });
                }
                if (!string.IsNullOrEmpty(data.SecretPassword))
                {
                    entity.SecretPassword = GetMd5SaltPassword(data.AdminName, data.SecretPassword);
                }
                var adminDO = entity.MappingTo<AdminInfoDO>();
                if (!string.IsNullOrEmpty(entity.HeadImageToken))
                {
                    adminDO.HeadImageToken = entity.HeadImageToken.ToUrlToken();
                }
                adminDO.BindUserCode = data.BindUserCode;
                adminDO.BindUserName = data.BindUserName;
                return await _adminDBRepository.UpdateOneAsync("Code", code, adminDO, extensionData, true);
            }
        }


        /// <summary>
        /// > 从请求中获取过滤器，然后从数据库中获取页面结果，最后将页面结果转换为DTO
        /// </summary>
        /// <param name="GetAdminPagesRequest">请求参数是接口的请求参数，也就是接口的参数。</param>
        public async Task<PageResult<AdminInfoDTO>> FindAdminPagesAsync(GetAdminPagesRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} FindAdminPagesAsync execute", $"{this.GetType().Name}"))
            {
                var customerFilter = GetFilter(request);
                var dbPage = new DBPageRequest();
                dbPage.CurrentPage = request.PageIndex;
                dbPage.PageSize = request.PageSize;
                dbPage.Filter = customerFilter;
                var pageResult = await _adminDBRepository.GetPages(dbPage);
                return PageResultConvert<AdminInfoDO, AdminInfoDTO>(pageResult);
            }
        }

        /// <summary>
        /// > 该函数用于获取查询请求的过滤条件
        /// </summary>
        /// <param name="GetAdminPagesRequest">请求参数类，用于接收前端发送的请求参数。</param>
        private FilterDefinition<MongoTable> GetFilter(GetAdminPagesRequest queryRequest)
        {
            List<FilterDefinition<MongoDataItem>> queryItemsFilterList = new();
            MongoCustomerFilter<MongoTable> customerFilter = new();
            MongoCustomerFilter<MongoDataItem> queryItemsFilter = new();
            var items = new List<string> { "AdminName", "FullName", "AdminCode" };
            if (!string.IsNullOrWhiteSpace(queryRequest.KeyWord))
            {
                foreach (var item in items)
                {
                    queryItemsFilter = new();
                    queryItemsFilter.AppendEq((item => item.Key, item));
                    queryItemsFilter.AppendRegex(item => item.Value, queryRequest.KeyWord, true, false);
                    queryItemsFilterList.Add(queryItemsFilter.AndCombine());
                }
            }
            if (queryItemsFilter.HasFilter)
            {
                var queryItemAllFilter = queryItemsFilter.OrCombine(queryItemsFilterList.ToArray());
                customerFilter.SubFilter(item => item.QueryData.QueryItems, queryItemAllFilter);
            }
            if (!string.IsNullOrEmpty(queryRequest.FatherCode))
            {
                customerFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.Or, ("Code", queryRequest.FatherCode), ("FatherCode", queryRequest.FatherCode));
            }
            if (!string.IsNullOrEmpty(queryRequest.RoleCode))
            {
                customerFilter.AppendArrayEq("BaseData.RoleCodes", queryRequest.RoleCode);
            }
            if (!queryRequest.IsShowSuperRoleCode)
            {
                customerFilter.AppendNotArrayEq("BaseData.RoleCodes", "AdminRoleDO01");
            }
            return customerFilter.AndCombine();
        }

        /// <summary>
        /// FindAdminListByRoleCodeAsync 用于按角色代码查找管理员列表
        /// </summary>
        /// <param name="roleCode">角色代码</param>
        /// <returns>
        /// AdminInfoDTO 对象的列表。
        /// </returns>
        public async Task<List<AdminInfoDTO>> FindAdminListByRoleCodeAsync(string roleCode)
        {
            using (new Performance(_performanceThreshold, "{0} FindAdminListByRoleCodeAsync execute", $"{this.GetType().Name}"))
            {
                var adminList = await _adminDBRepository.FindListAsync(new Dictionary<string, string>());
                var adminDOList = adminList.Where(a => a.RoleCodes != null && a.RoleCodes.Contains(roleCode)).ToList();
                return adminDOList.MappingTo<List<AdminInfoDTO>>();
            }
        }

        /// <summary>
        /// FindAdminListByRoleCodesAsync 用于按角色代码查找管理员列表
        /// </summary>
        /// <param name="roleCodes">需要查询的管理员角色代码列表</param>
        /// <returns>
        /// 列表<AdminInfoDTO>
        /// </returns>
        public async Task<List<AdminInfoDTO>> FindAdminListByRoleCodesAsync(List<string> roleCodes)
        {
            using (new Performance(_performanceThreshold, "{0} FindAdminListByRoleCodesAsync execute", $"{this.GetType().Name}"))
            {
                var result = new List<AdminInfoDTO>();
                if (roleCodes == null && roleCodes.Count == 0)
                {
                    return result;
                }
                var adminList = await _adminDBRepository.FindListAsync(new Dictionary<string, string>());
                foreach (var roleCode in roleCodes)
                {
                    var adminDOList = adminList.Where(a => a.RoleCodes != null && a.RoleCodes.Contains(roleCode)).ToList();
                    result.AddRange(adminDOList.MappingTo<List<AdminInfoDTO>>());
                }
                return result;
            }
        }

        /// <summary>
        /// FindAdminListByAdminCodesAsync
        /// </summary>
        /// <param name="adminCodes">要查询的管理员代码列表</param>
        /// <returns>
        /// AdminInfoDTO 对象的列表。
        /// </returns>
        public async Task<List<AdminInfoDTO>> FindAdminListByAdminCodesAsync(List<string> adminCodes)
        {
            using (new Performance(_performanceThreshold, "{0} FindAdminListByAdminCodesAsync execute", $"{this.GetType().Name}"))
            {
                var result = new List<AdminInfoDTO>();
                if (adminCodes == null && adminCodes.Count == 0)
                {
                    return result;
                }
                var dic = new Dictionary<string, List<string>>();
                dic.Add("Code", adminCodes);
                var adminList = await _adminDBRepository.FindInAsync(dic);
                result = adminList.MappingTo<List<AdminInfoDTO>>();
                return result;
            }
        }

        /// <summary>
        /// 更新用户锁定信息
        /// </summary>
        /// <param name="code">用户编码</param>
        /// <param name="LoginLockInfoDTO">锁定信息</param>
        /// <returns></returns>
        public async Task<bool> UpdateAdminLoginLockInfoAsync(string code, LoginLockInfoDTO loginLockInfoDTO)
        {
            var adminDO = await _adminDBRepository.FindOneAsync("Code", code);
            if (loginLockInfoDTO == null)
            {
                loginLockInfoDTO = new LoginLockInfoDTO();
            }
            adminDO.LoginLockInfo = loginLockInfoDTO.MappingTo<LoginLockInfoDO>();
            return await _adminDBRepository.UpdateOneAsync("Code", code, adminDO, null, true);
        }

        /// <summary>
        /// 查询CMS模板详情
        /// </summary>
        /// <param name="request">token请求实体</param>
        /// <returns></returns>
        public async Task<CMSTemplateResult> FindCMSTemplateDetailDBAsync(FindCMSTemplateDetailDBRequest request)
        {
            MongoCustomerFilter<MongoTable> mongoFilter = new MongoCustomerFilter<MongoTable>();
            mongoFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("Code", request.CMSTemplateCode));
            var cms = await _cmsTemplateDBRepository.FindOneAsync(mongoFilter.AndCombine());
            return new CMSTemplateResult { CMSBorderColor = cms.CMSBorderColor, CMSBorderShadow = cms.CMSBorderShadow, IsShowTop = cms.IsShowTop, CMSBgColor = cms.CMSBgColor, CMSBgImg = cms.CMSBgImg, IsDefault = cms.IsDefault, CMSTemplateCode = cms.CMSTemplateCode, CMSTemplateName = cms.CMSTemplateName, CMSTemplateStateType = cms.CMSTemplateStateType, CMSTemplateJson = cms.CMSTemplateJson };
        }

        /// <summary>
        /// 查询CMS模板详情
        /// </summary>
        /// <param name="request">token请求实体</param>
        /// <returns></returns>
        public async Task<CMSTemplateResult> GetCMSTemplateDetailByUserCodeAsync(string userCode)
        {
            string cmsTemplateCode = string.Empty;
            var user = await _userDBRepository.FindOneAsync("Code", userCode);
            if (user != null && !string.IsNullOrWhiteSpace(user.OrganizationCode))
            {
                var orgInfo = CacheMaintenance.Instance.Get<IOrganizationsManager>().Get(user.OrganizationCode);
                var rootOrganizationCode = orgInfo?.RootCode;
                if (!string.IsNullOrWhiteSpace(rootOrganizationCode))
                {
                    var organization = await _organizationDBRepository.FindOneAsync("Code", rootOrganizationCode);
                    cmsTemplateCode = organization?.CMSTemplateCode;
                }
            }
            MongoCustomerFilter<MongoTable> customerFilter = new();
            customerFilter.AppendIn("BaseData.IsDefault", new List<bool> { true });
            if (!string.IsNullOrWhiteSpace(cmsTemplateCode))
            {
                customerFilter.AppendIn("BaseData.Code", new List<string> { cmsTemplateCode });
            }
            var filter = customerFilter.OrCombine();
            var cmsTemplates = await _cmsTemplateDBRepository.FindAllAsync(filter);
            var cms = cmsTemplates.OrderBy(x => x.IsDefault).FirstOrDefault();
            if (cms != null)
            {
                return new CMSTemplateResult { CMSBorderShadow = cms.CMSBorderShadow, CMSBorderColor = cms.CMSBorderColor, IsShowTop = cms.IsShowTop, CMSBgColor = cms.CMSBgColor, CMSBgImg = cms.CMSBgImg, IsDefault = cms.IsDefault, CMSTemplateCode = cms.CMSTemplateCode, CMSTemplateName = cms.CMSTemplateName, CMSTemplateStateType = cms.CMSTemplateStateType, CMSTemplateJson = cms.CMSTemplateJson };
            }
            return null;
        }

        /// <summary>
        /// 查询CMS模版分页
        /// </summary>
        /// <param name="request">token请求实体</param>
        /// <returns></returns>
        public async Task<PageResult<CMSTemplateResult>> FindCMSTemplatePageDBAsync(FindCMSTemplatePageDBRequest request)
        {
            MongoCustomerFilter<MongoTable> mongoFilter = new MongoCustomerFilter<MongoTable>();
            if (!string.IsNullOrWhiteSpace(request.SearchKey))
            {
                mongoFilter.AppendQueryItemRegexFilter(MongoCombineTypeEnum.And, ("CMSTemplateName", request.SearchKey, true, false));
            }
            var pageData = await _cmsTemplateDBRepository.GetPages(new DBPageRequest
            {
                CurrentPage = request.PageIndex,
                PageSize = request.PageSize,
                Filter = mongoFilter.AndCombine()
            });
            return new PageResult<CMSTemplateResult>
            {
                PageIndex = pageData.CurrentPage,
                PageSize = pageData.PageSize,
                PageData = pageData.PageData.Select(f => new CMSTemplateResult
                {
                    CMSTemplateCode = f.CMSTemplateCode,
                    CMSTemplateName = f.CMSTemplateName,
                    CMSTemplateJson = f.CMSTemplateJson,
                    CMSTemplateStateType = f.CMSTemplateStateType,
                    CreateTime = f.CreateTime,
                    IsDefault = f.IsDefault,
                }).ToList(),
                TotalCount = pageData.DataCount
            };
        }

        /// <summary>
        /// 查询CMS统计接口
        /// </summary>
        /// <param name="request">token请求实体</param>
        /// <returns></returns>
        public async Task<IList<CMSStatisticRecordDTO>> FindCMSStatisticRecordsDBAsync(FindCMSStatisticDBRequest request)
        {
            MongoCustomerFilter<MongoTable> mongoFilter = new MongoCustomerFilter<MongoTable>();
            mongoFilter.AppendEq(("BaseData.StatisticsPlatform", StatisticsPlatformEnum.CMS));
            mongoFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("UserCode", request.UserCode));
            var cmsStatisticRecordList = await _statisticRecordDBRepository.FindAllAsync(mongoFilter.AndCombine());
            return cmsStatisticRecordList.Select(f => new CMSStatisticRecordDTO { StatisticJsonData = f.StatisticJsonData, AdminStatisticName = f.StatisticName }).ToList();
        }

        /// <summary>
        /// 查找等保信息
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageResult<GradingProtectionConfigDTO>> FindPageGradingProtectionConfigPageDBAsync(FindGradingProtectionConfigByPageRequest request)
        {
            var keyword = request.Keyword;
            var builder = Builders<GradingProtectionConfigEntity>.Filter;
            var filter = builder.Where(c => c.IsDelete == false);
            if (!string.IsNullOrEmpty(keyword))
            {
                keyword = EscapeExprSpecialWord(keyword);//字符串转义
                filter = filter & builder.Regex(c => c.Name, new BsonRegularExpression(keyword, "i"));
            }
            var dbPage = new DBPageRequest<GradingProtectionConfigEntity>();
            dbPage.CurrentPage = request.PageIndex;
            dbPage.PageSize = request.PageSize;
            dbPage.Filter = filter;
            var pageResult = await _gradingProtectionConfigDBRepository.GetPages(dbPage);
            return PageResultConvert<GradingProtectionConfigEntity, GradingProtectionConfigDTO>(pageResult);
        }
        private string EscapeExprSpecialWord(string keyword)
        {
            string[] fbsArr = { "\\", "$", "(", ")", "*", "+", ".", "[", "]", "?", "^", "{", "}", "|" };
            foreach (var key in fbsArr)
            {
                if (keyword.Contains(key))
                {
                    keyword = keyword.Replace(key, "\\" + key);
                }
            }
            return keyword;
        }
        /// <summary>
        /// 更新选定配置
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> UpdateGradingProtectionConfigEnableAsync(UpdateGradingProtectionConfigEnableRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateGradingProtectionConfigEnableAsync execute", "DatabaseService"))
            {
                var update = Builders<GradingProtectionConfigEntity>.Update
                .Set(c => c.ConfigType, (int)request.ConfigType)
                .Set(c => c.Value, request.Value)
                .Set(c => c.UpdateTime, DateTime.UtcNow);
                var filter = Builders<GradingProtectionConfigEntity>.Filter.Where(c =>
                request.Codes.Contains(c.Code) && c.IsDelete == false);
                return await _gradingProtectionConfigDBRepository.UpdateAllAsync(filter, update) > 0;
            }
        }

        /// <summary>
        /// 检查管理员密码是否正确
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> CheckAdminPasswordAsync(CheckAdminPasswordRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} CheckAdminPasswordAsync execute", $"{this.GetType().Name}"))
            {
                var adminInfo = await _adminDBRepository.FindOneAsync("Code", request.AdminCode);
                if (adminInfo.SecretPassword.StartsWith("ServerSalt_"))
                {
                    if (adminInfo.SecretPassword == GetMd5SaltPassword(adminInfo.AdminName, request.AdminPassword))
                    {
                        return true;
                    }
                }
                else
                {
                    if (adminInfo.SecretPassword == request.AdminPassword)
                    {
                        return true;
                    }
                }
                return false;
            }
        }

        /// <summary>
        /// 查找等保配置列表
        /// </summary>
        /// <param name="moduleCodes">配置编码集合</param>
        /// <returns></returns>
        public async Task<List<GradingProtectionConfigDTO>> FindGradingProtectionConfigListAsync(List<string> moduleCodes)
        {
            var filter = Builders<GradingProtectionConfigEntity>.Filter.Where(c => moduleCodes.Contains(c.Code) && c.IsDelete == false);
            var result = await _gradingProtectionConfigDBRepository.FindAllAsync(filter);
            return result.MappingTo<List<GradingProtectionConfigDTO>>();
        }

        /// <summary>
        /// 批量新增管理员数据
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>是否成功</returns>
        public async Task<bool> BatchInsertAdminDataDBAsync(BatchInsertAdminRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} BatchInsertAdminDataDBAsync execute", $"{this.GetType().Name}"))
            {
                long res = 0;
                if (request?.AdminMigratorys?.Count > 0)
                {
                    //找到相同的
                    var ids = request.AdminMigratorys.Select(c => c.AdminCode).Distinct().ToList();
                    var dic = new Dictionary<string, List<string>>();
                    dic.Add("Code", ids);
                    var adminList = await _adminDBRepository.FindInAsync(dic);
                    var adminRoleList = await _adminRoleDBRepository.FindAllAsync();
                    var superAdminRoleCode = (adminRoleList?.Find(c => c.Code == "AdminRoleDO01") ?? new AdminRoleDO()).Code;
                    var generalAdminRoleCode = (adminRoleList?.FindAll(c => c.Code != "AdminRoleDO01")?.OrderByDescending(c => c.CreateTime)?.FirstOrDefault() ?? new AdminRoleDO()).Code;
                    //求同存异
                    if (adminList?.Count > 0)
                    {
                        var existCodes = adminList.Select(c => c.Code).Distinct().ToList();
                        var updateData = request.AdminMigratorys.Where(c => existCodes.Contains(c.AdminCode));
                        var insertData = request.AdminMigratorys.Where(c => !existCodes.Contains(c.AdminCode));
                        if (insertData?.Count() > 0)
                        {
                            var names = insertData.Select(c => c.AdminName).Distinct().ToList();
                            dic = new Dictionary<string, List<string>>();
                            dic.Add("AdminName", names);
                            var nameList = await _adminDBRepository.FindInAsync(dic);
                            if (nameList?.Count > 0)
                            {
                                var existNames = nameList.Select(c => c.AdminName).Distinct().ToList();
                                var updateNameData = insertData.Where(c => existNames.Contains(c.AdminName));
                                var insertNameData = insertData.Where(c => !existNames.Contains(c.AdminName));
                                if (insertNameData?.Count() > 0)
                                {
                                    res = await BatchAddAdminData(insertNameData, superAdminRoleCode, generalAdminRoleCode);
                                }
                                if (updateNameData?.Count() > 0)
                                {
                                    Logger.WriteLineWarn($"AdminDBService BatchInsertAdminDataDBAsync Repeat Name, insertData: {JsonConvert.SerializeObject(updateNameData)}");
                                }
                            }
                            else
                            {
                                res = await BatchAddAdminData(insertData, superAdminRoleCode, generalAdminRoleCode);
                            }
                        }
                        if (updateData?.Count() > 0)
                        {
                            var req = updateData.Select(f =>
                            {
                                var updateEntity = f.MappingTo<AdminInfoDO>();
                                updateEntity.Code = updateEntity.AdminCode.Trim();
                                updateEntity.IsDelete = f.IsDelete;
                                if (f.AdminRole == AdminMigratoryRoleEnum.PrimaryAdmin01)
                                {
                                    updateEntity.RoleCodes = new List<string>() {
                                    superAdminRoleCode
                                };
                                }
                                else
                                {
                                    updateEntity.RoleCodes = new List<string>() {
                                    generalAdminRoleCode
                                };
                                }
                                return ("Code", f.AdminCode, updateEntity, string.Empty);
                            }).ToList();
                            res = await _adminDBRepository.UpdateAllAsync(req);
                        }
                    }
                    else
                    {
                        //新增
                        var names = request.AdminMigratorys.Select(c => c.AdminName).Distinct().ToList();
                        dic = new Dictionary<string, List<string>>();
                        dic.Add("AdminName", names);
                        var nameList = await _adminDBRepository.FindInAsync(dic);
                        if (nameList?.Count > 0)
                        {
                            var existNames = nameList.Select(c => c.AdminName).Distinct().ToList();
                            var updateNameData = request.AdminMigratorys.Where(c => existNames.Contains(c.AdminName));
                            var insertNameData = request.AdminMigratorys.Where(c => !existNames.Contains(c.AdminName));
                            if (insertNameData?.Count() > 0)
                            {
                                res = await BatchAddAdminData(insertNameData, superAdminRoleCode, generalAdminRoleCode);
                            }
                            if (updateNameData?.Count() > 0)
                            {
                                Logger.WriteLineWarn($"AdminDBService BatchInsertAdminDataDBAsync Repeat Name, AdminMigratorys: {JsonConvert.SerializeObject(updateNameData)}");
                            }
                        }
                        else
                        {
                            res = await BatchAddAdminData(request.AdminMigratorys, superAdminRoleCode, generalAdminRoleCode);
                        }
                    }
                }
                var result = res > 0 ? true : false;
                return result;
            }
        }

        /// <summary>
        /// 批量新增管理员数据
        /// </summary>
        private async Task<long> BatchAddAdminData(IEnumerable<AdminMigratoryInfo> insertData, string superAdminRoleCode, string generalAdminRoleCode)
        {
            //新增
            var req = insertData.Select(
                f =>
                {
                    var data = f.MappingTo<AdminInfoDO>();
                    data.Code = data.AdminCode.Trim();
                    data.IsDelete = f.IsDelete;
                    if (f.AdminRole == AdminMigratoryRoleEnum.PrimaryAdmin01)
                    {
                        data.RoleCodes = new List<string>() {
                            superAdminRoleCode
                        };
                    }
                    else
                    {
                        data.RoleCodes = new List<string>() {
                            generalAdminRoleCode
                        };
                    }
                    return (data, new Dictionary<string, string>(), string.Empty);
                }
            ).ToList();
            var res = await _adminDBRepository.InsertAllAsync(req);
            return res;
        }

        /// <summary>
        /// 更新密码
        /// </summary>
        /// <param name="code"></param>
        /// <param name="data"></param>
        /// <returns></returns>
        public async Task<bool> UpdateAdminPasswordAsync(string code, string password, IList<string> pwdRecords = null, DateTime? updateTime = null)
        {
            var adminDO = await _adminDBRepository.FindOneAsync("Code", code);
            if (password.StartsWith("Symmetry_"))
            {
                adminDO.SecretPassword = password;
            }
            else
            {
                adminDO.SecretPassword = GetMd5SaltPassword(adminDO.AdminName, password);
            }
            if (pwdRecords?.Count > 0)
            {
                adminDO.PasswordRecords = pwdRecords.ToList();
            }
            if (updateTime != null && updateTime != DateTime.MinValue && updateTime > Convert.ToDateTime("2000-1-1"))
            {
                adminDO.PasswordUpdateTime = updateTime.Value;
            }
            adminDO.LoginLockInfo = new LoginLockInfoDO();
            return await _adminDBRepository.UpdateOneAsync("Code", code, adminDO, "", false);
        }

        #region db version 1.2 upgrade 
        /// <summary>
        /// 权限多语言数据更新
        /// </summary>
        /// <returns></returns>
        public async Task<bool> UpdateAdminAuthDBAsync()
        {
            using (new Performance(_performanceThreshold, "{0} UpdateAdminAuthDBAsync execute", $"{this.GetType().Name}"))
            {
                var res = false;
                var adminFeatureList = await _adminFeaturesDBRepository.FindAllAsync();
                if (adminFeatureList?.Count > 0)
                {
                    var languages = new List<string>();
                    DirectoryInfo theFolder = new DirectoryInfo(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "Languge"));
                    DirectoryInfo[] dirInfo = theFolder.GetDirectories();//获取所在目录的文件夹
                    foreach (var item in dirInfo)
                    {
                        languages.Add(item.Name);
                    }
                    var dicLanguage = new Dictionary<string, JObject>();
                    foreach (var langaugeItem in languages)
                    {
                        var jsonContent = WingServerCommon.ResourceManage.ResourceManager.GetSetting(langaugeItem + "authority");
                        if (string.IsNullOrWhiteSpace(jsonContent))
                        {
                            ThrowCustomerException(CustomerRpcCode.ExceptionError, "Not find Authority Config");
                        }
                        JObject json = JObject.Parse(jsonContent);
                        dicLanguage.Add(langaugeItem, json);
                    }
                    //foreach (var item in adminFeatureList)
                    var updateDataList = adminFeatureList.Select(item =>
                    {
                        if (item != null && !string.IsNullOrWhiteSpace(item.FeatureCode) && (item.LanguageConfigs == null || item.LanguageConfigs.Count <= 0))
                        {
                            var tempList = new List<UserFeatureLanguageConfigDO>();
                            foreach (var langaugeItem in languages)
                            {
                                var jsonData = dicLanguage[langaugeItem];
                                var entity = new UserFeatureLanguageConfigDO()
                                {
                                    Language = langaugeItem,
                                    FeatureName = jsonData["Admin"][item.UniqueCode].ToString()
                                };
                                tempList.Add(entity);
                            }
                            item.LanguageConfigs = tempList;
                        }
                        return ("Code", item.Code, item, string.Empty);
                    }).ToList();
                    var result = await _adminFeaturesDBRepository.UpdateAllAsync(updateDataList);
                    if (result > 0)
                    {
                        res = true;
                    }
                }
                return res;
            }
        }

        /// <summary>
        /// 角色多语言数据更新
        /// </summary>
        /// <returns></returns>
        public async Task<bool> UpdateAdminRoleDBAsync()
        {
            using (new Performance(_performanceThreshold, "{0} UpdateAdminRoleDBAsync execute", $"{this.GetType().Name}"))
            {
                var res = false;
                var codes = new List<string>() { "AdminRoleDO01", "AdminRoleDO02" };
                var dic = new Dictionary<string, List<string>>();
                dic.Add("Code", codes);
                var findRoles = await _adminRoleDBRepository.FindInAsync(dic);
                if (findRoles?.Count > 0)
                {
                    foreach (var item in findRoles)
                    {
                        if (item.Code == "AdminRoleDO01")
                        {
                            item.LanguageConfigs = new List<AdminRoleLanguageConfigDO>
                            {
                                new AdminRoleLanguageConfigDO { Language = "zh-CN", RoleName = "超级管理员", Description = "拥有管理后台最高权限" },
                                new AdminRoleLanguageConfigDO { Language = "en-US", RoleName = "Super administrator", Description = "Has the highest authority to manage the backend" },
                                new AdminRoleLanguageConfigDO { Language = "ru-RU", RoleName = "Суперадминистратор", Description = "Обладает максимальными правами администрирования" }
                            };
                        }
                        else if (item.Code == "AdminRoleDO02")
                        {
                            item.LanguageConfigs = new List<AdminRoleLanguageConfigDO>
                            {
                                new AdminRoleLanguageConfigDO { Language = "zh-CN", RoleName = "管理员", Description = "一般管理员，只管理自己区域内的机构、设备、用户数据" },
                                new AdminRoleLanguageConfigDO { Language = "en-US", RoleName = "administrator", Description = "A general administrator who only manages institutions, devices, and user data within their own area" },
                                new AdminRoleLanguageConfigDO { Language = "ru-RU", RoleName = "Администратор", Description = "Общий администратор, который управляет только учреждениями, оборудованием, пользовательскими данными в своем регионе" }
                            };
                        }
                        res = await _adminRoleDBRepository.UpdateOneForSyncDBAsync("Code", item.Code, item, string.Empty, true, false);
                    }
                }
                return res;
            }
        }
        #endregion 
    }
}