using System.Threading.Tasks;
using WingInterfaceLibrary.DB.Request;
using WingInterfaceLibrary.DTO.AdminFeature;
using WingInterfaceLibrary.Interface.DBInterface;
using WingInterfaceLibrary.Request;
using WingMongoDBModule.DO;
using WingServerCommon.Mapper;
using WingServerCommon.Service;
using System.Collections.Generic;
using WingServerCommon.Utilities;
using System.Linq;
using System;
using JsonRpcLite.Log;

namespace WingMongoDBModule.Service
{
    public partial class DatabaseService : JsonRpcService, IAdminFeaturesDBService
    {
        /// <summary>
        /// > 通过代码查找管理功能
        /// </summary>
        /// <param name="code">要找到的管理功能的代码。</param>
        /// <returns>
        /// AdminFeatureInfoDTO
        /// </returns>
        public async Task<AdminFeatureInfoDTO> FindAdminFeatureByCodeAsync(string code)
        {
            using (new Performance(_performanceThreshold, "{0} FindAdminFeatureByCodeAsync execute", $"{this.GetType().Name}"))
            {
                var adminFeatureDO = await _adminFeaturesDBRepository.FindOneAsync("Code", code);
                return adminFeatureDO.MappingTo<AdminFeatureInfoDTO>();
            }
        }

        /// <summary>
        /// > 按名称查找管理功能
        /// </summary>
        /// <param name="name">功能的名称。</param>
        /// <returns>
        /// AdminFeatureInfoDTO
        /// </returns>
        public async Task<AdminFeatureInfoDTO> FindAdminFeatureByNameAsync(string name)
        {
            using (new Performance(_performanceThreshold, "{0} FindAdminFeatureByNameAsync execute", $"{this.GetType().Name}"))
            {
                var adminFeatureDO = await _adminFeaturesDBRepository.FindOneAsync("FeatureName", name);
                return adminFeatureDO.MappingTo<AdminFeatureInfoDTO>();
            }
        }

        /// <summary>
        /// > 将新的管理功能插入数据库
        /// </summary>
        /// <param name="CreateAdminFeatureDBRequest">包含要插入的数据的请求对象。</param>
        /// <returns>
        /// 新创建的管理功能的 ID。
        /// </returns>
        public async Task<string> InsertAdminFeatureAsync(CreateAdminFeatureDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} InsertAdminFeatureAsync execute", $"{this.GetType().Name}"))
            {
                var adminFeatureDO = request.Data.MappingTo<AdminFeatureDO>();
                return await _adminFeaturesDBRepository.InsertOneAsync(adminFeatureDO, request.PlatformDatas, request.ExtensionData);
            }
        }

        /// <summary>
        /// 通过代码更新管理功能
        /// </summary>
        /// <param name="code">要更新的管理功能的代码。</param>
        /// <param name="AdminFeatureInfoDTO">包含要更新的数据的数据传输对象。</param>
        /// <param name="extensionData">扩展数据用于扩展数据模型。它是 JSON 格式的字符串。</param>
        /// <returns>
        /// 一个布尔值。
        /// </returns>
        public async Task<bool> UpdateAdminFeatureByCodeAsync(string code, AdminFeatureInfoDTO data, string extensionData)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateAdminFeatureByCodeAsync execute", $"{this.GetType().Name}"))
            {
                var adminFeatureDO = data.MappingTo<AdminFeatureDO>();
                return await _adminFeaturesDBRepository.UpdateOneAsync("Code", code, adminFeatureDO, extensionData, true);
            }
        }

        /// <summary>
        /// > 此功能用于查找管理功能页面
        /// </summary>
        /// <param name="PageFilterRequest">这是包含页码、页面大小和搜索条件的请求对象。</param>
        /// <returns>
        /// AdminFeatureInfoDTO 的 PageCollection
        /// </returns>
        public async Task<PageCollection<AdminFeatureInfoDTO>> FindAdminFeaturePagesAsync(PageFilterRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} FindAdminFeaturePagesAsync execute", $"{this.GetType().Name}"))
            {
                var dbRequest = GetDBPageRequest(request);
                var pageInfo = await _adminFeaturesDBRepository.GetPages(dbRequest);
                return PageCollectionConvert<AdminFeatureDO, AdminFeatureInfoDTO>(pageInfo);
            }
        }

        /// <summary>
        /// 通过代码删除管理功能
        /// </summary>
        /// <param name="code">要删除的功能的代码。</param>
        /// <returns>
        /// 一个布尔值。
        /// </returns>
        public async Task<bool> DeleteAdminFeatureByCodeAsync(string code)
        {
            using (new Performance(_performanceThreshold, "{0} DeleteAdminFeatureByCodeAsync execute", $"{this.GetType().Name}"))
            {
                return await _adminFeaturesDBRepository.DeleteOneAsync("Code", code);
            }
        }

        /// <summary>
        /// > 通过代码查找管理功能
        /// </summary>
        /// <param name="codes">要查询的特征码列表</param>
        /// <returns>
        /// AdminFeatureInfoDTO 对象列表
        /// </returns>
        public async Task<List<AdminFeatureInfoDTO>> FindAdminFeaturesByCodesAsync(List<string> codes)
        {
            using (new Performance(_performanceThreshold, "{0} FindAdminFeaturesByCodesAsync execute", $"{this.GetType().Name}"))
            {
                var dic = new Dictionary<string, List<string>>();
                dic.Add("Code", codes);
                var featureList = await _adminFeaturesDBRepository.FindInAsync(dic);
                return featureList.MappingTo<List<AdminFeatureInfoDTO>>();
            }
        }

        /// <summary>
        /// > 通过代码查找管理功能
        /// </summary>
        /// <param name="codes">要查询的特征码列表</param>
        /// <returns>
        public async Task<bool> UpdateAdminFeatureDBAsync()
        {
            using (new Performance(_performanceThreshold, "{0} UpdateAdminFeatureDBAsync execute", $"{this.GetType().Name}"))
            {
                try
                {
                    WingServerCommon.Log.Logger.WriteLineInfo($"AdminDBService UpdateAdminFeatureDBAsync Starting...");
                    var res = false;
                    var featureInfo = await _adminFeaturesDBRepository.FindOneAsync("Code", "JPGL");
                    if (string.IsNullOrEmpty(featureInfo?.Code))
                    {
                        var addRequest = new List<(AdminFeatureDO adminFeatureDO, Dictionary<string, string> platformDatas, string extensionData)>();
                        var model = new AdminFeatureDO//对应  service
                        {
                            FeatureCode = "JPGL",
                            FatherCode = "",
                            FeatureName = "杏聆荟学院",
                            UniqueCode = "JPGL",//审核申请
                            LanguageConfigs = new List<UserFeatureLanguageConfigDO>() {
                                new UserFeatureLanguageConfigDO() {
                                    Language = "zh-CN",
                                    FeatureName = "杏聆荟学院"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "ru-RU",
                                    FeatureName = "Образование"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "en-US",
                                    FeatureName = "Education"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "es-ES",
                                    FeatureName = "Educación"
                                }
                            }
                        };
                        addRequest.Add((model, new Dictionary<string, string>(), ""));

                        model = new AdminFeatureDO//对应  service
                        {
                            FeatureCode = "JPCL",
                            FatherCode = "JPGL",
                            FeatureName = "课程列表",
                            UniqueCode = "JPCL",//审核申请
                            LanguageConfigs = new List<UserFeatureLanguageConfigDO>() {
                                new UserFeatureLanguageConfigDO() {
                                    Language = "zh-CN",
                                    FeatureName = "课程列表"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "ru-RU",
                                    FeatureName = "Список курсов"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "en-US",
                                    FeatureName = "Course List"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "es-ES",
                                    FeatureName = "Lista de cursos"
                                }
                            }
                        };
                        addRequest.Add((model, new Dictionary<string, string>(), ""));

                        model = new AdminFeatureDO//对应  service
                        {
                            FeatureCode = "KCZJLB",
                            FatherCode = "JPGL",
                            FeatureName = "专辑列表",
                            UniqueCode = "KCZJLB",//审核申请
                            LanguageConfigs = new List<UserFeatureLanguageConfigDO>() {
                                new UserFeatureLanguageConfigDO() {
                                    Language = "zh-CN",
                                    FeatureName = "专辑列表"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "ru-RU",
                                    FeatureName = "Список альбомов"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "en-US",
                                    FeatureName = "Album List"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "es-ES",
                                    FeatureName = "Lista de álbumes"
                                }
                            }
                        };
                        addRequest.Add((model, new Dictionary<string, string>(), ""));

                        model = new AdminFeatureDO//对应  service
                        {
                            FeatureCode = "JPUGL",
                            FatherCode = "JPGL",
                            FeatureName = "班级管理",
                            UniqueCode = "JPUGL",//审核申请
                            LanguageConfigs = new List<UserFeatureLanguageConfigDO>() {
                                new UserFeatureLanguageConfigDO() {
                                    Language = "zh-CN",
                                    FeatureName = "班级管理"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "ru-RU",
                                    FeatureName = "Управление группами"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "en-US",
                                    FeatureName = "Class Manager"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "es-ES",
                                    FeatureName = "Director de clase"
                                }
                            }
                        };
                        addRequest.Add((model, new Dictionary<string, string>(), ""));

                        model = new AdminFeatureDO//对应  service
                        {
                            FeatureCode = "JPCLL",
                            FatherCode = "JPGL",
                            FeatureName = "标签管理",
                            UniqueCode = "JPCLL",//审核申请
                            LanguageConfigs = new List<UserFeatureLanguageConfigDO>() {
                                new UserFeatureLanguageConfigDO() {
                                    Language = "zh-CN",
                                    FeatureName = "标签管理"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "ru-RU",
                                    FeatureName = "Управление метками"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "en-US",
                                    FeatureName = "Label Manage"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "es-ES",
                                    FeatureName = "Gestión de etiquetas"
                                }
                            }
                        };
                        addRequest.Add((model, new Dictionary<string, string>(), ""));

                        model = new AdminFeatureDO//对应  service
                        {
                            FeatureCode = "KCSPGL",
                            FatherCode = "JPGL",
                            FeatureName = "素材管理",
                            UniqueCode = "KCSPGL",//审核申请
                            LanguageConfigs = new List<UserFeatureLanguageConfigDO>() {
                                new UserFeatureLanguageConfigDO() {
                                    Language = "zh-CN",
                                    FeatureName = "素材管理"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "ru-RU",
                                    FeatureName = "Видеоматериалы"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "en-US",
                                    FeatureName = "Material management"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "es-ES",
                                    FeatureName = "Gestión de materiales"
                                }
                            }
                        };
                        addRequest.Add((model, new Dictionary<string, string>(), ""));

                        model = new AdminFeatureDO//对应  service
                        {
                            FeatureCode = "TMLB",
                            FatherCode = "JPGL",
                            FeatureName = "题目列表",
                            UniqueCode = "TMLB",//审核申请
                            LanguageConfigs = new List<UserFeatureLanguageConfigDO>() {
                                new UserFeatureLanguageConfigDO() {
                                    Language = "zh-CN",
                                    FeatureName = "题目列表"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "ru-RU",
                                    FeatureName = "Список вопросов"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "en-US",
                                    FeatureName = "Questions Manage"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "es-ES",
                                    FeatureName = "Gestión de problemas"
                                }
                            }
                        };
                        addRequest.Add((model, new Dictionary<string, string>(), ""));

                        model = new AdminFeatureDO//对应  service
                        {
                            FeatureCode = "SJGL",
                            FatherCode = "JPGL",
                            FeatureName = "试卷管理",
                            UniqueCode = "SJGL",//审核申请
                            LanguageConfigs = new List<UserFeatureLanguageConfigDO>() {
                                new UserFeatureLanguageConfigDO() {
                                    Language = "zh-CN",
                                    FeatureName = "试卷管理"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "ru-RU",
                                    FeatureName = "Проверка"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "en-US",
                                    FeatureName = "Examination paper"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "es-ES",
                                    FeatureName = "Examination paper"
                                }
                            }
                        };
                        addRequest.Add((model, new Dictionary<string, string>(), ""));
                        await _adminFeaturesDBRepository.InsertAllAsync(addRequest);
                    }

                    //医师，新增开始会诊权限，个性化设置
                    var adminRoleDO = await _adminRoleDBRepository.FindOneAsync("Code", "AdminRoleDO02");
                    if (!string.IsNullOrEmpty(adminRoleDO?.AdminGroupCode))
                    {
                        var adminAuthorityGroupDO = await _adminAuthorityGroupDBRepository.FindOneAsync("Code", adminRoleDO.AdminGroupCode);
                        if (adminAuthorityGroupDO.Features?.Count > 0)
                        {
                            adminAuthorityGroupDO.Features.Add("JPGL");
                            adminAuthorityGroupDO.Features.Add("JPCL");
                            adminAuthorityGroupDO.Features.Add("KCZJLB");
                            adminAuthorityGroupDO.Features.Add("JPUGL");
                            adminAuthorityGroupDO.Features.Add("JPCLL");
                            adminAuthorityGroupDO.Features.Add("KCSPGL");
                            adminAuthorityGroupDO.Features.Add("TMLB");
                            adminAuthorityGroupDO.Features.Add("SJGL");
                            adminAuthorityGroupDO.Features = adminAuthorityGroupDO.Features.Distinct().ToList();
                        }
                        else
                        {
                            adminAuthorityGroupDO.Features = new List<string>() { "JPGL", "JPCL", "KCZJLB", "JPUGL", "JPCLL", "KCSPGL", "TMLB", "SJGL" };
                        }
                        res = await _adminAuthorityGroupDBRepository.UpdateOneForSyncDBAsync("Code", adminAuthorityGroupDO.Code, adminAuthorityGroupDO, "", true, false);
                    }

                    WingServerCommon.Log.Logger.WriteLineInfo($"AdminDBService UpdateAdminFeatureDBAsync Ending...");
                    return res;
                }
                catch (Exception ex)
                {
                    WingServerCommon.Log.Logger.WriteLineError($"AdminDBService UpdateAdminFeatureDBAsync error, ex: {ex}");
                    return false;
                }
            }
        }

        /// <summary>
        /// 更新管理员角色西班牙语
        /// </summary>
        /// <returns>
        private async Task<bool> UpdateAdminRoleForESDBAsync()
        {
            using (new Performance(_performanceThreshold, "{0} UpdateAdminRoleForESDBAsync execute", $"{this.GetType().Name}"))
            {
                var res = false;
                try
                {
                    var adminRoleDOList = await _adminRoleDBRepository.FindAllAsync();
                    if (adminRoleDOList?.Count > 0)
                    {
                        foreach (var item in adminRoleDOList)
                        {
                            if (item.LanguageConfigs?.Count > 0)
                            {
                                if (!item.LanguageConfigs.Exists(c => c.Language == "es-ES"))
                                {
                                    if (item.Code == "AdminRoleDO02") //管理员
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new AdminRoleLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                RoleName = "cuidadores",
                                                Description = "Un administrador general que gestiona únicamente la organización, los equipos y los datos de los usuarios de su área."
                                            }
                                        );
                                    }
                                    else if (item.Code == "AdminRoleDO01") //超级管理员
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new AdminRoleLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                RoleName = "Superadministrador",
                                                Description = "Tiene la máxima autoridad para gestionar el fondo"
                                            }
                                        );
                                    }
                                    var result = await _adminRoleDBRepository.UpdateOneForSyncDBAsync("Code", item.Code, item, string.Empty, true, false);
                                }
                            }
                        }
                    }

                    var featureInfoList = await _adminFeaturesDBRepository.FindAllAsync();
                    if (featureInfoList?.Count > 0)
                    {
                        foreach (var item in featureInfoList)
                        {
                            if (item.LanguageConfigs?.Count > 0)
                            {
                                if (!item.LanguageConfigs.Any(c => c.Language == "es-ES"))
                                {
                                    if (item.Code == "JPGL")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Educación",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "JPCL")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Lista de cursos",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "KCZJLB")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Lista de álbumes",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "JPUGL")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Director de clase",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "JPCLL")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Gestión de etiquetas",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "KCSPGL")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Gestión de materiales",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "TMLB")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Gestión de problemas",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "SJGL")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Examination paper",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "JGGL")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Gestión de organizaciones",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "OrgDevices")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Institutional equipment",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "JGLB")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Lista de organizaciones",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "SBGL")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Gestión de equipos",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "SBLXLB")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Lista de tipos de equipos",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "PPXHLB")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Lista de marcas y modelos de equipos",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "SBLB")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Lista de equipos",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "HTXTGL")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Lista de administradores",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "HTQXLB")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Lista de permisos en segundo plano",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "HTGLYJS")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Funciones de administrador en segundo plano",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "HTGLYLB")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Lista de administradores en segundo plano",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "YHGL")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Gestión de usuarios",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "QXLB")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Lista de permisos",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "PTJS")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Roles de plataforma",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "YHLB")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Lista de usuarios",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "MBGL")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Gestión de plantillas",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "RPMGI")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Plantillas de informes",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "BGBQ")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Etiquetas de informes",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "CTKMB")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Lista de tesauros",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "CLWH")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Lista de entradas",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "AIGL")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Diagnóstico AI",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "AILB")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Aplicaciones AI",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "HTGL")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Gestión de Backstage",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "XTRZ")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Registro del sistema",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "DSFGL")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Configurar asistente",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "ServerPZ")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Configuración del servidor",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "ZXZH")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Cuenta en línea",
                                                Description = ""
                                            }
                                        );
                                    }
                                    var result = await _adminFeaturesDBRepository.UpdateOneAsync("Code", item.Code, item, string.Empty, true, false);
                                }
                            }
                        }
                    }

                    return res;
                }
                catch (Exception ex)
                {
                    WingServerCommon.Log.Logger.WriteLineError($"AdminFeatureDBService UpdateAdminRoleForESDBAsync error, ex: {ex}");
                    return res;
                }
            }
        }

        /// <summary>
        /// 更新管理端权限列表
        /// </summary>
        /// <returns>
        private async Task<bool> UpdateManagementAuthDBAsync()
        {
            using (new Performance(_performanceThreshold, "{0} UpdateManagementAuthDBAsync execute", $"{this.GetType().Name}"))
            {
                var res = false;
                try
                {
                    var featureInfo = await _adminFeaturesDBRepository.FindOneAsync("Code", "YHQZ");
                    if (string.IsNullOrEmpty(featureInfo?.Code))
                    {
                        var addRequest = new List<(AdminFeatureDO adminFeatureDO, Dictionary<string, string> platformDatas, string extensionData)>();
                        var model = new AdminFeatureDO//对应  service
                        {
                            FeatureCode = "YHQZ",
                            FatherCode = "YHGL",
                            FeatureName = "用户群组",
                            UniqueCode = "YHQZ",//审核申请
                            LanguageConfigs = new List<UserFeatureLanguageConfigDO>() {
                                new UserFeatureLanguageConfigDO() {
                                    Language = "zh-CN",
                                    FeatureName = "用户群组"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "ru-RU",
                                    FeatureName = "Группы пользователей"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "en-US",
                                    FeatureName = "User Groups"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "es-ES",
                                    FeatureName = "Grupos de usuarios"
                                }
                            }
                        };
                        addRequest.Add((model, new Dictionary<string, string>(), ""));

                        model = new AdminFeatureDO//对应  service
                        {
                            FeatureCode = "SBXHLB",
                            FatherCode = "SBGL",
                            FeatureName = "设备型号列表",
                            UniqueCode = "SBXHLB",//审核申请
                            LanguageConfigs = new List<UserFeatureLanguageConfigDO>() {
                                new UserFeatureLanguageConfigDO() {
                                    Language = "zh-CN",
                                    FeatureName = "设备型号列表"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "ru-RU",
                                    FeatureName = "Список моделей устройств"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "en-US",
                                    FeatureName = "Lista de modelos de dispositivos"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "es-ES",
                                    FeatureName = "Device Model List"
                                }
                            }
                        };
                        addRequest.Add((model, new Dictionary<string, string>(), ""));

                        model = new AdminFeatureDO//对应  service
                        {
                            FeatureCode = "WLWSBLB",
                            FatherCode = "SBGL",
                            FeatureName = "物联网设备列表",
                            UniqueCode = "WLWSBLB",//审核申请
                            LanguageConfigs = new List<UserFeatureLanguageConfigDO>() {
                                new UserFeatureLanguageConfigDO() {
                                    Language = "zh-CN",
                                    FeatureName = "物联网设备列表"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "ru-RU",
                                    FeatureName = "IOT Список устройств"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "en-US",
                                    FeatureName = "IOT Device List"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "es-ES",
                                    FeatureName = "IOT Lista de dispositivos"
                                }
                            }
                        };
                        addRequest.Add((model, new Dictionary<string, string>(), ""));

                        model = new AdminFeatureDO//对应  service
                        {
                            FeatureCode = "SBWZLB",
                            FatherCode = "SBGL",
                            FeatureName = "物联网文章",
                            UniqueCode = "SBWZLB",//审核申请
                            LanguageConfigs = new List<UserFeatureLanguageConfigDO>() {
                                new UserFeatureLanguageConfigDO() {
                                    Language = "zh-CN",
                                    FeatureName = "物联网文章"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "ru-RU",
                                    FeatureName = "Статьи"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "en-US",
                                    FeatureName = "Articles"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "es-ES",
                                    FeatureName = "Artículos"
                                }
                            }
                        };
                        addRequest.Add((model, new Dictionary<string, string>(), ""));

                        model = new AdminFeatureDO//对应  service
                        {
                            FeatureCode = "SBCZSC",
                            FatherCode = "SBGL",
                            FeatureName = "物联网操作指导",
                            UniqueCode = "SBCZSC",//审核申请
                            LanguageConfigs = new List<UserFeatureLanguageConfigDO>() {
                                new UserFeatureLanguageConfigDO() {
                                    Language = "zh-CN",
                                    FeatureName = "物联网操作指导"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "ru-RU",
                                    FeatureName = "Руководство по эксплуатации"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "en-US",
                                    FeatureName = "Operating Instructions"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "es-ES",
                                    FeatureName = "Instrucciones de operación"
                                }
                            }
                        };
                        addRequest.Add((model, new Dictionary<string, string>(), ""));

                        model = new AdminFeatureDO//对应  service
                        {
                            FeatureCode = "WLWSBGJLB",
                            FatherCode = "SBGL",
                            FeatureName = "物联网设备固件列表",
                            UniqueCode = "WLWSBGJLB",//审核申请
                            LanguageConfigs = new List<UserFeatureLanguageConfigDO>() {
                                new UserFeatureLanguageConfigDO() {
                                    Language = "zh-CN",
                                    FeatureName = "物联网设备固件列表"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "ru-RU",
                                    FeatureName = "Список версий прошивки устройства"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "en-US",
                                    FeatureName = "Device Firmware List"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "es-ES",
                                    FeatureName = "Lista de firmware del dispositivo"
                                }
                            }
                        };
                        addRequest.Add((model, new Dictionary<string, string>(), ""));

                        model = new AdminFeatureDO//对应  service
                        {
                            FeatureCode = "WLWSBBBLB",
                            FatherCode = "SBGL",
                            FeatureName = "物联网设备版本列表",
                            UniqueCode = "WLWSBBBLB",//审核申请
                            LanguageConfigs = new List<UserFeatureLanguageConfigDO>() {
                                new UserFeatureLanguageConfigDO() {
                                    Language = "zh-CN",
                                    FeatureName = "物联网设备版本列表"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "ru-RU",
                                    FeatureName = "Список версий устройства"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "en-US",
                                    FeatureName = "Device Version List"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "es-ES",
                                    FeatureName = "Lista de versiones del dispositivo"
                                }
                            }
                        };
                        addRequest.Add((model, new Dictionary<string, string>(), ""));

                        model = new AdminFeatureDO//对应  service
                        {
                            FeatureCode = "XMGL",
                            FatherCode = "JGGL",
                            FeatureName = "项目列表",
                            UniqueCode = "XMGL",//审核申请
                            LanguageConfigs = new List<UserFeatureLanguageConfigDO>() {
                                new UserFeatureLanguageConfigDO() {
                                    Language = "zh-CN",
                                    FeatureName = "项目列表"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "ru-RU",
                                    FeatureName = "Список проектов"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "en-US",
                                    FeatureName = "Project List"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "es-ES",
                                    FeatureName = "Lista de Proyectos"
                                }
                            }
                        };
                        addRequest.Add((model, new Dictionary<string, string>(), ""));

                        model = new AdminFeatureDO//对应  service
                        {
                            FeatureCode = "XMTS",
                            FatherCode = "JGGL",
                            FeatureName = "推送记录",
                            UniqueCode = "XMTS",//审核申请
                            LanguageConfigs = new List<UserFeatureLanguageConfigDO>() {
                                new UserFeatureLanguageConfigDO() {
                                    Language = "zh-CN",
                                    FeatureName = "推送记录"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "ru-RU",
                                    FeatureName = "Запись Отправки"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "en-US",
                                    FeatureName = "Push Record"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "es-ES",
                                    FeatureName = "Registro de Envío"
                                }
                            }
                        };
                        addRequest.Add((model, new Dictionary<string, string>(), ""));

                        model = new AdminFeatureDO//对应  service
                        {
                            FeatureCode = "BGTSPZ",
                            FatherCode = "HTGL",
                            FeatureName = "报告推送",
                            UniqueCode = "BGTSPZ",//审核申请
                            LanguageConfigs = new List<UserFeatureLanguageConfigDO>() {
                                new UserFeatureLanguageConfigDO() {
                                    Language = "zh-CN",
                                    FeatureName = "报告推送"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "ru-RU",
                                    FeatureName = "Отправка отчетов"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "en-US",
                                    FeatureName = "Report Push"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "es-ES",
                                    FeatureName = "Pulsar Informe"
                                }
                            }
                        };
                        addRequest.Add((model, new Dictionary<string, string>(), ""));

                        model = new AdminFeatureDO//对应  service
                        {
                            FeatureCode = "BGCF",
                            FatherCode = "HTGL",
                            FeatureName = "报告重发",
                            UniqueCode = "BGCF",//审核申请
                            LanguageConfigs = new List<UserFeatureLanguageConfigDO>() {
                                new UserFeatureLanguageConfigDO() {
                                    Language = "zh-CN",
                                    FeatureName = "报告重发"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "ru-RU",
                                    FeatureName = "Отправка доклада"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "en-US",
                                    FeatureName = "Report Resend"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "es-ES",
                                    FeatureName = "Reenviar informe"
                                }
                            }
                        };
                        addRequest.Add((model, new Dictionary<string, string>(), ""));

                        model = new AdminFeatureDO//对应  service
                        {
                            FeatureCode = "FWQGL",
                            FatherCode = "HTGL",
                            FeatureName = "服务器管理",
                            UniqueCode = "FWQGL",//审核申请
                            LanguageConfigs = new List<UserFeatureLanguageConfigDO>() {
                                new UserFeatureLanguageConfigDO() {
                                    Language = "zh-CN",
                                    FeatureName = "服务器管理"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "ru-RU",
                                    FeatureName = "Список серверов"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "en-US",
                                    FeatureName = "Server Management"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "es-ES",
                                    FeatureName = "Gestión del servidor"
                                }
                            }
                        };
                        addRequest.Add((model, new Dictionary<string, string>(), ""));

                        model = new AdminFeatureDO//对应  service
                        {
                            FeatureCode = "CMSTI",
                            FatherCode = "HTGL",
                            FeatureName = "CMS模板",
                            UniqueCode = "CMSTI",//审核申请
                            LanguageConfigs = new List<UserFeatureLanguageConfigDO>() {
                                new UserFeatureLanguageConfigDO() {
                                    Language = "zh-CN",
                                    FeatureName = "CMS模板"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "ru-RU",
                                    FeatureName = "Шаблоны CMS"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "en-US",
                                    FeatureName = "CMS template"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "es-ES",
                                    FeatureName = "Plantilla CMS"
                                }
                            }
                        };
                        addRequest.Add((model, new Dictionary<string, string>(), ""));

                        model = new AdminFeatureDO//对应  service
                        {
                            FeatureCode = "GRADI",
                            FatherCode = "HTGL",
                            FeatureName = "等保管理",
                            UniqueCode = "GRADI",//审核申请
                            LanguageConfigs = new List<UserFeatureLanguageConfigDO>() {
                                new UserFeatureLanguageConfigDO() {
                                    Language = "zh-CN",
                                    FeatureName = "等保管理"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "ru-RU",
                                    FeatureName = "Безопасность"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "en-US",
                                    FeatureName = "Grading Protection"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "es-ES",
                                    FeatureName = "Gestión de equivalencias"
                                }
                            }
                        };
                        addRequest.Add((model, new Dictionary<string, string>(), ""));

                        model = new AdminFeatureDO//对应  service
                        {
                            FeatureCode = "XTGGGL",
                            FatherCode = "HTGL",
                            FeatureName = "公告管理",
                            UniqueCode = "XTGGGL",//审核申请
                            LanguageConfigs = new List<UserFeatureLanguageConfigDO>() {
                                new UserFeatureLanguageConfigDO() {
                                    Language = "zh-CN",
                                    FeatureName = "公告管理"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "ru-RU",
                                    FeatureName = "Объявление"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "en-US",
                                    FeatureName = "Announcement Manager"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "es-ES",
                                    FeatureName = "Gestión del boletín"
                                }
                            }
                        };
                        addRequest.Add((model, new Dictionary<string, string>(), ""));

                        model = new AdminFeatureDO//对应  service
                        {
                            FeatureCode = "XTPZGL",
                            FatherCode = "HTGL",
                            FeatureName = "环境配置",
                            UniqueCode = "XTPZGL",//审核申请
                            LanguageConfigs = new List<UserFeatureLanguageConfigDO>() {
                                new UserFeatureLanguageConfigDO() {
                                    Language = "zh-CN",
                                    FeatureName = "环境配置"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "ru-RU",
                                    FeatureName = "Настройка системы"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "en-US",
                                    FeatureName = "Environment Manager"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "es-ES",
                                    FeatureName = "Configuración del entorno"
                                }
                            }
                        };
                        addRequest.Add((model, new Dictionary<string, string>(), ""));

                        model = new AdminFeatureDO//对应  service
                        {
                            FeatureCode = "DDJYJLLB",
                            FatherCode = "HTGL",
                            FeatureName = "交易记录",
                            UniqueCode = "DDJYJLLB",//审核申请
                            LanguageConfigs = new List<UserFeatureLanguageConfigDO>() {
                                new UserFeatureLanguageConfigDO() {
                                    Language = "zh-CN",
                                    FeatureName = "交易记录"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "ru-RU",
                                    FeatureName = "Записи транзакций"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "en-US",
                                    FeatureName = "Finance"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "es-ES",
                                    FeatureName = "Registro de transacción"
                                }
                            }
                        };
                        addRequest.Add((model, new Dictionary<string, string>(), ""));

                        await _adminFeaturesDBRepository.InsertAllAsync(addRequest);
                    }

                    //删除不合理的权限
                    var delRes = await _adminFeaturesDBRepository.DeleteOneAsync("Code", "ServerPZ");

                    //医师，新增开始会诊权限，个性化设置
                    var adminRoleDO = await _adminRoleDBRepository.FindOneAsync("Code", "AdminRoleDO02");
                    if (!string.IsNullOrEmpty(adminRoleDO?.AdminGroupCode))
                    {
                        var adminAuthorityGroupDO = await _adminAuthorityGroupDBRepository.FindOneAsync("Code", adminRoleDO.AdminGroupCode);
                        if (adminAuthorityGroupDO?.Features?.Count > 0)
                        {
                            adminAuthorityGroupDO.Features.Add("YHQZ");
                            adminAuthorityGroupDO.Features.Add("SBXHLB");
                            adminAuthorityGroupDO.Features.Add("WLWSBLB");
                            adminAuthorityGroupDO.Features.Add("SBWZLB");
                            adminAuthorityGroupDO.Features.Add("SBCZSC");
                            adminAuthorityGroupDO.Features.Add("WLWSBGJLB");
                            adminAuthorityGroupDO.Features.Add("WLWSBBBLB");
                            adminAuthorityGroupDO.Features.Add("XMGL");
                            adminAuthorityGroupDO.Features.Add("XMTS");
                            adminAuthorityGroupDO.Features.Add("BGTSPZ");
                            adminAuthorityGroupDO.Features.Add("BGCF");
                            adminAuthorityGroupDO.Features.Add("FWQGL");
                            adminAuthorityGroupDO.Features.Add("CMSTI");
                            adminAuthorityGroupDO.Features.Add("GRADI");
                            adminAuthorityGroupDO.Features.Add("XTGGGL");
                            adminAuthorityGroupDO.Features.Add("XTPZGL");
                            adminAuthorityGroupDO.Features.Add("DDJYJLLB");
                            if (adminAuthorityGroupDO.Features.Contains("ServerPZ"))
                            {
                                adminAuthorityGroupDO.Features.Remove("ServerPZ");
                            }
                            adminAuthorityGroupDO.Features = adminAuthorityGroupDO.Features.Distinct().ToList();
                        }
                        else
                        {
                            adminAuthorityGroupDO.Features = new List<string>() { "YHQZ", "SBXHLB", "WLWSBLB", "SBWZLB", "SBCZSC", "WLWSBGJLB", "WLWSBBBLB", "XMGL", "XMTS", "BGTSPZ",
                                                                                "BGCF", "FWQGL", "CMSTI", "GRADI", "XTGGGL", "XTPZGL", "DDJYJLLB" };
                        }
                        res = await _adminAuthorityGroupDBRepository.UpdateOneForSyncDBAsync("Code", adminAuthorityGroupDO.Code, adminAuthorityGroupDO, "", true, false);
                    }
                    return res;
                }
                catch (Exception ex)
                {
                    WingServerCommon.Log.Logger.WriteLineError($"AdminFeatureDBService UpdateManagementAuthDBAsync error, ex: {ex}");
                    return res;
                }
            }
        }

        /// <summary>
        /// > 通过代码查找管理功能
        /// </summary>
        /// <param name="codes">要查询的特征码列表</param>
        /// <returns>
        public async Task<bool> UpdateAdminFeatureForRODBAsync()
        {
            using (new Performance(_performanceThreshold, "{0} UpdateAdminFeatureForRODBAsync execute", $"{this.GetType().Name}"))
            {
                try
                {
                    WingServerCommon.Log.Logger.WriteLineInfo($"AdminDBService UpdateAdminFeatureForRODBAsync Starting...");
                    var res = false;
                    var featureInfo = await _adminFeaturesDBRepository.FindOneAsync("Code", "JPGL");
                    if (!string.IsNullOrEmpty(featureInfo?.Code))
                    {
                        featureInfo.LanguageConfigs.Add(new UserFeatureLanguageConfigDO()
                        {
                            Language = "ro-RO",
                            FeatureName = "Educație"
                        });
                        res = await _adminFeaturesDBRepository.UpdateOneAsync("Code", featureInfo.Code, featureInfo, string.Empty, true, false);
                    }

                    featureInfo = await _adminFeaturesDBRepository.FindOneAsync("Code", "JPCL");
                    if (!string.IsNullOrEmpty(featureInfo?.Code))
                    {
                        featureInfo.LanguageConfigs.Add(new UserFeatureLanguageConfigDO()
                        {
                            Language = "ro-RO",
                            FeatureName = "Lista cursurilor"
                        });
                        res = await _adminFeaturesDBRepository.UpdateOneAsync("Code", featureInfo.Code, featureInfo, string.Empty, true, false);
                    }

                    featureInfo = await _adminFeaturesDBRepository.FindOneAsync("Code", "KCZJLB");
                    if (!string.IsNullOrEmpty(featureInfo?.Code))
                    {
                        featureInfo.LanguageConfigs.Add(new UserFeatureLanguageConfigDO()
                        {
                            Language = "ro-RO",
                            FeatureName = "Lista albumelor"
                        });
                        res = await _adminFeaturesDBRepository.UpdateOneAsync("Code", featureInfo.Code, featureInfo, string.Empty, true, false);
                    }

                    featureInfo = await _adminFeaturesDBRepository.FindOneAsync("Code", "JPUGL");
                    if (!string.IsNullOrEmpty(featureInfo?.Code))
                    {
                        featureInfo.LanguageConfigs.Add(new UserFeatureLanguageConfigDO()
                        {
                            Language = "ro-RO",
                            FeatureName = "Gestionarea claselor"
                        });
                        res = await _adminFeaturesDBRepository.UpdateOneAsync("Code", featureInfo.Code, featureInfo, string.Empty, true, false);
                    }

                    featureInfo = await _adminFeaturesDBRepository.FindOneAsync("Code", "JPCLL");
                    if (!string.IsNullOrEmpty(featureInfo?.Code))
                    {
                        featureInfo.LanguageConfigs.Add(new UserFeatureLanguageConfigDO()
                        {
                            Language = "ro-RO",
                            FeatureName = "Gestionarea etichetelor"
                        });
                        res = await _adminFeaturesDBRepository.UpdateOneAsync("Code", featureInfo.Code, featureInfo, string.Empty, true, false);
                    }

                    featureInfo = await _adminFeaturesDBRepository.FindOneAsync("Code", "KCSPGL");
                    if (!string.IsNullOrEmpty(featureInfo?.Code))
                    {
                        featureInfo.LanguageConfigs.Add(new UserFeatureLanguageConfigDO()
                        {
                            Language = "ro-RO",
                            FeatureName = "Gestionarea materialelor"
                        });
                        res = await _adminFeaturesDBRepository.UpdateOneAsync("Code", featureInfo.Code, featureInfo, string.Empty, true, false);
                    }

                    featureInfo = await _adminFeaturesDBRepository.FindOneAsync("Code", "TMLB");
                    if (!string.IsNullOrEmpty(featureInfo?.Code))
                    {
                        featureInfo.LanguageConfigs.Add(new UserFeatureLanguageConfigDO()
                        {
                            Language = "ro-RO",
                            FeatureName = "Lista întrebărilor"
                        });
                        res = await _adminFeaturesDBRepository.UpdateOneAsync("Code", featureInfo.Code, featureInfo, string.Empty, true, false);
                    }

                    featureInfo = await _adminFeaturesDBRepository.FindOneAsync("Code", "SJGL");
                    if (!string.IsNullOrEmpty(featureInfo?.Code))
                    {
                        featureInfo.LanguageConfigs.Add(new UserFeatureLanguageConfigDO()
                        {
                            Language = "ro-RO",
                            FeatureName = "Gestionarea lucrărilor de examinare"
                        });
                        res = await _adminFeaturesDBRepository.UpdateOneAsync("Code", featureInfo.Code, featureInfo, string.Empty, true, false);
                    }

                    WingServerCommon.Log.Logger.WriteLineInfo($"AdminDBService UpdateAdminFeatureForRODBAsync Ending...");
                    return res;
                }
                catch (Exception ex)
                {
                    WingServerCommon.Log.Logger.WriteLineError($"AdminDBService UpdateAdminFeatureForRODBAsync error, ex: {ex}");
                    return false;
                }
            }
        }

        /// <summary>
        /// 更新管理员角色西班牙语
        /// </summary>
        /// <returns>
        private async Task<bool> UpdateAdminRoleForRODBAsync()
        {
            using (new Performance(_performanceThreshold, "{0} UpdateAdminRoleForRODBAsync execute", $"{this.GetType().Name}"))
            {
                var res = false;
                try
                {
                    var adminRoleDOList = await _adminRoleDBRepository.FindAllAsync();
                    if (adminRoleDOList?.Count > 0)
                    {
                        foreach (var item in adminRoleDOList)
                        {
                            if (item.LanguageConfigs?.Count > 0)
                            {
                                if (!item.LanguageConfigs.Exists(c => c.Language == "ro-RO"))
                                {
                                    if (item.Code == "AdminRoleDO02") //管理员
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new AdminRoleLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                RoleName = "administratori",
                                                Description = "Un administrator general care gestionează numai instituțiile, dispozitivele și datele utilizatorilor în propria zonă"
                                            }
                                        );
                                    }
                                    else if (item.Code == "AdminRoleDO01") //超级管理员
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new AdminRoleLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                RoleName = "Super administrator",
                                                Description = "Are cea mai înaltă autoritate pentru a gestiona backend-ul"
                                            }
                                        );
                                    }
                                    var result = await _adminRoleDBRepository.UpdateOneForSyncDBAsync("Code", item.Code, item, string.Empty, true, false);
                                }
                            }
                        }
                    }

                    var featureInfoList = await _adminFeaturesDBRepository.FindAllAsync();
                    if (featureInfoList?.Count > 0)
                    {
                        foreach (var item in featureInfoList)
                        {
                            if (item.LanguageConfigs?.Count > 0)
                            {
                                if (!item.LanguageConfigs.Any(c => c.Language == "ro-RO"))
                                {
                                    if (item.Code == "JPGL")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "educație",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "JPCL")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Lista de verificare a cursului",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "KCZJLB")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Lista albumelor",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "JPUGL")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "profesor de clasă",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "JPCLL")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Gestionarea etichetelor",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "KCSPGL")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Gestionarea materialelor",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "TMLB")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Gestionarea problemelor",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "SJGL")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Lucrarea examenului",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "JGGL")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Managementul organizațional",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "OrgDevices")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Echipamente instituționale",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "JGLB")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Lista organizațiilor",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "SBGL")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "gestionarea dispozitivelor",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "SBLXLB")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Lista tipurilor de echipamente",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "PPXHLB")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Lista mărcilor și modelelor de echipamente",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "SBLB")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Lista echipamentelor",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "HTXTGL")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "manageri",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "HTQXLB")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Lista permisiunilor de fundal",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "HTGLYJS")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Rolul administratorului backend",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "HTGLYLB")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Lista administratorilor backend",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "YHGL")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "gestionarea utilizatorilor",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "QXLB")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Lista licențelor",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "PTJS")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Rolul platformei",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "YHLB")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Rol Pratforme",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "MBGL")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Gestionarea șabloanelor",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "RPMGI")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Șablon de raport",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "BGBQ")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Eticheta raportului",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "CTKMB")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Lista sinonimelor",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "CLWH")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Lista elementelor",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "AIGL")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Diagnosticul inteligenței artificiale",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "AILB")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Aplicații de inteligență artificială",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "HTGL")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Gestionare backend",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "XTRZ")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Înregistrarea sistemului",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "DSFGL")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Expert de configurare",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "ServerPZ")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Configurare server",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "ZXZH")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Cont online",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "XMGL")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Lista proiectelor",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "XMTS")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Înregistrare push",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "SBXHLB")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Lista modelelor de echipamente",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "WLWSBLB")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Lista dispozitivelor IoT",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "SBWZLB")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Articole IoT",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "SBCZSC")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Ghid de funcționare a internetului obiectelor",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "WLWSBGJLB")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Lista firmware pentru dispozitivele IoT",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "WLWSBBBLB")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Lista versiunilor dispozitivelor IoT",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "YHQZ")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "grupuri de utilizatori",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "BGTSPZ")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Raportează push",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "BGCF")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Raportează din nou",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "FWQGL")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Gestionarea serverelor",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "CMSTI")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Șablon CMS",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "GRADI")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Gestionarea asigurărilor",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "XTGGGL")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Gestionarea anunțurilor",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "XTPZGL")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Configurarea mediului",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "DDJYJLLB")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Înregistrări tranzacții",
                                                Description = ""
                                            }
                                        );
                                    }
                                    var result = await _adminFeaturesDBRepository.UpdateOneAsync("Code", item.Code, item, string.Empty, true, false);
                                }
                            }
                        }
                    }

                    return res;
                }
                catch (Exception ex)
                {
                    WingServerCommon.Log.Logger.WriteLineError($"AdminFeatureDBService UpdateAdminRoleForRODBAsync error, ex: {ex}");
                    return res;
                }
            }
        }

        /// <summary>
        /// 更新管理端权限列表
        /// </summary>
        /// <returns>
        private async Task<bool> UpdateManagementAuthForRODBAsync()
        {
            using (new Performance(_performanceThreshold, "{0} UpdateManagementAuthForRODBAsync execute", $"{this.GetType().Name}"))
            {
                var res = false;
                try
                {
                    var featureInfo = await _adminFeaturesDBRepository.FindOneAsync("Code", "YHQZ");
                    if (!string.IsNullOrEmpty(featureInfo?.Code))
                    {
                        featureInfo.LanguageConfigs.Add(new UserFeatureLanguageConfigDO()
                        {
                            Language = "zh-CN",
                            FeatureName = "grupuri de utilizatori"
                        });
                        res = await _adminFeaturesDBRepository.UpdateOneAsync("Code", featureInfo.Code, featureInfo, string.Empty, true, false);
                    }

                    featureInfo = await _adminFeaturesDBRepository.FindOneAsync("Code", "SBXHLB");
                    if (!string.IsNullOrEmpty(featureInfo?.Code))
                    {
                        featureInfo.LanguageConfigs.Add(new UserFeatureLanguageConfigDO()
                        {
                            Language = "zh-CN",
                            FeatureName = "Lista modelelor de echipamente"
                        });
                        res = await _adminFeaturesDBRepository.UpdateOneAsync("Code", featureInfo.Code, featureInfo, string.Empty, true, false);
                    }

                    featureInfo = await _adminFeaturesDBRepository.FindOneAsync("Code", "WLWSBLB");
                    if (!string.IsNullOrEmpty(featureInfo?.Code))
                    {
                        featureInfo.LanguageConfigs.Add(new UserFeatureLanguageConfigDO()
                        {
                            Language = "zh-CN",
                            FeatureName = "Lista dispozitivelor IoT"
                        });
                        res = await _adminFeaturesDBRepository.UpdateOneAsync("Code", featureInfo.Code, featureInfo, string.Empty, true, false);
                    }

                    featureInfo = await _adminFeaturesDBRepository.FindOneAsync("Code", "SBWZLB");
                    if (!string.IsNullOrEmpty(featureInfo?.Code))
                    {
                        featureInfo.LanguageConfigs.Add(new UserFeatureLanguageConfigDO()
                        {
                            Language = "zh-CN",
                            FeatureName = "Articole IoT"
                        });
                        res = await _adminFeaturesDBRepository.UpdateOneAsync("Code", featureInfo.Code, featureInfo, string.Empty, true, false);
                    }


                    featureInfo = await _adminFeaturesDBRepository.FindOneAsync("Code", "SBCZSC");
                    if (!string.IsNullOrEmpty(featureInfo?.Code))
                    {
                        featureInfo.LanguageConfigs.Add(new UserFeatureLanguageConfigDO()
                        {
                            Language = "zh-CN",
                            FeatureName = "Instrucțiuni de utilizare"
                        });
                        res = await _adminFeaturesDBRepository.UpdateOneAsync("Code", featureInfo.Code, featureInfo, string.Empty, true, false);
                    }

                    featureInfo = await _adminFeaturesDBRepository.FindOneAsync("Code", "WLWSBGJLB");
                    if (!string.IsNullOrEmpty(featureInfo?.Code))
                    {
                        featureInfo.LanguageConfigs.Add(new UserFeatureLanguageConfigDO()
                        {
                            Language = "zh-CN",
                            FeatureName = "Lista firmware a dispozitivelor"
                        });
                        res = await _adminFeaturesDBRepository.UpdateOneAsync("Code", featureInfo.Code, featureInfo, string.Empty, true, false);
                    }


                    featureInfo = await _adminFeaturesDBRepository.FindOneAsync("Code", "WLWSBBBLB");
                    if (!string.IsNullOrEmpty(featureInfo?.Code))
                    {
                        featureInfo.LanguageConfigs.Add(new UserFeatureLanguageConfigDO()
                        {
                            Language = "zh-CN",
                            FeatureName = "Lista versiunilor dispozitivului"
                        });
                        res = await _adminFeaturesDBRepository.UpdateOneAsync("Code", featureInfo.Code, featureInfo, string.Empty, true, false);
                    }

                    featureInfo = await _adminFeaturesDBRepository.FindOneAsync("Code", "XMGL");
                    if (!string.IsNullOrEmpty(featureInfo?.Code))
                    {
                        featureInfo.LanguageConfigs.Add(new UserFeatureLanguageConfigDO()
                        {
                            Language = "zh-CN",
                            FeatureName = "Lista proiectelor"
                        });
                        res = await _adminFeaturesDBRepository.UpdateOneAsync("Code", featureInfo.Code, featureInfo, string.Empty, true, false);
                    }

                    featureInfo = await _adminFeaturesDBRepository.FindOneAsync("Code", "XMTS");
                    if (!string.IsNullOrEmpty(featureInfo?.Code))
                    {
                        featureInfo.LanguageConfigs.Add(new UserFeatureLanguageConfigDO()
                        {
                            Language = "zh-CN",
                            FeatureName = "Înregistrare push"
                        });
                        res = await _adminFeaturesDBRepository.UpdateOneAsync("Code", featureInfo.Code, featureInfo, string.Empty, true, false);
                    }

                    featureInfo = await _adminFeaturesDBRepository.FindOneAsync("Code", "BGTSPZ");
                    if (!string.IsNullOrEmpty(featureInfo?.Code))
                    {
                        featureInfo.LanguageConfigs.Add(new UserFeatureLanguageConfigDO()
                        {
                            Language = "zh-CN",
                            FeatureName = "Raportează impulsul"
                        });
                        res = await _adminFeaturesDBRepository.UpdateOneAsync("Code", featureInfo.Code, featureInfo, string.Empty, true, false);
                    }

                    featureInfo = await _adminFeaturesDBRepository.FindOneAsync("Code", "BGCF");
                    if (!string.IsNullOrEmpty(featureInfo?.Code))
                    {
                        featureInfo.LanguageConfigs.Add(new UserFeatureLanguageConfigDO()
                        {
                            Language = "zh-CN",
                            FeatureName = "Raportează din nou"
                        });
                        res = await _adminFeaturesDBRepository.UpdateOneAsync("Code", featureInfo.Code, featureInfo, string.Empty, true, false);
                    }

                    featureInfo = await _adminFeaturesDBRepository.FindOneAsync("Code", "FWQGL");
                    if (!string.IsNullOrEmpty(featureInfo?.Code))
                    {
                        featureInfo.LanguageConfigs.Add(new UserFeatureLanguageConfigDO()
                        {
                            Language = "zh-CN",
                            FeatureName = "Gestionarea serverelor"
                        });
                        res = await _adminFeaturesDBRepository.UpdateOneAsync("Code", featureInfo.Code, featureInfo, string.Empty, true, false);
                    }

                    featureInfo = await _adminFeaturesDBRepository.FindOneAsync("Code", "CMSTI");
                    if (!string.IsNullOrEmpty(featureInfo?.Code))
                    {
                        featureInfo.LanguageConfigs.Add(new UserFeatureLanguageConfigDO()
                        {
                            Language = "zh-CN",
                            FeatureName = "Șablon CMS"
                        });
                        res = await _adminFeaturesDBRepository.UpdateOneAsync("Code", featureInfo.Code, featureInfo, string.Empty, true, false);
                    }

                    featureInfo = await _adminFeaturesDBRepository.FindOneAsync("Code", "GRADI");
                    if (!string.IsNullOrEmpty(featureInfo?.Code))
                    {
                        featureInfo.LanguageConfigs.Add(new UserFeatureLanguageConfigDO()
                        {
                            Language = "zh-CN",
                            FeatureName = "Protecția clasificării"
                        });
                        res = await _adminFeaturesDBRepository.UpdateOneAsync("Code", featureInfo.Code, featureInfo, string.Empty, true, false);
                    }

                    featureInfo = await _adminFeaturesDBRepository.FindOneAsync("Code", "XTGGGL");
                    if (!string.IsNullOrEmpty(featureInfo?.Code))
                    {
                        featureInfo.LanguageConfigs.Add(new UserFeatureLanguageConfigDO()
                        {
                            Language = "zh-CN",
                            FeatureName = "Manager de anunțuri"
                        });
                        res = await _adminFeaturesDBRepository.UpdateOneAsync("Code", featureInfo.Code, featureInfo, string.Empty, true, false);
                    }

                    featureInfo = await _adminFeaturesDBRepository.FindOneAsync("Code", "XTPZGL");
                    if (!string.IsNullOrEmpty(featureInfo?.Code))
                    {
                        featureInfo.LanguageConfigs.Add(new UserFeatureLanguageConfigDO()
                        {
                            Language = "zh-CN",
                            FeatureName = "Manager de mediu"
                        });
                        res = await _adminFeaturesDBRepository.UpdateOneAsync("Code", featureInfo.Code, featureInfo, string.Empty, true, false);
                    }

                    featureInfo = await _adminFeaturesDBRepository.FindOneAsync("Code", "DDJYJLLB");
                    if (!string.IsNullOrEmpty(featureInfo?.Code))
                    {
                        featureInfo.LanguageConfigs.Add(new UserFeatureLanguageConfigDO()
                        {
                            Language = "zh-CN",
                            FeatureName = "Finance"
                        });
                        res = await _adminFeaturesDBRepository.UpdateOneAsync("Code", featureInfo.Code, featureInfo, string.Empty, true, false);
                    }

                    return res;
                }
                catch (Exception ex)
                {
                    WingServerCommon.Log.Logger.WriteLineError($"AdminFeatureDBService UpdateManagementAuthForRODBAsync error, ex: {ex}");
                    return res;
                }
            }
        }

        /// <summary>
        /// 添加一个管理端权限-科研版
        /// </summary>
        public async Task<bool> UpdateAdminFeatureForResearchProjectDBAsync()
        {
            using (new Performance(_performanceThreshold, "{0} UpdateAdminFeatureForResearchProjectDBAsync execute", $"{this.GetType().Name}"))
            {
                try
                {
                    WingServerCommon.Log.Logger.WriteLineInfo($"AdminDBService UpdateAdminFeatureForResearchProjectDBAsync Starting...");
                    var res = false;
                    var featureInfo = await _adminFeaturesDBRepository.FindOneAsync("Code", "KYYHLB");
                    if (string.IsNullOrEmpty(featureInfo?.Code))
                    {
                        var adminFeatureInfo = new AdminFeatureDO()
                        {
                            FeatureCode = "KYYHLB",
                            FeatureName = "账号管理(科研版)",
                            Description = "",
                            FatherCode = "YHGL",
                            UniqueCode = "KYYHLB"
                        };
                        adminFeatureInfo.LanguageConfigs = new List<UserFeatureLanguageConfigDO>(){
                            new UserFeatureLanguageConfigDO() {
                                Language = "zh-CN",
                                FeatureName = "账号管理(科研版)"
                            },
                            new UserFeatureLanguageConfigDO() {
                                Language = "en-US",
                                FeatureName = "Account Manager(Research)"
                            },
                            new UserFeatureLanguageConfigDO() {
                                Language = "es-ES",
                                FeatureName = "Account Manager(Research)"
                            },
                            new UserFeatureLanguageConfigDO() {
                                Language = "ru-RU",
                                FeatureName = "Account Manager(Research)"
                            },
                            new UserFeatureLanguageConfigDO() {
                                Language = "ro-RO",
                                FeatureName = "Account Manager(Research)"
                            }
                        };

                        var result = await _adminFeaturesDBRepository.InsertOneAsync(adminFeatureInfo, new Dictionary<string, string>(), string.Empty, false);
                        if (!string.IsNullOrEmpty(result))
                        {
                            res = true;
                        }
                    }
                    return res;
                }
                catch (Exception ex)
                {
                    WingServerCommon.Log.Logger.WriteLineError($"AdminDBService UpdateAdminFeatureForResearchProjectDBAsync error, ex: {ex}");
                    return false;
                }
            }
        }

        /// <summary>
        /// 添加一个管理端权限-科研版
        /// </summary>
        public async Task<bool> UpdateDongleFeatureForResearchProjectDBAsync()
        {
            using (new Performance(_performanceThreshold, "{0} UpdateDongleFeatureForResearchProjectDBAsync execute", $"{this.GetType().Name}"))
            {
                try
                {
                    WingServerCommon.Log.Logger.WriteLineInfo($"AdminDBService UpdateDongleFeatureForResearchProjectDBAsync Starting...");
                    var res = false;
                    var featureInfo = await _adminFeaturesDBRepository.FindOneAsync("Code", "KYDGYHLB");
                    if (string.IsNullOrEmpty(featureInfo?.Code))
                    {
                        var adminFeatureInfo = new AdminFeatureDO()
                        {
                            FeatureCode = "KYDGYHLB",
                            FeatureName = "智能卡管理",
                            Description = "",
                            FatherCode = "YHGL",
                            UniqueCode = "KYDGYHLB"
                        };
                        adminFeatureInfo.LanguageConfigs = new List<UserFeatureLanguageConfigDO>(){
                            new UserFeatureLanguageConfigDO() {
                                Language = "zh-CN",
                                FeatureName = "智能卡管理"
                            },
                            new UserFeatureLanguageConfigDO() {
                                Language = "en-US",
                                FeatureName = "Smart Card Management"
                            },
                            new UserFeatureLanguageConfigDO() {
                                Language = "es-ES",
                                FeatureName = "Gestión de Tarjetas Inteligentes"
                            },
                            new UserFeatureLanguageConfigDO() {
                                Language = "ru-RU",
                                FeatureName = "Управление смарт-картами"
                            },
                            new UserFeatureLanguageConfigDO() {
                                Language = "ro-RO",
                                FeatureName = "Administrare Cartele Inteligente"
                            }
                        };

                        var result = await _adminFeaturesDBRepository.InsertOneAsync(adminFeatureInfo, new Dictionary<string, string>(), string.Empty, false);
                        if (!string.IsNullOrEmpty(result))
                        {
                            res = true;
                        }
                    }
                    return res;
                }
                catch (Exception ex)
                {
                    WingServerCommon.Log.Logger.WriteLineError($"AdminDBService UpdateDongleFeatureForResearchProjectDBAsync error, ex: {ex}");
                    return false;
                }
            }
        }

        /// <summary>
        /// 更新管理员角色西班牙语
        /// </summary>
        /// <returns>
        private async Task<bool> UpdateAdminRoleForResearchProjectDBAsync()
        {
            using (new Performance(_performanceThreshold, "{0} UpdateAdminRoleForResearchProjectDBAsync execute", $"{this.GetType().Name}"))
            {
                var res = false;
                try
                {
                    //新增菜单，权限
                    var generalGroup = "G180EC8080CA043ADA7F153F751D6BDD7";
                    var systemGroup = "G180EC8080CB143ADA7F153F751D6BDD7";
                    var adminAuthorityGroupsInfo = await _adminAuthorityGroupDBRepository.FindOneAsync("Code", generalGroup);
                    if (string.IsNullOrEmpty(adminAuthorityGroupsInfo?.Code))
                    {
                        var model = new AdminAuthorityGroupDO()
                        {
                            Code = generalGroup,
                            Description = "",
                            AdminCodes = new List<string>(),
                            Features = new List<string>() {
                                "YHGL",
                                "KYYHLB",
                                "KYDGYHLB"
                            },
                            CreateAdmin = string.Empty
                        };
                        var result1 = await _adminAuthorityGroupDBRepository.InsertOneAsync(model, new Dictionary<string, string>(), string.Empty, false);
                    }
                    var adminRoleInfo = await _adminRoleDBRepository.FindOneAsync("RoleName", "科研软件普通用户角色");
                    var generalRoleCode = "";
                    var systemRoleCode = "";
                    if (string.IsNullOrEmpty(adminRoleInfo?.Code))
                    {
                        var model = new AdminRoleDO()
                        {
                            RoleName = "科研软件普通用户角色",
                            AdminGroupCode = generalGroup,
                            Description = "科研软件普通用户角色"
                        };
                        model.LanguageConfigs = new List<AdminRoleLanguageConfigDO>() {
                            new AdminRoleLanguageConfigDO() {
                                Language = "en-US",
                                RoleName = "Research Project General User Role",
                                Description = "Research Project General User Role",
                            },
                            new AdminRoleLanguageConfigDO() {
                                Language = "es-ES",
                                RoleName = "Research Project General User Role",
                                Description = "Research Project General User Role",
                            },
                            new AdminRoleLanguageConfigDO() {
                                Language = "ro-RO",
                                RoleName = "Research Project General User Role",
                                Description = "Research Project General User Role",
                            },
                            new AdminRoleLanguageConfigDO() {
                                Language = "ru-RU",
                                RoleName = "Research Project General User Role",
                                Description = "Research Project General User Role",
                            },
                            new AdminRoleLanguageConfigDO() {
                                Language = "zh-CN",
                                RoleName = "科研软件普通用户角色",
                                Description = "科研软件普通用户角色"
                            }
                        };
                        generalRoleCode = await _adminRoleDBRepository.InsertOneAsync(model, new Dictionary<string, string>(), string.Empty, false);
                    }
                    else
                    {
                        generalRoleCode = adminRoleInfo?.Code;
                    }

                    adminAuthorityGroupsInfo = await _adminAuthorityGroupDBRepository.FindOneAsync("Code", systemGroup);
                    if (string.IsNullOrEmpty(adminAuthorityGroupsInfo?.Code))
                    {
                        var model = new AdminAuthorityGroupDO()
                        {
                            Code = systemGroup,
                            Description = "",
                            AdminCodes = new List<string>(),
                            Features = new List<string>() {
                                "YHGL",
                                "KYYHLB",
                                "KYDGYHLB",
                                "QXLB",
                                "HTGL",
                                "ZXZH",
                                "XTRZ"
                            },
                            CreateAdmin = string.Empty
                        };
                        var result3 = await _adminAuthorityGroupDBRepository.InsertOneAsync(model, new Dictionary<string, string>(), string.Empty, false);
                    }
                    adminRoleInfo = await _adminRoleDBRepository.FindOneAsync("RoleName", "科研软件系统管理员角色");
                    if (string.IsNullOrEmpty(adminRoleInfo?.Code))
                    {
                        var model = new AdminRoleDO()
                        {
                            RoleName = "科研软件系统管理员角色",
                            AdminGroupCode = systemGroup,
                            Description = "科研软件系统管理员角色"
                        };
                        model.LanguageConfigs = new List<AdminRoleLanguageConfigDO>() {
                            new AdminRoleLanguageConfigDO() {
                                Language = "en-US",
                                RoleName = "Research Project System User Role",
                                Description = "Research Project System User Role",
                            },
                            new AdminRoleLanguageConfigDO() {
                                Language = "es-ES",
                                RoleName = "Research Project System User Role",
                                Description = "Research Project System User Role",
                            },
                            new AdminRoleLanguageConfigDO() {
                                Language = "ro-RO",
                                RoleName = "Research Project System User Role",
                                Description = "Research Project System User Role",
                            },
                            new AdminRoleLanguageConfigDO() {
                                Language = "ru-RU",
                                RoleName = "Research Project System User Role",
                                Description = "Research Project System User Role",
                            },
                            new AdminRoleLanguageConfigDO() {
                                Language = "zh-CN",
                                RoleName = "科研软件系统管理员角色",
                                Description = "科研软件系统管理员角色"
                            }
                        };
                        systemRoleCode = await _adminRoleDBRepository.InsertOneAsync(model, new Dictionary<string, string>(), string.Empty, false);
                    }
                    else
                    {
                        systemRoleCode = adminRoleInfo?.Code;
                    }

                    //新增用户
                    var generalAdmin = "A180EC801ACA043ADA7F153F751D6BDD7";
                    var systemAdmin = "A180EC802BCB143ADA7F153F751D6BDD7";
                    var adminInfo = await _adminDBRepository.FindOneAsync("Code", generalAdmin);
                    if (string.IsNullOrEmpty(adminInfo?.Code))
                    {
                        //密码：IDS@123
                        var model = new AdminInfoDO()
                        {
                            Code = generalAdmin,
                            AdminName = "Alberto",
                            SecretPassword = "Symmetry_gifhe34MaK==",
                            FullName = "Alberto",
                            Phone = "",
                            RoleCodes = new List<string>() { generalRoleCode }
                        };
                        var result5 = await _adminDBRepository.InsertOneAsync(model, new Dictionary<string, string>(), string.Empty, false);
                    }
                    adminInfo = await _adminDBRepository.FindOneAsync("Code", systemAdmin);
                    if (string.IsNullOrEmpty(adminInfo?.Code))
                    {
                        //密码：SM@#$abc!123
                        var model = new AdminInfoDO()
                        {
                            Code = systemAdmin,
                            AdminName = "SystemManager",
                            SecretPassword = "Symmetry_iOP08MfvmAavah8N",
                            FullName = "SystemManager",
                            Phone = "",
                            RoleCodes = new List<string>() { systemRoleCode }
                        };
                        var result5 = await _adminDBRepository.InsertOneAsync(model, new Dictionary<string, string>(), string.Empty, false);
                    }

                    var dongleAdmin = "0112AE81EE274B279137809495CD4FC8";
                    adminInfo = await _adminDBRepository.FindOneAsync("Code", dongleAdmin);
                    if (string.IsNullOrEmpty(adminInfo?.Code))
                    {
                        //密码：vinno123
                        await _adminDBRepository.InsertOneAsync(new AdminInfoDO
                        {
                            Code = dongleAdmin,
                            AdminName = "admin",
                            SecretPassword = "Symmetry_rAzIpAWLaxa=",
                            FullName = "admin",
                            Phone = "",
                            RoleCodes = new List<string>() { generalRoleCode }
                        }, new Dictionary<string, string>(), string.Empty, false);
                    }

                    res = true;
                    return res;
                }
                catch (Exception ex)
                {
                    WingServerCommon.Log.Logger.WriteLineError($"AdminFeatureDBService UpdateAdminRoleForResearchProjectDBAsync error, ex: {ex}");
                    return res;
                }
            }
        }
    }
}