using System;
using System.Collections.Generic;
using System.Threading.Tasks;
using WingInterfaceLibrary.DB.Request;
using WingInterfaceLibrary.Interface.DBInterface;
using WingServerCommon.Service;
using WingServerCommon.Mapper;
using WingInterfaceLibrary.DTO.Role;
using WingMongoDBModule.DO;
using WingInterfaceLibrary.Request;
using WingServerCommon.Utilities;

namespace WingMongoDBModule.Service
{
    public partial class DatabaseService : JsonRpcService, IAdminRoleDBService
    {
        /// <summary>
        /// 删除后台角色
        /// </summary>
        /// <param name="id">角色Id</param>
        /// <returns></returns>
        public async Task<bool> DeleteAdminRoleByCodeAsync(string code)
        {
            using (new Performance(_performanceThreshold, "{0} DeleteAdminRoleByCodeAsync execute", $"{this.GetType().Name}"))
            {
                return await _adminRoleDBRepository.DeleteOneAsync("Code", code);
            }
        }

        /// <summary>
        /// 查询后台角色
        /// </summary>
        /// <param name="code">角色编码</param>
        /// <returns></returns>
        public async Task<AdminRoleDTO> FindAdminRoleByCodeAsync(string code)
        {
            using (new Performance(_performanceThreshold, "{0} FindAdminRoleByCodeAsync execute", $"{this.GetType().Name}"))
            {
                var adminRoleDO = await _adminRoleDBRepository.FindOneAsync("Code", code);
                return adminRoleDO.MappingTo<AdminRoleDTO>();
            }
        }

        /// <summary>
        /// 查询后台角色
        /// </summary>
        /// <param name="name">角色名称</param>
        /// <returns></returns>
        public async Task<AdminRoleDTO> FindAdminRoleByNameAsync(string name)
        {
            using (new Performance(_performanceThreshold, "{0} FindAdminRoleByNameAsync execute", $"{this.GetType().Name}"))
            {
                var adminRoleDO = await _adminRoleDBRepository.FindOneAsync("AdminRoleName", name);
                return adminRoleDO.MappingTo<AdminRoleDTO>();
            }
        }


        /// <summary>
        /// > 获取管理员角色数据传输对象的页面集合
        /// </summary>
        /// <param name="PageFilterRequest">分页查询的请求参数，包含分页查询条件和分页查询参数。</param>
        /// <returns>
        /// AdminRoleDTO 的 PageCollection
        /// </returns>
        public async Task<PageCollection<AdminRoleDTO>> FindAdminRolePagesAsync(PageFilterRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} FindAdminRolePagesAsync execute", $"{this.GetType().Name}"))
            {
                var dbRequest = GetDBPageRequest(request);
                var adminRoleDO = await _adminRoleDBRepository.GetPages(dbRequest);
                return PageCollectionConvert<AdminRoleDO, AdminRoleDTO>(adminRoleDO);
            }
        }

        /// <summary>
        /// 插入后台角色
        /// </summary>
        /// <param name="AdminRoleDO">角色信息</param>
        /// <param name="extensionData">扩展信息</param>
        /// <returns></returns>
        public async Task<string> InsertAdminRoleAsync(CreateAdminRoleDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} InsertAdminRoleAsync execute", $"{this.GetType().Name}"))
            {
                return await _adminRoleDBRepository.InsertOneAsync(request.Data.MappingTo<AdminRoleDO>(), request.PlatformDatas, request.ExtensionData);
            }
        }

        /// <summary>
        /// 更新后台角色信息
        /// </summary>
        /// <param name="code">角色编码</param>
        /// <param name="data">角色数据</param>
        /// <param name="extensionData">扩展信息</param>
        /// <returns></returns>
        public async Task<bool> UpdateAdminRoleByCodeAsync(string code, AdminRoleDTO data, string extensionData)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateAdminRoleByCodeAsync execute", $"{this.GetType().Name}"))
            {
                return await _adminRoleDBRepository.UpdateOneAsync("Code", code, data.MappingTo<AdminRoleDO>(), extensionData);
            }
        }


        /// <summary>
        /// 用于通过管理员角色代码查找管理员角色列表
        /// </summary>
        /// <param name="adminRoleCodes">要查询的角色代码列表。如果为空，则查询所有角色。</param>
        /// <returns>
        /// AdminRoleDTO 对象的列表。
        /// </returns>
        public async Task<List<AdminRoleDTO>> FindAdminRoleListByCodesAsync(List<string> adminRoleCodes)
        {
            using (new Performance(_performanceThreshold, "{0} FindAdminRoleListByCodesAsync execute", $"{this.GetType().Name}"))
            {
                if (adminRoleCodes?.Count > 0)
                {
                    var dic = new Dictionary<string, List<string>>();
                    dic.Add("Code", adminRoleCodes);
                    var adminRoleList = await _adminRoleDBRepository.FindInAsync(dic);
                    return adminRoleList.MappingTo<List<AdminRoleDTO>>();
                }
                else
                {
                    var adminRoleList = await _adminRoleDBRepository.FindAllAsync(null);
                    return adminRoleList.MappingTo<List<AdminRoleDTO>>();
                }
            }
        }
    }
}