using System.Collections.Generic;
using System.Threading.Tasks;
using WingServerCommon.Service;
using WingServerCommon.Mapper;
using System.Linq;
using System;
using WingMongoDBModule.DO;
using WingInterfaceLibrary.DB.Request;
using WingMongoDBModule.Mongodb.Driver;
using WingMongoDBModule.Entity;
using MongoDB.Driver;
using WingInterfaceLibrary.Interface.DBInterface;
using WingInterfaceLibrary.Enum;
using WingInterfaceLibrary.DTO.AdminStatistic;
using WingInterfaceLibrary.DTO.Dictionary;
using WingServerCommon.Utilities;
using WingInterfaceLibrary.DTO.Common;

namespace WingMongoDBModule.Service
{
    public partial class DatabaseService : JsonRpcService, IAdminStatisticDBService
    {
        /// <summary>
        /// 查询管理端统计配置集合
        /// </summary>
        /// <returns></returns>
        public async Task<IList<AdminStatisticDTO>> GetAdminStatisticAsync(GetAdminStatisticDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetAdminStatisticAsync execute", $"{this.GetType().Name}"))
            {
                var dic = new Dictionary<string, string>();
                dic.Add("Enabled", bool.TrueString);
                var entities = await _adminStatisticDBRepository.FindListAsync(dic);
                return entities.MappingTo<List<AdminStatisticDTO>>();
            }
        }

        /// <summary>
        /// 查询管理员统计结果集合
        /// </summary>
        /// <returns></returns>
        public async Task<IList<AdminStatisticRecordDTO>> GetAdminStatisticRecordsAsync(GetAdminStatisticRecordsDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetAdminStatisticRecordsAsync execute", $"{this.GetType().Name}"))
            {
                var dic = new Dictionary<string, string>();
                dic.Add("AdminCode", request.AdminCode);
                var entities = await _adminStatisticRecordDBRepository.FindListAsync(dic);
                return entities.MappingTo<List<AdminStatisticRecordDTO>>();
            }
        }

        /// <summary>
        /// 查询已过期的统计数据
        /// </summary>
        /// <returns></returns>
        public async Task<IList<AdminStatisticRecordDTO>> GetExpiredRecordsAsync(GetExpiredRecordsDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetExpiredRecordsAsync execute", $"{this.GetType().Name}"))
            {
                MongoCustomerFilter<MongoTable> customerFilter = new();
                customerFilter.AppendLt("BaseData.NextHandleTime", DateTime.UtcNow);
                var filter = customerFilter.AndCombine();
                var entities = await _adminStatisticRecordDBRepository.FindAllAsync(filter);
                return entities.MappingTo<List<AdminStatisticRecordDTO>>();
            }
        }

        /// <summary>
        /// 删除管理员的统计数据
        /// </summary>
        /// <returns></returns>
        public async Task<bool> DeleteAdminStatisticRecordAsync(DeleteAdminStatisticRecordDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} DeleteAdminStatisticRecordAsync execute", $"{this.GetType().Name}"))
            {
                var dic = new Dictionary<string, string>();
                dic.Add("AdminCode", request.AdminCode);
                var entities = await _adminStatisticRecordDBRepository.FindListAsync(dic);
                if (entities.Any())
                {
                    var list = entities.Select(f => ("Code", f.Code)).ToList();
                    await _adminStatisticRecordDBRepository.RealDeleteAllAsync(list);
                }
                return true;
            }
        }

        /// <summary>
        /// 保存统计结果
        /// </summary>
        /// <returns></returns>
        public async Task<bool> SaveAdminStatisticRecordAsync(SaveAdminStatisticRecordDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} SaveAdminStatisticRecordAsync execute", $"{this.GetType().Name}"))
            {
                var dic = new Dictionary<string, string>();
                dic.Add("AdminCode", request.AdminCode);
                dic.Add("AdminStatisticCode", request.AdminStatisticCode);
                var entities = await _adminStatisticRecordDBRepository.FindListAsync(dic);
                if (entities.Count == 1)
                {
                    var updateEntity = entities.FirstOrDefault();
                    updateEntity.AdminStatisticCode = request.AdminStatisticCode;
                    updateEntity.AdminStatisticName = request.AdminStatisticName;
                    updateEntity.XAxisDatas = request.XAxisDatas;
                    updateEntity.YAxisDatas = request.YAxisDatas;
                    updateEntity.LastestUpdateTime = DateTime.UtcNow;
                    updateEntity.NextHandleTime = request.NextHandleTime;
                    updateEntity.ServerHost = request.ServerHost;
                    if (request.Children != null)
                    {
                        updateEntity.Children = request.Children.MappingTo<List<StatisticInfoDO>>();
                    }
                    return await _adminStatisticRecordDBRepository.UpdateOneAsync("Code", updateEntity.Code, updateEntity, string.Empty);
                }
                else if (entities.Count > 1)
                {
                    var list = entities.Select(f => ("Code", f.Code)).ToList();
                    await _adminStatisticRecordDBRepository.RealDeleteAllAsync(list);
                }
                var insertEntity = new AdminStatisticRecordDO();
                insertEntity.AdminCode = request.AdminCode;
                insertEntity.AdminStatisticCode = request.AdminStatisticCode;
                insertEntity.AdminStatisticName = request.AdminStatisticName;
                insertEntity.XAxisDatas = request.XAxisDatas;
                insertEntity.YAxisDatas = request.YAxisDatas;
                insertEntity.LastestUpdateTime = DateTime.UtcNow;
                insertEntity.NextHandleTime = request.NextHandleTime;
                insertEntity.ServerHost = request.ServerHost;
                if (request.Children != null)
                {
                    insertEntity.Children = request.Children.MappingTo<List<StatisticInfoDO>>();
                }
                await _adminStatisticRecordDBRepository.InsertOneAsync(insertEntity, new Dictionary<string, string>(), string.Empty);
                return true;
            }
        }

        /// <summary>
        /// 认证用户
        /// </summary>
        /// <param name="adminCode"></param>
        /// <returns></returns>
        public async Task<StatisticInfoDTO> GetUserActivateInfoStatisticAsync(GetUserActivateInfoStatisticDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetUserActivateInfoStatisticAsync execute", $"{this.GetType().Name}"))
            {
                var resultData = new StatisticInfoDTO();
                var adminEntity = await _adminDBRepository.FindOneAsync("Code", request.AdminCode);
                if (adminEntity != null && !string.IsNullOrWhiteSpace(adminEntity.Code))
                {
                    var isSuperAdmin = adminEntity.RoleCodes.Contains("AdminRoleDO01");
                    List<string> orgCodes = null;
                    if (!isSuperAdmin)
                    {
                        var organizations = await GetAdminOrganizationsAsync(adminEntity.Code);
                        orgCodes = organizations.Select(x => x.Code).ToList();
                    }
                    var statusList = new List<UserInfoStateEnum> { UserInfoStateEnum.Nonactivated, UserInfoStateEnum.Activated };
                    foreach (var activate in statusList)
                    {
                        var filter = GetFilter(activate, orgCodes);
                        var count = await _userDBRepository.CountDocumentsAsync(filter);
                        AddDataInStatisticInfo(resultData, activate.ToString(), count);
                    }
                }
                return resultData;
            }
        }

        private void AddDataInStatisticInfo(StatisticInfoDTO resultData, string x, double y)
        {
            resultData.XAxisDatas.Add(x);
            resultData.YAxisDatas.Add(new List<double> { y });
        }

        private void DefaultDataInStatisticInfo(StatisticInfoDTO resultData, string x, double y)
        {
            if (!resultData.XAxisDatas.Any())
            {
                AddDataInStatisticInfo(resultData, x, y);
            }
        }

        /// <summary>
        /// 我的待办
        /// </summary>
        /// <returns></returns>
        public async Task<StatisticInfoDTO> GetUserActivateLogStatisticAsync(GetUserActivateLogStatisticDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetUserActivateLogStatisticAsync execute", $"{this.GetType().Name}"))
            {
                var resultData = new StatisticInfoDTO();
                var adminEntity = await _adminDBRepository.FindOneAsync("Code", request.AdminCode);
                if (adminEntity != null && !string.IsNullOrWhiteSpace(adminEntity.Code))
                {
                    var isSuperAdmin = adminEntity.RoleCodes.Contains("AdminRoleDO01");
                    if (isSuperAdmin)
                    {
                        MongoCustomerFilter<MongoTable> mongoCustomerFilter = new();
                        List<(string, string)> eqFilters = new();
                        eqFilters.Add(("ApplyState", ApplyStateEnum.Applying.ToString()));
                        mongoCustomerFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, eqFilters.ToArray());
                        var count = await _identityApplyDBRepository.CountDocumentsAsync(mongoCustomerFilter.AndCombine());
                        AddDataInStatisticInfo(resultData, "applying", count);
                        return resultData;
                    }
                    else
                    {
                        var filter = new Dictionary<string, string>();
                        filter.Add("ApplyState", ApplyStateEnum.Applying.ToString());
                        var applyRecords = await _identityApplyDBRepository.FindListAsync(filter);
                        if (applyRecords.Any())
                        {
                            var userCodes = applyRecords.Select(x => x.UserCode).ToList();

                            var organizations = await GetAdminOrganizationsAsync(adminEntity.Code);
                            var orgCodes = organizations.Select(x => x.Code).ToList();

                            MongoCustomerFilter<MongoTable> customerFilter = new();
                            MongoCustomerFilter<MongoDataItem> queryItemsFilter = new();
                            queryItemsFilter.AppendEq((item => item.Key, "Code"));
                            queryItemsFilter.AppendIn(item => item.Value, userCodes);
                            if (queryItemsFilter.HasFilter)
                            {
                                customerFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
                            }
                            queryItemsFilter = new();
                            queryItemsFilter.AppendEq((item => item.Key, "OrganizationCode"));
                            queryItemsFilter.AppendIn(item => item.Value, orgCodes.ToList());
                            if (queryItemsFilter.HasFilter)
                            {
                                customerFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
                            }
                            var count = await _userDBRepository.CountDocumentsAsync(customerFilter.AndCombine());
                            AddDataInStatisticInfo(resultData, "applying", count);
                        }
                    }
                }
                DefaultDataInStatisticInfo(resultData, "applying", 0);
                return resultData;
            }
        }

        /// <summary>
        /// 认证角色
        /// </summary>
        /// <returns></returns>
        public async Task<StatisticInfoDTO> GetUserRoleStatisticAsync(GetUserRoleStatisticDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetUserRoleStatisticAsync execute", $"{this.GetType().Name}"))
            {
                var resultData = new StatisticInfoDTO();
                var adminEntity = await _adminDBRepository.FindOneAsync("Code", request.AdminCode);
                if (adminEntity != null && !string.IsNullOrWhiteSpace(adminEntity.Code))
                {
                    var isSuperAdmin = adminEntity.RoleCodes.Contains("AdminRoleDO01");
                    List<string> orgCodes = null;
                    if (!isSuperAdmin)
                    {
                        var organizations = await GetAdminOrganizationsAsync(adminEntity.Code);
                        orgCodes = organizations.Select(x => x.Code).ToList();
                    }
                    var roleList = await _roleDBRepository.FindAllAsync();
                    foreach (var roleDO in roleList)
                    {
                        var filter = GetFilter(roleDO.Code, orgCodes);
                        var count = await _userDBRepository.CountDocumentsAsync(filter);
                        AddDataInStatisticInfo(resultData, roleDO.Code, count);
                    }
                }
                return resultData;
            }
        }

        /// <summary>
        /// 机构分布
        /// </summary>
        /// <returns></returns>
        public async Task<StatisticInfoDTO> GetOrganizationStatisticAsync(GetOrganizationStatisticDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetOrganizationStatisticAsync execute", $"{this.GetType().Name}"))
            {
                var resultData = new StatisticInfoDTO();
                var adminEntity = await _adminDBRepository.FindOneAsync("Code", request.AdminCode);
                if (adminEntity != null && !string.IsNullOrWhiteSpace(adminEntity.Code))
                {
                    var isSuperAdmin = adminEntity.RoleCodes.Contains("AdminRoleDO01");
                    List<string> orgCodes = null;
                    if (!isSuperAdmin)
                    {
                        var organizations = await GetAdminOrganizationsAsync(adminEntity.Code);
                        orgCodes = organizations.Select(x => x.Code).ToList();
                    }
                    var statusList = new List<OrganizationPatientTypeEnum> { OrganizationPatientTypeEnum.Person, OrganizationPatientTypeEnum.Animals };
                    foreach (var patientType in statusList)
                    {
                        var filter = GetFilter(patientType, orgCodes);
                        var count = await _organizationDBRepository.CountDocumentsAsync(filter);
                        AddDataInStatisticInfo(resultData, patientType.ToString(), count);
                    }
                }
                return resultData;
            }
        }

        /// <summary>
        /// 机型分布，设备型号分布
        /// </summary>
        /// <returns></returns>
        public async Task<StatisticInfoDTO> GetDeviceStatisticAsync(GetDeviceStatisticDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetDeviceStatisticAsync execute", $"{this.GetType().Name}"))
            {
                var resultData = new StatisticInfoDTO
                {
                    Children = new List<StatisticInfoDTO>()
                };
                var adminEntity = await _adminDBRepository.FindOneAsync("Code", request.AdminCode);
                if (adminEntity != null && !string.IsNullOrWhiteSpace(adminEntity.Code))
                {
                    var isSuperAdmin = adminEntity.RoleCodes.Contains("AdminRoleDO01");
                    List<DeviceInfoDO> deviceInfos = new List<DeviceInfoDO>();
                    if (isSuperAdmin)
                    {
                        deviceInfos = await _deviceInfoDBRepository.FindAllAsync();
                    }
                    else
                    {
                        var organizations = await GetAdminOrganizationsAsync(adminEntity.Code);
                        var dic = new Dictionary<string, List<string>>();
                        dic.Add("Code", organizations.Select(x => x.Code).ToList());
                        deviceInfos = await _deviceInfoDBRepository.FindInAsync(dic);
                    }
                    var deviceTypeList = await FindDictionaryItemsAsync(new FindDictionaryItemsDBRequest { DictionaryType = (int)DictionaryTypeEnum.DeviceType });
                    var deviceModelList = await FindDictionaryItemsAsync(new FindDictionaryItemsDBRequest { DictionaryType = (int)DictionaryTypeEnum.DeviceModel });
                    foreach (var deviceTypeInfo in deviceTypeList)
                    {
                        var count = deviceInfos.Count(x => x.DeviceType == deviceTypeInfo.DictionaryCode);
                        AddDataInStatisticInfo(resultData, deviceTypeInfo.Value, count);
                        var deviceModels = deviceModelList.Where(x => x.ParentCode == deviceTypeInfo.DictionaryCode)?.ToList() ?? new List<DictionaryDTO>();
                        var xAxisDatas = new List<string>();
                        var yAxisDatas = new List<List<double>>();
                        var dataItemList = new List<DataItemDTO>();
                        foreach (var deviceModel in deviceModels)
                        {
                            var value = deviceInfos.Count(x => x.DeviceType == deviceTypeInfo.DictionaryCode && x.DeviceModel == deviceModel.Value);
                            var dataItem = new DataItemDTO() {
                                Key = deviceModel.Value,
                                Value = value.ToString()
                            };
                            dataItemList.Add(dataItem);
                        }
                        if (dataItemList?.Count > 0) {
                            var orderbyDatas = dataItemList.OrderByDescending(c => Convert.ToDouble(c.Value)).ToList();
                            var rangeDatas = new List<DataItemDTO>();
                            if (orderbyDatas.Count > 6) {
                                rangeDatas = orderbyDatas.GetRange(0, 6);
                            }
                            else {
                                rangeDatas = orderbyDatas;
                            }
                            foreach(var item in rangeDatas)
                            {
                                xAxisDatas.Add(item.Key);
                                yAxisDatas.Add(new List<double> { Convert.ToDouble(item.Value) });
                            }
                        }
                        resultData.Children.Add(new StatisticInfoDTO
                        {
                            XAxisDatas = xAxisDatas,
                            YAxisDatas = yAxisDatas,
                        });
                    }

                }
                return resultData;
            }
        }

        private FilterDefinition<MongoTable> GetFilter(UserInfoStateEnum activate, IList<string> orgCodes = null)
        {
            MongoCustomerFilter<MongoTable> customerFilter = new();
            if (activate == UserInfoStateEnum.Nonactivated)
            {
                customerFilter.AppendSizeLt("BaseData.RoleCodes", 1);
            }
            else
            {
                customerFilter.AppendSizeGte("BaseData.RoleCodes", 1);
            }
            if (orgCodes != null)
            {
                customerFilter.AppendIn("BaseData.OrganizationCode", orgCodes.ToList());
            }
            return customerFilter.AndCombine();
        }

        private FilterDefinition<MongoTable> GetFilter(string roleCode, IList<string> orgCodes = null)
        {
            MongoCustomerFilter<MongoTable> customerFilter = new();
            MongoCustomerFilter<MongoDataItem> queryItemsFilter = new();
            queryItemsFilter.AppendEq((item => item.Key, "UserState"));
            queryItemsFilter.AppendRegex(item => item.Value, UserInfoStateEnum.Activated.ToString(), true);

            queryItemsFilter = new();
            customerFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
            customerFilter.AppendArrayEq("BaseData.RoleCodes", roleCode);

            if (orgCodes != null)
            {
                queryItemsFilter = new();
                queryItemsFilter.AppendEq((item => item.Key, "OrganizationCode"));
                queryItemsFilter.AppendIn(item => item.Value, orgCodes.ToList());
                if (queryItemsFilter.HasFilter)
                {
                    customerFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
                }
            }
            return customerFilter.AndCombine();
        }

        private FilterDefinition<MongoTable> GetFilter(OrganizationPatientTypeEnum patientType, IList<string> orgCodes = null)
        {
            MongoCustomerFilter<MongoTable> customerFilter = new();
            customerFilter.AppendIn("BaseData.PatientType", new List<int> { (int)patientType });
            customerFilter.AppendIn("BaseData.OrganizationType", new List<int> { (int)OrganizationTypeEnum.Corporation });
            if (orgCodes != null)
            {
                customerFilter.AppendIn("BaseData.Code", orgCodes.ToList());
            }
            return customerFilter.AndCombine();
        }
    }
}