using System.Threading.Tasks;
using WingServerCommon.Service;
using WingInterfaceLibrary.DTO.Message;
using WingInterfaceLibrary.Request.DBRequest;
using WingInterfaceLibrary.Result;
using WingInterfaceLibrary.Interface.DBInterface;
using WingServerCommon.Utilities;
using WingMongoDBModule.Entities;
using WingServerCommon.Mapper;
using System;
using MongoDB.Driver;
using MongoDB.Bson;
using WingMongoDBModule.Request;
using System.Collections.Generic;
using WingInterfaceLibrary.Enum;

namespace WingMongoDBModule.Service
{
    public partial class DatabaseService : JsonRpcService, IAnnouncementDBService
    {
        /// <summary>
        /// 增加公告内容接口
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> AddAnnouncementDBAsync(AddAnnouncementDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} AddAnnouncementDBAsync execute", "DatabaseService"))
            { 
                var entity = request.MappingTo<AnnouncementInfoEntity>();
                var result = await _announcementInfoDBRepository.InsertOneAsync(entity);
                if (!string.IsNullOrEmpty(result))
                {
                    return true;
                }
                else
                {
                    return false;
                }
            }
        }

        /// <summary>
        /// 根据用户Code获取公告内容列表接口
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageResult<AnnouncementInfoDTO>> QueryAnnouncementsDBByPageAsync(QueryAnnouncementsDBRequest request)
        {
            var keyword = request.Keyword;
            var builder = Builders<AnnouncementInfoEntity>.Filter;
            var filter = builder.Where(c => c.IsDelete == false);
            if (!string.IsNullOrEmpty(request.Language))
            {
                var languageFilter = builder.ElemMatch(x => x.LanguageConfigs, Builders<AnnouncementLanguageConfigEntity>.Filter.Eq(c => c.Language, request.Language));
                filter = filter & languageFilter;
            }
            if (!string.IsNullOrEmpty(keyword))
            {
                keyword = EscapeExprSpecialWord(keyword);//字符串转义
                var titleFilter = builder.ElemMatch(x => x.LanguageConfigs, Builders<AnnouncementLanguageConfigEntity>.Filter.Regex(c => c.Title, new BsonRegularExpression(keyword, "i")));
                var contentFilter = builder.ElemMatch(x => x.LanguageConfigs, Builders<AnnouncementLanguageConfigEntity>.Filter.Regex(c => c.Content, new BsonRegularExpression(keyword, "i")));
                filter = filter & (titleFilter | contentFilter);
            }
            var announcementStatusInt = Convert.ToInt32(request.AnnouncementStatus);
             if (announcementStatusInt > 0)
            {
                filter = filter & builder.Where(c => c.AnnouncementStatus == request.AnnouncementStatus);
            }
            var announcementTypeInt = Convert.ToInt32(request.AnnouncementType);
            if (announcementTypeInt > 0)
            {
                filter = filter & builder.Where(c => c.AnnouncementType == request.AnnouncementType);
            }
            if (request.StartTime > DateTime.MinValue)
            {
                filter = filter & builder.Where(x => x.CreateTime >= request.StartTime);
            }
            if (request.EndTime > DateTime.MinValue)
            {
                filter = filter & builder.Where(x => x.CreateTime < request.EndTime.AddSeconds(1));
            }
            if (request.QueryType == 1)
            {
                filter = filter & builder.Where(x => x.AnnouncementType == AnnouncementTypeEnum.Carousel || x.AnnouncementType == AnnouncementTypeEnum.Information);
            }
            else if (request.QueryType == 2)
            {
                filter = filter & builder.Where(x => x.AnnouncementType == AnnouncementTypeEnum.OperateVideo || x.AnnouncementType == AnnouncementTypeEnum.Question);
            }
            else
            {
                filter = filter & builder.Where(x => x.AnnouncementType == AnnouncementTypeEnum.Broadcast || x.AnnouncementType == AnnouncementTypeEnum.Maintenance || x.AnnouncementType == AnnouncementTypeEnum.WXOfficialAccountCarousel);
            }
            if (!string.IsNullOrWhiteSpace(request.DeviceType))
            {
                filter = filter & builder.Where(x => x.DeviceType == request.DeviceType);
            }
            var dbPage = new DBPageRequest<AnnouncementInfoEntity>();
            dbPage.CurrentPage = request.CurrentPage;
            dbPage.PageSize = request.PageSize;
            dbPage.Filter = filter;
            var pageResult = await _announcementInfoDBRepository.GetPages(dbPage);
            return PageResultConvert<AnnouncementInfoEntity, AnnouncementInfoDTO>(pageResult);
        }

        /// <summary>
        /// 根据公告Code获取内容接口
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<AnnouncementInfoDTO> GetAnnouncementDBAsync(GetAnnouncementDBRequest request)        
        {
            using (new Performance(_performanceThreshold, "{0} GetAnnouncementDBAsync execute", "DatabaseService"))
            { 
                var filter = Builders<AnnouncementInfoEntity>.Filter.Where(c => c.Code == request.AnnouncementCode && c.IsDelete == false);
                var findEntity = await _announcementInfoDBRepository.FindOneAsync(filter);
                var result = findEntity.MappingTo<AnnouncementInfoDTO>();
                return result;
            }
        }

        /// <summary>
        /// 更新公告内容接口
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> UpdateAnnouncementDBAsync(UpdateAnnouncementDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateAnnouncementDBAsync execute", "DatabaseService"))
            { 
                var filter = Builders<AnnouncementInfoEntity>.Filter.Where(c => c.Code == request.Code && c.IsDelete == false);
                var findEntity = await _announcementInfoDBRepository.FindOneAsync(filter) ?? new AnnouncementInfoEntity();
                if (!string.IsNullOrEmpty(findEntity.Code))
                {
                    var announcementStatusInt = Convert.ToInt32(request.AnnouncementStatus);
                    if (announcementStatusInt > 0)
                    {
                        findEntity.AnnouncementStatus = request.AnnouncementStatus;
                    }
                    var announcementTypeInt = Convert.ToInt32(request.AnnouncementType);
                    if (announcementTypeInt > 0)
                    {
                        findEntity.AnnouncementType = request.AnnouncementType;
                    }
                    findEntity.Sort = request.Sort;
                    findEntity.FileToken = request.FileToken;
                    findEntity.DeviceType = request.DeviceType;

                    
                    findEntity.Poster = request.Poster;
                    findEntity.Duration = request.Duration;
                    findEntity.VodFileId = request.VodFileId;
                    findEntity.VideoSize = request.VideoSize;
                    findEntity.VideoToken = request.VideoToken;
                    if (request.LanguageConfigs?.Count > 0)
                    {
                        findEntity.LanguageConfigs = request.LanguageConfigs.MappingTo<List<AnnouncementLanguageConfigEntity>>();
                    }
                    findEntity.UpdateTime = DateTime.UtcNow;
                    var result = await _announcementInfoDBRepository.UpdateOneAsync("Code", findEntity.Code, findEntity);
                    return result;
                }
                else
                {
                    return false;
                }
            }
        }

        /// <summary>
        /// 删除公告内容接口
        /// </summary>
        /// <returns></returns>
        public async Task<bool> DeleteAnnouncementDBByCodeAsync(string code)
        {
            using (new Performance(_performanceThreshold, "{0} DeleteAnnouncementDBByCodeAsync execute", $"{this.GetType().Name}"))
            {
                if (!string.IsNullOrWhiteSpace(code))
                {
                    // var filter = Builders<AnnouncementInfoEntity>.Filter.Where(c => c.Code == code && c.IsDelete == false);
                    // var findEntity = await _announcementInfoDBRepository.FindOneAsync(filter) ?? new AnnouncementInfoEntity();
                    // if (!string.IsNullOrEmpty(findEntity.Code))
                    // {
                    //     if (findEntity.AnnouncementStatus == WingInterfaceLibrary.Enum.AnnouncementStatusEnum.Released)
                    //     {
                    //         return false;
                    //     }
                    // }
                    var result = await _announcementInfoDBRepository.DeleteOneAsync("Code", code);
                    return result;
                }
                return false;
            }
        }

        /// <summary>
        /// 根据用户Code获取公告内容列表接口
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<IList<AnnouncementInfoDTO>> GetAnnouncementsAsync(GetAnnouncementsRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetAnnouncementsAsync execute", $"{this.GetType().Name}"))
            {
                var filter = Builders<AnnouncementInfoEntity>.Filter.Where(w=>w.IsDelete == false && w.AnnouncementStatus == AnnouncementStatusEnum.Released);
                if (request.AnnouncementType == AnnouncementTypeEnum.WXOfficialAccountCarousel)
                {
                    filter = filter & Builders<AnnouncementInfoEntity>.Filter.Where(w => w.AnnouncementType == AnnouncementTypeEnum.WXOfficialAccountCarousel);
                }
                var sort = Builders<AnnouncementInfoEntity>.Sort.Descending(o=>o.CreateTime);
                var entities = await _announcementInfoDBRepository.FindAllAsync(filter, new FindOptions<AnnouncementInfoEntity>{ Sort = sort, Limit = request.Top });
                return entities?.MappingTo<List<AnnouncementInfoDTO>>();
            }
        }
    }
}