using System.Collections.Generic;
using System.Threading.Tasks;
using WingInterfaceLibrary.DTO.CMSStatistic;
using WingInterfaceLibrary.Enum;
using WingServerCommon.Service;
using WingInterfaceLibrary.Interface.DBInterface;
using WingInterfaceLibrary.Request.DBRequest;
using WingServerCommon.Mapper;
using WingServerCommon.Utilities;
using WingMongoDBModule.DO;
using WingInterfaceLibrary.DTO.Message;
using MongoDB.Driver;
using WingMongoDBModule.Entities;

namespace WingMongoDBModule.Service
{
    public partial class DatabaseService : JsonRpcService, ICMSTemplateDBService
    {
        /// <summary>
        /// 获取CMS模板列表
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<IList<CMSTemplateDTO>> GetCMSTemplatesDBAsync(GetCMSTemplatesDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetCMSTemplatesDBAsync execute", $"{this.GetType().Name}"))
            {
                var queryCMSTemplateStatus = request.QueryCMSTemplateStatus;
                var dic = new Dictionary<string, string>();
                if (queryCMSTemplateStatus != QueryCMSTemplateStatusTypeEnum.All)
                {
                    dic.Add("CMSTemplateStateType", queryCMSTemplateStatus.ToString());
                }
                var entities = await _cmsTemplateDBRepository.FindListAsync(dic);
                return entities.MappingTo<List<CMSTemplateDTO>>();
            }
        }

        /// <summary>
        /// 模板名称获取模板
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<CMSTemplateDTO> GetCMSTemplateDBByNameDBAsync(string name)
        {
            using (new Performance(_performanceThreshold, "{0} GetCMSTemplateDBByNameDBAsync execute", $"{this.GetType().Name}"))
            {
                var template = await _cmsTemplateDBRepository.FindOneAsync("Name", name);
                return template?.MappingTo<CMSTemplateDTO>();
            }
        }

        /// <summary>
        /// 增加CMS模板
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> AddCMSTemplatesDBAsync(AddCMSTemplatesDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} AddCMSTemplatesDBAsync execute", $"{this.GetType().Name}"))
            {
                await _cmsTemplateDBRepository.InsertOneAsync(request.Data.MappingTo<CMSTemplateDO>(), request.PlatformDatas, request.ExtensionData);
                return true;
            }
        }

        /// <summary>
        /// 更新CMS模板
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> UpdateCMSTemplatesDBAsync(UpdateCMSTemplatesDBRequest request)
        {
            if (string.IsNullOrWhiteSpace(request.Data.CMSTemplateCode))
            {
                return false;
            }
            using (new Performance(_performanceThreshold, "{0} UpdateCMSTemplatesDBAsync execute", $"{this.GetType().Name}"))
            {
                var template = await _cmsTemplateDBRepository.FindOneAsync("Code", request.Data.CMSTemplateCode);
                if (template == null)
                {
                    return false;
                }
                template.CMSTemplateName = request.Data.CMSTemplateName;
                template.CMSTemplateStateType = request.Data.CMSTemplateStateType;
                template.CMSTemplateJson = request.Data.CMSTemplateJson;
                template.IsDefault = request.Data.IsDefault;
                await _cmsTemplateDBRepository.UpdateOneAsync("Code", template.Code, template, string.Empty);
                return true;
            }
        }

        /// <summary>
        /// 更新CMS模板
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> UpdateCMSTemplatesBgDBAsync(UpdateCMSTemplatesDBRequest request)
        {
            if (string.IsNullOrWhiteSpace(request.Data.CMSTemplateCode))
            {
                return false;
            }
            using (new Performance(_performanceThreshold, "{0} UpdateCMSTemplatesBgDBAsync execute", $"{this.GetType().Name}"))
            {
                var template = await _cmsTemplateDBRepository.FindOneAsync("Code", request.Data.CMSTemplateCode);
                if (template == null)
                {
                    return false;
                }
                template.CMSBgColor = request.Data.CMSBgColor;
                template.CMSBgImg = request.Data.CMSBgImg;
                template.IsShowTop = request.Data.IsShowTop;
                template.CMSBorderColor = request.Data.CMSBorderColor;
                template.CMSBorderShadow = request.Data.CMSBorderShadow;
                await _cmsTemplateDBRepository.UpdateOneAsync("Code", template.Code, template, string.Empty);
                return true;
            }
        }

        /// <summary>
        /// 删除CMS模板请求
        /// </summary>
        /// <returns></returns>
        public async Task<bool> DeleteCMSTemplatesDBAsync(string code)
        {
            using (new Performance(_performanceThreshold, "{0} DeleteCMSTemplatesDBAsync execute", $"{this.GetType().Name}"))
            {
                return await _cmsTemplateDBRepository.DeleteOneAsync("Code", code);
            }
        }

        /// <summary>
        /// 通过code 获取CMS模板
        /// </summary>
        /// <returns></returns>
        public async Task<CMSTemplateDTO> GetCMSTemplateDBByCodeAsync(string code)
        {
            using (new Performance(_performanceThreshold, "{0} GetCMSTemplateDBByCodeAsync execute", $"{this.GetType().Name}"))
            {
                var template = await _cmsTemplateDBRepository.FindOneAsync("Code", code);
                return template?.MappingTo<CMSTemplateDTO>();
            }
        }
    }
}