using System.Collections.Concurrent;
using System.Linq;
using System;
using System.Collections.Generic;
using System.Threading.Tasks;
using MongoDB.Driver;
using WingInterfaceLibrary.DTO.Management;
using WingInterfaceLibrary.Interface.DBInterface;
using WingInterfaceLibrary.Request;
using WingInterfaceLibrary.Request.DBRequest;
using WingMongoDBModule.DO;
using WingMongoDBModule.Mongodb.Driver;
using WingServerCommon.Config;
using WingServerCommon.Config.Parameters;
using WingServerCommon.Log;
using WingServerCommon.Mapper;
using WingServerCommon.Service;
using WingServerCommon.Utilities;
using System.Threading;
using WingMongoDBModule.Entity;

namespace WingMongoDBModule.Service
{
    public partial class DatabaseService : JsonRpcService, IDBLogService
    {
        private ConcurrentDictionary<string, int>  _tokenSourceDic = new ConcurrentDictionary<string, int>();
        private static string _logDBName = ConfigurationManager.GetParammeter<StringParameter>("Log", "DBName").Value;

        private static string _collectionName = ConfigurationManager.GetParammeter<StringParameter>("Log", "CollectionName").Value;

        /// <summary>
        /// 分页查询
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageCollection<DBLog>> FindDBLogPagesAsync(GetDBLogPagesRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} FindDBLogPagesAsync execute", $"{this.GetType().Name}"))
            {
                var db = MongoRepertoryCenter.Instance.GetDatabase(_logDBName);
                var collection = db.GetCollection<LogEntry>(_collectionName);
                var sort = Builders<LogEntry>.Sort.Descending("Timestamp");
                var filter = GetFilter(request);
                var count = await collection.CountAsync(filter);//CountDocuments很慢
                var cursor = await collection.FindAsync(filter, new FindOptions<LogEntry> { NoCursorTimeout = true, Skip = (request.CurrentPage - 1) * request.PageSize, Limit = request.PageSize, Sort = sort });
                var pageCollection = new PageCollection<DBLog>();
                pageCollection.CurrentPage = request.CurrentPage;
                pageCollection.PageSize = request.PageSize;
                pageCollection.DataCount = (int)count;
                var logList = await cursor.ToListAsync();
                pageCollection.PageData = logList.MappingTo<List<DBLog>>();
                return pageCollection;
            }
        }

        /// <summary>
        /// 查询待删除日志数量
        /// </summary>
        /// <param name="request"></param>
        /// <returns>是否删除成功</returns>
        public async Task<WaitDeleteLogsDTO> QueryWaitDeleteLogsCountDBAsync(QueryWaitDeleteLogsRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} QueryWaitDeleteLogsCountDBAsync execute", $"{this.GetType().Name}"))
            {
                var result = new WaitDeleteLogsDTO()
                {
                    WaitDeleteLogs = new List<WaitDeleteLogBaseDTO>() 
                    {
                        new WaitDeleteLogBaseDTO() {
                            DaysAgo = 1,
                            Percentage = 0
                        },
                        new WaitDeleteLogBaseDTO() {
                            DaysAgo = 7,
                            Percentage = 0
                        },
                        new WaitDeleteLogBaseDTO() {
                            DaysAgo = 30,
                            Percentage = 0
                        },
                        new WaitDeleteLogBaseDTO() {
                            DaysAgo = 90,
                            Percentage = 0
                        },
                        new WaitDeleteLogBaseDTO() {
                            DaysAgo = 180,
                            Percentage = 0
                        },
                        new WaitDeleteLogBaseDTO() {
                            DaysAgo = 365,
                            Percentage = 0
                        },
                    }
                };
                var adminDO = await _adminDBRepository.FindOneAsync("Code", request.AdminCode);
                if (!string.IsNullOrEmpty(adminDO?.AdminCode))
                {
                    if (adminDO.ClearLogOperateList?.Exists(c => c.ServerHost == request.ServerHost) == true)
                    {
                        //存在
                        var findEntity = adminDO.ClearLogOperateList.FirstOrDefault(c => c.ServerHost == request.ServerHost) ?? new ClearLogOperateDO();
                        result.DaysAgo = CleartTimeTransferDays(findEntity.LastClearTime);
                        result.RequestServerHost = request.ServerHost;
                        result.LastClearTime = findEntity.LastClearTime;
                        result.LastClearOperateTime = findEntity.LastClearOperateTime;
                    }
                }
                if (!string.IsNullOrWhiteSpace(request.ServerHost))
                {
                    var key = request.ServerHost;
                    if (_tokenSourceDic.ContainsKey(key))
                    {
                        //表示正在删除
                        var percentage = _tokenSourceDic[key];
                        if (percentage < 5)
                        {
                            percentage = 5;
                        }
                        result.IsLockClear = true;
                        foreach(var item in result.WaitDeleteLogs)
                        {
                            if (item.DaysAgo == result.DaysAgo)
                            {
                                item.Percentage = percentage;
                            }
                        }
                    }
                }
                return result;
            }
        }

        /// <summary>
        /// 清理时时间转换为天数
        /// </summary>
        /// <param name="clearTime">清理时间</param>
        /// <returns></returns>
        private int CleartTimeTransferDays(DateTime clearTime)
        {
            var days = 0;
            TimeSpan daysSpan = new TimeSpan(DateTime.UtcNow.Ticks - clearTime.Ticks);
            var factDays = daysSpan.Days;
            //  1,7,30,90,180,365,730
            if (factDays >= 365)
            {
                days = 365;
            }
            else if (factDays >= 180)
            {
                days = 180;
            }
            else if (factDays >= 90)
            {
                days = 90;
            }
            else if (factDays >= 30)
            {
                days = 30;
            }
            else if (factDays >= 7)
            {
                days = 7;
            }
            else
            {
                days = 1;
            }
            return days;
        }


        /// <summary>
        /// 查询删除了多少
        /// </summary>
        /// <param name="request"></param>
        /// <returns>是否删除成功</returns>
        public async Task<long> QueryDeletedLogsCountDBAsync(DeleteDBLogsRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} QueryDeletedLogsCountDBAsync execute", $"{this.GetType().Name}"))
            {
                try
                {
                    var percentage = 0;
                    if (!string.IsNullOrWhiteSpace(request.ServerHost))
                    {
                        var key = request.ServerHost;
                        if (_tokenSourceDic.ContainsKey(key))
                        {
                            //表示要删除
                            percentage = _tokenSourceDic[key];
                        }
                        else
                        {
                            percentage = 100;
                        }
                    }
                    return percentage;
                    // var db = MongoRepertoryCenter.Instance.GetDatabase(_logDBName);
                    // var collection = db.GetCollection<LogEntry>(_collectionName);
                    // var dataRows = await collection.CountDocumentsAsync(f => f.Timestamp < DateTime.UtcNow.AddDays(-request.DaysAgo));
                    // return dataRows;
                }
                catch (Exception ex)
                {
                    Logger.WriteLineError($"IDBLogService QueryDeletedLogsCountDBAsync, DaysAgo: {request.DaysAgo}, err: {ex}");
                    return 0;
                }
            }
        }

        /// <summary>
        /// 取消删除日志
        /// </summary>
        /// <param name="request"></param>
        /// <returns>是否删除成功</returns>
        public async Task<bool> CancelDeleteLogsDBAsync(BatchDeleteDBLogsRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} CancelDeleteLogsDBAsync execute", $"{this.GetType().Name}"))
            {
                try
                {
                    var key = request.ServerHost;
                    if (_tokenSourceDic.ContainsKey(key))
                    {
                        //表示要删除
                        _tokenSourceDic.TryRemove(key, out _);
                    }
                    return true;
                }
                catch (Exception ex)
                {
                    Logger.WriteLineError($"IDBLogService CancelDeleteLogsDBAsync, err: {ex}");
                    return false;
                }
            }
        }

        /// <summary>
        /// 删除日志数据，适合大批量
        /// </summary>
        /// <param name="request"></param>
        /// <returns>是否删除成功</returns>
        public async Task<bool> DeleteManyLogsDBAsync(BatchDeleteDBLogsRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} DeleteManyLogsDBAsync execute", $"{this.GetType().Name}"))
            {
                try
                {
                    var res = true;
                    var beginOperateTime = DateTime.UtcNow;
                    var db = MongoRepertoryCenter.Instance.GetDatabase(_logDBName);
                    var collection = db.GetCollection<LogEntry>(_collectionName);
                    long batchNum = 5000;//每次批量处理5000个
                    var dataRows = await collection.CountDocumentsAsync(f => f.Timestamp < DateTime.UtcNow.AddDays(-request.DaysAgo));                  
                    //异步，循环处理{
                    var key = request.ServerHost;
                    if (!_tokenSourceDic.ContainsKey(key))
                    {
                        //表示要删除
                        _tokenSourceDic.TryAdd(key, 5);
                    }
                    BatchDeleteLogDatas(dataRows, batchNum, collection, request, beginOperateTime);    
                    return res;
                }
                catch (Exception ex)
                {
                    Logger.WriteLineError($"IDBLogService DeleteManyLogsDBAsync, DaysAgo: {request.DaysAgo}, err: {ex}");
                    return false;
                }
            }
        }

        /// <summary>
        /// 记录操作日志
        /// </summary>
        /// <param name="request">删除操作类</param>
        /// <param name="beginOperateTime">开始操作时间</param>   
        /// <returns></returns>
        private async Task<bool> UpdateAdminClearTime(BatchDeleteDBLogsRequest request, DateTime beginOperateTime)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateAdminClearTime execute", $"{this.GetType().Name}"))
            {
                var res = false;
                MongoCustomerFilter<MongoTable> customerFilter = new();
                MongoCustomerFilter<MongoDataItem> queryItemsFilter = new();
                customerFilter.AppendArrayEq("BaseData.RoleCodes", "AdminRoleDO01");
                var adminList = await _adminDBRepository.FindAllAsync(customerFilter.AndCombine(), new FindOptions<MongoTable> { NoCursorTimeout = true });
                if (adminList?.Count > 0)
                {
                    foreach(var adminDO in adminList) 
                    {
                        if (adminDO.ClearLogOperateList?.Exists(c => c.ServerHost == request.ServerHost) == true)
                        {
                            //存在
                            var findEntity = adminDO.ClearLogOperateList.FirstOrDefault(c => c.ServerHost == request.ServerHost) ?? new ClearLogOperateDO();
                            findEntity.ServerHost = request.ServerHost;
                            findEntity.LastClearTime = DateTime.UtcNow.AddDays(-request.DaysAgo);
                            findEntity.LastClearOperateTime = beginOperateTime;
                        }
                        else
                        {
                            var clearTimeDo = new ClearLogOperateDO()
                            {
                                ServerHost = request.ServerHost,
                                LastClearTime = DateTime.UtcNow.AddDays(-request.DaysAgo),
                                LastClearOperateTime = beginOperateTime
                            };
                            adminDO.ClearLogOperateList.Add(clearTimeDo);
                        }
                        res = await _adminDBRepository.UpdateOneAsync("Code", adminDO.Code, adminDO, string.Empty);
                    }
                }
                return res;
            }
        }

        /// <summary>
        /// 循环删除数据
        /// </summary>
        /// <param name="dataRows">总条数</param>
        /// <param name="batchNum">批量大小</param>   
        /// <param name="collection">表操作对象</param>   
        /// <param name="daysAgo">提前天数</param>           
        /// <returns></returns>
        private async Task BatchDeleteLogDatas(long dataRows, long batchNum, IMongoCollection<LogEntry> collection, BatchDeleteDBLogsRequest request, DateTime beginOperateTime)
        {
            var key = request.ServerHost;
            double logCount = dataRows;
            var index = 0;
            while (dataRows > 0) 
            {
                if (index == 0)
                {
                    await UpdateAdminClearTime(request, beginOperateTime);
                    index++;
                }
                if (dataRows < batchNum) {
                    batchNum = dataRows;
                }
                int limitSize = (int)batchNum;
                MongoCustomerFilter<LogEntry> customerFilter = new();
                customerFilter.AppendLt("Timestamp", DateTime.UtcNow.AddDays(-request.DaysAgo));
                var queryFilter = customerFilter.AndCombine();
                var cursor = await collection.FindAsync(queryFilter, new FindOptions<LogEntry> { NoCursorTimeout = true, Limit = limitSize});
                var deleteIds = new List<MongoDB.Bson.ObjectId>();
                await cursor.ForEachAsync(c => {
                    deleteIds.Add(c.Id);
                });
                MongoCustomerFilter<LogEntry> subFilter = new();
                subFilter.AppendIn("Id", deleteIds);
                var deleteFilter = subFilter.AndCombine();
                DeleteResult deleteResult = await collection.DeleteManyAsync(deleteFilter);
                if (deleteResult.DeletedCount > 0)
                {
                    dataRows = dataRows - batchNum;
                }
                var percentage = 5;
                var value = 100 - Convert.ToInt32(Convert.ToDouble(dataRows) / logCount * 100);
                if (value < 5)
                {
                    percentage = 5;
                }
                else
                {
                    percentage = value;
                }
                if (_tokenSourceDic.ContainsKey(key))
                {
                    _tokenSourceDic[key] = percentage;
                }
                else
                {
                    _tokenSourceDic.TryAdd(key, percentage);
                }
            }
            if (dataRows == 0) 
            {
                await UpdateAdminClearTime(request, beginOperateTime);
            }
            //清除缓存
            if (_tokenSourceDic.ContainsKey(key))
            {
                //表示要删除
                _tokenSourceDic.TryRemove(key, out _);
            }
        }

        /// <summary>
        /// 删除日志数据，适合小批量
        /// </summary>
        /// <param name="request">删除范围</param>
        /// <returns>是否删除成功</returns>
        public async Task<bool> DeleteDBLogsAsync(DeleteDBLogsRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} DeleteDBLogsAsync execute", $"{this.GetType().Name}"))
            {
                var db = MongoRepertoryCenter.Instance.GetDatabase(_logDBName);
                var collection = db.GetCollection<LogEntry>(_collectionName);
                DeleteResult deleteResult = await collection.DeleteManyAsync(f => f.Timestamp < DateTime.UtcNow.AddDays(-request.DaysAgo));
                return deleteResult.DeletedCount > 0;
            }
        }

        /// <summary>
        /// 该函数用于获取查询请求的过滤条件
        /// </summary>
        /// <param name="GetDBLogPagesRequest">请求参数类，用于接收前端发送的请求参数。</param>
        /// <returns>过滤条件</returns>
        private FilterDefinition<LogEntry> GetFilter(GetDBLogPagesRequest queryRequest)
        {
            MongoCustomerFilter<LogEntry> customerFilter = new();

            if (!string.IsNullOrEmpty(queryRequest.Level))
            {
                customerFilter.AppendEq(("Level", queryRequest.Level));
            }
            if (queryRequest.StartTime > new DateTime(1901, 1, 1))
            {
                customerFilter.AppendGte("Timestamp", queryRequest.StartTime.ToUniversalTime());
            }
            if (queryRequest.EndTime > new DateTime(1901, 1, 1))
            {
                customerFilter.AppendLt("Timestamp", queryRequest.EndTime.ToUniversalTime());
            }
            if (!string.IsNullOrWhiteSpace(queryRequest.KeyWord))
            {
                customerFilter.AppendRegexForKey("RenderedMessage", queryRequest.KeyWord, true);
            }
            if (!string.IsNullOrWhiteSpace(queryRequest.Exclude))
            {
                customerFilter.AppendReverseRegexForKey("RenderedMessage", queryRequest.Exclude, true);
            }
            return customerFilter.AndCombine();
        }

    }
}

