using WingServerCommon.Service;
using WingMongoDBModule.DBSync;
using System;
using WingServerCommon.Config;
using System.Collections.Generic;
using System.Threading.Tasks;
using WingServerCommon.Log;
using WingInterfaceLibrary.Request.AIDiagnosis;
using System.Linq;
using WingInterfaceLibrary.Request.Examine;
using WingInterfaceLibrary.Request.Record;
using WingInterfaceLibrary.Enum;
using WingServerCommon.Interfaces.Cache;

namespace WingMongoDBModule.Service
{

    public partial class DatabaseService : JsonRpcService
    {
        /// <summary>
        /// 执行OperationLog后续触发操作
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="opLog"></param>
        private async void OnExecuteOpLogEvent(object sender, OperationLog opLog)
        {
            try
            {
                await ExecuteRemedicalCoverImg(opLog).ConfigureAwait(false);
                await ExecuteSystemConfig(opLog).ConfigureAwait(false);
                await ExecuteDistributedServerConfig(opLog).ConfigureAwait(false);
                await ExecuteRemedicalAIFirst(opLog).ConfigureAwait(false);
                await ExecuteRemedical(opLog).ConfigureAwait(false);
                await ExecuteRemedicalThyroid(opLog).ConfigureAwait(false);
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"DatabaseService.ExecuteOpLog OnExecuteOpLogEvent error, ex:{ex}");
            }
        }

        /// <summary>
        /// token异常兜底机制
        /// </summary>
        /// <param name="code"></param>
        /// <returns></returns>
        private async Task<bool> OnTokenRemove(string code)
        {
            var token = await FindTokenByValueAsync(code);
            var loginSources = GetLoginSource(token.LoginSource);
            var oldDBTokens = await FindTokensByClentIdAsync(token.ClientId);
            oldDBTokens = oldDBTokens.Where(x => loginSources.Contains(x.LoginSource) && x.Code != code)?.ToList();
            foreach (var item in oldDBTokens)
            {
                //不同步oplog
                await _tokensDBRepository.RealDeleteOneAsync("Code", item.Code, false);
                CacheMaintenance.Instance.Get<ITokensManager>().Remove(item.Code);
            }
            return true;
        }

        private List<LoginSource> GetLoginSource(LoginSource loginSource)
        {
            var list = new List<LoginSource>();
            switch (loginSource)
            {
                case LoginSource.PC:
                case LoginSource.Web:
                    list.Add(LoginSource.PC);
                    list.Add(LoginSource.Web);
                    break;
                case LoginSource.Mobile:
                case LoginSource.Pad:
                    list.Add(LoginSource.Mobile);
                    list.Add(LoginSource.Pad);
                    break;
                default:
                    list.Add(loginSource);
                    break;
            }
            return list;
        }


        /// <summary>
        /// 环境变量
        /// </summary>
        /// <param name="opLog"></param>
        /// <returns></returns>
        private async Task ExecuteSystemConfig(OperationLog opLog)
        {
            if (opLog.CollectionName == _SystemConfigDBRepository.CollectionName)
            {
                Logger.WriteLineInfo($"DatabaseService.ExecuteOpLog ExecuteSystemConfig");
                await LoadEnvironmentConfig();
            }
        }

        /// <summary>
        /// 服务器配置
        /// </summary>
        /// <param name="opLog"></param>
        /// <returns></returns>
        private async Task ExecuteDistributedServerConfig(OperationLog opLog)
        {
            if (opLog.CollectionName == _distributedServerInfoDBRepository.CollectionName)
            {
                Logger.WriteLineInfo($"DatabaseService.ExecuteOpLog ExecuteDistributedServerConfig");
                await LoadDistributedServerConfig();
            }
        }

        /// <summary>
        /// 云端图像
        /// </summary>
        /// <param name="opLog"></param>
        /// <returns></returns>
        private async Task ExecuteRemedical(OperationLog opLog)
        {
            if (opLog.CollectionName == _remedicalDBRepository.CollectionName)
            {
                await ExecuteRemedicalSaveCarotidSurfaceImages(opLog);
            }
        }

        /// <summary>
        /// 老平台同步过来的需要插队AI处理的数据
        /// </summary>
        /// <param name="opLog"></param>
        /// <returns></returns>
        private async Task ExecuteRemedicalAIFirst(OperationLog opLog)
        {
            try
            {
                if (opLog.CollectionName == _remedicalDBRepository.CollectionName)
                {
                    var content = opLog.BsonContent.Replace(" ", "");
                    if (content.Contains("DiagnosisConclusion\":1") && content.Contains("DiagnosisResult\":null")
                        && !string.IsNullOrWhiteSpace(opLog.Code)
                        && MatchCurrentStorage(opLog.BsonContent)
                        && content.Contains("OldPlatformNeedAI\":true")
                    )
                    {
                        var remedicalCode = opLog.Code.Split(',')[0];
                        var remedical = await _remedicalDBRepository.FindOneAsync("Code", remedicalCode);
                        var addToRemedicalDiagnosisRequest = new AddToRemedicalDiagnosisRequest { RemedicalCode = remedicalCode };
                        await _remedicalService.AddToRemedicalDiagnosisJumpQueueAsync(addToRemedicalDiagnosisRequest);
                    }
                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"DatabaseService.ExecuteOpLog ExecuteRemedicalAIFirst error, ex:{ex}");
            }
        }

        /// <summary>
        /// 甲状腺
        /// </summary>
        /// <param name="opLog"></param>
        /// <returns></returns>
        private async Task ExecuteRemedicalThyroid(OperationLog opLog)
        {
            try
            {
                if (opLog.CollectionName == _remedicalDBRepository.CollectionName)
                {
                    var content = opLog.BsonContent.Replace(" ", "");
                    if ((opLog.BsonContent.Contains("Thyroid") || opLog.BsonContent.Contains("甲状腺")) && content.Contains("DiagnosisConclusion\":1") && content.Contains("DiagnosisResult\":null")
                        && !string.IsNullOrWhiteSpace(opLog.Code)
                        && MatchCurrentStorage(opLog.BsonContent)
                        && !content.Contains("OldPlatformNeedAI\":true")
                        )
                    {
                        var remedicalCode = opLog.Code.Split(',')[0];
                        var remedical = await _remedicalDBRepository.FindOneAsync("Code", remedicalCode);
                        var addToRemedicalDiagnosisRequest = new AddToRemedicalDiagnosisRequest { RemedicalCode = remedicalCode };
                        await _remedicalService.AddToRemedicalDiagnosisAsync(addToRemedicalDiagnosisRequest);
                    }
                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"DatabaseService.ExecuteOpLog ExecuteRemedicalThyroid error, ex:{ex}");
            }
        }


        /// <summary>
        /// 生成颈动脉3D 6个面图片
        /// </summary>
        /// <param name="opLog"></param>
        /// <returns></returns>
        private async Task ExecuteRemedicalSaveCarotidSurfaceImages(OperationLog opLog)
        {
            try
            {
                // 修改颈动脉结果
                var opLogContent = opLog.BsonContent.Replace(" ", "");
                if (opLogContent.Contains("\"SurfaceFile\":\"http") && opLogContent.Contains("\"SurfaceImageList\":[]")
                    && !string.IsNullOrWhiteSpace(opLog.Code)
                    && MatchCurrentStorage(opLog.BsonContent))
                {
                    var remedicalCode = opLog.Code.Split(',')[0];
                    var remedical = await _remedicalDBRepository.FindOneAsync("Code", remedicalCode);
                    var carotidResult = remedical?.CarotidResult;
                    var surfaceFileUrl = carotidResult?.SurfaceFile;
                    if (carotidResult != null && !string.IsNullOrWhiteSpace(surfaceFileUrl)
                        && MatchCurrentStorage(surfaceFileUrl)
                        && (carotidResult.SurfaceImageList == null || carotidResult.SurfaceImageList.Count != 6))
                    {
                        // 根据颈动脉体数据生成3D 6个面图片
                        Logger.WriteLineInfo($"DatabaseService.ExecuteOpLog SaveCarotidSurfaceImages, start upload surface images, remedical code:{remedicalCode}, surface file:{surfaceFileUrl}");
                        var createCarotidSurfaceImagesResult = await _aiDiagnosisService.CreateCarotidSurfaceImagesAsync(new CreateCarotidSurfaceImagesRequest { SurfaceFileUrl = surfaceFileUrl });
                        var surfaceImageList = createCarotidSurfaceImagesResult?.SurfaceImages;
                        if (surfaceImageList != null && surfaceImageList.Any())
                        {
                            remedical.CarotidResult.SurfaceImageList = surfaceImageList;
                            await _remedicalDBRepository.UpdateOneAsync("Code", remedicalCode, remedical, "");
                            await AddPushRemedicalAsync(remedical);
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"DatabaseService.ExecuteOpLog SaveCarotidSurfaceImages error, ex:{ex}");
            }
        }

        /// <summary>
        /// 老平台同步过来的生成首帧大图
        /// </summary>
        /// <param name="opLog"></param>
        /// <returns></returns>
        private async Task ExecuteRemedicalCoverImg(OperationLog opLog)
        {
            try
            {
                if (opLog.CollectionName == _remedicalDBRepository.CollectionName)
                {
                    var content = opLog.BsonContent.Replace(" ", "");
                    if ((content.Contains("CoverImageUrl\":null") || content.Contains("CoverImageUrl\":\"\"")) && !string.IsNullOrWhiteSpace(opLog.Code)
                    && MatchCurrentStorage(opLog.BsonContent))
                    {
                        var remedicalCode = opLog.Code.Split(',')[0];
                        var remedical = await _remedicalDBRepository.FindOneAsync("Code", remedicalCode);
                        //没有首帧大图的数据
                        //插入生成缩略图队列
                        var addToVidProcessQueueRequest = new AddToVidProcessQueueRequest();
                        addToVidProcessQueueRequest.OriginImageUrl = remedical.TerminalImages.OriginImageUrl;
                        addToVidProcessQueueRequest.RemedicalCode = remedical.RemedicalCode;
                        addToVidProcessQueueRequest.RecordCode = remedical.RecordCode;
                        addToVidProcessQueueRequest.IsNeedCoverImg = true;
                        if (string.IsNullOrEmpty(remedical.TerminalImages.PreviewUrl))
                        {
                            addToVidProcessQueueRequest.IsNeedThumbnailUrl = true;
                        }
                        await _remedicalService.AddToVidProcessQueueAsync(addToVidProcessQueueRequest);
                    }
                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"DatabaseService.ExecuteOpLog ExecuteRemedicalAIFirst error, ex:{ex}");
            }
        }

        /// <summary>
        /// 是否与当前存储匹配
        /// </summary>
        /// <param name="value"></param>
        /// <returns></returns>
        private bool MatchCurrentStorage(string value)
        {
            if (!string.IsNullOrWhiteSpace(value))
            {
                var cfsServerUrl = EnvironmentConfigs.Storage.CFSServerUrl.Replace("http://", "").Replace("https://", "");
                var cdnServerUrl = EnvironmentConfigs.Storage.CDNServerUrl.Replace("http://", "").Replace("https://", "");
                return value.Contains(cfsServerUrl) || value.Contains(cdnServerUrl);
            }
            return false;
        }
    }
}