using WingServerCommon.Service;
using WingServerCommon.Mapper;
using System.Threading.Tasks;
using System.Collections.Generic;
using WingMongoDBModule.DBSync;
using WingMongoDBModule.Mongodb.Driver;
using WingInterfaceLibrary.OpLog;
using WingInterfaceLibrary.Interface.DBInterface;
using WingServerCommon.Utilities;
using WingServerCommon.Config;
using Newtonsoft.Json;
using WingInterfaceLibrary.DB.Request;
using System.Linq;
using System;
using WingServerCommon.Log;

namespace WingMongoDBModule.Service
{
    public partial class DatabaseService : JsonRpcService, IOpLogDBService
    {
        private List<long> _executedOpLogIds = new List<long>();

        /// <summary>副服务器获取主服务器OpLogs同步数据库</summary>
        /// <param name="timeout"></param>
        /// <param name="callback"></param>
        /// <returns></returns>
        private async Task StartSyncDBFromMasterServer(int timeout, Action callback)
        {
            try
            {
                await Task.Delay(timeout);
                callback.Invoke();
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"DatabaseService.OpLog {ConfigurationManager.Host} - StartSyncDBFromMasterServer fail {ex}!");
            }
            await StartSyncDBFromMasterServer(timeout, callback);
        }

        private async void OnSyncDBFromMaster()
        {
            try
            {
                var currentCursor = SlaveOperationLogDbClient.Instance.CurrentMasterCursor;
                _executedOpLogIds.RemoveAll(x => x <= currentCursor);
                var request = new GetOpLogsFormMasterRequest()
                {
                    Cursor = currentCursor,
                    SourceUrl = ConfigurationManager.Host
                };
                var opLogs = await _masterInteractionCenterService.GetOpLogsFromMasterAsync(request);
                long cursor = 0;
                foreach (var logMsg in opLogs)
                {
                    var logId = logMsg.Id;
                    if (_executedOpLogIds.Contains(logId))
                    {
                        continue;
                    }
                    var opLog = new OperationLog();
                    opLog.BsonContent = logMsg.BsonContent;
                    opLog.FilterContent = logMsg.FilterContent;
                    opLog.CollectionName = logMsg.CollectionName;
                    opLog.CreateTime = logMsg.CreateTime;
                    opLog.ActionType = logMsg.ActionType;
                    opLog.Code = logMsg.Code;
                    opLog.IsSimple=logMsg.IsSimple;
                    var isSuccess = await MongoRepertoryCenter.Instance.ReceiveBsonLog(opLog);
                    if (isSuccess && cursor < logId)
                    {
                        _executedOpLogIds.Add(logId);
                        cursor = logId;
                    }
                }
                if (cursor > 0)
                {
                    SlaveOperationLogDbClient.Instance.CurrentMasterCursor = cursor;
                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"{ConfigurationManager.Host} - OnSyncDBFromMaster fail {ex}!");
            }
        }

        /// <summary>插队完成业务数据的同步</summary>
        /// <param name="code">业务编码</param>
        /// <returns></returns>
        public async Task<bool> SyncCompleteOpLogsAsync(SyncCompleteOpLogsRequest request)
        {
            try
            {
                var localHost = ConfigurationManager.Host;
                var opLogs = request.Oplogs;
                if (opLogs.Any(x => x.SourceUrl != localHost))//oplogs只有当前服务器则不处理
                {
                    foreach (var logMsg in opLogs)
                    {
                        var logId = logMsg.Id;
                        if (logId <= SlaveOperationLogDbClient.Instance.CurrentMasterCursor)
                        {
                            continue;
                        }
                        if (_executedOpLogIds.Contains(logId))
                        {
                            continue;
                        }

                        var opLog = new OperationLog();
                        opLog.BsonContent = logMsg.BsonContent;
                        opLog.FilterContent = logMsg.FilterContent;
                        opLog.CollectionName = logMsg.CollectionName;
                        opLog.CreateTime = logMsg.CreateTime;
                        opLog.ActionType = logMsg.ActionType;
                        opLog.Code = logMsg.Code;
                        opLog.IsSimple=logMsg.IsSimple;
                        var isSuccess = await MongoRepertoryCenter.Instance.ReceiveBsonLog(opLog);
                        if (isSuccess)
                        {
                            _executedOpLogIds.Add(logId);
                        }
                    }
                    return true;
                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"DatabaseService.OpLog ComplateOplogsByCode fail, code:{string.Join(",", request.Oplogs)}, err:{ex}");
            }
            return false;
        }

        /// <summary>
        /// > 该函数从从节点接收操作日志并写入主节点
        /// </summary>
        /// <param name="SyncOpLogToMasterRequest"></param>
        /// <returns>
        /// 一个布尔值。
        /// </returns>
        public async Task<bool> SyncOpLogAsync(SyncOpLogToMasterRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} SyncOpLogAsync execute", $"{this.GetType().Name}"))
            {

                var oplog = new OperationLog
                {
                    ActionType = request.ActionType,
                    BsonContent = request.BsonContent,
                    CollectionName = request.CollectionName,
                    CreateTime = request.CreateTime,
                    FilterContent = request.FilterContent,
                    SourceUrl = request.SourceUrl,
                    Code = request.Code,
                    IsSimple=request.IsSimple
                };
                return await MongoRepertoryCenter.Instance.ReceiveBsonLog(oplog);
            }
        }


        /// <summary>
        /// > GetOpLogsFormMasterRequest 是一个请求对象，用于从MongoDB数据库中查询操作日志数据
        /// </summary>
        /// <param name="GetOpLogsFormMasterRequest">request参数是接口的请求参数，用于传递接口的参数。</param>
        /// <returns>
        /// OperationLogDTO 对象的列表。
        /// </returns>
        public async Task<List<OperationLogDTO>> GetOpLogAsync(GetOpLogsFormMasterRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetOpLogAsync execute", $"{this.GetType().Name}"))
            {
                var oplogs = await MongoRepertoryCenter.Instance.GetOpLogsAsync(request);
                return oplogs.MappingTo<List<OperationLogDTO>>();
            }
        }


        /// <summary>
        /// > GetOpLogsByCodesFormMasterRequest 是一个请求对象，用于根据业务编码从MongoDB数据库中查询操作日志数据
        /// </summary>
        /// <param name="request">request参数是接口的请求参数，用于传递接口的参数。</param>
        /// <returns>
        /// OperationLogDTO 对象的列表。
        /// </returns>
        public async Task<List<OperationLogDTO>> GetOpLogsByCodesAsync(GetOpLogsByCodesFormMasterRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetOpLogsByCodesAsync execute", $"{this.GetType().Name}"))
            {
                var oplogs = await MongoRepertoryCenter.Instance.GetOpLogsByCodesAsync(request);
                return oplogs.MappingTo<List<OperationLogDTO>>();
            }
        }

    }
}