using System.Collections.Generic;
using System.Threading.Tasks;
using WingServerCommon.Service;
using System.Linq;
using WingMongoDBModule.DO;
using WingInterfaceLibrary.DB.Request;
using WingMongoDBModule.Mongodb.Driver;
using WingMongoDBModule.Entity;
using WingInterfaceLibrary.Result;
using WingMongoDBModule.Request;
using WingInterfaceLibrary.DTO.DiagnosisModule;
using WingInterfaceLibrary.Interface.DBInterface;
using MongoDB.Driver;
using WingServerCommon.Mapper;
using WingServerCommon.Utilities;
using MongoDB.Driver.GeoJsonObjectModel;

namespace WingMongoDBModule.Service
{
    public partial class DatabaseService : JsonRpcService, IDiagnosisModuleDBService
    {
        /// <summary>
        /// 查询AI应用集合
        /// </summary>
        /// <param name="codes"></param>
        /// <returns></returns>
        public async Task<List<DiagnosisModuleDTO>> FindDiagnosisModuleByCodesAsync(List<string> codes)
        {
            using (new Performance(_performanceThreshold, "{0} FindDiagnosisModuleByCodesAsync execute", $"{this.GetType().Name}"))
            {
                var dic = new Dictionary<string, List<string>>();
                dic.Add("Code", codes);
                var entities = await _diagnosisModuleDBRepository.FindInAsync(dic);
                if (!_isCarotidOn)
                {
                    entities = entities.Where(x => !x.DiagnosisModule.Contains("Carotid")).ToList();
                }
                return entities.MappingTo<List<DiagnosisModuleDTO>>();
            }
        }

        /// <summary>
        /// 查询AI应用集合
        /// </summary>
        /// <param name="diagnosisModules"></param>
        /// <returns></returns>
        public async Task<List<DiagnosisModuleDTO>> FindDiagnosisModuleByModulesAsync(List<string> diagnosisModules)
        {
            using (new Performance(_performanceThreshold, "{0} FindDiagnosisModuleByModulesAsync execute", $"{this.GetType().Name}"))
            {
                var dic = new Dictionary<string, List<string>>();
                dic.Add("DiagnosisModule", diagnosisModules);
                var entities = await _diagnosisModuleDBRepository.FindInAsync(dic);
                if (!_isCarotidOn)
                {
                    entities = entities.Where(x => !x.DiagnosisModule.Contains("Carotid")).ToList();
                }
                return entities.MappingTo<List<DiagnosisModuleDTO>>();
            }
        }

        /// <summary>增量添加AI应用</summary>
        /// <returns></returns>
        public async Task<bool> UpdateDiagnosisModulesAsync(UpdateDiagnosisModulesDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateDiagnosisModulesAsync execute", $"{this.GetType().Name}"))
            {
                var list = await _diagnosisModuleDBRepository.FindAllAsync();
                var insertModules = new List<DiagnosisModuleDO>();
                foreach (var item in request.DiagnosisModules)
                {
                    if (!list.Any(x => x.DiagnosisModule == item))
                    {
                        insertModules.Add(new DiagnosisModuleDO { DiagnosisModule = item });
                    }
                }
                if (insertModules.Any())
                {
                    await _diagnosisModuleDBRepository.InsertAllAsync(insertModules.Select(x => (x, new Dictionary<string, string>(), string.Empty)).ToList());
                }
                return true;
            }
        }

        /// <summary>
        /// 分页查询AI应用
        /// </summary>
        /// <returns></returns>
        public async Task<PageResult<DiagnosisModuleDTO>> GetDiagnosisModuleByPageAsync(GetDiagnosisModuleByPageDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetDiagnosisModuleByPageAsync execute", $"{this.GetType().Name}"))
            {
                MongoCustomerFilter<MongoTable> mongoCustomerFilter = new();

                List<(string, string, bool, bool)> regexFilters = new();
                List<(string, string)> eqFilters = new();
                if (regexFilters.Count > 0)
                {
                    mongoCustomerFilter.AppendQueryItemRegexFilter(MongoCombineTypeEnum.And, regexFilters.ToArray());
                }
                if (eqFilters.Count > 0)
                {
                    mongoCustomerFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, eqFilters.ToArray());
                }
                var filter = mongoCustomerFilter.AndCombine();
                var dbPage = new DBPageRequest();
                dbPage.CurrentPage = request.CurrentPage;
                dbPage.PageSize = request.PageSize;
                dbPage.Filter = filter;
                var pageResult = await _diagnosisModuleDBRepository.GetPages(dbPage);
                if (!_isCarotidOn)
                {
                    if (pageResult.PageData != null && pageResult.PageData.Count > 0)
                    {
                        if (pageResult.PageData.Any(x => x.DiagnosisModule.Contains("Carotid")))
                        {
                            pageResult.PageData = pageResult.PageData.Where(x => !x.DiagnosisModule.Contains("Carotid")).ToList();
                            pageResult.DataCount = pageResult.DataCount - 1;
                        }
                    }
                }
                return PageResultConvert<DiagnosisModuleDO, DiagnosisModuleDTO>(pageResult);
            }
        }

        /// <summary>开启/关闭AI应用</summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> UpdateEnableStateAsync(UpdateDiagnosisModuleEnableStateDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateEnableStateAsync execute", $"{this.GetType().Name}"))
            {
                if (request.DiagnosisModules == null || !request.DiagnosisModules.Any())
                {
                    return false;
                }
                var modules = await FindDiagnosisModuleByModulesAsync(request.DiagnosisModules);
                if (modules?.Any() != true)
                {
                    return false;
                }

                MongoCustomerFilter<MongoDataItem> subFilter = new();
                subFilter.AppendEq((item => item.Key, "Code"));
                subFilter.AppendIn(item => item.Value, modules.Select(x => x.DiagnosisModuleCode).ToList());

                MongoCustomerFilter<MongoTable> filter = new();
                filter.SubFilter<MongoDataItem>(item => item.QueryData.QueryItems, subFilter.AndCombine());

                MongoCustomerUpdate<MongoTable> update = new();
                update.SetField("BaseData.Enabled", request.Enabled.ToString());
                return await _diagnosisModuleDBRepository.UpdateAllAsync(filter.AndCombine(), update.AllUpdateDefinition) > 0;
            }
        }

        /// <summary>
        /// 查询机构AI应用集合
        /// </summary>
        /// <returns></returns>
        public async Task<List<DiagnosisModuleDTO>> GetOrganizationDiagnosisModulesAsync(GetOrganizationDiagnosisModulesDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetOrganizationDiagnosisModulesAsync execute", $"{this.GetType().Name}"))
            {
                if (string.IsNullOrWhiteSpace(request.OrganizationCode))
                {
                    return new List<DiagnosisModuleDTO>();
                }
                var resultData = new List<DiagnosisModuleDTO>();
                var modules = await _diagnosisModuleDBRepository.FindAllAsync();
                modules = modules.Where(x => x.Enabled)?.ToList() ?? new List<DiagnosisModuleDO>();
                var organizationEntity = await _organizationDBRepository.FindOneAsync("Code", request.OrganizationCode);
                foreach (var module in modules)
                {
                    var dto = new DiagnosisModuleDTO { DiagnosisModule = module.DiagnosisModule, DiagnosisModuleCode = module.DiagnosisModuleCode };
                    if (module.Enabled && organizationEntity?.DiagnosisModules?.Contains(module.DiagnosisModuleCode) == true)
                    {
                        dto.Enabled = true;
                    }
                    resultData.Add(dto);
                }
                if (!_isCarotidOn)
                {
                    resultData = resultData.Where(x => !x.DiagnosisModule.Contains("Carotid")).ToList();
                }
                return resultData;
            }
        }

        /// <summary>保存机构AI应用集合</summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> UpdateOrganizationDiagnosisModulesAsync(UpdateOrganizationDiagnosisModulesDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateOrganizationDiagnosisModulesAsync execute", $"{this.GetType().Name}"))
            {
                var organizationEntity = await _organizationDBRepository.FindOneAsync("Code", request.OrganizationCode);
                if (organizationEntity != null && !string.IsNullOrWhiteSpace(organizationEntity.Code))
                {
                    var newModules = await FindDiagnosisModuleByModulesAsync(request.DiagnosisModules);
                    var newModuleCodes = newModules?.Select(x => x.DiagnosisModuleCode) ?? new List<string>();
                    var oldModuleCodes = organizationEntity.DiagnosisModules ?? new List<string>();
                    var insertModuleCodes = newModuleCodes.Except(oldModuleCodes).ToList();

                    organizationEntity.DiagnosisModules = newModuleCodes.ToList();
                    var res = await _organizationDBRepository.UpdateOneAsync("Code", organizationEntity.Code, organizationEntity, string.Empty);
                    if (res)
                    {
                        await AddDiagnosisModuleByOrganization(organizationEntity.Code, insertModuleCodes);
                        return true;
                    }
                }
                return false;
            }
        }

        private async Task AddDiagnosisModuleByOrganization(string organizationCode, List<string> insertModules)
        {
            if (!insertModules.Any())
            {
                return;
            }
            var dic = new Dictionary<string, string>();
            dic.Add("OrganizationCode", organizationCode);
            var deviceList = await _deviceInfoDBRepository.FindListAsync(dic);

            var deviceCodes = deviceList.Select(x => x.Code).ToList();
            if (deviceCodes.Any())
            {
                MongoCustomerFilter<MongoDataItem> subFilter = new();
                subFilter.AppendEq((item => item.Key, "Code"));
                subFilter.AppendIn(item => item.Value, deviceCodes);

                MongoCustomerFilter<MongoTable> filter = new();
                filter.SubFilter<MongoDataItem>(item => item.QueryData.QueryItems, subFilter.AndCombine());

                MongoCustomerUpdate<MongoTable> update = new();
                update.AddArrayToSubItem("BaseData.DiagnosisModules", insertModules);
                await _deviceInfoDBRepository.UpdateAllAsync(filter.AndCombine(), update.AllUpdateDefinition);
            }
        }

        /// <summary>
        /// 查询设备AI应用集合
        /// </summary>
        /// <returns></returns>
        public async Task<List<DiagnosisModuleDTO>> GetDeviceDiagnosisModulesAsync(GetDeviceDiagnosisModulesDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetDeviceDiagnosisModulesAsync execute", $"{this.GetType().Name}"))
            {
                var resultData = new List<DiagnosisModuleDTO>();
                var deviceEntity = await _deviceInfoDBRepository.FindOneAsync("Code", request.DeviceCode);
                if (deviceEntity == null || string.IsNullOrWhiteSpace(deviceEntity.Code) || string.IsNullOrWhiteSpace(deviceEntity.OrganizationCode))
                {
                    return resultData;
                }
                var modules = await GetOrganizationDiagnosisModulesAsync(new GetOrganizationDiagnosisModulesDBRequest { OrganizationCode = deviceEntity.OrganizationCode });
                foreach (var module in modules)
                {
                    if (module.Enabled)
                    {
                        var dto = new DiagnosisModuleDTO { DiagnosisModule = module.DiagnosisModule, DiagnosisModuleCode = module.DiagnosisModuleCode };
                        if (deviceEntity?.DiagnosisModules?.Contains(module.DiagnosisModuleCode) == true)
                        {
                            dto.Enabled = true;
                        }
                        resultData.Add(dto);
                    }
                }
                if (!_isCarotidOn)
                {
                    resultData = resultData.Where(x => !x.DiagnosisModule.Contains("Carotid")).ToList();
                }
                return resultData;
            }
        }

        /// <summary>设置设备AI应用启用状态</summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> UpdateDeviceDiagnosisModuleStateAsync(UpdateDeviceDiagnosisModuleStateDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateDeviceDiagnosisModuleStateAsync execute", $"{this.GetType().Name}"))
            {
                var requestModules = await FindDiagnosisModuleByModulesAsync(new List<string> { request.DiagnosisModule });
                var requestModule = requestModules.FirstOrDefault();
                var deviceEntity = await _deviceInfoDBRepository.FindOneAsync("Code", request.DeviceCode);
                if (requestModule != null && deviceEntity != null && !string.IsNullOrWhiteSpace(deviceEntity.Code))
                {
                    var diagnosisModule = deviceEntity.DiagnosisModules ?? new List<string>();
                    if (request.Enabled)
                    {
                        if (diagnosisModule.Contains(requestModule.DiagnosisModuleCode))
                        {
                            return true;
                        }
                        diagnosisModule.Add(requestModule.DiagnosisModuleCode);
                    }
                    else
                    {
                        if (!diagnosisModule.Contains(requestModule.DiagnosisModuleCode))
                        {
                            return true;
                        }
                        diagnosisModule.RemoveAll(x => x == requestModule.DiagnosisModuleCode);
                    }
                    deviceEntity.DiagnosisModules = diagnosisModule;
                    return await _deviceInfoDBRepository.UpdateOneAsync("Code", deviceEntity.Code, deviceEntity, string.Empty);
                }
                return false;
            }
        }

        /// <summary>设置设备已启用AI应用集合</summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> UpdateDeviceDiagnosisModulesDBAsync(UpdateDeviceDiagnosisModulesDBRequest request)
        {
            var deviceCode = request.DeviceCode;
            var requestModules = request.DiagnosisModules;
            using (new Performance(_performanceThreshold, "{0} UpdateDeviceDiagnosisModulesAsync execute", $"{this.GetType().Name}"))
            {
                var enabledModuleCodes = new List<string>();
                if (requestModules != null && requestModules.Any())
                {
                    var modules = await FindDiagnosisModuleByModulesAsync(requestModules);
                    enabledModuleCodes = modules?.Select(x => x.DiagnosisModuleCode)?.ToList() ?? new List<string>();
                }
                var deviceEntity = await _deviceInfoDBRepository.FindOneAsync("Code", deviceCode);
                if (deviceEntity != null && !string.IsNullOrWhiteSpace(deviceEntity.Code))
                {
                    deviceEntity.DiagnosisModules = enabledModuleCodes;
                    return await _deviceInfoDBRepository.UpdateOneAsync("Code", deviceEntity.Code, deviceEntity, string.Empty);
                }
                return false;
            }
        }
    }
}