using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Threading.Tasks;
using MongoDB.Bson;
using MongoDB.Driver;
using WingInterfaceLibrary.DB.Request;
using WingInterfaceLibrary.DTO.Vital;
using WingInterfaceLibrary.Enum.VitalEnum;
using WingInterfaceLibrary.Interface.DBVitalInterface;
using WingInterfaceLibrary.Request;
using WingInterfaceLibrary.Request.DBVitalRequest;
using WingInterfaceLibrary.Result;
using WingMongoDBModule.Entities;
using WingMongoDBModule.Mongodb.Driver;
using WingMongoDBModule.Request;
using WingServerCommon.Log;
using WingServerCommon.Mapper;
using WingServerCommon.Service;

namespace WingMongoDBModule.Service
{
    public partial class DatabaseService : JsonRpcService, IVitalElectrocardiogramDBService
    {
        public async Task<string> CreateElectrocardiogramDBAsync(CreateElectrocardiogramDBRequest request)
        {
            var entity = request.MappingTo<ElectrocardiogramEntity>();
            var result = await _electrocardiogramDBRepository.InsertOneAsync(entity);
            return result;
        }

        public async Task<ElectrocardiogramRecord> GetElectrocardiogramDBByPhysicalExamNumberDBAsync(string physicalExamNumber)
        {
            var filter = Builders<ElectrocardiogramEntity>.Filter.Eq(x => x.PhysicalExamNumber, physicalExamNumber);
            var entity = await _electrocardiogramDBRepository.FindOneAsync(filter);
            var result = entity.MappingTo<ElectrocardiogramRecord>();
            return result;
        }

        public async Task<PageCollection<ElectrocardiogramRecord>> GetElectrocardiogramDBPageAsync(GetElectrocardiogramDBPageRequest request)
        {
            var keyword = request.Keyword;
            var builder = Builders<ElectrocardiogramEntity>.Filter;
            var filter = builder.Where(c => c.IsDelete == false);
            var orFilter = builder.Where(c => c.OrganizationCode == request.OrgCode);
            if (!string.IsNullOrEmpty(keyword))
            {
                keyword = EscapeExprSpecialWord(keyword);//字符串转义
                if (!string.IsNullOrEmpty(keyword))
                {
                    keyword = EscapeExprSpecialWord(keyword);//字符串转义
                    var nameFilter = builder.Regex(c => c.PatientName, new BsonRegularExpression(keyword, "i"));
                    var IDCardNoFilter = builder.Regex(c => c.CardNo, new BsonRegularExpression(keyword, "i"));
                    var physicalExamNumberFilter = builder.Regex(c => c.PhysicalExamNumber, new BsonRegularExpression(keyword, "i"));
                    var phoneFilter = builder.Regex(c => c.Phone, new BsonRegularExpression(keyword, "i"));
                    filter = filter & (nameFilter | IDCardNoFilter | physicalExamNumberFilter | phoneFilter);
                }
            }
            if (request.Filter.Any(x => x.Key == "UserEcgCode"))
            {
                var userEcgCode = request.Filter.FirstOrDefault(x => x.Key == "UserEcgCode").Value;
                if (!string.IsNullOrWhiteSpace(userEcgCode))
                {
                    orFilter = orFilter | builder.Where(x => x.DeviceCode == userEcgCode);
                }
            }
            //下级机构提交的远程心电记录
            if (request.IsQuerySubordinate && request.ChildrenOrgCodes != null && request.ChildrenOrgCodes.Count > 0)
            {
                var orgFilter = builder.In(c => c.OrganizationCode, request.ChildrenOrgCodes);
                orFilter = orFilter | orgFilter;
            }
            var startDate = request.StartTime ?? new DateTime(1900, 1, 1);
            var endDate = request.EndTime ?? new DateTime(1900, 1, 1);
            if (startDate > Convert.ToDateTime("1900-2-1"))
            {
                filter = filter & builder.Where(c => c.CreateTime >= startDate);
            }
            if (endDate > Convert.ToDateTime("1900-2-1"))
            {
                filter = filter & builder.Where(c => c.CreateTime < endDate.AddDays(1));
            }
            if (request.States != null && request.States.Count > 0)
            {
                filter = filter & builder.In(x => x.ExamState, request.States);
            }
            var dbPage = new DBPageRequest<ElectrocardiogramEntity>();
            dbPage.CurrentPage = request.PageIndex;
            dbPage.PageSize = request.PageSize;
            dbPage.Filter = filter & orFilter;
            var entitys = await _electrocardiogramDBRepository.GetPages(dbPage);
            var result = entitys.MappingTo<PageCollection<ElectrocardiogramRecord>>();
            return result;
        }

        public async Task<ElectrocardiogramRecord> GetExamElectrocardiogramDBAsync(string code)
        {
            var filter = Builders<ElectrocardiogramEntity>.Filter.Eq(x => x.Code, code);
            var entity = await _electrocardiogramDBRepository.FindOneAsync(filter);
            var result = entity.MappingTo<ElectrocardiogramRecord>();
            return result;
        }


        public async Task<bool> UpdateExamElectrocardiogramDBAsync(ElectrocardiogramRecord record)
        {
            var consultationRecordEntity = record.MappingTo<ElectrocardiogramEntity>();
            consultationRecordEntity.UpdateTime = DateTime.UtcNow;
            var result = await _electrocardiogramDBRepository.UpdateOneAsync("Code", record.Code, consultationRecordEntity);
            return result;
        }


        public async Task<bool> RemoveElectrocardiogramDBAsync(string code)
        {
            var result = await _electrocardiogramDBRepository.DeleteOneAsync("Code", code);
            return result;
        }

        /// <summary>
        /// 分页查询增量心电记录集合
        /// </summary>
        /// <returns></returns>
        public async Task<PageResult<ElectrocardiogramRecord>> FindElectrocardiogramRecordByUpdateTimeDBAsync(FindCollectionByTimeDBRequest request)
        {
            var startTime = request.StartTime;
            var pageIndex = request.PageIndex;
            var pageSize = request.PageSize;
            var builder = Builders<ElectrocardiogramEntity>.Filter;
            var filter = builder.Where(c => c.IsDelete == false);
            filter = filter & builder.Where(c => c.UpdateTime > request.StartTime);
            var sort = Builders<ElectrocardiogramEntity>.Sort.Ascending(v => v.UpdateTime);
            var dbPage = new DBPageRequest<ElectrocardiogramEntity>();
            dbPage.CurrentPage = request.PageIndex;
            dbPage.PageSize = request.PageSize;
            dbPage.Filter = filter;
            dbPage.Sort = sort;
            var pageResult = await _electrocardiogramDBRepository.GetPages(dbPage, onlyNotDeleted: false);
            var result = new PageResult<ElectrocardiogramRecord>();
            result.PageIndex = pageResult.CurrentPage;
            result.PageSize = pageResult.PageSize;
            result.TotalCount = pageResult.DataCount;
            result.PageData = pageResult.PageData?.MappingTo<List<ElectrocardiogramRecord>>() ?? new List<ElectrocardiogramRecord>();
            return result;
        }

        /// <summary>
        /// 保存心电记录
        /// </summary>
        /// <returns></returns>
        public async Task<bool> SaveElectrocardiogramRecordDBAsync(ElectrocardiogramRecord record)
        {
            var code = record.Code;
            var recordEntity = record.MappingTo<ElectrocardiogramEntity>();
            if (string.IsNullOrWhiteSpace(code))
            {
                await _electrocardiogramDBRepository.InsertOneAsync(recordEntity);
                return true;
            }
            var entity = await _electrocardiogramDBRepository.FindOneAsync("Code", code);
            if (entity != null && !string.IsNullOrWhiteSpace(entity.Code))
            {
                await _electrocardiogramDBRepository.UpdateOneAsync("Code", code, recordEntity);
            }
            else
            {
                await _electrocardiogramDBRepository.InsertOneAsync(recordEntity);
            }
            return true;
        }
    }
}