using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using WingInterfaceLibrary.DB.Request;
using WingInterfaceLibrary.DTO.FaceRecognition;
using WingInterfaceLibrary.DTO.Vital;
using WingInterfaceLibrary.Request;
using WingInterfaceLibrary.Interface.DBInterface;
using WingServerCommon.Mapper;
using WingServerCommon.Service;
using MongoDB.Bson;
using MongoDB.Driver;
using WingMongoDBModule.Entities;
using WingMongoDBModule.Request;

namespace WingMongoDBModule.Service
{
    public partial class DatabaseService : JsonRpcService, IFaceRecognitionDBService
    {
        /// <summary>
        /// 添加人脸识别数据
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<string> AddFaceRecognitionAsync(AddFaceRecognitionRequest request)
        {
            var faceRecognitionEntity = new FaceRecognitionEntity
            {
                PersonId = request.PersonId,
                Data = request.Data,
                IdRegModelVersion = request.IdRegModelVersion,
                ImagePath = request.ImagePath
            };
            var personFace = await _faceRecognitionDBRepository.FindOneAsync("PersonId", request.PersonId, false);
            if(personFace!=null)
            {
                var filter = Builders<FaceRecognitionEntity>.Filter.Eq(f => f.Code, personFace.Code);
                var update = Builders<FaceRecognitionEntity>.Update
                    .Set(f => f.IsDelete, false)
                    .Set(f => f.Data, personFace.Data)
                    .Set(f => f.Size, personFace.Size)
                    .Set(f => f.IdRegModelVersion, personFace.IdRegModelVersion)
                    .Set(f => f.ImagePath, personFace.ImagePath);    
                await _faceRecognitionDBRepository.UpdateOneAsync(filter, update);
                return personFace.Code;
            }
            else
            {
                return await _faceRecognitionDBRepository.InsertOneAsync(faceRecognitionEntity);
            }
        }

        /// <summary>
        /// 根据code查询人脸识别数据
        /// </summary>
        /// <param name="code"></param>
        /// <returns></returns>
        public async Task<FaceRecognitionDTO> FindFaceRecognitionByCodeAsync(string code)
        {
            var filter = Builders<FaceRecognitionEntity>.Filter.Eq(f => f.Code, code);
            var entity = await _faceRecognitionDBRepository.FindOneAsync(filter);
            if (entity != null && !string.IsNullOrWhiteSpace(entity.Code))
            {
                var dto = new FaceRecognitionDTO
                {
                    Code = entity.Code,
                    Data = entity.Data,
                    IdRegModelVersion = entity.IdRegModelVersion,
                    ImagePath = entity.ImagePath,
                    PersonId = entity.PersonId,
                    Size = entity.Size
                };
                return dto;
            }
            return null;
        }

        /// <summary>
        /// 修改人脸识别数据
        /// </summary>
        /// <param name="personFullInfo"></param>
        /// <returns></returns>
        public async Task<bool> UpdateFaceRecognitionAsync(UpdateFaceRecognitionRequest faceRecognition)
        {
            var filter = Builders<FaceRecognitionEntity>.Filter.Eq(f => f.Code, faceRecognition.Code);
            var update = Builders<FaceRecognitionEntity>.Update
                .Set(f => f.PersonId, faceRecognition.PersonId)
                .Set(f => f.Data, faceRecognition.Data)
                .Set(f => f.Size, faceRecognition.Size)
                .Set(f => f.IdRegModelVersion, faceRecognition.IdRegModelVersion)
                .Set(f => f.ImagePath, faceRecognition.ImagePath);    
            var result = await _faceRecognitionDBRepository.UpdateOneAsync(filter, update) > 0;
            return result;
        }

        /// <summary>
        /// 获取人脸识别数据列表分页
        /// </summary>
        /// <param name="request">分页请求</param>
        /// <returns>返回人脸识别数据列表</returns>
        public async Task<PageCollection<FaceRecognitionDTO>> GetFaceRecognitionPageAsync(GetFaceRecognitionPageRequest request)
        {
            var builder = Builders<FaceRecognitionEntity>.Filter;
            var filter = builder.Where(c => c.IsDelete == false && c.UpdateTime >= request.StartTime);
            var pageFilter = new DBPageRequest<FaceRecognitionEntity>
            {
                CurrentPage = request.PageIndex,
                PageSize = request.PageSize,
                Filter = filter,
                Sort = Builders<FaceRecognitionEntity>.Sort.Descending(x => x.UpdateTime)
            };
            var result = await _faceRecognitionDBRepository.GetPages(pageFilter);
            var newPage = new PageCollection<FaceRecognitionDTO>
            {
                CurrentPage = result.CurrentPage,
                PageSize = result.PageSize,
                DataCount = result.DataCount,
                PageData = result.PageData.Select(f => new FaceRecognitionDTO
                {
                    Code = f.Code,
                    Data = f.Data,
                    IdRegModelVersion = f.IdRegModelVersion,
                    ImagePath = f.ImagePath,
                    PersonId = f.PersonId,
                    Size = f.Size
                }).ToList()
            };
            return newPage;
        }

        /// <summary>
        /// 根据code删除人脸识别数据
        /// </summary>
        /// <param name="code"></param>
        /// <returns></returns>
        public async Task<bool> DeleteFaceRecognitionAsync(string code)
        {
            var result = await _faceRecognitionDBRepository.DeleteOneAsync("Code", code);
            return result;
        }

        /// <summary>
        /// 根据personId删除人脸识别数据
        /// </summary>
        /// <param name="code"></param>
        /// <returns></returns>
        public async Task<bool> DeleteFaceRecognitionByPersonIdAsync(string personId)
        {
            var result = await _faceRecognitionDBRepository.DeleteOneAsync("PersonId", personId);
            return result;
        }
    }
}