using System;
using System.Collections.Generic;
using System.Threading.Tasks;
using JsonRpcLite.Services;
using WingMongoDBModule.DO;
using WingInterfaceLibrary.DTO.IdentityApply;
using WingServerCommon.Service;
using WingInterfaceLibrary.Interface.DBInterface;
using WingInterfaceLibrary.DB.Request;
using WingServerCommon.Mapper;
using WingInterfaceLibrary.Request;
using WingServerCommon.Utilities;

namespace WingMongoDBModule.Service
{
    public partial class DatabaseService : JsonRpcService, IIdentityApplyDBService
    {

        /// <summary>
        /// 删除身份申请
        /// </summary>
        /// <param name="id">Id</param>
        /// <returns>是否成功</returns>
        public async Task<bool> DeleteIdentityApplyByCodeAsync(string code)
        {
            using (new Performance(_performanceThreshold, "{0} DeleteIdentityApplyByCodeAsync execute", $"{this.GetType().Name}"))
            {
                return await _identityApplyDBRepository.DeleteOneAsync("Code", code);
            }
        }

        /// <summary>
        /// 通过编码查找身份申请
        /// </summary>
        /// <param name="code">申请编码</param>
        /// <returns>申请信息</returns>
        public async Task<IdentityApplyDTO> FindIdentityApplyByCodeAsync(string code)
        {
            using (new Performance(_performanceThreshold, "{0} FindIdentityApplyByCodeAsync execute", $"{this.GetType().Name}"))
            {
                var identityApplyDTO = await _identityApplyDBRepository.FindOneAsync("Code", code);
                return identityApplyDTO.MappingTo<IdentityApplyDTO>();
            }
        }

        /// <summary>
        /// 通过用户名查找申请信息
        /// </summary>
        /// <param name="name">用户名</param>
        /// <returns>申请信息</returns>
        public async Task<IdentityApplyDTO> FindIdentityApplyByNameAsync(string name)
        {
            using (new Performance(_performanceThreshold, "{0} FindIdentityApplyByNameAsync execute", $"{this.GetType().Name}"))
            {
                var identityApplyDTO = await _identityApplyDBRepository.FindOneAsync("RealName", name);
                return identityApplyDTO.MappingTo<IdentityApplyDTO>();
            }
        }

        /// <summary>
        /// 查找申请信息（分页）
        /// </summary>
        /// <param name="request">分页信息</param>
        /// <returns>申请信息列表</returns>
        public async Task<PageCollection<IdentityApplyDTO>> FindIdentityApplyPagesAsync(PageFilterRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} FindIdentityApplyPagesAsync execute", $"{this.GetType().Name}"))
            {
                var dbRequest = GetDBPageRequest(request);
                var pageInfo = await _identityApplyDBRepository.GetPages(dbRequest);
                return PageCollectionConvert<IdentityApplyInfoDO, IdentityApplyDTO>(pageInfo);
            }
        }

        public async Task<List<IdentityApplyDTO>> GetIdentityApplyByUserCodeAsync(string userCode)
        {
            using (new Performance(_performanceThreshold, "{0} GetIdentityApplyByUserCodeAsync execute", $"{this.GetType().Name}"))
            {
                var dic = new Dictionary<string, string>();
                dic.Add("UserCode", userCode);
                var identityApplyList = await _identityApplyDBRepository.FindListAsync(dic);
                return identityApplyList.MappingTo<List<IdentityApplyDTO>>();
            }
        }

        /// <summary>
        /// 添加申请信息
        /// </summary>
        /// <param name="identityApplyDO">申请信息</param>
        /// <param name="extensionData">扩展信息</param>
        /// <returns>Id</returns>
        public async Task<string> InsertIdentityApplyAsync(CreateIdentityApplyDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} InsertIdentityApplyAsync execute", $"{this.GetType().Name}"))
            {
                var identityApplyDO = request.Data.MappingTo<IdentityApplyInfoDO>();
                return await _identityApplyDBRepository.InsertOneAsync(identityApplyDO, request.PlatformDatas, request.ExtensionData);
            }
        }

        /// <summary>
        /// 根据编码更新申请信息
        /// </summary>
        /// <param name="code">申请编码</param>
        /// <param name="data">申请信息</param>
        /// <returns>是否成功</returns>
        public async Task<bool> UpdateIdentityApplyByCodeAsync(string code, IdentityApplyDTO data, string extensionData)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateIdentityApplyByCodeAsync execute", $"{this.GetType().Name}"))
            {
                var identityApplyDO = data.MappingTo<IdentityApplyInfoDO>();
                return await _identityApplyDBRepository.UpdateOneAsync("Code", code, identityApplyDO, extensionData);
            }
        }

        /// <summary>
        /// 根据userCodes查询申请信息列表
        /// </summary>
        /// <param name="userCodes"></param>
        /// <returns></returns>
        public async Task<List<IdentityApplyDTO>> FindIdentityApplyListByUserCodeAsync(List<string> userCodes)
        {
            using (new Performance(_performanceThreshold, "{0} FindIdentityApplyListByUserCodeAsync execute", $"{this.GetType().Name}"))
            {
                var dic = new Dictionary<string, List<string>>();
                dic.Add("UserCode", userCodes);
                var identityApplyList = await _identityApplyDBRepository.FindInAsync(dic);
                return identityApplyList.MappingTo<List<IdentityApplyDTO>>();
            }
        }
    }
}