using System;
using System.Collections.Generic;
using System.Threading.Tasks;
using WingInterfaceLibrary.DB.Request;
using WingInterfaceLibrary.Interface.DBInterface;
using WingServerCommon.Service;
using WingInterfaceLibrary.Enum;
using WingServerCommon.Mapper;
using WingInterfaceLibrary.Request;
using System.Linq;
using WingServerCommon.Utilities;
using WingInterfaceLibrary.DTO.Record;
using WingInterfaceLibrary.DTO.Lab;
using WingInterfaceLibrary.Result;
using WingMongoDBModule.Entities;
using MongoDB.Driver;

namespace WingMongoDBModule.Service
{
    public partial class DatabaseService : JsonRpcService, ILabDBService
    {

        // <summary>
        /// 创建文件夹
        /// </summary>
        /// <returns></returns>
        public async Task<string> CreateLabFolderDBAsync(CreateLabFolderDBRequest request)
        {
            var filter = Builders<LabFolderEntity>.Filter.Where(f => f.Name == request.Name && f.ParentCode == request.ParentCode && f.CreatorCode == request.CreatorCode);
            var folder = await _labFolderDBRepository.FindOneAsync(filter);
            if (folder == null)
            {
                return await _labFolderDBRepository.InsertOneAsync(new Entities.LabFolderEntity
                {
                    Name = request.Name,
                    ParentCode = request.ParentCode,
                    CreatorCode = request.CreatorCode
                });
            }
            else
            {
                return folder.Code;
            }
        }

        // <summary>
        /// 创建上传记录
        /// </summary>
        /// <returns></returns>
        public async Task<string> CreateLabUploadRecordDBAsync(CreateLabUploadRecordDBRequest request)
        {
            return await _labUploadRecordRepository.InsertOneAsync(new Entities.LabUploadRecordEntity
            {
                FolderCode = request.FolderCode,
                IsComprehensiveAnalysis = request.IsComprehensiveAnalysis,
                IsSeparateAnalysis = request.IsSeparateAnalysis,
                UploadType = request.UploadType
            });
        }

        /// <summary>
        /// 分页查询图像列表
        /// </summary>
        /// <returns></returns>
        public async Task<PageResult<RemedicalInfoDTO>> GetFileListByFolderDBAsync(GetFileListByFolderDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetFileListByFolderDBAsync execute", "ILabDBService"))
            {
                var folderCode = request.FolderCode;
                var filterRequest = new PageFilterRequest();
                filterRequest.CurrentPage = request.CurrentPage;
                filterRequest.PageSize = request.PageSize;
                filterRequest.Filter = new Dictionary<string, string>();
                filterRequest.Filter.Add("RecordCode", folderCode);
                var dbResult = await GetRemedicalPagesByRecordCodeRAsync(filterRequest);
                foreach (var remedical in dbResult.PageData)
                {
                    if (remedical.TerminalImages != null)
                    {
                        remedical.TerminalImages.RecommendedDownloadMode = GetRecommendedDownloadMode(remedical.DeviceCode, remedical.TerminalImages.ImageSize);
                    }
                    if (remedical.CarotidResult != null)
                    {
                        remedical.CarotidResult.RecommendedDownloadMode = RecommendedDownloadModeEnum.Origin;
                    }
                }
                return dbResult;
            }
        }

        /// <summary>
        /// 获取推荐下载模式
        /// </summary>
        /// <param name="deviceCode"></param>
        /// <returns></returns>
        private RecommendedDownloadModeEnum GetRecommendedDownloadMode(string deviceCode, long fileSize)
        {
            RecommendedDownloadModeEnum recommendedDownloadMode = RecommendedDownloadModeEnum.Origin;
            if (fileSize >= 0 && fileSize < _downloadOriginalLowestValue)
            {
                recommendedDownloadMode = RecommendedDownloadModeEnum.CDN;
            }
            return recommendedDownloadMode;
        }

        // <summary>
        /// 查询文件夹列表
        /// </summary>
        /// <returns></returns>
        public async Task<IList<LabFolderDTO>> GetLabFolderListDBAsync(GetLabFolderListDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetLabFolderListDBAsync execute", "ILabDBService"))
            {
                var creatorCode = request.CreatorCode;
                var filter = Builders<LabFolderEntity>.Filter.Where(x => x.CreatorCode == creatorCode);
                var dbResult = await _labFolderDBRepository.FindAllAsync(filter);
                var result = dbResult.MappingTo<List<LabFolderDTO>>();
                return result;
            }
        }

        /// <summary>
        /// 创建实验室文件转换状态信息
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> CreateLabConvertStateDBAsync(CreateLabConvertStateDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} CreateLabConvertStateDBAsync execute", "ILabDBService"))
            {
                var result = await _labConvertStateDBRepository.InsertOneAsync(new LabConvertStateEntity
                {
                    CreatorCode = request.CreatorCode,
                    RecordCode = request.RecordCode,
                    IsTemplate = request.IsTemplate,
                    LastExecuteTime = request.LastExecuteTime
                });
                return !string.IsNullOrWhiteSpace(result);
            }
        }

        /// <summary>
        /// 设置实验室文件转换状态
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> SetLabConvertStateDBAsync(SetLabConvertStateDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} SetLabConvertStateDBAsync execute", "ILabDBService"))
            {
                var updateFilter = Builders<LabConvertStateEntity>.Filter.Eq(f => f.Code, request.Code);
                var update = Builders<LabConvertStateEntity>.Update.Set(s => s.LastExecuteTime, request.LastExecuteTime);
                var result = await _labConvertStateDBRepository.UpdateOneAsync(updateFilter, update);
                return result > 0;
            }
        }

        /// <summary>
        /// 创建实验室转换错误记录信息
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> AddConvertErrorInfoStateDBAsync(AddConvertErrorInfoStateDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} AddConvertErrorInfoStateDBAsync execute", "ILabDBService"))
            {
                var filter = Builders<LabConvertStateEntity>.Filter.Eq(f => f.RecordCode, request.Code);
                var convertState = await _labConvertStateDBRepository.FindOneAsync(filter);
                if(convertState != null)
                {
                    var updateFilter = Builders<LabConvertStateEntity>.Filter.Eq(f=>f.Code, convertState.Code);
                    if (convertState.DownloadErrorFiles == null || convertState.DownloadErrorFiles.Count == 0)
                    {
                        var update = Builders<LabConvertStateEntity>.Update.Set(s => s.ConvertErrorFiles, request.ConvertErrorFiles);
                        var result = await _labConvertStateDBRepository.UpdateOneAsync(updateFilter, update);
                        return result > 0;
                    }
                    else
                    {
                        var update = Builders<LabConvertStateEntity>.Update.AddToSetEach(s => s.ConvertErrorFiles, request.ConvertErrorFiles);
                        var result = await _labConvertStateDBRepository.UpdateOneAsync(updateFilter, update);
                        return result > 0;
                    }
                }
                return false;
            }
        }

        /// <summary>
        /// 创建实验室下载错误记录信息
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> AddDownloadErrorInfoStateDBAsync(AddDownloadErrorInfoStateDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} AddDownloadErrorInfoStateDBAsync execute", "ILabDBService"))
            {
                var filter = Builders<LabConvertStateEntity>.Filter.Eq(f => f.RecordCode, request.Code);
                var convertState = await _labConvertStateDBRepository.FindOneAsync(filter);
                if(convertState != null)
                {
                    var updateFilter = Builders<LabConvertStateEntity>.Filter.Eq(f=>f.Code, convertState.Code);
                    if (convertState.DownloadErrorFiles == null || convertState.DownloadErrorFiles.Count == 0)
                    {
                        var update = Builders<LabConvertStateEntity>.Update.Set(s => s.DownloadErrorFiles, request.DownloadErrorFiles);
                        var result = await _labConvertStateDBRepository.UpdateOneAsync(updateFilter, update);
                        return result > 0;
                    }
                    else
                    {
                        var update = Builders<LabConvertStateEntity>.Update.AddToSetEach(s => s.DownloadErrorFiles, request.DownloadErrorFiles);
                        var result = await _labConvertStateDBRepository.UpdateOneAsync(updateFilter, update);
                        return result > 0;
                    }
                }
                return false;
            }
        }

        /// <summary>
        /// 删除实验室转换错误记录信息
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> RemoveConvertErrorInfoStateDBAsync(RemoveConvertErrorInfoStateDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} RemoveConvertErrorInfoStateDBAsync execute", "ILabDBService"))
            {
                var filter = Builders<LabConvertStateEntity>.Filter.Eq(f => f.RecordCode, request.Code);
                var convertState = await _labConvertStateDBRepository.FindOneAsync(filter);
                if(convertState != null)
                {
                    var updateFilter = Builders<LabConvertStateEntity>.Filter.Eq(f=>f.Code, convertState.Code);
                    var errorFiles = convertState.ConvertErrorFiles.Where(w =>
                        !request.ConvertErrorFiles.Any(a =>
                            a.RecordCode == w.RecordCode
                            && a.FolderCode == w.FolderCode
                            && a.FilePath == w.FilePath
                        )
                    );
                    var update = Builders<LabConvertStateEntity>.Update.Set(s => s.ConvertErrorFiles, errorFiles);
                    var result = await _labConvertStateDBRepository.UpdateOneAsync(updateFilter, update);
                    return result > 0;
                }
                return false;
            }
        }


        /// <summary>
        /// 删除实验室下载错误记录信息
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> RemoveDownloadErrorInfoStateDBAsync(RemoveDownloadErrorInfoStateDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} RemoveDownloadErrorInfoStateDBAsync execute", "ILabDBService"))
            {
                var filter = Builders<LabConvertStateEntity>.Filter.Eq(f => f.RecordCode, request.Code);
                var convertState = await _labConvertStateDBRepository.FindOneAsync(filter);
                if (convertState != null)
                {
                    var updateFilter = Builders<LabConvertStateEntity>.Filter.Eq(f=>f.Code, convertState.Code);
                    var errorFiles = convertState.DownloadErrorFiles.Where(w =>
                        !request.DownloadErrorFiles.Any(a =>
                            a.FolderCode == w.FolderCode
                            && a.FileUploadInfos == w.FileUploadInfos
                            && a.FileName == w.FileName
                        )
                    );
                    var update = Builders<LabConvertStateEntity>.Update.Set(s => s.DownloadErrorFiles, errorFiles);
                    var result = await _labConvertStateDBRepository.UpdateOneAsync(updateFilter, update);
                    return result > 0;
                }
                return false;
            }
        }


        /// <summary>
        /// 获取实验室文件处理记录
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<List<LabConvertStateDTO>> GetLabConvertStateDBAsync(GetLabConvertStateDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} RemoveConvertErrorInfoStateDBAsync execute", "ILabDBService"))
            {
                var filter = Builders<LabConvertStateEntity>.Filter.Where(f => 
                    f.CreatorCode == request.CreatorCode &&
                    f.LastExecuteTime < DateTime.UtcNow &&
                    (f.ConvertErrorFiles.Count > 0 || f.DownloadErrorFiles.Count>0)
                );
                var convertStates = await _labConvertStateDBRepository.FindAllAsync(filter);
                return convertStates.Select(s => new LabConvertStateDTO
                {
                    Code = s.Code,
                    LastExecuteTime = s.LastExecuteTime,
                    RecordCode = s.RecordCode,
                    CreatorCode = s.CreatorCode,
                    IsTemplate = s.IsTemplate,
                    DownloadErrorFiles = s.DownloadErrorFiles,
                    ConvertErrorFiles = s.ConvertErrorFiles
                }).ToList();
            }
        }
    }
}