using System.Collections.Generic;
using System.Threading.Tasks;
using WingServerCommon.Service;
using WingInterfaceLibrary.Interface.DBInterface;
using WingServerCommon.Mapper;
using WingMongoDBModule.DO;
using WingInterfaceLibrary.DB.Request;
using WingServerCommon.Utilities;
using WingInterfaceLibrary.DTO.LiveRoom;
using System.Linq;
using WingInterfaceLibrary.Enum;
using WingMongoDBModule.Entities;
using WingInterfaceLibrary.Enum.CommentEnum;
using MongoDB.Driver;
using System;

namespace WingMongoDBModule.Service
{
    public partial class DatabaseService : JsonRpcService, ILiveRoomDBService
    {
        /// <summary>
        /// 查询房间详情
        /// </summary>
        /// <returns></returns>
        public async Task<LiveRoomDTO> GetLiveRoomByCodeAsync(GetLiveRoomByCodeDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetLiveRoomByCodeAsync execute", $"{this.GetType().Name}"))
            {
                var entity = await _liveRoomDBRepository.FindOneAsync("Code", request.LiveRoomCode);
                return entity.MappingTo<LiveRoomDTO>();
            }
        }

        /// <summary>
        /// 批量查询房间详情
        /// </summary>
        /// <param name="codes">The specified codes</param>
        /// <returns></returns>
        public async Task<List<LiveRoomDTO>> GetLiveRoomWithCodesAsync(GetLiveRoomWithCodesDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetLiveRoomWithCodesAsync execute", $"{this.GetType().Name}"))
            {
                var dic = new Dictionary<string, List<string>>();
                dic.Add("Code", request.Codes);
                var allDatas = await _liveRoomDBRepository.FindInAsync(dic);
                return allDatas.MappingTo<List<LiveRoomDTO>>();
            }
        }

        /// <summary>
        /// 查询直播中的房间集合
        /// </summary>
        /// <returns></returns>
        public async Task<List<LiveRoomDTO>> GetLiveRoomsAsync(GetLiveRoomsDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetLiveRoomsAsync execute", $"{this.GetType().Name}"))
            {
                var filter = Builders<LiveRoomEntity>.Filter.Where(c => (c.Status == LiveRoomStatus.Initiating || c.Status == LiveRoomStatus.ConnectionTimeout || c.Status == LiveRoomStatus.Connected) && c.IsDelete == false);
                var entities = await _liveRoomDBRepository.FindAllAsync(filter);
                var liveRooms = entities.MappingTo<List<LiveRoomDTO>>();
                return liveRooms;
            }
        }

        /// <summary>
        /// 查询业务关联房间详情
        /// </summary>
        /// <returns></returns>
        public async Task<LiveRoomDTO> GetBusinessLiveRoomAsync(GetBusinessLiveRoomDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetBusinessLiveRoomAsync execute", $"{this.GetType().Name}"))
            {
                var dic = new Dictionary<string, string>();
                dic.Add("RelatedCode", request.RelatedCode);
                var entities = await _liveRoomDBRepository.FindListAsync(dic);
                var liveRooms = entities.MappingTo<List<LiveRoomDTO>>();
                return liveRooms.FirstOrDefault();
            }
        }

        /// <summary>
        /// 保存房间详情
        /// </summary>
        /// <returns></returns>
        public async Task<string> AddOrUpdateLiveRoomAsync(AddOrUpdateLiveRoomDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} AddOrUpdateLiveRoomAsync execute", $"{this.GetType().Name}"))
            {
                var liveRoom = request.Data;
                var entity = await _liveRoomDBRepository.FindOneAsync("RelatedCode", liveRoom.RelatedCode);
                if (entity == null || string.IsNullOrWhiteSpace(entity.Code))
                {
                    entity = new LiveRoomEntity();
                    entity.Code = liveRoom.RelatedCode;
                    entity.RelatedCode = liveRoom.RelatedCode;
                    entity.InitiatorCode = liveRoom.InitiatorCode;
                    entity.RtcRoomId = liveRoom.RtcRoomId;
                    entity.SdkAppId = liveRoom.SdkAppId;
                    entity.BusinessModule = liveRoom.BusinessModule;
                    entity.Status = liveRoom.Status;
                    if (liveRoom.DeviceInfos != null && liveRoom.DeviceInfos.Any())
                    {
                        entity.DeviceInfos = liveRoom.DeviceInfos.MappingTo<List<LiveMemberDO>>();
                    }
                    if (liveRoom.UserInfos != null && liveRoom.UserInfos.Any())
                    {
                        entity.UserInfos = liveRoom.UserInfos.MappingTo<List<LiveMemberDO>>();
                    }
                    entity.ViewerInfos = new List<LiveViewerDO>();
                    if (liveRoom.ViewerInfos != null && liveRoom.ViewerInfos.Any())
                    {
                        entity.ViewerInfos = liveRoom.ViewerInfos.MappingTo<List<LiveViewerDO>>();
                    }
                    return await _liveRoomDBRepository.InsertOneAsync(entity);
                }
                else
                {
                    entity.RelatedCode = liveRoom.RelatedCode;
                    entity.InitiatorCode = liveRoom.InitiatorCode;
                    entity.RtcRoomId = liveRoom.RtcRoomId;
                    entity.SdkAppId = liveRoom.SdkAppId;
                    entity.BusinessModule = liveRoom.BusinessModule;
                    entity.Status = liveRoom.Status;
                    if (liveRoom.DeviceInfos != null && liveRoom.DeviceInfos.Any())
                    {
                        entity.DeviceInfos = liveRoom.DeviceInfos.MappingTo<List<LiveMemberDO>>();
                    }
                    if (liveRoom.UserInfos != null && liveRoom.UserInfos.Any())
                    {
                        entity.UserInfos = liveRoom.UserInfos.MappingTo<List<LiveMemberDO>>();
                    }
                    entity.ViewerInfos = new List<LiveViewerDO>();
                    if (liveRoom.ViewerInfos != null && liveRoom.ViewerInfos.Any())
                    {
                        entity.ViewerInfos = liveRoom.ViewerInfos.MappingTo<List<LiveViewerDO>>();
                    }
                    var result = await _liveRoomDBRepository.UpdateOneAsync("Code", entity.Code, entity);
                    if (result)
                    {
                        return entity.Code;
                    }
                }
                return string.Empty;
            }
        }

        /// <summary>
        /// 保存用户直播状态
        /// </summary>
        /// <returns></returns>
        public async Task<bool> AddOrUpdateLiveMemberAsync(AddOrUpdateLiveMemberDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} AddOrUpdateLiveMemberAsync execute", $"{this.GetType().Name}"))
            {
                var entity = await _liveRoomDBRepository.FindOneAsync("Code", request.LiveRoomCode);
                if (entity != null && !string.IsNullOrWhiteSpace(entity.Code))
                {
                    if (request.MemberType == LiveMemberEnum.Device)
                    {
                        var liveMember = entity.DeviceInfos.FirstOrDefault(m => m.Code == request.MemberCode)
                                ?? new LiveMemberDO { Code = request.MemberCode, MemberType = request.MemberType };
                        liveMember.Name = request.Name;
                        liveMember.HeadImageToken = request.HeadImageToken;
                        liveMember.Status = request.Status;
                        liveMember.MuteOpened = request.MuteOpened;
                        liveMember.VideoOpened = request.VideoOpened;
                        liveMember.SortLevel = request.SortLevel;
                        liveMember.LoginSource = request.LoginSource;
                        liveMember.InviterCode = request.InviterCode;
                        liveMember.BackgroundRole = request.BackgroundRole;
                        liveMember.VideoDeviceInfos = request.VideoDeviceInfos?.MappingTo<List<VideoDeviceInfoDO>>() ?? new List<VideoDeviceInfoDO>();
                        if (request.LiveData != null)
                        {
                            liveMember.LiveData = request.LiveData.MappingTo<LiveDataDO>();
                        }
                        if (!entity.DeviceInfos.Any(m => m.Code == request.MemberCode))
                        {
                            entity.DeviceInfos.Add(liveMember);
                        }
                    }
                    else if (request.MemberType == LiveMemberEnum.User)
                    {
                        var liveMember = entity.UserInfos.FirstOrDefault(m => m.Code == request.MemberCode)
                                ?? new LiveMemberDO { Code = request.MemberCode, MemberType = request.MemberType };
                        liveMember.Name = request.Name;
                        liveMember.HeadImageToken = request.HeadImageToken;
                        liveMember.Status = request.Status;
                        liveMember.MuteOpened = request.MuteOpened;
                        liveMember.VideoOpened = request.VideoOpened;
                        liveMember.InviterCode = request.InviterCode;
                        liveMember.IsControllingParameter = request.IsControllingParameter;
                        liveMember.SortLevel = request.SortLevel;
                        liveMember.LoginSource = request.LoginSource;
                        liveMember.BackgroundRole = request.BackgroundRole;
                        liveMember.VideoDeviceInfos = new List<VideoDeviceInfoDO>();
                        if (request.LiveData != null)
                        {
                            liveMember.LiveData = request.LiveData.MappingTo<LiveDataDO>();
                        }
                        if (!entity.UserInfos.Any(m => m.Code == request.MemberCode))
                        {
                            entity.UserInfos.Add(liveMember);
                        }
                    }
                    // entity.Status = request.RoomStatus;
                    return await _liveRoomDBRepository.UpdateOneAsync("Code", entity.Code, entity);
                }
                return false;
            }
        }

        /// <summary>
        /// 移除用户或者超声机
        /// </summary>
        /// <returns></returns>
        public async Task<bool> RemoveLiveMemberAsync(RemoveLiveMemberDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} RemoveLiveMemberAsync execute", $"{this.GetType().Name}"))
            {
                var entity = await _liveRoomDBRepository.FindOneAsync("Code", request.LiveRoomCode);
                if (entity != null && !string.IsNullOrWhiteSpace(entity.Code))
                {
                    if (request.MemberType == LiveMemberEnum.Device)
                    {
                        if (entity.DeviceInfos?.Count > 0)
                        {
                            var newDeviceList = new List<LiveMemberDO>();
                            foreach (var deviceItem in entity.DeviceInfos)
                            {
                                if (deviceItem.Code != request.MemberCode)
                                {
                                    newDeviceList.Add(deviceItem);
                                }
                            }
                            entity.DeviceInfos = newDeviceList;
                        }
                    }
                    else if (request.MemberType == LiveMemberEnum.User)
                    {
                        if (entity.UserInfos?.Count > 0)
                        {
                            var newUserList = new List<LiveMemberDO>();
                            foreach (var userItem in entity.UserInfos)
                            {
                                if (userItem.Code != request.MemberCode)
                                {
                                    newUserList.Add(userItem);
                                }
                            }
                            entity.UserInfos = newUserList;
                        }
                    }
                    return await _liveRoomDBRepository.UpdateOneAsync("Code", entity.Code, entity);
                }
                return false;
            }
        }

        /// <summary>
        /// 删除房间
        /// </summary>
        /// <returns></returns>
        public async Task<bool> DeleteLiveRoomAsync(DeleteLiveRoomDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} DeleteLiveRoomAsync execute", $"{this.GetType().Name}"))
            {
                var code = request.LiveRoomCode;
                var result = await _liveRoomDBRepository.DeleteOneAsync("Code", code);
                return result;
            }
        }

        /// <summary>
        /// 记录直播观看人信息
        /// </summary>
        /// <returns></returns>
        public async Task<bool> AddOrUpdateViewerAsync(AddOrUpdateViewerDBRequest request)
        {
            var liveRoomCode = request.LiveRoomCode;
            var userCode = request.UserCode;
            var userName = request.UserName;
            using (new Performance(_performanceThreshold, "{0} AddOrUpdateViewerAsync execute", $"{this.GetType().Name}"))
            {
                var entity = await _liveRoomDBRepository.FindOneAsync("Code", liveRoomCode);
                if (entity != null && !string.IsNullOrWhiteSpace(entity.Code))
                {
                    var reviewers = entity.ViewerInfos;
                    var current = reviewers.FirstOrDefault(x => x.UserCode == userCode);
                    if (current == null)
                    {
                        current = new LiveViewerDO
                        {
                            UserCode = userCode,
                            Name = userName,
                            LastReportTime = DateTime.UtcNow,
                        };
                        reviewers.Add(current);
                    }
                    else
                    {
                        current.Name = userName;
                        current.LastReportTime = DateTime.UtcNow;
                    }
                    entity.ViewerInfos = reviewers;
                    return await _liveRoomDBRepository.UpdateOneAsync("Code", entity.Code, entity);
                }
                return false;
            }
        }

        /// <summary>
        /// 删除直播观看人信息
        /// </summary>
        /// <returns></returns>
        public async Task<bool> DeleteViewerAsync(DeleteViewerDBRequest request)
        {
            var liveRoomCode = request.LiveRoomCode;
            var userCode = request.UserCode;
            using (new Performance(_performanceThreshold, "{0} DeleteViewerAsync execute", $"{this.GetType().Name}"))
            {
                var entity = await _liveRoomDBRepository.FindOneAsync("Code", liveRoomCode);
                if (entity != null && !string.IsNullOrWhiteSpace(entity.Code))
                {
                    var reviewers = entity.ViewerInfos;
                    if (!reviewers.Any(x => x.UserCode == userCode))
                    {
                        return true;
                    }
                    else
                    {
                        reviewers.RemoveAll(x => x.UserCode == userCode);
                        entity.ViewerInfos = reviewers;
                        return await _liveRoomDBRepository.UpdateOneAsync("Code", entity.Code, entity);
                    }
                }
                return false;
            }
        }
    }
}