using System.Linq;
using System;
using System.Threading.Tasks;
using MongoDB.Driver;
using WingInterfaceLibrary.DTO.Measure;
using WingInterfaceLibrary.Interface.DBInterface;
using WingInterfaceLibrary.Request.DBRequest;
using WingMongoDBModule.Entities;
using WingServerCommon.Mapper;
using WingServerCommon.Service;
using WingServerCommon.Utilities;
using System.Collections.Generic;

namespace WingMongoDBModule.Service
{
    /// <summary>
    /// 测量结果记录导出服务
    /// </summary>
    public partial class DatabaseService : JsonRpcService, IMeasureExportRecordDBService
    {
        /// <summary>
        /// 查询测量结果记录导出服务
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns></returns>
        public async Task<MeasureExportRecordDTO> GetMeasureExportRecordsByConditionAsync(QueryMeasureExportRecordDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetMeasureExportRecordsByConditionAsync execute", $"{this.GetType().Name}"))
            {
                var filter = Builders<MeasureExportRecordEntity>.Filter.Where(c => c.IsDelete == false && c.LanguageCode == request.LanguageCode && c.RecordType == request.RecordType);
                var remedicalMeasuredInfoCodes = request.RemedicalMeasuredInfos.Select(c => c.RemedicalMeasuredInfoCode).Distinct().ToList();
                var sorts = request.RemedicalMeasuredInfos.Select(c => c.SortNum).Distinct().ToList();
                var inFilter = Builders<MeasureExportRecordEntity>.Filter.ElemMatch(c => c.RemedicalMeasuredInfos, Builders<MeasureExportDetailEntity>.Filter.In(d => d.RemedicalMeasuredInfoCode, remedicalMeasuredInfoCodes));
                //var inFilter = Builders<MeasureExportRecordEntity>.Filter.AnyIn(c => c.RemedicalMeasuredInfoCodes, request.RemedicalMeasuredInfoCodes);
                filter = filter & inFilter;                
                var findList = await _measureExportRecordDBRepository.FindAllAsync(filter);
                var findEntity = new MeasureExportRecordDTO();
                if (findList?.Count > 0)
                {
                    foreach(var item in findList.OrderByDescending(c => c.CreateTime))
                    {
                        var codes = item.RemedicalMeasuredInfos.Select(d => d.RemedicalMeasuredInfoCode).Distinct().ToList();
                        var sortNums = item.RemedicalMeasuredInfos.Select(d => d.SortNum).Distinct().ToList();
                        if (item.RemedicalMeasuredInfos.Count == request.RemedicalMeasuredInfos.Count 
                        && codes.All(t => remedicalMeasuredInfoCodes.Any(b => b == t)) 
                        && sortNums.All(t => sorts.Any(b => b == t)))
                        {
                            findEntity = item.MappingTo<MeasureExportRecordDTO>();
                            break;
                        }
                    }
                }
                return findEntity;
            }
        }

        /// <summary>
        /// 查询分享图像记录导出服务
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns></returns>
        public async Task<MeasureExportRecordDTO> GetShareImagesRecordsByConditionAsync(QueryMeasureExportRecordDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetShareImagesRecordsByConditionAsync execute", $"{this.GetType().Name}"))
            {
                var filter = Builders<MeasureExportRecordEntity>.Filter.Where(c => c.IsDelete == false && c.LanguageCode == request.LanguageCode && c.RecordType == request.RecordType);
                var fileTokens = request.RemedicalMeasuredInfos.Select(c => c.SourceFileToken).Distinct().ToList();
                var sorts = request.RemedicalMeasuredInfos.Select(c => c.SortNum).Distinct().ToList();
                var inFilter = Builders<MeasureExportRecordEntity>.Filter.ElemMatch(c => c.RemedicalMeasuredInfos, Builders<MeasureExportDetailEntity>.Filter.In(d => d.SourceFileToken, fileTokens));
                filter = filter & inFilter;                
                var findList = await _measureExportRecordDBRepository.FindAllAsync(filter);
                var findEntity = new MeasureExportRecordDTO();
                if (findList?.Count > 0)
                {
                    foreach(var item in findList.OrderByDescending(c => c.CreateTime))
                    {
                        var codes = item.RemedicalMeasuredInfos.Select(d => d.SourceFileToken).Distinct().ToList();
                        var sortNums = item.RemedicalMeasuredInfos.Select(d => d.SortNum).Distinct().ToList();
                        if (item.RemedicalMeasuredInfos.Count == request.RemedicalMeasuredInfos.Count 
                        && codes.All(t => fileTokens.Any(b => b == t)) 
                        && sortNums.All(t => sorts.Any(b => b == t)))
                        {
                            findEntity = item.MappingTo<MeasureExportRecordDTO>();
                            break;
                        }
                    }
                }
                return findEntity;
            }
        }

        /// <summary>
        /// 新增测量结果记录导出服务
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns></returns>
        public async Task<string> AddMeasureExportRecordAsync(AddMeasureExportRecordDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} AddMeasureExportRecordAsync execute", $"{this.GetType().Name}"))
            {
                var entity = new MeasureExportRecordEntity()
                {
                    LanguageCode = request.LanguageCode,
                    RemedicalMeasuredInfos = request.RemedicalMeasuredInfos.MappingTo<List<MeasureExportDetailEntity>>(),
                    ExportFileToken = request.ExportFileToken,
                    RecordType = request.RecordType,
                    IsCompleted = false,
                    RecordCode = request.RecordCode
                };
                entity.CreateTime = DateTime.UtcNow;
                var recordCode = await _measureExportRecordDBRepository.InsertOneAsync(entity);
                // if (!string.IsNullOrEmpty(recordCode))
                // {
                //     return true;
                // }
                // return false;
                return recordCode;
            }
        }

        /// <summary>
        /// 修改图像操作记录
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns></returns>
        public async Task<bool> UpdateMeasureExportRecordAsync(UpdateMeasureExportRecordDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} AddMeasureExportRecordAsync execute", $"{this.GetType().Name}"))
            {
                var res = false;
                var findEntity = await _measureExportRecordDBRepository.FindOneAsync("Code", request.Code);
                if (!string.IsNullOrEmpty(findEntity?.Code))
                {
                    findEntity.RemedicalMeasuredInfos = request.RemedicalMeasuredInfos.MappingTo<List<MeasureExportDetailEntity>>();
                    findEntity.LanguageCode = request.LanguageCode;
                    findEntity.RecordType = request.RecordType;
                    findEntity.IsCompleted = request.IsCompleted;
                    findEntity.UpdateTime = DateTime.UtcNow;
                    res = await _measureExportRecordDBRepository.UpdateOneAsync("Code", findEntity.Code, findEntity, true);
                }
                return res;
            }
        }

        /// <summary>
        /// 查询分享图像列表记录
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns></returns>
        public async Task<MeasureExportRecordDTO> GetMeasureExportRecordByCodeAsync(string code)
        {
            using (new Performance(_performanceThreshold, "{0} GetMeasureExportRecordByCodeAsync execute", $"{this.GetType().Name}"))
            {
                var findEntity = await _measureExportRecordDBRepository.FindOneAsync("Code", code);
                var res = findEntity.MappingTo<MeasureExportRecordDTO>();
                return res;
            }
        }

        /// <summary>
        /// 查询分享图像列表记录
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns></returns>
        public async Task<List<MeasureExportRecordDTO>> GetMeasureExportRecordByRecordCodeAsync(string recordCode, WingInterfaceLibrary.Enum.MeasureImageRecordTypeEnum recordType)
        {
            using (new Performance(_performanceThreshold, "{0} GetMeasureExportRecordByRecordCodeAsync execute", $"{this.GetType().Name}"))
            {
                var filter = Builders<MeasureExportRecordEntity>.Filter.Where(c => c.IsDelete == false && c.RecordCode == recordCode && c.RecordType == recordType);
                var findList = await _measureExportRecordDBRepository.FindAllAsync(filter);
                var res = findList.MappingTo<List<MeasureExportRecordDTO>>();
                return res;
            }
        }
    }
}