using System.Linq;
using System.Threading.Tasks;
using WingServerCommon.Service;
using WingInterfaceLibrary.Request;
using WingInterfaceLibrary.DTO.Message;
using WingInterfaceLibrary.Interface.DBInterface;
using WingInterfaceLibrary.DB.Request;
using WingMongoDBModule.Entities;
using System;
using WingServerCommon.Mapper;
using WingServerCommon.Utilities;
using MongoDB.Driver;
using MongoDB.Bson;
using WingMongoDBModule.Request;
using WingInterfaceLibrary.Enum;
using WingInterfaceLibrary.Result;
using System.Collections.Generic;

namespace WingMongoDBModule.Service
{
    public partial class DatabaseService : JsonRpcService, IMessageDBService
    {
        /// <summary>
        /// 通过UserCode获取消息列表接口
        /// </summary>
        /// <returns></returns>
        public async Task<PageResult<MessageInfoDTO>> GetMessagesPageDBAsync(GetMessagesDBPageRequest request)
        {
            var keyword = request.Keyword;
            var builder = Builders<MessageInfoEntity>.Filter;
            var clientIdFilter = builder.ElemMatch(x => x.ClientInfos, Builders<ClientInfoEntity>.Filter.Eq(c => c.ClientId, request.ClientId));
            var filter = builder.Where(c => c.IsDelete == false) & clientIdFilter;
            if (!string.IsNullOrEmpty(keyword))
            {
                keyword = EscapeExprSpecialWord(keyword);//字符串转义
                filter = filter & builder.Regex(c => c.Content, new BsonRegularExpression(keyword, "i"));
            }
            if (request.Status == QueryCMSMessageStatusEnum.UnRead || request.Status == QueryCMSMessageStatusEnum.Read)
            {
                bool isReaded = false;
                if (request.Status == QueryCMSMessageStatusEnum.Read)
                {
                    isReaded = true;
                }
                var readFilter = builder.ElemMatch(x => x.ClientInfos, Builders<ClientInfoEntity>.Filter.Eq(c => c.IsReaded, isReaded));
                filter = filter & readFilter;
            }
            if (request.NotificationTypes!=null && request.NotificationTypes.Count > 0)
            {
                filter = filter & builder.Where(c => request.NotificationTypes.Contains(c.NotificationType));
            }
            if (!string.IsNullOrEmpty(request.RelevanceCode))
            {
                filter = filter & builder.Where(c => c.RelevanceCode == request.RelevanceCode);
            }
            if (request.StartTime > DateTime.MinValue)
            {
                filter = filter & builder.Where(x => x.NotifyTime >= request.StartTime);
            }
            if (request.EndTime > DateTime.MinValue)
            {
                filter = filter & builder.Where(x => x.NotifyTime < request.EndTime.AddSeconds(1));
            }
            var dbPage = new DBPageRequest<MessageInfoEntity>();
            dbPage.CurrentPage = request.CurrentPage;
            dbPage.PageSize = request.PageSize;
            dbPage.Filter = filter;

            var sort1 = Builders<MessageInfoEntity>.Sort.Ascending("ClientInfos.IsReaded");
            var sort2 = Builders<MessageInfoEntity>.Sort.Descending(f => f.NotifyTime);
            dbPage.Sort = Builders<MessageInfoEntity>.Sort.Combine(sort1, sort2);
            var pageResult = await _messageInfoDBRepository.GetPages(dbPage);
            return PageResultConvert<MessageInfoEntity, MessageInfoDTO>(pageResult);
        }

        /// <summary>
        /// 增加消息列表接口
        /// </summary>
        /// <returns></returns>
        public async Task<bool> AddMessageInfoDBAsync(AddMessagesDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} AddMessageInfoDBAsync execute", "DatabaseService"))
            {
                var config = request.MappingTo<MessageInfoEntity>();
                var result = await _messageInfoDBRepository.InsertOneAsync(config);
                if (!string.IsNullOrEmpty(result))
                {
                    return true;
                }
                else
                {
                    return false;
                }
            }
        }

        /// <summary>
        /// 通过UserCode获取消息列表接口
        /// </summary>
        /// <returns></returns>
        public async Task<MessageInfoDTO> GetMessageDBByConditionAsync(FindMessagesDBRequest request)
        {
            var builder = Builders<MessageInfoEntity>.Filter;
            var clientIdFilter = builder.ElemMatch(x => x.ClientInfos, Builders<ClientInfoEntity>.Filter.Eq(c => c.ClientId, request.ClientId));
            var filter = builder.Where(c => c.IsDelete == false && c.ReceiverType == request.ReceiverType 
                                            && c.TransactionType == request.TransactionType) & clientIdFilter;
            if (request.NotificationTypes!=null && request.NotificationTypes.Count > 0)
            {
                filter = filter & builder.Where(c => request.NotificationTypes.Contains(c.NotificationType));
            }
            if (!string.IsNullOrEmpty(request.RelevanceCode))
            {
                filter = filter & builder.Where(c => c.RelevanceCode == request.RelevanceCode);
            }
            var findEntity = await _messageInfoDBRepository.FindOneAsync(filter);
            var result = findEntity.MappingTo<MessageInfoDTO>();
            return result;
        }

        /// <summary>
        /// 更新消息信息接口
        /// </summary>
        /// <returns></returns>
        public async Task<bool> UpdateMessageInfoDBAsync(UpdateMessagesDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateMessageInfoDBAsync execute", "DatabaseService"))
            {
                var filter = Builders<MessageInfoEntity>.Filter.Where(c => c.Code == request.MessageCode && c.IsDelete == false);
                var result = await _messageInfoDBRepository.FindOneAsync(filter);
                var res = false;
                if (result != null && !string.IsNullOrEmpty(result.Code) && result.ClientInfos != null && result.ClientInfos.Count > 0)
                {
                    foreach (var item in result.ClientInfos)
                    {
                        if (item.ClientId == request.ClientId)
                        {
                            item.DeliveryTime = DateTime.UtcNow;
                            if (request.IsReaded && !item.IsReaded)
                            {
                                item.ReadTime = DateTime.UtcNow;
                                item.IsReaded = true;
                            }
                            break;
                        }
                    }
                    var update = Builders<MessageInfoEntity>.Update
                    .Set(c => c.ClientInfos, result.ClientInfos)
                    .Set(c => c.UpdateTime, DateTime.UtcNow);
                    res = await _messageInfoDBRepository.UpdateOneAsync(filter, update) > 0;
                }
                return res;
            }
        }

        /// <summary>
        ///批量更新消息状态信息列表接口
        /// </summary>
        /// <returns></returns>
        public async Task<bool> UpdateMessagesStatusDBAsync(UpdateMessagesStatusDBequest request)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateMessagesStatusDBAsync execute", "DatabaseService"))
            {
                var subFilter = Builders<ClientInfoEntity>.Filter.Where(w => w.ClientId == request.ClientId && !w.IsReaded);
                var filter1 = Builders<MessageInfoEntity>.Filter.ElemMatch(f => f.ClientInfos, subFilter);
                var filter2 = Builders<MessageInfoEntity>.Filter.Where(c => request.MessageCodes.Contains(c.Code) && c.IsDelete == false);
                var filterCombine = Builders<MessageInfoEntity>.Filter.And(filter1, filter2);
                var result = await _messageInfoDBRepository.FindAllAsync(filterCombine);
                var res = false;
                if (result != null && result.Count > 0)
                {
                    foreach (var message in result)
                    {
                        var messageFilter = Builders<MessageInfoEntity>.Filter.Where(w => w.Code == message.Code);
                        var pullOne = Builders<MessageInfoEntity>.Update.PullFilter(f => f.ClientInfos, subFilter);
                        await _messageInfoDBRepository.UpdateOneAsync(messageFilter, pullOne);
                        var clientInfo = message.ClientInfos.FirstOrDefault(f => f.ClientId == request.ClientId);
                        clientInfo.IsReaded = true;
                        var pushOne = Builders<MessageInfoEntity>.Update.Push(f => f.ClientInfos, clientInfo);
                        await _messageInfoDBRepository.UpdateOneAsync(messageFilter, pushOne);
                    }
                    res = true;
                }
                return res;
            }
        }

        /// <summary>
        /// 获取未读消息数量
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<List<string>> GetNoReadMessagesDBAsync(GetNoReadMessagesDBRequest request)
        {
            if (request.NotificationTypes==null || request.NotificationTypes.Count == 0)
            {
                ThrowCustomerException(CustomerRpcCode.NotificationParamError, "NotificationParamError");
            }
            var subFilter = Builders<ClientInfoEntity>.Filter.Where(w => w.ClientId == request.ClientId 
                && !w.IsReaded 
                // && w.DeliveryTime >= DateTime.UtcNow.AddYears(-1) 
                // && w.DeliveryTime < DateTime.UtcNow.AddSeconds(1) 
            );
            var filter1 = Builders<MessageInfoEntity>.Filter.ElemMatch(f => f.ClientInfos, subFilter);
            var filter2 = Builders<MessageInfoEntity>.Filter.Where(c => c.IsDelete == false && c.NotifyTime >= DateTime.UtcNow.AddYears(-1) && c.NotifyTime < DateTime.UtcNow.AddSeconds(1));
            var filter3 = Builders<MessageInfoEntity>.Filter.In(c=>c.NotificationType, request.NotificationTypes);
            var filterCombine = Builders<MessageInfoEntity>.Filter.And(filter1, filter2, filter3);
            var messageCodeProjection = Builders<MessageInfoEntity>.Projection.Include(f => f.Code);
            var result = await _messageInfoDBRepository.FindAllAsync(filterCombine, new FindOptions<MessageInfoEntity> { Projection = messageCodeProjection });
            return result?.Select(f => f.Code)?.ToList();
        }
    }
}