﻿using System.IO;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using JsonRpcLite.Services;
using WingMongoDBModule.DO;
using WingInterfaceLibrary.DB.Request;
using WingInterfaceLibrary.Interface.DBInterface;
using WingServerCommon.Service;
using WingInterfaceLibrary.DTO.Organization;
using WingInterfaceLibrary.Enum;
using WingServerCommon.Mapper;
using WingMongoDBModule.Mongodb.Driver;
using WingMongoDBModule.Entity;
using WingInterfaceLibrary.Request;
using WingInterfaceLibrary.Request.DBRequest;
using WingMongoDBModule.Common;
using MongoDB.Driver;
using WingMongoDBModule.Request;
using WingServerCommon.Utilities;
using WingServerCommon.Interfaces.Cache;
using WingInterfaceLibrary.Request.Management;
using WingServerCommon.Log;
using Newtonsoft.Json;
using WingMongoDBModule.Entities;
using WingInterfaceLibrary.Result;
using WingInterfaceLibrary.Request.User;
using System.Collections;
using MongoDB.Driver.GeoJsonObjectModel;

namespace WingMongoDBModule.Service
{
    public partial class DatabaseService : JsonRpcService, IOrganizationDBService
    {

        /// <summary>
        /// > 该函数在数据库中创建一个组织
        /// </summary>
        /// <param name="CreateOrganizationDBRequest">这是传递给服务的请求对象。</param>
        /// <returns>
        /// 返回值是一个字符串。
        /// </returns>
        public async Task<string> CreateOrganizationAsync(CreateOrganizationDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} CreateOrganizationAsync execute", $"{this.GetType().Name}"))
            {
                var organizationDO = request.Data.MappingTo<OrganizationDO>();
                if (!string.IsNullOrEmpty(request.Data.LogoUrl))
                {
                    organizationDO.LogoUrl = request.Data.LogoUrl.ToUrlToken();
                }
                if (organizationDO.ReferralOrganizationCodes?.Any() != true)
                {
                    organizationDO.ReferralOrganizationCodes = null;
                }
                //验证是否有重复名称
                MongoCustomerFilter<MongoTable> customerFilter = new();
                MongoCustomerFilter<MongoDataItem> subFilter = new();
                subFilter.AppendEq((item => item.Key, "OrganizationName"));
                subFilter.AppendEq((item => item.Value, organizationDO.OrganizationName));
                customerFilter.SubFilter(item => item.QueryData.QueryItems, subFilter.AndCombine());
                subFilter.ClearFilter();
                var isFound = await _organizationDBRepository.FindOneAsync(customerFilter.AndCombine());
                if (isFound != null && isFound.OrganizationName == organizationDO.OrganizationName)
                {
                    throw new RpcException((int)CustomerRpcCode.OrganizationNameDuplicate, "OrganizationName Duplicate", "OrganizationName Duplicate");
                }
                return await _organizationDBRepository.InsertOneAsync(organizationDO, request.PlatformDatas, request.ExtensionData, true);
            }
        }

        /// <summary>
        /// > 这是创建组织的功能
        /// </summary>
        /// <param name="CreateOrganizationsDBRequest"></param>
        /// <returns>
        /// 返回值是长的。
        /// </returns>
        public async Task<long> CreateOrganizationsAsync(CreateOrganizationsDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} CreateOrganizationsAsync execute", $"{this.GetType().Name}"))
            {
                var req = new List<(OrganizationDO organizationDO, Dictionary<string, string> platformDatas, string extensionData)>();
                foreach (var info in request.Infos)
                {
                    info.Data.OrganizationName = info.Data.OrganizationName.Trim();
                    MongoCustomerFilter<MongoTable> customerFilter = new();
                    MongoCustomerFilter<MongoDataItem> subFilter = new();
                    //验证是否有重复名称
                    subFilter.AppendEq((item => item.Key, "OrganizationName"));
                    subFilter.AppendEq((item => item.Value, info.Data.OrganizationName));
                    customerFilter.SubFilter(item => item.QueryData.QueryItems, subFilter.AndCombine());
                    subFilter.ClearFilter();
                    subFilter.AppendEq((item => item.Key, "ParentCode"));
                    subFilter.AppendEq((item => item.Value, info.Data.ParentCode));
                    customerFilter.SubFilter(item => item.QueryData.QueryItems, subFilter.AndCombine());
                    subFilter.ClearFilter();
                    var isFound = await _organizationDBRepository.FindOneAsync(customerFilter.AndCombine());
                    if (isFound != null && isFound.OrganizationName == info.Data.OrganizationName)
                    {
                        throw new RpcException((int)CustomerRpcCode.OrganizationNameDuplicate, "OrganizationName Duplicate", "OrganizationName Duplicate");
                    }
                    var organizationDO = info.Data.MappingTo<OrganizationDO>();
                    if (organizationDO.ReferralOrganizationCodes?.Any() != true)
                    {
                        organizationDO.ReferralOrganizationCodes = null;
                    }
                    var shortCode = "";
                    for (var i = 0; i < 5; i++)
                    {
                        shortCode = GeneralShortCode();
                        if (!string.IsNullOrWhiteSpace(shortCode))
                        {
                            var orgInfo = await _organizationDBRepository.FindOneAsync("ShortCode", shortCode);
                            if (orgInfo == null || string.IsNullOrWhiteSpace(orgInfo.Code))
                            {
                                break;
                            }
                        }
                        shortCode = string.Empty;
                    }
                    organizationDO.ShortCode = shortCode;
                    req.Add((organizationDO, info.PlatformDatas, info.ExtensionData));
                }
                return await _organizationDBRepository.InsertAllAsync(req);
            }
        }

        /// <summary>
        /// 它通过代码从数据库中获取组织并将其作为 DTO 返回
        /// </summary>
        /// <param name="GetOrganizationDBRequest">这是传递给服务的请求对象。</param>
        /// <returns>
        /// 正在返回 OrganizationDTO。
        /// </returns>
        public async Task<OrganizationDTO> GetOrganizationDetailAsync(GetOrganizationDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetOrganizationDetailAsync execute", $"{this.GetType().Name}"))
            {
                var organizationShortCode = request.ShortCode?.Trim() ?? string.Empty;
                if (string.IsNullOrWhiteSpace(request.OrganizationCode) && string.IsNullOrWhiteSpace(organizationShortCode))
                {
                    return null;
                }
                OrganizationDO organizationDO = null;
                if (!string.IsNullOrEmpty(organizationShortCode))
                {
                    organizationDO = await _organizationDBRepository.FindOneAsync("ShortCode", organizationShortCode.ToUpper());
                    if (organizationDO == null)
                    {
                        if (!string.IsNullOrEmpty(request.OrganizationCode))
                        {
                            organizationDO = await _organizationDBRepository.GetOrganizationByCodeAsync(request.OrganizationCode);
                        }
                    }
                    if (organizationDO == null)
                    {
                        ThrowCustomerException(CustomerRpcCode.OrgShortCodeError, "Organization short code error");
                        return null;
                    }
                }
                else
                {
                    if (!string.IsNullOrEmpty(request.OrganizationCode))
                    {
                        organizationDO = await _organizationDBRepository.GetOrganizationByCodeAsync(request.OrganizationCode);
                    }
                }
                if (organizationDO == null)
                {
                    return null;
                }
                if (organizationDO.ReferralLimitHours <= 0)
                {
                    var settingDO = await GetReferralRecordSettingAsync(new GetReferralRecordSettingDBRequest { });
                    organizationDO.ReferralLimitHours = settingDO.LimitHours;
                }

                var dto = organizationDO.MappingTo<OrganizationDTO>();
                var orgs = new List<OrganizationDO>();
                if (!string.IsNullOrWhiteSpace(organizationDO.ParentCode))
                {
                    var parOrganization = await _organizationDBRepository.GetOrganizationByCodeAsync(organizationDO.ParentCode);
                    if (parOrganization != null)
                    {
                        dto.ParentOrganizationName = parOrganization.OrganizationName;
                        orgs.Add(parOrganization);
                    }
                }
                if (organizationDO.ReferralOrganizationCodes?.Any() == true)
                {
                    var dic = new Dictionary<string, List<string>>();
                    dic.Add("Code", organizationDO.ReferralOrganizationCodes);
                    var referralOrganizations = await _organizationDBRepository.FindInAsync(dic);

                    foreach (var orgCode in organizationDO.ReferralOrganizationCodes)
                    {
                        if (!orgs.Any(a => a.Code == orgCode))
                        {
                            var org = referralOrganizations.FirstOrDefault(x => x.OrganizationCode == orgCode);
                            if (org != null)
                            {
                                orgs.Add(org);
                            }
                        }
                    }
                }
                dto.ReferralOrganizationCodes = orgs.Select(x => x.Code).Where(x => x != dto.OrganizationCode).Distinct().ToList();
                dto.ReferralOrganizations = orgs.Select(x => new ReferralOrganizationDTO { OrganizationCode = x.Code, OrganizationName = x.OrganizationName }).ToList();

                return dto;
            }
        }


        /// <summary>
        /// 根据名称查询机构（不含对接机构）
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<OrganizationDTO> GetOrganizationDetailByNameAsync(GetOrganizationByNameDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetOrganizationDetailAsync execute", $"{this.GetType().Name}"))
            {
                string name = request.OrganizationName;
                if (string.IsNullOrWhiteSpace(name))
                {
                    return null;
                }
                var organizationDO = await _organizationDBRepository.FindOneAsync("OrganizationName", name);
                if (organizationDO == null)
                {
                    return null;
                }
                var dto = organizationDO.MappingTo<OrganizationDTO>();
                return dto;
            }
        }


        /// <summary>
        /// > 从数据库中获取组织页面
        /// </summary>
        /// <param name="PageFilterRequest"></param>
        /// <returns>
        /// OrganizationDTO 的 PageCollection
        /// </returns>
        public async Task<PageCollection<OrganizationDTO>> GetOrganizationPageAsync(PageFilterRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetOrganizationPageAsync execute", $"{this.GetType().Name}"))
            {
                var dbRequest = GetOrgDBPageRequest(request);
                var pageInfos = await _organizationDBRepository.GetPages(dbRequest);
                return pageInfos.MappingTo<PageCollection<OrganizationDTO>>();
            }
        }


        /// <summary>
        /// > 该函数用于将PageFilterRequest对象转换为DBPageRequest对象
        /// </summary>
        /// <param name="PageFilterRequest"></param>
        /// <returns>
        /// 一个 DBPageRequest 对象。
        /// </returns>
        internal DBPageRequest GetOrgDBPageRequest(PageFilterRequest request)
        {
            var customerFilter = new MongoCustomerFilter<MongoTable>();
            var mongoDataItemFilter = new MongoCustomerFilter<MongoDataItem>();
            var keys = request.Filter.Select(f => f.Key);
            if (!keys.Contains("Isinvented"))
            {
                request.Filter.Add("Isinvented", "False");
            }
            foreach (var para in request.Filter)
            {
                if (para.Key == "Isinvented")
                {
                    if (para.Value?.ToLower() == "true")
                    {
                        customerFilter.AppendEq(("BaseData.Isinvented", true));
                    }
                    else
                    {
                        customerFilter.AppendNot(("BaseData.Isinvented", true));
                    }
                }
                else if (para.Key == "Director")
                {
                    customerFilter.AppendArrayEq("BaseData.Directors", para.Value);
                }
                else if (para.Key == "AssignedAdmin")
                {
                    customerFilter.AppendArrayEq("BaseData.AssignedAdmins", para.Value);
                }
                else
                {
                    if (request.IsFuzzy)
                    {
                        mongoDataItemFilter.AppendEq((item => item.Key, para.Key));
                        mongoDataItemFilter.AppendRegex(item => item.Value, para.Value, request.IsFuzzy);
                    }
                    else
                    {
                        mongoDataItemFilter.AppendEq((item => item.Key, para.Key), (item => item.Value, para.Value));
                    }
                    customerFilter.SubFilter(item => item.QueryData.QueryItems, mongoDataItemFilter.AndCombine());
                    mongoDataItemFilter.ClearFilter();
                }
            }
            SortDefinitionBuilder<MongoTable> builderSort = Builders<MongoTable>.Sort;
            SortDefinition<MongoTable> sort = builderSort.Descending("BaseData.CreateTime");
            var dbPageRequest = new DBPageRequest();
            dbPageRequest.CurrentPage = request.CurrentPage;
            dbPageRequest.PageSize = request.PageSize;
            dbPageRequest.Sort = sort;
            dbPageRequest.Filter = customerFilter.AndCombine();
            return dbPageRequest;
        }

        /// <summary>
        /// 它从数据库中删除具有请求中指定的代码的所有组织
        /// </summary>
        /// <param name="RemoveOrganizationsDBRequest">这是传递给服务的请求对象。</param>
        /// <returns>
        /// 删除的记录数。
        /// </returns>
        public async Task<long> RemoveOrganizationsAsync(RemoveOrganizationsDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} RemoveOrganizationsAsync execute", $"{this.GetType().Name}"))
            {
                if (request.OrganizationCodes != null && request.OrganizationCodes.Count > 0)
                {
                    request.OrganizationCodes = request.OrganizationCodes.Where(x => x != "VINNO_FIS").ToList();
                }
                var list = request.OrganizationCodes.Select(f => ("Code", f)).ToList();
                var deviceFilter = new MongoCustomerFilter<MongoTable>();
                deviceFilter.AppendIn("BaseData.DepartmentCode", request.OrganizationCodes);
                var allFindDevice = await _deviceInfoDBRepository.FindAllAsync(deviceFilter.AndCombine());
                if (allFindDevice != null && allFindDevice.Count > 0)
                {
                    foreach (var device in allFindDevice)
                    {
                        device.DepartmentCode = string.Empty;
                        await _deviceInfoDBRepository.UpdateOneAsync("Code", device.Code, device, string.Empty, true);
                    }
                }
                return await _organizationDBRepository.DeleteAllAsync(list);
            }
        }

        /// <summary>
        /// 根据codes列表查询所有子部门科室(包括自身)
        /// </summary>
        /// <param name="codes">组织编码</param>
        /// <returns></returns>
        public async Task<List<string>> SearchNeedOrgByCodesAsync(List<string> codes)
        {
            using (new Performance(_performanceThreshold, "{0} SearchNeedOrgByCodesAsync execute", $"{this.GetType().Name}"))
            {
                if (codes == null || codes.Count == 0)
                {
                    return new List<string>();
                }
                var codeList = codes;
                MongoCustomerFilter<MongoTable> filter = new();
                MongoCustomerFilter<MongoDataItem> subfilter = new();
                subfilter.AppendEq((item => item.Key, "Code"));
                subfilter.AppendIn(item => item.Value, codeList);
                filter.SubFilter(item => item.QueryData.QueryItems, subfilter.AndCombine());
                var datas = await _organizationDBRepository.FindAllAsync(filter.AndCombine());
                var allDataFilter = new Dictionary<string, string>();
                allDataFilter.Add("IsDelete", "False");
                var allDatas = await _organizationDBRepository.FindListAsync(allDataFilter);
                allDatas = allDatas.Where(o => o.OrganizationType != OrganizationTypeEnum.Corporation).ToList();
                foreach (var item in datas)
                {
                    SearchUnCorporationOrgByParentCodes(allDatas, item, ref codeList);
                }
                return codeList;
            }
        }

        /// <summary>
        /// 查询子集（部门科室类型）
        /// </summary>
        /// <param name="organizations">所有非医院数据</param>
        /// <param name="organizationDO"></param>
        /// <param name="organizationDOList"></param>
        private void SearchUnCorporationOrgByParentCodes(List<OrganizationDO> organizations, OrganizationDO organizationDO, ref List<string> organizationCodeList)
        {
            var childList = organizations.Where(o => o.ParentCode == organizationDO.OrganizationCode && o.OrganizationCode != organizationDO.OrganizationCode);
            if (childList != null && childList.Count() > 0)
            {
                organizationCodeList.AddRange(childList.Select(c => c.Code));
                foreach (var item in childList)
                {
                    SearchUnCorporationOrgByParentCodes(organizations, item, ref organizationCodeList);
                }
            }
        }

        /// <summary>
        /// 根据rootcode列表查询所有相关机构
        /// </summary>
        /// <param name="rootCodes">父级编码</param>
        /// <returns></returns>
        public async Task<List<OrganizationDTO>> SearchOrgByRootCodesAsync(List<string> rootCodes)
        {
            using (new Performance(_performanceThreshold, "{0} SearchOrgByRootCodesAsync execute", $"{this.GetType().Name}"))
            {
                if (rootCodes == null)
                {
                    return new List<OrganizationDTO>();
                }
                rootCodes = rootCodes.Where(x => !string.IsNullOrWhiteSpace(x)).ToList();
                if (rootCodes.Count == 0)
                {
                    return new List<OrganizationDTO>();
                }
                MongoCustomerFilter<MongoTable> filter = new();
                MongoCustomerFilter<MongoDataItem> subfilter = new();
                subfilter.AppendEq((item => item.Key, "RootCode"));
                subfilter.AppendIn(item => item.Value, rootCodes);
                filter.SubFilter(item => item.QueryData.QueryItems, subfilter.AndCombine());
                var datas = await _organizationDBRepository.FindAllAsync(filter.AndCombine());
                return datas.MappingTo<List<OrganizationDTO>>();
            }
        }

        /// <summary>
        /// 查询机构信息
        /// </summary>
        /// <param name="keyword">关键字</param>
        /// <param name="parentCode">父级编码</param>
        /// <param name="organizationType">组织类型</param>
        /// <returns></returns>
        public async Task<List<OrganizationDTO>> SearchOrganizationsAsync(string keyword, string parentCode, string parentShortCode, OrganizationTypeEnum organizationType)
        {
            using (new Performance(_performanceThreshold, "{0} SearchOrganizationsAsync execute", $"{this.GetType().Name}"))
            {
                MongoCustomerFilter<MongoTable> filter = new();
                MongoCustomerFilter<MongoDataItem> subfilter = new();
                filter.AppendNot(("BaseData.Isinvented", true));
                if (!string.IsNullOrWhiteSpace(keyword))
                {
                    subfilter.AppendEq((item => item.Key, "OrganizationName"));
                    subfilter.AppendRegex(item => item.Value, keyword, true, false);
                    filter.SubFilter(item => item.QueryData.QueryItems, subfilter.AndCombine());
                    subfilter.ClearFilter();
                }
                if (!string.IsNullOrWhiteSpace(parentCode))
                {
                    subfilter.AppendEq((item => item.Key, "ParentCode"), (item => item.Value, parentCode));
                    filter.SubFilter(item => item.QueryData.QueryItems, subfilter.AndCombine());
                    subfilter.ClearFilter();
                }
                if (!string.IsNullOrWhiteSpace(parentShortCode))
                {
                    var orgInfo = await _organizationDBRepository.FindOneAsync("ShortCode", parentShortCode.ToUpper());
                    if (!string.IsNullOrEmpty(orgInfo?.OrganizationCode))
                    {
                        subfilter.AppendEq((item => item.Key, "ParentCode"), (item => item.Value, orgInfo.OrganizationCode));
                        filter.SubFilter(item => item.QueryData.QueryItems, subfilter.AndCombine());
                        subfilter.ClearFilter();
                    }
                    else
                    {
                        subfilter.AppendEq((item => item.Key, "ParentCode"), (item => item.Value, "NotExist"));
                        filter.SubFilter(item => item.QueryData.QueryItems, subfilter.AndCombine());
                        subfilter.ClearFilter();
                    }
                }
                subfilter.AppendEq((item => item.Key, "OrganizationType"), (item => item.Value, organizationType.ToString()));
                filter.SubFilter(item => item.QueryData.QueryItems, subfilter.AndCombine());
                subfilter.ClearFilter();
                var datas = await _organizationDBRepository.FindAllAsync(filter.AndCombine());
                return datas.MappingTo<List<OrganizationDTO>>();
            }
        }

        /// <summary>
        /// 分页查询机构信息
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageResult<OrganizationDTO>> SearchOrganizationPagesDBAsync(SearchOrganizationsRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} SearchOrganizationPagesDBAsync execute", $"{this.GetType().Name}"))
            {
                var keyword = request.Keyword;
                var parentCode = request.ParentCode;
                var organizationType = (OrganizationTypeEnum)((int)request.OrganizationType);
                MongoCustomerFilter<MongoTable> filter = new();
                MongoCustomerFilter<MongoDataItem> subfilter = new();
                filter.AppendNot(("BaseData.Isinvented", true));
                if (!string.IsNullOrWhiteSpace(keyword))
                {
                    subfilter.AppendEq((item => item.Key, "OrganizationName"));
                    subfilter.AppendRegex(item => item.Value, keyword, true, false);
                    filter.SubFilter(item => item.QueryData.QueryItems, subfilter.AndCombine());
                    subfilter.ClearFilter();
                }
                if (!string.IsNullOrWhiteSpace(parentCode))
                {
                    subfilter.AppendEq((item => item.Key, "ParentCode"), (item => item.Value, parentCode));
                    filter.SubFilter(item => item.QueryData.QueryItems, subfilter.AndCombine());
                    subfilter.ClearFilter();
                }
                subfilter.AppendEq((item => item.Key, "OrganizationType"), (item => item.Value, organizationType.ToString()));
                filter.SubFilter(item => item.QueryData.QueryItems, subfilter.AndCombine());
                subfilter.ClearFilter();
                var dbPage = new DBPageRequest();
                dbPage.CurrentPage = request.PageIndex;
                dbPage.PageSize = request.PageSize;
                dbPage.Filter = filter.AndCombine();
                var datas = await _organizationDBRepository.GetPages(dbPage);
                var result = new PageResult<OrganizationDTO>();
                result.PageIndex = datas.CurrentPage;
                result.PageSize = datas.PageSize;
                result.TotalCount = datas.DataCount;
                result.PageData = datas.PageData.MappingTo<List<OrganizationDTO>>();
                return result;
            }
        }

        private async Task<List<OrganizationDO>> GetOrganizationDOListByKeyword(string keyword)
        {
            var list = new List<OrganizationDO>();
            if (string.IsNullOrWhiteSpace(keyword))
            {
                return list;
            }
            MongoCustomerFilter<MongoTable> filter = new();
            MongoCustomerFilter<MongoDataItem> subfilter = new();
            if (!string.IsNullOrWhiteSpace(keyword))
            {
                subfilter.AppendEq((item => item.Key, "OrganizationName"));
                subfilter.AppendRegex(item => item.Value, keyword, true, false);
                filter.SubFilter(item => item.QueryData.QueryItems, subfilter.AndCombine());
                subfilter.ClearFilter();
            }
            filter.SubFilter(item => item.QueryData.QueryItems, subfilter.AndCombine());
            subfilter.ClearFilter();
            var datas = await _organizationDBRepository.FindAllAsync(filter.AndCombine());
            return datas;
        }

        /// <summary>
        /// 通过 医院类型  找到所有上级机构
        /// </summary>
        /// <param name="organizationCode">当前机构编号</param>
        /// <returns>剔除了当前机构编号的所有上级医院</returns>
        public async Task<List<OrganizationDTO>> FindAllOrganizationByTypeAsync(string organizationCode)
        {
            using (new Performance(_performanceThreshold, "{0} FindAllOrganizationByTypeAsync execute", $"{this.GetType().Name}"))
            {
                var dic = new Dictionary<string, string>();
                dic.Add("OrganizationType", OrganizationTypeEnum.Corporation.ToString());
                var datas = await _organizationDBRepository.FindListAsync(dic);
                var doList = datas.Where(w => w.Code.ToString() != organizationCode && !w.Isinvented).ToList();
                return doList.MappingTo<List<OrganizationDTO>>();
            }
        }

        /// <summary>
        /// 查询机构列表
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<List<OrganizationDTO>> GetOrganizationListAsync(GetOrganizationListDBRequest request)
        {
            MongoCustomerFilter<MongoTable> customerFilter = new();
            if (request.OrganizationType != null)
            {
                customerFilter.AppendIn("BaseData.OrganizationType", new List<int> { (int)request.OrganizationType });
            }
            if (request.PatientType != null)
            {
                customerFilter.AppendIn("BaseData.PatientType", new List<int> { (int)request.PatientType });
            }
            if (request.Isinvented != null)
            {
                var boolString = request.Isinvented == true ? bool.TrueString : bool.FalseString;
                customerFilter.AppendIn("BaseData.Isinvented", new List<bool> { request.Isinvented ?? false });
            }
            var filter = customerFilter.AndCombine();
            var entities = await _organizationDBRepository.FindAllAsync(filter);
            return entities.MappingTo<List<OrganizationDTO>>();
        }

        /// <summary>
        /// 保存机构信息
        /// </summary>
        /// <param name="id">Id</param>
        /// <param name="model">机构基础信息对象</param>
        /// <returns>是否成功</returns>
        public async Task<bool> SaveOrganizationAsync(string code, OrganizationDTO model, bool canNull = false, bool modifyReferralOrgs = false)
        {
            using (new Performance(_performanceThreshold, "{0} SaveOrganizationAsync execute", $"{this.GetType().Name}"))
            {
                // if (code == "VINNO_FIS")
                // {
                //     return false;
                // }
                var organizationDO = await _organizationDBRepository.FindOneAsync("Code", code);
                if (organizationDO != null && !string.IsNullOrWhiteSpace(organizationDO.Code))
                {
                    var data = model.MappingTo<OrganizationDO>();
                    if (!string.IsNullOrEmpty(model.LogoUrl))
                    {
                        data.LogoUrl = model.LogoUrl.ToUrlToken();
                        //防止ReferralOrganizationCodes被AutoMapping自动排序
                        data.ReferralOrganizationCodes = model.ReferralOrganizationCodes;
                    }
                    if (!modifyReferralOrgs)
                    {
                        data.ReferralOrganizationCodes = organizationDO.ReferralOrganizationCodes;
                    }
                    if (!string.IsNullOrEmpty(model.DefaultReportTemplateCode))
                    {
                        data.DefaultReportTemplateCode = model.DefaultReportTemplateCode;
                    }
                    if (!string.IsNullOrEmpty(model.DefaultThesaurusCode))
                    {
                        data.DefaultThesaurusCode = model.DefaultThesaurusCode;
                    }
                    var res = await _organizationDBRepository.UpdateOneAsync("Code", code, data, string.Empty, canNull);
                    CacheMaintenance.Instance.Get<IOrganizationsManager>().Remove(code);
                    return res;
                }
                return false;
            }
        }

        private async Task<List<OrganizationDO>> FindOrganizationDOListByCodesAsync(List<string> organizationCodes)
        {
            MongoCustomerFilter<MongoTable> filter = new();
            filter.AppendQueryItemInFilter(MongoCombineTypeEnum.Or, "Code", organizationCodes);
            //查询所有指定的Organization
            var entities = await _organizationDBRepository.FindAllAsync(filter.AndCombine());
            return entities;
        }

        public async Task<List<OrganizationDTO>> FindOrganizationDTOListByCodesAsync(List<string> organizationCodes)
        {
            if (organizationCodes == null || organizationCodes.Count == 0)
            {
                return new List<OrganizationDTO>();
            }
            MongoCustomerFilter<MongoTable> filter = new();
            filter.AppendQueryItemInFilter(MongoCombineTypeEnum.Or, "Code", organizationCodes);
            //查询所有指定的Organization
            var entities = await _organizationDBRepository.FindAllAsync(filter.AndCombine());
            var result = entities.MappingTo<List<OrganizationDTO>>();
            return result;
        }

        /// <summary>
        /// 根据机构codes获取机构
        /// </summary>
        /// <param name="organizationCodes"></param>
        /// <returns></returns>
        public async Task<List<OrganizationDTO>> FindOrganizationListByCodesAsync(List<string> organizationCodes)
        {
            using (new Performance(_performanceThreshold, "{0} FindOrganizationListByCodesAsync execute", $"{this.GetType().Name}"))
            {
                if (organizationCodes == null || organizationCodes.Count == 0)
                {
                    return new List<OrganizationDTO>();
                }
                MongoCustomerFilter<MongoTable> filter = new();
                filter.AppendQueryItemInFilter(MongoCombineTypeEnum.Or, "Code", organizationCodes);
                //查询所有指定的Organization
                var entities = await _organizationDBRepository.FindAllAsync(filter.AndCombine());
                //查询所有指定的Organization的上级组织编码
                var parentOrganizationCodes = entities.Select(f => f.ParentCode).ToList();
                //可转诊医院
                var parentEntities = await GetParentEntities(parentOrganizationCodes);
                //转诊超时时间
                if (entities.Any(x => x.ReferralLimitHours <= 0))
                {
                    var settingDO = await GetReferralRecordSettingAsync(new GetReferralRecordSettingDBRequest { });
                    foreach (var org in entities)
                    {
                        if (org.ReferralLimitHours <= 0)
                        {
                            org.ReferralLimitHours = settingDO.LimitHours;
                        }
                    }
                }
                var dtos = entities.MappingTo<List<OrganizationDTO>>();

                foreach (var dto in dtos)
                {
                    var thisOrg = entities.FirstOrDefault(x => x.Code == dto.OrganizationCode);
                    if (thisOrg.ReferralOrganizationCodes == null)
                    {
                        var parentOrganizations = GetParentOrganizations(parentEntities, thisOrg);
                        dto.ReferralOrganizationCodes = parentOrganizations.Select(x => x.Code).ToList();
                        dto.ReferralOrganizations = parentOrganizations.Select(x => new ReferralOrganizationDTO { OrganizationCode = x.OrganizationCode, OrganizationName = x.OrganizationName }).ToList();

                    }
                    else
                    {
                        var referralOrgList = entities.Where(x => thisOrg.ReferralOrganizationCodes.Contains(x.Code))?.ToList() ?? new List<OrganizationDO>();
                        dto.ReferralOrganizationCodes = referralOrgList.Select(x => x.Code).ToList();
                        dto.ReferralOrganizations = referralOrgList.Select(x => new ReferralOrganizationDTO { OrganizationCode = x.OrganizationCode, OrganizationName = x.OrganizationName }).ToList();
                    }
                }

                return dtos;
            }
        }

        private async Task GetReferralOrganizations(OrganizationDO organizationDO, OrganizationDTO dto)
        {
            if (organizationDO.ReferralOrganizationCodes == null)
            {
                var referralOrganizations = await GetParentCorporationsAsync(new FindParentCorporationsDBRequest
                {
                    OrganizationCode = organizationDO.Code
                });
                dto.ReferralOrganizationCodes = referralOrganizations.Select(x => x.OrganizationCode).ToList();
                dto.ReferralOrganizations = referralOrganizations.Select(x => new ReferralOrganizationDTO { OrganizationCode = x.OrganizationCode, OrganizationName = x.OrganizationName }).ToList();
            }
            else if (organizationDO.ReferralOrganizationCodes.Any())
            {
                var dic = new Dictionary<string, List<string>>();
                dic.Add("Code", organizationDO.ReferralOrganizationCodes);
                var referralOrganizations = await _organizationDBRepository.FindInAsync(dic);
                dto.ReferralOrganizationCodes = referralOrganizations.Select(x => x.Code).ToList();
                dto.ReferralOrganizations = referralOrganizations.Select(x => new ReferralOrganizationDTO { OrganizationCode = x.Code, OrganizationName = x.OrganizationName }).ToList();
            }
        }

        /// <summary>
        /// 更新组织名称
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> UpdateOrganizationNameAsync(UpdateOrganizationNameDBRequest request)
        {
            if (request.OrganizationCode == "VINNO_FIS")
            {
                return false;
            }
            using (new Performance(_performanceThreshold, "{0} UpdateOrganizationNameAsync execute", $"{this.GetType().Name}"))
            {
                var result = await _organizationDBRepository.UpdateOrganizationName(request);
                CacheMaintenance.Instance.Get<IOrganizationsManager>().Remove(request.OrganizationCode);
                return result;
            }
        }

        /// <summary>
        /// 后台管理网站通过已分配管理员获取医院
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageCollection<OrganizationDTO>> GetManageOrganizationPageAsync(GetManageOrganizationDBPageRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetManageOrganizationPageAsync execute", $"{this.GetType().Name}"))
            {
                List<FilterDefinition<MongoDataItem>> queryItemsFilterList = new();
                List<FilterDefinition<MongoTable>> customerFilterList = new();
                List<FilterDefinition<MongoTable>> tempCustomerFilterList = new();
                MongoCustomerFilter<MongoTable> customerFilter = new();
                MongoCustomerFilter<MongoDataItem> queryItemsFilter = new();
                if (request.AssignedAdmins != null && request.AssignedAdmins.Count > 0)
                {
                    customerFilter.AppendArrayEq("BaseData.AssignedAdmins", request.AssignedAdmins.FirstOrDefault());
                }
                foreach (var kv in request.Filter)
                {
                    queryItemsFilter = new();
                    if (kv.Key == "Isinvented" && !string.IsNullOrWhiteSpace(kv.Value))
                    {
                        if (kv.Value?.ToLower() == "true")
                        {
                            customerFilter.AppendEq(("BaseData.Isinvented", true));
                        }
                        else
                        {
                            customerFilter.AppendNot(("BaseData.Isinvented", true));
                        }
                    }
                    else if (!string.IsNullOrWhiteSpace(kv.Value))
                    {
                        if (kv.Key == "OrganizationName")
                        {
                            List<AdminInfoDO> adminList = await _adminDBRepository.FindAdminsByNameAsync(kv.Value);
                            if (adminList.Any())
                            {
                                var tempCustomerFilter = new MongoCustomerFilter<MongoTable>();
                                tempCustomerFilter.AppendAnyIn("BaseData.AssignedAdmins", adminList.Select(x => x.AdminCode).ToList());
                                tempCustomerFilterList.Add(tempCustomerFilter.AndCombine());
                            }
                            if (!string.IsNullOrWhiteSpace(kv.Key) && !string.IsNullOrWhiteSpace(kv.Value))
                            {
                                MongoCustomerFilter<MongoDataItem> tempQueryItemsFilter = new();
                                tempQueryItemsFilter.AppendEq((item => item.Key, kv.Key));
                                tempQueryItemsFilter.AppendRegex(item => item.Value, kv.Value, true, false);
                                queryItemsFilterList.Add(tempQueryItemsFilter.AndCombine());

                                tempQueryItemsFilter = new();
                                tempQueryItemsFilter.AppendEq((item => item.Key, "ShortCode"));
                                tempQueryItemsFilter.AppendEq((item => item.Value, kv.Value.ToUpper()));
                                queryItemsFilterList.Add(tempQueryItemsFilter.AndCombine());
                            }
                            if (queryItemsFilterList.Count > 0)
                            {
                                var tempCustomerFilter = new MongoCustomerFilter<MongoTable>();
                                var queryItemAllFilter = queryItemsFilter.OrCombine(queryItemsFilterList.ToArray());
                                tempCustomerFilter.SubFilter(item => item.QueryData.QueryItems, queryItemAllFilter);
                                tempCustomerFilterList.Add(tempCustomerFilter.AndCombine());
                            }
                        }
                        else
                        {
                            if (!string.IsNullOrWhiteSpace(kv.Key) && !string.IsNullOrWhiteSpace(kv.Value))
                            {
                                queryItemsFilter.AppendEq((item => item.Key, kv.Key));
                                queryItemsFilter.AppendRegex(item => item.Value, kv.Value, true, false);
                                if (queryItemsFilter.HasFilter)
                                    customerFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
                            }
                        }
                    }
                }
                if (tempCustomerFilterList.Count > 0)
                {
                    var tempCustomerFilter = new MongoCustomerFilter<MongoTable>();
                    customerFilterList.Add(tempCustomerFilter.OrCombine(tempCustomerFilterList.ToArray()));
                }
                PageCollection<OrganizationDTO> result = new PageCollection<OrganizationDTO>();
                result.CurrentPage = request.CurrentPage;
                result.PageSize = request.PageSize;
                result.DataCount = (int)await _organizationDBRepository.CountDocumentsAsync(customerFilter.AndCombine(customerFilterList.ToArray()));
                SortDefinitionBuilder<MongoTable> builderSort = Builders<MongoTable>.Sort;
                SortDefinition<MongoTable> sort = builderSort.Descending("BaseData.CreateTime");
                var orgInfoList = await _organizationDBRepository.FindAllAsync(customerFilter.AndCombine(customerFilterList.ToArray()),
                new FindOptions<MongoTable> { NoCursorTimeout = true, Skip = (request.CurrentPage - 1) * request.PageSize, Limit = request.PageSize, Sort = sort });
                if (orgInfoList != null && orgInfoList.Count > 0)
                {
                    result.PageData = orgInfoList.Select(v => v.MappingTo<OrganizationDTO>()).ToList();
                }
                return result;
            }
        }

        /// <summary>
        /// 查询上级医院集合
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<List<OrganizationDTO>> GetParentCorporationsAsync(FindParentCorporationsDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetParentCorporationsAsync execute", $"{this.GetType().Name}"))
            {
                MongoCustomerFilter<MongoTable> filter = new();
                //查询所有指定的Organization的上级组织编码
                var entity = await _organizationDBRepository.FindOneAsync("Code", request.OrganizationCode);
                //可转诊医院
                var parentEntities = await GetParentEntities(new List<string> { entity.ParentCode });
                return parentEntities.MappingTo<List<OrganizationDTO>>();
            }
        }

        /// <summary>
        /// 查询父级机构
        /// </summary>
        /// <param name="parentCodes"></param>
        /// <returns></returns>
        private async Task<List<OrganizationDO>> GetParentEntities(List<string> parentCodes)
        {
            MongoCustomerFilter<MongoTable> filter = new();
            //父级机构
            var parentEntities = new List<OrganizationDO>();
            var allOrganizationCodes = parentCodes;
            var parentOrganizationCodes = parentCodes;
            while (parentOrganizationCodes.Count(f => !string.IsNullOrWhiteSpace(f)) > 0)
            {
                filter.AppendQueryItemInFilter(MongoCombineTypeEnum.Or, "Code", parentOrganizationCodes);
                filter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And
                , ("OrganizationType", OrganizationTypeEnum.Corporation.ToString())
                , ("Isinvented", Boolean.FalseString));
                var newParentEntities = await _organizationDBRepository.FindAllAsync(filter.AndCombine());
                newParentEntities = newParentEntities.Where(x => !allOrganizationCodes.Contains(x.ParentCode)).ToList();
                if (newParentEntities == null || newParentEntities.Count == 0)
                {
                    break;
                }
                allOrganizationCodes.AddRange(newParentEntities.Select(f => f.ParentCode));
                parentEntities.AddRange(newParentEntities);
                parentOrganizationCodes = newParentEntities.Select(f => f.ParentCode).ToList();
            }
            return parentEntities;
        }

        private List<OrganizationDO> GetParentOrganizations(List<OrganizationDO> source, OrganizationDO thisOrg)
        {
            if (!string.IsNullOrWhiteSpace(thisOrg.ParentCode))
            {
                var parOrg = source.FirstOrDefault(x => x.Code == thisOrg.ParentCode);
                if (parOrg != null)
                {
                    var parList = GetParentOrganizations(source, parOrg);
                    parList.Add(parOrg);
                    return parList;
                }
            }
            return new List<OrganizationDO>();
        }

        /// <summary>
        /// 查询管理员关联机构集合
        /// </summary>
        /// <param name="adminCode"></param>
        /// <returns></returns>
        private async Task<IList<OrganizationDO>> GetAdminOrganizationsAsync(string adminCode)
        {
            MongoCustomerFilter<MongoTable> customerFilter = new();
            customerFilter.AppendArrayEq("BaseData.AssignedAdmins", adminCode);
            var entities = await _organizationDBRepository.FindAllAsync(customerFilter.AndCombine(), new FindOptions<MongoTable> { NoCursorTimeout = true });
            return entities;
        }

        /// <summary>
        /// CMS首页假数据模块列表请求
        /// </summary>
        /// <param name="organizationCode"></param>
        /// <returns></returns>
        public async Task<StatisticDetailSettingDTO> GetCMSFakeDataAsync(string organizationCode)
        {
            using (new Performance(_performanceThreshold, "{0} GetCMSFakeDataAsync execute", $"{this.GetType().Name}"))
            {
                var organizationDO = await _organizationDBRepository.FindOneAsync("Code", organizationCode);
                if (organizationDO != null)
                {
                    var result = new StatisticDetailSettingDTO();
                    result.CMSTemplateCode = organizationDO.CMSTemplateCode;
                    return result;
                }
            }
            return new StatisticDetailSettingDTO();
        }

        /// <summary>
        /// CMS首页假数据模块列表请求
        /// </summary>
        /// <param name="organizationCode"></param>
        /// <returns></returns>
        public async Task<bool> UpdateCMSFakeDataAsync(UpdateCMSFakeDataDBRequest updateCMSFakeDataDBRequest)
        {
            var code = updateCMSFakeDataDBRequest.OrganizationCode;
            using (new Performance(_performanceThreshold, "{0} UpdateCMSFakeDataAsync execute", $"{this.GetType().Name}"))
            {
                var organizationDO = await _organizationDBRepository.FindOneAsync("Code", code);
                if (organizationDO != null)
                {
                    organizationDO.CMSTemplateCode = updateCMSFakeDataDBRequest.CMSTemplateCode;
                    return await _organizationDBRepository.UpdateOneAsync("Code", code, organizationDO, string.Empty, true);
                }
            }
            return false;
        }

        /// <summary>
        /// 批量新增机构数据
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>是否成功</returns>
        public async Task<bool> BatchInsertOrganizationDataDBAsync(BatchInsertOrganizationRequest request)
        {
            bool res = false;
            if (request?.OrganizationMigratorys?.Count > 0)
            {
                //找到相同的
                var ids = request.OrganizationMigratorys.Select(c => c.OrganizationCode).Distinct().ToList();
                var dic = new Dictionary<string, List<string>>();
                dic.Add("Code", ids);
                var organizationList = await _organizationDBRepository.FindInAsync(dic);
                //求同存异 
                var moduleList = await GetAllDiagnosisModuleList();
                if (organizationList?.Count > 0)
                {
                    var existCodes = organizationList.Select(c => c.Code).Distinct().ToList();
                    var updateData = request.OrganizationMigratorys.Where(c => existCodes.Contains(c.OrganizationCode));
                    var insertData = request.OrganizationMigratorys.Where(c => !existCodes.Contains(c.OrganizationCode));
                    if (insertData?.Count() > 0)
                    {
                        //新增
                        var names = insertData.Where(c => !string.IsNullOrEmpty(c.OrganizationName)).Select(c => c.OrganizationName).Distinct().ToList();
                        dic = new Dictionary<string, List<string>>();
                        dic.Add("OrganizationName", names);
                        var nameList = await _organizationDBRepository.FindInAsync(dic);
                        if (nameList?.Count > 0)
                        {
                            var existNames = nameList.Select(c => c.OrganizationName).Distinct().ToList();
                            var updateNameData = insertData.Where(c => existNames.Contains(c.OrganizationName));
                            var insertNameData = insertData.Where(c => !existNames.Contains(c.OrganizationName));
                            if (insertNameData?.Count() > 0)
                            {
                                res = await BatchAddOrganizationData(insertNameData, moduleList, request.SyncType);
                            }
                            if (updateNameData?.Count() > 0)
                            {
                                Logger.WriteLineWarn($"OrganizationDBService BatchInsertOrganizationDataDBAsync Repeat Name, insertData: {JsonConvert.SerializeObject(updateNameData)}");
                            }
                        }
                        else
                        {
                            res = await BatchAddOrganizationData(insertData, moduleList, request.SyncType);
                        }
                    }
                    if (updateData?.Count() > 0)
                    {
                        var req = updateData.Select(f =>
                        {
                            var updateEntity = f.MappingTo<OrganizationDO>();
                            updateEntity.Code = f.OrganizationCode.Trim();
                            updateEntity.IsDelete = f.IsDelete;
                            updateEntity.IsEncryptedShow = false;
                            //updateEntity.DefaultReportTemplateCode = f.DefaultTemplateId;
                            var findOrg = organizationList.FirstOrDefault(c => c.OrganizationCode == updateEntity.Code);
                            if (!string.IsNullOrEmpty(findOrg?.ShortCode))
                            {
                                updateEntity.ShortCode = findOrg.ShortCode;
                            }
                            updateEntity.DiagnosisModules = DealWithDiagnosisModules(moduleList, f.DiagnosisModule);
                            if (request.SyncType == SyncDBEnum.Synchronize)
                            {
                                res = _organizationDBRepository.UpdateOneForSyncDBAsync("Code", updateEntity.Code, updateEntity, null).Result;
                            }
                            return updateEntity;
                        }).ToList();
                        if (request.SyncType == SyncDBEnum.Migrate)
                        {
                            res = await _organizationDBRepository.UpdateAllWithoutOplogLogAsync(req);
                        }
                        foreach (var item in updateData)
                        {
                            CacheMaintenance.Instance.Get<IOrganizationsManager>().Remove(item.OrganizationCode);
                        }
                    }
                }
                else
                {
                    //新增
                    var names = request.OrganizationMigratorys.Where(c => !string.IsNullOrEmpty(c.OrganizationName)).Select(c => c.OrganizationName).Distinct().ToList();
                    dic = new Dictionary<string, List<string>>();
                    dic.Add("OrganizationName", names);
                    var nameList = await _organizationDBRepository.FindInAsync(dic);
                    if (nameList?.Count > 0)
                    {
                        var existNames = nameList.Select(c => c.OrganizationName).Distinct().ToList();
                        var updateNameData = request.OrganizationMigratorys.Where(c => existNames.Contains(c.OrganizationName));
                        var insertNameData = request.OrganizationMigratorys.Where(c => !existNames.Contains(c.OrganizationName));
                        if (insertNameData?.Count() > 0)
                        {
                            res = await BatchAddOrganizationData(insertNameData, moduleList, request.SyncType);
                        }
                        if (updateNameData?.Count() > 0)
                        {
                            Logger.WriteLineWarn($"OrganizationDBService BatchInsertOrganizationDataDBAsync Repeat Name, OrganizationMigratorys: {JsonConvert.SerializeObject(updateNameData)}");
                        }
                    }
                    else
                    {
                        res = await BatchAddOrganizationData(request.OrganizationMigratorys, moduleList, request.SyncType);
                    }
                }

                foreach (var item in request.OrganizationMigratorys)
                {
                    if (!string.IsNullOrEmpty(item.DefaultTemplateId))
                    {
                        if (item.ReportTemplateIds?.Count > 0)
                        {
                            if (!item.ReportTemplateIds.Contains(item.DefaultTemplateId))
                            {
                                item.ReportTemplateIds.Add(item.DefaultTemplateId);
                            }
                            else
                            {
                                //包含。不处理
                            }
                        }
                        else
                        {
                            item.ReportTemplateIds = new List<string>() { item.DefaultTemplateId };
                        }
                    }
                    if (item.ReportTemplateIds?.Count > 0)
                    {
                        //获取所有的报告数据
                        var ReportTemplateIdsDic = new Dictionary<string, List<string>>();
                        ReportTemplateIdsDic.Add("Code", item.ReportTemplateIds.ToList());
                        var entities = await _reportTemplateDBRepository.FindInAsync(ReportTemplateIdsDic);
                        if (entities?.Count > 0)
                        {
                            entities = entities.Where(c => c.ReportTemplateType == ReportTemplateTypeEnum.Default)?.ToList() ?? new List<ReportTemplateDO>();
                        }
                        foreach (var idItem in item.ReportTemplateIds)
                        {
                            // var findTemplate = entities.FirstOrDefault(c => c.Code == idItem) ?? new ReportTemplateDO();
                            // if (!string.IsNullOrEmpty(findTemplate?.Code))
                            // {
                            //     findTemplate.IsDefault = true;
                            //     findTemplate.ReportTemplateStateType = ReportTemplateStatusTypeEnum.Published;
                            //     findTemplate.OrganizationCode = item.OrganizationCode;
                            //     var res1 = await _reportTemplateDBRepository.UpdateOneAsync("Code", findTemplate.ReportTemplateCode, findTemplate, string.Empty, true);
                            // }
                            //检查是否存在
                            var reportDic = new Dictionary<string, string>();
                            reportDic.Add("ReferenceReportTemplateCode", idItem);
                            reportDic.Add("OrganizationCode", item.OrganizationCode.Trim());
                            reportDic.Add("ReportTemplateType", ReportTemplateTypeEnum.Organization.ToString());
                            var reportTemplateList = await _reportTemplateDBRepository.FindListAsync(reportDic);
                            if (reportTemplateList?.Count > 0)
                            {
                                //找到了。不更新
                                var findTemplate = reportTemplateList.FirstOrDefault();
                                if (!string.IsNullOrEmpty(findTemplate?.Code))
                                {
                                    findTemplate.IsDefault = true;
                                    findTemplate.ReportTemplateStateType = ReportTemplateStatusTypeEnum.Published;
                                    findTemplate.OrganizationCode = item.OrganizationCode;
                                    var res1 = await _reportTemplateDBRepository.UpdateOneAsync("Code", findTemplate.ReportTemplateCode, findTemplate, string.Empty, true);
                                    //更新机构默认
                                    if (item.DefaultTemplateId == findTemplate.ReferenceReportTemplateCode)
                                    {
                                        var orgInfo = await _organizationDBRepository.FindOneAsync("Code", item.OrganizationCode);
                                        if (!string.IsNullOrEmpty(orgInfo?.Code))
                                        {
                                            orgInfo.DefaultReportTemplateCode = findTemplate.Code;
                                            var res2 = await _organizationDBRepository.UpdateOneAsync("Code", orgInfo.Code, orgInfo, string.Empty);
                                        }
                                    }
                                }
                            }
                            else
                            {
                                var findEntity = entities?.FirstOrDefault(d => d.ReportTemplateCode == idItem) ?? new ReportTemplateDO();
                                if (!string.IsNullOrEmpty(findEntity.Code))
                                {
                                    //新增
                                    var dbRequest = new AddReportTemplateDBRequest();
                                    dbRequest.Data = new WingInterfaceLibrary.DTO.ReportTemplate.ReportTemplateDTO();
                                    dbRequest.Data.IsDefault = true;
                                    dbRequest.Data.ReportTemplateName = findEntity.ReportTemplateName;
                                    dbRequest.Data.ReportTemplateStateType = ReportTemplateStatusTypeEnum.Published;
                                    dbRequest.Data.ReferenceReportTemplateCode = findEntity.Code;
                                    if (findEntity.TemplatePreviewList?.Count > 0)
                                    {
                                        dbRequest.Data.TemplatePreviewList = findEntity.TemplatePreviewList.MappingTo<List<WingInterfaceLibrary.DTO.Report.ReportPreviewDTO>>();
                                    }
                                    dbRequest.Data.ReportTemplateJson = findEntity.ReportTemplateJson ?? "{}";
                                    dbRequest.Data.ReportDatasJson = findEntity.ReportDatasJson ?? "{}";
                                    dbRequest.Data.ReportTemplateType = ReportTemplateTypeEnum.Organization;
                                    dbRequest.Data.LanguageCode = findEntity.LanguageCode;
                                    dbRequest.Data.OrganizationCode = item.OrganizationCode.Trim();
                                    dbRequest.Data.ReportTemplateUsageType = findEntity.ReportTemplateUsageType;
                                    var resp = await AddReportTemplateAsync(dbRequest);
                                    //更新机构默认
                                    if (resp != null && item.DefaultTemplateId == dbRequest.Data.ReferenceReportTemplateCode)
                                    {
                                        var orgInfo = await _organizationDBRepository.FindOneAsync("Code", item.OrganizationCode);
                                        if (!string.IsNullOrEmpty(orgInfo?.Code))
                                        {
                                            orgInfo.DefaultReportTemplateCode = resp.ReportTemplateCode;
                                            var res2 = await _organizationDBRepository.UpdateOneAsync("Code", orgInfo.Code, orgInfo, string.Empty);
                                        }
                                    }
                                }
                                else
                                {
                                    Logger.WriteLineWarn($"OrganizationDBService BatchInsertOrganizationDataDBAsync ReportTemplateSyn Error, ReportTemplateCode: {idItem}");
                                }
                            }
                        }
                    }
                }
            }
            return res;
        }

        /// <summary>
        /// 批量新增机构数据
        /// </summary>
        private async Task<bool> BatchAddOrganizationData(IEnumerable<OrganizationMigratoryInfo> insertData, IList<DiagnosisModuleDO> moduleList, SyncDBEnum syncType)
        {
            var req = insertData.Select(
                f =>
                {
                    var data = f.MappingTo<OrganizationDO>();
                    data.Code = f.OrganizationCode.Trim();
                    data.IsDelete = f.IsDelete;
                    data.IsEncryptedShow = false;
                    // data.DefaultReportTemplateCode = f.DefaultTemplateId;
                    data.DiagnosisModules = DealWithDiagnosisModules(moduleList, f.DiagnosisModule);

                    var shortCode = "";
                    for (var i = 0; i < 20; i++)
                    {
                        shortCode = GeneralShortCode();
                        if (!string.IsNullOrWhiteSpace(shortCode))
                        {
                            var orgInfo = _organizationDBRepository.FindOneAsync("ShortCode", shortCode).Result;
                            if (orgInfo == null || string.IsNullOrWhiteSpace(orgInfo.Code))
                            {
                                break;
                            }
                        }
                        shortCode = string.Empty;
                    }
                    data.ShortCode = shortCode;
                    if (syncType == SyncDBEnum.Synchronize)
                    {
                        var r = _organizationDBRepository.InsertOneForAsyncDBAsync(data, new Dictionary<string, string>(), "").Result;
                    }
                    return data;
                }
            ).ToList();
            var res = true;
            if (syncType == SyncDBEnum.Migrate)
            {
                res = await _organizationDBRepository.InsertAllWithoutOplogLogAsync(req);
            }
            return res;
        }

        /// <summary>
        /// 批量新增医联体
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>是否成功</returns>
        public async Task<bool> BatchInsertMedicalConsortiumDBAsync(BatchInsertMedicalConsortiumRequest request)
        {
            try
            {
                var addRequest = new List<(OrganizationDO organizationDO, Dictionary<string, string> platformDatas, string extensionData)>();
                var updateRequest = new List<(string key, string value, OrganizationDO data, string extensionData)>();
                List<string> orgCodes = request.MedicalConsortiumDetails?.Select(x => x.Code).ToList();
                if (orgCodes != null && orgCodes.Count > 0)
                {
                    var dic = new Dictionary<string, List<string>>();
                    dic.Add("Code", orgCodes);
                    var organizationList = await _organizationDBRepository.FindInAsync(dic);
                    var existsCodes = organizationList.Select(x => x.Code);
                    foreach (var item in request.MedicalConsortiumDetails)
                    {
                        var organization = organizationList.FirstOrDefault(x => x.Code == item.Code);
                        if (organization == null || string.IsNullOrWhiteSpace(organization.Code))
                        {
                            organization = new OrganizationDO();
                            organization.Code = item.Code;
                            organization.OrganizationName = item.Name;
                            organization.RootCode = item.Code;
                            organization.State = OrganizationStateEnum.Audited;
                            organization.PatientType = item.PatientType;
                            addRequest.Add((organization, new Dictionary<string, string>(), ""));
                        }
                        else
                        {
                            updateRequest.Add((organization.Code, organization.Code, organization, ""));
                        }
                    }
                }
                if (addRequest.Count > 0)
                {
                    await _organizationDBRepository.InsertAllAsync(addRequest);
                }
                if (updateRequest.Count > 0)
                {
                    await _organizationDBRepository.UpdateAllAsync(updateRequest);
                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn("SyncBatchRemedicalRecordsDBAsync," + ex);
            }
            return true;
        }

        /// <summary>
        /// 取该医院组织的子孙医院
        /// </summary>
        /// <param name="organizations">全部组织</param>
        /// <param name="organizationCode">当前组织Code</param>
        /// <param name="needOrganizations">全部子孙组织</param>
        /// <returns></returns>
        public async Task<List<OrganizationDTO>> GetAllChildOrganizations(string rootOrganizationCode)
        {
            var org = await _organizationDBRepository.FindOneAsync("Code", rootOrganizationCode);
            var needOrganizations = new List<OrganizationDO>();
            GetAllChildOrganizationDO(org, ref needOrganizations);
            var result = needOrganizations.MappingTo<List<OrganizationDTO>>();
            return result;
        }

        /// <summary>
        /// 取该医院组织的子孙医院
        /// </summary>
        /// <param name="organizations">全部组织</param>
        /// <param name="organizationCode">当前组织Code</param>
        /// <param name="needOrganizations">全部子孙组织</param>
        /// <returns></returns>
        internal void GetAllChildOrganizationDO(OrganizationDO organization, ref List<OrganizationDO> needOrganizations)
        {
            var organizationCode = organization.OrganizationCode;
            //查到父级为此机构的机构列表
            var organizationFilter = new Dictionary<string, string>();
            organizationFilter.Add("ParentCode", organizationCode);
            organizationFilter.Add("OrganizationType", OrganizationTypeEnum.Corporation.ToString());

            var filter = new MongoCustomerFilter<MongoTable>();
            MongoCustomerFilter<MongoDataItem> subfilter = new();
            subfilter.AppendEq((item => item.Key, "ParentCode"), (item => item.Value, organizationCode));
            filter.SubFilter(item => item.QueryData.QueryItems, subfilter.AndCombine());
            subfilter.ClearFilter();

            subfilter.AppendEq((item => item.Key, "OrganizationType"), (item => item.Value, OrganizationTypeEnum.Corporation.ToString()));
            filter.SubFilter(item => item.QueryData.QueryItems, subfilter.AndCombine());
            subfilter.ClearFilter();
            var organizations = _organizationDBRepository.FindAllAsync(filter.AndCombine()).Result;
            var childList = organizations.Where(o => o.ParentCode == organizationCode && o.OrganizationCode != organizationCode);
            if (childList != null && childList.Count() > 0)
            {
                var intersectedList = needOrganizations.Select(n => n.OrganizationCode).Intersect(childList.Select(c => c.OrganizationCode));
                //避免循环设置上级：a->b->c->a
                if (intersectedList.Count() == 0)
                {
                    needOrganizations.AddRange(childList);
                    foreach (var item in childList)
                    {
                        GetAllChildOrganizationDO(item, ref needOrganizations);
                    }
                }

            }
        }

        /// <summary>
        /// 根据用户和设备列表获取组织
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageCollection<OrganizationSimplifyDTO>> GetOrganizationByUserAndDevicesDBAsync(GetOrganizationByUserAndDevicesDBRequest request)
        {
            bool isIncloudReferral = request.IsIncloudReferral;//是否包含转诊申请机构
            var organizationList = new List<string>();
            var userInfo = await FindUserByCodeAsync(request.UserCode);
            if (userInfo != null)
            {
                organizationList.Add(userInfo.OrganizationCode);
            }
            if (request.OrgCodes != null && request.OrgCodes.Count > 0)
            {
                var orgDevices = await FindDeviceInfoListByOrganizationCodesAsync(request.OrgCodes.Distinct().ToList());
                if (orgDevices != null && orgDevices.Count > 0)
                {
                    foreach (var device in orgDevices)
                    {
                        request.BindDeviceCodes.Add(device.DeviceCode);
                    }
                }
            }
            if (request.BindDeviceCodes != null && request.BindDeviceCodes.Count > 0)
            {
                var deviceInfoList = await FindDeviceInfoListByCodeListAsync(request.BindDeviceCodes.Distinct().ToList());
                if (deviceInfoList != null && deviceInfoList.Count > 0)
                {
                    foreach (var device in deviceInfoList)
                    {
                        organizationList.Add(device.OrganizationCode);
                    }
                }
            }
            if (organizationList.Count == 0)
            {
                return new PageCollection<OrganizationSimplifyDTO>
                {
                    CurrentPage = request.CurrentPage,
                    PageSize = request.PageSize,
                    DataCount = 0,
                    PageData = new List<OrganizationSimplifyDTO>()
                };
            }
            organizationList = organizationList.Distinct().ToList();
            var customerFilter = new MongoCustomerFilter<MongoTable>();
            if (!string.IsNullOrWhiteSpace(request.Keyword))
            {
                customerFilter.AppendQueryItemRegexFilter(MongoCombineTypeEnum.And, ("OrganizationName", request.Keyword, true, false));
            }
            if (isIncloudReferral)
            {
                //转诊申请机构   机构负责人、普通员工
                List<ReferralDataEntity> referralDatas = new List<ReferralDataEntity>();
                if (userInfo.IsDirector)
                {
                    FilterDefinition<ReferralDataEntity> referralDataFliter = Builders<ReferralDataEntity>.Filter.Where(x => x.ReferralInOrganizationCode == userInfo.RootOrganizationCode && x.ReferralStatus != RecordReferralStatusEnum.Withdrawn);
                    referralDatas = await _referralDataDBRepository.FindAllAsync(referralDataFliter);
                }
                else
                {
                    FilterDefinition<ReferralDataEntity> referralDataFliter = Builders<ReferralDataEntity>.Filter.Where(x => x.ReferralInUserCode == userInfo.UserCode && x.ReferralStatus != RecordReferralStatusEnum.Withdrawn);
                    referralDatas = await _referralDataDBRepository.FindAllAsync(referralDataFliter);
                }
                if (referralDatas.Count > 0)
                {
                    //转诊申请机构
                    organizationList.AddRange(referralDatas.Select(x => x.ReferralOutOrganizationCode));
                    var recordCodes = referralDatas.Select(x => x.RecordCode).ToList();
                    MongoCustomerFilter<MongoTable> recordFilter = new();
                    MongoCustomerFilter<MongoDataItem> queryItemsFilter = new();
                    queryItemsFilter.ClearFilter();
                    queryItemsFilter.AppendEq((item => item.Key, "Code"));
                    queryItemsFilter.AppendIn(item => item.Value, recordCodes);
                    recordFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
                    var records = _recordInfoDBRepository.FindAllAsync(recordFilter.AndCombine()).Result;
                    var devices = await FindDeviceInfoListByCodeListAsync(records.Select(x => x.DeviceCode).ToList());
                    if (devices.Count > 0)
                    {
                        //转诊设备所在机构，用来查设备
                        organizationList.AddRange(devices.Select(x => x.OrganizationCode));
                    }
                }
            }
            customerFilter.AppendQueryItemInFilter(MongoCombineTypeEnum.Or, "Code", organizationList);
            var dbPage = new DBPageRequest();
            dbPage.CurrentPage = request.CurrentPage;
            dbPage.PageSize = request.PageSize;
            dbPage.Filter = customerFilter.AndCombine();
            var findPage = await _organizationDBRepository.GetPages(dbPage);
            var pageData = new PageCollection<OrganizationSimplifyDTO>();
            pageData.CurrentPage = findPage.CurrentPage;
            pageData.PageSize = findPage.PageSize;
            pageData.DataCount = findPage.DataCount;
            pageData.PageData = findPage.PageData.Select(s => new OrganizationSimplifyDTO
            {
                OrganizationCode = s.OrganizationCode,
                OrganizationName = s.OrganizationName
            }).ToList();
            return pageData;
        }

        /// <summary>
        /// 按机构唯一码字搜索组织
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns></returns>
        public async Task<OrganizationDTO> QueryOrganizationsByShortCodeDBAsync(QueryOrganizationsRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} QueryOrganizationsByShortCodeDBAsync execute", $"{this.GetType().Name}"))
            {
                string shortCode = request.ShortCode;
                if (string.IsNullOrWhiteSpace(shortCode))
                {
                    ThrowCustomerException(CustomerRpcCode.OrgNotExist, "Organization not exist");
                    return null;
                }
                var organizationDO = await _organizationDBRepository.FindOneAsync("ShortCode", shortCode.ToUpper());
                if (organizationDO == null)
                {
                    ThrowCustomerException(CustomerRpcCode.OrgShortCodeEmpty, "Organization short code empty");
                    return null;
                }
                var dto = organizationDO.MappingTo<OrganizationDTO>();
                return dto;
            }
        }

        /// <summary>
        /// 查询所有机构Code列表
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns></returns>
        public async Task<List<string>> GetAllOrganizationCodesDBAsync(GetUpgradeBusinessDataRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetAllOrganizationCodesDBAsync execute", $"{this.GetType().Name}"))
            {
                var list = new List<string>();
                if (request.Category == 0)
                {
                    var dic = new Dictionary<string, string>();
                    dic.Add("IsDelete", "False");
                    var organizationDOList = await _organizationDBRepository.FindListAsync(dic);
                    if (organizationDOList?.Count > 0)
                    {
                        list = organizationDOList.Select(c => c.OrganizationCode).Distinct().ToList();
                    }
                }
                else if (request.Category == 1)
                {
                    MongoCustomerFilter<MongoTable> filter = new();
                    filter.AppendSizeLt("BaseData.RoleCodes", 1);
                    var userDOList = await _userDBRepository.FindAllAsync(filter.AndCombine());
                    if (userDOList?.Count > 0)
                    {
                        list = userDOList.Select(c => c.UserCode).Distinct().ToList();
                    }
                }
                return list;
            }
        }

        /// <summary>
        /// 所有机构:添加短码字段
        /// </summary>
        /// <returns>是否成功</returns>
        public async Task<bool> UpdateOrganizationsDBAsync(ModifyUserBindDeviceOrganizationRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateOrganizationsDBAsync execute", $"{this.GetType().Name}"))
            {
                var res = false;
                if (request.OrganizationCodes?.Count > 0)
                {
                    var dic = new Dictionary<string, List<string>>();
                    dic.Add("Code", request.OrganizationCodes.ToList());
                    var organizationDOList = await _organizationDBRepository.FindInAsync(dic);
                    if (organizationDOList?.Count > 0)
                    {
                        foreach (var item in organizationDOList)
                        {
                            var shortCode = "";
                            for (var i = 0; i < 20; i++)
                            {
                                shortCode = GeneralShortCode();
                                if (!string.IsNullOrWhiteSpace(shortCode))
                                {
                                    var orgInfo = await _organizationDBRepository.FindOneAsync("ShortCode", shortCode);
                                    if (orgInfo == null || string.IsNullOrWhiteSpace(orgInfo.Code))
                                    {
                                        break;
                                    }
                                }
                                shortCode = string.Empty;
                            }
                            item.ShortCode = shortCode;
                            item.UpdateTime = DateTime.UtcNow;
                            res = await _organizationDBRepository.UpdateOneAsync("Code", item.Code, item, "", true);
                            CacheMaintenance.Instance.Get<IOrganizationsManager>().Remove(item.Code);
                        }
                    }
                }
                return res;
            }
        }

        /// <summary>
        /// 修改即时会诊配置
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> UpdateEmergencySettingDBAsync(UpdateEmergencySettingDBRequest request)
        {
            string organizationCode = request.OrganizationCode;
            bool emergencyNeedFill = request.EmergencyNeedFill;
            string emergencySettingVersion = request.EmergencySettingVersion;
            string emergencySettingJson = request.EmergencySettingJson;
            using (new Performance(_performanceThreshold, "{0} UpdateEmergencySettingDBAsync execute", $"{this.GetType().Name}"))
            {
                var organizationDO = await _organizationDBRepository.FindOneAsync("Code", organizationCode);
                if (organizationDO != null)
                {
                    organizationDO.EmergencyNeedFill = emergencyNeedFill;
                    organizationDO.EmergencySettingVersion = emergencySettingVersion;
                    organizationDO.EmergencySettingJson = emergencySettingJson;
                    var result = await _organizationDBRepository.UpdateOneAsync("Code", organizationCode, organizationDO, string.Empty, true);
                    CacheMaintenance.Instance.Get<IOrganizationsManager>().Remove(organizationCode);
                    return result;
                }
            }
            return false;
        }
    
        /// <summary>
        /// 监听收费服务包是否过期
        /// </summary>
        /// <returns></returns>
        public async Task<bool> SetOrganizationResourcePackageExpired()
        { 
            using (new Performance(_performanceThreshold, "{0} SetOrganizationResourcePackageExpired execute", $"{this.GetType().Name}"))
            {
                var result = false;
                MongoCustomerFilter<MongoTable> customerFilter = new();
                customerFilter.AppendSizeGte("BaseData.RoBindChargeResourcePackageseCodes", 1);
                var orgList = await _organizationDBRepository.FindAllAsync(customerFilter.AndCombine());
                if (orgList?.Count > 0)
                {
                    foreach(var item in orgList)
                    {
                        if (item?.BindChargeResourcePackages?.Count > 0)
                        {
                            var indexList = new List<int>();        
                            var index = 0;
                            foreach(var packageInfo in item.BindChargeResourcePackages)
                            {
                                if (DateTime.UtcNow > packageInfo.EndTime.AddDays(1))
                                {
                                    //表示已过期
                                    indexList.Add(index);
                                }
                                index++;
                            }
                            if (indexList?.Count > 0)
                            {
                                foreach(var curIndex in indexList)
                                {
                                    item.BindChargeResourcePackages.RemoveAt(curIndex);
                                }
                                //更新db
                                result = await _organizationDBRepository.UpdateOneAsync("Code", item.Code, item, string.Empty, true);
                            }
                        }
                    }
                }
                return result;
            }
        }
    }
}
