using System;
using System.Threading.Tasks;
using MongoDB.Driver;
using WingInterfaceLibrary.DB.Request;
using WingInterfaceLibrary.DTO.Package;
using WingInterfaceLibrary.Interface.DBInterface;
using WingMongoDBModule.Entities;
using WingServerCommon.Log;
using WingServerCommon.Mapper;
using WingServerCommon.Service;
using WingServerCommon.Utilities;

namespace WingMongoDBModule.Service
{
    public partial class DatabaseService : JsonRpcService, IPackagesDBService
    {
        /// <summary>
        /// 查询软件安装包
        /// </summary>
        /// <returns></returns>
        public async Task<PackagesDTO> GetPackageDBAsync(GetPackageDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetPackageDBAsync execute", $"{this.GetType().Name}"))
            {
                var code = request.PackageCode;
                var builder = Builders<PackagesEntity>.Filter;
                var filter = builder.Where(c => c.Code == code);
                var entity = await _packagesDBRepository.FindOneAsync(filter) ?? new PackagesEntity();
                return entity.MappingTo<PackagesDTO>();
            }
        }

        /// <summary>
        /// 根据唯一标识查询软件安装包
        /// </summary>
        /// <returns></returns>
        public async Task<PackagesDTO> GetPackageByUniquedCodeDBAsync(GetPackageByUniquedCodeDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetPackageByUniquedCodeDBAsync execute", $"{this.GetType().Name}"))
            {
                var uniquedCode = request.UniquedCode;
                var builder = Builders<PackagesEntity>.Filter;
                var filter = builder.Where(c => c.UniquedCode == uniquedCode);
                var entity = await _packagesDBRepository.FindOneAsync(filter) ?? new PackagesEntity();
                return entity.MappingTo<PackagesDTO>();
            }
        }

        /// <summary>
        /// 新增/更新软件安装包
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<string> AddOrUpdatePackageDBAsync(PackagesDTO request)
        {
            using (new Performance(_performanceThreshold, "{0} AddOrUpdatePackageDBAsync execute", $"{this.GetType().Name}"))
            {
                try
                {
                    var entity = request.MappingTo<PackagesEntity>();
                    var uniquedCode = request.UniquedCode;
                    var builder = Builders<PackagesEntity>.Filter;
                    var filter = builder.Where(c => c.UniquedCode == uniquedCode);
                    var existedEntity = await _packagesDBRepository.FindOneAsync(filter);
                    if (existedEntity != null && !string.IsNullOrWhiteSpace(existedEntity.Code))
                    {
                        existedEntity.Name = entity.Name;
                        existedEntity.Version = entity.Version;
                        existedEntity.UniquedCode = entity.UniquedCode;
                        existedEntity.Description = entity.Description;
                        existedEntity.FileUrl = entity.FileUrl;
                        existedEntity.FileSize = entity.FileSize;
                        existedEntity.UploadUserCode = entity.UploadUserCode;
                        existedEntity.UpdateTime = entity.UpdateTime;
                        await _packagesDBRepository.UpdateOneAsync("Code", existedEntity.Code, existedEntity);
                        return existedEntity.Code;
                    }
                    else
                    {
                        return await _packagesDBRepository.InsertOneAsync(entity);
                    }
                }
                catch (Exception ex)
                {
                    Logger.WriteLineWarn($"PackagesDBService AddOrUpdatePackageDBAsync error, ex:{ex}");
                }
                return string.Empty;
            }
        }

    }
}