using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using MongoDB.Bson;
using MongoDB.Driver;
using WingInterfaceLibrary.DB.Request;
using WingInterfaceLibrary.DTO.Management;
using WingInterfaceLibrary.DTO.Payment;
using WingInterfaceLibrary.Enum;
using WingInterfaceLibrary.Enum.CourseEnum;
using WingInterfaceLibrary.Interface.DBInterface;
using WingInterfaceLibrary.Result;
using WingMongoDBModule.Entities;
using WingMongoDBModule.Request;
using WingServerCommon.Mapper;
using WingServerCommon.Service;

namespace WingMongoDBModule.Service
{
    public partial class DatabaseService : JsonRpcService, IPaymentDBService
    {
        /// <summary>
        /// 创建订单
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<CreateOrderResultDTO> CreateOrderAsync(CreateOrderDBRequest request)
        {
            var newCode = await _orderDBRepository.InsertOneAsync(new Entities.OrderEntity
            {
                OrderUserCode = request.OrderUserCode,
                OrderAmount = request.OrderAmount,
                OrderTitle = request.OrderTitle,
                OrderType = request.OrderType,
                ProductCode = request.ProductCode,
                TPORderCode = request.TPORderCode
            });
            return new CreateOrderResultDTO { OrderCode = newCode };
        }

        /// <summary>
        /// 更新订单
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<UpdateOrderResultDTO> UpdateOrderAsync(UpdateOrderDBRequest request)
        {
            var filter = Builders<Entities.OrderEntity>.Filter.Eq(f => f.Code, request.OrderCode);
            var update = Builders<Entities.OrderEntity>.Update.Combine();
            if (!string.IsNullOrWhiteSpace(request.TPORderCode))
            {
                update = update.Set(s => s.TPORderCode, request.TPORderCode);
            }
            if (request.PayStatus == PayStatusEnum.Paid)
            {
                update = update.Set(s => s.PayStatus, request.PayStatus);
            }
            var modifiedCount = await _orderDBRepository.UpdateOneAsync(filter, update);
            return new UpdateOrderResultDTO { IsSuccess = modifiedCount > 0 };
        }

        /// <summary>
        /// 新增支付记录信息接口
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<CreatePaymentRecordResultDTO> CreatePaymentRecordAsync(CreatePaymentRecordDBRequest request)
        {
            var newCode = await _paymentRecordDBRepository.InsertOneAsync(new Entities.PaymentRecordEntity
            {
                Code = request.Code,
                PayAmount = request.PayAmount,
                PayTime = request.PayTime,
                PayType = request.PayType,
                PayStatus = request.PayStatus,
                OrderCode = request.OrderCode,
                PayReqContent = request.PayReqContent,
                PayResContent = request.PayResContent
            });
            return new CreatePaymentRecordResultDTO { PaymentRecordCode = newCode };
        }

        /// <summary>
        /// 创建产品
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<CreateProductDTO> CreateProductAsync(CreateProductDBRequest request)
        {
            var newCode = await _productDBRepository.InsertOneAsync(new Entities.ProductEntity
            {
                ProductType = request.ProductType,
                Price = request.Price,
                Name = request.Name,
                HeadImage = request.HeadImage,
                BelongerCode = request.BelongerCode,
                RelationInfo = request.RelationInfo,
                Status = request.Status
            });
            return new CreateProductDTO { ProductCode = newCode };
        }

        /// <summary>
        /// 根据关联编码获取商品详情
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<ProductDetailDTO> GetProductByRelationInfoAsync(GetProductByRelationInfoDBRequest request)
        {
            var result = await _productDBRepository.FindOneAsync("RelationInfo", request.RelationInfo);
            return result.MappingTo<ProductDetailDTO>();
        }

        /// <summary>
        /// 获取订单详情
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<OrderDetailDTO> GetOrderDetailAsync(GetOrderDetailDBRequest request)
        {
            var result = await _orderDBRepository.FindOneAsync("Code", request.OrderCode);
            return result.MappingTo<OrderDetailDTO>();
        }

        /// <summary>
        /// 获取订单列表
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageResult<OrderDetailDTO>> GetOrderPagesAsync(GetOrderPagesDBRequest request)
        {
            var pageRequest = new DBPageRequest<OrderEntity>();
            pageRequest.CurrentPage = request.CurrentPage;
            pageRequest.PageSize = request.PageSize;
            BsonRegularExpression regularExp = null;
            if (!string.IsNullOrWhiteSpace(request.KeyWord))
            {
                var pattern = ReplaceSpecialCharacter(request.KeyWord);
                regularExp = new BsonRegularExpression((request.IsFuzzy ? $".*{pattern}.*" : pattern), "i");
                var filter1 = Builders<OrderEntity>.Filter.Regex(f => f.OrderTitle, regularExp);
                var filter2 = Builders<OrderEntity>.Filter.Regex(f => f.Code, regularExp);
                pageRequest.Filter = Builders<OrderEntity>.Filter.Or(filter1, filter2);
            }
            if(request.PayStatus!=null)
            {
                if(pageRequest.Filter!=null)
                {
                    pageRequest.Filter = pageRequest.Filter & Builders<OrderEntity>.Filter.Eq(f=>f.PayStatus, request.PayStatus);
                }
                else
                {
                    pageRequest.Filter = Builders<OrderEntity>.Filter.Eq(f=>f.PayStatus, request.PayStatus);
                }
            }
            var result = await _orderDBRepository.GetPages(pageRequest);
            if(result==null)
            {
                return new PageResult<OrderDetailDTO>();
            }
            var userNames = await GetUserInfoByCodeFromCache(result.PageData.Select(s=>s.OrderUserCode).ToList());
            var dto = PageResultConvert<OrderEntity, OrderDetailDTO>(result);
            dto?.PageData?.ForEach(e=>{
                e.OrderUserName = userNames?.FirstOrDefault(w=>w.Code == e.OrderUserCode)?.Name;
            });
            return dto;
        }

        /// <summary>
        /// 获取支付记录
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageResult<PaymentRecordDTO>> GetPaymentRecordsAsync(GetPaymentRecordsDBRequest request)
        {
            var pageRequest = new DBPageRequest<PaymentRecordEntity>();
            pageRequest.CurrentPage = request.CurrentPage;
            pageRequest.PageSize = request.PageSize;
            BsonRegularExpression regularExp = null;
            if (!string.IsNullOrWhiteSpace(request.KeyWord))
            {
                var pattern = ReplaceSpecialCharacter(request.KeyWord);
                regularExp = new BsonRegularExpression((request.IsFuzzy ? $".*{pattern}.*" : pattern), "i");
                pageRequest.Filter = Builders<PaymentRecordEntity>.Filter.Regex(f => f.OrderCode, regularExp);
            }
            else
            {
                pageRequest.Filter = Builders<PaymentRecordEntity>.Filter.Empty;
            }
            var result = await _paymentRecordDBRepository.GetPages(pageRequest);
            return PageResultConvert<PaymentRecordEntity, PaymentRecordDTO>(result);
        }

        /// <summary>
        /// 更新支付记录信息接口
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<UpdatePaymentRecordResultDTO> UpdatePaymentRecordAsync(UpdatePaymentRecordDBRequest request)
        {
            var filter = Builders<PaymentRecordEntity>.Filter.Eq(f => f.Code, request.Code);
            var update = Builders<PaymentRecordEntity>.Update
            .Set(s => s.OrderCode, request.OrderCode)
            .Set(s => s.PayStatus, request.PayStatus)
            .Set(s => s.PayTime, request.PayTime);
            if (!string.IsNullOrWhiteSpace(request.PayReqContent))
            {
                update = update.Set(s => s.PayReqContent, request.PayReqContent);
            }
            if (!string.IsNullOrWhiteSpace(request.PayResContent))
            {
                update = update.Set(s => s.PayResContent, request.PayResContent);
            }
            var modifiedCount = await _paymentRecordDBRepository.UpdateOneAsync(filter, update);
            return new UpdatePaymentRecordResultDTO { IsSuccess = modifiedCount > 0 };
        }

        /// <summary>
        /// 更新商品信息
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<UpdateProductDTO> UpdateProductAsync(UpdateProductDBRequest request)
        {
            var filter = Builders<ProductEntity>.Filter.Eq(f => f.Code, request.Code);
            var update = Builders<ProductEntity>.Update
            .Set(f => f.BelongerCode, request.BelongerCode)
            .Set(f => f.HeadImage, request.HeadImage)
            .Set(f => f.Name, request.Name)
            .Set(f => f.Price, request.Price)
            .Set(f => f.ProductType, request.ProductType)
            .Set(f => f.RelationInfo, request.RelationInfo)
            .Set(f => f.Status, request.Status);
            var result = await _productDBRepository.UpdateOneAsync(filter, update);
            return new UpdateProductDTO
            {
                IsSuccess = result > 0
            };
        }

        /// <summary>
        /// 根据关联信息修改商品信息
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<UpdateProductDTO> UpdateProductByRelationAsync(UpdateProductByRelationDBRequest request)
        {
            var relationInfo = request.RelationInfo;
            var result = new UpdateProductDTO();
            var productEntity = await _productDBRepository.FindOneAsync("RelationInfo", relationInfo);
            if (productEntity == null || string.IsNullOrWhiteSpace(productEntity.Code))
            {
                //需要支付的产品，根据业务审核状态自动确认上架、下架
                if (request.Price > 0)
                {
                    var createProductResult = await CreateProductAsync(new CreateProductDBRequest
                    {
                        ProductType = request.ProductType,
                        Price = request.Price,
                        Name = request.Name,
                        HeadImage = request.HeadImage,
                        BelongerCode = request.BelongerCode,
                        RelationInfo = request.RelationInfo,
                        Status = request.Status,
                    });
                    result.IsSuccess = !string.IsNullOrWhiteSpace(createProductResult.ProductCode);
                }
                else
                {
                    result.IsSuccess = true;
                }
            }
            else
            {
                var filter = Builders<ProductEntity>.Filter.Eq(f => f.Code, productEntity.Code);
                var update = Builders<ProductEntity>.Update
                .Set(f => f.BelongerCode, request.BelongerCode)
                .Set(f => f.HeadImage, request.HeadImage)
                .Set(f => f.Name, request.Name)
                .Set(f => f.Price, request.Price)
                .Set(f => f.ProductType, request.ProductType)
                .Set(f => f.RelationInfo, request.RelationInfo)
                .Set(f => f.Status, request.Status);
                var updateCourseResult = await _productDBRepository.UpdateOneAsync(filter, update);
                result.IsSuccess = updateCourseResult > 0;
            }
            return result;
        }

        /// <summary>
        /// 支付成功结果处理
        /// </summary>
        /// <returns></returns>
        public async Task<PaySuccessDBResult> PaySuccessAsync(PaySuccessDBRequest request)
        {
            var orderCode = request.PaymentOrderCode;
            var orderEntity = await _orderDBRepository.FindOneAsync("Code", orderCode);
            var result = new PaySuccessDBResult { };
            if (orderEntity != null && !string.IsNullOrWhiteSpace(orderEntity.Code))
            {
                var productEntity = await _productDBRepository.FindOneAsync("Code", orderEntity.ProductCode);
                if (productEntity != null && !string.IsNullOrWhiteSpace(productEntity.Code))
                {
                    result.PaymentOrderCode = orderEntity.Code;
                    result.BelongerCode = productEntity.BelongerCode;
                    result.RelationInfo = productEntity.RelationInfo;
                    result.OrderTitle = orderEntity.OrderTitle;
                    result.OrderUserCode = orderEntity.OrderUserCode;
                    result.OrderAmount = orderEntity.OrderAmount;
                    result.PayTime = orderEntity.PayTime;
                    result.Success = await CoursePaySuccessAsync(productEntity.RelationInfo, orderEntity.OrderUserCode);
                }
            }
            return result;
        }

        /// <summary>
        /// 课程支付成功处理
        /// </summary>
        /// <param name="courseCode"></param>
        /// <param name="userCode"></param>
        /// <returns></returns>
        private async Task<bool> CoursePaySuccessAsync(string courseCode, string userCode)
        {
            if (!string.IsNullOrWhiteSpace(courseCode))
            {
                var signFilter = Builders<SignStudentEntity>.Filter.Where(c=> c.CourseOrAlbumCode == courseCode
                    && c.StudentCode == userCode
                );
                var allSignStudent = await _signStudentDBRepository.FindOneAsync(signFilter);
                var allSignStudentCode = allSignStudent.Code;
                var updateFilter = Builders<SignStudentEntity>.Filter.Eq(f=>f.Code, allSignStudentCode);
                var updateSign = Builders<SignStudentEntity>.Update.Set(s=>s.IsPay, true);
                return await _signStudentDBRepository.UpdateOneAsync(updateFilter, updateSign) > 0;
            }
            return false;
        }

        /// <summary>
        /// 插入支付链接信息
        /// </summary>
        /// <param name="orderCode"></param>
        /// <param name="payUrl"></param>
        /// <returns></returns>
        public async Task<InsertPayUrlDBResult> InsertPayUrlAsync(InsertPayUrlDBRequest request)
        {
            var payUrl = await _payUrlDBRepository.FindOneAsync("PaymentOrderCode", request.PaymentOrderCode);
            if(payUrl!=null)
            {
                await _payUrlDBRepository.DeleteOneAsync("Code", payUrl.Code);
            }
            var newPayUrl = new PayUrlInfo();
            newPayUrl.PaymentOrderCode = request.PaymentOrderCode;
            newPayUrl.PayLink = request.PayLink;
            newPayUrl.PayShortLink = request.PayShortLink;
            if(!string.IsNullOrWhiteSpace(await _payUrlDBRepository.InsertOneAsync(newPayUrl)))
            {
                return new InsertPayUrlDBResult{ IsSuccess = true, PayShortUrl = newPayUrl.PayShortLink };
            }
            return new InsertPayUrlDBResult{ IsSuccess = false, PayShortUrl = string.Empty };
        }

        /// <summary>
        /// 获取支付链接信息
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<GetPayUrlDBResult> GetPayUrlAsync(GetPayUrlDBRequest request)
        {
            var paymentOrderCode = request.PaymentOrderCode;
            var result = await _payUrlDBRepository.FindOneAsync("PaymentOrderCode", paymentOrderCode);
            return new GetPayUrlDBResult{
                PayUrl = result?.PayLink
            };
        }

        private string ReplaceSpecialCharacter(string str)
        {
            if (string.IsNullOrWhiteSpace(str))
            {
                return str;
            }
            return str
            .Replace("(", "\\(")
            .Replace(")", "\\)")
            .Replace("+", "\\+")
            .Replace("<", "\\<")
            .Replace(">", "\\>")
            .Replace("[", "\\[")
            .Replace("]", "\\]")
            .Replace("*", "\\*")
            .Replace(".", "\\.");
        }
    }
}