using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using JsonRpcLite.Services;
using WingInterfaceLibrary.DB.Request;
using WingInterfaceLibrary.DTO.Position;
using WingInterfaceLibrary.Enum;
using WingInterfaceLibrary.Interface.DBInterface;
using WingMongoDBModule.DO;
using WingMongoDBModule.Entity;
using WingMongoDBModule.Mongodb.Driver;
using WingServerCommon.Config;
using WingServerCommon.Config.Parameters;
using WingServerCommon.Mapper;
using WingServerCommon.Service;
using WingServerCommon.Utilities;

namespace WingMongoDBModule.Service
{
    public partial class DatabaseService : JsonRpcService, IPositionDBService
    {
        /// <summary>
        /// > 将职位列表添加到数据库
        /// </summary>
        /// <param name="CreatePositionsDBRequest">客户端发送给服务器的请求对象。</param>
        /// <returns>
        /// 受影响的行数。
        /// </returns>
        public async Task<long> AddPositionsAsync(CreatePositionsDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} AddPositionsAsync execute", "DatabaseService"))
            {
                var req = request.Infos.Select(
                    f => {
                        var data = f.Data.MappingTo<UserPositionDO>();
                        data.PositionName = data.PositionName.Trim();
                        return (data, f.PlatformDatas, f.ExtensionData);
                    }
                ).ToList();
                MongoCustomerFilter<MongoTable> customerFilter = new();
                MongoCustomerFilter<MongoDataItem> subFilter = new();
                //验证是否有重复名称
                subFilter.AppendEq((item => item.Key, "PositionName"));
                subFilter.AppendIn(item => item.Value, req.Select(f=>f.Item1.PositionName).ToList());
                customerFilter.SubFilter(item => item.QueryData.QueryItems, subFilter.AndCombine());
                subFilter.ClearFilter();
                var isFound = await _positionDBRepository.FindOneAsync(customerFilter.AndCombine());
                if(isFound!=null && !string.IsNullOrWhiteSpace(isFound.PositionName) 
                && request.Infos.Any(i=>i.Data.OrganizationCode == isFound.OrganizationCode && i.Data.PositionName == isFound.PositionName))
                {
                    throw new RpcException((int)CustomerRpcCode.PositionNameDuplicate,"PositionName Duplicate","PositionName Duplicate");
                }
                return await _positionDBRepository.InsertAllAsync(req, true);
            }
        }


        /// <summary>
        /// 根据职位代码、组织代码、部门代码获取职位信息
        /// </summary>
        /// <param name="GetPositionsDBRequest">请求参数是一个自定义类，包含您需要查询的参数。</param>
        /// <returns>
        /// PositionDTO 对象的列表。
        /// </returns>
        public async Task<List<PositionDTO>> GetPositionsAsync(GetPositionsDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetPositionsAsync execute", "DatabaseService"))
            {
                MongoCustomerFilter<MongoTable> customerFilter = new();
                MongoCustomerFilter<MongoDataItem> itemFilter = new();
                if (request.PositionCodes != null && request.PositionCodes.Count(f => !string.IsNullOrWhiteSpace(f)) > 0)
                {
                    itemFilter.AppendEq((item => item.Key, "Code"));
                    itemFilter.AppendIn(item => item.Value, request.PositionCodes);
                    var itemFilters = itemFilter.AndCombine();
                    customerFilter.SubFilter(item => item.QueryData.QueryItems, itemFilters);
                    itemFilter.ClearFilter();
                }
                if (request.OrganizationCodes != null && request.OrganizationCodes.Count(f => !string.IsNullOrWhiteSpace(f)) > 0)
                {
                    itemFilter.AppendEq((item => item.Key, "OrganizationCode"));
                    itemFilter.AppendIn(item => item.Value, request.OrganizationCodes);
                    var itemFilters = itemFilter.AndCombine();
                    customerFilter.SubFilter(item => item.QueryData.QueryItems, itemFilters);
                    itemFilter.ClearFilter();
                }
                if (!string.IsNullOrWhiteSpace(request.FitDepartmentCode))
                {

                    itemFilter.AppendEq((item => item.Key, "FitDepartmentCode"), (item => item.Value, request.FitDepartmentCode));
                    var itemFilters = itemFilter.AndCombine();
                    customerFilter.SubFilter(item => item.QueryData.QueryItems, itemFilters);
                    itemFilter.ClearFilter();
                }
                var positionList = await _positionDBRepository.FindAllAsync(customerFilter.AndCombine());
                return positionList.MappingTo<List<PositionDTO>>();
            }
        }


        /// <summary>
        /// > 此函数从数据库中删除与给定位置代码匹配的所有位置
        /// </summary>
        /// <param name="RemovePositionDBRequest">包含要删除的位置代码列表的请求对象。</param>
        /// <returns>
        /// 删除的职位数。
        /// </returns>
        public async Task<long> RemovePositionsAsync(RemovePositionDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} RemovePositionsAsync execute", "DatabaseService"))
            {
                var infos = request.PositionCodes.Select(f => ("Code", f)).ToList();
                return await _positionDBRepository.DeleteAllAsync(infos);
            }
        }


        /// <summary>
        /// > 更新请求中的所有职位
        /// </summary>
        /// <param name="UpdatePositionsDBRequest">将传递给方法的请求对象。</param>
        /// <returns>
        /// 更新的记录数。
        /// </returns>
        public async Task<long> UpdatePositionsAsync(UpdatePositionsDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} UpdatePositionsAsync execute", "DatabaseService"))
            {
                var req = request.Positions.Select(f => ("Code", f.Data.PositionCode, f.Data.MappingTo<UserPositionDO>(), f.ExtensionData)).ToList();
                return await _positionDBRepository.UpdateAllAsync(req);
            }
        }


        /// <summary>
        /// 通过组织代码获取职位列表
        /// </summary>
        /// <param name="organizationCodes">待查询职位的组织机构代码列表。</param>
        /// <returns>
        /// PositionDTO 对象的列表。
        /// </returns>
        public async Task<List<PositionDTO>> GetPositionsByOrganizationCodesAsync(List<string> organizationCodes)
        {
            using (new Performance(_performanceThreshold, "{0} GetPositionsByOrganizationCodesAsync execute", "DatabaseService"))
            {
                var dic = new Dictionary<string, List<string>>();
                if (organizationCodes.Count > 0)
                {
                    dic.Add("OrganizationCode", organizationCodes);
                }
                var positionList = await _positionDBRepository.FindInAsync(dic);
                return positionList.MappingTo<List<PositionDTO>>();
            }
        }


        /// <summary>
        /// 更新职位名称
        /// </summary>
        /// <param name="UpdatePositionNameDBRequest">客户端发送给服务器的请求对象。</param>
        /// <returns>
        /// 一个布尔值。
        /// </returns>
        public async Task<bool> UpdatePositionNameAsync(UpdatePositionNameDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} UpdatePositionNameAsync execute", "DatabaseService"))
            {
                return await _positionDBRepository.UpdatePositionName(request);
            }
        }
    }
}