using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using MongoDB.Bson;
using MongoDB.Driver;
using WingInterfaceLibrary.DB.Request;
using WingInterfaceLibrary.DTO.Project;
using WingInterfaceLibrary.Enum;
using WingInterfaceLibrary.Interface.DBInterface;
using WingInterfaceLibrary.Result;
using WingMongoDBModule.Entities;
using WingMongoDBModule.Request;
using WingServerCommon.Log;
using WingServerCommon.Mapper;
using WingServerCommon.Service;

namespace WingMongoDBModule.Service
{
    public partial class DatabaseService : JsonRpcService, IProjectDBService
    {
        /// <summary>
        /// 分页查询项目配置
        /// </summary>
        /// <returns></returns>
        public async Task<PageResult<ProjectInfoDTO>> GetProjectByPageDBAsync(GetProjectByPageDBRequest request)
        {
            var pageIndex = request.PageIndex;
            var pageSize = request.PageSize;
            var keyword = request.Keyword;

            var builder = Builders<ProjectInfoEntity>.Filter;
            var filter = builder.Where(c => c.IsDelete == false);
            if (!string.IsNullOrEmpty(keyword))
            {
                keyword = EscapeExprSpecialWord(keyword);//字符串转义
                var nameFilter = builder.Regex(c => c.Name, new BsonRegularExpression(keyword, "i"));
                filter = filter & nameFilter;
            }
            var sort = Builders<ProjectInfoEntity>.Sort.Descending(v => v.CreateTime);
            var pageResult = await _projectInfoDBRepository.GetPages(new DBPageRequest<ProjectInfoEntity>
            {
                CurrentPage = pageIndex,
                PageSize = pageSize,
                Filter = filter,
                Sort = sort,
            });
            return PageResultConvert<ProjectInfoEntity, ProjectInfoDTO>(pageResult);
        }

        /// <summary>
        /// 查询项目配置详情
        /// </summary>
        /// <returns></returns>
        public async Task<ProjectInfoDTO> GetProjectDBAsync(GetProjectDBRequest request)
        {
            var code = request.Code;
            var builder = Builders<ProjectInfoEntity>.Filter;
            var filter = builder.Where(c => c.Code == code);
            var entity = await _projectInfoDBRepository.FindOneAsync(filter) ?? new ProjectInfoEntity();
            return entity.MappingTo<ProjectInfoDTO>();
        }

        /// <summary>
        /// 新增/更新项目配置
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<string> AddOrUpdateProjectDBAsync(ProjectInfoDTO request)
        {
            try
            {
                var entity = request.MappingTo<ProjectInfoEntity>();
                if (!string.IsNullOrWhiteSpace(entity.Code))
                {
                    var code = entity.Code;
                    var existedEntity = await _projectInfoDBRepository.FindOneAsync("Code", code, false);
                    if (existedEntity != null && !string.IsNullOrWhiteSpace(existedEntity.Code))
                    {
                        var organizationInfoChanged = !ArraysAreEqual(entity.OrganizationInfos, existedEntity.OrganizationInfos);
                        var organizationPosterChanged = !ArraysAreEqual(entity.PosterInfos.Where(x => x.DataType == ProjectDataTypeEnum.Organization)?.ToList() ?? new List<ProjectPosterEntity>(), existedEntity.PosterInfos.Where(x => x.DataType == ProjectDataTypeEnum.Organization)?.ToList() ?? new List<ProjectPosterEntity>());
                        var userPosterChanged = !ArraysAreEqual(entity.PosterInfos.Where(x => x.DataType == ProjectDataTypeEnum.User)?.ToList() ?? new List<ProjectPosterEntity>(), existedEntity.PosterInfos.Where(x => x.DataType == ProjectDataTypeEnum.User)?.ToList() ?? new List<ProjectPosterEntity>());
                        var devicePosterChanged = !ArraysAreEqual(entity.PosterInfos.Where(x => x.DataType == ProjectDataTypeEnum.Device)?.ToList() ?? new List<ProjectPosterEntity>(), existedEntity.PosterInfos.Where(x => x.DataType == ProjectDataTypeEnum.Device)?.ToList() ?? new List<ProjectPosterEntity>());

                        existedEntity.Name = entity.Name;
                        existedEntity.PushType = entity.PushType;
                        existedEntity.ContactName = entity.ContactName;
                        existedEntity.ContactPhone = entity.ContactPhone;
                        existedEntity.OrganizationInfos = entity.OrganizationInfos;
                        existedEntity.PosterInfos = entity.PosterInfos;
                        await _projectInfoDBRepository.UpdateOneAsync("Code", code, existedEntity);
                        await GetProjectInfosByOrganizationAsync("", true);

                        if (organizationInfoChanged)
                        {
                            await AddPushOrganizationsAsync(existedEntity);
                            await AddPushDevicesAsync(existedEntity);
                            await AddPushUsersAsync(existedEntity);
                        }
                        else
                        {
                            if (organizationPosterChanged)
                            {
                                await AddPushOrganizationsAsync(existedEntity);
                            }
                            if (userPosterChanged)
                            {
                                await AddPushDevicesAsync(existedEntity);
                            }
                            if (devicePosterChanged)
                            {
                                await AddPushUsersAsync(existedEntity);
                            }
                        }

                        return code;
                    }
                }
                else
                {
                    var result = await _projectInfoDBRepository.InsertOneAsync(entity);
                    await GetProjectInfosByOrganizationAsync("", true);
                    return result;
                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"ProjectDBService AddOrUpdateProjectDBAsync error, ex:{ex}");
            }
            return string.Empty;
        }

        /// <summary>
        /// 删除项目配置
        /// </summary>
        /// <returns></returns>
        public async Task<bool> DeleteProjectDBAsync(DeleteProjectDBRequest request)
        {
            var result = await _projectInfoDBRepository.DeleteOneAsync("Code", request.Code);
            return result;
        }

        /// <summary>
        /// 查询所有项目配置
        /// </summary>
        /// <returns></returns>
        public async Task<List<ProjectInfoDTO>> GetProjectAllDBAsync()
        {
            var builder = Builders<ProjectInfoEntity>.Filter;
            var filter = builder.Where(c => c.IsDelete == false);
            var entities = await _projectInfoDBRepository.FindAllAsync(filter);
            return entities.MappingTo<List<ProjectInfoDTO>>();
        }
    }
}