using System;
using System.Collections.Concurrent;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Threading.Tasks;
using MongoDB.Driver;
using MongoDB.Driver.GeoJsonObjectModel;
using WingInterfaceLibrary.DB.Request;
using WingInterfaceLibrary.DTO;
using WingInterfaceLibrary.DTO.Common;
using WingInterfaceLibrary.DTO.Consultation;
using WingInterfaceLibrary.DTO.Report;
using WingInterfaceLibrary.Enum;
using WingInterfaceLibrary.Interface.DBInterface;
using WingInterfaceLibrary.Request.DBRequest;
using WingInterfaceLibrary.Request.Examine;
using WingInterfaceLibrary.Request.Report;
using WingMongoDBModule.Common;
using WingMongoDBModule.DO;
using WingMongoDBModule.Entities;
using WingServerCommon.Config;
using WingServerCommon.Config.Parameters;
using WingServerCommon.Interfaces.Cache;
using WingServerCommon.Log;
using WingServerCommon.Service;
using WingServerCommon.Utilities;
using WingServerCommon.Utilities.Executors;

namespace WingMongoDBModule.Service
{
    public partial class DatabaseService : JsonRpcService, IProjectPushRecordDBService
    {
        private DateTime _refreshTime = DateTime.MinValue;
        private List<ProjectInfoEntity> _projectAll = new List<ProjectInfoEntity>();
        private bool _isChinaMainland = true;
        private readonly SequenceExecutor<ProjectPushRecordEntity> _projectPushRecordsExecutor = new SequenceExecutor<ProjectPushRecordEntity>("ProjectPushRecordsExecutor");

        /// <summary>
        /// 查询机构匹配的项目集合
        /// </summary>
        /// <param name="organizationCode"></param>
        /// <returns></returns>
        private async Task<List<ProjectInfoEntity>> GetProjectInfosByOrganizationAsync(string organizationCode, bool refresh = false)
        {
            if (_refreshTime <= DateTime.UtcNow.AddMinutes(-3) || refresh)
            {
                _refreshTime = DateTime.UtcNow;
                var builder = Builders<ProjectInfoEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false);
                var entities = await _projectInfoDBRepository.FindAllAsync(filter);
                _projectAll = entities?.ToList() ?? new List<ProjectInfoEntity>();
                Logger.WriteLineInfo($"ProjectPushRecordDBService FindProjectAll, projectCount:{_projectAll.Count}, projectNames:{string.Join(";", _projectAll.Select(x => x.Name))}");
            }
            return _projectAll.Where(x => x.OrganizationInfos?.Any(o => o.OrganizationCode == organizationCode) ?? false)?.ToList() ?? new List<ProjectInfoEntity>();
        }

        /// <summary>
        /// 开始推送到第三方厂商
        /// </summary>
        /// <param name="entity"></param>
        /// <returns></returns>
        private bool PushRecord(ProjectPushRecordEntity entity)
        {
            try
            {
                _projectPosterManager.Post(entity);
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"ProjectPushRecordDBService PushRecord error, projectName:{entity.ProjectName}, type:{entity.DataType}, dataCode:{entity.DataCode}, name:{entity.DataName}, ex:{ex}");
            }
            return true;
        }

        /// <summary>
        /// Server启动，自动发送未处理的记录
        /// </summary>
        /// <returns></returns>
        private async Task<bool> SendProjectPushRecordByDBAsync()
        {
            try
            {
                if (_isDistributed && !_isMaster)
                {
                    return false;
                }
                var builder = Builders<ProjectPushRecordEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false && c.PushStatus == ProjectPushStatusEnum.NotSend);
                var entities = await _projectPushRecordDBRepository.FindAllAsync(filter);
                if (entities?.Any() ?? false)
                {
                    Logger.WriteLineInfo($"ProjectPushRecordDBService SendProjectPushRecordByDBAsync, not send record count:{entities.Count}");
                    foreach (var entity in entities)
                    {
                        _projectPushRecordsExecutor.Add(PushRecord, entity);
                    }
                }
                else
                {
                    Logger.WriteLineInfo($"ProjectPushRecordDBService SendProjectPushRecordByDBAsync, not send record is empty");
                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"ProjectPushRecordDBService SendProjectPushRecordByDBAsync error, ex:{ex}");
            }
            return true;
        }

        private async Task<bool> InitYinChuanProjectAsync()
        {
            try
            {
                if (_isDistributed && !_isMaster)
                {
                    return false;
                }
                var yinChuanProjectPath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "YinChuanProject.json");
                if (File.Exists(yinChuanProjectPath))
                {
                    var projectString = File.ReadAllText(yinChuanProjectPath);
                    if (!string.IsNullOrWhiteSpace(projectString))
                    {
                        var projectInfo = Newtonsoft.Json.JsonConvert.DeserializeObject<ProjectInfoEntity>(projectString);
                        var projectCode = projectInfo.Code;

                        var builder = Builders<ProjectInfoEntity>.Filter;
                        var filter = builder.Where(c => c.Code == projectCode);
                        var entity = await _projectInfoDBRepository.FindOneAsync(filter);
                        if (entity == null || string.IsNullOrWhiteSpace(entity.Code))
                        {
                            await _projectInfoDBRepository.InsertOneAsync(projectInfo);
                            Logger.WriteLineInfo($"ProjectPushRecordDBService InitYinChuanProjectAsync success");
                            await GetProjectInfosByOrganizationAsync("", true);
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"ProjectPushRecordDBService InitYinChuanProjectAsync error, ex:{ex}");
            }
            return true;
        }

        #region 检查、添加到发送记录

        private async Task AddPushOrganizationsAsync(ProjectInfoEntity projectInfo)
        {
            if (projectInfo.OrganizationInfos?.Any() ?? false)
            {
                var organizationPoster = projectInfo.PosterInfos.FirstOrDefault(x => x.DataType == ProjectDataTypeEnum.Organization);
                if (organizationPoster != null)
                {
                    var organizationCodes = projectInfo.OrganizationInfos.Select(x => x.OrganizationCode).ToList();
                    var organizations = await FindOrganizationDTOListByCodesAsync(organizationCodes);
                    Logger.WriteLineInfo($"ProjectPushRecordDBService AddPushOrganizationsAsync, projectName:{projectInfo.Name}, orgCount:{organizations.Count}, pushUrl:{organizationPoster.ApiUrl}");
                    foreach (var organization in organizations)
                    {
                        var projectPushRecord = new ProjectPushRecordEntity(projectInfo, organizationPoster)
                        {
                            DataCode = organization.OrganizationCode,
                            DataName = organization.OrganizationName,
                            IsDistributeHandle = _isDistributed
                        };
                        await _projectPushRecordDBRepository.InsertOneAsync(projectPushRecord);
                        _projectPushRecordsExecutor.Add(PushRecord, projectPushRecord);
                    }
                }
            }
            Logger.WriteLineInfo($"ProjectPushRecordDBService AddPushOrganizationsAsync finished, projectName:{projectInfo.Name}");
        }

        private async Task AddPushDevicesAsync(ProjectInfoEntity projectInfo)
        {
            if (projectInfo.OrganizationInfos?.Any() ?? false)
            {
                var devicePoster = projectInfo.PosterInfos.FirstOrDefault(x => x.DataType == ProjectDataTypeEnum.Device);
                if (devicePoster != null)
                {
                    var organizationCodes = projectInfo.OrganizationInfos.Select(x => x.OrganizationCode).ToList();
                    var devices = await FindDeviceInfoListByOrganizationCodesAsync(organizationCodes);
                    Logger.WriteLineInfo($"ProjectPushRecordDBService AddPushDevicesAsync, projectName:{projectInfo.Name}, deviceCount:{devices.Count}, pushUrl:{devicePoster.ApiUrl}");
                    foreach (var device in devices)
                    {
                        var projectPushRecord = new ProjectPushRecordEntity(projectInfo, devicePoster)
                        {
                            DataCode = device.DeviceCode,
                            DataName = device.DisplayName,
                            IsDistributeHandle = _isDistributed
                        };
                        await _projectPushRecordDBRepository.InsertOneAsync(projectPushRecord);
                        _projectPushRecordsExecutor.Add(PushRecord, projectPushRecord);
                    }
                }
            }
            Logger.WriteLineInfo($"ProjectPushRecordDBService AddPushDevicesAsync finished, projectName:{projectInfo.Name}");
        }

        private async Task<bool> AddPushDeviceAsync(DeviceInfoDO deviceInfo)
        {
            var organizationCode = deviceInfo.OrganizationCode;
            var dataCode = deviceInfo.Code;

            var projectInfos = await GetProjectInfosByOrganizationAsync(organizationCode);
            if (projectInfos.Any())
            {
                Logger.WriteLineInfo($"ProjectPushRecordDBService AddPushDeviceAsync start, dataCode:{dataCode}, deviceName:{deviceInfo.ShortCode}, projectNames:{string.Join(";", projectInfos.Select(x => x.Name))}");
                foreach (var projectInfo in projectInfos)
                {
                    var devicePoster = projectInfo.PosterInfos.FirstOrDefault(x => x.DataType == ProjectDataTypeEnum.Device);
                    if (devicePoster != null)
                    {
                        var projectPushRecord = new ProjectPushRecordEntity(projectInfo, devicePoster)
                        {
                            DataCode = dataCode,
                            DataName = deviceInfo.ShortCode,
                            IsDistributeHandle = false
                        };
                        await _projectPushRecordDBRepository.InsertOneAsync(projectPushRecord);
                        _projectPushRecordsExecutor.Add(PushRecord, projectPushRecord);
                    }
                }
            }
            return true;
        }

        private async Task AddPushUsersAsync(ProjectInfoEntity projectInfo)
        {
            if (projectInfo.OrganizationInfos?.Any() ?? false)
            {
                var userPoster = projectInfo.PosterInfos.FirstOrDefault(x => x.DataType == ProjectDataTypeEnum.User);
                if (userPoster != null)
                {
                    var organizationCodes = projectInfo.OrganizationInfos.Select(x => x.OrganizationCode).ToList();
                    var users = await FindUserListByOrganizationCodesAsync(organizationCodes);
                    Logger.WriteLineInfo($"ProjectPushRecordDBService AddPushUsersAsync, projectName:{projectInfo.Name}, userCount:{users.Count}, pushUrl:{userPoster.ApiUrl}");
                    foreach (var user in users)
                    {
                        var projectPushRecord = new ProjectPushRecordEntity(projectInfo, userPoster)
                        {
                            DataCode = user.UserCode,
                            DataName = user.DisplayName,
                            IsDistributeHandle = _isDistributed
                        };
                        await _projectPushRecordDBRepository.InsertOneAsync(projectPushRecord);
                        _projectPushRecordsExecutor.Add(PushRecord, projectPushRecord);
                    }
                }
            }
            Logger.WriteLineInfo($"ProjectPushRecordDBService AddPushUsersAsync finished, projectName:{projectInfo.Name}");
        }

        private async Task<bool> AddPushUserAsync(UserInfoDO userInfo)
        {
            var organizationCode = userInfo.OrganizationCode;
            var dataCode = userInfo.Code;

            var userDTO = await FindUserDTOByCodeAsync(dataCode, false);
            var projectInfos = await GetProjectInfosByOrganizationAsync(organizationCode);
            if (projectInfos.Any())
            {
                Logger.WriteLineInfo($"ProjectPushRecordDBService AddPushUserAsync start, dataCode:{dataCode}, userName:{userDTO.DisplayName}, projectNames:{string.Join(";", projectInfos.Select(x => x.Name))}");
                foreach (var projectInfo in projectInfos)
                {
                    var userPoster = projectInfo.PosterInfos.FirstOrDefault(x => x.DataType == ProjectDataTypeEnum.User);
                    if (userPoster != null)
                    {
                        var projectPushRecord = new ProjectPushRecordEntity(projectInfo, userPoster)
                        {
                            DataCode = dataCode,
                            DataName = userDTO.DisplayName,
                            IsDistributeHandle = false
                        };
                        await _projectPushRecordDBRepository.InsertOneAsync(projectPushRecord);
                        _projectPushRecordsExecutor.Add(PushRecord, projectPushRecord);
                    }
                }
            }
            return true;
        }

        private async Task AddPushPatientAsync(PatientInfoDO patientInfo, List<DataItem> relationDatas = null)
        {
            var organizationCode = patientInfo.RootOrganizationCode;
            var patientName = GetPatientValue(patientInfo.PatientData, new List<string> { "Name", "AnimalInfoName" }, true);
            var devicePatientId = GetPatientValue(patientInfo.PatientData, new List<string> { "ID", "AnimalInfoID" });
            var dataCode = patientInfo.PatientCode;

            var projectInfos = await GetProjectInfosByOrganizationAsync(organizationCode);
            if (projectInfos.Any())
            {
                Logger.WriteLineInfo($"ProjectPushRecordDBService AddPushPatientAsync start, dataCode:{dataCode}, devicePatientId:{devicePatientId}, patientName:{patientName}, projectNames:{string.Join(";", projectInfos.Select(x => x.Name))}");
                foreach (var projectInfo in projectInfos)
                {
                    var patientPoster = projectInfo.PosterInfos.FirstOrDefault(x => x.DataType == ProjectDataTypeEnum.Patient);
                    if (patientPoster != null)
                    {
                        var projectPushRecord = new ProjectPushRecordEntity(projectInfo, patientPoster)
                        {
                            DataCode = dataCode,
                            DataName = $"{devicePatientId}_{patientName}".Trim('_'),
                            RelationDatas = relationDatas ?? new List<DataItem>(),
                            IsDistributeHandle = false
                        };
                        await _projectPushRecordDBRepository.InsertOneAsync(projectPushRecord);
                        _projectPushRecordsExecutor.Add(PushRecord, projectPushRecord);
                    }
                }
            }
        }

        private async Task AddPushRemoteRecordAsync(RecordInfoDO recordInfo)
        {
            var organizationCode = recordInfo.RootOrganizationCode;
            var patientName = GetPatientValue(recordInfo.PatientInfo, new List<string> { "Name", "AnimalInfoName" }, true);
            var devicePatientId = GetPatientValue(recordInfo.PatientInfo, new List<string> { "ID", "AnimalInfoID" });
            var dataCode = recordInfo.RecordCode;

            var projectInfos = await GetProjectInfosByOrganizationAsync(organizationCode);
            if (projectInfos.Any())
            {
                Logger.WriteLineInfo($"ProjectPushRecordDBService AddPushRemoteRecordAsync start, dataCode:{dataCode}, devicePatientId:{devicePatientId}, patientName:{patientName}, projectNames:{string.Join(";", projectInfos.Select(x => x.Name))}");
                foreach (var projectInfo in projectInfos)
                {
                    var remoteRecordPoster = projectInfo.PosterInfos.FirstOrDefault(x => x.DataType == ProjectDataTypeEnum.RemoteRecord);
                    if (remoteRecordPoster != null)
                    {
                        var projectPushRecord = new ProjectPushRecordEntity(projectInfo, remoteRecordPoster)
                        {
                            DataCode = dataCode,
                            DataName = $"{devicePatientId}_{patientName}".Trim('_'),
                            IsDistributeHandle = false
                        };
                        await _projectPushRecordDBRepository.InsertOneAsync(projectPushRecord);
                        _projectPushRecordsExecutor.Add(PushRecord, projectPushRecord);

                        var patient = await _patientInfoDBRepository.FindOneAsync("Code", recordInfo.PatientCode);
                        if (patient != null && !string.IsNullOrWhiteSpace(patient.PatientCode))
                        {
                            var recordCodeItem = new DataItem { Key = "RecordId", Value = dataCode };
                            await AddPushPatientAsync(patient, new List<DataItem> { recordCodeItem });
                        }
                    }
                }
            }
        }

        private async Task AddPushRemedicalAsync(RemedicalDO remedicalInfo)
        {
            if (string.IsNullOrWhiteSpace(remedicalInfo.TerminalImages?.OriginImageUrl)
                || string.IsNullOrWhiteSpace(remedicalInfo.TerminalImages?.PreviewUrl)
                || string.IsNullOrWhiteSpace(remedicalInfo.TerminalImages?.CoverImageUrl))
            {
                return;
            }
            try
            {
                var recordCode = remedicalInfo.RecordCode;
                var recordInfo = FindRecordInfoByCodeAsync(recordCode).Result;
                var organizationCode = recordInfo.RootOrganizationCode;
                var patientName = GetPatientValue(recordInfo.PatientInfo, new List<string> { "Name", "AnimalInfoName" }, true);
                var devicePatientId = GetPatientValue(recordInfo.PatientInfo, new List<string> { "ID", "AnimalInfoID" });
                var dataCode = remedicalInfo.Code;

                var projectInfos = await GetProjectInfosByOrganizationAsync(organizationCode);
                if (projectInfos.Any())
                {
                    Logger.WriteLineInfo($"ProjectPushRecordDBService AddPushRemedicalAsync start, dataCode:{dataCode}, devicePatientId:{devicePatientId}, patientName:{patientName}, projectNames:{string.Join(";", projectInfos.Select(x => x.Name))}");
                    foreach (var projectInfo in projectInfos)
                    {
                        var remedicalPoster = projectInfo.PosterInfos.FirstOrDefault(x => x.DataType == ProjectDataTypeEnum.Remedical);
                        if (remedicalPoster != null)
                        {
                            var projectPushRecord = new ProjectPushRecordEntity(projectInfo, remedicalPoster)
                            {
                                DataCode = dataCode,
                                DataName = $"{devicePatientId}_{patientName}".Trim('_'),
                                IsDistributeHandle = false
                            };
                            await _projectPushRecordDBRepository.InsertOneAsync(projectPushRecord);
                            _projectPushRecordsExecutor.Add(PushRecord, projectPushRecord);
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"ProjectPushRecordDBService Manager AddPushRemedicalAsync failed, ex:{ex}");
            }
        }

        private async Task AddPushConsultationAsync(ConsultationRecordEntity consultationInfo)
        {
            var dataCode = consultationInfo.Code;
            var organizationCodes = new List<string> { consultationInfo.ApplyOrganizationCode, consultationInfo.ExpertOrganizationCode };
            var organizations = await FindOrganizationDTOListByCodesAsync(organizationCodes);

            var pushedProjects = new List<string>();
            foreach (var organization in organizations)
            {
                var patientName = GetPatientValue(consultationInfo.PatientDatas?.ToList(), new List<string> { "Name", "AnimalInfoName" }, true);
                var devicePatientId = GetPatientValue(consultationInfo.PatientDatas?.ToList(), new List<string> { "ID", "AnimalInfoID" });

                var projectInfos = await GetProjectInfosByOrganizationAsync(organization.OrganizationCode);
                if (projectInfos.Any())
                {
                    Logger.WriteLineInfo($"ProjectPushRecordDBService AddPushConsultationAsync start, dataCode:{dataCode}, orgName:{organization.OrganizationName}, patientName:{patientName}, projectNames:{string.Join(";", projectInfos.Select(x => x.Name))}");
                    foreach (var projectInfo in projectInfos)
                    {
                        if (pushedProjects.Contains(projectInfo.Code))
                        {
                            continue;
                        }
                        var consultationPoster = projectInfo.PosterInfos.FirstOrDefault(x => x.DataType == ProjectDataTypeEnum.Consultation);
                        if (consultationPoster != null)
                        {
                            pushedProjects.Add(projectInfo.Code);
                            var projectPushRecord = new ProjectPushRecordEntity(projectInfo, consultationPoster)
                            {
                                DataCode = dataCode,
                                DataName = $"{devicePatientId}_{patientName}".Trim('_'),
                                IsDistributeHandle = false
                            };
                            await _projectPushRecordDBRepository.InsertOneAsync(projectPushRecord);
                            _projectPushRecordsExecutor.Add(PushRecord, projectPushRecord);
                        }
                    }
                }
            }
        }

        private async Task AddPushReportAsync(ReportDO reportInfo)
        {
            if (reportInfo.ReportPreviewList == null || !reportInfo.ReportPreviewList.Any())
            {
                return;
            }
            var patientName = CryptionHelper.Decrypt(reportInfo.PatientName);
            var devicePatientId = reportInfo.PatientId;
            var dataCode = reportInfo.Code;
            var reportUserCode = reportInfo.ReportUserCode;
            var reportUserDO = await FindUserDTOByCodeAsync(reportUserCode, true);
            var organizationCode = reportUserDO.RootOrganizationCode;

            var projectInfos = await GetProjectInfosByOrganizationAsync(organizationCode);
            if (projectInfos.Any())
            {
                Logger.WriteLineInfo($"ProjectPushRecordDBService AddPushReportAsync start, dataCode:{dataCode}, devicePatientId:{devicePatientId}, patientName:{patientName}, projectNames:{string.Join(";", projectInfos.Select(x => x.Name))}");
                foreach (var projectInfo in projectInfos)
                {
                    var reportPoster = projectInfo.PosterInfos.FirstOrDefault(x => x.DataType == ProjectDataTypeEnum.Report);
                    if (reportPoster != null)
                    {
                        var projectPushRecord = new ProjectPushRecordEntity(projectInfo, reportPoster)
                        {
                            DataCode = dataCode,
                            DataName = $"{devicePatientId}_{patientName}".Trim('_'),
                            IsDistributeHandle = false
                        };
                        await _projectPushRecordDBRepository.InsertOneAsync(projectPushRecord);
                        _projectPushRecordsExecutor.Add(PushRecord, projectPushRecord);
                    }
                }
            }
        }

        #endregion

        private string GetPatientValue(List<DataItemDTO> patientData, List<string> patientKeys, bool decrypt = false)
        {
            if (patientData != null && patientData.Any() && patientKeys != null && patientKeys.Any())
            {
                foreach (var patientKey in patientKeys)
                {
                    foreach (var patientItem in patientData)
                    {
                        if (patientItem.Key == patientKey && !string.IsNullOrWhiteSpace(patientItem.Value))
                        {
                            if (decrypt)
                            {
                                return CryptionHelper.Decrypt(patientItem.Value);
                            }
                            return patientItem.Value;
                        }
                    }
                }
            }
            return string.Empty;
        }

        private string GetPatientValue(List<DataItem> patientData, List<string> patientKeys, bool decrypt = false)
        {
            if (patientData != null && patientData.Any() && patientKeys != null && patientKeys.Any())
            {
                foreach (var patientKey in patientKeys)
                {
                    foreach (var patientItem in patientData)
                    {
                        if (patientItem.Key == patientKey && !string.IsNullOrWhiteSpace(patientItem.Value))
                        {
                            if (decrypt)
                            {
                                return CryptionHelper.Decrypt(patientItem.Value);
                            }
                            return patientItem.Value;
                        }
                    }
                }
            }
            return string.Empty;
        }

        private Tuple<int, string, string> GetPatientGender(string gender, string language)
        {
            if (gender == PatientGenderEnum.Male.ToString())
            {
                return new Tuple<int, string, string>(1, "Male", language == "zh-CN" ? "男" : "Male");
            }
            else if (gender == PatientGenderEnum.Female.ToString())
            {
                return new Tuple<int, string, string>(2, "Female", language == "zh-CN" ? "女" : "Female");
            }
            else if (gender == AnimalGenderEnum.AnimalInfoMale.ToString())
            {
                return new Tuple<int, string, string>(1, "Male", language == "zh-CN" ? "雄" : "Male");
            }
            else if (gender == AnimalGenderEnum.AnimalInfoFemale.ToString())
            {
                return new Tuple<int, string, string>(2, "Female", language == "zh-CN" ? "雌" : "Female");
            }
            else
            {
                return new Tuple<int, string, string>(0, "NotFilled", language == "zh-CN" ? "未填写" : "NotFilled");
            }
        }

        private ConcurrentDictionary<string, object> GetPatientDataByCode(ConcurrentDictionary<string, object> patientDatas, string patientCode, string language)
        {
            if (!string.IsNullOrWhiteSpace(patientCode))
            {
                var patient = FindPatientByCodeAsync(patientCode).Result;
                if (patient != null && !string.IsNullOrWhiteSpace(patient.PatientCode))
                {
                    var patientName = GetPatientValue(patient.PatientData, new List<string> { "Name", "AnimalInfoName" }, true);
                    var devicePatientId = GetPatientValue(patient.PatientData, new List<string> { "ID", "AnimalInfoID" });
                    var gender = GetPatientValue(patient.PatientData, new List<string> { "Sex", "AnimalInfoSex" });
                    var gengerItems = GetPatientGender(gender, language);
                    var ageValue = GetPatientAge(patient.PatientData, language);
                    var birthday = GetPatientValue(patient.PatientData, new List<string> { "Birthday" });
                    patientDatas.TryAdd("PatientCode", patientCode);
                    patientDatas.TryAdd("PatientName", patientName);
                    patientDatas.TryAdd("DevicePatientId", devicePatientId);
                    patientDatas.TryAdd("Gender", gengerItems.Item1);
                    patientDatas.TryAdd("GenderType", gengerItems.Item1);
                    patientDatas.TryAdd("GenderValue", gengerItems.Item2);
                    patientDatas.TryAdd("GenderLanguage", gengerItems.Item3);
                    patientDatas.TryAdd("PatientGender", gengerItems.Item1);
                    patientDatas.TryAdd("Age", ageValue);
                    patientDatas.TryAdd("PatientAge", ageValue);
                    patientDatas.TryAdd("Birthday", birthday);
                    var scanDoctorName = GetPatientValue(patient.PatientData, new List<string> { "ScanDoctor" });
                    if (!string.IsNullOrWhiteSpace(scanDoctorName))
                    {
                        patientDatas.TryAdd("ScanDoctorName", scanDoctorName);
                        patientDatas.TryAdd("ScanDoctor", scanDoctorName);
                    }

                    patientDatas.TryAdd("PatientId", devicePatientId);
                    patientDatas.TryAdd("Name", patientName);
                }
            }
            return patientDatas;
        }

        private bool SavePushResult(string pushRecordCode, ProjectPushStatusEnum pushStatus, string pushDatas, string resultContent)
        {
            var recordDTO = GetProjectPushRecordDBAsync(new GetProjectPushRecordDBRequest { Code = pushRecordCode }).Result;
            if (recordDTO != null && !String.IsNullOrWhiteSpace(recordDTO.Code) && recordDTO.PushStatus != pushStatus)
            {
                recordDTO.PushStatus = pushStatus;
                recordDTO.PushTime = DateTime.UtcNow;
                recordDTO.PushDatas = pushDatas;
                recordDTO.ResultContent = resultContent;
                var result = AddOrUpdateProjectPushRecordDBAsync(recordDTO).Result;
                return !string.IsNullOrWhiteSpace(result);
            }
            return false;
        }

        private ConcurrentDictionary<string, object> GetPushDatas(ProjectPushRecordEntity pushRecordInfo, string language)
        {
            var template = pushRecordInfo.PushTemplate;
            var dataCode = pushRecordInfo.DataCode;
            if (string.IsNullOrWhiteSpace(template) || string.IsNullOrWhiteSpace(dataCode))
            {
                return new ConcurrentDictionary<string, object>();
            }

            var pushDatas = new ConcurrentDictionary<string, object>();
            if (pushRecordInfo.DataType == ProjectDataTypeEnum.Organization)
            {
                var entities = FindOrganizationDTOListByCodesAsync(new List<string> { dataCode }).Result;
                var entity = entities.FirstOrDefault();
                if (entity != null && !string.IsNullOrWhiteSpace(entity.OrganizationCode))
                {
                    pushDatas.TryAdd("CreateTime", entity.CreateTime);
                    pushDatas.TryAdd("OrganizationCode", entity.OrganizationCode);
                    pushDatas.TryAdd("OrganizationName", entity.OrganizationName);
                    pushDatas.TryAdd("Description", entity.OrganizationName);
                    var project = GetProjectDBAsync(new GetProjectDBRequest { Code = pushRecordInfo.ProjectCode }).Result;
                    var thirdOrganizationCode = project?.OrganizationInfos?.FirstOrDefault(x => x.OrganizationCode == entity.OrganizationCode)?.ThirdCode ?? string.Empty;
                    pushDatas.TryAdd("ThirdOrganizationCode", thirdOrganizationCode);

                    pushDatas.TryAdd("PushHospitalId", thirdOrganizationCode);
                    pushDatas.TryAdd("Name", entity.OrganizationName);
                    pushDatas.TryAdd("Id", entity.OrganizationCode);
                }
            }
            else if (pushRecordInfo.DataType == ProjectDataTypeEnum.Device)
            {
                var entity = FindDeviceInfoByCodeAsync(dataCode).Result;
                if (entity != null && !string.IsNullOrWhiteSpace(entity.DeviceCode))
                {
                    pushDatas.TryAdd("CreateTime", entity.CreateTime);
                    pushDatas.TryAdd("DeviceCode", entity.DeviceCode);
                    pushDatas.TryAdd("OrganizationCode", entity.OrganizationCode);
                    pushDatas.TryAdd("Name", entity.Name);
                    pushDatas.TryAdd("ShortCode", entity.ShortCode);
                    pushDatas.TryAdd("DisplayName", entity.DisplayName);
                    pushDatas.TryAdd("Description", entity.Description);
                    pushDatas.TryAdd("DeviceModel", entity.DeviceModel);

                    pushDatas.TryAdd("HospitalId", entity.OrganizationCode);
                    pushDatas.TryAdd("Model", entity.DeviceModel);
                    pushDatas.TryAdd("Id", entity.DeviceCode);
                }
            }
            else if (pushRecordInfo.DataType == ProjectDataTypeEnum.User)
            {
                var entity = FindUserByCodeAsync(dataCode).Result;
                if (entity != null && !string.IsNullOrWhiteSpace(entity.UserCode))
                {
                    pushDatas.TryAdd("CreateTime", entity.CreateTime);
                    pushDatas.TryAdd("UserCode", entity.UserCode);
                    pushDatas.TryAdd("OrganizationCode", entity.RootOrganizationCode);
                    pushDatas.TryAdd("UserName", entity.UserName);
                    pushDatas.TryAdd("DisplayName", entity.DisplayName);
                    pushDatas.TryAdd("Phone", entity.Phone);

                    pushDatas.TryAdd("Name", entity.UserName);
                    pushDatas.TryAdd("HospitalId", entity.RootOrganizationCode);
                    pushDatas.TryAdd("Id", entity.UserCode);
                }
            }
            else if (pushRecordInfo.DataType == ProjectDataTypeEnum.Patient)
            {
                var entity = FindPatientByCodeAsync(dataCode).Result;
                if (entity != null && !string.IsNullOrWhiteSpace(entity.PatientCode))
                {
                    pushDatas.TryAdd("CreateTime", entity.CreateTime);
                    pushDatas.TryAdd("OrganizationCode", entity.RootOrganizationCode);
                    pushDatas = GetPatientDataByCode(pushDatas, dataCode, language);

                    if (pushRecordInfo.RelationDatas?.Any() ?? false)
                    {
                        foreach (var item in pushRecordInfo.RelationDatas)
                        {
                            pushDatas.TryAdd(item.Key, item.Value);
                        }
                    }
                }
            }
            else if (pushRecordInfo.DataType == ProjectDataTypeEnum.RemoteRecord)
            {
                var entity = FindRecordInfoByCodeAsync(dataCode).Result;
                if (entity != null && !string.IsNullOrWhiteSpace(entity.RecordCode))
                {
                    pushDatas.TryAdd("CreateTime", entity.CreateTime);
                    pushDatas.TryAdd("RecordCode", entity.RecordCode);
                    pushDatas.TryAdd("OrganizationCode", entity.RootOrganizationCode);
                    var organizationCache = CacheMaintenance.Instance.Get<IOrganizationsManager>().Get(entity.RootOrganizationCode);
                    pushDatas.TryAdd("OrganizationName", organizationCache?.OrganizationName);
                    pushDatas.TryAdd("DeviceCode", entity.DeviceCode);
                    var deviceCache = CacheMaintenance.Instance.Get<IDeviceInfosManager>().Get(entity.DeviceCode);
                    pushDatas.TryAdd("DeviceName", deviceCache?.DisplayName);
                    pushDatas = GetPatientDataByCode(pushDatas, entity.PatientCode, language);
                    pushDatas.TryAdd("CreateType", entity.CreateType.ToString());

                    pushDatas.TryAdd("ExamId", entity.RecordCode);
                    pushDatas.TryAdd("HosptialId", entity.RootOrganizationCode);
                    pushDatas.TryAdd("TerminalId", entity.DeviceCode);
                    pushDatas.TryAdd("Id", entity.RecordCode);
                }
            }
            else if (pushRecordInfo.DataType == ProjectDataTypeEnum.Remedical)
            {
                var entity = GetRemedicalByCodeAsync(dataCode, false).Result;
                if (entity != null && !string.IsNullOrWhiteSpace(entity.RemedicalCode))
                {
                    pushDatas.TryAdd("CreateTime", entity.CreateTime);
                    pushDatas.TryAdd("RemedicalCode", entity.RemedicalCode);
                    pushDatas.TryAdd("Application", entity.Application);
                    pushDatas.TryAdd("ApplicationCategory", entity.ApplicationCategory);
                    pushDatas.TryAdd("OriginImageUrl", entity.TerminalImages?.OriginImageUrl);
                    pushDatas.TryAdd("CoverImageUrl", entity.TerminalImages?.CoverImageUrl);
                    pushDatas.TryAdd("PreviewUrl", entity.TerminalImages?.PreviewUrl);
                    pushDatas.TryAdd("ImageUrl", entity.TerminalImages?.ImageUrl);
                    pushDatas.TryAdd("ImageSize", entity.TerminalImages?.ImageSize);
                    pushDatas.TryAdd("FileDataType", entity.FileDataType.ToString());

                    var recordCode = entity.RecordCode;
                    var record = FindRecordInfoByCodeAsync(recordCode).Result;
                    if (record != null)
                    {
                        pushDatas.TryAdd("OrganizationCode", record.RootOrganizationCode);
                        pushDatas = GetPatientDataByCode(pushDatas, record.PatientCode, language);
                    }
                    pushDatas.TryAdd("RecordCode", recordCode);

                    pushDatas.TryAdd("FileToken", entity.TerminalImages?.OriginImageUrl);
                    pushDatas.TryAdd("Id", entity.RemedicalCode);
                    pushDatas.TryAdd("TerminalRecordId", recordCode);
                }
            }
            else if (pushRecordInfo.DataType == ProjectDataTypeEnum.Consultation)
            {
                var entity = FindConsultationRecordByCodeAsync(new ConsultationRecordCodeDBRequest { Code = dataCode }).Result;
                if (entity != null && !string.IsNullOrWhiteSpace(entity.ConsultationCode))
                {
                    pushDatas.TryAdd("CreateTime", entity.CreateTime);
                    pushDatas.TryAdd("ConsultationCode", entity.ConsultationCode);
                    pushDatas.TryAdd("ConsultationTime", entity.ConsultationTime);
                    pushDatas.TryAdd("ApplyOrganizationCode", entity.ApplyOrganizationCode);
                    pushDatas.TryAdd("ApplyOrganizationName", entity.ApplyOrganizationName);
                    pushDatas.TryAdd("ExpertOrganizationCode", entity.ExpertOrganizationCode);
                    pushDatas.TryAdd("ExpertOrganizationName", entity.ExpertOrganizationName);
                    pushDatas.TryAdd("ApplyUserCode", entity.ApplyUserCode);
                    pushDatas.TryAdd("ApplyUserName", entity.ApplyUserName);
                    pushDatas.TryAdd("ExpertUserCode", entity.ExpertUserCode);
                    pushDatas.TryAdd("ExpertUserName", entity.ExpertUserName);
                    var scanPostion = entity.ScanPositions.FirstOrDefault() ?? string.Empty;
                    pushDatas.TryAdd("ScanPostion", scanPostion);

                    pushDatas.TryAdd("DeviceCode", entity.DeviceCode);
                    pushDatas.TryAdd("DeviceName", entity.DisplayName);
                    pushDatas = GetPatientDataByCode(pushDatas, entity.PatientCode, language);
                    if (!pushDatas.ContainsKey("ScanDoctorName"))
                    {
                        pushDatas.TryAdd("ScanUserCode", entity.ScanUserCode);
                        pushDatas.TryAdd("ScanUserName", entity.ScanUserName);
                    }
                    pushDatas.TryAdd("ConsultationStatus", (int)entity.ConsultationStatus);
                    var imageFiles = entity.ConsultationFileList?.Where(x => x.FileDataType == RemedicalFileDataTypeEnum.VinnoVidSingle || x.FileDataType == RemedicalFileDataTypeEnum.ThirdVidSingle || x.FileDataType == RemedicalFileDataTypeEnum.Image)?.ToList() ?? new List<ConsultationFileDTO>();
                    pushDatas.TryAdd("ImageInfos", imageFiles.Select(x => new { FileUrl = x.SourceUrl, PreviewUrl = x.PreviewImageUrl, CoverUrl = x.CoverImageUrl }));
                    pushDatas.TryAdd("Images", imageFiles.Select(x => x.SourceUrl).ToList());
                    var videoFiles = entity.ConsultationFileList?.Where(x => x.FileDataType == RemedicalFileDataTypeEnum.VinnoVidMovie || x.FileDataType == RemedicalFileDataTypeEnum.ThirdVidMovie)?.ToList() ?? new List<ConsultationFileDTO>();
                    pushDatas.TryAdd("VideoInfos", imageFiles.Select(x => new { FileUrl = x.SourceUrl, PreviewUrl = x.PreviewImageUrl, CoverUrl = x.CoverImageUrl }));
                    pushDatas.TryAdd("Videos", imageFiles.Select(x => x.SourceUrl).ToList());

                    pushDatas.TryAdd("AppointmentTime", entity.ConsultationTime);
                    pushDatas.TryAdd("CentralHospitalId", entity.ExpertOrganizationCode);
                    pushDatas.TryAdd("CheckPoint", scanPostion);
                    pushDatas.TryAdd("Id", entity.ConsultationCode);
                    pushDatas.TryAdd("ConsultationNumber", "");
                    pushDatas.TryAdd("DoctorId", entity.ApplyUserCode);
                    pushDatas.TryAdd("ExpertId", entity.ExpertUserCode);
                    pushDatas.TryAdd("PrimaryHosptialId", entity.ApplyOrganizationCode);
                    pushDatas.TryAdd("State", (int)entity.ConsultationStatus);
                    pushDatas.TryAdd("TermianlId", entity.DeviceCode);
                }
            }
            else if (pushRecordInfo.DataType == ProjectDataTypeEnum.Report)
            {
                var entity = GetReportByCodeAsync(new GetReportByCodeDBRequest { ReportCode = dataCode }).Result;
                if (entity != null && !string.IsNullOrWhiteSpace(entity.ReportCode))
                {
                    if (entity.DiagnosisLabels.Contains("VAid"))
                    {
                        var newReportDatasJson = _remedicalService.GetReportElementByLanguageAsync(new GetReportElementByLanguageRequest
                        {
                            ReportDatasJosn = entity.ReportDatasJson,
                            Language = language,
                        }).Result;
                        entity.ReportDatasJson = newReportDatasJson;
                    }
                    pushDatas.TryAdd("CreateTime", entity.CreateTime);
                    pushDatas.TryAdd("ReportCode", entity.ReportCode);
                    pushDatas.TryAdd("ReportUserCode", entity.ReportUserCode);
                    pushDatas.TryAdd("ReportUserName", entity.ReportUserName);
                    pushDatas.TryAdd("RecordCode", entity.RecordCode);
                    pushDatas.TryAdd("PatientCode", entity.PatientCode);
                    // pushDatas = GetPatientDataByCode(pushDatas, entity.PatientCode, language);

                    pushDatas.TryAdd("ExamId", entity.RecordCode);
                    pushDatas.TryAdd("PatientExamDate", entity.CreateTime);
                    pushDatas.TryAdd("Id", entity.ReportCode);
                    pushDatas.TryAdd("CreateUserId", entity.ReportUserCode);

                    pushDatas.TryAdd("Images", GetReportElementArray(entity.ReportDatasJson, "ReportImages"));
                    pushDatas.TryAdd("ReportDescription", GetReportElementValue(entity.ReportDatasJson, "ReportDescription"));
                    pushDatas.TryAdd("ReportSummary", GetReportElementValue(entity.ReportDatasJson, "ReportSummary"));
                    pushDatas.TryAdd("ReportPhysician", GetReportElementValue(entity.ReportDatasJson, "ReportPhysician"));
                    pushDatas.TryAdd("HospitalName", GetReportElementValue(entity.ReportDatasJson, "HospitalName"));
                    pushDatas.TryAdd("PatientName", GetReportElementValue(entity.ReportDatasJson, "PatientName"));
                    pushDatas.TryAdd("PatientGender", GetReportElementValue(entity.ReportDatasJson, "PatientGender"));
                    pushDatas.TryAdd("PatientAge", GetReportElementValue(entity.ReportDatasJson, "PatientAge"));
                    pushDatas.TryAdd("PatientId", GetReportElementValue(entity.ReportDatasJson, "PatientId"));
                    pushDatas.TryAdd("InspectionPhysician", GetReportElementValue(entity.ReportDatasJson, "InspectionPhysician"));
                }
            }

            return pushDatas;
        }

        private string GetReportElementValue(string reportElementValues, string name)
        {
            try
            {
                var list = Newtonsoft.Json.JsonConvert.DeserializeObject<List<ReportElementStringDTO>>(reportElementValues);
                return list.FirstOrDefault(x => x.Name == name)?.Value?.ToString() ?? string.Empty;
            }
            catch (Exception ex)
            {
                return string.Empty;
            }
        }

        private List<string> GetReportElementArray(string reportElementValues, string name)
        {
            try
            {
                var list = Newtonsoft.Json.JsonConvert.DeserializeObject<List<ReportElementArrayDTO>>(reportElementValues);
                var result = list.FirstOrDefault(x => x.Name == name)?.Value;
                if (result != null && result is List<string> array)
                {
                    return array?.ToList() ?? new List<string>();
                }
                return new List<string>();
            }
            catch (Exception ex)
            {
                return new List<string>();
            }
        }

    }
}