using System;
using System.Collections.Concurrent;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using MongoDB.Bson;
using MongoDB.Driver;
using WingInterfaceLibrary.DB.Request;
using WingInterfaceLibrary.DTO.Project;
using WingInterfaceLibrary.Interface.DBInterface;
using WingInterfaceLibrary.Result;
using WingMongoDBModule.Entities;
using WingMongoDBModule.Request;
using WingServerCommon.Log;
using WingServerCommon.Mapper;
using WingServerCommon.Service;

namespace WingMongoDBModule.Service
{
    public partial class DatabaseService : JsonRpcService, IProjectPushRecordDBService
    {
        /// <summary>
        /// 分页查询项目数据推送记录
        /// </summary>
        /// <returns></returns>
        public async Task<PageResult<ProjectPushRecordDTO>> GetProjectPushRecordByPageDBAsync(GetProjectPushRecordByPageDBRequest request)
        {
            var pageIndex = request.PageIndex;
            var pageSize = request.PageSize;

            var builder = Builders<ProjectPushRecordEntity>.Filter;
            var filter = builder.Where(c => c.IsDelete == false);
            if (!string.IsNullOrEmpty(request.ProjectName))
            {
                var projectName = EscapeExprSpecialWord(request.ProjectName);
                var nameFilter = builder.Regex(c => c.ProjectName, new BsonRegularExpression(projectName, "i"));
                filter = filter & nameFilter;
            }
            if (!string.IsNullOrEmpty(request.Keyword))
            {
                var keyword = EscapeExprSpecialWord(request.Keyword);
                var dataCodeFilter = builder.Regex(c => c.DataCode, new BsonRegularExpression(keyword, "i"));
                var dataNameFilter = builder.Regex(c => c.DataName, new BsonRegularExpression(keyword, "i"));
                filter = filter & (dataCodeFilter | dataNameFilter);
            }
            if (request.DataType != null)
            {
                var nameFilter = builder.Where(c => c.DataType == request.DataType);
                filter = filter & nameFilter;
            }
            if (request.PushStatus != null)
            {
                var nameFilter = builder.Where(c => c.PushStatus == request.PushStatus);
                filter = filter & nameFilter;
            }
            var sort = Builders<ProjectPushRecordEntity>.Sort.Descending(v => v.CreateTime);
            var pageResult = await _projectPushRecordDBRepository.GetPages(new DBPageRequest<ProjectPushRecordEntity>
            {
                CurrentPage = pageIndex,
                PageSize = pageSize,
                Filter = filter,
                Sort = sort,
            });
            if (pageResult != null && pageResult.PageData != null && pageResult.PageData.Any())
            {
                foreach (var data in pageResult.PageData)
                {
                    data.PushDatas = string.Empty;
                    data.ResultContent = string.Empty;
                }
            }
            return PageResultConvert<ProjectPushRecordEntity, ProjectPushRecordDTO>(pageResult);
        }

        /// <summary>
        /// 查询项目数据推送记录详情
        /// </summary>
        /// <returns></returns>
        public async Task<ProjectPushRecordDTO> GetProjectPushRecordDBAsync(GetProjectPushRecordDBRequest request)
        {
            var code = request.Code;
            var builder = Builders<ProjectPushRecordEntity>.Filter;
            var filter = builder.Where(c => c.Code == code);
            var entity = await _projectPushRecordDBRepository.FindOneAsync(filter) ?? new ProjectPushRecordEntity();
            return entity.MappingTo<ProjectPushRecordDTO>();
        }

        /// <summary>
        /// 新增/更新项目数据推送记录
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<string> AddOrUpdateProjectPushRecordDBAsync(ProjectPushRecordDTO request)
        {
            try
            {
                var entity = request.MappingTo<ProjectPushRecordEntity>();
                if (!string.IsNullOrWhiteSpace(entity.Code))
                {
                    var code = entity.Code;
                    var existedEntity = await _projectPushRecordDBRepository.FindOneAsync("Code", code, false);
                    if (existedEntity != null && !string.IsNullOrWhiteSpace(existedEntity.Code))
                    {
                        existedEntity.ProjectName = entity.ProjectName;
                        existedEntity.PushType = entity.PushType;
                        existedEntity.DataType = entity.DataType;
                        existedEntity.ApiUrl = entity.ApiUrl;
                        existedEntity.DataName = entity.DataName;
                        existedEntity.DataCode = entity.DataCode;
                        existedEntity.RelationDatas = entity.RelationDatas;
                        existedEntity.PushStatus = entity.PushStatus;
                        existedEntity.PushTime = entity.PushTime;
                        existedEntity.PushDatas = entity.PushDatas;
                        existedEntity.ResultContent = entity.ResultContent;
                        await _projectPushRecordDBRepository.UpdateOneAsync("Code", code, existedEntity);
                        return code;
                    }
                }
                else
                {
                    var result = await _projectPushRecordDBRepository.InsertOneAsync(entity);
                    _projectPushRecordsExecutor.Add(PushRecord, entity);
                    return result;
                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"ProjectPushRecordDBService AddOrUpdateProjectPushRecordDBAsync error, ex:{ex}");
            }
            return string.Empty;
        }

        /// <summary>
        /// 删除项目数据推送记录
        /// </summary> 
        /// <param name="code">项目项目数据推送记录编码</param>
        /// <returns></returns>
        public async Task<bool> DeleteProjectPushRecordDBAsync(DeleteProjectPushRecordDBRequest request)
        {
            var result = await _projectPushRecordDBRepository.DeleteOneAsync("Code", request.Code);
            return result;
        }

    }
}