using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using JsonRpcLite.Services;
using WingInterfaceLibrary.DB.Request;
using WingInterfaceLibrary.DTO.Rank;
using WingInterfaceLibrary.Enum;
using WingInterfaceLibrary.Interface.DBInterface;
using WingMongoDBModule.DO;
using WingMongoDBModule.Entity;
using WingMongoDBModule.Mongodb.Driver;
using WingServerCommon.Mapper;
using WingServerCommon.Service;
using WingServerCommon.Utilities;

namespace WingMongoDBModule.Service
{
    public partial class DatabaseService : JsonRpcService, IRankDBService
    {

        /// <summary>
        /// > 将用户等级列表添加到数据库
        /// </summary>
        /// <param name="CreateRanksDBRequest">客户端发送给服务器的请求对象。</param>
        /// <returns>
        /// 受影响的行数。
        /// </returns>
        public async Task<long> AddRanksAsync(CreateRanksDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} AddRanksAsync execute", $"{this.GetType().Name}"))
            {
                var req = request.Infos.Select(
                    f => 
                    {
                        var data = f.Data.MappingTo<UserRankDO>();
                        data.RankName = data.RankName.Trim();
                        return (data, f.PlatformDatas, f.ExtensionData);
                    }
                ).ToList();
                //验证是否有重复名称
                MongoCustomerFilter<MongoTable> customerFilter = new();
                MongoCustomerFilter<MongoDataItem> subFilter = new();
                subFilter.AppendEq((item => item.Key, "RankName"));
                subFilter.AppendIn(item => item.Value, req.Select(f=>f.Item1.RankName).ToList());
                customerFilter.SubFilter(item => item.QueryData.QueryItems, subFilter.AndCombine());
                subFilter.ClearFilter();
                var isFound = await _rankDBRepository.FindOneAsync(customerFilter.AndCombine());
                if(isFound!=null && !string.IsNullOrWhiteSpace(isFound.RankName)
                && request.Infos.Any(i=>i.Data.OrganizationCode == isFound.OrganizationCode && i.Data.RankName == isFound.RankName))
                {
                    throw new RpcException((int)CustomerRpcCode.RankNameDuplicate,"RankName Duplicate","RankName Duplicate");
                }
                return await _rankDBRepository.InsertAllAsync(req, true);
            }
        }


        /// <summary>
        /// 从数据库中获取所有等级
        /// </summary>
        /// <param name="GetRanksDBRequest">请求参数，它是继承自 BaseRequest 类的类。</param>
        /// <returns>
        /// RankDTO 对象列表
        /// </returns>
        public async Task<List<RankDTO>> GetAllRanksAsync(GetRanksDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetAllRanksAsync execute", $"{this.GetType().Name}"))
            {
                var dic = new Dictionary<string, List<string>>();
                if (request.RankCodes != null && request.RankCodes.Count(f => !string.IsNullOrWhiteSpace(f)) > 0)
                {
                    dic.Add("Code", request.RankCodes);
                }
                if (request.OrganizationCodes != null && request.OrganizationCodes.Count(f => !string.IsNullOrWhiteSpace(f)) > 0)
                {
                    dic.Add("OrganizationCode", request.OrganizationCodes);
                }
                var rankList = await _rankDBRepository.FindInAsync(dic);
                return rankList.MappingTo<List<RankDTO>>();
            }
        }


        /// <summary>
        /// > 通过代码获得排名
        /// </summary>
        /// <param name="rankCode">要搜索的排名代码。</param>
        /// <returns>
        /// 排名DTO
        /// </returns>
        public async Task<RankDTO> GetRankByCodeAsync(string rankCode)
        {
            using (new Performance(_performanceThreshold, "{0} GetRankByCodeAsync execute", $"{this.GetType().Name}"))
            {
                var result = await _rankDBRepository.FindOneAsync("Code", rankCode);
                return result.MappingTo<RankDTO>();
            }
        }


        /// <summary>
        /// > 此功能从数据库中删除排名
        /// </summary>
        /// <param name="RemoveRanksDBRequest">将传递给方法的请求对象。</param>
        /// <returns>
        /// 删除的行数。
        /// </returns>
        public async Task<long> RemoveRanksAsync(RemoveRanksDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} RemoveRanksAsync execute", $"{this.GetType().Name}"))
            {
                var infos = request.RankCodes.Select(f => ("Code", f)).ToList();
                return await _rankDBRepository.DeleteAllAsync(infos);
            }
        }


        /// <summary>
        /// > 更新数据库中的所有排名
        /// </summary>
        /// <param name="UpdateRanksDBRequest">将传递给服务的请求对象。</param>
        /// <returns>
        /// 更新的记录数。
        /// </returns>
        public async Task<long> UpdateRanksAsync(UpdateRanksDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateRanksAsync execute", "DatabaseService"))
            {
                var req = request.Ranks.Select(f => ("Code", f.Data.RankCode, f.Data.MappingTo<UserRankDO>(), f.ExtensionData)).ToList();
                return await _rankDBRepository.UpdateAllAsync(req);
            }
        }


        /// <summary>
        /// 按组织代码获取所有职级
        /// </summary>
        /// <param name="organizationCodes">要查询的职级组织机构代码列表。</param>
        /// <returns>
        /// RankDTO 对象列表
        /// </returns>
        public async Task<List<RankDTO>> GetAllRanksByOrganizationCodeAsync(List<string> organizationCodes)
        {
            using (new Performance(_performanceThreshold, "{0} GetAllRanksByOrganizationCodeAsync execute", $"{this.GetType().Name}"))
            {
                var dic = new Dictionary<string, List<string>>();
                if (organizationCodes != null && organizationCodes.Count > 0)
                {
                    dic.Add("OrganizationCode", organizationCodes);
                }
                var result = await _rankDBRepository.FindInAsync(dic);
                return result.MappingTo<List<RankDTO>>();
            }
        }


        /// <summary>
        /// > UpdateRankNameAsync 使用给定的排名代码更新排名的排名名称
        /// </summary>
        /// <param name="UpdateRankNameDBRequest">这是将传递给方法的请求对象。</param>
        /// <returns>
        /// 一个布尔值。
        /// </returns>
        public async Task<bool> UpdateRankNameAsync(UpdateRankNameDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateRankNameAsync execute", "DatabaseService"))
            {
                return await _rankDBRepository.UpdateRankName(request);
            }
        }
    }
}