using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using JsonRpcLite.Services;
using WingInterfaceLibrary.DB.Request;
using WingInterfaceLibrary.DTO.Record;
using WingInterfaceLibrary.DTO.User;
using WingInterfaceLibrary.Enum;
using WingInterfaceLibrary.Interface.DBInterface;
using WingInterfaceLibrary.Request;
using WingInterfaceLibrary.Result;
using WingMongoDBModule.Common;
using WingMongoDBModule.DO;
using WingMongoDBModule.Entity;
using WingMongoDBModule.Mongodb.Driver;
using WingMongoDBModule.Request;
using WingServerCommon.Interfaces.Cache;
using WingServerCommon.Mapper;
using WingServerCommon.Service;
using WingServerCommon.Utilities;
using WingInterfaceLibrary.Request.DBRequest;
using WingInterfaceLibrary.DTO.DataCenter;
using WingInterfaceLibrary.DTO.Common;
using MongoDB.Driver;
using MongoDB.Bson;
using WingMongoDBModule.Entities;
using WingInterfaceLibrary.Request.Record;
using WingServerCommon.Log;

namespace WingMongoDBModule.Service
{
    public partial class DatabaseService : JsonRpcService, IRecordsInfoDBService
    {
        /// <summary>
        /// > 通过代码查找记录
        /// </summary>
        /// <param name="code">要查询的记录的代码</param>
        /// <returns>
        /// 一个 RecordInfoDTO 对象
        /// </returns>
        public async Task<RecordInfoDTO> FindRecordInfoByCodeAsync(string code)
        {
            using (new Performance(_performanceThreshold, "{0} FindRecordInfoByCodeAsync execute", $"{this.GetType().Name}"))
            {
                var recordDO = await _recordInfoDBRepository.FindOneAsync("Code", code);
                return recordDO.MappingTo<RecordInfoDTO>();
            }
        }

        /// <summary>
        /// 通过设备检查代码查找记录
        /// </summary>
        /// <param name="code">要查询的记录的代码</param>
        /// <returns>
        /// 一个 RecordInfoDTO 对象
        /// </returns>
        public async Task<RecordInfoDTO> FindRecordInfoByDeviceExamIdAsync(string code)
        {
            using (new Performance(_performanceThreshold, "{0} FindRecordInfoByCodeAsync execute", $"{this.GetType().Name}"))
            {
                MongoCustomerFilter<MongoTable> customerFilter = new();//常规查询
                customerFilter.AppendEq(("BaseData.DeviceExamId", code));
                var filter = customerFilter.AndCombine();
                var recordDO = await _recordInfoDBRepository.FindOneAsync(filter);
                return recordDO.MappingTo<RecordInfoDTO>();
            }
        }


        /// <summary>
        /// 通过设备检查代码和项目查找记录
        /// </summary>
        /// <param name="code">要查询的记录的代码</param>
        /// <returns>
        /// 一个 RecordInfoDTO 对象
        /// </returns>
        public async Task<RecordInfoDTO> FindRecordInfoByDeviceExamIdAndProjectCodeAsync(FindRecordInfoByDeviceExamIdAndProjectCodeRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} FindRecordInfoByDeviceExamIdAndProjectCodeAsync execute", $"{this.GetType().Name}"))
            {
                string deviceExamId = request.DeviceExamId;
                string projectCode = request.ProjectCode;
                MongoCustomerFilter<MongoTable> customerFilter = new();//常规查询
                customerFilter.AppendEq(("BaseData.DeviceExamId", deviceExamId));
                customerFilter.AppendEq(("BaseData.ProjectCode", projectCode));
                var filter = customerFilter.AndCombine();
                var recordDO = await _recordInfoDBRepository.FindOneAsync(filter);
                var result = recordDO.MappingTo<RecordInfoDTO>();
                return result;
            }
        }


        /// <summary>
        /// 通过设备检查代码和项目查找记录
        /// </summary>
        /// <param name="code">要查询的记录的代码</param>
        /// <returns>
        /// 一个 RecordInfoDTO 对象
        /// </returns>
        public async Task<RecordInfoDTO> FindRecordInfoByDeviceExamIdAndPatientCodeAsync(FindRecordInfoByDeviceExamIdAndPatientCodeRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} FindRecordInfoByDeviceExamIdAndProjectCodeAsync execute", $"{this.GetType().Name}"))
            {
                string deviceExamId = request.DeviceExamId;
                string patientCode = request.PatientCode;
                MongoCustomerFilter<MongoTable> customerFilter = new();//常规查询
                customerFilter.AppendEq(("BaseData.DeviceExamId", deviceExamId));
                customerFilter.AppendEq(("BaseData.PatientCode", patientCode));
                var filter = customerFilter.AndCombine();
                var recordDO = await _recordInfoDBRepository.FindOneAsync(filter);
                var result = recordDO.MappingTo<RecordInfoDTO>();
                return result;
            }
        }

        /// <summary>
        /// 插入记录信息
        /// </summary>
        /// <param name="CreateRecordInfoDBRequest">客户端发送给服务器的请求对象。</param>
        /// <returns>
        /// 创建的记录的 ID。
        /// </returns>
        public async Task<string> InsertRecordInfoAsync(CreateRecordInfoDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} InsertRecordInfoAsync execute", $"{this.GetType().Name}"))
            {
                var entity = request.Data.MappingTo<RecordInfoDO>();
                var result = await _recordInfoDBRepository.InsertOneAsync(entity, request.PlatformDatas, request.ExtensionData);
                //await UpdatePatientUpdateTime(request.Data.PatientCode);
                await AddPushRemoteRecordAsync(entity);
                return result;
            }
        }

        /// <summary>
        /// 更新检查
        /// </summary>
        /// <param name="code">角色编码</param>
        /// <param name="data">角色数据</param>
        /// <param name="extensionData">扩展信息</param>
        /// <returns></returns>
        public async Task<bool> UpdateRecordInfoByCodeAsync(string code, RecordInfoDTO data, string extensionData, bool canNull = false)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateRecordInfoByCodeAsync execute", $"{this.GetType().Name}"))
            {
                var entity = data.MappingTo<RecordInfoDO>();
                entity.UpdateTime = DateTime.UtcNow;
                var result = await _recordInfoDBRepository.UpdateOneAsync("Code", code, entity, extensionData, canNull);
                // await AddPushRemoteRecordAsync(entity);
                return result;
            }
        }


        /// <summary>
        /// 通过代码删除记录信息
        /// </summary>
        /// <param name="code">要删除的记录的代码。</param>
        /// <returns>
        /// 一个布尔值。
        /// </returns>
        public async Task<bool> DeleteRecordInfoByCodeAsync(string code)
        {
            using (new Performance(_performanceThreshold, "{0} DeleteRecordInfoByCodeAsync execute", $"{this.GetType().Name}"))
            {
                var result = await _recordInfoDBRepository.DeleteOneAsync("Code", code);
                if (result)
                {
                    await UpdatePatientUrmLabelAsync(code);
                    var record = await _recordInfoDBRepository.FindOneAsync("Code", code, false);
                    if (!string.IsNullOrWhiteSpace(record.ProjectCode))
                    {
                        var remedicals = await GetRecordRemedicalAsync(code);
                        if (remedicals != null && remedicals.Count > 0)
                        {
                            var secondDirectory = GetSecondDirectory();
                            foreach (var item in remedicals)
                            {
                                try
                                {
                                    var originImageFilePath = GetFilePath(item.TerminalImages.OriginImageUrl, secondDirectory);
                                    DeleteFile(originImageFilePath);
                                    var imageFilePath = GetFilePath(item.TerminalImages.ImageUrl, secondDirectory);
                                    DeleteFile(imageFilePath);
                                    var coverImageFilePath = GetFilePath(item.TerminalImages.CoverImageUrl, secondDirectory);
                                    DeleteFile(coverImageFilePath);
                                    var previewFilePath = GetFilePath(item.TerminalImages.PreviewUrl, secondDirectory);
                                    DeleteFile(previewFilePath);
                                    foreach (var imgDetail in item.ImgList)
                                    {
                                        var imgDetailFilePath = GetFilePath(imgDetail.FileToken, secondDirectory);
                                        DeleteFile(imgDetailFilePath);
                                    }
                                }
                                catch (Exception ex)
                                {
                                    Logger.WriteLineError($"delete file error:{item.RemedicalCode},{ex}");
                                }
                            }
                            var remedicalMeasuredInfos = await GetRemedicalMeasuredInfosByRemedicalCodesAsync(remedicals.Select(x => x.RemedicalCode).ToList());
                            if (remedicalMeasuredInfos != null && remedicalMeasuredInfos.Count > 0)
                            {
                                foreach (var item in remedicalMeasuredInfos)
                                {
                                    try
                                    {
                                        var measuredFilePath = GetFilePath(item.MeasuredFileToken, secondDirectory);
                                        DeleteFile(measuredFilePath);
                                        var previewFilePath = GetFilePath(item.PreviewFileToken, secondDirectory);
                                        DeleteFile(previewFilePath);
                                    }
                                    catch (Exception ex)
                                    {
                                        Logger.WriteLineError($"delete file remedicalMeasuredInfos error:{item.RemedicalMeasuredInfoCode},{ex}");
                                    }
                                }
                            }
                        }
                    }
                }
                return result;
            }
        }

        /// <summary>
        /// 分页查询增量检查集合
        /// </summary>
        /// <returns></returns>
        public async Task<PageResult<RecordInfoDTO>> FindRecordInfosByUpdateTimeDBAsync(FindCollectionByTimeDBRequest request)
        {
            var startTime = request.StartTime;
            var pageIndex = request.PageIndex;
            var pageSize = request.PageSize;
            MongoCustomerFilter<MongoTable> customerFilter = new();//常规查询

            customerFilter.AppendGt("BaseData.UpdateTime", startTime);
            var filter = customerFilter.AndCombine();
            var pagedResult = await _recordInfoDBRepository.GetPages(new DBPageRequest
            {
                CurrentPage = pageIndex,
                PageSize = pageSize,
                Filter = filter,
                Sort = Builders<MongoTable>.Sort.Ascending("BaseData.UpdateTime"),
            }, onlyNotDeleted: false);
            var result = new PageResult<RecordInfoDTO>();
            result.PageIndex = pagedResult.CurrentPage;
            result.PageSize = pagedResult.PageSize;
            result.TotalCount = pagedResult.DataCount;
            result.PageData = pagedResult.PageData?.MappingTo<List<RecordInfoDTO>>() ?? new List<RecordInfoDTO>();
            return result;
        }

        /// <summary>
        /// 保存检查记录
        /// </summary>
        /// <returns></returns>
        public async Task<bool> SaveRecordInfoDBAsync(RecordInfoDTO request)
        {
            var code = request.RecordCode;
            var recordDO = request.MappingTo<RecordInfoDO>();
            var entity = await _recordInfoDBRepository.FindOneAsync("Code", code);
            if (entity != null && !string.IsNullOrWhiteSpace(entity.Code))
            {
                // recordDO.UpdateTime = DateTime.UtcNow;
                await _recordInfoDBRepository.UpdateOneAsync("Code", code, recordDO, "");
            }
            else
            {
                await _recordInfoDBRepository.InsertOneAsync(recordDO, new Dictionary<string, string>(), "");
            }
            return true;
        }

        /// <summary>
        /// 分页查询超声检查记录
        /// </summary>
        /// <returns></returns>
        public async Task<PageResult<SimpleRecordInfoDTO>> FindRecordByPageAsync(FindRecordByPageDBRequest request)
        {
            var userCode = request.UserCode;
            var orgCode = request.OrganizationCode;
            var queryState = request.RecordQueryState;
            var processState = request.RecordProcessState;
            var language = request.Language;
            var keyword = request.KeyWord;
            var startTime = request.StartTime;
            var endTime = request.EndTime;
            var pageIndex = request.CurrentPage;
            var pageSize = request.PageSize;

            MongoCustomerFilter<MongoTable> customerFilter = new();//常规查询
            MongoCustomerFilter<MongoTable> customerKeyWordFilter = new();//关键字查询
            MongoCustomerFilter<MongoDataItem> queryItemsFilter = new();
            if (!string.IsNullOrWhiteSpace(orgCode))
            {
                customerFilter.AppendEq(("BaseData.VitalOrganizationCode", orgCode));
            }
            if (startTime > new DateTime(2010, 1, 1))
            {
                customerFilter.AppendGt("BaseData.CreateTime", startTime.Date);
            }
            if (endTime > new DateTime(2010, 1, 1))
            {
                customerFilter.AppendLt("BaseData.CreateTime", endTime.Date.AddDays(1));
            }
            if (processState == RecordProcessStateEnum.Wait)
            {
                customerFilter.AppendEq(("BaseData.IsUserReport", false));
            }
            else if (processState == RecordProcessStateEnum.Done)
            {
                customerFilter.AppendEq(("BaseData.IsUserReport", true));
            }
            if (queryState != RecordQueryStateEnum.All)
            {
                customerFilter.AppendEq(("BaseData.RecordStatus", queryState.GetHashCode() - 1));
            }
            if (!string.IsNullOrWhiteSpace(keyword))
            {
                var keywordDictionary = new Dictionary<string, string>();
                var tempKeyWord = CryptionHelper.EncryptOneByOne(keyword);
                keywordDictionary.Add("Name", tempKeyWord);
                keywordDictionary.Add("Phone", tempKeyWord);
                keywordDictionary.Add("Age", keyword);
                keywordDictionary.Add("Gender", keyword);
                var pageFilterRequest = new GetPatientsByOrgPageRequest()
                {
                    KeywordOrFilter = keywordDictionary,
                    IsFuzzy = true
                };
                var patientList = GetPatientsDBAsync(pageFilterRequest).Result;
                queryItemsFilter = new();
                queryItemsFilter.AppendEq((item => item.Key, "PatientCode"));
                if (patientList == null || patientList.Count == 0)
                {
                    queryItemsFilter.AppendEq((item => item.Value, "-1"));
                    if (queryItemsFilter.HasFilter)
                    {
                        customerKeyWordFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
                    }
                }
                else
                {
                    queryItemsFilter.AppendIn(item => item.Value, patientList.Select(p => p.PatientCode).ToList());
                    if (queryItemsFilter.HasFilter)
                    {
                        customerKeyWordFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
                    }
                }
                customerKeyWordFilter.AppendRegexForKey("BaseData.DevicePatientID", keyword, true);

            }
            var filter = customerFilter.AndCombine() & customerKeyWordFilter.OrCombine();
            var pagedResult = await _recordInfoDBRepository.GetPages(new DBPageRequest
            {
                CurrentPage = pageIndex,
                PageSize = pageSize,
                Filter = filter,
            });
            var resultData = new List<SimpleRecordInfoDTO>();
            if (pagedResult.PageData?.Any() ?? false)
            {
                var patients = await FindPatientInfoListByCodeListAsync(pagedResult.PageData.Select(x => x.PatientCode).ToList());
                var devices = await FindDeviceInfoListByCodeListAsync(pagedResult.PageData.Select(x => x.DeviceCode).ToList());
                var orgs = await FindOrganizationDTOListByCodesAsync(pagedResult.PageData.Select(x => x.RootOrganizationCode).ToList());

                foreach (var item in pagedResult.PageData)
                {
                    var detail = new SimpleRecordInfoDTO();
                    detail = item.MappingTo<SimpleRecordInfoDTO>();
                    var patient = patients.FirstOrDefault(x => x.PatientCode == item.PatientCode);
                    var device = devices.FirstOrDefault(x => x.DeviceCode == item.DeviceCode);
                    var organization = orgs.FirstOrDefault(x => x.OrganizationCode == item.RootOrganizationCode);
                    if (patient != null)
                    {
                        detail.PatientName = string.IsNullOrWhiteSpace(patient.Name) ? GetNotFilled(language) : CryptionHelper.NameEncrypt(patient.Name, false);
                        detail.Age = GetPatientAge(patient.PatientData, language);
                        detail.Sex = GetPatientGender(patient.PatientData, language);
                    }
                    detail.DeviceName = device?.DisplayName ?? string.Empty;
                    detail.DeviceHeadPicUrl = device?.HeadPicUrl;
                    detail.RootOrganizationName = organization?.OrganizationName;
                    var isCanCreateReport = item.RecordStatus == RecordStatusEnum.Uploaded || item.RecordStatus == RecordStatusEnum.NotReport || item.RecordStatus == RecordStatusEnum.Completed;
                    var existUserReport = await ExistRecordReportsByUserAsync(item.RecordCode, userCode);
                    detail.CanCreateReport = isCanCreateReport && !existUserReport;
                    //采图
                    detail.IsCollecting = item.IsCollecting;
                    detail.CanCollcetImg = GetCanCollcetImg(patient == null ? false : patient.IsValid, item.RecordStatus, item.IsCollecting, device?.DeviceType, item.IsFromDicom);
                    detail.CustomDoctor = item.CustomDoctor;
                    detail.CustomOrganzation = item.CustomOrganzation;
                    detail.EquipmentSN = item.EquipmentSN;
                    //截图版分时诊断
                    var isCanScreenshot = item.CreateType == RecordCreateTypeEnum.Reservation && string.IsNullOrWhiteSpace(item.DeviceCode) && item.CreatorCode == userCode;
                    detail.CanScreenshot = isCanScreenshot && (detail.RecordStatus == RecordStatusEnum.NotScanned || detail.RecordStatus == RecordStatusEnum.Uploaded);
                    //质控
                    detail.IsQualityControlled = item.IsImgQualityControlled || item.IsReportQualityControlled;
                    detail.Score = item.ImgScore + item.ReportScore;
                    resultData.Add(detail);
                }
            }

            var result = new PageResult<SimpleRecordInfoDTO>();
            result.PageIndex = pagedResult.CurrentPage;
            result.PageSize = pagedResult.PageSize;
            result.TotalCount = pagedResult.DataCount;
            result.PageData = resultData;
            return result;
        }

        /// <summary>
        /// 查询检查记录大列表（分页）
        /// </summary>
        /// <param name="request">查询检查记录列表（分页）请求实体</param>
        /// <returns></returns>
        public async Task<PageResult<SimpleRecordInfoDTO>> FindRecordPagesAsync(FindFullRecordInfoPagesRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} FindRecordPagesAsync execute", $"{this.GetType().Name}"))
            {
                var language = request.Language;
                var dbPage = new DBPageRequest();
                if (request.PatientCodes != null && request.PatientCodes.Count > 0)
                {
                    var customerFilter = await GetPatientRecordInfoPagesFilter(request);
                    dbPage.Filter = customerFilter;
                }
                else
                {
                    var customerFilter = await GetRecordInfoPagesFilter(request);
                    dbPage.Filter = customerFilter;
                }
                dbPage.CurrentPage = request.CurrentPage;
                dbPage.PageSize = request.PageSize;
                if (request.Filter.Any(x => x.Key == "Sort"))
                {
                    var value = request.Filter.FirstOrDefault(x => x.Key == "Sort").Value;
                    if (value == "StartCollectingTime desc")
                    {
                        dbPage.Sort = Builders<MongoTable>.Sort.Descending("BaseData.StartCollectingTime");
                    }
                }
                var recordResult = await _recordInfoDBRepository.GetPages(dbPage);
                PageResult<SimpleRecordInfoDTO> result = new PageResult<SimpleRecordInfoDTO>();
                result.PageIndex = recordResult.CurrentPage;
                result.PageSize = recordResult.PageSize;
                result.TotalCount = recordResult.DataCount;
                result.PageData = new List<SimpleRecordInfoDTO>();
                if (recordResult.PageData != null && recordResult.PageData.Count > 0)
                {
                    var userInfo = await FindUserByCodeAsync(request.UserCode);
                    if (userInfo != null && !string.IsNullOrWhiteSpace(userInfo.UserCode))
                    {
                        var orgRequest = new GetOrganizationDBRequest { OrganizationCode = userInfo.RootOrganizationCode };
                        var rootOrg = await GetOrganizationDetailAsync(orgRequest) ?? new WingInterfaceLibrary.DTO.Organization.OrganizationDTO() { IsEncryptedShow = false };
                        var patients = await FindPatientInfoListByCodeListAsync(recordResult.PageData.Select(x => x.PatientCode).ToList());
                        var devices = await FindDeviceInfoListByCodeListAsync(recordResult.PageData.Select(x => x.DeviceCode).ToList());
                        var orgs = await FindOrganizationDTOListByCodesAsync(recordResult.PageData.Select(x => x.RootOrganizationCode).ToList());
                        Dictionary<string, List<string>> filter = new Dictionary<string, List<string>>();
                        filter.Add("RecordCode", recordResult.PageData.Select(x => x.RecordCode).ToList());
                        var referralDatas = await _referralDataDBRepository.FindInAsync(filter);
                        foreach (var item in recordResult.PageData)
                        {
                            var detail = new SimpleRecordInfoDTO();
                            detail = item.MappingTo<SimpleRecordInfoDTO>();
                            var patient = patients.FirstOrDefault(x => x.PatientCode == item.PatientCode);
                            var device = devices.FirstOrDefault(x => x.DeviceCode == item.DeviceCode);
                            var organization = orgs.FirstOrDefault(x => x.OrganizationCode == item.RootOrganizationCode);
                            if (patient != null)
                            {
                                detail.PatientName = string.IsNullOrWhiteSpace(patient.Name) ? GetNotFilled(language) : CryptionHelper.NameEncrypt(patient.Name, rootOrg.IsEncryptedShow);
                                detail.Age = GetPatientAge(patient.PatientData, language);
                                detail.Sex = GetPatientGender(patient.PatientData, language);
                            }
                            detail.DeviceName = device?.DisplayName ?? string.Empty;
                            detail.DeviceHeadPicUrl = device?.HeadPicUrl;
                            detail.RootOrganizationName = organization?.OrganizationName;
                            if (item.IsReferral)
                            {
                                var itemReferralDatas = referralDatas.Where(x => x.RecordCode == item.Code);
                                if (itemReferralDatas.Any(x => x.ReferralInOrganizationCode == userInfo.RootOrganizationCode && x.ReferralStatus != RecordReferralStatusEnum.Withdrawn))
                                {
                                    detail.ReferralType = ReferralTypeEnum.ReferralIn;
                                }
                                //如果再转给了别人则会覆盖成已转出
                                if (itemReferralDatas.Any(x => x.ReferralOutOrganizationCode == userInfo.RootOrganizationCode && x.ReferralStatus != RecordReferralStatusEnum.Withdrawn))
                                {
                                    detail.ReferralType = ReferralTypeEnum.ReferralOut;
                                }
                            }
                            //var isCanCreateReport = (item.RecordStatus == RecordStatusEnum.Uploaded || item.RecordStatus == RecordStatusEnum.NotReport || item.RecordStatus == RecordStatusEnum.Completed);
                            //var existUserReport = await ExistRecordReportsByUserAsync(item.RecordCode, request.UserCode);
                            detail.CanCreateReport = true;//isCanCreateReport && !existUserReport;
                            //采图
                            detail.IsCollecting = item.IsCollecting;
                            detail.CanCollcetImg = GetCanCollcetImg(patient == null ? false : patient.IsValid, item.RecordStatus, item.IsCollecting, device?.DeviceType, item.IsFromDicom);
                            detail.CustomDoctor = item.CustomDoctor;
                            detail.CustomOrganzation = item.CustomOrganzation;
                            detail.EquipmentSN = item.EquipmentSN;
                            //截图版分时诊断
                            var isCanScreenshot = item.CreateType == RecordCreateTypeEnum.Reservation && string.IsNullOrWhiteSpace(item.DeviceCode) && item.CreatorCode == userInfo.UserCode;
                            detail.CanScreenshot = isCanScreenshot && (detail.RecordStatus == RecordStatusEnum.NotScanned || detail.RecordStatus == RecordStatusEnum.Uploaded);
                            //质控
                            detail.IsQualityControlled = item.IsImgQualityControlled || item.IsReportQualityControlled;
                            detail.Score = item.ImgScore + item.ReportScore;
                            result.PageData.Add(detail);
                        }
                    }
                    else
                    {
                        var patients = await FindPatientInfoListByCodeListAsync(recordResult.PageData.Select(x => x.PatientCode).ToList());
                        var devices = await FindDeviceInfoListByCodeListAsync(recordResult.PageData.Select(x => x.DeviceCode).ToList());
                        var orgs = await FindOrganizationDTOListByCodesAsync(recordResult.PageData.Select(x => x.RootOrganizationCode).ToList());
                        Dictionary<string, List<string>> filter = new Dictionary<string, List<string>>();
                        filter.Add("RecordCode", recordResult.PageData.Select(x => x.RecordCode).ToList());
                        var referralDatas = await _referralDataDBRepository.FindInAsync(filter);
                        foreach (var item in recordResult.PageData)
                        {
                            var detail = new SimpleRecordInfoDTO();
                            detail = item.MappingTo<SimpleRecordInfoDTO>();
                            var patient = patients.FirstOrDefault(x => x.PatientCode == item.PatientCode);
                            var device = devices.FirstOrDefault(x => x.DeviceCode == item.DeviceCode);
                            var organization = orgs.FirstOrDefault(x => x.OrganizationCode == item.RootOrganizationCode);
                            if (patient != null)
                            {
                                detail.PatientName = string.IsNullOrWhiteSpace(patient.Name) ? GetNotFilled(language) : CryptionHelper.NameEncrypt(patient.Name, organization.IsEncryptedShow);
                                detail.Age = GetPatientAge(patient.PatientData, language);
                                detail.Sex = GetPatientGender(patient.PatientData, language);
                            }
                            detail.DeviceName = device?.DisplayName ?? string.Empty;
                            detail.DeviceHeadPicUrl = device?.HeadPicUrl;
                            detail.RootOrganizationName = organization?.OrganizationName;
                            //var isCanCreateReport = (item.RecordStatus == RecordStatusEnum.Uploaded || item.RecordStatus == RecordStatusEnum.NotReport || item.RecordStatus == RecordStatusEnum.Completed);
                            //var existUserReport = await ExistRecordReportsByUserAsync(item.RecordCode, request.UserCode);
                            detail.CanCreateReport = true;//isCanCreateReport && !existUserReport;
                            //采图
                            detail.IsCollecting = item.IsCollecting;
                            detail.CanCollcetImg = GetCanCollcetImg(patient == null ? false : patient.IsValid, item.RecordStatus, item.IsCollecting, device?.DeviceType, item.IsFromDicom);
                            detail.CustomDoctor = item.CustomDoctor;
                            detail.CustomOrganzation = item.CustomOrganzation;
                            detail.EquipmentSN = item.EquipmentSN;
                            //截图版分时诊断
                            detail.CanScreenshot = false;
                            //质控
                            detail.IsQualityControlled = item.IsImgQualityControlled || item.IsReportQualityControlled;
                            detail.Score = item.ImgScore + item.ReportScore;
                            result.PageData.Add(detail);
                        }
                    }
                }
                return result;
            }
        }

        /// <summary>
        /// 查询检查记录列表（分页）
        /// </summary>
        /// <param name="request">查询检查记录列表（分页）请求实体</param>
        /// <returns></returns>
        public async Task<PageCollection<RecordInfoDTO>> FindRecordInfoPagesAsync(FindRecordInfoPagesRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} FindRecordInfoPagesAsync execute", $"{this.GetType().Name}"))
            {
                FilterDefinition<MongoTable> customerFilter = await GetRecordInfoPagesFilter(request);
                var dbPage = new DBPageRequest();
                dbPage.CurrentPage = request.CurrentPage;
                dbPage.PageSize = request.PageSize;
                dbPage.Filter = customerFilter;
                if (request.Filter.Any(x => x.Key == "Sort"))
                {
                    var value = request.Filter.FirstOrDefault(x => x.Key == "Sort").Value;
                    if (value == "StartCollectingTime desc")
                    {
                        dbPage.Sort = Builders<MongoTable>.Sort.Descending("BaseData.StartCollectingTime");
                    }
                }
                var recordResult = await _recordInfoDBRepository.GetPages(dbPage);
                return PageCollectionConvert<RecordInfoDO, RecordInfoDTO>(recordResult);
            }
        }

        /// <summary>
        /// 查询完整数据检查记录列表（分页）请求实体
        /// </summary>
        /// <param name="request">查询检查记录列表（分页）请求实体</param>
        /// <returns></returns>
        public async Task<PageResult<GetRecordsPageDTO>> FindFullRecordInfoPagesAsync(FindFullRecordInfoPagesRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} FindFullRecordInfoPagesAsync execute", $"{this.GetType().Name}"))
            {
                var result = new PageResult<GetRecordsPageDTO>();
                var customerFilter = await GetRecordInfoPagesFilter(request);
                var dbPage = new DBPageRequest();
                dbPage.CurrentPage = request.CurrentPage;
                dbPage.PageSize = request.PageSize;
                dbPage.Filter = customerFilter;
                var list = await _recordInfoDBRepository.GetPages(dbPage);
                if (list.DataCount > 0)
                {
                    var userInfos = await FindUserListByUserCodesAsync(list.PageData.Select(r => r.CreatorCode).ToList());
                    var devices = await FindDeviceInfoListByCodeListAsync(list.PageData.Select(r => r.DeviceCode).ToList());
                    var recordCodes = list.PageData.Select(r => r.RecordCode).ToList();
                    var patientCodes = list.PageData.Select(r => r.PatientCode).ToList();
                    var reports = await GetReportsByRecordCodesAsync(recordCodes);
                    var remedicalLists = await GetRemedicalListByRecordCodeAsync(recordCodes);
                    var patients = await FindPatientInfoListByCodeListAsync(patientCodes);
                    foreach (var item in list.PageData)
                    {
                        var device = devices.FirstOrDefault(d => d.DeviceCode == item.DeviceCode);
                        var itemUserInfo = userInfos.FirstOrDefault(u => u.UserCode == item.CreatorCode) ?? new UserDTO();
                        var organizationName = await GetOrganizationNameByCondition(itemUserInfo, item.OrganizationCode, item.RootOrganizationCode);
                        var detail = new GetRecordsPageDTO();
                        detail.CreateTime = item.CreateTime;
                        detail.DeptName = organizationName ?? string.Empty;
                        detail.CreatorName = string.IsNullOrWhiteSpace(itemUserInfo?.FullName) ? itemUserInfo?.UserName : itemUserInfo?.FullName;
                        detail.DeviceName = device?.Name;
                        detail.DisplayName = device?.DisplayName;
                        detail.DeviceCode = device?.DeviceCode;
                        detail.ReportNum = reports.Count(r => r.RecordCode == item.RecordCode).ToString();
                        detail.RecordCode = item.RecordCode;
                        detail.RecordStatus = item.RecordStatus;
                        detail.IsRead = item.ReadUsers.Contains(request.UserCode);
                        detail.DiagnosisStatus = item.DiagnosisStatus;
                        detail.DiagnosisInfos = item.DiagnosisInfos.MappingTo<List<DiagnosisInfoDTO>>();
                        var isCanCreateReport = (item.RecordStatus == RecordStatusEnum.Uploaded || item.RecordStatus == RecordStatusEnum.NotReport || item.RecordStatus == RecordStatusEnum.Completed);
                        var existUserReport = await ExistRecordReportsByUserAsync(item.RecordCode, request.UserCode);
                        detail.CanCreateReport = isCanCreateReport && !existUserReport;
                        //采图
                        var patient = patients.FirstOrDefault(x => x.PatientCode == item.PatientCode);
                        detail.IsCollecting = item.IsCollecting;
                        detail.CanCollcetImg = GetCanCollcetImg(patient.IsValid, item.RecordStatus, item.IsCollecting, device?.DeviceType, item.IsFromDicom);
                        var remedicalList = remedicalLists.Where(r => r.RecordCode == item.RecordCode);
                        if (remedicalList != null && remedicalList.Count() > 0)
                        {
                            foreach (var remedical in remedicalList)
                            {
                                foreach (var remedicalItem in remedical.RemedicalItemList)
                                {
                                    if (remedicalItem.RemedicalList != null && remedicalItem.RemedicalList.Count > 0)
                                    {
                                        detail.RemedicalList.AddRange(remedicalItem.RemedicalList);
                                    }
                                }
                            }
                        }
                        result.PageData.Add(detail);
                        result.PageIndex = list.CurrentPage;
                        result.PageSize = list.PageSize;
                        result.TotalCount = list.DataCount;
                    }
                }
                return result;
            }
        }

        public async Task<IList<GetRecordsPageDTO>> GetRecordsPageDBAsync(GetRecordsPageDBRequest request)
        {
            var userCode = request.OperatorCode;
            var records = request.Records;

            var resultData = new List<GetRecordsPageDTO>();
            if (records != null && records.Any())
            {
                var userInfos = await FindUserListByUserCodesAsync(records.Select(r => r.CreatorCode).ToList());
                var devices = await FindDeviceInfoListByCodeListAsync(records.Select(r => r.DeviceCode).ToList());
                var recordCodes = records.Select(r => r.RecordCode).ToList();
                var patientCodes = records.Select(r => r.PatientCode).ToList();
                var reports = await GetReportsByRecordCodesAsync(recordCodes);
                var remedicalLists = await GetRemedicalListByRecordCodeAsync(recordCodes);
                var patients = await FindPatientInfoListByCodeListAsync(patientCodes);
                foreach (var item in records)
                {
                    var device = devices.FirstOrDefault(d => d.DeviceCode == item.DeviceCode);
                    var itemUserInfo = userInfos.FirstOrDefault(u => u.UserCode == item.CreatorCode) ?? new UserDTO();
                    var organizationName = await GetOrganizationNameByCondition(itemUserInfo, item.OrganizationCode, item.RootOrganizationCode);
                    var detail = new GetRecordsPageDTO();
                    detail.CreateTime = item.CreateTime;
                    detail.DeptName = organizationName ?? string.Empty;
                    detail.CreatorName = string.IsNullOrWhiteSpace(itemUserInfo?.FullName) ? itemUserInfo?.UserName : itemUserInfo?.FullName;
                    detail.DeviceName = device?.Name;
                    detail.DisplayName = device?.DisplayName;
                    detail.DeviceCode = device?.DeviceCode;
                    detail.ReportNum = reports.Count(r => r.RecordCode == item.RecordCode).ToString();
                    detail.RecordCode = item.RecordCode;
                    detail.RecordStatus = item.RecordStatus;
                    detail.IsRead = item.ReadUsers.Contains(userCode);
                    detail.DiagnosisStatus = item.DiagnosisStatus;
                    detail.DiagnosisInfos = item.DiagnosisInfos.MappingTo<List<DiagnosisInfoDTO>>();
                    var isCanCreateReport = (item.RecordStatus == RecordStatusEnum.Uploaded || item.RecordStatus == RecordStatusEnum.NotReport || item.RecordStatus == RecordStatusEnum.Completed);
                    var existUserReport = await ExistRecordReportsByUserAsync(item.RecordCode, userCode);
                    detail.CanCreateReport = isCanCreateReport && !existUserReport;
                    //采图
                    var patient = patients.FirstOrDefault(x => x.PatientCode == item.PatientCode);
                    detail.IsCollecting = item.IsCollecting;
                    detail.CanCollcetImg = GetCanCollcetImg(patient.IsValid, item.RecordStatus, item.IsCollecting, device?.DeviceType, item.IsFromDicom);
                    var remedicalList = remedicalLists.Where(r => r.RecordCode == item.RecordCode);
                    if (remedicalList != null && remedicalList.Count() > 0)
                    {
                        foreach (var remedical in remedicalList)
                        {
                            foreach (var remedicalItem in remedical.RemedicalItemList)
                            {
                                if (remedicalItem.RemedicalList != null && remedicalItem.RemedicalList.Count > 0)
                                {
                                    detail.RemedicalList.AddRange(remedicalItem.RemedicalList);
                                }
                            }
                        }
                    }
                    resultData.Add(detail);
                }
            }
            return resultData;
        }

        /// <summary>
        /// 是否可采图
        /// 已开启则不能点，已传图、已结束扫查则不能点,非魔盒不能截图版，未登记不能采图
        /// </summary>
        /// <param name="recordStatus"></param>
        /// <param name="isCollecting"></param>
        /// <returns></returns>
        private bool GetCanCollcetImg(bool patientIsvalid, RecordStatusEnum recordStatus, bool isCollecting, string deviceType, bool isFromDicom)
        {
            if (!patientIsvalid)
            {
                return false;
            }
            if (deviceType == null || deviceType.ToLower() != "sonopost")
            {
                return false;
            }
            if (isCollecting)
            {
                return false;
            }
            if (isFromDicom)
            {
                return false;
            }
            if (recordStatus == RecordStatusEnum.Uploaded || recordStatus == RecordStatusEnum.Completed || recordStatus == RecordStatusEnum.NotReport)
            {
                return false;
            }
            return true;
        }

        /// <summary>
        /// 获取机构名称，上下逻辑一致
        /// </summary>
        /// <param name="request">查询检查记录列表（分页）请求实体</param>
        /// <returns></returns>
        private async Task<string> GetOrganizationNameByCondition(UserDTO creatorInfo, string organizationCode, string rootOrganizationCode)
        {
            var organizationName = string.Empty;
            if (!string.IsNullOrEmpty(organizationCode))
            {
                var orgInfo = CacheMaintenance.Instance.Get<IOrganizationsManager>().Get(organizationCode);
                if (orgInfo != null)
                {
                    if (!string.IsNullOrEmpty(orgInfo.OrganizationName))
                    {
                        organizationName = orgInfo.OrganizationName;
                    }
                }
            }
            if (string.IsNullOrEmpty(organizationName) && !string.IsNullOrEmpty(rootOrganizationCode))
            {
                var org = CacheMaintenance.Instance.Get<IOrganizationsManager>().Get(rootOrganizationCode);
                if (org != null && !string.IsNullOrEmpty(org.OrganizationName))
                {
                    organizationName = org.OrganizationName;
                }
            }

            return organizationName;
        }

        /// <summary>
        /// 查询检查记录数量
        /// </summary>
        /// <param name="request">查询检查记录列表（分页）请求实体</param>
        /// <returns></returns>
        public async Task<long> FindRecordInfoNumAsync(FindRecordInfoNumRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} FindRecordInfoNumAsync execute", $"{this.GetType().Name}"))
            {
                var filterRequest = new FindRecordInfoPagesRequest();
                filterRequest.Filter = request.Filter;
                filterRequest.DeviceCodes = request.DeviceCodes;
                filterRequest.PatientCodes = request.PatientCodes;
                var customerFilter = await GetRecordInfoPagesFilter(filterRequest);
                var result = await _recordInfoDBRepository.CountDocumentsAsync(customerFilter);
                return result;
            }
        }





        /// <summary>
        /// 获取检查记录列表（分页）条件拼接
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        private async Task<FilterDefinition<MongoTable>> GetRecordInfoPagesFilter(FindRecordInfoPagesRequest request)
        {
            var queryFilter = request.Filter;
            var deviceCodes = request.DeviceCodes;
            var patientCodes = request.PatientCodes;
            var organizationCodes = request.OrganizationCodes;
            var recordCodes = request.RecordCodes;
            string userCode = request.UserCode;
            var recordProcessState = request.RecordProcessState;

            MongoCustomerFilter<MongoTable> customerFilter = new();//常规查询
            MongoCustomerFilter<MongoDataItem> queryItemsFilter = new();
            MongoCustomerFilter<MongoTable> customerOrFilter = new();//用户不同权限查询
            MongoCustomerFilter<MongoTable> deviceAndOrgFilter = new();//绑定的设备和机构
            MongoCustomerFilter<MongoTable> tabOrFilter = new();//待处理已处理已转出查询处理
            MongoCustomerFilter<MongoTable> referralOutFilter = new();//已转出
            MongoCustomerFilter<MongoTable> processedFilter = new();//已处理
            MongoCustomerFilter<MongoTable> customerKeyWordFilter = new();//关键字查询
            if (queryFilter != null && queryFilter.Count > 0)
            {
                foreach (var filter in queryFilter)
                {
                    if (filter.Key == "UserCode")
                    {   //查询未读数据
                        customerFilter.AppendNotArrayEq("BaseData.ReadUsers", filter.Value);
                    }
                    else if (filter.Key == "CreateTime>")
                    {
                        var startTime = filter.Value.ToDateTime();
                        if (startTime > new DateTime(2000, 1, 1))
                        {
                            customerFilter.AppendGt("BaseData.CreateTime", startTime);
                        }
                    }
                    else if (filter.Key == "CreateTime<=")
                    {
                        var endTime = filter.Value.ToDateTime();
                        if (endTime > new DateTime(2000, 1, 1))
                        {
                            customerFilter.AppendLte("BaseData.CreateTime", endTime);
                        }
                    }
                    else if (filter.Key == "PatientName")
                    {
                        var patientList = GetPatientListByQueryDataAsync(new PageFilterRequest
                        {
                            PageSize = 999999,
                            CurrentPage = 1,
                            IsFuzzy = true,
                            Filter = new Dictionary<string, string> { ["Name"] = filter.Value }
                        });
                        queryItemsFilter = new();
                        queryItemsFilter.AppendEq((item => item.Key, "PatientCode"));
                        if (patientList == null || patientList.Result == null || patientList.Result.PageData.Count == 0)
                        {
                            queryItemsFilter.AppendEq((item => item.Value, "-1"));
                            if (queryItemsFilter.HasFilter)
                            {
                                customerFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
                            }
                        }
                        else
                        {
                            queryItemsFilter.AppendIn(item => item.Value, patientList.Result.PageData.Select(p => p.PatientCode).ToList());
                            if (queryItemsFilter.HasFilter)
                            {
                                customerFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
                            }
                        }
                    }
                    else if (filter.Key == "KeyWord")
                    {
                        //关键字 支持Name、DevicePatientID、手机号、年龄、性别
                        var keyword = filter.Value;
                        var keywordfilter = new Dictionary<string, string>();
                        var keywordDictionary = new Dictionary<string, string>();
                        var tempKeyWord = CryptionHelper.EncryptOneByOne(keyword);
                        keywordDictionary.Add("Name", tempKeyWord);
                        keywordDictionary.Add("Phone", tempKeyWord);
                        //keywordDictionary.Add("DevicePatientID", keyword);
                        keywordDictionary.Add("Age", keyword);
                        keywordDictionary.Add("Gender", keyword);
                        var pageFilterRequest = new GetPatientsByOrgPageRequest()
                        {
                            KeywordOrFilter = keywordDictionary,
                            IsFuzzy = true
                        };
                        var patientList = GetPatientsDBAsync(pageFilterRequest).Result;
                        queryItemsFilter = new();
                        queryItemsFilter.AppendEq((item => item.Key, "PatientCode"));
                        if (patientList == null || patientList.Count == 0)
                        {
                            queryItemsFilter.AppendEq((item => item.Value, "-1"));
                            if (queryItemsFilter.HasFilter)
                            {
                                customerKeyWordFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
                            }
                        }
                        else
                        {
                            queryItemsFilter.AppendIn(item => item.Value, patientList.Select(p => p.PatientCode).ToList());
                            if (queryItemsFilter.HasFilter)
                            {
                                customerKeyWordFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
                            }
                        }
                        customerKeyWordFilter.AppendRegexForKey("BaseData.DevicePatientID", keyword, true);
                    }
                    else if (filter.Key == "RecordStatus" && filter.Value == QueryRecordStatusEnum.NotCompleted.ToString())
                    {
                        queryItemsFilter = new();
                        queryItemsFilter.AppendEq((item => item.Key, "RecordStatus"));
                        queryItemsFilter.AppendIn(item => item.Value, new List<string> { RecordStatusEnum.NotScanned.ToString(), RecordStatusEnum.Uploaded.ToString(), RecordStatusEnum.NotReport.ToString() });
                        if (queryItemsFilter.HasFilter)
                        {
                            customerFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
                        }
                    }
                    else if (filter.Key == "ExamStatus")//查询状态
                    {
                        queryItemsFilter = new();
                        queryItemsFilter.AppendEq((item => item.Key, "RecordStatus"));
                        if (filter.Value == ExamStatusEnum.Wait.ToString())
                        {
                            queryItemsFilter.AppendIn(item => item.Value, new List<string> { RecordStatusEnum.NotScanned.ToString(), RecordStatusEnum.Uploaded.ToString() });
                        }
                        else
                        {
                            queryItemsFilter.AppendIn(item => item.Value, new List<string> { RecordStatusEnum.NotReport.ToString(), RecordStatusEnum.Completed.ToString() });
                        }
                        if (queryItemsFilter.HasFilter)
                        {
                            customerFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
                        }
                    }
                    else if (filter.Key == "IsCollecting")//查询采图状态
                    {
                        if (filter.Value?.ToLower() == "true")
                        {
                            customerFilter.AppendEq(("BaseData.IsCollecting", true));
                        }
                        else
                        {
                            customerFilter.AppendEq(("BaseData.IsCollecting", false));
                        }
                    }
                    else
                    {
                        if (filter.Key == "Sort")
                        {
                            continue;
                        }
                        queryItemsFilter = new();
                        queryItemsFilter.AppendEq((item => item.Key, filter.Key));
                        queryItemsFilter.AppendRegex(item => item.Value, filter.Value, true);
                        if (queryItemsFilter.HasFilter)
                        {
                            customerFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
                        }
                    }
                }
            }
            if (deviceCodes != null && deviceCodes.Count > 0)
            {
                queryItemsFilter = new();
                queryItemsFilter.AppendEq((item => item.Key, "DeviceCode"));
                queryItemsFilter.AppendIn(item => item.Value, deviceCodes);
                if (queryItemsFilter.HasFilter)
                {
                    customerFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
                }
            }
            if (organizationCodes != null && organizationCodes.Count > 0)
            {
                queryItemsFilter = new();
                queryItemsFilter.AppendEq((item => item.Key, "RootOrganizationCode"));
                queryItemsFilter.AppendIn(item => item.Value, organizationCodes);
                if (queryItemsFilter.HasFilter)
                {
                    customerFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
                }
            }
            if (recordCodes != null && recordCodes.Count > 0)
            {
                queryItemsFilter = new();
                queryItemsFilter.AppendEq((item => item.Key, "Code"));
                queryItemsFilter.AppendIn(item => item.Value, recordCodes);
                if (queryItemsFilter.HasFilter)
                {
                    customerFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
                }
            }
            if (patientCodes != null && patientCodes.Count > 0)
            {
                queryItemsFilter = new();
                queryItemsFilter.AppendEq((item => item.Key, "PatientCode"));
                queryItemsFilter.AppendIn(item => item.Value, patientCodes);
                if (queryItemsFilter.HasFilter)
                {
                    customerFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
                }
            }

            //用户可以查到的数据应该是个人分配的设备+自己创建
            if (!string.IsNullOrWhiteSpace(userCode))
            {
                var userInfo = await FindUserByCodeAsync(userCode);
                if (!request.NotFilterByFeature)
                {
                    var userOrganizationCodes = new List<string> { userInfo.RootOrganizationCode };
                    //如果是机构负责人
                    if (userInfo.IsDirector)
                    {
                        var org = await _organizationDBRepository.FindOneAsync("Code", userInfo.RootOrganizationCode);
                        //查到该机构所有的子机构
                        var needOrganizations = new List<OrganizationDO>();
                        GetAllChildOrganizationDO(org, ref needOrganizations);
                        if (needOrganizations.Count > 0)
                        {
                            userOrganizationCodes.AddRange(needOrganizations.Select(x => x.Code));
                        }
                        queryItemsFilter.ClearFilter();
                        queryItemsFilter.AppendEq((item => item.Key, "RootOrganizationCode"));
                        queryItemsFilter.AppendIn(item => item.Value, userOrganizationCodes);
                        customerOrFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());

                        //本机构及下级机构的人创建的
                        if (userOrganizationCodes.Count > 0)
                        {
                            //查询包含部门在内的机构
                            MongoCustomerFilter<MongoTable> orgCustomerFilter = new();
                            MongoCustomerFilter<MongoDataItem> orgQueryItemsFilter = new();
                            orgQueryItemsFilter.AppendEq((item => item.Key, "RootCode"));
                            orgQueryItemsFilter.AppendIn(item => item.Value, userOrganizationCodes);
                            orgCustomerFilter.SubFilter(item => item.QueryData.QueryItems, orgQueryItemsFilter.AndCombine());
                            var organizations = await _organizationDBRepository.FindAllAsync(orgCustomerFilter.AndCombine());
                            //查询用户
                            MongoCustomerFilter<MongoTable> userCustomerFilter = new();
                            MongoCustomerFilter<MongoDataItem> userQueryItemsFilter = new();
                            userQueryItemsFilter.ClearFilter();
                            userQueryItemsFilter.AppendEq((item => item.Key, "OrganizationCode"));
                            userQueryItemsFilter.AppendIn(item => item.Value, organizations.Select(x => x.OrganizationCode).ToList());
                            userCustomerFilter.SubFilter(item => item.QueryData.QueryItems, userQueryItemsFilter.AndCombine());
                            var users = await _userDBRepository.FindAllAsync(userCustomerFilter.AndCombine());
                            queryItemsFilter.ClearFilter();
                            queryItemsFilter.AppendEq((item => item.Key, "CreatorCode"));
                            queryItemsFilter.AppendIn(item => item.Value, users.Select(x => x.Code).ToList());
                            customerOrFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
                        }
                    }
                    //自己创建的
                    queryItemsFilter.ClearFilter();
                    queryItemsFilter.AppendEq((item => item.Key, "CreatorCode"), (item => item.Value, userInfo.UserCode));
                    customerOrFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
                    //分配自己的设备和机构对应的检查的病人
                    if (userInfo.BindDevices != null && userInfo.BindDevices.Count > 0)
                    {
                        MongoCustomerFilter<MongoDataItem> recordItemFilter = new();
                        recordItemFilter.AppendEq((item => item.Key, "DeviceCode"));
                        recordItemFilter.AppendIn(item => item.Value, userInfo.BindDevices.ToList());
                        deviceAndOrgFilter.SubFilter(item => item.QueryData.QueryItems, recordItemFilter.AndCombine());
                        recordItemFilter.ClearFilter();
                        recordItemFilter.AppendEq((item => item.Key, "RootOrganizationCode"));
                        recordItemFilter.AppendIn(item => item.Value, userInfo.BindDeviceOrganizations.ToList());
                        deviceAndOrgFilter.SubFilter(item => item.QueryData.QueryItems, recordItemFilter.AndCombine());
                    }
                    //包括转入的
                    List<ReferralDataEntity> referralDatas = new List<ReferralDataEntity>();
                    //机构负责人查询转入本机构的，其他人查询转入本人的
                    if (userInfo.IsDirector)
                    {
                        FilterDefinition<ReferralDataEntity> referralDataFliter = Builders<ReferralDataEntity>.Filter.Where(x => x.ReferralInOrganizationCode == userInfo.RootOrganizationCode && x.ReferralStatus != RecordReferralStatusEnum.Withdrawn);
                        referralDatas = await _referralDataDBRepository.FindAllAsync(referralDataFliter);
                    }
                    else
                    {
                        FilterDefinition<ReferralDataEntity> referralDataFliter = Builders<ReferralDataEntity>.Filter.Where(x => x.ReferralInUserCode == userInfo.UserCode && x.ReferralStatus != RecordReferralStatusEnum.Withdrawn);
                        referralDatas = await _referralDataDBRepository.FindAllAsync(referralDataFliter);
                    }
                    if (referralDatas.Count > 0)
                    {
                        var records = referralDatas.Select(x => x.RecordCode).ToList();
                        if (records != null && records.Count() > 0)
                        {
                            queryItemsFilter.ClearFilter();
                            queryItemsFilter.AppendEq((item => item.Key, "Code"));
                            queryItemsFilter.AppendIn(item => item.Value, records);
                            customerOrFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
                        }
                    }
                }
                //以上是全部可以看到的数据，下面做筛选过滤
                if (recordProcessState != RecordProcessStateEnum.All)//查询状态
                {
                    //待扫查、已传图、已扫查、已报告（仅出了AI报告的检查），以及其他医院转入的检查（加上转入标识）
                    if (recordProcessState == RecordProcessStateEnum.Wait)
                    {
                        //医生没有出报告的
                        tabOrFilter.AppendNot(("BaseData.IsUserReport", true));
                        //转入未处理的
                        FilterDefinition<ReferralDataEntity> waitReferralDataFliter = Builders<ReferralDataEntity>.Filter.Where(x => x.ReferralInOrganizationCode == userInfo.RootOrganizationCode && x.ReferralStatus == RecordReferralStatusEnum.ReferralIn);
                        var waitReferralDatas = await _referralDataDBRepository.FindAllAsync(waitReferralDataFliter);
                        var waitRecords = waitReferralDatas.Select(x => x.RecordCode).ToList();
                        if (waitRecords != null && waitRecords.Count() > 0)
                        {
                            queryItemsFilter.ClearFilter();
                            queryItemsFilter.AppendEq((item => item.Key, "Code"));
                            queryItemsFilter.AppendIn(item => item.Value, waitRecords);
                            tabOrFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
                        }
                        //但是不包括已转出
                        var subReferralOutFilter = Builders<ReferralDataEntity>.Filter.Where(x => x.ReferralOutOrganizationCode == userInfo.RootOrganizationCode && x.ReferralStatus != RecordReferralStatusEnum.Withdrawn);
                        var referralOutDatas = await _referralDataDBRepository.FindAllAsync(subReferralOutFilter);
                        var referralOutRecords = referralOutDatas.Select(x => x.RecordCode).ToList();
                        if (referralOutRecords != null && referralOutRecords.Count() > 0)
                        {
                            queryItemsFilter.ClearFilter();
                            queryItemsFilter.AppendEq((item => item.Key, "Code"));
                            queryItemsFilter.AppendIn(item => item.Value, referralOutRecords);
                            referralOutFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
                        }

                    }
                    else if (recordProcessState == RecordProcessStateEnum.Done)
                    {
                        //医生出了诊断报告
                        processedFilter.AppendEq(("BaseData.IsUserReport", true));
                        //不包括已转出
                        var subReferralOutFilter = Builders<ReferralDataEntity>.Filter.Where(x => x.ReferralOutOrganizationCode == userInfo.RootOrganizationCode && x.ReferralStatus != RecordReferralStatusEnum.Withdrawn);
                        var referralOutDatas = await _referralDataDBRepository.FindAllAsync(subReferralOutFilter);
                        var referralOutRecords = referralOutDatas.Select(x => x.RecordCode).ToList();
                        if (referralOutRecords != null && referralOutRecords.Count() > 0)
                        {
                            queryItemsFilter.ClearFilter();
                            queryItemsFilter.AppendEq((item => item.Key, "Code"));
                            queryItemsFilter.AppendIn(item => item.Value, referralOutRecords);
                            referralOutFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
                        }
                    }
                    //该检查已被转出
                    else if (recordProcessState == RecordProcessStateEnum.ReferralOut)
                    {
                        //todo 应该只关心自己机构转出的，子机构的不需要看到的
                        var subFilter = Builders<ReferralDataEntity>.Filter.Where(x => x.ReferralOutOrganizationCode == userInfo.RootOrganizationCode && x.ReferralStatus != RecordReferralStatusEnum.Withdrawn);
                        var referralOutDatas = await _referralDataDBRepository.FindAllAsync(subFilter);
                        var referralOutRecords = referralOutDatas.Select(x => x.RecordCode).ToList();
                        if (referralOutRecords != null && referralOutRecords.Count() > 0)
                        {
                            queryItemsFilter.ClearFilter();
                            queryItemsFilter.AppendEq((item => item.Key, "Code"));
                            queryItemsFilter.AppendIn(item => item.Value, referralOutRecords);
                            tabOrFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
                        }
                        else
                        {
                            queryItemsFilter.ClearFilter();
                            queryItemsFilter.AppendEq((item => item.Key, "Code"), (item => item.Value, "-1"));
                            tabOrFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
                        }
                    }
                }
            }
            //常规查询：如传入usercode需根据用户来判断查询权限：自己创建的和本机构内共享，外加绑定的设备和机构 Builders<MongoTable>.Filter.Or(customerOrFilter.OrCombine(), deviceAndOrgFilter.AndCombine())
            //待处理的：转入的、医生没有出报告的（tabOrFilter）、但是不包括已转出referralOutFilter.NotCombine()
            //已处理：医生出了诊断报告（tabOrFilter）、但是不包括已转出referralOutFilter.NotCombine()
            //已转出：该检查已被转出（tabOrFilter）
            if (deviceAndOrgFilter.Count() > 0)
            {
                return customerFilter.AndCombine() & customerKeyWordFilter.OrCombine() & processedFilter.AndCombine() & referralOutFilter.NotCombine() & tabOrFilter.OrCombine() & Builders<MongoTable>.Filter.Or(customerOrFilter.OrCombine(), deviceAndOrgFilter.AndCombine());
            }
            else
            {
                return customerFilter.AndCombine() & customerKeyWordFilter.OrCombine() & processedFilter.AndCombine() & referralOutFilter.NotCombine() & tabOrFilter.OrCombine() & customerOrFilter.OrCombine();
            }
        }

        /// <summary>
        /// 获取病人的检查记录列表（分页）条件拼接-只有病人条件  病人的所有历史检查查询
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        private async Task<FilterDefinition<MongoTable>> GetPatientRecordInfoPagesFilter(FindRecordInfoPagesRequest request)
        {
            var patientCodes = request.PatientCodes;
            MongoCustomerFilter<MongoTable> customerFilter = new();//常规查询
            MongoCustomerFilter<MongoDataItem> queryItemsFilter = new();
            if (patientCodes != null && patientCodes.Count > 0)
            {
                queryItemsFilter = new();
                queryItemsFilter.AppendEq((item => item.Key, "PatientCode"));
                queryItemsFilter.AppendIn(item => item.Value, patientCodes);
                if (queryItemsFilter.HasFilter)
                {
                    customerFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
                }
            }
            return customerFilter.AndCombine();
        }


        /// <summary>
        /// 根据记录codes查询记录信息列表
        /// </summary>
        /// <param name="recordCodes">记录列表</param>
        /// <returns>信息列表</returns>
        public async Task<List<RecordInfoDTO>> FindRecordsByRecordCodesAsync(List<string> recordCodes)
        {
            using (new Performance(_performanceThreshold, "{0} FindRecordsByRecordCodesAsync execute", $"{this.GetType().Name}"))
            {
                Dictionary<string, List<string>> filter = new Dictionary<string, List<string>>();
                filter.Add("Code", recordCodes);
                var recordDO = await _recordInfoDBRepository.FindInAsync(filter);
                return recordDO.MappingTo<List<RecordInfoDTO>>();
            }
        }

        /// <summary>
        /// 更改指定设备上传到虚拟机构的检查记录的医院
        /// </summary>
        /// <param name="deviceCodes">设备编码</param>
        /// <param name="newOrgCode">医院编码</param>
        /// <returns></returns>
        public async Task<bool> UpdateRecordsWithOrgCodeAsync(List<string> deviceCodes, string newOrgCode)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateRecordsWithOrgCodeAsync execute", $"{this.GetType().Name}"))
            {
                var deviceInfos = new Dictionary<string, List<string>>();
                deviceInfos.Add("DeviceCode", deviceCodes);
                var records = await _recordInfoDBRepository.FindInAsync(deviceInfos);
                records = records.Where(r => r.OrganizationCode == "VINNO_FIS").ToList();//只迁移设备在虚拟机构产生的数据
                if (records != null)
                {
                    var orgRequest = new GetOrganizationDBRequest();
                    orgRequest.OrganizationCode = newOrgCode;
                    var newOrg = await GetOrganizationDetailAsync(orgRequest);
                    foreach (var item in records)
                    {
                        item.RootOrganizationCode = newOrg?.RootCode;
                        item.OrganizationCode = newOrg.OrganizationCode;
                    }
                }
                if (records.Count > 0)
                {
                    var req = records.Select(f => ("Code", f.Code, f, "")).ToList();
                    return await _recordInfoDBRepository.UpdateAllAsync(req) > 0;
                }
                else
                {
                    return true;
                }
            }
        }

        /// <summary>
        /// 查询未完成AI诊断的检查记录
        /// </summary>
        /// <returns></returns>
        public async Task<List<RecordInfoDTO>> GetDiagnosisInProcessRecordAsync()
        {
            using (new Performance(_performanceThreshold, "{0} GetDiagnosisInProcessRecordAsync execute", $"{this.GetType().Name}"))
            {
                MongoCustomerFilter<MongoTable> customerFilter = new();
                customerFilter.AppendIn("BaseData.DiagnosisStatus", new List<int> { (int)DiagnosisStatusEnum.Under });
                var filter = customerFilter.AndCombine();
                var entities = await _recordInfoDBRepository.FindAllAsync(filter);

                var minCreateTime = DateTime.UtcNow.Date.AddDays(-2);
                entities = entities.Where(x => x.CreateTime >= minCreateTime && (x.RecordStatus == RecordStatusEnum.NotReport || x.RecordStatus == RecordStatusEnum.Completed))?.ToList() ?? new List<RecordInfoDO>();
                return entities.MappingTo<List<RecordInfoDTO>>();
            }
        }

        /// <summary>
        /// 根据病人ID查询检查记录-健康体检
        /// </summary>
        /// <returns></returns>
        public async Task<RecordInfoDTO> GetVitalRecordByPatientIdDBAsync(GetVitalRecordByPatientIdDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetRecordByPatientIdDBAsync execute", $"{this.GetType().Name}"))
            {
                var patientId = request.DevicePatientId;
                if (!string.IsNullOrWhiteSpace(patientId))
                {
                    MongoCustomerFilter<MongoTable> customerFilter = new();
                    customerFilter.AppendIn("BaseData.DevicePatientID", new List<string> { patientId });
                    var filter = customerFilter.AndCombine();
                    var entity = (await _recordInfoDBRepository.FindAllAsync(filter))?.FirstOrDefault(x => !string.IsNullOrWhiteSpace(x.VitalOrganizationCode));
                    if (entity != null && !string.IsNullOrWhiteSpace(entity.Code))
                    {
                        return entity.MappingTo<RecordInfoDTO>();
                    }
                }
                return null;
            }
        }

        /// <summary>
        /// 保存检查AI诊断结果
        /// </summary>
        /// <returns></returns>
        public async Task<bool> UpdateRecordDiagnosisResultAsync(UpdateRecordDiagnosisResultDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateRecordDiagnosisResultAsync execute", $"{this.GetType().Name}"))
            {
                var recordDO = await _recordInfoDBRepository.FindOneAsync("Code", request.RecordCode);
                if (recordDO != null && !string.IsNullOrWhiteSpace(recordDO.Code))
                {
                    var oldInfos = recordDO.DiagnosisInfos;
                    foreach (var info in oldInfos)
                    {
                        var new_info = request.DiagnosisInfos.FirstOrDefault(x => x.DiagnosisOrgan == info.DiagnosisOrgan);
                        if (new_info != null)
                        {
                            info.Conclusion = new_info.Conclusion;
                            if (info.ReportStatus != DiagnosisReportStatusEnum.Reported)
                            {
                                info.ReportStatus = new_info.ReportStatus;
                            }
                        }
                    }
                    foreach (var info in request.DiagnosisInfos)
                    {
                        var old_info = oldInfos.FirstOrDefault(x => x.DiagnosisOrgan == info.DiagnosisOrgan);
                        if (old_info == null)
                        {
                            oldInfos.Add(new DiagnosisInfoDO
                            {
                                DiagnosisOrgan = info.DiagnosisOrgan,
                                Conclusion = info.Conclusion,
                                ReportStatus = info.ReportStatus,
                            });
                        }
                    }
                    recordDO.DiagnosisInfos = oldInfos;
                    if (!oldInfos.Any() || oldInfos.All(x => x.ReportStatus == DiagnosisReportStatusEnum.NotRequired))
                    {
                        recordDO.DiagnosisStatus = DiagnosisStatusEnum.NotRequired;
                    }
                    else if (oldInfos.All(x => x.ReportStatus == DiagnosisReportStatusEnum.Reported))
                    {
                        recordDO.DiagnosisStatus = DiagnosisStatusEnum.Completed;
                    }
                    else
                    {
                        recordDO.DiagnosisStatus = DiagnosisStatusEnum.Under;
                    }
                    if (recordDO.RecordStatus != RecordStatusEnum.Completed
                        && (recordDO.DiagnosisInfos.Any(x => x.ReportStatus == DiagnosisReportStatusEnum.Reported)))
                    {
                        recordDO.RecordStatus = RecordStatusEnum.Completed;
                    }
                    return await _recordInfoDBRepository.UpdateOneAsync("Code", request.RecordCode, recordDO, string.Empty);
                }
                return false;
            }
        }


        /// <summary>
        /// 根据病人codes查询检查
        /// </summary>
        /// <param name="patientCodes"></param>
        /// <returns></returns>
        internal async Task<List<RecordInfoDTO>> FindRecordInfosByPatientCodesAsync(List<string> patientCodes, DateTime startTime, DateTime endTime)
        {
            if (patientCodes == null || patientCodes.Count == 0)
            {
                return new List<RecordInfoDTO>();
            }
            MongoCustomerFilter<MongoTable> customerFilter = new();
            MongoCustomerFilter<MongoDataItem> queryItemsFilter = new();
            queryItemsFilter.AppendEq((item => item.Key, "PatientCode"));
            queryItemsFilter.AppendIn(item => item.Value, patientCodes);
            if (queryItemsFilter.HasFilter)
            {
                customerFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
            }
            if (startTime > DateTime.MinValue)
            {
                customerFilter.AppendGte("BaseData.CreateTime", startTime);
            }
            if (endTime > DateTime.MinValue)
            {
                customerFilter.AppendLt("BaseData.CreateTime", endTime);
            }
            var recordResult = await _recordInfoDBRepository.FindAllAsync(customerFilter.AndCombine());
            return recordResult.MappingTo<List<RecordInfoDTO>>();
        }

        /// <summary>
        /// 查询转诊配置
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<ReferralRecordSettingDTO> GetReferralRecordSettingAsync(GetReferralRecordSettingDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetReferralRecordSettingAsync execute", $"{this.GetType().Name}"))
            {
                var entities = await _referralRecordSettingDBRepository.FindAllAsync();
                if (entities == null || !entities.Any())
                {
                    var entity = new ReferralRecordSettingDO();
                    entity.ReferralRecordSettingCode = await _referralRecordSettingDBRepository.InsertOneAsync(entity, new Dictionary<string, string>(), string.Empty);
                    return entity.MappingTo<ReferralRecordSettingDTO>();
                }
                return entities.FirstOrDefault().MappingTo<ReferralRecordSettingDTO>();
            }
        }

        /// <summary>
        /// 保存转诊配置
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> UpdateReferralRecordSettingAsync(UpdateReferralRecordSettingDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateReferralRecordSettingAsync execute", $"{this.GetType().Name}"))
            {
                var entities = await _referralRecordSettingDBRepository.FindAllAsync();
                if (entities?.Any() == true)
                {
                    var entity = entities.FirstOrDefault();
                    entity.LimitHours = request.LimitHours;
                    return await _referralRecordSettingDBRepository.UpdateOneAsync("Code", entity.Code, entity, string.Empty);
                }
                return false;
            }
        }

        public async Task<bool> ExistRecordReportsByUserAsync(string recordCode, string userCode)
        {
            MongoCustomerFilter<MongoTable> filter = new MongoCustomerFilter<MongoTable>();
            filter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("RecordCode", recordCode), ("ReportUserCode", userCode));
            return await _reportDBRepository.Exists(filter.AndCombine());
        }

        /// <summary>
        /// 获取诊断数据发送给数据中心
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<IList<RemoteDiagnosisDTO>> FindRemoteDiagnosisByDateCenterDBAsync(FindRemoteDiagnosisByDateCenterDBRequest request)
        {
            var startTime = request.StartTime;
            var endTime = request.EndTime;
            MongoCustomerFilter<MongoTable> customerFilter = new();
            customerFilter.AppendGt("BaseData.CreateTime", startTime);
            customerFilter.AppendGt("BaseData.UpdateTime", startTime);
            customerFilter.AppendLte("BaseData.CreateTime", endTime);
            customerFilter.AppendLte("BaseData.UpdateTime", endTime);
            var recordResult = await _recordInfoDBRepository.FindAllAsync(customerFilter.AndCombine());
            var remoteDiagnosisList = new List<RemoteDiagnosisDTO>();
            if (recordResult.Count > 0)
            {
                //获取病人信息
                var patientCodes = recordResult.Select(v => v.PatientCode).ToList();
                var filter = new Dictionary<string, List<string>>();
                filter.Add("Code", patientCodes);
                var patientList = await _patientInfoDBRepository.FindInAsync(filter);
                //获取医院信息 
                var rootOrganizationCodes = recordResult.Select(v => v.RootOrganizationCode).ToList();
                filter = new Dictionary<string, List<string>>();
                filter.Add("Code", rootOrganizationCodes);
                var organizationList = await _organizationDBRepository.FindInAsync(filter);
                //获取设备信息
                var deviceCodeCodes = recordResult.Select(v => v.DeviceCode).ToList();
                filter = new Dictionary<string, List<string>>();
                filter.Add("Code", deviceCodeCodes);
                var deviceList = await _deviceInfoDBRepository.FindInAsync(filter);
                //获取诊断记录
                var remedicalCodes = recordResult.Select(v => v.Code).ToList();
                filter = new Dictionary<string, List<string>>();
                filter.Add("RecordCode", remedicalCodes);
                var remedicalList = await _remedicalDBRepository.FindInAsync(filter);
                //获取医院信息
                var orgCodeList = deviceList?.Select(v => v.OrganizationCode).ToList();
                filter = new Dictionary<string, List<string>>();
                filter.Add("Code", orgCodeList);
                var orgList = await _organizationDBRepository.FindInAsync(filter);
                foreach (var item in recordResult)
                {
                    var organization = organizationList.FirstOrDefault(c => c.Code == item.RootOrganizationCode);
                    var device = deviceList.FirstOrDefault(c => c.Code == item.DeviceCode);
                    var patient = patientList.FirstOrDefault(c => c.Code == item.PatientCode);
                    var fileList = remedicalList.Where(v => v.RecordCode == item.Code).ToList();
                    var org = orgList.FirstOrDefault(c => c.Code == item.RootOrganizationCode);
                    remoteDiagnosisList.Add(new RemoteDiagnosisDTO()
                    {
                        ExamId = item.RecordCode,
                        Code = item.RecordCode,
                        CreateTime = item.CreateTime,
                        UpdateTime = item.UpdateTime,
                        OrganizationInfo = new OrganizationView()
                        {
                            Code = organization?.Code,
                            Name = organization?.OrganizationName,
                            Description = organization?.Description,
                            //DepartmentCode=
                            //DepartmentName 
                            LogoUrl = organization?.LogoUrl
                            //    CountryCode =
                            //    CountryName =
                            //    ProvinceCode =
                            //    ProvinceName =
                            //    CityCode =
                            //    CityName =
                        },
                        DeviceInfo = new DeviceView()
                        {
                            Code = device?.Code,
                            Name = device?.Name,
                            Description = device?.Description,
                            UniqueCode = device?.ShortCode,
                            OrganizationCode = device?.OrganizationCode,
                            OrganizationName = org?.OrganizationCode,
                            DeviceType = device?.DeviceType,
                            DeviceModel = device?.DeviceModel,
                        },
                        PatientDataInfo = patient?.PatientData,
                        PatientType = patient?.PatientType,
                        ReadUsers = item.ReadUsers,
                        AssociatedExamCodes = item.AssociatedExamCodes,
                        PatientInfoExtList = item.PatientInfoExtList?.MappingTo<List<WingInterfaceLibrary.DTO.Record.PatientInfoExt>>(),
                        CreateType = item.CreateType,
                        Tags = item.Tags,
                        CreatorCode = item.CreatorCode,
                        RecordStatus = item.RecordStatus,

                        // QualifiedInfo =
                        DiagnosisInfos = item.DiagnosisInfos.Select(v => new DiagnosisView()
                        {
                            Organ = v.DiagnosisOrgan,
                            DiagnosisStatus = v.Conclusion
                        }).ToList(),
                        FileInfos = fileList.Select(v => new RemoteDiagnosisFileView()
                        {
                            Code = v.Code,
                            Application = v.Application,
                            DataType = v.FileDataType,
                            ImageLocationInfo = new ImageLocationView()
                            {
                                Group = v.ImageLocation?.Group,
                                Position = v.ImageLocation?.Position,
                                Quadrant = v.ImageLocation?.Quadrant
                            },
                            CdnUrl = v.TerminalImages?.ImageUrl,
                            PreviewUrl = v.TerminalImages?.PreviewUrl,
                            ConverUrl = v.TerminalImages?.CoverImageUrl,
                            // CommentInfos  =v.CommentResult?null:new CommentView ()
                            // {

                            // },
                        }).ToList()
                    });
                }
            }
            return remoteDiagnosisList;
        }

        /// <summary>
        /// 查询病人关联检查记录
        /// </summary>
        /// <param name="patientCodes"></param>
        /// <returns></returns>
        public async Task<IList<RecordInfoDTO>> GetPatientRecordsDBAsync(IList<string> patientCodes)
        {
            var records = new List<RecordInfoDO>();
            if (patientCodes != null && patientCodes.Any())
            {
                if (patientCodes.Count == 1)
                {
                    Dictionary<string, string> filter = new Dictionary<string, string>();
                    filter.Add("PatientCode", patientCodes.FirstOrDefault());
                    var entities = await _recordInfoDBRepository.FindListAsync(filter);
                    if (entities != null && entities.Any())
                    {
                        records = entities.ToList();
                    }
                }
                else
                {
                    Dictionary<string, List<string>> filter = new Dictionary<string, List<string>>();
                    filter.Add("PatientCode", patientCodes.ToList());
                    var entities = await _recordInfoDBRepository.FindInAsync(filter);
                    if (entities != null && entities.Any())
                    {
                        records = entities.ToList();
                    }
                }
            }
            return records.MappingTo<List<RecordInfoDTO>>();
        }

        /// <summary>
        /// 根据设备查询检查记录
        /// </summary>
        /// <param name="deviceCode"></param>
        /// <returns></returns>
        public async Task<IList<RecordInfoDTO>> FindRecordInfosByDeviceCodeAsync(string deviceCode)
        {
            if (string.IsNullOrWhiteSpace(deviceCode))
            {
                return new List<RecordInfoDTO>();
            }
            MongoCustomerFilter<MongoTable> customerFilter = new();
            MongoCustomerFilter<MongoDataItem> queryItemsFilter = new();
            queryItemsFilter.AppendEq((item => item.Key, "DeviceCode"), (item => item.Value, deviceCode));
            if (queryItemsFilter.HasFilter)
            {
                customerFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
            }
            var filter = customerFilter.AndCombine();
            var entities = await _recordInfoDBRepository.FindAllAsync(filter);
            return entities.MappingTo<List<RecordInfoDTO>>();
        }

        /// <summary>
        /// 根据recordcode查询转诊列表
        /// </summary>
        /// <param name="request">根据recordcode查询转诊列表</param>
        /// <returns>转诊历史</returns>
        public async Task<List<ReferralData>> FindReferralDatasByRecordCodeDBAsync(string recordCode)
        {
            FilterDefinition<ReferralDataEntity> filter = Builders<ReferralDataEntity>.Filter.Where(x => x.RecordCode == recordCode);
            var referralDatas = await _referralDataDBRepository.FindAllAsync(filter);
            var result = referralDatas.MappingTo<List<ReferralData>>();
            return result;
        }

        /// <summary>
        /// 新增转诊记录
        /// </summary>
        /// <param name=""> </param>
        /// <returns>
        public async Task<string> InsertReferralDataAsync(ReferralData referralData)
        {
            using (new Performance(_performanceThreshold, "{0} InsertReferralDataAsync execute", $"{this.GetType().Name}"))
            {
                var referralDataEntity = referralData.MappingTo<ReferralDataEntity>();
                var result = await _referralDataDBRepository.InsertOneAsync(referralDataEntity);
                return result;
            }
        }

        public async Task<bool> UpdateReferralDataAsync(ReferralData data, bool canNull = false)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateReferralDataAsync execute", $"{this.GetType().Name}"))
            {
                var referralDataEntity = data.MappingTo<ReferralDataEntity>();
                referralDataEntity.UpdateTime = DateTime.UtcNow;
                var result = await _referralDataDBRepository.UpdateOneAsync("Code", referralDataEntity.Code, referralDataEntity, canNull);
                return result;
            }
        }

        /// <summary>
        /// 查询转诊历史
        /// </summary>
        /// <param name="request">查询查询转诊历史</param>
        /// <returns>转诊历史</returns>
        public async Task<List<ReferralHistoryDetail>> FindReferralHistoryDBAsync(FindReferralHistoryDBRequest request)
        {
            var result = new List<ReferralHistoryDetail>();
            FilterDefinition<ReferralDataEntity> filter = Builders<ReferralDataEntity>.Filter.Where(x => x.RecordCode == request.RecordCode);
            var referralDatas = await _referralDataDBRepository.FindAllAsync(filter);
            var recordInfo = await FindRecordInfoByCodeAsync(request.RecordCode);
            var patientName = "";
            if (recordInfo != null && recordInfo.PatientInfo?.Count > 0 && recordInfo.PatientInfo.Exists(d => d.Key == "Name"))
            {
                var dataItem = recordInfo.PatientInfo.FirstOrDefault(c => c.Key == "Name") ?? new DataItemDTO();
                if (string.IsNullOrWhiteSpace(dataItem.Value))
                {
                    patientName = GetNotFilled(request.LanguageCode);
                }
                else
                {
                    var orgRequest = new GetOrganizationDBRequest { OrganizationCode = request.OrganizationCode };
                    var rootOrg = await GetOrganizationDetailAsync(orgRequest);
                    patientName = CryptionHelper.NameEncrypt(dataItem.Value, rootOrg.IsEncryptedShow);
                }
            }
            var organizationCodes = referralDatas.Select(x => x.ReferralInOrganizationCode).ToList();
            organizationCodes.AddRange(referralDatas.Select(x => x.ReferralOutOrganizationCode));
            var organizations = await FindOrganizationDTOListByCodesAsync(organizationCodes.Where(x => !string.IsNullOrWhiteSpace(x)).ToList());
            var userCodes = referralDatas.Select(x => x.ReferralInUserCode).ToList();
            userCodes.AddRange(referralDatas.Select(x => x.ReferralOutUserCode));
            List<UserDTO> users = await FindUserListByUserCodesAsync(userCodes.Where(x => !string.IsNullOrWhiteSpace(x)).ToList());
            foreach (var item in referralDatas)
            {
                var referralOutOrganization = organizations.FirstOrDefault(x => x.OrganizationCode == item.ReferralOutOrganizationCode);
                var referralInOrganization = organizations.FirstOrDefault(x => x.OrganizationCode == item.ReferralInOrganizationCode);
                var referralOutUser = users.FirstOrDefault(x => x.UserCode == item.ReferralOutUserCode);
                var referralInUser = users.FirstOrDefault(x => x.UserCode == item.ReferralInUserCode);
                var referralHistoryDetail = new ReferralHistoryDetail();
                referralHistoryDetail.ReferralOutUserCode = item.ReferralOutUserCode;
                referralHistoryDetail.ReferralOutOrganizationCode = item.ReferralOutOrganizationCode;
                referralHistoryDetail.ReferralInUserCode = item.ReferralInUserCode;
                referralHistoryDetail.ReferralInOrganizationCode = item.ReferralInOrganizationCode;
                referralHistoryDetail.ReferralStatus = item.ReferralStatus;
                referralHistoryDetail.ReferralTime = item.ReferralTime;
                referralHistoryDetail.SubjectMatter = item.SubjectMatter;
                referralHistoryDetail.PatientName = patientName;
                referralHistoryDetail.ReferralOutOrganizationName = referralOutOrganization?.OrganizationName;
                referralHistoryDetail.ReferralOutUserName = referralOutUser?.DisplayName;
                referralHistoryDetail.ReferralInOrganizationName = referralInOrganization?.OrganizationName;
                referralHistoryDetail.ReferralInUserName = referralInUser?.DisplayName;
                result.Add(referralHistoryDetail);
            }
            return result;
        }

        /// <summary>
        /// 根据项目询检查记录
        /// </summary>
        /// <param name="deviceCode"></param>
        /// <returns></returns>
        async Task<IList<RecordInfoDTO>> FindRecordInfosByProjectCodeAsync(string projectCode)
        {
            if (string.IsNullOrWhiteSpace(projectCode))
            {
                return new List<RecordInfoDTO>();
            }
            MongoCustomerFilter<MongoTable> customerFilter = new();//常规查询
            customerFilter.AppendEq(("BaseData.ProjectCode", projectCode));
            var filter = customerFilter.AndCombine();
            var entities = await _recordInfoDBRepository.FindAllAsync(filter);
            var result = entities.MappingTo<List<RecordInfoDTO>>();
            return result;
        }

        /// <summary>
        /// 根据项目询检查记录
        /// </summary>
        /// <param name="deviceCode"></param>
        /// <returns></returns>
        async Task<IList<RecordInfoDO>> FindRecordInfosByProjectCodesAsync(List<string> projectCodes)
        {
            if (projectCodes == null || projectCodes.Count <= 0)
            {
                return new List<RecordInfoDO>();
            }
            MongoCustomerFilter<MongoTable> customerFilter = new();//常规查询
            customerFilter.AppendIn("BaseData.ProjectCode", projectCodes);
            var filter = customerFilter.AndCombine();
            var entities = await _recordInfoDBRepository.FindAllAsync(filter);
            return entities;
        }
    }
}