using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using MongoDB.Bson;
using MongoDB.Driver;
using WingInterfaceLibrary.DB.Request;
using WingInterfaceLibrary.DTO.Vital;
using WingInterfaceLibrary.Interface.DBInterface;
using WingInterfaceLibrary.Request;
using WingInterfaceLibrary.Request.Vital;
using WingInterfaceLibrary.Result;
using WingMongoDBModule.Entities;
using WingMongoDBModule.Request;
using WingServerCommon.Mapper;
using WingServerCommon.Service;

namespace WingMongoDBModule.Service
{
    public partial class DatabaseService : JsonRpcService, IRegisterInfoDBService
    {
        public async Task<string> AddRegisterInfoAsync(RegisterInfoDTO request)
        {
            var entity = request.MappingTo<RegisterInfoEntity>();
            var result = await _registerInfoDBRepository.InsertOneAsync(entity);
            return result;
        }

        public async Task<bool> DeleteRegisterInfoAsync(string code)
        {
            var filter = Builders<RegisterInfoEntity>.Filter.Eq(f => f.Code, code);
            var update = Builders<RegisterInfoEntity>.Update
                .Set(f => f.IsDelete, true)
                .Set(f => f.UpdateTime, DateTime.UtcNow);
            var result = await _registerInfoDBRepository.UpdateOneAsync(filter, update) > 0;
            return result;
        }

        public async Task<bool> DeleteRegisterInfoByIdCardNoDBAsync(DeleteRegisterInfoByIdCardNoDBRequest request)
        {
            var idCardNo = request.IDCardNo;
            var builder = Builders<RegisterInfoEntity>.Filter;
            var filter = builder.Where(c => c.IsDelete == false && c.IDCardNo == idCardNo);
            var entities = await _registerInfoDBRepository.FindAllAsync(filter);
            foreach (var entity in entities)
            {
                var filter1 = Builders<RegisterInfoEntity>.Filter.Eq(f => f.Code, entity.Code);
                var update = Builders<RegisterInfoEntity>.Update
                    .Set(f => f.IsDelete, true)
                    .Set(f => f.UpdateTime, DateTime.UtcNow);
                await _registerInfoDBRepository.UpdateOneAsync(filter1, update);
            }
            return true;
        }

        public async Task<RegisterInfoDTO> FindRegisterInfoByCodeAsync(string code)
        {
            var filter = Builders<RegisterInfoEntity>.Filter.Eq(f => f.Code, code);
            var entity = await _registerInfoDBRepository.FindOneAsync(filter);
            if (entity != null && !string.IsNullOrWhiteSpace(entity.Code))
            {
                var dto = entity.MappingTo<RegisterInfoDTO>();
                return dto;
            }
            return null;
        }

        public async Task<PageCollection<RegisterInfoDTO>> GetRegisterInfoPageAsync(GetRegisterInfoPageDBRequest request)
        {
            var keyword = request.Keyword;

            var builder = Builders<RegisterInfoEntity>.Filter;
            var filter = builder.Where(c => c.IsDelete == false);
            if (!string.IsNullOrEmpty(keyword))
            {
                keyword = EscapeExprSpecialWord(keyword);//字符串转义
                var nameFilter = builder.Regex(c => c.Name, new BsonRegularExpression(keyword, "i"));
                var IDCardNoFilter = builder.Regex(c => c.IDCardNo, new BsonRegularExpression(keyword, "i"));
                var physicalExamNumberFilter = builder.Regex(c => c.PhysicalExamNumber, new BsonRegularExpression(keyword, "i"));
                var phoneFilter = builder.Regex(c => c.Phone, new BsonRegularExpression(keyword, "i"));
                filter = filter & (nameFilter | IDCardNoFilter | physicalExamNumberFilter | phoneFilter);
            }
            if (!string.IsNullOrWhiteSpace(request.OrganizationCode))
            {
                var orgFilter = builder.Regex(c => c.OrganizationCode, request.OrganizationCode);
                filter = filter & orgFilter;
            }
            if (!string.IsNullOrWhiteSpace(request.IDCardNo))
            {
                var cardNoFilter = builder.Where(c => c.IDCardNo == request.IDCardNo);
                filter = filter & cardNoFilter;
            }
            if (request.ProjectType != null)
            {
                var projectTypeFilter = builder.Where(c => c.ProjectType == request.ProjectType);
                filter = filter & projectTypeFilter;
            }
            if (request.StartTime >= _defaultTime)
            {
                var startTimeFilter = builder.Where(x => x.CreateTime >= request.StartTime);
                filter = filter & startTimeFilter;
            }
            if (request.EndTime >= _defaultTime)
            {
                var endTimeFilter = builder.Where(x => x.CreateTime < request.EndTime);
                filter = filter & endTimeFilter;
            }

            var pageFilter = new DBPageRequest<RegisterInfoEntity>
            {
                CurrentPage = request.PageIndex,
                PageSize = request.PageSize,
                Filter = filter,
                Sort = Builders<RegisterInfoEntity>.Sort.Descending(x => x.CreateTime)
            };
            var result = await _registerInfoDBRepository.GetPages(pageFilter);
            return result.MappingTo<PageCollection<RegisterInfoDTO>>();
        }

        /// <summary>
        /// 修改登记信息
        /// </summary>
        /// <param name="registerInfoDTO"></param>
        /// <returns></returns>
        public async Task<bool> UpdateRegisterInfoAsync(RegisterInfoDTO registerInfoDTO)
        {
            var persons = new List<RegisterInfoEntity>();

            var filter = Builders<RegisterInfoEntity>.Filter.Eq(f => f.Code, registerInfoDTO.Code);
            var update = Builders<RegisterInfoEntity>.Update
                // .Set(f => f.PhysicalExamNumber, registerInfoDTO.PhysicalExamNumber)
                .Set(f => f.IDCardNo, registerInfoDTO.IDCardNo)
                .Set(f => f.Name, registerInfoDTO.Name)
                .Set(f => f.Sex, registerInfoDTO.Sex)
                .Set(f => f.Age, registerInfoDTO.Age)
                .Set(f => f.Phone, registerInfoDTO.Phone)
                .Set(f => f.Adress, registerInfoDTO.Adress)
                .Set(f => f.State, registerInfoDTO.State)
                .Set(f => f.Birthday, registerInfoDTO.Birthday)
                .Set(f => f.UpdateTime, DateTime.UtcNow);
            var result = await _registerInfoDBRepository.UpdateOneAsync(filter, update) > 0;
            return result;
        }

        /// <summary>
        /// 修改结果和建议信息
        /// </summary>
        /// <param name="Code"></param>
        /// <param name="resultsAndSuggestions"></param>
        /// <returns></returns>
        public async Task<bool> UpdateResultsAndSuggestionsAsync(string code, string resultsAndSuggestions, string healthGuidance, List<DataItemDTO> diagnosis)
        {
            var filter = Builders<RegisterInfoEntity>.Filter.Eq(f => f.Code, code);
            var update = Builders<RegisterInfoEntity>.Update
                .Set(f => f.UpdateTime, DateTime.UtcNow);
            if (!string.IsNullOrEmpty(resultsAndSuggestions))
            {
                update = update.Set(c => c.ResultsAndSuggestions, resultsAndSuggestions);
            }
            if (!string.IsNullOrEmpty(healthGuidance))
            {
                update = update.Set(c => c.HealthGuidance, healthGuidance);
            }
            if (diagnosis?.Count > 0)
            {
                update = update.Set(c => c.DiagnosisExceptionResult, diagnosis);
            }
            var result = await _registerInfoDBRepository.UpdateOneAsync(filter, update) > 0;
            return result;
        }

        /// <summary>
        /// 根据身份证获取居民登记记录
        /// </summary>
        /// <returns></returns>
        public async Task<List<RegisterInfoDTO>> GetPersonRegisteredInfosDBAsync(GetPersonRegisteredInfosDBRequest request)
        {
            var idCardNo = request.IDCardNo;
            var organizationCode = request.OrganizationCode;
            var startTime = DateTime.Now.Date;
            var endTime = DateTime.Now.AddDays(1).Date;

            var builder = Builders<RegisterInfoEntity>.Filter;
            var filter = builder.Where(c => c.IsDelete == false && c.IDCardNo == idCardNo);
            if (!string.IsNullOrWhiteSpace(organizationCode))
            {
                var orgFilter = builder.Where(x => x.OrganizationCode == organizationCode);
                filter = filter & orgFilter;
            }
            var startTimeFilter = builder.Where(x => x.CreateTime >= startTime);
            filter = filter & startTimeFilter;
            var endTimeFilter = builder.Where(x => x.CreateTime < endTime);
            filter = filter & endTimeFilter;
            var entities = await _registerInfoDBRepository.FindAllAsync(filter);
            return entities?.MappingTo<List<RegisterInfoDTO>>()?.ToList() ?? new List<RegisterInfoDTO>();
        }

        /// <summary>
        /// 查询该机构当天已经登记了多少人
        /// </summary>
        /// <returns></returns>
        public async Task<int> GetRegisterNumberAsync(string organizationCode)
        {
            var todayDate = DateTime.Today;
            var createStartTime = todayDate.ToUniversalTime();
            var orgFilter = Builders<RegisterInfoEntity>.Filter.Eq(f => f.OrganizationCode, organizationCode);
            var dateFilter = Builders<RegisterInfoEntity>.Filter.Gte(f => f.CreateTime, createStartTime);
            var filter = Builders<RegisterInfoEntity>.Filter.And(orgFilter, dateFilter);
            var entitys = await _registerInfoDBRepository.FindAllAsync(filter, onlyNotDeleted: false);
            var num = entitys.Count;
            return num;
        }

        /// <summary>
        /// 查询所有已体检完成未出报告的登记
        /// </summary>
        /// <returns></returns>
        public async Task<IList<RegisterInfoDTO>> GetInspectedRegisterInfoListDBAsync(List<string> physicalExamNumbers, string cardNo = "", bool isQueryReportState = false)
        {
            var dataList = new List<RegisterInfoDTO>();
            var filter = Builders<RegisterInfoEntity>.Filter.Where(c => c.IsDelete == false);
            if (physicalExamNumbers?.Count > 0)
            {
                filter = filter & Builders<RegisterInfoEntity>.Filter.Where(c => physicalExamNumbers.Contains(c.PhysicalExamNumber));
            }
            if (!string.IsNullOrEmpty(cardNo))
            {
                filter = filter & Builders<RegisterInfoEntity>.Filter.Where(c => c.IDCardNo == cardNo);
            }
            if (isQueryReportState)
            {
                filter = filter & Builders<RegisterInfoEntity>.Filter.Where(c => (c.State == WingInterfaceLibrary.Enum.VitalEnum.ExamStateEnum.Inspected ||
                (c.State == WingInterfaceLibrary.Enum.VitalEnum.ExamStateEnum.Reported && c.UpdateTime > DateTime.UtcNow.AddHours(-1))));
            }
            else
            {
                filter = filter & Builders<RegisterInfoEntity>.Filter.Where(c => c.State == WingInterfaceLibrary.Enum.VitalEnum.ExamStateEnum.Inspected);
            }
            var entitys = await _registerInfoDBRepository.FindAllAsync(filter);
            if (entitys?.Count > 0)
            {
                dataList = entitys.MappingTo<List<RegisterInfoDTO>>();
            }
            return dataList;
        }

        /// <summary>
        /// 查询体检信息
        /// </summary>
        /// <returns></returns>
        public async Task<RegisterPersonInfoDTO> GetRegisterPersonInfoByPhysicalExamNumberDBAsync(GetRegisterPersonRequest request)
        {
            //注册
            var dataInfo = new RegisterPersonInfoDTO();
            var filter = Builders<RegisterInfoEntity>.Filter.Where(c => c.IsDelete == false && c.PhysicalExamNumber == request.PhysicalExamNumber);
            var entity = await _registerInfoDBRepository.FindOneAsync(filter);
            if (!string.IsNullOrEmpty(entity?.Code))
            {
                dataInfo = entity.MappingTo<RegisterPersonInfoDTO>();
                var personFilter = Builders<RegisterPersonInfoEntity>.Filter.Where(c => c.IsDelete == false && c.CardID == dataInfo.IDCardNo);
                var personEntity = await _registerPersonInfoDBRepository.FindOneAsync(personFilter);
                if (!string.IsNullOrEmpty(personEntity?.Code))
                {
                    dataInfo.HeadImg = personEntity.HeadImg;
                    dataInfo.Gender = personEntity.Sex;
                }
                if (entity.Birthday > DateTime.MinValue)
                {
                    dataInfo.Age = CalculateAge(entity.Birthday).ToString();
                }
            }
            return dataInfo;
        }

        /// <summary>
        /// 分页查询增量登记集合
        /// </summary>
        /// <returns></returns>
        public async Task<PageResult<RegisterPersonInfoDTO>> FindRegisterPersonInfosByUpdateTimeDBAsync(FindCollectionByTimeDBRequest request)
        {
            var startTime = request.StartTime;
            var pageIndex = request.PageIndex;
            var pageSize = request.PageSize;

            var builder = Builders<RegisterInfoEntity>.Filter;
            var filter = builder.Where(c => c.UpdateTime > startTime);

            var pageFilter = new DBPageRequest<RegisterInfoEntity>
            {
                CurrentPage = request.PageIndex,
                PageSize = request.PageSize,
                Filter = filter,
                Sort = Builders<RegisterInfoEntity>.Sort.Ascending(x => x.UpdateTime)
            };
            var pagedResult = await _registerInfoDBRepository.GetPages(pageFilter, onlyNotDeleted: false);

            var result = new PageResult<RegisterPersonInfoDTO>();
            result.PageIndex = pagedResult.CurrentPage;
            result.PageSize = pagedResult.PageSize;
            result.TotalCount = pagedResult.DataCount;
            result.PageData = pagedResult.PageData?.MappingTo<List<RegisterPersonInfoDTO>>() ?? new List<RegisterPersonInfoDTO>();
            return result;
        }

        /// <summary>
        /// 保存登记记录
        /// </summary>
        /// <returns></returns>
        public async Task<bool> SaveRegisterPersonInfoDBAsync(RegisterPersonInfoDTO request)
        {
            var code = request.Code;
            var registerDO = request.MappingTo<RegisterInfoEntity>();

            var filter = Builders<RegisterInfoEntity>.Filter.Eq(f => f.Code, code);
            var entity = await _registerInfoDBRepository.FindOneAsync(filter);
            if (entity != null && !string.IsNullOrWhiteSpace(entity.Code))
            {
                var filter2 = Builders<RegisterInfoEntity>.Filter.Eq(f => f.Code, code);
                var update = Builders<RegisterInfoEntity>.Update
                    // .Set(f => f.PhysicalExamNumber, registerDO.PhysicalExamNumber)
                    .Set(f => f.IDCardNo, registerDO.IDCardNo)
                    .Set(f => f.Name, registerDO.Name)
                    .Set(f => f.Sex, registerDO.Sex)
                    .Set(f => f.Age, registerDO.Age)
                    .Set(f => f.Phone, registerDO.Phone)
                    .Set(f => f.Adress, registerDO.Adress)
                    .Set(f => f.State, registerDO.State)
                    .Set(f => f.Birthday, registerDO.Birthday)
                    .Set(f => f.IsDelete, registerDO.IsDelete)
                    .Set(f => f.CreateTime, registerDO.CreateTime)
                    .Set(f => f.UpdateTime, registerDO.UpdateTime);
                await _registerInfoDBRepository.UpdateOneAsync(filter2, update);
            }
            else
            {
                await _registerInfoDBRepository.InsertOneAsync(registerDO);
            }
            return true;
        }

        /// <summary>
        /// 根据检测编号获取最新的体检登记记录
        /// </summary>
        /// <returns></returns>
        public async Task<RegisterInfoDTO> GetRegisterInfoByExamNumberDBAsync(GetRegisterInfoByExamNumberDBRequest request)
        {
            var orgCode = request.OrganizationCode;
            var examNumber = request.ExamNumber;
            if (!string.IsNullOrWhiteSpace(orgCode) && !string.IsNullOrWhiteSpace(examNumber))
            {
                var filter = Builders<RegisterInfoEntity>.Filter.Where(c => c.IsDelete == false && c.OrganizationCode == orgCode && c.ExamNumber == examNumber);
                if (request.ExamTime > new DateTime(2000, 1, 1))
                {
                    var startTime = request.ExamTime.Date.ToUniversalTime();
                    var endTime = startTime.AddDays(1);
                    filter = filter & Builders<RegisterInfoEntity>.Filter.Where(x => x.CreateTime >= startTime && x.CreateTime < endTime);
                }
                var entities = await _registerInfoDBRepository.FindAllAsync(filter);
                if (entities?.Any() ?? false)
                {
                    var entity = entities.OrderByDescending(x => x.CreateTime).FirstOrDefault();
                    return entity.MappingTo<RegisterInfoDTO>();
                }
            }
            return null;
        }

        /// <summary>
        /// 根据检测编号获取最新的体检登记记录
        /// </summary>
        /// <returns></returns>
        public async Task<List<RegisterInfoDTO>> GetRegisterInfosByExamNumbersDBAsync(GetRegisterInfosByExamNumbersDBRequest request)
        {
            var orgCode = request.OrganizationCode;
            var examNumbers = new List<string>();

            var resultData = new List<RegisterInfoDTO>();
            if (!string.IsNullOrWhiteSpace(orgCode) && examNumbers != null && examNumbers.Any())
            {
                var filter = Builders<RegisterInfoEntity>.Filter.Where(c => c.IsDelete == false && c.OrganizationCode == orgCode && examNumbers.Contains(c.ExamNumber));
                if (request.ExamTime > new DateTime(2000, 1, 1))
                {
                    var startTime = request.ExamTime.Date.ToUniversalTime();
                    var endTime = startTime.AddDays(1);
                    filter = filter & Builders<RegisterInfoEntity>.Filter.Where(x => x.CreateTime >= startTime && x.CreateTime < endTime);
                }
                var entities = await _registerInfoDBRepository.FindAllAsync(filter);
                if (entities?.Any() ?? false)
                {
                    foreach (var item in examNumbers)
                    {
                        var entity = entities.Where(x => x.ExamNumber == item)?.OrderByDescending(x => x.CreateTime)?.FirstOrDefault();
                        if (entity != null && !string.IsNullOrWhiteSpace(entity.PhysicalExamNumber))
                        {
                            resultData.Add(entity.MappingTo<RegisterInfoDTO>());
                        }
                    }
                }
            }
            return resultData;
        }

        /// <summary>
        /// 获取年龄
        /// </summary>
        /// <returns></returns>
        private int CalculateAge(DateTime birthDate)
        {
            DateTime currentDate = DateTime.Now; // 获取当前日期时间            
            int years = currentDate.Year - birthDate.Year; // 计算年份之差            
            if (currentDate < birthDate.AddYears(years)) // 如果还未过完指定年份
                return --years; // 则将年份减去1            
            return years; // 否则返回正常的年份
        }
    }
}