using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using MongoDB.Driver;
using WingInterfaceLibrary.DTO.Comment;
using WingInterfaceLibrary.DTO.DiagnosisResult;
using WingInterfaceLibrary.Interface.DBInterface;
using WingInterfaceLibrary.Request.DBRequest;
using WingMongoDBModule.Entities;
using WingServerCommon.Mapper;
using WingServerCommon.Service;
using WingServerCommon.Utilities;

namespace WingMongoDBModule.Service
{
    public partial class DatabaseService : JsonRpcService, IRemedicalAISelectedInfoDBService
    {
        /// <summary>
        /// 查询AI图像详情
        /// </summary>
        /// <returns></returns>
        public async Task<RemedicalAISelectedInfoDTO> GetRemedicalAISelectedInfoAsync(GetRemedicalAISelectedInfoDBRequest request)
        {
            var code = request.Code;
            using (new Performance(_performanceThreshold, "{0} GetRemedicalAISelectedInfoAsync execute", $"{this.GetType().Name}"))
            {
                var filter = Builders<RemedicalAISelectedInfoEntity>.Filter.Where(x => x.Code == code);
                var entity = await _remedicalAISelectedInfoDBRepository.FindOneAsync(filter);
                if (entity != null)
                {
                    return entity.MappingTo<RemedicalAISelectedInfoDTO>();
                }
                return null;
            }
        }

        /// <summary>
        /// 查询病人AI图像集合
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<List<RemedicalAISelectedInfoDTO>> GetPatientAISelectedInfosAsync(GetPatientAISelectedInfosDBRequest request)
        {
            var patientCode = request.PatientCode;
            var recordCode = request.RecordCode;
            var remedicalCode = request.RemedicalCode;
            var index = request.Index;
            using (new Performance(_performanceThreshold, "{0} GetPatientAISelectedInfosAsync execute", $"{this.GetType().Name}"))
            {
                var filterDefinitions = new List<FilterDefinition<RemedicalAISelectedInfoEntity>>();
                if (!string.IsNullOrWhiteSpace(patientCode))
                {
                    filterDefinitions.Add(Builders<RemedicalAISelectedInfoEntity>.Filter.Where(x => x.PatientCode == patientCode));
                }
                if (!string.IsNullOrWhiteSpace(recordCode))
                {
                    filterDefinitions.Add(Builders<RemedicalAISelectedInfoEntity>.Filter.Where(x => x.RecordCode == recordCode));
                }
                if (!string.IsNullOrWhiteSpace(remedicalCode))
                {
                    filterDefinitions.Add(Builders<RemedicalAISelectedInfoEntity>.Filter.Where(x => x.RemedicalCode == remedicalCode));
                }
                if (index > 0)
                {
                    filterDefinitions.Add(Builders<RemedicalAISelectedInfoEntity>.Filter.Where(x => x.FrameIndex == index));
                }
                var filter = Builders<RemedicalAISelectedInfoEntity>.Filter.And(filterDefinitions);
                var entities = await _remedicalAISelectedInfoDBRepository.FindAllAsync(filter);
                return entities.MappingTo<List<RemedicalAISelectedInfoDTO>>();
            }
        }

        /// <summary>
        /// 保存AI图像详情
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<string> SaveRemedicalAISelectedInfoAsync(RemedicalAISelectedInfoDTO request)
        {
            var code = request.RemedicalAISelectedInfoCode;
            if (string.IsNullOrWhiteSpace(code))
            {
                var entity = request.MappingTo<RemedicalAISelectedInfoEntity>();
                var entityCode = await _remedicalAISelectedInfoDBRepository.InsertOneAsync(entity);
                return entityCode;
            }
            else
            {
                var filter = Builders<RemedicalAISelectedInfoEntity>.Filter.Where(x => x.Code == code);
                var entity = await _remedicalAISelectedInfoDBRepository.FindOneAsync(filter);
                if (entity != null)
                {
                    var updateEntity = request.MappingTo<RemedicalAISelectedInfoEntity>();
                    updateEntity.Code = entity.Code;
                    var result = await _remedicalAISelectedInfoDBRepository.UpdateOneAsync("Code", code, updateEntity, true);
                    if(result)
                    {
                        return entity.Code;
                    }
                }
            }
            return string.Empty;;
        }

        /// <summary>
        /// 保存VID图像地址
        /// </summary>
        /// <returns></returns>
        public async Task<bool> UpdateRemedicalAISelectedFileInfoAsync(UpdateRemedicalAISelectedFileInfoDBRequest request)
        {
            var code = request.RemedicalAISelectedInfoCode;
            if (!string.IsNullOrWhiteSpace(code))
            {
                var filter = Builders<RemedicalAISelectedInfoEntity>.Filter.Where(x => x.Code == code);
                var entity = await _remedicalAISelectedInfoDBRepository.FindOneAsync(filter);
                if (entity != null)
                {
                    entity.OrginalFileToken = request.OrginalFileToken;
                    entity.CdnFileToken = request.CdnFileToken;
                    // entity.AIFileToken = request.AIFileToken;
                    // entity.AICdnFileToken = request.AICdnFileToken;
                    // entity.PreviewFileToken = request.PreviewFileToken;
                    return await _remedicalAISelectedInfoDBRepository.UpdateOneAsync("Code", code, entity, true);
                }
            }
            return false;
        }

    }
}