using System;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using JsonRpcLite.Services;
using MongoDB.Driver;
using WingInterfaceLibrary.DB.Request;
using WingInterfaceLibrary.DTO.Comment;
using WingInterfaceLibrary.DTO.Common;
using WingInterfaceLibrary.DTO.DataCenter;
using WingInterfaceLibrary.DTO.Measure;
using WingInterfaceLibrary.DTO.Organization;
using WingInterfaceLibrary.DTO.Patient;
using WingInterfaceLibrary.DTO.Record;
using WingInterfaceLibrary.Enum;
using WingInterfaceLibrary.Interface.DBInterface;
using WingInterfaceLibrary.Request;
using WingInterfaceLibrary.Request.Comment;
using WingInterfaceLibrary.Request.DBCopy;
using WingInterfaceLibrary.Request.DBRequest;
using WingInterfaceLibrary.Request.Device;
using WingInterfaceLibrary.Request.Examine;
using WingInterfaceLibrary.Request.Record;
using WingInterfaceLibrary.Result;
using WingInterfaceLibrary.Result.Record;
using WingMongoDBModule.Common;
using WingMongoDBModule.DO;
using WingMongoDBModule.Entity;
using WingMongoDBModule.Mongodb.Driver;
using WingMongoDBModule.Request;
using WingServerCommon.Log;
using WingServerCommon.Mapper;
using WingServerCommon.Service;
using WingServerCommon.Utilities;

namespace WingMongoDBModule.Service
{
    public partial class DatabaseService : JsonRpcService, IRemedicalDBService
    {
        /// <summary>
        /// 获取分时诊断图像服务接口列表
        /// </summary>
        /// <param name="recordCodes">记录编码列表</param>
        /// <returns>记录对应的数据列表<returns>
        public async Task<List<RemedicalListResult>> GetRemedicalListByRecordCodeAsync(List<string> recordCodes)
        {
            var recordCodesString = recordCodes == null ? "" : string.Join(',', recordCodes);
            using (new Performance(_performanceThreshold, "{0} GetRemedicalListByRecordCodeAsync execute", $"{this.GetType().Name},recordCodes:{recordCodesString}"))
            {
                if (recordCodes?.Count > 0)
                {
                    var dic = new Dictionary<string, List<string>>();
                    dic.Add("RecordCode", recordCodes);
                    var remedicalList = await _remedicalDBRepository.FindInAsync(dic);
                    var remedicalDTOList = remedicalList.MappingTo<List<RemedicalInfoDTO>>();
                    var resList = new List<RemedicalListResult>();
                    if (remedicalDTOList?.Count > 0)
                    {
                        var recordFilter = new Dictionary<string, List<string>>();
                        recordFilter.Add("Code", recordCodes);
                        var recordList = await _recordInfoDBRepository.FindInAsync(recordFilter);
                        foreach (var recordCode in recordCodes)
                        {
                            var record = recordList.FirstOrDefault(x => x.Code == recordCode);
                            if (record == null)
                            {
                                continue;
                            }
                            var model = new RemedicalListResult()
                            {
                                RecordCode = recordCode
                            };
                            var applicationTypeList = remedicalDTOList.FindAll(c => c.RecordCode == recordCode)?.Select(c => (c.ApplicationCategory ?? string.Empty) + "-" + (c.Application ?? string.Empty))?.Distinct()?.ToList();
                            if (applicationTypeList == null || applicationTypeList.Count <= 0)
                            {
                                continue;
                            }
                            foreach (var applicationType in applicationTypeList)
                            {
                                // if (applicationType.Trim() == "-")
                                // {
                                //     continue;
                                // }
                                var rList = remedicalDTOList.FindAll(c => c.RecordCode == recordCode && ((c.ApplicationCategory ?? string.Empty) + "-" + (c.Application ?? string.Empty)) == applicationType);
                                if (rList == null || rList.Count <= 0)
                                {
                                    continue;
                                }
                                //列表不返回AI诊断结果
                                rList.ForEach(x => x.DiagnosisResult = string.Empty);
                                var remedicalInfoDTO = rList.First();
                                var applicationCategory = remedicalInfoDTO?.ApplicationCategory ?? string.Empty;
                                var application = remedicalInfoDTO.Application ?? string.Empty;
                                var patientScanTypeList = record.PatientInfoExtList?.Select(x => x.PatientScanType)?.ToList() ?? new List<string>();
                                var remedicalItem = new RemedicalItemList()
                                {
                                    PatientScanTypeDesc = applicationType,
                                    PatientScanTypeList = patientScanTypeList,
                                    ApplicationCategory = applicationCategory,
                                    Application = application,
                                    RemedicalList = rList
                                };
                                model.RemedicalItemList.Add(remedicalItem);
                            }
                            resList.Add(model);
                        }
                    }
                    return resList;
                }
                return null;
            }
        }

        /// <summary>
        /// 获取分时诊断图像服务接口列表
        /// </summary>
        /// <param name="recordCodes">记录编码列表</param>
        /// <returns>记录对应的数据列表<returns>
        public async Task<List<RemedicalInfoDTO>> GetAllRemedicalListAsync()
        {
            using (new Performance(_performanceThreshold, "{0} GetAllRemedicalListAsync execute", $"{this.GetType().Name}"))
            {
                var dic = new Dictionary<string, string>();
                dic.Add("IsDelete", "False");
                var remedicalList = await _remedicalDBRepository.FindListAsync(dic);
                var remedicalDTOList = remedicalList.MappingTo<List<RemedicalInfoDTO>>();
                //列表不返回AI诊断结果
                remedicalDTOList.ForEach(x => x.DiagnosisResult = string.Empty);
                return remedicalDTOList;
            }
        }

        /// <summary>
        /// 新增分时诊断图像
        /// </summary>
        /// <param name="request">新增分时诊断图像请求实体</param>
        /// <returns>图像code</returns>
        public async Task<string> InsertRemedicalInfoAsync(CreateRemedicalInfoDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} InsertRemedicalInfoAsync execute", $"{this.GetType().Name}"))
            {
                var entity = request.Data.MappingTo<RemedicalDO>();
                var remedicalCode = await _remedicalDBRepository.InsertOneAsync(entity, request.PlatformDatas, request.ExtensionData);
                if (entity.BusinessType == BusinessTypeEnum.RemoteDiagnosis)
                {
                    await AddPushRemedicalAsync(entity);
                }
                if (entity.ImgList != null && entity.ImgList.Any())
                {
                    await UpdatePatientUrmLabelAsync(entity.RecordCode);
                }
                return remedicalCode;
            }
        }

        /// <summary>
        /// 更新样本是否URM标识
        /// </summary>
        /// <param name="recordCode"></param>
        /// <returns></returns>
        private async Task UpdatePatientUrmLabelAsync(string recordCode)
        {
            try
            {
                var record = await _recordInfoDBRepository.FindOneAsync("Code", recordCode, false);
                if (!string.IsNullOrWhiteSpace(record?.PatientCode))
                {
                    var patient = await _patientInfoDBRepository.FindOneAsync("Code", record.PatientCode);
                    if (patient != null && !string.IsNullOrWhiteSpace(patient.Code))
                    {
                        Dictionary<string, string> recordFilter = new Dictionary<string, string>
                            {
                                { "PatientCode", patient.Code },
                                { "IsDelete", "False" }
                            };
                        var recordInfos = await _recordInfoDBRepository.FindListAsync(recordFilter);
                        if (recordInfos?.Any() ?? false)
                        {
                            var remedicalFilter = new Dictionary<string, List<string>>
                                {
                                    { "RecordCode", recordInfos.Select(x => x.RecordCode).ToList() }
                                };
                            var remedicalInfos = await _remedicalDBRepository.FindInAsync(remedicalFilter);
                            var isUrm = remedicalInfos != null && remedicalInfos.Any(x => x.ImgList != null && x.ImgList.Any(i => i.ImgType == 0));
                            if (patient.IsUrm != isUrm)
                            {
                                patient.IsUrm = isUrm;
                                await _patientInfoDBRepository.UpdateOneAsync("Code", patient.Code, patient, "");
                            }
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"RemedicalDBServce UpdatePatientUrmLabelAsync failed, ex:{ex}");
            }
        }

        public async Task<RemedicalInfoDTO> GetRemedicalByCodeAsync(string code, bool existDiagnosis = true)
        {
            using (new Performance(_performanceThreshold, "{0} GetRemedicalByCodeAsync execute", $"{this.GetType().Name}"))
            {
                var remedicalDO = await _remedicalDBRepository.FindOneAsync("Code", code);
                var dto = remedicalDO.MappingTo<RemedicalInfoDTO>();
                if (!existDiagnosis)
                {
                    dto.DiagnosisResult = string.Empty;
                }
                if (dto?.ImgList?.Count > 0)
                {
                    foreach(var item in dto.ImgList)
                    {
                        item.CoverImgUrl = item.CoverImgUrl.ToLocalUrl();
                        item.FileToken = item.FileToken.ToLocalUrl();
                        item.PreviewFileUrl = item.PreviewFileUrl.ToLocalUrl();
                    }
                }
                if (dto?.TerminalImages != null)
                {
                    dto.TerminalImages.CoverImageUrl = dto.TerminalImages.CoverImageUrl.ToLocalUrl();
                    dto.TerminalImages.ImageUrl = dto.TerminalImages.ImageUrl.ToLocalUrl();
                    dto.TerminalImages.OriginImageUrl = dto.TerminalImages.OriginImageUrl.ToLocalUrl();
                    dto.TerminalImages.PreviewUrl = dto.TerminalImages.PreviewUrl.ToLocalUrl();
                }
                return dto;
            }
        }

        public async Task<bool> ModifyRemedicalInfoAsync(string code, RemedicalInfoDTO data, string extensionData)
        {
            using (new Performance(_performanceThreshold, "{0} ModifyRemedicalInfoAsync execute", $"{this.GetType().Name}"))
            {
                var remedicalDO = await _remedicalDBRepository.FindOneAsync("Code", code);
                if (remedicalDO != null && !string.IsNullOrWhiteSpace(remedicalDO.RemedicalCode))
                {
                    data.DiagnosisResult = remedicalDO.DiagnosisResult ?? string.Empty;
                }
                var entity = data.MappingTo<RemedicalDO>();
                var result = await _remedicalDBRepository.UpdateOneAsync("Code", code, entity, extensionData);
                await AddPushRemedicalAsync(entity);
                return result;
            }
        }

        /// <summary>
        /// 查询检查检查关联所有图像
        /// </summary>
        /// <param name="recordCode"></param>
        /// <returns></returns>
        public async Task<List<RemedicalInfoDTO>> GetRecordRemedicalAsync(string recordCode)
        {
            using (new Performance(_performanceThreshold, "{0} GetRecordRemedicalAsync execute", $"{this.GetType().Name}"))
            {
                var dic = new Dictionary<string, string>();
                dic.Add("RecordCode", recordCode);
                var entities = await _remedicalDBRepository.FindListAsync(dic);
                return entities.MappingTo<List<RemedicalInfoDTO>>();
            }
        }

        internal async Task<PageResult<RemedicalInfoDTO>> GetRemedicalPagesByRecordCodeRAsync(PageFilterRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetRecordRemedicalPagesAsync execute", $"{this.GetType().Name}"))
            {
                var recordCode = request.Filter.FirstOrDefault(x => x.Key == "RecordCode").Value;
                MongoCustomerFilter<MongoTable> mongoCustomerFilter = new();
                mongoCustomerFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("RecordCode", recordCode));
                var filter = mongoCustomerFilter.AndCombine();
                PageResult<RemedicalInfoDTO> result = new PageResult<RemedicalInfoDTO>();
                result.PageIndex = request.CurrentPage;
                result.PageSize = request.PageSize;
                result.TotalCount = (int)await _remedicalDBRepository.CountDocumentsAsync(filter);
                SortDefinitionBuilder<MongoTable> builderSort = Builders<MongoTable>.Sort;
                SortDefinition<MongoTable> sort = builderSort.Descending("BaseData.CreateTime");
                var dbResult = await _remedicalDBRepository.FindAllAsync(filter,
                    new FindOptions<MongoTable>
                    {
                        NoCursorTimeout = true,
                        Skip = (request.CurrentPage - 1) * request.PageSize,
                        Limit = request.PageSize,
                        Sort = sort
                    }
                );
                //列表不返回AI诊断结果
                dbResult.ForEach(x => x.DiagnosisResult = string.Empty);
                result.PageData = dbResult.MappingTo<List<RemedicalInfoDTO>>();
                return result;
            }
        }

        /// <summary>
        /// 保存图像AI诊断结果
        /// </summary>
        /// <param name="code"></param>
        /// <param name="conclusion"></param>
        /// <param name="diagnosisResult"></param>
        /// <param name="diagnosisOrgans"></param>
        /// <returns></returns>
        public async Task<bool> UpdateDiagnosisResult(UpdateDiagnosisResultRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateDiagnosisResult execute", $"{this.GetType().Name}"))
            {
                var remedicalDO = await _remedicalDBRepository.FindOneAsync("Code", request.RemedicalCode);
                if (remedicalDO != null && !string.IsNullOrWhiteSpace(remedicalDO.Code))
                {
                    remedicalDO.DiagnosisConclusion = request.DiagnosisConclusion;
                    remedicalDO.DiagnosisResult = request.DiagnosisResult ?? string.Empty;
                    remedicalDO.DiagnosisOrgans = request.DiagnosisOrgans ?? new List<DiagnosisOrganEnum>();
                    if (request.CarotidResult != null)
                    {
                        remedicalDO.CarotidResult = request.CarotidResult.MappingTo<CarotidResultDO>();
                    }
                    return await _remedicalDBRepository.UpdateOneAsync("Code", request.RemedicalCode, remedicalDO, string.Empty);
                }
                return false;
            }
        }

        /// <summary>
        /// 保存图像AI诊断结果For同步数据
        /// </summary>
        /// <param name="code"></param>
        /// <param name="conclusion"></param>
        /// <param name="diagnosisResult"></param>
        /// <param name="diagnosisOrgans"></param>
        /// <returns></returns>
        public async Task<bool> UpdateDiagnosisForSyncDBResult(UpdateDiagnosisResultRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateDiagnosisResult execute", $"{this.GetType().Name}"))
            {
                var updateFilter = new MongoCustomerFilter<MongoTable>();
                updateFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("Code", request.RemedicalCode));
                MongoCustomerUpdate<MongoTable> update = new MongoCustomerUpdate<MongoTable>();
                update.SetField("BaseData.DiagnosisConclusion", request.DiagnosisConclusion);
                update.SetField("BaseData.DiagnosisResult", request.DiagnosisResult ?? string.Empty);
                update.SetField("BaseData.DiagnosisOrgans", request.DiagnosisOrgans ?? new List<DiagnosisOrganEnum>());
                if (request.CarotidResult != null)
                {
                    update.SetField("BaseData.CarotidResult", request.CarotidResult.MappingTo<CarotidResultDO>());
                }
                var result = await _remedicalDBRepository.UpdateOneAsync(updateFilter.AndCombine(), update.AllUpdateDefinition);
                return result > 0;
            }
        }

        /// <summary>
        /// 根据codes查询图像
        /// </summary>
        /// <param name="codes"></param>
        /// <returns></returns>
        public async Task<List<RemedicalInfoDTO>> GetRemedicalInfoListByCodes(IList<string> codes)
        {
            Dictionary<string, List<string>> filterRemedical = new Dictionary<string, List<string>>();
            filterRemedical.Add("Code", codes.ToList());
            var remedicalDatas = await _remedicalDBRepository.FindInAsync(filterRemedical);
            var result = remedicalDatas.MappingTo<List<RemedicalInfoDTO>>();
            return result;
        }

        /// <summary>
        /// 获取用户自定义测量配置项
        /// </summary>
        /// <param name="userId"></param>
        /// <returns></returns>
        public async Task<UserDefinedMeasureInfoDTO> GetUserDefinedMeasureInfoAsync(string userId)
        {
            using (new Performance(_performanceThreshold, "{0} GetUserDefinedMeasureInfoAsync execute", $"{this.GetType().Name}"))
            {
                var measureInfo = await _measureInfoRepository.FindOneAsync("UserId", userId);
                if (measureInfo != null)
                {
                    return measureInfo.MappingTo<UserDefinedMeasureInfoDTO>();
                }
                return null;
            }
        }
        /// <summary>
        /// 保存测量设置
        /// </summary>
        /// <param name="userId"></param>
        /// <param name="measureSystemSettingDict"></param>
        /// <returns></returns>
        public async Task<bool> SaveMeasureSystemSettingAsync(string userId, Dictionary<string, string> measureSystemSettingDict)
        {
            using (new Performance(_performanceThreshold, "{0} SaveMeasureSystemSettingAsync execute", $"{this.GetType().Name}"))
            {
                var measureInfo = await _measureInfoRepository.FindOneAsync("UserId", userId);
                if (measureInfo != null)
                {
                    foreach (var item in measureSystemSettingDict)
                    {
                        if (measureInfo.SystemSettings.ContainsKey(item.Key))
                        {
                            measureInfo.SystemSettings[item.Key] = item.Value;
                        }
                        else
                        {
                            measureInfo.SystemSettings.TryAdd(item.Key, item.Value);
                        }
                    }
                    await _measureInfoRepository.UpdateOneAsync("Code", measureInfo.Code, measureInfo, string.Empty);
                }
                else
                {
                    measureInfo = new UserDefinedMeasureInfoDO()
                    {
                        UserId = userId,
                        SystemSettings = measureSystemSettingDict
                    };
                    await _measureInfoRepository.InsertOneAsync(measureInfo, new Dictionary<string, string>(), "");
                }
                return true;
            }
        }

        /// <summary>
        /// 保存用户自定义测量配置项请求
        /// </summary>
        /// <param name="userId"></param>
        /// <param name="UserDefinedMeasureApplicationDTO"></param>
        /// <returns></returns>
        public async Task<bool> SaveUserDefinedMeasureApplicationAsync(string userId, UserDefinedMeasureApplicationDTO measureApplication)
        {
            using (new Performance(_performanceThreshold, "{0} SaveUserDefinedMeasureApplicationAsync execute", $"{this.GetType().Name}"))
            {
                var measureInfo = await _measureInfoRepository.FindOneAsync("UserId", userId);
                if (measureInfo != null)
                {
                    var existMeasureApplication = measureInfo.MeasureApplications.FirstOrDefault(v => v.Id == measureApplication.Id);
                    if (existMeasureApplication != null)
                    {
                        foreach (var mode in measureApplication.Modes)
                        {
                            var existMode = existMeasureApplication.Modes.FirstOrDefault(v => v.ModeName == mode.ModeName);

                            if (existMode != null)
                            {
                                foreach (var item in mode.WorkingGroups)
                                {
                                    var existGroup = existMode.WorkingGroups.FirstOrDefault(v => v.Name == item.Name);
                                    if (existGroup != null)
                                    {
                                        foreach (var folder in item.Folders)
                                        {
                                            var existFolder = existGroup.Folders.FirstOrDefault(v => v.Name == folder.Name);
                                            if (existFolder != null)
                                            {
                                                // 找到这个模式。在判断下面的测量项
                                                if (!string.IsNullOrEmpty(existFolder.ItemMetaJson))
                                                {
                                                    var existItemMetas = Newtonsoft.Json.JsonConvert.DeserializeObject<List<ItemMetaDTO>>(existFolder.ItemMetaJson);
                                                    if (existItemMetas.Count > 0)
                                                    {
                                                        foreach (var itemMetaInput in folder.ItemMetas)
                                                        {
                                                            var existItemMeta = existItemMetas.Find(c => c.Name == itemMetaInput.Name);
                                                            if (!string.IsNullOrEmpty(existItemMeta?.Name))
                                                            {
                                                                //存在，移除。在添加
                                                                existItemMetas.Remove(existItemMeta);
                                                            }
                                                            existItemMetas.Add(itemMetaInput);
                                                        }
                                                        existFolder.WorkingItemNames = folder.WorkingItemNames;
                                                        existFolder.ItemMetaJson = Newtonsoft.Json.JsonConvert.SerializeObject(existItemMetas);
                                                    }
                                                    else
                                                    {
                                                        existGroup.Folders.Remove(existFolder);
                                                        existGroup.Folders.Add(folder.MappingTo<UserDefinedMeasureFolderDO>());
                                                    }
                                                }
                                                else
                                                {
                                                    existGroup.Folders.Remove(existFolder);
                                                    existGroup.Folders.Add(folder.MappingTo<UserDefinedMeasureFolderDO>());
                                                }
                                            }
                                            else
                                            {
                                                existGroup.Folders.Add(folder.MappingTo<UserDefinedMeasureFolderDO>());
                                            }
                                        }
                                    }
                                    else
                                    {
                                        existMode.WorkingGroups.Add(item.MappingTo<UserDefinedMeasureGroupDO>());
                                    }
                                }
                            }
                            else
                            {
                                existMeasureApplication.Modes.Add(mode.MappingTo<UserDefinedMeasureModeDO>());
                            }
                        }
                    }
                    else
                    {
                        measureInfo.MeasureApplications.Add(measureApplication.MappingTo<UserDefinedMeasureApplicationDO>());
                    }
                    await _measureInfoRepository.UpdateOneAsync("Code", measureInfo.Code, measureInfo, string.Empty);
                }
                else
                {
                    measureInfo = new UserDefinedMeasureInfoDO()
                    {
                        UserId = userId,
                        MeasureApplications = new List<UserDefinedMeasureApplicationDO>()
                    {
                        measureApplication.MappingTo<UserDefinedMeasureApplicationDO>()
                    }
                    };
                    await _measureInfoRepository.InsertOneAsync(measureInfo, new Dictionary<string, string>(), "");
                }
                return true;
            }
        }
        /// <summary>
        /// 保存用户自定义注释项
        /// </summary>
        /// <param name="userId"></param>
        /// <param name="UserDefinedCommentDataDTO"></param>
        /// <returns></returns>
        public async Task<bool> SaveUserDefinedCommentAsync(string userId, UserDefinedCommentDataDTO commentDataDTO)
        {

            using (new Performance(_performanceThreshold, "{0} SaveUserDefinedCommentAsync execute", $"{this.GetType().Name}"))
            {
                var measureInfo = await _measureInfoRepository.FindOneAsync("UserId", userId);
                if (measureInfo != null)
                {
                    var commentItem = measureInfo.CommentItems.FirstOrDefault(v =>
                                      v.ApplicationName == commentDataDTO.ApplicationName &&
                                       v.LanguageCode == commentDataDTO.LanguageCode &&
                                        v.CategoryName == commentDataDTO.CategoryName);
                    if (commentItem != null)
                    {
                        measureInfo.CommentItems.Remove(commentItem);
                        commentItem = commentDataDTO.MappingTo<UserDefinedCommentDataDO>();
                        measureInfo.CommentItems.Add(commentItem);
                    }
                    else
                    {
                        measureInfo.CommentItems.Add(commentDataDTO.MappingTo<UserDefinedCommentDataDO>());
                    }
                    await _measureInfoRepository.UpdateOneAsync("Code", measureInfo.Code, measureInfo, string.Empty);
                }
                else
                {
                    measureInfo = new UserDefinedMeasureInfoDO()
                    {
                        UserId = userId,
                        CommentItems = new List<UserDefinedCommentDataDO>() { commentDataDTO.MappingTo<UserDefinedCommentDataDO>() }
                    };
                    await _measureInfoRepository.InsertOneAsync(measureInfo, new Dictionary<string, string>(), "");
                }
                return true;
            }
        }

        /// <summary>
        /// 导出诊断数据
        /// </summary>
        /// <param name="request">导出诊断数据请求实体</param>
        /// <returns></returns>
        public async Task<ExportRemedicalDataResult> ExportRemedicalDataDBAsync(ExportRemedicalDataDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} ExportRemedicalDataDBAsync execute", $"{this.GetType().Name}"))
            {
                var result = new ExportRemedicalDataResult();
                var records = new List<RecordInfoDTO>();
                var patients = await FindPatientInfoListByCodeListAsync(request.PatientCodes);
                foreach (var item in patients)
                {
                    item.Name = CryptionHelper.Decrypt(item.Name);
                }
                //patients = patients.Where(p => !string.IsNullOrWhiteSpace(p.Name)).ToList();
                if (request.IsExportRecord)
                {
                    records = await FindRecordInfosByPatientCodesAsync(patients.Select(p => p.PatientCode).ToList(), request.StartTime, request.EndTime);
                    foreach (var item in patients)
                    {
                        var recordData = new RecordData();
                        recordData.PatientCode = item.PatientCode;
                        recordData.PatientName = string.IsNullOrWhiteSpace(item.Name) ? GetNotFilled(request.LanguageCode) : item.Name;
                        recordData.RecordItemList = records.Where(r => r.PatientCode == item.PatientCode).ToList();
                        foreach (var recordInfo in recordData.RecordItemList)
                        {
                            SetEncryptedShow(item.PatientType, recordInfo.PatientInfo);
                        }
                        result.RecordList.Add(recordData);
                    }
                }
                if (request.IsExportReport)
                {
                    var reports = await GetReportByPatientCodesAsync(patients.Select(p => p.PatientCode).ToList(), request.StartTime, request.EndTime);
                    foreach (var item in patients)
                    {
                        var reportData = new ReportData();
                        reportData.PatientCode = item.PatientCode;
                        reportData.PatientName = string.IsNullOrWhiteSpace(item.Name) ? GetNotFilled(request.LanguageCode) : item.Name;
                        reportData.ReportItemList = new List<ReportItem>();
                        var itemReportList = reports.Where(r => r.PatientCode == item.PatientCode);
                        foreach (var reportDTO in itemReportList)
                        {
                            var reportItem = new ReportItem();
                            if (reportDTO.ReportPreviewList.Any(x => x.FileType == UploadFileTypeEnum.PDF))
                            {
                                //有pdf
                                reportItem = new ReportItem { ReportCode = reportDTO.ReportCode, RecordCode = reportDTO.RecordCode, FileToken = reportDTO.ReportPreviewList.FirstOrDefault(x => x.FileType == UploadFileTypeEnum.PDF)?.FileToken, CreateTime = reportDTO.CreateTime, ReportTime = reportDTO.ReportTime };
                            }
                            else
                            {
                                //没有pdf
                                reportItem = new ReportItem { ReportCode = reportDTO.ReportCode, RecordCode = reportDTO.RecordCode, FileToken = reportDTO.ReportPreviewList.FirstOrDefault()?.FileToken, CreateTime = reportDTO.CreateTime, ReportTime = reportDTO.ReportTime };
                            }
                            reportData.ReportItemList.Add(reportItem);
                        }
                        result.ReportList.Add(reportData);
                    }

                }
                if (request.IsExportRemedicalData)
                {
                    if (!request.IsExportRecord)
                    {
                        records = await FindRecordInfosByPatientCodesAsync(patients.Select(p => p.PatientCode).ToList(), request.StartTime, request.EndTime);
                    }
                    var recordCodes = records.Select(r => r.RecordCode).ToList();
                    var remedicalDatas = await GetRecordRemedicalsByRecordsAsync(recordCodes, request.StartTime, request.EndTime);
                    foreach (var item in patients)
                    {
                        var itemRecords = records.Where(r => r.PatientCode == item.PatientCode);
                        var itemRecordCodes = itemRecords.Select(r => r.RecordCode);
                        var itemRemedicalDatas = remedicalDatas.Where(r => itemRecordCodes.Contains(r.RecordCode));
                        var remedicalData = new RemedicalData();
                        remedicalData.PatientCode = item.PatientCode;
                        remedicalData.PatientName = string.IsNullOrWhiteSpace(item.Name) ? GetNotFilled(request.LanguageCode) : item.Name;
                        remedicalData.RemedicalItemList = itemRemedicalDatas.Select(x => new RemedicalItem { RecordCode = x.RecordCode, FileToken = x.TerminalImages.ImageUrl, FileDataType = x.FileDataType }).ToList();
                        result.RemedicalList.Add(remedicalData);
                    }
                }
                return result;
            }
        }

        private void SetEncryptedShow(string patientInfoType, List<DataItemDTO> patientData)
        {
            if (patientInfoType == OrganizationPatientTypeEnum.Animals.ToString())
            {
                if (patientData.FirstOrDefault(x => x.Key == "AnimalInfoName") != null && !string.IsNullOrWhiteSpace(patientData.FirstOrDefault(x => x.Key == "AnimalInfoName").Value))
                {
                    patientData.FirstOrDefault(x => x.Key == "AnimalInfoName").Value = CryptionHelper.Decrypt(patientData.FirstOrDefault(x => x.Key == "AnimalInfoName").Value);
                }
                if (patientData.FirstOrDefault(x => x.Key == "AnimalInfoPhone") != null && !string.IsNullOrWhiteSpace(patientData.FirstOrDefault(x => x.Key == "AnimalInfoPhone").Value))
                {
                    patientData.FirstOrDefault(x => x.Key == "AnimalInfoPhone").Value = CryptionHelper.Decrypt(patientData.FirstOrDefault(x => x.Key == "AnimalInfoPhone").Value);
                }
                if (patientData.FirstOrDefault(x => x.Key == "AnimalInfoIdentityCard") != null && !string.IsNullOrWhiteSpace(patientData.FirstOrDefault(x => x.Key == "AnimalInfoIdentityCard").Value))
                {
                    patientData.FirstOrDefault(x => x.Key == "AnimalInfoIdentityCard").Value = CryptionHelper.Decrypt(patientData.FirstOrDefault(x => x.Key == "AnimalInfoIdentityCard").Value);
                }
            }
            else
            {
                if (patientData.FirstOrDefault(x => x.Key == "Name") != null && !string.IsNullOrWhiteSpace(patientData.FirstOrDefault(x => x.Key == "Name").Value))
                {
                    patientData.FirstOrDefault(x => x.Key == "Name").Value = CryptionHelper.Decrypt(patientData.FirstOrDefault(x => x.Key == "Name").Value);
                }
                if (patientData.FirstOrDefault(x => x.Key == "Phone") != null && !string.IsNullOrWhiteSpace(patientData.FirstOrDefault(x => x.Key == "Phone").Value))
                {
                    patientData.FirstOrDefault(x => x.Key == "Phone").Value = CryptionHelper.Decrypt(patientData.FirstOrDefault(x => x.Key == "Phone").Value);
                }
                if (patientData.FirstOrDefault(x => x.Key == "IdentityCard") != null && !string.IsNullOrWhiteSpace(patientData.FirstOrDefault(x => x.Key == "IdentityCard").Value))
                {
                    patientData.FirstOrDefault(x => x.Key == "IdentityCard").Value = CryptionHelper.Decrypt(patientData.FirstOrDefault(x => x.Key == "IdentityCard").Value);
                }
                if (patientData.FirstOrDefault(x => x.Key == "InsuranceCode") != null && !string.IsNullOrWhiteSpace(patientData.FirstOrDefault(x => x.Key == "InsuranceCode").Value))
                {
                    patientData.FirstOrDefault(x => x.Key == "InsuranceCode").Value = CryptionHelper.Decrypt(patientData.FirstOrDefault(x => x.Key == "InsuranceCode").Value);
                }
            }

        }

        /// <summary>
        /// 查询检查检查关联所有图像
        /// </summary>
        /// <param name="recordCode"></param>
        /// <returns></returns>
        internal async Task<List<RemedicalInfoDTO>> GetRecordRemedicalsByRecordsAsync(List<string> recordCodes, DateTime startTime, DateTime endTime)
        {
            if (recordCodes == null || recordCodes.Count == 0)
            {
                return new List<RemedicalInfoDTO>();
            }
            MongoCustomerFilter<MongoTable> customerFilter = new();
            MongoCustomerFilter<MongoDataItem> queryItemsFilter = new();
            queryItemsFilter.AppendEq((item => item.Key, "RecordCode"));
            queryItemsFilter.AppendIn(item => item.Value, recordCodes);
            if (queryItemsFilter.HasFilter)
            {
                customerFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
            }
            if (startTime > DateTime.MinValue)
            {
                customerFilter.AppendGte("BaseData.CreateTime", startTime);
            }
            if (endTime > DateTime.MinValue)
            {
                customerFilter.AppendLt("BaseData.CreateTime", endTime);
            }
            var entities = await _remedicalDBRepository.FindAllAsync(customerFilter.AndCombine());
            return entities.MappingTo<List<RemedicalInfoDTO>>();
        }

        /// <summary>
        /// 查询转诊记录
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<ReferralRecordDTO> GetReferralRecordInfoAsync(GetReferralRecordByCodeDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetReferralRecordInfoAsync execute", $"{this.GetType().Name}"))
            {
                var referralRecord = await _referralRecordDBRepository.FindOneAsync("Code", request.Code);
                var dto = referralRecord.MappingTo<ReferralRecordDTO>();
                return dto;
            }
        }

        /// <summary>
        /// 查询转诊记录
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        internal async Task<List<ReferralRecordDTO>> GetReferralRecordInfosAsync(IList<string> codes)
        {
            if (codes == null || codes.Count == 0)
            {
                return new List<ReferralRecordDTO>();
            }
            var dic = new Dictionary<string, List<string>>();
            dic.Add("Code", codes.ToList());
            var referralRecords = await _referralRecordDBRepository.FindInAsync(dic);
            var list = referralRecords.MappingTo<List<ReferralRecordDTO>>();
            return list;
        }

        /// <summary>
        /// 创建转诊记录
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<string> CreateReferralRecordAsync(CreateReferralRecordDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} CreateReferralRecordAsync execute", $"{this.GetType().Name}"))
            {
                var patientCode = request.Data.PatientCode;
                var examRecords = await FindRecordInfosByPatientCodesAsync(new List<string> { patientCode }, DateTime.MinValue, DateTime.MinValue);
                if (examRecords == null || !examRecords.Any())
                {
                    ThrowCustomerException(CustomerRpcCode.ExamRecordsIsEmpty, "Exam Records Is Empty");
                }
                //转诊创建报告code
                var reportCodeList = new List<string>();
                //转诊链路
                var referralCodeList = new List<string>();
                if (request.Data.ReferralCodeList != null && request.Data.ReferralCodeList.Count() > 0)
                {
                    var getReferralRecordByCodeDBRequest = new GetReferralRecordByCodeDBRequest { Code = request.Data.ReferralCodeList.FirstOrDefault() };
                    //上个转诊单
                    var lastReferralRecord = await GetReferralRecordInfoAsync(getReferralRecordByCodeDBRequest);
                    if (lastReferralRecord.ReferralStatus != ReferralStatusEnum.Accepted)
                    {
                        //已同意才可以继续转诊
                        ThrowCustomerException(CustomerRpcCode.ReferralNotAccepted, "Referral Not Accepted");
                    }
                    var userInfoDTO = await FindUserByCodeAsync(request.Data.Creator);
                    if (lastReferralRecord.ReferralInUserCode != request.Data.Creator && !(userInfoDTO.RootOrganizationCode == lastReferralRecord.ReferralInOrganizationCode && userInfoDTO.IsDirector))
                    {
                        //不是转入医生且不是转入机构的机构负责人 无权操作此转诊单
                        ThrowCustomerException(CustomerRpcCode.CanNotReferral, "Can Not Referral");
                    }
                    //当前转诊链路所有转诊单
                    var referralRecordInfos = await GetReferralRecordInfosAsync(lastReferralRecord.ReferralCodeList);
                    var users = referralRecordInfos.Select(r => r.ReferralInUserCode).ToList();
                    users.AddRange(referralRecordInfos.Select(r => r.Creator));
                    if (users.Contains(request.Data.ReferralInUserCode))
                    {
                        //循环转诊
                        ThrowCustomerException(CustomerRpcCode.ReferralIsCircle, "Referral Is Circle");
                    }
                    //copy时把上一个人写的报告带上
                    if (lastReferralRecord.CreateReportCodeList != null && lastReferralRecord.CreateReportCodeList.Count > 0)
                    {
                        reportCodeList.AddRange(lastReferralRecord.CreateReportCodeList.Select(c => c.ReportCode));
                    }
                    referralCodeList = lastReferralRecord.ReferralCodeList;
                }
                var patientInfo = await FindPatientByCodeAsync(patientCode);
                //必须是已登记病人
                if (patientInfo == null || patientInfo.IsValid == false)
                {
                    ThrowCustomerException(CustomerRpcCode.PatientInfoIsNotValid, "Patient Is NotValid");
                }
                //验证病人是否有未结束的扫查
                if (examRecords.Any(x => x.RecordStatus == RecordStatusEnum.NotScanned || x.RecordStatus == RecordStatusEnum.Uploaded))
                {
                    ThrowCustomerException(CustomerRpcCode.ExistUnCompleteScan, "Exist UnComplete Scan");
                }
                //验证此机构是否可转入
                var user = await FindUserByCodeAsync(request.Data.Creator);
                var organization = await GetOrganizationDetailAsync(new GetOrganizationDBRequest { OrganizationCode = user.RootOrganizationCode });
                if (!organization.ReferralOrganizationCodes.Contains(request.Data.ReferralInOrganizationCode))
                {
                    ThrowCustomerException(CustomerRpcCode.NoRightToReferralInThisOrg, "No Right To ReferralIn This Org");
                }
                var referralRecordDO = request.Data.MappingTo<ReferralRecordDO>();
                referralRecordDO.Code = CodeCreator.CreateCode("ReferralRecordDO");
                referralCodeList.Add(referralRecordDO.Code);
                referralRecordDO.ReferralCodeList = referralCodeList;
                referralRecordDO.PatientInfo = patientInfo.MappingTo<PatientInfoDO>();
                referralRecordDO.PatientInfo.IsReferral = false;//转诊单刚申请时为未转诊，同意后改为已转诊
                var reports = await GetReportByPatientCodeAsync(patientCode);
                if (examRecords.Count > 0)
                {
                    var recordCodeList = examRecords.Select(x => x.RecordCode).ToList();
                    referralRecordDO.RecordCodeList = recordCodeList;
                    var remedicalDatas = await GetRecordRemedicalsByRecordsAsync(examRecords.Select(x => x.RecordCode).ToList(), DateTime.MinValue, DateTime.MinValue);
                    if (remedicalDatas.Count > 0)
                    {
                        referralRecordDO.RemedicalDataCodeList = remedicalDatas.Select(x => x.RemedicalCode).ToList();
                    }
                }
                if (reports.Count > 0)
                {
                    reportCodeList.AddRange(reports.Select(x => x.ReportCode).ToList());
                }
                referralRecordDO.ReportCodeList = reportCodeList.Where(r => !string.IsNullOrWhiteSpace(r)).ToList();
                await _referralRecordDBRepository.InsertOneAsync(referralRecordDO, new Dictionary<string, string>(), "");
                return referralRecordDO.Code;
            }
        }

        /// <summary>
        /// 修改转诊记录
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<string> ModifyReferralRecordAsync(CreateReferralRecordDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} ModifyReferralRecordAsync execute", $"{this.GetType().Name}"))
            {
                var referralRecordDO = await _referralRecordDBRepository.FindOneAsync("Code", request.Data.Code);
                if (referralRecordDO.ReferralStatus == ReferralStatusEnum.Wait || referralRecordDO.ReferralStatus == ReferralStatusEnum.Accepted)
                {
                    ThrowCustomerException(CustomerRpcCode.ReferralCanNotModify, "Referral status can not modify");
                }
                if (referralRecordDO.ReferralCodeList.Count > 1)
                {
                    request.Data.ReferralCodeList.Add(referralRecordDO.ReferralCodeList.Where(r => r != referralRecordDO.Code)?.LastOrDefault());
                }
                var patientCode = referralRecordDO.PatientCode;
                var examRecords = await FindRecordInfosByPatientCodesAsync(new List<string> { patientCode }, DateTime.MinValue, DateTime.MinValue);
                //转诊创建报告code
                var reportCodeList = new List<string>();
                //转诊链路
                var referralCodeList = new List<string>();
                if (request.Data.ReferralCodeList != null && request.Data.ReferralCodeList.Count() > 0)
                {
                    var getReferralRecordByCodeDBRequest = new GetReferralRecordByCodeDBRequest { Code = request.Data.ReferralCodeList.FirstOrDefault() };
                    //上个转诊单
                    var lastReferralRecord = await GetReferralRecordInfoAsync(getReferralRecordByCodeDBRequest);
                    if (lastReferralRecord.ReferralStatus != ReferralStatusEnum.Accepted)
                    {
                        //已同意才可以继续转诊
                        ThrowCustomerException(CustomerRpcCode.ReferralNotAccepted, "Referral Not Accepted");
                    }
                    //当前转诊链路所有转诊单
                    var referralRecordInfos = await GetReferralRecordInfosAsync(lastReferralRecord.ReferralCodeList);
                    var users = referralRecordInfos.Select(r => r.ReferralInUserCode).ToList();
                    users.AddRange(referralRecordInfos.Select(r => r.Creator));
                    if (users.Contains(request.Data.ReferralInUserCode))
                    {
                        //循环转诊
                        ThrowCustomerException(CustomerRpcCode.ReferralIsCircle, "Referral Is Circle");
                    }
                    //copy时把上一个人写的报告带上
                    if (lastReferralRecord.CreateReportCodeList != null && lastReferralRecord.CreateReportCodeList.Count > 0)
                    {
                        reportCodeList.AddRange(lastReferralRecord.CreateReportCodeList.Select(c => c.ReportCode));
                    }
                    referralCodeList = lastReferralRecord.ReferralCodeList;
                }
                var patientInfo = await FindPatientByCodeAsync(patientCode);
                //必须是已登记病人
                if (patientInfo == null || patientInfo.IsValid == false)
                {
                    ThrowCustomerException(CustomerRpcCode.PatientInfoIsNotValid, "Patient Is NotValid");
                }

                //验证病人是否有未结束的扫查
                if (examRecords.Any(x => x.RecordStatus == RecordStatusEnum.NotScanned || x.RecordStatus == RecordStatusEnum.Uploaded))
                {
                    ThrowCustomerException(CustomerRpcCode.ExistUnCompleteScan, "Exist UnComplete Scan");
                }
                //验证此机构是否可转入
                var user = await FindUserByCodeAsync(referralRecordDO.Creator);
                var organization = await GetOrganizationDetailAsync(new GetOrganizationDBRequest { OrganizationCode = user.RootOrganizationCode });
                if (!organization.ReferralOrganizationCodes.Contains(referralRecordDO.ReferralInOrganizationCode))
                {
                    ThrowCustomerException(CustomerRpcCode.NoRightToReferralInThisOrg, "No Right To ReferralIn This Org");
                }
                referralCodeList.Add(referralRecordDO.Code);
                referralRecordDO.SubjectMatter = request.Data.SubjectMatter;
                referralRecordDO.ReferralInOrganizationCode = request.Data.ReferralInOrganizationCode;
                referralRecordDO.ReferralInUserCode = request.Data.ReferralInUserCode;
                referralRecordDO.ReferralOutTime = request.Data.ReferralOutTime;
                referralRecordDO.ReferralCodeList = referralCodeList;
                referralRecordDO.PatientInfo = patientInfo.MappingTo<PatientInfoDO>();
                referralRecordDO.PatientInfo.IsReferral = false;
                var reports = await GetReportByPatientCodeAsync(patientCode);
                if (examRecords.Count > 0)
                {
                    var recordCodeList = examRecords.Select(x => x.RecordCode).ToList();
                    referralRecordDO.RecordCodeList = recordCodeList;
                    var remedicalDatas = await GetRecordRemedicalsByRecordsAsync(examRecords.Select(x => x.RecordCode).ToList(), DateTime.MinValue, DateTime.MinValue);
                    if (remedicalDatas.Count > 0)
                    {
                        referralRecordDO.RemedicalDataCodeList = remedicalDatas.Select(x => x.RemedicalCode).ToList();
                    }
                }
                if (reports.Count > 0)
                {
                    reportCodeList.AddRange(reports.Select(x => x.ReportCode).ToList());
                }
                referralRecordDO.ReportCodeList = reportCodeList.Where(r => !string.IsNullOrWhiteSpace(r)).ToList();
                referralRecordDO.ReferralStatus = ReferralStatusEnum.Wait;
                await _referralRecordDBRepository.UpdateOneAsync("Code", referralRecordDO.Code, referralRecordDO, "");
                return referralRecordDO.Code;
            }
        }


        /// <summary>
        /// 根据状态查询待处理记录
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<List<ReferralRecordDTO>> GetReferralRecordList(GetReferralRecordDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetReferralRecordList execute", $"{this.GetType().Name}"))
            {
                var filter = new Dictionary<string, string>();
                filter.Add("ReferralStatus", request.ReferralStatus.ToString());
                var referralRecordList = await _referralRecordDBRepository.FindListAsync(filter);
                return referralRecordList.MappingTo<List<ReferralRecordDTO>>();
            }
        }

        /// <summary>
        /// 更新转诊记录信息
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> UpdateReferralRecordInfo(ReferralRecordDTO request)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateReferralRecordInfo execute", $"{this.GetType().Name}"))
            {
                var referralRecordDO = request.MappingTo<ReferralRecordDO>();
                return await _referralRecordDBRepository.UpdateOneAsync("Code", referralRecordDO.Code, referralRecordDO, "");
            }
        }

        /// <summary>
        /// 分页查询转诊记录列表
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageResult<ReferralRecordDTO>> QueryReferralRecordListAsync(QueryReferralRecordDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} QueryReferralRecordListAsync execute", $"{this.GetType().Name}"))
            {
                var allFilter = await GetReferralRecordsFilter(request);
                PageResult<ReferralRecordDTO> result = new PageResult<ReferralRecordDTO>();
                result.PageIndex = request.CurrentPage;
                result.PageSize = request.PageSize;
                result.TotalCount = (int)await _referralRecordDBRepository.CountDocumentsAsync(allFilter);
                SortDefinitionBuilder<MongoTable> builderSort = Builders<MongoTable>.Sort;
                SortDefinition<MongoTable> sort = builderSort.Descending("BaseData.ReferralOutTime");//builderSort.Ascending("BaseData.PatientInfo.IsValid").Descending("BaseData.ReferralOutTime");
                var patientList = await _referralRecordDBRepository.FindAllAsync(allFilter,
                    new FindOptions<MongoTable>
                    {
                        NoCursorTimeout = true,
                        Skip = (request.CurrentPage - 1) * request.PageSize,
                        Limit = request.PageSize,
                        Sort = sort
                    }
                );
                //是否转诊逻辑
                var entities = new List<ReferralRecordDO>();
                if (patientList.Count > 0)
                {
                    var userDTO = await FindUserByCodeAsync(request.UserCode);
                    MongoCustomerFilter<MongoTable> customerFilter = new();
                    customerFilter.AppendAnyIn("BaseData.ReferralCodeList", patientList.Select(p => p.Code).ToList());
                    entities = await _referralRecordDBRepository.FindAllAsync(customerFilter.AndCombine(), new FindOptions<MongoTable> { NoCursorTimeout = true });
                    foreach (var item in patientList)
                    {
                        if (item.PatientInfo.IsReferral)
                        {
                            if (!entities.Any(e => e.ReferralCodeList.Contains(item.Code) && e.Code != item.Code && e.ReferralOutOrganizationCode == userDTO.RootOrganizationCode && e.ReferralStatus == ReferralStatusEnum.Accepted))
                            {
                                item.PatientInfo.IsReferral = false;
                            }
                        }
                    }
                }
                result.PageData = patientList.MappingTo<List<ReferralRecordDTO>>();
                return result;
            }
        }

        /// <summary>
        /// 查询转诊记录过滤条件
        /// </summary>
        /// <param name="request">请求参数</param>
        /// <returns></returns>
        private async Task<FilterDefinition<MongoTable>> GetReferralRecordsFilter(QueryReferralRecordDBRequest request)
        {
            MongoCustomerFilter<MongoTable> customerChildFilter1 = new();
            MongoCustomerFilter<MongoTable> customerFilter = new();
            MongoCustomerFilter<MongoTable> baseDataOrFilter = new();
            MongoCustomerFilter<MongoDataItem> itemFilter = new();
            var positionCodeInfo = request.Filter.FirstOrDefault(f => f.Key == "PositionCode");
            //主管
            if (!string.IsNullOrWhiteSpace(positionCodeInfo.Value) && positionCodeInfo.Value == "PositionCode01")
            {
                var users = await FindUserListByOrganizationCodesAsync(request.OrgCodes);
                if (users.Count() > 0)
                {
                    customerFilter.AppendIn("BaseData.ReferralInUserCode", users.Select(u => u.UserCode).ToList());
                }
                else
                {
                    customerFilter.AppendEq(("BaseData.Code", "-1"));
                }
            }
            else if (request.OrgCodes?.Count > 0)//机构负责人
            {
                customerFilter.AppendIn("BaseData.ReferralInOrganizationCode", request.OrgCodes);
            }
            else//普通员工
            {
                var keyinfo = request.Filter.FirstOrDefault(v => v.Key == "CreatorCode");
                customerChildFilter1.AppendEq(("BaseData.ReferralInUserCode", keyinfo.Value));
            }
            MongoCustomerFilter<MongoTable> sumItemFilter = new();
            if (request.KeywordOrFilter?.Count > 0)
            {
                foreach (var kv in request.KeywordOrFilter)
                {
                    sumItemFilter.AppendRegex("BaseData.PatientInfo." + kv.Key, kv.Value, true, false);
                }
            }
            itemFilter.AppendEq((item => item.Key, "IsDelete"), (item => item.Value, "False"));
            customerFilter.SubFilter(item => item.QueryData.QueryItems, itemFilter.AndCombine());
            itemFilter.AppendEq((item => item.Key, "ReferralStatus"));
            itemFilter.AppendIn(item => item.Value, new List<string>() { ReferralStatusEnum.Wait.ToString(), ReferralStatusEnum.Accepted.ToString(), ReferralStatusEnum.Rejected.ToString() });
            customerFilter.SubFilter(item => item.QueryData.QueryItems, itemFilter.AndCombine());
            if (request.IsValid.GetHashCode() != 0)
            {
                if (request.IsValid == WingInterfaceLibrary.Enum.PatientValidStatusEnum.CheckOut)
                {
                    customerFilter.AppendEq(("BaseData.PatientInfo.IsValid", "False"));
                }
                else
                {
                    customerFilter.AppendEq(("BaseData.PatientInfo.IsValid", "True"));
                }
            }
            if (request.StartTime > DateTime.MinValue)
            {
                customerFilter.AppendGte("BaseData.ReferralOutTime", request.StartTime);
            }
            if (request.EndTime > DateTime.MinValue)
            {
                customerFilter.AppendLt("BaseData.ReferralOutTime", request.EndTime.AddDays(1));
            }
            var resFilter = baseDataOrFilter.AndCombine(sumItemFilter.OrCombine(), customerChildFilter1.OrCombine());
            return customerFilter.AndCombine(resFilter);
        }

        /// <summary>
        /// 查询患者转诊历史
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<List<ReferralRecordDTO>> GetPatientReferralRecordAsync(GetPatientReferralRecordDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetPatientReferralRecordAsync execute", $"{this.GetType().Name}"))
            {
                var patientCode = request.PatientCode;
                var dic = new Dictionary<string, string>();
                dic.Add("PatientCode", patientCode);
                var entities = await _referralRecordDBRepository.FindListAsync(dic);
                //根据转出时间倒叙
                entities = entities.OrderByDescending(x => x.ReferralOutTime).ToList();
                return entities.MappingTo<List<ReferralRecordDTO>>();
            }
        }

        /// <summary>
        /// 查询转诊转入历史
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<List<ReferralRecordDTO>> GetReferralInRecordAsync(GetReferralInRecordDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetReferralInRecordAsync execute", $"{this.GetType().Name}"))
            {
                var referralCode = request.ReferralRecordCode;
                MongoCustomerFilter<MongoTable> customerFilter = new();
                customerFilter.AppendArrayEq("BaseData.ReferralCodeList", referralCode);
                var entities = await _referralRecordDBRepository.FindAllAsync(customerFilter.AndCombine(), new FindOptions<MongoTable> { NoCursorTimeout = true });
                //根据转出时间倒叙
                entities = entities.OrderByDescending(x => x.ReferralOutTime).ToList();
                return entities.MappingTo<List<ReferralRecordDTO>>();
            }
        }

        /// <summary>
        /// 更新用户自定义测量
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> UpdateUserDefinedMeasureInfoAsync(UpdateUserDefinedMeasureInfoDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateUserDefinedMeasureInfoAsync execute", $"{this.GetType().Name}"))
            {
                var userCode = request.UserCode;
                var measureInfo = await _measureInfoRepository.FindOneAsync("UserId", userCode);
                if (measureInfo != null)
                {
                    measureInfo = request.UserDefinedMeasureInfo.MappingTo<UserDefinedMeasureInfoDO>();
                    return await _measureInfoRepository.UpdateOneAsync("UserId", userCode, measureInfo, string.Empty);
                }
                return false;
            }
        }

        /// <summary>
        /// 同步检查记录
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> SyncBatchRemedicalRecordsDBAsync(SyncBatchRemedicalRecordRequest request)
        {
            try
            {
                SyncDBEnum syncType = request.SyncType;
                var insertList = new List<RecordInfoDO>();
                var updateList = new List<RecordInfoDO>();
                var orgs = new List<OrganizationDO>();
                var orgCodes = request?.RemedicalRecordList?.Select(x => x.OrganizationCode).ToList();
                if (orgCodes != null && orgCodes.Count() > 0)
                {
                    MongoCustomerFilter<MongoTable> filter = new();
                    filter.AppendQueryItemInFilter(MongoCombineTypeEnum.Or, "Code", orgCodes);
                    orgs = await _organizationDBRepository.FindAllAsync(filter.AndCombine());
                }
                var recordCodes = request.RemedicalRecordList.Select(x => x.Code).ToList();
                Dictionary<string, List<string>> filterRecord = new Dictionary<string, List<string>>();
                filterRecord.Add("Code", recordCodes);
                var recordInfos = await _recordInfoDBRepository.FindInAsync(filterRecord);
                Dictionary<string, List<string>> filterPatient = new Dictionary<string, List<string>>();
                filterPatient.Add("RootOrganizationCode", orgCodes);
                var patients = await _patientInfoDBRepository.FindInAsync(filterPatient);
                foreach (var remedicalRecord in request.RemedicalRecordList)
                {
                    try
                    {
                        var isUpdate = false;
                        var recordCode = remedicalRecord.Code;
                        //判断数据是否已存在
                        var recordInfo = recordInfos.FirstOrDefault(x => x.RecordCode == remedicalRecord.Code);
                        if (recordInfo == null || string.IsNullOrWhiteSpace(recordInfo.RecordCode))
                        {
                            recordInfo = new RecordInfoDO();
                            recordInfo.RecordCode = recordCode;
                        }
                        else
                        {
                            isUpdate = true;
                        }
                        var creator = remedicalRecord.Creator;
                        recordInfo.IsDelete = remedicalRecord.IsDelete;
                        //注意创建时间、修改时间
                        recordInfo.CreateTime = remedicalRecord.CreateTime;
                        recordInfo.CreatorCode = creator;
                        recordInfo.UpdateTime = remedicalRecord.UpdateTime;
                        recordInfo.CreateType = string.IsNullOrWhiteSpace(creator) ? RecordCreateTypeEnum.Reservation : RecordCreateTypeEnum.Normal;
                        recordInfo.DeviceCode = remedicalRecord.DeviceCode;
                        var org = orgs.FirstOrDefault(x => x.OrganizationCode == remedicalRecord.OrganizationCode);
                        if (org == null || string.IsNullOrWhiteSpace(org.OrganizationCode))
                        {
                            Logger.WriteLineWarn($"IRemedicalDBService-SyncBatchRemedicalRecordsDBAsync org is not exist,recordCode:{recordCode},OrganizationCode:{remedicalRecord.OrganizationCode}");
                            continue;
                        }
                        else
                        {
                            if (org.PatientType == OrganizationPatientTypeEnum.Animals)
                            {
                                var patientData = remedicalRecord.PatientInfo;
                                if (patientData.FirstOrDefault(x => x.Key == "Name") != null && !string.IsNullOrWhiteSpace(patientData.FirstOrDefault(x => x.Key == "Name").Value))
                                {
                                    patientData.FirstOrDefault(x => x.Key == "Name").Key = "AnimalInfoName";
                                }
                                if (patientData.FirstOrDefault(x => x.Key == "Phone") != null && !string.IsNullOrWhiteSpace(patientData.FirstOrDefault(x => x.Key == "Phone").Value))
                                {
                                    patientData.FirstOrDefault(x => x.Key == "Phone").Key = "AnimalInfoPhone";
                                }
                                if (patientData.FirstOrDefault(x => x.Key == "ID") != null && !string.IsNullOrWhiteSpace(patientData.FirstOrDefault(x => x.Key == "ID").Value))
                                {
                                    patientData.FirstOrDefault(x => x.Key == "ID").Key = "AnimalInfoID";
                                }
                                if (patientData.FirstOrDefault(x => x.Key == "Sex") != null && !string.IsNullOrWhiteSpace(patientData.FirstOrDefault(x => x.Key == "Sex").Value))
                                {
                                    patientData.FirstOrDefault(x => x.Key == "Sex").Key = "AnimalInfoSex";
                                    patientData.FirstOrDefault(x => x.Key == "AnimalInfoSex").Value = "AnimalInfo" + patientData.FirstOrDefault(x => x.Key == "AnimalInfoSex").Value;
                                }
                            }
                        }
                        recordInfo.PatientType = org.PatientType.ToString();
                        var devicePatientId = GetPatientId(remedicalRecord.PatientInfo, org.PatientType);
                        recordInfo.DevicePatientID = devicePatientId;
                        //注意加密和创建时间、修改时间
                        SetEncryptedData(remedicalRecord.PatientInfo);
                        recordInfo.PatientCode = await AddOrUpdatePatientForSync(patients, remedicalRecord.PatientInfo, remedicalRecord.DeviceCode, org, remedicalRecord.CreateTime, remedicalRecord.UpdateTime); //使用姓名+病人ID+机构ID来识别病人唯一性
                        recordInfo.PatientInfo = remedicalRecord.PatientInfo.MappingTo<List<DataItem>>();//存储病人快照信息
                        recordInfo.OrganizationCode = org.OrganizationCode;
                        recordInfo.RootOrganizationCode = org.RootCode;
                        //AI
                        recordInfo.DiagnosisStatus = remedicalRecord.DiagnosisStatus;
                        var diagnosisInfos = recordInfo.DiagnosisInfos?.ToList() ?? new List<DiagnosisInfoDO>();
                        if (remedicalRecord.AIDiagnosisInfos != null && remedicalRecord.AIDiagnosisInfos.Any())
                        {
                            var list = remedicalRecord.AIDiagnosisInfos.MappingTo<List<DiagnosisInfoDO>>();
                            foreach (var item in list)
                            {
                                if (!diagnosisInfos.Any(x => x.DiagnosisOrgan == item.DiagnosisOrgan))
                                {
                                    diagnosisInfos.Add(item);
                                }
                            }
                        }
                        recordInfo.DiagnosisInfos = diagnosisInfos;
                        recordInfo.RecordStatus = syncType == SyncDBEnum.Migrate ? RecordStatusEnum.Uploaded : RecordStatusEnum.NotScanned;
                        if (remedicalRecord.IsUploadReport)
                        {
                            recordInfo.RecordStatus = RecordStatusEnum.Completed;
                        }
                        recordInfo.CustomDoctor = remedicalRecord.CustomDoctor;
                        recordInfo.CustomOrganzation = remedicalRecord.CustomOrganzation;
                        recordInfo.EquipmentSN = remedicalRecord.EquipmentSN;
                        if (string.IsNullOrWhiteSpace(recordInfo.DeviceExamId))
                        {
                            recordInfo.DeviceExamId = remedicalRecord.PatientInfo?.FirstOrDefault(x => x.Key == "ExamId")?.Value?.ToString() ?? string.Empty;
                        }
                        if (isUpdate)
                        {
                            if (syncType == SyncDBEnum.Migrate)
                            {
                                updateList.Add(recordInfo);
                            }
                            else
                            {
                                await _recordInfoDBRepository.UpdateOneForSyncDBAsync("Code", recordInfo.RecordCode, recordInfo, null);
                                await AddPushRemoteRecordAsync(recordInfo);
                            }
                        }
                        else
                        {
                            if (syncType == SyncDBEnum.Migrate)
                            {
                                recordInfo.IsImgQualityControlled = remedicalRecord.IsImgQualityControlled;
                                recordInfo.ImgScore = remedicalRecord.ImgScore;
                                recordInfo.IsReportQualityControlled = remedicalRecord.IsReportQualityControlled;
                                recordInfo.ReportScore = remedicalRecord.ReportScore;
                                insertList.Add(recordInfo);
                            }
                            else
                            {
                                await _recordInfoDBRepository.InsertOneForAsyncDBAsync(recordInfo, new Dictionary<string, string>(), "");
                            }
                        }
                    }
                    catch (Exception ex)
                    {
                        Logger.WriteLineWarn("foreach SyncBatchRemedicalRecordsDBAsync," + ex);
                    }
                }
                //迁移or同步
                if (syncType == SyncDBEnum.Migrate)
                {
                    if (updateList.Count > 0)
                    {
                        await _recordInfoDBRepository.UpdateAllWithoutOplogLogAsync(updateList);
                    }
                    if (insertList.Count > 0)
                    {
                        await _recordInfoDBRepository.InsertAllWithoutOplogLogAsync(insertList);
                    }
                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn("SyncBatchRemedicalRecordsDBAsync," + ex);
            }
            return true;
        }

        /// <summary>
        /// 分页查询增量远程图像集合
        /// </summary>
        /// <returns></returns>
        public async Task<PageResult<RemedicalInfoDTO>> FindRemedicalByUpdateTimeDBAsync(FindCollectionByTimeDBRequest request)
        {
            var startTime = request.StartTime;
            var pageIndex = request.PageIndex;
            var pageSize = request.PageSize;
            MongoCustomerFilter<MongoTable> customerFilter = new();//常规查询

            customerFilter.AppendGt("BaseData.UpdateTime", startTime);
            var filter = customerFilter.AndCombine();
            var pagedResult = await _remedicalDBRepository.GetPages(new DBPageRequest
            {
                CurrentPage = pageIndex,
                PageSize = pageSize,
                Filter = filter,
                Sort = Builders<MongoTable>.Sort.Ascending("BaseData.UpdateTime"),
            }, onlyNotDeleted: false);
            var result = new PageResult<RemedicalInfoDTO>();
            result.PageIndex = pagedResult.CurrentPage;
            result.PageSize = pagedResult.PageSize;
            result.TotalCount = pagedResult.DataCount;
            result.PageData = pagedResult.PageData?.MappingTo<List<RemedicalInfoDTO>>() ?? new List<RemedicalInfoDTO>();
            return result;
        }

        /// <summary>
        /// 保存远程图像记录
        /// </summary>
        /// <returns></returns>
        public async Task<bool> SaveRemedicalDBAsync(RemedicalInfoDTO request)
        {
            var code = request.RemedicalCode;
            var remedicalDO = request.MappingTo<RemedicalDO>();
            var entity = await _patientInfoDBRepository.FindOneAsync("Code", code);
            if (entity != null && !string.IsNullOrWhiteSpace(entity.Code))
            {
                await _remedicalDBRepository.UpdateOneAsync("Code", code, remedicalDO, "");
            }
            else
            {
                await _remedicalDBRepository.InsertOneAsync(remedicalDO, new Dictionary<string, string>(), "");
                if (remedicalDO.ImgList != null && remedicalDO.ImgList.Any())
                {
                    await UpdatePatientUrmLabelAsync(remedicalDO.RecordCode);
                }
            }
            return true;
        }

        /// <summary>
        /// 同步检查图像
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> SyncBatchUploadRemedicalDatasDBAsync(SyncBatchUploadRemedicalDatasRequest request)
        {
            try
            {
                SyncDBEnum syncType = request.SyncType;
                var list = new List<RemedicalDO>();
                var remedicalCodes = request?.SyncUploadRemedicalDataList?.Select(x => x.Code);
                Dictionary<string, List<string>> filterRemedical = new Dictionary<string, List<string>>();
                filterRemedical.Add("Code", remedicalCodes.ToList());
                var remedicalDatas = await _remedicalDBRepository.FindInAsync(filterRemedical);
                var remedicalDOList = new List<RemedicalDO>();
                foreach (var syncUploadRemedicalData in request.SyncUploadRemedicalDataList)
                {
                    try
                    {
                        var dataCode = syncUploadRemedicalData.Code;
                        bool isOldPlatformNeedAI = syncUploadRemedicalData.OldPlatformNeedAI;
                        bool isThyroid = false;
                        //判断数据是否已存在
                        var dataDO = remedicalDatas.FirstOrDefault(x => x.Code == dataCode);
                        if (dataDO != null && !string.IsNullOrWhiteSpace(dataDO.RemedicalCode))
                        {
                            if (syncType == SyncDBEnum.Synchronize)
                            {
                                if (dataDO.TerminalImages?.PreviewUrl != syncUploadRemedicalData.PreviewFileToken)
                                {
                                    var examDataUrl = syncUploadRemedicalData.FileToken;
                                    if (dataDO.TerminalImages == null)
                                    {
                                        dataDO.TerminalImages = new TerminalImageDO();
                                    }
                                    dataDO.TerminalImages.OriginImageUrl = examDataUrl;
                                    if (string.IsNullOrWhiteSpace(dataDO.TerminalImages.ImageUrl))
                                    {
                                        dataDO.TerminalImages.ImageUrl = syncUploadRemedicalData.ImageUrl;
                                    }
                                    if (string.IsNullOrWhiteSpace(dataDO.TerminalImages.PreviewUrl))
                                    {
                                        dataDO.TerminalImages.PreviewUrl = syncUploadRemedicalData.PreviewFileToken;
                                    }
                                    if (string.IsNullOrWhiteSpace(dataDO.TerminalImages.CoverImageUrl))
                                    {
                                        dataDO.TerminalImages.CoverImageUrl = syncUploadRemedicalData.CoverImageToken;
                                    }
                                    if (dataDO.ImageLocation == null)
                                    {
                                        dataDO.ImageLocation = syncUploadRemedicalData.ImageLocation.MappingTo<ImageLocationDO>();
                                    }
                                }
                                dataDO.OrganDiagnosisInfos = syncUploadRemedicalData.OrganDiagnosisInfos;
                                isThyroid = dataDO.OrganDiagnosisInfos.Contains(DiagnosisOrganEnum.Thyroid.ToString());
                                //AI中状态
                                if (isOldPlatformNeedAI || (isThyroid && dataDO.DiagnosisConclusion == DiagnosisConclusionEnum.NotRequired))
                                {
                                    var findDeviceDiagnosisRequest = new FindDeviceDiagnosisRequest { DeviceCode = dataDO.DeviceCode };
                                    var diagnosisOrgans = _remedicalService.FindDeviceDiagnosisAsync(findDeviceDiagnosisRequest);
                                    if (diagnosisOrgans.Any())
                                    {
                                        dataDO.DiagnosisConclusion = DiagnosisConclusionEnum.InProcess;
                                        Logger.WriteLineInfo($"SyncBatchUploadRemedicalDatasDBAsync record code:{dataDO.RecordCode}, diagnosis modules:{string.Join(";", diagnosisOrgans.Select(x => x.ToString()))}");
                                    }
                                    else
                                    {
                                        Logger.WriteLineInfo($"SyncBatchUploadRemedicalDatasDBAsync never config diagnosis modules, record code:{dataDO.RecordCode}");
                                    }
                                }
                                await _remedicalDBRepository.UpdateOneForSyncDBAsync("Code", dataDO.RemedicalCode, dataDO, null);
                                await AddPushRemedicalAsync(dataDO);
                            }
                        }
                        else
                        {
                            dataDO = new RemedicalDO();
                            dataDO.RemedicalCode = syncUploadRemedicalData.Code;
                            dataDO.CreateTime = syncUploadRemedicalData.CreateTime;
                            dataDO.DeviceCode = syncUploadRemedicalData.DeviceCode;
                            dataDO.RecordCode = syncUploadRemedicalData.ExamCode;
                            //dataDO.PatientScanType = scanTypeDesc;
                            dataDO.ApplicationCategory = syncUploadRemedicalData.ApplicationCategory;
                            dataDO.Application = syncUploadRemedicalData.Application;
                            dataDO.TerminalImages = new TerminalImageDO();
                            var sourceUrl = syncUploadRemedicalData.FileToken;
                            dataDO.TerminalImages.OriginImageUrl = sourceUrl;
                            dataDO.TerminalImages.ImageUrl = syncUploadRemedicalData.ImageUrl;
                            dataDO.TerminalImages.PreviewUrl = syncUploadRemedicalData.PreviewFileToken;
                            dataDO.TerminalImages.CoverImageUrl = syncUploadRemedicalData.CoverImageToken;
                            //dataDTO.TerminalImages.ImageSize = request.FileSize;
                            dataDO.FileDataType = syncUploadRemedicalData.FileDataType;
                            dataDO.ImageLocation = syncUploadRemedicalData.ImageLocation.MappingTo<ImageLocationDO>();
                            dataDO.OrganDiagnosisInfos = syncUploadRemedicalData.OrganDiagnosisInfos;
                            dataDO.OldPlatformNeedAI = isOldPlatformNeedAI;
                            isThyroid = dataDO.OrganDiagnosisInfos.Contains(DiagnosisOrganEnum.Thyroid.ToString());
                            if (syncType == SyncDBEnum.Migrate)
                            {
                                dataDO.QualityControlDatas = syncUploadRemedicalData.QualityControlDatas;
                                remedicalDOList.Add(dataDO);
                            }
                            else
                            {
                                if (!string.IsNullOrWhiteSpace(dataDO.Code) && (isThyroid || isOldPlatformNeedAI))
                                {
                                    var findDeviceDiagnosisRequest = new FindDeviceDiagnosisRequest { DeviceCode = dataDO.DeviceCode };
                                    var diagnosisOrgans = _remedicalService.FindDeviceDiagnosisAsync(findDeviceDiagnosisRequest);
                                    if (diagnosisOrgans.Any())
                                    {
                                        dataDO.DiagnosisConclusion = DiagnosisConclusionEnum.InProcess;
                                        Logger.WriteLineInfo($"SyncBatchUploadRemedicalDatasDBAsync record code:{dataDO.RecordCode}, diagnosis modules:{string.Join(";", diagnosisOrgans.Select(x => x.ToString()))}");
                                    }
                                    else
                                    {
                                        Logger.WriteLineInfo($"SyncBatchUploadRemedicalDatasDBAsync never config diagnosis modules, record code:{dataDO.RecordCode}");
                                    }
                                }
                                await _remedicalDBRepository.InsertOneForAsyncDBAsync(dataDO, new Dictionary<string, string>(), "");
                                await AddPushRemedicalAsync(dataDO);
                                //生成首帧大图
                                if (string.IsNullOrWhiteSpace(dataDO.TerminalImages.CoverImageUrl) && MatchCurrentStorage(dataDO.TerminalImages.OriginImageUrl))
                                {
                                    //没有首帧大图的数据
                                    //插入生成缩略图队列
                                    var addToVidProcessQueueRequest = new AddToVidProcessQueueRequest();
                                    addToVidProcessQueueRequest.OriginImageUrl = dataDO.TerminalImages.OriginImageUrl;
                                    addToVidProcessQueueRequest.RemedicalCode = dataDO.RemedicalCode;
                                    addToVidProcessQueueRequest.RecordCode = dataDO.RecordCode;
                                    addToVidProcessQueueRequest.IsNeedCoverImg = true;
                                    if (string.IsNullOrEmpty(dataDO.TerminalImages.PreviewUrl))
                                    {
                                        addToVidProcessQueueRequest.IsNeedThumbnailUrl = true;
                                    }
                                    //addToVidProcessQueueRequest.isNeedThumbnailUrl = true;
                                    _remedicalService.AddToVidProcessQueueAsync(addToVidProcessQueueRequest);
                                }

                            }
                        }
                        if (syncType == SyncDBEnum.Synchronize)
                        {
                            if (!string.IsNullOrWhiteSpace(dataDO.Code) && MatchCurrentStorage(dataDO.TerminalImages.OriginImageUrl))
                            {
                                if (isOldPlatformNeedAI)//需要优先做AI
                                {
                                    var addToRemedicalDiagnosisRequest = new AddToRemedicalDiagnosisRequest { RemedicalCode = dataDO.RemedicalCode };
                                    await _remedicalService.AddToRemedicalDiagnosisJumpQueueAsync(addToRemedicalDiagnosisRequest);
                                }
                                else if (isThyroid)//甲状腺AI处理
                                {
                                    var addToRemedicalDiagnosisRequest = new AddToRemedicalDiagnosisRequest { RemedicalCode = dataDO.RemedicalCode };
                                    await _remedicalService.AddToRemedicalDiagnosisAsync(addToRemedicalDiagnosisRequest);
                                }
                            }
                        }
                    }
                    catch (Exception ex)
                    {
                        Logger.WriteLineWarn("foreach SyncBatchUploadRemedicalDatasDBAsync," + ex);
                    }
                }
                //迁移or同步
                if (syncType == SyncDBEnum.Migrate)
                {
                    if (remedicalDOList.Count > 0)
                    {
                        await _remedicalDBRepository.InsertAllWithoutOplogLogAsync(remedicalDOList);
                    }
                }
                // 更新检查记录为已传图
                var recordCodes = request?.SyncUploadRemedicalDataList?.Select(x => x.ExamCode)?.Distinct()?.ToList() ?? new List<string>();
                foreach (var recordCode in recordCodes)
                {
                    var recordDO = await _recordInfoDBRepository.FindOneAsync("Code", recordCode);
                    if (recordDO.RecordStatus == RecordStatusEnum.NotScanned)
                    {
                        recordDO.RecordStatus = RecordStatusEnum.Uploaded;
                        await _recordInfoDBRepository.UpdateOneAsync("Code", recordDO.Code, recordDO, "");
                        await AddPushRemoteRecordAsync(recordDO);
                    }
                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"IRemedicalDBService-SyncBatchUploadRemedicalDatasDBAsync,{string.Join(',', request.SyncUploadRemedicalDataList.Select(x => x.Code))}," + ex);
            }
            return true;
        }


        private string GetPatientId(List<DataItemDTO> source, OrganizationPatientTypeEnum patientType)
        {
            if (patientType == OrganizationPatientTypeEnum.Animals)
            {
                return GetValue(source, "AnimalInfoID");
            }
            return GetValue(source, "ID");
        }

        private string GetValue(List<DataItemDTO> source, string key)
        {
            var dataItem = source.FirstOrDefault(x => x.Key == key);
            return dataItem?.Value ?? string.Empty;
        }

        /// <summary>
        /// 新增未登记病人(同步用)
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        private async Task<string> AddOrUpdatePatientForSync(List<PatientInfoDO> patientInfos, List<DataItemDTO> requestPatientInfo, string deviceCode, OrganizationDO orgInfo, DateTime? createTime, DateTime? updateTime)
        {
            var patientInfo = new PatientInfoDO();
            var name = "";
            var phone = "";
            var identityCard = "";
            var gender = 0;
            var devicePatientID = "";
            var insuranceCode = "";
            if (orgInfo.PatientType == OrganizationPatientTypeEnum.Animals)
            {
                name = GetValue(requestPatientInfo, "AnimalInfoName");
                phone = GetValue(requestPatientInfo, "AnimalInfoPhone");
                identityCard = GetValue(requestPatientInfo, "AnimalInfoIdentityCard");
                devicePatientID = GetValue(requestPatientInfo, "AnimalInfoID");
                var animalGender = GetValue(requestPatientInfo, "AnimalInfoSex");
                if (!string.IsNullOrWhiteSpace(animalGender))
                {
                    var isSexEnumValid = Enum.IsDefined(typeof(AnimalGenderEnum), animalGender);
                    if (!isSexEnumValid)
                    {
                        ThrowCustomerException(CustomerRpcCode.SexError, "Sex is error");
                    }
                    gender = Enum.Parse(typeof(AnimalGenderEnum), animalGender).GetHashCode();
                }
            }
            else
            {
                name = GetValue(requestPatientInfo, "Name");
                phone = GetValue(requestPatientInfo, "Phone");
                identityCard = GetValue(requestPatientInfo, "IdentityCard");
                insuranceCode = GetValue(requestPatientInfo, "InsuranceCode");
                devicePatientID = GetValue(requestPatientInfo, "ID");
                var sex = GetValue(requestPatientInfo, "Sex");
                if (!string.IsNullOrWhiteSpace(sex))
                {
                    var isSexEnumValid = Enum.IsDefined(typeof(PatientGenderEnum), sex);
                    if (!isSexEnumValid)
                    {
                        ThrowCustomerException(CustomerRpcCode.SexError, "Sex is error");
                    }
                    patientInfo.Gender = Enum.Parse(typeof(PatientGenderEnum), sex).GetHashCode();
                }
            }
            // if (string.IsNullOrWhiteSpace(devicePatientID))
            // {
            //     ThrowCustomerException(CustomerRpcCode.IDIsNullOrWhiteSpace, "IDIsNullOrWhiteSpace");
            // }
            //已有此病人
            var patientOld = patientInfos.FirstOrDefault(x => x.Name == name && x.DevicePatientIDs.Contains(devicePatientID) && x.RootOrganizationCode == orgInfo.RootCode);
            if (patientOld != null && !string.IsNullOrWhiteSpace(patientOld.PatientCode))
            {
                return patientOld.PatientCode;
            }
            //新病人
            patientInfo.Name = name;
            patientInfo.Phone = phone;
            patientInfo.IdentityCard = identityCard;
            patientInfo.InsuranceCode = insuranceCode;
            patientInfo.Gender = gender;
            patientInfo.DevicePatientIDs = new List<string> { devicePatientID };
            patientInfo.PatientData = requestPatientInfo;
            patientInfo.IsValid = true;
            patientInfo.OrganizationCode = orgInfo.OrganizationCode;//设备所在机构
            patientInfo.RootOrganizationCode = orgInfo.RootCode;//医院
            patientInfo.CreatorCode = "";//设备创建的没有创建人
            patientInfo.DeviceCode = deviceCode;
            patientInfo.PatientType = orgInfo.PatientType.ToString();
            if (createTime != null)
            {
                patientInfo.CreateTime = (DateTime)createTime;
            }
            if (updateTime != null)
            {
                patientInfo.UpdateTime = (DateTime)updateTime;
            }
            patientInfo.Code = await _patientInfoDBRepository.InsertOneForAsyncDBAsync(patientInfo, new Dictionary<string, string>(), "");
            patientInfos.Add(patientInfo);
            return patientInfo.Code;
        }


        /// <summary>
        /// 数据加密
        /// </summary>
        /// <param name="patientData"></param>
        private void SetEncryptedData(List<DataItemDTO> patientData)
        {
            if (patientData.FirstOrDefault(x => x.Key == "AnimalInfoName") != null && !string.IsNullOrWhiteSpace(patientData.FirstOrDefault(x => x.Key == "AnimalInfoName").Value))
            {
                patientData.FirstOrDefault(x => x.Key == "AnimalInfoName").Value = CryptionHelper.EncryptOneByOne(patientData.FirstOrDefault(x => x.Key == "AnimalInfoName").Value);
            }
            if (patientData.FirstOrDefault(x => x.Key == "AnimalInfoPhone") != null && !string.IsNullOrWhiteSpace(patientData.FirstOrDefault(x => x.Key == "AnimalInfoPhone").Value))
            {
                patientData.FirstOrDefault(x => x.Key == "AnimalInfoPhone").Value = CryptionHelper.EncryptOneByOne(patientData.FirstOrDefault(x => x.Key == "AnimalInfoPhone").Value);
            }
            if (patientData.FirstOrDefault(x => x.Key == "AnimalInfoIdentityCard") != null && !string.IsNullOrWhiteSpace(patientData.FirstOrDefault(x => x.Key == "AnimalInfoIdentityCard").Value))
            {
                patientData.FirstOrDefault(x => x.Key == "AnimalInfoIdentityCard").Value = CryptionHelper.EncryptOneByOne(patientData.FirstOrDefault(x => x.Key == "AnimalInfoIdentityCard").Value);
            }
            if (patientData.FirstOrDefault(x => x.Key == "Name") != null && !string.IsNullOrWhiteSpace(patientData.FirstOrDefault(x => x.Key == "Name").Value))
            {
                patientData.FirstOrDefault(x => x.Key == "Name").Value = CryptionHelper.EncryptOneByOne(patientData.FirstOrDefault(x => x.Key == "Name").Value);
            }
            if (patientData.FirstOrDefault(x => x.Key == "Phone") != null && !string.IsNullOrWhiteSpace(patientData.FirstOrDefault(x => x.Key == "Phone").Value))
            {
                patientData.FirstOrDefault(x => x.Key == "Phone").Value = CryptionHelper.EncryptOneByOne(patientData.FirstOrDefault(x => x.Key == "Phone").Value);
            }
            if (patientData.FirstOrDefault(x => x.Key == "IdentityCard") != null && !string.IsNullOrWhiteSpace(patientData.FirstOrDefault(x => x.Key == "IdentityCard").Value))
            {
                patientData.FirstOrDefault(x => x.Key == "IdentityCard").Value = CryptionHelper.EncryptOneByOne(patientData.FirstOrDefault(x => x.Key == "IdentityCard").Value);
            }
            if (patientData.FirstOrDefault(x => x.Key == "InsuranceCode") != null && !string.IsNullOrWhiteSpace(patientData.FirstOrDefault(x => x.Key == "InsuranceCode").Value))
            {
                patientData.FirstOrDefault(x => x.Key == "InsuranceCode").Value = CryptionHelper.EncryptOneByOne(patientData.FirstOrDefault(x => x.Key == "InsuranceCode").Value);
            }
        }
        /// <summary>
        /// 查询诊断图像增量数据
        /// </summary>
        /// <param name="startTime"></param>
        /// <param name="endTime"></param>
        /// <returns></returns>
        public async Task<IList<FileDiagnosisResultDTO>> FindRemedicalsDataCenterAsync(DateTime startTime, DateTime endTime)
        {
            MongoCustomerFilter<MongoTable> customerFilter = new();
            customerFilter.AppendGte("BaseData.CreateTime", startTime);
            customerFilter.AppendGte("BaseData.UpdateTime", startTime);
            customerFilter.AppendLte("BaseData.CreateTime", endTime);
            customerFilter.AppendLte("BaseData.UpdateTime", endTime);
            var remedicalResult = await _remedicalDBRepository.FindAllAsync(customerFilter.AndCombine());
            var result = new List<FileDiagnosisResultDTO>();
            if (remedicalResult.Count > 0)
            {
                result = remedicalResult.MappingTo<List<FileDiagnosisResultDTO>>();
            }
            return result;
        }

        /// <summary>
        /// 设备端根据设备检查id验证检查是否存在接口
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>是否存在：true-存在； false-不存在</returns>
        public async Task<bool> DeviceValidateExamIsExistDBAsync(DeviceValidateExamResultRequest request)
        {
            return false;
        }

        /// <summary>
        /// 查询科研项目关联所有图像
        /// </summary>
        /// <param name="recordCode"></param>
        /// <returns></returns>
        async Task<List<RemedicalInfoDTO>> GetRemedicalsByProjectCodeAsync(string projectCode)
        {
            using (new Performance(_performanceThreshold, "{0} GetRemedicalsByProjectCodeAsync execute", $"{this.GetType().Name}"))
            {
                var dic = new Dictionary<string, string>();
                dic.Add("ProjectCode", projectCode);
                var entities = await _remedicalDBRepository.FindListAsync(dic);
                return entities.MappingTo<List<RemedicalInfoDTO>>();
            }
        }

        /// <summary>
        /// 查询科研项目关联所有图像
        /// </summary>
        /// <param name="recordCode"></param>
        /// <returns></returns>
        async Task<List<RemedicalDO>> GetRemedicalsByProjectCodesAsync(List<string> projectCodes)
        {
            using (new Performance(_performanceThreshold, "{0} GetRemedicalsByProjectCodesAsync execute", $"{this.GetType().Name}"))
            {
                var dic = new Dictionary<string, List<string>>();
                dic.Add("ProjectCode", projectCodes);
                var entities = await _remedicalDBRepository.FindInAsync(dic);
                return entities;
            }
        }


        /// <summary>
        /// 申请小动物测量包试用接口
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>是否存在：true-存在； false-不存在</returns>
        public async Task<ApplyChargeMeasureProbationResult> ApplyChargeMeasureProbationDBAsync(ApplyChargeMeasureProbationRequest request)
        {
            //根据手机号，查询是否有试用过,查询账号，和手机号
            if (!string.IsNullOrWhiteSpace(request.Phone))
            {
                MongoCustomerFilter<MongoTable> customerFilter = new();
                List<FilterDefinition<MongoDataItem>> queryItemsFilterList = new();
                var items = new List<string> { "UserName", "Phone" };
                foreach (var item in items)
                {
                    var queryItemsFilter = new MongoCustomerFilter<MongoDataItem>();
                    queryItemsFilter.AppendEq((item => item.Key, item));
                    queryItemsFilter.AppendEq((item => item.Value, request.Phone));
                    queryItemsFilterList.Add(queryItemsFilter.AndCombine());
                }
                MongoCustomerFilter<MongoDataItem> queryAll = new();
                customerFilter.SubFilter(item => item.QueryData.QueryItems, queryAll.OrCombine(queryItemsFilterList.ToArray()));
                var userList = await _userDBRepository.FindAllAsync(customerFilter.AndCombine());
                {
                    bool isProbationed = false;
                    foreach (var item in userList)
                    {
                        var userResourcePackageInfo = item.BindChargeResourcePackages.FirstOrDefault(c => c.BussinessCode == "VET") ?? new BindChargeResourcePackageDO();
                        if (!string.IsNullOrEmpty(userResourcePackageInfo?.ResourcePackageCode))
                        {
                            var packageInfo = await GetChargeResourcePackageByCodeDBAsync(userResourcePackageInfo.ResourcePackageCode);
                            if (packageInfo.ResourcePackageType == ChargeResourcePackageTypeEnum.Probation)
                            {
                                isProbationed = true;
                                break;
                            }
                        }
                    }
                    if (isProbationed)
                    {
                        //加错误码，已申请过 todo
                        ThrowCustomerException(CustomerRpcCode.ProbationMeasureResourcePackageExpired, "RecordCode cannot be empty");
                    }
                    //绑定注册，token获取当前账号，绑定到手机号上，然后更新状态
                }
            }
            return null;
        }

        /// <summary>
        /// 用户自定义保存体标项请求
        /// </summary>
        /// <param name="request">用户自定义保存体标项请求实体</param>
        /// <returns></returns>
        public async Task<bool> SaveUserDefinedBodyPatternDBAsync(SaveUserDefinedBodyPatternDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} SaveUserDefinedBodyPatternDBAsync execute", $"{this.GetType().Name}"))
            {
                var res = false;
                var measureInfo = await _measureInfoRepository.FindOneAsync("UserId", request.UserCode);
                if (measureInfo != null)
                {
                    measureInfo.BodyPatterns = request.BodyPatterns.MappingTo<List<UserDefinedBodyPatternDataDO>>();
                    res = await _measureInfoRepository.UpdateOneAsync("Code", measureInfo.Code, measureInfo, string.Empty, true);
                }
                else
                {
                    measureInfo = new UserDefinedMeasureInfoDO()
                    {
                        UserId = request.UserCode,
                        BodyPatterns = request.BodyPatterns.MappingTo<List<UserDefinedBodyPatternDataDO>>()
                    };
                    var identity = await _measureInfoRepository.InsertOneAsync(measureInfo, new Dictionary<string, string>(), "");
                    if (!string.IsNullOrEmpty(identity))
                    {
                        res = true;
                    }
                }
                return res;
            }
        }

        // /// <summary>
        // /// 重置用户保存体标项请求
        // /// </summary>
        // /// <param name="request">重置用户保存体标项请求</param>
        // /// <returns></returns>
        // public async Task<bool> ResetUserBodyPatternsDBAsync(SaveUserDefinedBodyPatternDBRequest request)
        // {
        //     return false;
        // }
    }
}