using System.Collections.Generic;
using System.Threading.Tasks;
using WingServerCommon.Service;
using WingServerCommon.Mapper;
using System.Linq;
using System;
using WingMongoDBModule.DO;
using WingInterfaceLibrary.DB.Request;
using WingMongoDBModule.Mongodb.Driver;
using WingMongoDBModule.Entity;
using MongoDB.Driver;
using WingInterfaceLibrary.Interface.DBInterface;
using WingInterfaceLibrary.Enum;
using WingInterfaceLibrary.DTO.AdminStatistic;
using WingInterfaceLibrary.DTO.Dictionary;
using WingInterfaceLibrary.DTO.Record;
using WingServerCommon.Utilities;

namespace WingMongoDBModule.Service
{
    public partial class DatabaseService : JsonRpcService, IRemedicalMeasuredInfoDBService
    {
        /// <summary>
        /// 查询客户端测量图像数据详情
        /// </summary>
        /// <returns></returns>
        public async Task<RemedicalMeasuredInfoDTO> GetRemedicalMeasuredInfoByCodeAsync(GetRemedicalMeasuredInfoByCodeDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetRemedicalMeasuredInfoByCodeAsync execute", $"{this.GetType().Name}"))
            {
                var entity = await _remedicalMeasuredInfoDBRepository.FindOneAsync("Code", request.RemedicalMeasuredInfoCode);
                return entity.MappingTo<RemedicalMeasuredInfoDTO>();
            }
        }

        /// <summary>
        /// 查询检查关联的客户端测量图像数据集合
        /// </summary>
        /// <returns></returns>
        public async Task<List<RemedicalMeasuredInfoDTO>> GetRemedicalMeasuredInfosByRemedicalCodeAsync(GetRemedicalMeasuredInfosByRemedicalCodeDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetRemedicalMeasuredInfosByRemedicalCodeAsync execute", $"{this.GetType().Name}"))
            {
                var dic = new Dictionary<string, string>();
                if (!string.IsNullOrWhiteSpace(request.UserCode))
                {
                    dic.Add("UserCode", request.UserCode);
                }
                dic.Add("RemedicalCode", request.RemedicalCode);
                var entities = await _remedicalMeasuredInfoDBRepository.FindListAsync(dic);
                return entities.MappingTo<List<RemedicalMeasuredInfoDTO>>();
            }
        }

        /// <summary>
        /// 查询检查关联的客户端测量图像数据集合
        /// </summary>
        /// <returns></returns>
        public async Task<List<RemedicalMeasuredInfoDTO>> GetRemedicalMeasuredInfosByRecordCodeAsync(GetRemedicalMeasuredInfosByRecordCodeDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetRemedicalMeasuredInfosByRecordCodeAsync execute", $"{this.GetType().Name}"))
            {
                var dic = new Dictionary<string, string>();
                if (!string.IsNullOrWhiteSpace(request.UserCode))
                {
                    dic.Add("UserCode", request.UserCode);
                }
                dic.Add("RecordCode", request.RecordCode);
                var entities = await _remedicalMeasuredInfoDBRepository.FindListAsync(dic);
                return entities.MappingTo<List<RemedicalMeasuredInfoDTO>>();
            }
        }

        /// <summary>
        /// 查询检查关联的客户端测量图像数据集合
        /// </summary>
        /// <returns></returns>
        public async Task<long> GetRemedicalMeasuredCountByConditionAsync(GetRemedicalMeasuredInfosByConditionDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetRemedicalMeasuredCountByConditionAsync execute", $"{this.GetType().Name}"))
            {
                long count = 0; ;
                if (request.RemedicalMeasuredInfoCodes?.Count > 0)
                {
                    var customerFilter = new MongoCustomerFilter<MongoTable>();
                    var mongoDataItemFilter = new MongoCustomerFilter<MongoDataItem>();
                    mongoDataItemFilter.AppendEq((item => item.Key, "Code"));
                    mongoDataItemFilter.AppendIn(item => item.Value, request.RemedicalMeasuredInfoCodes.ToList());
                    customerFilter.SubFilter(item => item.QueryData.QueryItems, mongoDataItemFilter.AndCombine());
                    count = await _remedicalMeasuredInfoDBRepository.CountDocumentsAsync(customerFilter.AndCombine());
                }
                else if (!string.IsNullOrEmpty(request.RecordCode))
                {
                    var customerFilter = new MongoCustomerFilter<MongoTable>();
                    var mongoDataItemFilter = new MongoCustomerFilter<MongoDataItem>();
                    mongoDataItemFilter.AppendEq((item => item.Key, "Code"));
                    mongoDataItemFilter.AppendEq((item => item.Key, request.RecordCode));
                    customerFilter.SubFilter(item => item.QueryData.QueryItems, mongoDataItemFilter.AndCombine());
                    count = await _remedicalMeasuredInfoDBRepository.CountDocumentsAsync(customerFilter.AndCombine());
                }
                return count;
            }
        }

        /// <summary>
        /// 查询检查关联的客户端测量图像数据集合
        /// </summary>
        /// <returns></returns>
        public async Task<List<RemedicalMeasuredInfoDTO>> GetRemedicalMeasuredInfosByConditionAsync(GetRemedicalMeasuredInfosByConditionDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetRemedicalMeasuredInfosByConditionAsync execute", $"{this.GetType().Name}"))
            {
                var list = new List<RemedicalMeasuredInfoDO>();
                if (request.RemedicalMeasuredInfoCodes?.Count > 0)
                {
                    var dic = new Dictionary<string, List<string>>();
                    dic.Add("Code", request.RemedicalMeasuredInfoCodes.Distinct().ToList());
                    list = await _remedicalMeasuredInfoDBRepository.FindInAsync(dic);
                }
                else if (!string.IsNullOrEmpty(request.RecordCode))
                {
                    var dic = new Dictionary<string, string>();
                    dic.Add("RecordCode", request.RecordCode);
                    list = await _remedicalMeasuredInfoDBRepository.FindListAsync(dic);
                }
                if (list?.Count > 0)
                {
                    list = list.OrderBy(c => c.CreateTime).ToList();
                }
                return list.MappingTo<List<RemedicalMeasuredInfoDTO>>();
            }
        }

        /// <summary>
        /// 添加客户端测量图像数据
        /// </summary>
        /// <returns></returns>
        public async Task<string> AddRemedicalMeasuredInfoAsync(AddRemedicalMeasuredInfoDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} AddRemedicalMeasuredInfoAsync execute", $"{this.GetType().Name}"))
            {
                if (string.IsNullOrWhiteSpace(request.RemedicalCode))
                {
                    return string.Empty;
                }
                var remedicals = await GetRemedicalWithCodesAsync(new List<string> { request.RemedicalCode });
                if (remedicals?.Any() != true)
                {
                    return string.Empty;
                }
                var entity = new RemedicalMeasuredInfoDO();
                entity.UserCode = request.UserCode;
                entity.RecordCode = remedicals.FirstOrDefault().RecordCode;
                entity.RemedicalCode = request.RemedicalCode;
                entity.FrameIndex = request.FrameIndex;
                entity.MeasuredFileToken = request.MeasuredFileToken;
                entity.PreviewFileToken = request.PreviewFileToken;
                entity.MeasuredData = request.MeasuredData;
                entity.Model = request.Model;
                return await _remedicalMeasuredInfoDBRepository.InsertOneAsync(entity, new Dictionary<string, string>(), string.Empty);
            }
        }

        /// <summary>
        /// 批量添加客户端测量图像数据
        /// </summary>
        /// <returns></returns>
        public async Task<bool> AddRemedicalMeasuredInfoByBathAsync(AddRemedicalMeasuredInfoByBathDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} AddRemedicalMeasuredInfoByBathAsync execute", $"{this.GetType().Name}"))
            {
                var remedicalCodes = request.RemedicalMeasuredInfos.Select(x => x.RemedicalCode).ToList();
                var remedicals = await GetRemedicalWithCodesAsync(remedicalCodes);
                if (remedicals?.Any() != true)
                {
                    return false;
                }
                var insertDos = new List<RemedicalMeasuredInfoDO>();
                foreach (var info in request.RemedicalMeasuredInfos)
                {
                    var remedical = remedicals.FirstOrDefault(x => x.RemedicalCode == info.RemedicalCode);
                    var entity = new RemedicalMeasuredInfoDO();
                    entity.UserCode = info.UserCode;
                    entity.RecordCode = remedical?.RecordCode ?? string.Empty;
                    entity.RemedicalCode = info.RemedicalCode;
                    entity.FrameIndex = info.FrameIndex;
                    entity.MeasuredFileToken = info.MeasuredFileToken;
                    entity.PreviewFileToken = info.PreviewFileToken;
                    entity.MeasuredData = info.MeasuredData;
                    entity.Model = info.Model;
                    entity.RemedicalOriginalCode = remedical?.RemedicalCode ?? string.Empty;
                    entity.UrmOriginalCode = remedical?.UrmOriginalCode ?? string.Empty;
                    
                    insertDos.Add(entity);
                }
                if (insertDos.Any())
                {
                    var req = insertDos.Select(f => (f, new Dictionary<string, string>(), string.Empty)).ToList();
                    await _remedicalMeasuredInfoDBRepository.InsertAllAsync(req);
                    return true;
                }
                return false;
            }
        }

        /// <summary>
        /// 修改客户端测量图像数据
        /// </summary>
        /// <returns></returns>
        public async Task<bool> UpdateRemedicalMeasuredInfoAsync(UpdateRemedicalMeasuredInfoDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateRemedicalMeasuredInfoAsync execute", $"{this.GetType().Name}"))
            {
                var code = request.RemedicalMeasuredInfoCode;
                var entity = await _remedicalMeasuredInfoDBRepository.FindOneAsync("Code", code);
                if (entity != null && !string.IsNullOrWhiteSpace(entity.Code))
                {
                    entity.UserCode = request.UserCode;
                    //entity.RecordCode = request.RecordCode;
                    entity.RemedicalCode = request.RemedicalCode;
                    entity.FrameIndex = request.FrameIndex;
                    entity.MeasuredFileToken = request.MeasuredFileToken;
                    entity.PreviewFileToken = request.PreviewFileToken;
                    entity.MeasuredData = request.MeasuredData;
                    return await _remedicalMeasuredInfoDBRepository.UpdateOneAsync("Code", entity.Code, entity, string.Empty);
                }
                return false;
            }
        }

        private async Task<List<RemedicalDO>> GetRemedicalWithCodesAsync(List<string> codes)
        {
            if (codes?.Any() == true)
            {
                var dic = new Dictionary<string, List<string>>();
                dic.Add("Code", codes.Distinct().ToList());
                return await _remedicalDBRepository.FindInAsync(dic);
            }
            return new List<RemedicalDO>();
        }

        /// <summary>
        /// 查询图像关联的客户端测量图像数据集合
        /// </summary>
        /// <returns></returns>
        async Task<List<RemedicalMeasuredInfoDTO>> GetRemedicalMeasuredInfosByRemedicalCodesAsync(IList<string> remedicalCodes)
        {
            using (new Performance(_performanceThreshold, "{0} GetRemedicalMeasuredInfosByRemedicalCodesAsync execute", $"{this.GetType().Name}"))
            {
                if (remedicalCodes == null || remedicalCodes.Count == 0)
                {
                    return new List<RemedicalMeasuredInfoDTO>();
                }
                var customerFilter = new MongoCustomerFilter<MongoTable>();
                var mongoDataItemFilter = new MongoCustomerFilter<MongoDataItem>();
                mongoDataItemFilter.AppendEq((item => item.Key, "RemedicalCode"));
                mongoDataItemFilter.AppendIn(item => item.Value, remedicalCodes.ToList());
                customerFilter.SubFilter(item => item.QueryData.QueryItems, mongoDataItemFilter.AndCombine());
                var entities = await _remedicalMeasuredInfoDBRepository.FindAllAsync(customerFilter.AndCombine());
                return entities.MappingTo<List<RemedicalMeasuredInfoDTO>>();
            }
        }

    }
}