using System.Collections.Generic;
using System.Threading.Tasks;
using WingServerCommon.Service;
using WingInterfaceLibrary.Interface.DBInterface;
using WingServerCommon.Mapper;
using WingMongoDBModule.DO;
using WingInterfaceLibrary.DB.Request;
using WingMongoDBModule.Mongodb.Driver;
using WingMongoDBModule.Entity;
using WingInterfaceLibrary.Result;
using WingMongoDBModule.Request;
using WingInterfaceLibrary.DTO.Report;
using WingInterfaceLibrary.Enum;
using System;
using System.Linq;
using MongoDB.Driver;
using WingServerCommon.Utilities;
using WingInterfaceLibrary.Request.Management;
using WingServerCommon.Interfaces.Cache;

namespace WingMongoDBModule.Service
{
    public partial class DatabaseService : JsonRpcService, IReportPosterDBService
    {
        /// <summary>
        /// 查询报告转发厂商配置详情
        /// </summary>
        /// <returns></returns>
        public async Task<ReportPosterDTO> GetReportPosterByCodeAsync(GetReportPosterByCodeDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetReportPosterByCodeAsync execute", $"{this.GetType().Name}"))
            {
                var entity = await _reportPosterDBRepository.FindOneAsync("Code", request.ReportPosterCode);
                return entity.MappingTo<ReportPosterDTO>();
            }
        }

        /// <summary>
        /// Get report post info list from 
        /// </summary>
        /// <param name="codes">The specified codes</param>
        /// <returns></returns>
        public async Task<List<ReportPosterDTO>> GetReportPostersWithCodesAsync(List<string> codes)
        {
            using (new Performance(_performanceThreshold, "{0} GetReportPostersWithCodesAsync execute", $"{this.GetType().Name}"))
            {
                var dic = new Dictionary<string, List<string>>();
                dic.Add("Code", codes);
                var allDatas = await _reportPosterDBRepository.FindInAsync(dic);
                return allDatas.MappingTo<List<ReportPosterDTO>>();
            }
        }

        /// <summary>
        /// 查询所有报告转发厂商配置详情
        /// </summary>
        /// <returns></returns>
        public async Task<List<ReportPosterDTO>> GetReportPosterAllAsync()
        {
            using (new Performance(_performanceThreshold, "{0} GetReportPosterAllAsync execute", $"{this.GetType().Name}"))
            {
                var entity = await _reportPosterDBRepository.FindAllAsync();
                return entity.MappingTo<List<ReportPosterDTO>>();
            }
        }

        /// <summary>
        /// 分页查询报告转发厂商配置详情
        /// </summary>
        /// <returns></returns>
        public async Task<PageResult<ReportPosterDTO>> GetReportPosterByPageAsync(GetReportPosterByPageDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetReportPosterByPageAsync execute", $"{this.GetType().Name}"))
            {
                MongoCustomerFilter<MongoTable> mongoCustomerFilter = new();
                if (!string.IsNullOrWhiteSpace(request.Keyword))
                {
                    mongoCustomerFilter.AppendRegexForKey("BaseData.ReceiverName", request.Keyword, true);
                }
                var filter = mongoCustomerFilter.AndCombine();
                var dbPage = new DBPageRequest();
                dbPage.CurrentPage = request.CurrentPage;
                dbPage.PageSize = request.PageSize;
                dbPage.Filter = filter;
                var pageResult = await _reportPosterDBRepository.GetPages(dbPage);
                return PageResultConvert<ReportPosterDO, ReportPosterDTO>(pageResult);
            }
        }

        /// <summary>
        /// 查询关联设备Codes
        /// </summary>
        /// <returns></returns>
        public async Task<List<string>> GetRelatedDeviceCodesAsync(GetRelatedDeviceCodesDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetRelatedDeviceCodesAsync execute", $"{this.GetType().Name}"))
            {
                var relatedDevices = await GetRelatedDeviceAsync(request.ReportPosterCode);
                return relatedDevices.Select(x => x.Code).ToList();
            }
        }

        private async Task<List<DeviceInfoDO>> GetRelatedDeviceAsync(string reportPosterCode)
        {
            if (!string.IsNullOrWhiteSpace(reportPosterCode))
            {
                MongoCustomerFilter<MongoTable> customerFilter = new();
                customerFilter.AppendArrayEq("BaseData.ReportPosterCodes", reportPosterCode);
                var relatedDevices = await _deviceInfoDBRepository.FindAllAsync(customerFilter.AndCombine(), new FindOptions<MongoTable> { NoCursorTimeout = true });
                return relatedDevices;
            }
            return new List<DeviceInfoDO>();
        }

        /// <summary>
        /// 添加报告转发厂商配置详情
        /// </summary>
        /// <returns></returns>
        public async Task<string> AddReportPosterAsync(AddReportPosterDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} AddReportPosterAsync execute", $"{this.GetType().Name}"))
            {
                var entity = new ReportPosterDO();
                entity.ReceiverName = request.ReceiverName;
                entity.TargetUrl = request.TargetUrl;
                entity.Contact = request.Contact;
                entity.Phone = request.Phone;
                entity.PosterType = request.PosterType;
                entity.ReportFormat = request.ReportFormat;
                entity.PostTimeout = request.PostTimeout;
                entity.Template = request.Template;
                return await _reportPosterDBRepository.InsertOneAsync(entity, new Dictionary<string, string>(), string.Empty);
            }
        }

        /// <summary>
        /// 修改报告转发厂商配置详情
        /// </summary>
        /// <returns></returns>
        public async Task<bool> UpdateReportPosterAsync(UpdateReportPosterDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateReportPosterAsync execute", $"{this.GetType().Name}"))
            {
                var entity = await _reportPosterDBRepository.FindOneAsync("Code", request.ReportPosterCode);
                if (entity != null && !string.IsNullOrWhiteSpace(entity.Code))
                {
                    entity.ReceiverName = request.ReceiverName;
                    entity.TargetUrl = request.TargetUrl;
                    entity.Contact = request.Contact;
                    entity.Phone = request.Phone;
                    entity.PosterType = request.PosterType;
                    entity.ReportFormat = request.ReportFormat;
                    entity.PostTimeout = request.PostTimeout;
                    entity.Template = request.Template;
                    return await _reportPosterDBRepository.UpdateOneAsync("Code", entity.Code, entity, string.Empty);
                }
                return false;
            }
        }

        /// <summary>
        /// 删除报告转发厂商配置详情
        /// </summary>
        /// <returns></returns>
        public async Task<bool> DeleteReportPosterAsync(DeleteReportPosterDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} DeleteReportPosterAsync execute", $"{this.GetType().Name}"))
            {
                var reportPosterCode = request.ReportPosterCode;
                var result = await _reportPosterDBRepository.DeleteOneAsync("Code", reportPosterCode);
                if (result)
                {
                    var relatedDevices = await GetRelatedDeviceAsync(reportPosterCode);
                    foreach (var deviceInfo in relatedDevices)
                    {
                        var reportPosterCodes = deviceInfo.ReportPosterCodes;
                        reportPosterCodes.Remove(reportPosterCode);
                        deviceInfo.ReportPosterCodes = reportPosterCodes;
                        await _deviceInfoDBRepository.UpdateOneAsync("Code", deviceInfo.Code, deviceInfo, string.Empty);
                    }
                }
                return result;
            }
        }

        /// <summary>
        /// 设置报告转发厂商关联设备集合 
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<List<string>> UpdateReportPosterDevicesAsync(UpdateReportPosterDevicesDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateReportPosterDevicesAsync execute", $"{this.GetType().Name}"))
            {
                var updateDeviceCodes = new List<string>();
                var entity = await _reportPosterDBRepository.FindOneAsync("Code", request.ReportPosterCode);
                if (entity != null && !string.IsNullOrWhiteSpace(entity.Code))
                {
                    var relatedDevices = await GetRelatedDeviceAsync(entity.Code);

                    var relatedDeviceCodes = relatedDevices.Select(x => x.Code);
                    var insertDeviceCodes = request.DeviceIds.Except(relatedDeviceCodes);

                    foreach (var deviceInfo in relatedDevices)
                    {
                        if (!request.DeviceIds.Contains(deviceInfo.Code))
                        {
                            var reportPosterCodes = deviceInfo.ReportPosterCodes;
                            reportPosterCodes.Remove(entity.Code);
                            deviceInfo.ReportPosterCodes = reportPosterCodes;
                            await _deviceInfoDBRepository.UpdateOneAsync("Code", deviceInfo.Code, deviceInfo, string.Empty);
                            updateDeviceCodes.Add(deviceInfo.Code);
                        }
                    }
                    if (insertDeviceCodes?.Any() == true)
                    {
                        var dic = new Dictionary<string, List<string>>();
                        dic.Add("Code", insertDeviceCodes.ToList());
                        var insertDevices = await _deviceInfoDBRepository.FindInAsync(dic);
                        foreach (var deviceInfo in insertDevices)
                        {
                            var reportPosterCodes = deviceInfo.ReportPosterCodes;
                            reportPosterCodes.Add(entity.Code);
                            deviceInfo.ReportPosterCodes = reportPosterCodes;
                            await _deviceInfoDBRepository.UpdateOneAsync("Code", deviceInfo.Code, deviceInfo, string.Empty);
                            updateDeviceCodes.Add(deviceInfo.Code);
                        }
                    }

                }
                return updateDeviceCodes;
            }
        }

        /// <summary>
        ///  分页查询报告转发记录请求
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageResult<ReportPushRecordDTO>> GetReportPushRecordsByPageAsync(GetReportPushRecordsByPageDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetReportPushRecordsByPageAsync execute", $"{this.GetType().Name}"))
            {
                MongoCustomerFilter<MongoTable> mongoCustomerFilter = new();
                List<(string, string, bool, bool)> regexFilters = new();
                List<(string, string)> eqFilters = new();
                if (!string.IsNullOrWhiteSpace(request.Keyword))
                {
                    regexFilters.Add(("PatientName", CryptionHelper.Encrypt(request.Keyword), true, false));
                    regexFilters.Add(("PatientName", request.Keyword, true, false));
                    regexFilters.Add(("PatientId", request.Keyword, true, false));
                    regexFilters.Add(("PoseterConfigName", request.Keyword, true, false));
                }
                var state = request.QueryState;
                if (state != PushDataStateEnum.All)
                {
                    eqFilters.Add(("State", Enum.GetName(state)));
                }
                if (regexFilters.Count > 0)
                {
                    mongoCustomerFilter.AppendQueryItemRegexFilter(MongoCombineTypeEnum.Or, regexFilters.ToArray());
                }
                if (eqFilters.Count > 0)
                {
                    mongoCustomerFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, eqFilters.ToArray());
                }
                var filter = mongoCustomerFilter.AndCombine();
                var dbPage = new DBPageRequest();
                dbPage.CurrentPage = request.CurrentPage;
                dbPage.PageSize = request.PageSize;
                dbPage.Filter = filter;
                var pageResult = await _reportPushRecordsDBRepository.GetPages(dbPage);
                return PageResultConvert<ReportPushRecordDO, ReportPushRecordDTO>(pageResult);
            }
        }

        /// <summary>
        /// 增加报告推送记录请求
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<ReportPushRecordDTO> AddReportPushRecordAsync(AddReportPushRecordDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} AddReportPushRecordAsync execute", $"{this.GetType().Name}"))
            {
                var reportPushRecordDO = request.Data?.MappingTo<ReportPushRecordDO>();
                if (reportPushRecordDO != null)
                {
                    var reportCode = await _reportPushRecordsDBRepository.InsertOneAsync(reportPushRecordDO, new Dictionary<string, string>(), string.Empty);
                    var entity = await _reportPushRecordsDBRepository.FindOneAsync("Code", reportCode);
                    return entity.MappingTo<ReportPushRecordDTO>();
                }
                return null;
            }
        }

        /// <summary>
        /// 重置转发记录状态
        /// </summary>
        /// <param name="code"></param>
        /// <returns></returns>
        public async Task<bool> ResetRecordAsync(string code)
        {
            using (new Performance(_performanceThreshold, "{0} ResetRecordAsync execute", $"{this.GetType().Name}"))
            {
                var entity = await _reportPushRecordsDBRepository.FindOneAsync("Code", code);
                if (entity != null && !string.IsNullOrWhiteSpace(entity.Code))
                {
                    entity.State = PushDataStateEnum.UnDo;
                    entity.PushResultToken = string.Empty;
                    entity.PushResponse = string.Empty;
                    return await _reportPushRecordsDBRepository.UpdateOneAsync("Code", entity.Code, entity, string.Empty);
                }
                return false;
            }
        }

        /// <summary>
        /// 通过报告Id更新转发报告记录状态
        /// </summary>
        /// <param name="reportId"></param>
        /// <param name="state"></param>
        /// <returns></returns>
        public async Task<ReportPushRecordDTO> UpdatePushRecordStateByReportIdAsync(string reportId, string posterId, PushDataStateEnum state)
        {
            using (new Performance(_performanceThreshold, "{0} UpdatePushRecordStateByReportIdAsync execute", $"{this.GetType().Name}"))
            {
                var mongoCustomerFilter = new MongoCustomerFilter<MongoTable>();
                mongoCustomerFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("ReportId", reportId));
                mongoCustomerFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("PoseterConfigId", reportId));
                var entities = await _reportPushRecordsDBRepository.FindAllAsync(mongoCustomerFilter.AndCombine());

                var entity = entities.FirstOrDefault();
                if (entity != null && !string.IsNullOrWhiteSpace(entity.Code))
                {
                    entity.State = state;
                    await _reportPushRecordsDBRepository.UpdateOneAsync("Code", entity.Code, entity, string.Empty);
                }
                else
                {
                    return null;
                }
                var existEntity = await _reportPushRecordsDBRepository.FindOneAsync("Code", entity.Code);
                return existEntity.MappingTo<ReportPushRecordDTO>();
            }
        }
        /// <summary>
        /// 更新报告推送记录请求
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<ReportPushRecordDTO> UpdateReportPushRecordAsync(UpdateReportPushRecordDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateReportPushRecordAsync execute", $"{this.GetType().Name}"))
            {
                var entity = await _reportPushRecordsDBRepository.FindOneAsync("Code", request.PushRecordCode);
                if (entity != null && !string.IsNullOrWhiteSpace(entity.Code))
                {
                    entity.State = request.State;
                    entity.PushResultToken = request.PushResultToken;
                    entity.PushResponse = request.PushResponse;
                    entity.UpdateTime = DateTime.UtcNow;
                    await _reportPushRecordsDBRepository.UpdateOneAsync("Code", entity.Code, entity, string.Empty);
                }
                else
                {
                    return null;
                }
                var existEntity = await _reportPushRecordsDBRepository.FindOneAsync("Code", entity.Code);
                return existEntity.MappingTo<ReportPushRecordDTO>();
            }
        }
        /// <summary>
        /// 查询状态
        /// </summary>
        /// <param name="state"></param>
        /// <returns></returns>
        public async Task<List<ReportPushRecordDTO>> GetAllReportPushRecordsByStateAsync(PushDataStateEnum state)
        {
            using (new Performance(_performanceThreshold, "{0} GetAllReportPushRecordsByStateAsync execute", $"{this.GetType().Name}"))
            {
                MongoCustomerFilter<MongoTable> mongoCustomerFilter = new();
                List<(string, string)> eqFilters = new();
                eqFilters.Add(("State", Enum.GetName(state)));
                mongoCustomerFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, eqFilters.ToArray());
                var filter = mongoCustomerFilter.AndCombine();
                var list = await _reportPushRecordsDBRepository.FindAllAsync(filter);
                return list.MappingTo<List<ReportPushRecordDTO>>();
            }
        }
        /// <summary>
        /// 根据记录集合查询数据
        /// </summary>
        /// <param name="codes"></param>
        /// <returns></returns>
        public async Task<List<ReportPushRecordDTO>> GetReportPushRecordsByCodesAsync(List<string> codes)
        {
            using (new Performance(_performanceThreshold, "{0} GetReportPushRecordsByCodesAsync execute", $"{this.GetType().Name}"))
            {
                var dic = new Dictionary<string, List<string>>();
                dic.Add("Code", codes);
                var list = await _reportPushRecordsDBRepository.FindInAsync(dic);
                return list.MappingTo<List<ReportPushRecordDTO>>();
            }
        }

        /// <summary>
        /// 批量新增报告推送数据
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>是否成功</returns>
        public async Task<bool> BatchInsertPosterConfigsDataAsync(BatchInsertPosterConfigRequest request)
        {
            long res = 0;
            if (request?.PosterConfigMigratorys?.Count > 0)
            {
                //找到相同的
                var ids = request.PosterConfigMigratorys.Select(c => c.Code).Distinct().ToList();
                var dic = new Dictionary<string, List<string>>();
                dic.Add("Code", ids);
                var reportPosterList = await _reportPosterDBRepository.FindInAsync(dic);
                //求同存异
                if (reportPosterList?.Count > 0)
                {
                    var existCodes = reportPosterList.Select(c => c.Code).Distinct().ToList();
                    var updateData = request.PosterConfigMigratorys.Where(c => existCodes.Contains(c.Code));
                    var insertData = request.PosterConfigMigratorys.Where(c => !existCodes.Contains(c.Code));
                    if (insertData?.Count() > 0)
                    {
                        //新增
                        var req = insertData.Select(
                            f =>
                            {
                                var data = f.MappingTo<ReportPosterDO>();
                                data.Code = f.Code.Trim();
                                data.IsDelete = f.IsDelete;
                                return (data, new Dictionary<string, string>(), string.Empty);
                            }
                        ).ToList();
                        res = await _reportPosterDBRepository.InsertAllAsync(req);
                    }
                    if (updateData?.Count() > 0)
                    {
                        var req = updateData.Select(f =>
                        {
                            var updateEntity = f.MappingTo<ReportPosterDO>();
                            updateEntity.Code = f.Code.Trim();
                            updateEntity.IsDelete = f.IsDelete;
                            return ("Code", f.Code, updateEntity, string.Empty);
                        }).ToList();
                        res = await _reportPosterDBRepository.UpdateAllAsync(req);
                        foreach (var updateInfo in updateData)
                        {
                            CacheMaintenance.Instance.Get<IReportPostersManager>().Remove(updateInfo.Code);
                        }
                    }
                }
                else
                {
                    //新增
                    var req = request.PosterConfigMigratorys.Select(
                        f =>
                        {
                            var data = f.MappingTo<ReportPosterDO>();
                            data.Code = f.Code.Trim();
                            data.IsDelete = f.IsDelete;
                            return (data, new Dictionary<string, string>(), string.Empty);
                        }
                    ).ToList();
                    res = await _reportPosterDBRepository.InsertAllAsync(req);
                }
            }
            var result = res > 0 ? true : false;
            return result;
        }
    }
}