
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Threading.Tasks;
using AutoMapper.Internal;
using MongoDB.Bson;
using MongoDB.Driver;
using Newtonsoft.Json;
using WingInterfaceLibrary.DB.Request;
using WingInterfaceLibrary.DTO.Record;
using WingInterfaceLibrary.DTO.User;
using WingInterfaceLibrary.Enum;
using WingInterfaceLibrary.Interface.DBInterface;
using WingInterfaceLibrary.Request;
using WingInterfaceLibrary.Request.DBRequest;
using WingInterfaceLibrary.ResearchEdition;
using WingMongoDBModule.Common;
using WingMongoDBModule.Entities;
using WingMongoDBModule.Entity;
using WingMongoDBModule.Mongodb.Driver;
using WingMongoDBModule.Request;
using WingServerCommon.Interfaces.Cache;
using WingServerCommon.Log;
using WingServerCommon.Mapper;
using WingServerCommon.Service;
using WingServerCommon.Utilities;

namespace WingMongoDBModule.Service
{
    public partial class DatabaseService : JsonRpcService, IResearchProjectDBService
    {
        /// <summary>
        /// 查询项目列表
        /// </summary>
        /// <param name="request">查询项目列表请求实体</param>
        /// <returns></returns>
        /// <errorCodes></errorCodes>
        public async Task<IList<ResearchProjectBaseDTO>> GetResearchProjectListDBAsync(GetResearchProjectListDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetResearchProjectListDBAsync execute", "DatabaseService"))
            {
                var keyword = request.Keyword;
                var builder = Builders<ResearchProjectEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false);
                if (!string.IsNullOrEmpty(keyword))
                {
                    keyword = EscapeExprSpecialWord(keyword);//字符串转义
                    var nameFilter = builder.Eq(x => x.Name, new BsonRegularExpression(keyword, "i"));
                    filter = filter & nameFilter;
                }
                if (request.SearchType == ResearchProjectSearchTypeEnum.ICreated && !string.IsNullOrEmpty(request.UserCode))
                {
                    filter = filter & builder.Where(x => x.BelongUserCode == request.UserCode);
                }
                else if (request.SearchType == ResearchProjectSearchTypeEnum.IJoined && !string.IsNullOrEmpty(request.UserCode))
                {
                    filter = filter & builder.Where(x => x.UserCodes.Contains(request.UserCode) && x.BelongUserCode != request.UserCode);
                }
                else if (request.SearchType == ResearchProjectSearchTypeEnum.Undefined && !string.IsNullOrEmpty(request.UserCode))
                {
                    filter = filter & builder.Where(x => x.BelongUserCode == request.UserCode || x.UserCodes.Contains(request.UserCode) || x.Code == "BaseVetProject");
                }
                else if (request.SearchType == ResearchProjectSearchTypeEnum.Default)
                {
                    filter = filter & builder.Where(x => x.Code == "BaseVetProject");
                }
                var dataList = await _researchProjectDBRepository.FindAllAsync(filter);
                return dataList.MappingTo<List<ResearchProjectBaseDTO>>();
            }
        }

        /// <summary>
        /// 验证用户项目是否有项目权限或者项目是否存在
        /// </summary>
        /// <param name="request">查询项目详情请求实体</param>
        /// <returns></returns>
        /// <errorCodes></errorCodes>
        public async Task<bool> ValidateResearchProjectAuthDBAsync(GetResearchProjectDetailRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetResearchProjectDetailDBAsync execute", "DatabaseService"))
            {
                if (request.Code == "BaseVetProject")
                {
                    return true;
                }
                var builder = Builders<ResearchProjectEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false);
                if (!string.IsNullOrEmpty(request.Code))
                {
                    var codeFilter = builder.Eq(x => x.Code, request.Code);
                    filter = filter & codeFilter;
                }
                var dataInfo = await _researchProjectDBRepository.FindOneAsync(filter);
                if (!string.IsNullOrEmpty(dataInfo?.Code))
                {
                    if (!dataInfo.UserCodes.Contains(request.UserCode))
                    {
                        ThrowCustomerException(CustomerRpcCode.NotExistCurrentProjectAuthority, "Not exist current project authority");
                    }
                }
                else
                {
                    //项目不存在
                    ThrowCustomerException(CustomerRpcCode.CurrentProjectNotExist, "Current project not exist");
                }
                return true;
            }
        }

        /// <summary>
        /// 查询项目详情
        /// </summary>
        /// <param name="request">查询项目详情请求实体</param>
        /// <returns></returns>
        /// <errorCodes></errorCodes>
        public async Task<ResearchProjectDTO> GetResearchProjectDetailDBAsync(GetResearchProjectDetailRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetResearchProjectDetailDBAsync execute", "DatabaseService"))
            {
                var builder = Builders<ResearchProjectEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false);
                if (!string.IsNullOrEmpty(request.Code))
                {
                    var codeFilter = builder.Eq(x => x.Code, request.Code);
                    filter = filter & codeFilter;
                }
                var dataInfo = await _researchProjectDBRepository.FindOneAsync(filter);
                var result = new ResearchProjectDTO();
                if (!string.IsNullOrEmpty(dataInfo?.Code))
                {
                    result = dataInfo.MappingTo<ResearchProjectDTO>();
                    result.Users = new List<WingInterfaceLibrary.DTO.User.UserBaseDTO>();
                    if (dataInfo.UserCodes?.Count > 0)
                    {
                        var userList = await GetUserListByUserCodesAsync(dataInfo.UserCodes);
                        if (userList != null && userList.Any())
                        {
                            foreach (var findUserInfo in userList)
                            {
                                result.Users.Add(new WingInterfaceLibrary.DTO.User.UserBaseDTO()
                                {

                                    Phone = findUserInfo.Phone,
                                    Email = findUserInfo.Email,
                                    UserCode = findUserInfo.UserCode,
                                    UserName = findUserInfo.UserName,
                                    FullName = findUserInfo.FullName,
                                    HeadImageUrl = findUserInfo.HeadImageToken,
                                });

                            }
                        }
                    }

                }
                return result;
            }
        }

        /// <summary>
        /// 创建项目
        /// </summary>
        /// <param name="request">创建项目请求实体</param>
        /// <returns></returns>
        public async Task<string> CreateResearchProjectDBAsync(CreateResearchProjectDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} CreateResearchProjectDBAsync execute", "DatabaseService"))
            {

                var builder = Builders<ResearchProjectEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false && c.Name == request.Name);
                var existEntity = await _researchProjectDBRepository.Exists(filter);
                if (!existEntity)
                {
                    var entity = request.MappingTo<ResearchProjectEntity>();
                    entity.CreateTime = DateTime.UtcNow;
                    entity.UpdateTime = DateTime.UtcNow;
                    entity.OperateTime = DateTime.UtcNow;
                    entity.OperatorCode = request.BelongUserCode;
                    entity.OperatorName = request.CreatorName;
                    entity.BelongUserCode = request.BelongUserCode;
                    entity.CreatorName = request.CreatorName;
                    entity.PatientType = request.PatientType;
                    entity.Introduction = request.Introduction;
                    entity.UserCodes = new List<string> { request.BelongUserCode };
                    var projectCode = await _researchProjectDBRepository.InsertOneAsync(entity);
                    return projectCode;
                }
                else
                {
                    ThrowCustomerException(CustomerRpcCode.DataRepeat, "Data repeat error");
                    return "";
                }
            }
        }

        /// <summary>
        /// 修改项目
        /// </summary>
        /// <param name="request">创建项目请求实体</param>
        /// <returns></returns>
        public async Task<bool> UpdateResearchProjectDBAsync(UpdateResearchProjectDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateResearchProjectDBAsync execute", "DatabaseService"))
            {
                var res = false;
                if (request.Code == "BaseVetProject")
                {
                    return res;
                }
                var sameNameBuilder = Builders<ResearchProjectEntity>.Filter;
                var sameNameFilter = sameNameBuilder.Where(c => c.IsDelete == false && c.Name == request.Name);
                var sameNameDataInfo = await _researchProjectDBRepository.FindOneAsync(sameNameFilter);
                if (sameNameDataInfo != null && !string.IsNullOrWhiteSpace(sameNameDataInfo.Code))
                {
                    ThrowCustomerException(CustomerRpcCode.DataRepeat, "Data repeat error");
                }
                var builder = Builders<ResearchProjectEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false && c.Code == request.Code);
                var dataInfo = await _researchProjectDBRepository.FindOneAsync(filter);
                if (!string.IsNullOrEmpty(dataInfo?.Code))
                {
                    dataInfo.UpdateTime = DateTime.UtcNow;
                    dataInfo.OperateTime = DateTime.UtcNow;
                    dataInfo.OperatorCode = dataInfo.OperatorCode;
                    dataInfo.OperatorName = dataInfo.OperatorName;
                    dataInfo.BelongUserCode = dataInfo.BelongUserCode;
                    dataInfo.CreatorName = dataInfo.CreatorName;
                    dataInfo.PatientType = request.PatientType;
                    dataInfo.Name = request.Name;
                    dataInfo.Introduction = request.Introduction;
                    res = await _researchProjectDBRepository.UpdateOneAsync("Code", dataInfo.Code, dataInfo);
                    //时间轴
                    var task = new TaskEntity()
                    {
                        BusinessId = "",
                        TaskName = "",
                        MainTaskCode = "",
                        Description = "",
                        CreatorName = dataInfo.CreatorName,
                        OperatorCode = dataInfo.BelongUserCode,
                        OperatorName = dataInfo.CreatorName,
                        OperateTime = DateTime.UtcNow,
                        Status = VTaskStatus.Completed,
                        TaskType = VTaskType.ProjectUpdate,
                        CreateTime = DateTime.UtcNow,
                        UpdateTime = DateTime.UtcNow,
                        BusinessCode = dataInfo.Code,
                    };
                    await _taskDBRepository.InsertOneAsync(task);

                }
                return res;
            }
        }

        /// <summary>
        /// 删除项目
        /// </summary>
        /// <param name="request">创建项目请求实体</param>
        /// <returns></returns>
        public async Task<bool> DeleteResearchProjectDBAsync(DeleteResearchProjectDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} DeleteResearchProjectDBAsync execute", "DatabaseService"))
            {
                if (request.Code == "BaseVetProject")
                {
                    return false;
                }
                var projectCode = request.Code;
                var result = await _researchProjectDBRepository.DeleteOneAsync("Code", projectCode);
                //删除文件
                if (result)
                {
                    var remedicals = await GetRemedicalsByProjectCodeAsync(projectCode);
                    if (remedicals != null && remedicals.Count > 0)
                    {
                        var secondDirectory = GetSecondDirectory();
                        foreach (var item in remedicals)
                        {
                            try
                            {
                                var originImageFilePath = GetFilePath(item.TerminalImages.OriginImageUrl, secondDirectory);
                                DeleteFile(originImageFilePath);
                                var imageFilePath = GetFilePath(item.TerminalImages.ImageUrl, secondDirectory);
                                DeleteFile(imageFilePath);
                                var coverImageFilePath = GetFilePath(item.TerminalImages.CoverImageUrl, secondDirectory);
                                DeleteFile(coverImageFilePath);
                                var previewFilePath = GetFilePath(item.TerminalImages.PreviewUrl, secondDirectory);
                                DeleteFile(previewFilePath);
                                foreach (var imgDetail in item.ImgList)
                                {
                                    var imgDetailFilePath = GetFilePath(imgDetail.FileToken, secondDirectory);
                                    DeleteFile(imgDetailFilePath);
                                }
                            }
                            catch (Exception ex)
                            {
                                Logger.WriteLineError($"delete file error:{item.RemedicalCode},{ex}");
                            }
                        }
                        var remedicalMeasuredInfos = await GetRemedicalMeasuredInfosByRemedicalCodesAsync(remedicals.Select(x => x.RemedicalCode).ToList());
                        if (remedicalMeasuredInfos != null && remedicalMeasuredInfos.Count > 0)
                        {
                            foreach (var item in remedicalMeasuredInfos)
                            {
                                try
                                {
                                    var measuredFilePath = GetFilePath(item.MeasuredFileToken, secondDirectory);
                                    DeleteFile(measuredFilePath);
                                    var previewFilePath = GetFilePath(item.PreviewFileToken, secondDirectory);
                                    DeleteFile(previewFilePath);
                                }
                                catch (Exception ex)
                                {
                                    Logger.WriteLineError($"delete file remedicalMeasuredInfos error:{item.RemedicalMeasuredInfoCode},{ex}");
                                }
                            }
                        }
                    }
                }
                return result;
            }
        }

        string GetSecondDirectory()
        {
            var secondDirectory = "";
            if (!string.IsNullOrEmpty(_serverUrl))
            {
                var tempServerUrl = _serverUrl.TrimEnd('/');
                if (tempServerUrl.Split('/').Length > 0)
                {
                    var arrayList = tempServerUrl.Split('/');
                    secondDirectory = arrayList[(arrayList.Length - 1)];
                }
            }
            return secondDirectory;
        }

        string GetFilePath(string fileUrl, string secondDirectory)
        {
            var originImageArrayList = fileUrl.Split('/');
            var originImageFileDate = originImageArrayList[(originImageArrayList.Length - 2)];
            var originImageFileName = originImageArrayList[(originImageArrayList.Length - 1)];
            var filePath = Path.Combine(_storageDisk, secondDirectory, originImageFileDate, originImageFileName);
            return filePath;
        }

        void DeleteFile(string filePath)
        {
            if (File.Exists(filePath))
            {
                File.Delete(filePath);
            }
        }


        /// <summary>
        /// 邀请成员
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> InviteResearchProjectMembersDBAsync(ResearchProjectMembersDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} InviteResearchProjectMembersDBAsync execute", "DatabaseService"))
            {
                var res = false;
                if (request.Code == "BaseVetProject")
                {
                    return res;
                }
                var builder = Builders<ResearchProjectEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false && c.Code == request.Code);
                var dataInfo = await _researchProjectDBRepository.FindOneAsync(filter);
                if (!string.IsNullOrEmpty(dataInfo?.Code))
                {
                    dataInfo.UserCodes = dataInfo.UserCodes ?? new List<string>();
                    dataInfo.UserCodes.AddRange(request.MemberCodes);
                    dataInfo.UserCodes = dataInfo.UserCodes.Distinct().ToList();
                    dataInfo.OperateTime = DateTime.UtcNow;
                    dataInfo.OperatorCode = request.UserCode;
                    dataInfo.OperatorName = request.UserName;
                    res = await _researchProjectDBRepository.UpdateOneAsync("Code", dataInfo.Code, dataInfo);
                    //保存邀请记录用于时间轴
                    if (res)
                    {
                        if (request.MemberCodes != null && request.MemberCodes.Count > 0)
                        {
                            var task = new TaskEntity()
                            {
                                BusinessId = "",
                                TaskName = "",
                                MainTaskCode = "",
                                Description = "",
                                CreatorName = request.UserName,
                                OperatorCode = request.UserCode,
                                OperatorName = request.UserName,
                                OperateTime = DateTime.UtcNow,
                                Status = VTaskStatus.Completed,
                                TaskType = VTaskType.InviteUser,
                                CreateTime = DateTime.UtcNow,
                                UpdateTime = DateTime.UtcNow,
                                BusinessCode = dataInfo.Code,
                                JsonData = string.Join(",", request.MemberCodes)
                            };
                            await _taskDBRepository.InsertOneAsync(task);
                        }
                    }
                }
                return res;
            }
        }

        /// <summary>
        /// 移除成员
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> RemoveResearchProjectMembersDBAsync(ResearchProjectMembersDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} RemoveResearchProjectMembersRequest execute", "DatabaseService"))
            {
                var res = false;
                if (request.Code == "BaseVetProject")
                {
                    return res;
                }
                var builder = Builders<ResearchProjectEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false && c.Code == request.Code);
                var dataInfo = await _researchProjectDBRepository.FindOneAsync(filter);
                if (!string.IsNullOrEmpty(dataInfo?.Code))
                {
                    foreach (var item in request.MemberCodes)
                    {
                        if (dataInfo.UserCodes.Contains(item))
                        {
                            dataInfo.UserCodes.Remove(item);
                        }
                    }
                    dataInfo.OperateTime = DateTime.UtcNow;
                    dataInfo.OperatorCode = request.UserCode;
                    dataInfo.OperatorName = request.UserName;
                    res = await _researchProjectDBRepository.UpdateOneAsync("Code", dataInfo.Code, dataInfo);
                    //保存移除记录用于时间轴
                    if (res)
                    {
                        if (request.MemberCodes != null && request.MemberCodes.Count > 0)
                        {
                            var task = new TaskEntity()
                            {
                                BusinessId = "",
                                TaskName = "",
                                MainTaskCode = "",
                                Description = "",
                                CreatorName = request.UserName,
                                OperatorCode = request.UserCode,
                                OperatorName = request.UserName,
                                OperateTime = DateTime.UtcNow,
                                Status = VTaskStatus.Completed,
                                TaskType = VTaskType.RemoveUser,
                                CreateTime = DateTime.UtcNow,
                                UpdateTime = DateTime.UtcNow,
                                BusinessCode = dataInfo.Code,
                                JsonData = string.Join(",", request.MemberCodes)
                            };
                            await _taskDBRepository.InsertOneAsync(task);
                        }
                    }
                }
                return res;
            }
        }

        /// <summary>
        /// 创建任务
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<string> CreateResearchProjectTaskDBAsync(CreateResearchProjectTaskDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} CreateResearchProjectTaskDBAsync execute", "DatabaseService"))
            {
                var res = "";
                var builder = Builders<TaskEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false && c.TaskName == request.Name);
                var existEntity = await _taskDBRepository.Exists(filter);
                if (!existEntity)
                {
                    var model = new TaskMainEntity()
                    {
                        CreatorName = request.UserName,
                        OperatorCode = request.UserCode,
                        OperatorName = request.UserName,
                        OperateTime = DateTime.UtcNow,
                        Status = VTaskStatus.Unhandled,
                        TaskType = request.TaskType,
                        TaskName = request.Name,
                        CreateTime = DateTime.UtcNow,
                        UpdateTime = DateTime.UtcNow,
                        BusinessCode = request.BusinessCode
                    };
                    res = await _taskMainDBRepository.InsertOneAsync(model);
                    foreach (var item in request.TaskDetailList)
                    {
                        var task = new TaskEntity()
                        {
                            BusinessId = item.Id,
                            TaskName = item.Name,
                            MainTaskCode = res,
                            Description = item.Description,
                            CreatorName = request.UserName,
                            OperatorCode = request.UserCode,
                            OperatorName = request.UserName,
                            OperateTime = DateTime.UtcNow,
                            Status = VTaskStatus.Unhandled,
                            TaskType = request.TaskType,
                            CreateTime = DateTime.UtcNow,
                            UpdateTime = DateTime.UtcNow,
                            BusinessCode = request.BusinessCode
                        };
                        await _taskDBRepository.InsertOneAsync(task);
                    }
                }
                else
                {
                    ThrowCustomerException(CustomerRpcCode.DataRepeat, "Data repeat error");
                }
                return res;
            }
        }

        /// <summary>
        /// 更新任务进度状态
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> UpdateResearchProjectTaskDBAsync(UpdateResearchProjectTaskDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateResearchProjectTaskDBAsync execute", "DatabaseService"))
            {
                var res = false;
                var builder = Builders<TaskMainEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false && c.Code == request.MainTaskCode);
                //主任务
                var dataInfo = await _taskMainDBRepository.FindOneAsync(filter);
                if (!string.IsNullOrEmpty(dataInfo?.Code))
                {
                    //子任务
                    var taskBuilder = Builders<TaskEntity>.Filter;
                    var taskFilter = taskBuilder.Where(c => c.IsDelete == false && c.BusinessId == request.Id && c.MainTaskCode == request.MainTaskCode);
                    var task = await _taskDBRepository.FindOneAsync(taskFilter);
                    task.Result = request.Result;
                    task.Status = request.Status;
                    task.Rate = request.Rate;
                    await _taskDBRepository.UpdateOneAsync("Code", task.Code, task);
                    //主任务
                    var taskAllBuilder = Builders<TaskEntity>.Filter;
                    var taskAllFilter = taskAllBuilder.Where(c => c.IsDelete == false && c.MainTaskCode == dataInfo.Code);
                    var taskList = await _taskDBRepository.FindAllAsync(taskAllFilter);
                    if (!taskList.Any(x => x.Status != VTaskStatus.Completed && x.Status != VTaskStatus.Canceled))
                    {
                        dataInfo.Status = VTaskStatus.Completed;
                        dataInfo.EndTime = DateTime.UtcNow;
                    }
                    dataInfo.OperatorCode = request.UserCode;
                    dataInfo.OperatorName = request.UserName;
                    dataInfo.OperateTime = DateTime.UtcNow;
                    res = await _taskMainDBRepository.UpdateOneAsync("Code", dataInfo.Code, dataInfo);
                }
                return res;
            }
        }

        /// <summary>
        /// 查询任务列表
        /// </summary>
        /// <returns></returns>
        public async Task<IList<TaskBaseDTO>> GetMainTaskListDBAsync(GetMainTaskListDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetTaskListDBAsync execute", "DatabaseService"))
            {
                var resultList = new List<TaskBaseDTO>();
                var builder = Builders<TaskMainEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false && c.OperatorCode == request.UserCode);
                if (request.Status == VTaskStatus.Completed)
                {
                    var statusFilter = builder.Where(c => c.Status == request.Status);
                    filter = filter & statusFilter;
                }
                else if (request.Status == VTaskStatus.Processing)
                {
                    var statusFilter = builder.Where(c => c.Status == VTaskStatus.Processing || c.Status == VTaskStatus.Unhandled);
                    filter = filter & statusFilter;
                }
                else if (request.Status == VTaskStatus.Unhandled)
                {
                }
                else
                {
                    var statusFilter = builder.Where(c => c.Status != VTaskStatus.Unhandled);
                    filter = filter & statusFilter;
                }
                if (!string.IsNullOrEmpty(request.KeyWord))
                {
                    var nameFilter = builder.Where(c => c.TaskName.Contains(request.KeyWord));
                    filter = filter & nameFilter;

                }
                var dataList = await _taskMainDBRepository.FindAllAsync(filter);
                if (dataList?.Count > 0)
                {
                    resultList = dataList.MappingTo<List<TaskBaseDTO>>();
                }
                return resultList;
            }
        }

        /// <summary>
        /// 查询任务详情
        /// </summary>
        /// <returns></returns>
        public async Task<MainTaskDTO> GetMainTaskDetailDBAsync(GetMainTaskListDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetTaskDetailDBAsync execute", "DatabaseService"))
            {
                var resultInfo = new MainTaskDTO();
                var builder = Builders<TaskMainEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false && c.Code == request.TaskCode);
                var dataInfo = await _taskMainDBRepository.FindOneAsync(filter);
                //增加
                string projectName = "";
                string patientName = "";
                if (!string.IsNullOrEmpty(dataInfo?.Code))
                {
                    resultInfo = dataInfo.MappingTo<MainTaskDTO>();
                }
                if (!string.IsNullOrEmpty(dataInfo?.BusinessCode))
                {
                    var researchProjectBuilder = Builders<ResearchProjectEntity>.Filter;
                    var researchProjectFilter = researchProjectBuilder.Where(c => c.IsDelete == false && c.Code == dataInfo.BusinessCode);
                    var researchProjectInfo = await _researchProjectDBRepository.FindOneAsync(researchProjectFilter);
                    if (!string.IsNullOrEmpty(researchProjectInfo?.Code))
                    {
                        projectName = researchProjectInfo.Name;
                    }
                }
                if (!string.IsNullOrEmpty(dataInfo?.PatientCode))
                {
                    var patientInfo = await _patientInfoDBRepository.FindOneAsync("Code", dataInfo.PatientCode);
                    if (!string.IsNullOrEmpty(patientInfo?.Code))
                    {
                        patientName = CryptionHelper.NameEncrypt(patientInfo.Name, false);
                    }
                    if (string.IsNullOrEmpty(patientName))
                    {
                        patientName = "未命名";
                    }
                }
                var taskAllBuilder = Builders<TaskEntity>.Filter;
                var taskAllFilter = taskAllBuilder.Where(c => c.IsDelete == false && c.MainTaskCode == dataInfo.Code);
                var taskList = await _taskDBRepository.FindAllAsync(taskAllFilter);
                if (taskList != null && taskList.Count > 0)
                {
                    var remedicalRecords = new List<DO.RemedicalDO>();
                    var remedicalCodes = taskList.Where(c => !string.IsNullOrEmpty(c.BusinessCode))?.Select(c => c.BusinessCode)?.Distinct()?.ToList();
                    if (remedicalCodes?.Count > 0)
                    {
                        var dic = new Dictionary<string, List<string>>();
                        dic.Add("Code", remedicalCodes);
                        remedicalRecords = await _remedicalDBRepository.FindInAsync(dic);
                    }
                    resultInfo.TaskDetailList = new List<TaskDetail>();
                    foreach (var item in taskList)
                    {
                        var taskDetail = new TaskDetail();
                        taskDetail.TaskCode = item.Code;
                        taskDetail.Id = item.BusinessId;
                        taskDetail.Name = item.TaskName;
                        taskDetail.Rate = item.Rate;
                        taskDetail.Result = item.Result;
                        taskDetail.Status = item.Status;
                        taskDetail.Description = item.Description;
                        taskDetail.BusinessCode = item.BusinessCode;
                        if (remedicalRecords?.Count > 0)
                        {
                            var findEntity = remedicalRecords.FirstOrDefault(c => c.Code == item.BusinessCode);
                            if (!string.IsNullOrEmpty(findEntity?.Code) && findEntity.CreateTime > DateTime.MinValue)
                            {
                                var dateName = findEntity.CreateTime.ToUTCToBeijing().ToString("yyyy-MM-dd HH:mm:ss");
                                taskDetail.Description = projectName + "_" + patientName + "_" + dateName;
                            }
                            else
                            {
                                taskDetail.Description = projectName + "_" + patientName;
                            }
                        }
                        else
                        {
                            taskDetail.Description = projectName + "_" + patientName;
                        }
                        resultInfo.TaskDetailList.Add(taskDetail);
                    }
                }
                return resultInfo;
            }
        }

        /// <summary>
        /// 根据项目查询任务列表
        /// </summary>
        /// <returns></returns>
        public async Task<IList<TaskBaseDTO>> GetMainTaskListByProjectDBAsync(GetTaskListByProjectRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetTaskListByProjectDBAsync execute", "DatabaseService"))
            {
                var resultInfo = new List<TaskBaseDTO>();
                var builder = Builders<TaskMainEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false && c.BusinessCode == request.ProjectCode);
                var dataInfos = await _taskMainDBRepository.FindAllAsync(filter);
                if (dataInfos?.Count > 0)
                {
                    resultInfo = dataInfos.MappingTo<List<TaskBaseDTO>>();
                }
                return resultInfo;
            }
        }

        /// <summary>
        /// 查询所有用户集合（分页）
        /// </summary>
        /// <param name="request">查询机构专家集合请求实体</param>
        /// <returns></returns>
        /// <errorCodes></errorCodes>
        public async Task<PageCollection<UserBaseDTO>> GetUserBaseInfoPagesDBAsync(FindUserPageByOrgsRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetTaskListByProjectDBAsync execute", "DatabaseService"))
            {
                var filter = GetFilter(request);
                var dbPage = new DBPageRequest();
                dbPage.CurrentPage = request.CurrentPage;
                dbPage.PageSize = request.PageSize;
                dbPage.Filter = filter;
                var result = await _userDBRepository.GetPages(dbPage);
                return PageCollectionConvert<DO.UserInfoDO, UserBaseDTO>(result);
            }
        }

        /// <summary>
        /// 查询项目数据
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageCollection<ResearchProjectDataDTO>> GetResearchProjectDataPagesDBAsync(GetResearchProjectDatasRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetResearchProjectDataPagesDBAsync execute", "DatabaseService"))
            {
                var dbPage = new PageCollection<ResearchProjectDataDTO>();
                dbPage.CurrentPage = request.PageIndex;
                dbPage.PageSize = request.PageSize;
                var keyword = request.KeyWord;
                var builder = Builders<ResearchProjectEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false && c.Code == request.ProjectCode);
                var dataInfo = await _researchProjectDBRepository.FindOneAsync(filter);
                if (!string.IsNullOrEmpty(dataInfo?.Code))
                {
                    var keywordfilter = new Dictionary<string, string>();
                    if (!string.IsNullOrWhiteSpace(keyword))
                    {
                        var tempKeyWord = CryptionHelper.EncryptOneByOne(keyword.Trim());
                        keywordfilter.Add("Name", tempKeyWord);
                        keywordfilter.Add("Phone", tempKeyWord);
                        keywordfilter.Add("IdentityCard", tempKeyWord);
                    }
                    var pageFilterRequest = new GetPatientsByOrgPageRequest()
                    {
                        KeywordOrFilter = keywordfilter,
                        CurrentPage = request.PageIndex,
                        PageSize = request.PageSize,
                        IsFuzzy = true,
                        ProjectCode = request.ProjectCode
                    };
                    var sort = Builders<MongoTable>.Sort.Descending("BaseData.CreateTime");
                    if (request.Sort == 1)
                    {
                        sort = Builders<MongoTable>.Sort.Ascending("BaseData.CreateTime");
                    }
                    var patients = await GetPatientPagesAsync(pageFilterRequest, sort);
                    if (patients != null && patients.PageData.Count > 0)
                    {
                        dbPage.DataCount = patients.DataCount;
                        var records = await FindRecordInfosByProjectCodeAsync(request.ProjectCode);
                        var remedicals = await GetRemedicalsByProjectCodeAsync(request.ProjectCode);
                        var devices = await FindDeviceInfoListByCodeListAsync(patients.PageData.Select(x => x.DeviceCode).ToList());
                        dbPage.PageData = new List<ResearchProjectDataDTO>();
                        foreach (var item in patients.PageData)
                        {
                            var researchProjectDataDTO = new ResearchProjectDataDTO();
                            var itemRecords = records.Where(x => x.PatientCode == item.PatientCode);
                            if (itemRecords != null && itemRecords.Count() > 0)
                            {
                                var recordCodes = itemRecords.Select(x => x.RecordCode);
                                if (recordCodes != null && recordCodes.Count() > 0)
                                {
                                    var itemRemedicals = remedicals.Where(x => recordCodes.Contains(x.RecordCode));
                                    var imgList = itemRemedicals.SelectMany(x => x.ImgList.Where(x => x.ImgType == 0 || x.IsShow == true));
                                    var fileSize = imgList.DistinctBy(x => x.FileToken).Sum(i => i.FileSize);
                                    fileSize += itemRemedicals.DistinctBy(x => x.TerminalImages.OriginImageUrl).Sum(x => x.TerminalImages.ImageSize);
                                    researchProjectDataDTO.FileSize = fileSize.ToString();
                                }
                            }
                            researchProjectDataDTO.CreateTime = item.CreateTime;
                            researchProjectDataDTO.DevicePatientId = item.DevicePatientIDs.FirstOrDefault();
                            researchProjectDataDTO.PatientCode = item.PatientCode;
                            researchProjectDataDTO.IsUrm = item.IsUrm;
                            researchProjectDataDTO.PatientName = item.Name;
                            var device = devices.FirstOrDefault(x => x.DeviceCode == item.DeviceCode);
                            researchProjectDataDTO.DeviceShortCode = device?.ShortCode;
                            dbPage.PageData.Add(researchProjectDataDTO);
                        }
                    }
                }
                return dbPage;
            }
        }

        /// <summary>
        /// 获取URM关联图像集合
        /// </summary>
        /// <returns></returns>
        public async Task<List<SampleRemedicalInfoDTO>> GetUrmRemedicalInfosDBAsync(string urmRemedicalCode)
        {
            var sampleRemedicalInfoList = new List<SampleRemedicalInfoDTO>();
            if (urmRemedicalCode.Length > 32)//视频
            {
                var tempCode = urmRemedicalCode.Substring(0, 32);
                MongoCustomerFilter<MongoTable> remedicalFilter = new();
                remedicalFilter.AppendEq(("BaseData.Code", tempCode));
                var remedical = await _remedicalDBRepository.FindOneAsync(remedicalFilter.AndCombine());
                urmRemedicalCode = remedical.UrmOriginalCode;
            }
            MongoCustomerFilter<MongoTable> originalRemedicalFilter1 = new();
            originalRemedicalFilter1.AppendEq(("BaseData.Code", urmRemedicalCode));
            var originalRemedical1 = await _remedicalDBRepository.FindOneAsync(originalRemedicalFilter1.AndCombine());
            if (originalRemedical1 == null || string.IsNullOrWhiteSpace(originalRemedical1.Code))//测量图
            {
                MongoCustomerFilter<MongoTable> remedicalCodeMeasuredFilter = new();
                remedicalCodeMeasuredFilter.AppendEq(("BaseData.Code", urmRemedicalCode));
                var remedicalMeasure = await _remedicalMeasuredInfoDBRepository.FindOneAsync(remedicalCodeMeasuredFilter.AndCombine());
                if (remedicalMeasure != null && !string.IsNullOrWhiteSpace(remedicalMeasure.Code))
                {
                    urmRemedicalCode = remedicalMeasure.UrmOriginalCode;
                }
            }
            if (!string.IsNullOrWhiteSpace(urmRemedicalCode))
            {
                MongoCustomerFilter<MongoTable> originalRemedicalFilter = new();
                originalRemedicalFilter.AppendEq(("BaseData.Code", urmRemedicalCode));
                var originalRemedical = await _remedicalDBRepository.FindOneAsync(originalRemedicalFilter.AndCombine());
                var originalRemedicalData = originalRemedical.MappingTo<RemedicalInfoDTO>();
                sampleRemedicalInfoList.Add(ConvertToSampleRemedical(originalRemedicalData));//原图
                MongoCustomerFilter<MongoTable> remedicalFilter = new();
                remedicalFilter.AppendEq(("BaseData.UrmOriginalCode", urmRemedicalCode));
                var remedicalList = await _remedicalDBRepository.FindAllAsync(remedicalFilter.AndCombine());
                var recordRemedicalDatas = remedicalList.MappingTo<List<RemedicalInfoDTO>>();

                MongoCustomerFilter<MongoTable> remedicalMeasuredFilter = new();
                remedicalMeasuredFilter.AppendEq(("BaseData.UrmOriginalCode", urmRemedicalCode));
                var remedicalMeasuredList = await _remedicalMeasuredInfoDBRepository.FindAllAsync(remedicalMeasuredFilter.AndCombine());
                var remedicalMeasuredInfos = remedicalMeasuredList.MappingTo<List<RemedicalMeasuredInfoDTO>>();

                if (recordRemedicalDatas?.Count > 0)
                {
                    foreach (var urmResult in recordRemedicalDatas)
                    {
                        sampleRemedicalInfoList.Add(ConvertToSampleRemedical(urmResult));//分析结果

                        sampleRemedicalInfoList.AddRange(ConvertVideoToSampleRemedical(urmResult));//视频
                        var itemRemedicalMeasuredInfos1 = remedicalMeasuredInfos.Where(x => x.RemedicalOriginalCode == urmResult.RemedicalCode)
                            ?.OrderBy(x => x.CreateTime)?.ToList() ?? new List<RemedicalMeasuredInfoDTO>();
                        foreach (var itemRemedicalMeasuredInfo in itemRemedicalMeasuredInfos1)
                        {
                            sampleRemedicalInfoList.Add(ConvertToSampleRemedical(itemRemedicalMeasuredInfo));//测量
                        }
                    }
                }
                else
                {
                    if (remedicalMeasuredInfos?.Count > 0)
                    {
                        var itemRemedicalMeasuredInfos1 = remedicalMeasuredInfos.OrderBy(x => x.CreateTime).ToList();
                        foreach (var itemRemedicalMeasuredInfo in itemRemedicalMeasuredInfos1)
                        {
                            sampleRemedicalInfoList.Add(ConvertToSampleRemedical(itemRemedicalMeasuredInfo));//测量
                        }
                    }
                    else
                    {
                        remedicalMeasuredFilter = new();
                        remedicalMeasuredFilter.AppendEq(("BaseData.RemedicalOriginalCode", urmRemedicalCode));
                        remedicalMeasuredList = await _remedicalMeasuredInfoDBRepository.FindAllAsync(remedicalMeasuredFilter.AndCombine());
                        remedicalMeasuredInfos = remedicalMeasuredList.MappingTo<List<RemedicalMeasuredInfoDTO>>();
                        if (remedicalMeasuredInfos?.Count > 0)
                        {
                            var itemRemedicalMeasuredInfos1 = remedicalMeasuredInfos.OrderBy(x => x.CreateTime).ToList();
                            foreach (var itemRemedicalMeasuredInfo in itemRemedicalMeasuredInfos1)
                            {
                                sampleRemedicalInfoList.Add(ConvertToSampleRemedical(itemRemedicalMeasuredInfo));//测量
                            }
                        }
                    }
                }

                // var orginalUrmRemedicalInfos = recordRemedicalDatas.Where(r => r.ImgList != null && r.ImgList.Any(x => x.ImgType == 0) && r.ImgList.Any(x => x.ImgType != 0))
                //     ?.OrderBy(x => x.CreateTime)?.ToList() ?? new List<RemedicalInfoDTO>();
                // foreach (var original in orginalUrmRemedicalInfos)
                // {
                //     sampleRemedicalInfoList.Add(ConvertToSampleRemedical(original));//分析结果
                //     var urmRemedicalInfos = recordRemedicalDatas.Where(r => r.RemedicalOriginalCode == original.RemedicalCode)
                //         ?.OrderBy(x => x.CreateTime)?.ToList() ?? new List<RemedicalInfoDTO>();

                //     foreach (var urm in urmRemedicalInfos)
                //     {
                //         sampleRemedicalInfoList.Add(ConvertToSampleRemedical(urm));//
                //         var videoRemedicalInfos = recordRemedicalDatas.Where(r => r.RemedicalOriginalCode == urm.RemedicalCode && r.ImgList != null && r.ImgList.Any(x => x.ImgType == 4))
                //             ?.OrderBy(x => x.CreateTime)?.ToList() ?? new List<RemedicalInfoDTO>();
                //         foreach (var video in videoRemedicalInfos)
                //         {
                //             var info = ConvertVideoToSampleRemedical(video);
                //             if (info != null)
                //             {
                //                 sampleRemedicalInfoList.Add(info);
                //             }
                //         }
                //         var itemRemedicalMeasuredInfos1 = remedicalMeasuredInfos.Where(x => x.UrmOriginalCode == urm.RemedicalCode)
                //             ?.OrderBy(x => x.CreateTime)?.ToList() ?? new List<RemedicalMeasuredInfoDTO>();
                //         foreach (var itemRemedicalMeasuredInfo in itemRemedicalMeasuredInfos1)
                //         {
                //             sampleRemedicalInfoList.Add(ConvertToSampleRemedical(itemRemedicalMeasuredInfo));
                //         }
                //     }
                // }
            }
            return sampleRemedicalInfoList;
        }

        /// <summary>
        /// GetSampleDetailDBList
        /// </summary>
        /// <param name="patientCode"></param>
        /// <returns></returns>
        public async Task<IList<SampleDetail>> GetSampleDetailDBList(IList<string> patientCodes)
        {
            var sampleDetailList = new List<SampleDetail>();
            var records = await GetPatientRecordsDBAsync(patientCodes);
            if (records != null && records.Count > 0)
            {
                var remedicalDatas = await GetRecordRemedicalsByRecordsAsync(records.Select(x => x.RecordCode).ToList(), DateTime.MinValue, DateTime.MinValue);
                var remedicalMeasuredInfos = new List<RemedicalMeasuredInfoDTO>();
                //查询测量数据
                if (remedicalDatas != null && remedicalDatas.Count > 0)
                {
                    remedicalDatas = remedicalDatas.OrderBy(x => x.CreateTime).ToList();
                    remedicalMeasuredInfos = await GetRemedicalMeasuredInfosByRemedicalCodesAsync(remedicalDatas.Select(x => x.RemedicalCode).ToList());
                }
                foreach (var record in records)
                {
                    var sampleDetail = new SampleDetail();
                    sampleDetail.RecordCode = record.RecordCode;
                    sampleDetail.RecordTime = record.CreateTime;
                    sampleDetail.PatientInfoExtList = record.PatientInfoExtList;
                    sampleDetail.PatientInfo = record.PatientInfo;
                    sampleDetail.SampleRemedicalInfoList = new List<SampleRemedicalInfoDTO>();

                    var recordRemedicalDatas = remedicalDatas.Where(x => x.RecordCode == record.RecordCode)?.ToList() ?? new List<RemedicalInfoDTO>();
                    if (recordRemedicalDatas != null && recordRemedicalDatas.Count > 0)
                    {
                        foreach (var remedical in recordRemedicalDatas)
                        {
                            sampleDetail.SampleRemedicalInfoList.Add(ConvertToSampleRemedical(remedical));

                            var videoList = ConvertVideoToSampleRemedical(remedical);
                            foreach (var item in videoList)
                            {
                                sampleDetail.SampleRemedicalInfoList.Add(item);
                            }
                            var itemRemedicalMeasuredInfos = remedicalMeasuredInfos.Where(x => x.RemedicalCode == remedical.RemedicalCode)
                                        ?.OrderBy(x => x.CreateTime)?.ToList() ?? new List<RemedicalMeasuredInfoDTO>();
                            foreach (var itemRemedicalMeasuredInfo in itemRemedicalMeasuredInfos)
                            {
                                sampleDetail.SampleRemedicalInfoList.Add(ConvertToSampleRemedical(itemRemedicalMeasuredInfo));
                            }
                        }
                        var sampleRemedicalList = new List<SampleRemedicalInfoDTO>();
                        var groupUrms = sampleDetail.SampleRemedicalInfoList.GroupBy(x => x.UrmOriginalCode).OrderBy(x => x.OrderBy(r => r.CreateTime).FirstOrDefault().CreateTime).ToList();
                        foreach (var group in groupUrms)
                        {
                            var groupItems = group.OrderByDescending(x => x.IsURM && !x.IsAnalysisResult).ThenByDescending(x => x.IsURM).ThenByDescending(x => x.SampleRemedicalMeasuredList == null).ThenBy(x => x.CreateTime).ToList();
                            sampleRemedicalList.AddRange(groupItems);
                        }
                        foreach (var item in sampleRemedicalList)
                        {
                            if (item.UrmOriginalCode == item.BusinessCode)
                            {
                                item.UrmOriginalCode = string.Empty;
                            }
                        }
                        sampleDetail.SampleRemedicalInfoList = sampleRemedicalList;
                        sampleDetail.DataCount = sampleDetail.SampleRemedicalInfoList.Count;
                    }
                    sampleDetailList.Add(sampleDetail);
                }
            }
            return sampleDetailList;
        }


        /// <summary>
        /// 转入数据接口
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> TransferDataDBAsync(TransferDataDBRequest request)
        {
            var patientCodes = request.PatientCodes;
            var projectCode = request.ProjectCode;
            var userCode = request.UserCode;
            var userName = request.UserName;
            //转项目，小动物科研的只会有小动物类型不会有人用类型，todo在创建项目的时候就要注意
            var patients = await FindPatientInfoListByCodeListAsync(patientCodes);
            var records = await FindRecordInfosByPatientCodesAsync(patientCodes, DateTime.MinValue, DateTime.MinValue);
            var remedicalDatas = await GetRecordRemedicalsByRecordsAsync(records.Select(x => x.RecordCode).ToList(), DateTime.MinValue, DateTime.MinValue);
            foreach (var patient in patients)
            {
                patient.ProjectCode = projectCode;
                await UpdatePatientsAsync(patient.PatientCode, patient, null);
            }
            foreach (var record in records)
            {
                record.ProjectCode = projectCode;
                await UpdateRecordInfoByCodeAsync(record.RecordCode, record, null);
            }
            foreach (var remedical in remedicalDatas)
            {
                remedical.ProjectCode = projectCode;
                await ModifyRemedicalInfoAsync(remedical.RemedicalCode, remedical, null);
            }
            //转出时间轴
            var task = new TaskEntity()
            {
                BusinessId = "",
                TaskName = "",
                MainTaskCode = "",
                Description = "",
                CreatorName = request.UserName,
                OperatorCode = request.UserCode,
                OperatorName = request.UserName,
                OperateTime = DateTime.UtcNow,
                Status = VTaskStatus.Completed,
                TaskType = VTaskType.TransferOut,
                CreateTime = DateTime.UtcNow,
                UpdateTime = DateTime.UtcNow,
                BusinessCode = "BaseVetProject",
                JsonData = string.Join(",", patients.Select(x => x.Name))
            };
            await _taskDBRepository.InsertOneAsync(task);
            //转入时间轴
            var taskIn = new TaskEntity()
            {
                BusinessId = "",
                TaskName = "",
                MainTaskCode = "",
                Description = "",
                CreatorName = request.UserName,
                OperatorCode = request.UserCode,
                OperatorName = request.UserName,
                OperateTime = DateTime.UtcNow,
                Status = VTaskStatus.Completed,
                TaskType = VTaskType.TransferIn,
                CreateTime = DateTime.UtcNow,
                UpdateTime = DateTime.UtcNow,
                BusinessCode = projectCode,
                JsonData = string.Join(",", patients.Select(x => x.Name))
            };
            await _taskDBRepository.InsertOneAsync(taskIn);
            return true;
        }

        private SampleRemedicalInfoDTO ConvertToSampleRemedical(RemedicalInfoDTO remedicalData)
        {
            var sampleRemedicalInfoDTO = new SampleRemedicalInfoDTO();
            sampleRemedicalInfoDTO.IsURM = remedicalData.ImgList.Any(x => x.ImgType == 0);
            sampleRemedicalInfoDTO.ImgList = remedicalData.ImgList;
            sampleRemedicalInfoDTO.IsAnalysisResult = remedicalData.ImgList.Any(x => x.ImgType != 0);
            sampleRemedicalInfoDTO.Application = remedicalData.Application;
            sampleRemedicalInfoDTO.ApplicationCategory = remedicalData.ApplicationCategory;
            sampleRemedicalInfoDTO.BusinessCode = remedicalData.RemedicalCode;
            sampleRemedicalInfoDTO.CreateTime = remedicalData.CreateTime;
            sampleRemedicalInfoDTO.FileDataType = (SampleRemedicalFileDataTypeEnum)remedicalData.FileDataType.GetHashCode();
            sampleRemedicalInfoDTO.ImageSize = remedicalData.TerminalImages.ImageSize;
            sampleRemedicalInfoDTO.ImageUrl = remedicalData.TerminalImages.OriginImageUrl.ToLocalUrl();
            sampleRemedicalInfoDTO.CoverImgUrl = remedicalData.TerminalImages.CoverImageUrl.ToLocalUrl();
            sampleRemedicalInfoDTO.PreviewUrl = remedicalData.TerminalImages.PreviewUrl.ToLocalUrl();
            sampleRemedicalInfoDTO.UrmOriginalCode = string.IsNullOrWhiteSpace(remedicalData.UrmOriginalCode) ? remedicalData.RemedicalCode : remedicalData.UrmOriginalCode;
            sampleRemedicalInfoDTO.RemedicalOriginalCode = remedicalData.RemedicalOriginalCode;
            if (sampleRemedicalInfoDTO.ImgList?.Count > 0)
            {
                foreach (var item in sampleRemedicalInfoDTO.ImgList)
                {
                    item.FileToken = item.FileToken.ToLocalUrl();
                    item.CoverImgUrl = item.CoverImgUrl.ToLocalUrl();
                    item.PreviewFileUrl = item.PreviewFileUrl.ToLocalUrl();
                }
            }
            return sampleRemedicalInfoDTO;
        }

        private List<SampleRemedicalInfoDTO> ConvertVideoToSampleRemedical(RemedicalInfoDTO remedicalData)
        {
            var videoList = new List<SampleRemedicalInfoDTO>();
            if (remedicalData.ImgList != null && remedicalData.ImgList.Count > 0)
            {
                foreach (var img in remedicalData.ImgList)
                {
                    if (!img.IsShow)
                    {
                        continue;
                    }
                    var imgSampleRemedicalInfoDTO = new SampleRemedicalInfoDTO();
                    imgSampleRemedicalInfoDTO.Application = remedicalData.Application;
                    imgSampleRemedicalInfoDTO.ApplicationCategory = remedicalData.ApplicationCategory;
                    imgSampleRemedicalInfoDTO.BusinessCode = remedicalData.RemedicalCode + img.ImgType + img.CreateTime.ToString("yyyyMMddHHmmssfff");
                    imgSampleRemedicalInfoDTO.CreateTime = img.CreateTime < new DateTime(2000, 1, 1) ? remedicalData.CreateTime : img.CreateTime;
                    imgSampleRemedicalInfoDTO.FileDataType = GetSampleRemedicalFileDataTypeEnumByImgType(img.ImgType);
                    imgSampleRemedicalInfoDTO.ImageSize = img.FileSize;
                    imgSampleRemedicalInfoDTO.ImageUrl = img.FileToken.ToLocalUrl();
                    imgSampleRemedicalInfoDTO.PreviewUrl = img.PreviewFileUrl.ToLocalUrl();
                    imgSampleRemedicalInfoDTO.CoverImgUrl = img.CoverImgUrl.ToLocalUrl();
                    imgSampleRemedicalInfoDTO.UrmOriginalCode = string.IsNullOrWhiteSpace(remedicalData.UrmOriginalCode) ? remedicalData.RemedicalCode : remedicalData.UrmOriginalCode;
                    imgSampleRemedicalInfoDTO.RemedicalOriginalCode = remedicalData.RemedicalCode;
                    if (imgSampleRemedicalInfoDTO?.ImgList?.Count > 0)
                    {
                        foreach (var item in imgSampleRemedicalInfoDTO.ImgList)
                        {
                            item.FileToken = item.FileToken.ToLocalUrl();
                            item.CoverImgUrl = item.CoverImgUrl.ToLocalUrl();
                            item.PreviewFileUrl = item.PreviewFileUrl.ToLocalUrl();
                        }
                    }
                    videoList.Add(imgSampleRemedicalInfoDTO);
                }
            }
            return videoList;
        }

        private SampleRemedicalInfoDTO ConvertToSampleRemedical(RemedicalMeasuredInfoDTO itemRemedicalMeasuredInfo)
        {
            var measuredSampleRemedicalInfoDTO = new SampleRemedicalInfoDTO();
            //measuredSampleRemedicalInfoDTO.Application = remedicalData.Application;
            //measuredSampleRemedicalInfoDTO.ApplicationCategory = remedicalData.ApplicationCategory;
            measuredSampleRemedicalInfoDTO.BusinessCode = itemRemedicalMeasuredInfo.RemedicalMeasuredInfoCode;
            measuredSampleRemedicalInfoDTO.CreateTime = itemRemedicalMeasuredInfo.CreateTime;
            measuredSampleRemedicalInfoDTO.FileDataType = SampleRemedicalFileDataTypeEnum.MeasurementImage;
            measuredSampleRemedicalInfoDTO.ImageUrl = itemRemedicalMeasuredInfo.MeasuredFileToken.ToLocalUrl();
            measuredSampleRemedicalInfoDTO.PreviewUrl = itemRemedicalMeasuredInfo.PreviewFileToken.ToLocalUrl();
            measuredSampleRemedicalInfoDTO.CoverImgUrl = itemRemedicalMeasuredInfo.MeasuredFileToken.ToLocalUrl();
            measuredSampleRemedicalInfoDTO.UrmOriginalCode = string.IsNullOrWhiteSpace(itemRemedicalMeasuredInfo.UrmOriginalCode) ? itemRemedicalMeasuredInfo.RemedicalCode : itemRemedicalMeasuredInfo.UrmOriginalCode;
            measuredSampleRemedicalInfoDTO.RemedicalOriginalCode = itemRemedicalMeasuredInfo.RemedicalOriginalCode;
            measuredSampleRemedicalInfoDTO.SampleRemedicalMeasuredList = new List<SampleRemedicalMeasuredInfoDTO>()
            {
                new SampleRemedicalMeasuredInfoDTO() {
                    MeasuredData = itemRemedicalMeasuredInfo.MeasuredData
                }
            };
            return measuredSampleRemedicalInfoDTO;
        }

        public SampleRemedicalFileDataTypeEnum GetSampleRemedicalFileDataTypeEnumByImgType(int imgType)
        {
            //0-URM源文件，1-URM分析Den图像，2-URM分析Dir图像，3-URM分析Vel图像，4-URM分析视频
            if (imgType == 4)
            {
                return SampleRemedicalFileDataTypeEnum.AnalysisVideo;
            }
            else
            {
                return SampleRemedicalFileDataTypeEnum.MeasurementImage;
            }
        }

        /// <summary>
        /// 根据项目查询时间轴
        /// </summary>
        /// <returns></returns>
        public async Task<IList<TimeAxisDetail>> GetTimeAxisByProjectDBAsync(string projectCode)
        {
            var list = new List<TimeAxisDetail>();
            var builder = Builders<ResearchProjectEntity>.Filter;
            var filter = builder.Where(c => c.IsDelete == false && c.Code == projectCode);
            //项目，项目创建
            var projectInfo = await _researchProjectDBRepository.FindOneAsync(filter);
            var projectCreateTimeAxisDetail = new TimeAxisDetail();
            projectCreateTimeAxisDetail.OperatorName = projectInfo.CreatorName;
            projectCreateTimeAxisDetail.OperateTime = projectInfo.CreateTime;
            projectCreateTimeAxisDetail.TimeAxisType = TimeAxisDetailEnum.ProjectCreate;
            list.Add(projectCreateTimeAxisDetail);
            //超声机图像，超声机上传
            var remedicals = await GetRemedicalsByProjectCodeAsync(projectCode);
            var records = await FindRecordInfosByProjectCodeAsync(projectCode);
            var devides = await FindDeviceInfoListByCodeListAsync(remedicals.Select(x => x.DeviceCode).ToList());
            foreach (var item in remedicals)
            {
                //设备上传的才有设备编码
                if (string.IsNullOrWhiteSpace(item.DeviceCode))
                {
                    continue;
                }
                var record = records.FirstOrDefault(x => x.RecordCode == item.RecordCode);
                if (record == null || string.IsNullOrWhiteSpace(record.RecordCode))
                {
                    continue;//检查已被删除
                }
                var deviceUploadTimeAxisDetail = new TimeAxisDetail();
                deviceUploadTimeAxisDetail.OperatorName = devides.FirstOrDefault(x => x.DeviceCode == record?.DeviceCode)?.DisplayName;
                deviceUploadTimeAxisDetail.OperateTime = item.CreateTime;
                deviceUploadTimeAxisDetail.TimeAxisType = TimeAxisDetailEnum.DeviceUpload;
                var deviceUploadTimeAxisJson = new DeviceUploadTimeAxisJson();
                deviceUploadTimeAxisJson.PatientCode = record?.PatientCode ?? string.Empty;
                deviceUploadTimeAxisJson.RecordCode = item.RecordCode;
                deviceUploadTimeAxisJson.RemedicalCode = item.RemedicalCode;
                deviceUploadTimeAxisDetail.DataJson = JsonConvert.SerializeObject(deviceUploadTimeAxisJson);
                list.Add(deviceUploadTimeAxisDetail);
            }
            //邀请用户、移除用户、删除样本、删除检查、修改项目，转出，转入
            var tasks = await GetTaskListByBusinessCodeDBAsync(projectCode);
            var userTasks = tasks.Where(x => x.TaskType == VTaskType.InviteUser || x.TaskType == VTaskType.RemoveUser || x.TaskType == VTaskType.DeleteRecord || x.TaskType == VTaskType.DeleteSample || x.TaskType == VTaskType.ProjectUpdate || x.TaskType == VTaskType.TransferOut || x.TaskType == VTaskType.TransferIn);
            foreach (var item in userTasks)
            {
                var userTimeAxisDetail = new TimeAxisDetail();
                userTimeAxisDetail.OperatorName = item.CreatorName;
                userTimeAxisDetail.OperateTime = item.CreateTime;
                if (item.TaskType == VTaskType.InviteUser)
                {
                    userTimeAxisDetail.TimeAxisType = TimeAxisDetailEnum.InviteUser;
                }
                else if (item.TaskType == VTaskType.RemoveUser)
                {
                    userTimeAxisDetail.TimeAxisType = TimeAxisDetailEnum.RemoveUser;
                }
                else if (item.TaskType == VTaskType.DeleteSample)
                {
                    userTimeAxisDetail.TimeAxisType = TimeAxisDetailEnum.DeleteSample;
                }
                else if (item.TaskType == VTaskType.DeleteRecord)
                {
                    userTimeAxisDetail.TimeAxisType = TimeAxisDetailEnum.DeleteRecord;
                }
                else if (item.TaskType == VTaskType.ProjectUpdate)
                {
                    userTimeAxisDetail.TimeAxisType = TimeAxisDetailEnum.ProjectUpdate;
                }
                else if (item.TaskType == VTaskType.TransferOut)
                {
                    userTimeAxisDetail.TimeAxisType = TimeAxisDetailEnum.TransferOut;
                }
                else if (item.TaskType == VTaskType.TransferIn)
                {
                    userTimeAxisDetail.TimeAxisType = TimeAxisDetailEnum.TransferIn;
                }
                if (userTimeAxisDetail.TimeAxisType == TimeAxisDetailEnum.InviteUser || userTimeAxisDetail.TimeAxisType == TimeAxisDetailEnum.RemoveUser)
                {
                    var userCodes = item.JsonData.Split(',').ToList();
                    var users = await FindUsersByUserCodesAsync(userCodes);
                    var inviteUserTimeAxisJsons = new List<InviteUserTimeAxisJson>();
                    if (users != null && users.Count() > 0)
                    {
                        foreach (var user in users)
                        {
                            var inviteUserTimeAxisJson = new InviteUserTimeAxisJson();
                            inviteUserTimeAxisJson.UserCode = user.UserCode;
                            inviteUserTimeAxisJson.Name = !string.IsNullOrWhiteSpace(user.FullName) ? user.FullName : user.UserName;
                            inviteUserTimeAxisJsons.Add(inviteUserTimeAxisJson);
                        }
                    }
                    userTimeAxisDetail.DataJson = JsonConvert.SerializeObject(inviteUserTimeAxisJsons);
                }
                else
                {
                    userTimeAxisDetail.DataJson = item.JsonData;
                }
                list.Add(userTimeAxisDetail);
            }
            //用户导入、用户导出、分析
            var mainTasks = await GetMainTaskListByProjectDBAsync(new GetTaskListByProjectRequest { ProjectCode = projectCode });
            var tempTasks = await GetTaskListByMainTaskCodesDBAsync(mainTasks.Select(x => x.Code).ToList());
            foreach (var mainTask in mainTasks)
            {
                if (mainTask.TaskType == VTaskType.Analysis)//分析
                {
                    var startanalysisTimeAxisDetail = new TimeAxisDetail();
                    startanalysisTimeAxisDetail.OperatorName = mainTask.CreatorName;
                    startanalysisTimeAxisDetail.OperateTime = mainTask.CreateTime;
                    startanalysisTimeAxisDetail.TimeAxisType = TimeAxisDetailEnum.Startanalysis;
                    list.Add(startanalysisTimeAxisDetail);
                    if (mainTask.Status == VTaskStatus.Completed)
                    {
                        var nalysisCompleteTimeAxisDetail = new TimeAxisDetail();
                        nalysisCompleteTimeAxisDetail.OperatorName = mainTask.CreatorName;
                        nalysisCompleteTimeAxisDetail.OperateTime = mainTask.EndTime;
                        nalysisCompleteTimeAxisDetail.TimeAxisType = TimeAxisDetailEnum.NalysisComplete;
                        list.Add(nalysisCompleteTimeAxisDetail);
                    }
                }
                else if (mainTask.TaskType == VTaskType.UserImport)//用户导入
                {
                    if (mainTask.Status == VTaskStatus.Completed)
                    {
                        var userImportTimeAxisDetail = new TimeAxisDetail();
                        userImportTimeAxisDetail.OperatorName = mainTask.OperatorName;
                        userImportTimeAxisDetail.OperateTime = mainTask.OperateTime;
                        userImportTimeAxisDetail.TimeAxisType = TimeAxisDetailEnum.UserImport;
                        userImportTimeAxisDetail.DataJson = tempTasks.Count(x => x.MainTaskCode == mainTask.Code).ToString();
                        list.Add(userImportTimeAxisDetail);
                    }
                }
                else if (mainTask.TaskType == VTaskType.UserExport)//用户导出
                {
                    if (mainTask.Status == VTaskStatus.Completed)
                    {
                        var userExportTimeAxisDetail = new TimeAxisDetail();
                        userExportTimeAxisDetail.OperatorName = mainTask.OperatorName;
                        userExportTimeAxisDetail.OperateTime = mainTask.OperateTime;
                        userExportTimeAxisDetail.TimeAxisType = TimeAxisDetailEnum.UserExport;
                        userExportTimeAxisDetail.DataJson = tempTasks.Count(x => x.MainTaskCode == mainTask.Code).ToString();
                        list.Add(userExportTimeAxisDetail);
                    }
                }
            }
            list = list.OrderByDescending(x => x.OperateTime).ToList();
            return list;
        }

        /// <summary>
        /// 样本数据导出列表db接口
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<ProjectSampleDataExportResult> GetProjectSampleDataExportDBAsync(GetProjectSampleDataExportRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetProjectSampleDataExportDBAsync execute", "DatabaseService"))
            {
                var result = new ProjectSampleDataExportResult();
                var builder = Builders<ResearchProjectEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false);
                var projectInfo = new ResearchProjectEntity();
                if (!string.IsNullOrEmpty(request.ProjectCode))
                {
                    var nameFilter = builder.Where(c => c.Code == request.ProjectCode);
                    filter = filter & nameFilter;
                    projectInfo = await _researchProjectDBRepository.FindOneAsync(filter);
                }
                if (!string.IsNullOrEmpty(projectInfo?.Code))
                {
                    result.CreateTime = projectInfo.CreateTime;
                    result.Name = projectInfo.Name;
                    result.PatientType = projectInfo.PatientType;
                }

                var sampleDetailInfoList = new List<SampleDetailInfo>();
                var records = await GetPatientRecordsDBAsync(request.PatientCodes);
                if (records != null && records.Count > 0)
                {
                    var remedicalDatas = await GetRecordRemedicalsByRecordsAsync(records.Select(x => x.RecordCode).ToList(), DateTime.MinValue, DateTime.MinValue);
                    var remedicalMeasuredInfos = new List<RemedicalMeasuredInfoDTO>();
                    //查询测量数据
                    if (remedicalDatas != null && remedicalDatas.Count > 0)
                    {
                        remedicalDatas = remedicalDatas.OrderBy(x => x.CreateTime).ToList();
                        remedicalMeasuredInfos = await GetRemedicalMeasuredInfosByRemedicalCodesAsync(remedicalDatas.Select(x => x.RemedicalCode).ToList());
                    }
                    foreach (var patientCode in request.PatientCodes)
                    {
                        var recordInfo = records.FirstOrDefault(c => c.PatientCode == patientCode);
                        if (string.IsNullOrEmpty(recordInfo?.PatientCode))
                        {
                            continue;
                        }
                        var sampleDetailInfo = new SampleDetailInfo()
                        {
                            PatientInfoExtList = recordInfo.PatientInfoExtList,
                            PatientInfo = recordInfo.PatientInfo,
                            SampleRemedicalBasicList = new List<SampleRemedicalBasicInfo>()
                        };
                        var patientRecords = records.Where(c => c.PatientCode == patientCode)?.ToList();
                        if (patientRecords?.Count > 0)
                        {
                            foreach (var record in patientRecords)
                            {
                                var sampleRemedicalBasicInfo = new SampleRemedicalBasicInfo()
                                {
                                    RemedicalName = record.CreateTime.ToString("yyyy-MM-dd"),
                                    RecordCode = record.RecordCode,
                                    SampleRemedicalInfoList = new List<SampleRemedicalInfoDTO>()
                                };
                                var filterRemedicalDatas = remedicalDatas.Where(c => c.RecordCode == record.RecordCode)?.ToList();
                                if (filterRemedicalDatas?.Count > 0)
                                {
                                    foreach (var remedicalItem in filterRemedicalDatas)
                                    {
                                        var sampleRemedicalInfoDTO = new SampleRemedicalInfoDTO();
                                        sampleRemedicalInfoDTO.IsURM = remedicalItem.ImgList.Any(x => x.ImgType == 0);
                                        sampleRemedicalInfoDTO.ImgList = remedicalItem.ImgList;
                                        if (sampleRemedicalInfoDTO.ImgList?.Count > 0)
                                        {
                                            foreach (var item in sampleRemedicalInfoDTO.ImgList)
                                            {
                                                item.CoverImgUrl = item.CoverImgUrl.ToLocalUrl();
                                                item.FileToken = item.FileToken.ToLocalUrl();
                                                item.PreviewFileUrl = item.PreviewFileUrl.ToLocalUrl();
                                            }
                                        }
                                        sampleRemedicalInfoDTO.IsAnalysisResult = remedicalItem.ImgList.Any(x => x.ImgType != 0);
                                        sampleRemedicalInfoDTO.Application = remedicalItem.Application;
                                        sampleRemedicalInfoDTO.ApplicationCategory = remedicalItem.ApplicationCategory;
                                        sampleRemedicalInfoDTO.BusinessCode = remedicalItem.RemedicalCode;
                                        sampleRemedicalInfoDTO.CreateTime = remedicalItem.CreateTime;
                                        sampleRemedicalInfoDTO.FileDataType = (SampleRemedicalFileDataTypeEnum)remedicalItem.FileDataType.GetHashCode();
                                        sampleRemedicalInfoDTO.ImageSize = remedicalItem.TerminalImages.ImageSize;
                                        sampleRemedicalInfoDTO.ImageUrl = remedicalItem.TerminalImages.OriginImageUrl.ToLocalUrl();
                                        sampleRemedicalInfoDTO.CoverImgUrl = remedicalItem.TerminalImages.CoverImageUrl.ToLocalUrl();
                                        sampleRemedicalInfoDTO.PreviewUrl = remedicalItem.TerminalImages.PreviewUrl.ToLocalUrl();
                                        sampleRemedicalInfoDTO.SampleRemedicalMeasuredList = new List<SampleRemedicalMeasuredInfoDTO>();
                                        //添加测量项
                                        var itemRemedicalMeasuredInfos = remedicalMeasuredInfos.FindAll(x => x.RemedicalCode == remedicalItem.RemedicalCode);
                                        if (itemRemedicalMeasuredInfos?.Count > 0)
                                        {
                                            foreach (var itemRemedicalMeasuredInfo in itemRemedicalMeasuredInfos)
                                            {
                                                var measuredSampleRemedicalInfoDTO = new SampleRemedicalMeasuredInfoDTO();
                                                measuredSampleRemedicalInfoDTO.RemedicalMeasuredInfoCode = itemRemedicalMeasuredInfo.RemedicalMeasuredInfoCode;
                                                measuredSampleRemedicalInfoDTO.CreateTime = itemRemedicalMeasuredInfo.CreateTime;
                                                measuredSampleRemedicalInfoDTO.FrameIndex = itemRemedicalMeasuredInfo.FrameIndex;
                                                measuredSampleRemedicalInfoDTO.MeasuredFileToken = itemRemedicalMeasuredInfo.MeasuredFileToken.ToLocalUrl();
                                                measuredSampleRemedicalInfoDTO.PreviewFileToken = itemRemedicalMeasuredInfo.PreviewFileToken.ToLocalUrl();
                                                measuredSampleRemedicalInfoDTO.MeasuredData = itemRemedicalMeasuredInfo.MeasuredData;
                                                measuredSampleRemedicalInfoDTO.Model = itemRemedicalMeasuredInfo.Model;
                                                sampleRemedicalInfoDTO.SampleRemedicalMeasuredList.Add(measuredSampleRemedicalInfoDTO);
                                            }
                                        }
                                        sampleRemedicalBasicInfo.SampleRemedicalInfoList.Add(sampleRemedicalInfoDTO);
                                        if (remedicalItem.ImgList != null && remedicalItem.ImgList.Count > 0)
                                        {
                                            foreach (var img in remedicalItem.ImgList)
                                            {
                                                if (!img.IsShow)
                                                {
                                                    continue;
                                                }
                                                var imgSampleRemedicalInfoDTO = new SampleRemedicalInfoDTO();
                                                imgSampleRemedicalInfoDTO.Application = remedicalItem.Application;
                                                imgSampleRemedicalInfoDTO.ApplicationCategory = remedicalItem.ApplicationCategory;
                                                imgSampleRemedicalInfoDTO.BusinessCode = remedicalItem.RemedicalCode + img.ImgType;
                                                imgSampleRemedicalInfoDTO.CreateTime = remedicalItem.CreateTime;
                                                imgSampleRemedicalInfoDTO.FileDataType = GetSampleRemedicalFileDataTypeEnumByImgType(img.ImgType);
                                                imgSampleRemedicalInfoDTO.ImageSize = img.FileSize;
                                                imgSampleRemedicalInfoDTO.ImageUrl = img.FileToken.ToLocalUrl(); ;
                                                imgSampleRemedicalInfoDTO.PreviewUrl = img.PreviewFileUrl.ToLocalUrl(); ;
                                                imgSampleRemedicalInfoDTO.CoverImgUrl = img.CoverImgUrl.ToLocalUrl(); ;
                                                sampleRemedicalBasicInfo.SampleRemedicalInfoList.Add(imgSampleRemedicalInfoDTO);
                                            }
                                        }
                                    }
                                }
                                sampleDetailInfo.SampleRemedicalBasicList.Add(sampleRemedicalBasicInfo);
                            }
                        }
                        sampleDetailInfoList.Add(sampleDetailInfo);
                    }
                }
                result.SampleDetailList = sampleDetailInfoList;
                return result;
            }
        }


        /// <summary>
        /// 查询所有科研病例样本列表
        /// </summary>
        /// <param name="request">病例样本搜索实体</param>
        /// <returns></returns>
        /// <errorCodes></errorCodes>
        public async Task<PageCollection<PatientSampleDataDTO>> GetAllPatientSamplePagesDBAsync(GetRRUserPagesRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetAllPatientSamplePagesDBAsync execute", "DatabaseService"))
            {
                var dbPage = new PageCollection<PatientSampleDataDTO>();
                dbPage.CurrentPage = request.PageIndex;
                dbPage.PageSize = request.PageSize;
                dbPage.PageData = new List<PatientSampleDataDTO>();
                var builder = Builders<ResearchProjectEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false);
                var projectList = await _researchProjectDBRepository.FindAllAsync(filter);
                if (projectList?.Count > 0)
                {
                    var keyword = request.KeyWord;
                    var keywordfilter = new Dictionary<string, string>();
                    if (!string.IsNullOrWhiteSpace(keyword))
                    {
                        var tempKeyWord = CryptionHelper.EncryptOneByOne(keyword.Trim());
                        keywordfilter.Add("Name", tempKeyWord);
                        keywordfilter.Add("Phone", tempKeyWord);
                        keywordfilter.Add("IdentityCard", tempKeyWord);
                    }
                    var tempProjectCodes = projectList.Select(c => c.Code).Distinct().ToList();
                    var pageFilterRequest = new GetPatientsByOrgPageRequest()
                    {
                        KeywordOrFilter = keywordfilter,
                        CurrentPage = request.PageIndex,
                        PageSize = request.PageSize,
                        IsFuzzy = true,
                        ProjectCodes = tempProjectCodes
                    };
                    var sort = Builders<MongoTable>.Sort.Descending("BaseData.CreateTime");
                    var patients = await GetPatientPagesAsync(pageFilterRequest, sort);
                    if (patients != null && patients.PageData.Count > 0)
                    {
                        dbPage.DataCount = patients.DataCount;
                        var projectCodes = patients.PageData.Where(c => !string.IsNullOrEmpty(c.ProjectCode)).Select(c => c.ProjectCode).Distinct().ToList();
                        //查询所有的项目
                        var records = await FindRecordInfosByProjectCodesAsync(projectCodes);
                        var remedicals = await GetRemedicalsByProjectCodesAsync(projectCodes);
                        foreach (var item in patients.PageData)
                        {
                            var researchProjectDataDTO = new PatientSampleDataDTO();
                            var itemRecords = records.Where(x => x.PatientCode == item.PatientCode);
                            if (itemRecords != null && itemRecords.Count() > 0)
                            {
                                var recordCodes = itemRecords.Select(x => x.RecordCode);
                                if (recordCodes != null && recordCodes.Count() > 0)
                                {
                                    var itemRemedicals = remedicals.Where(x => recordCodes.Contains(x.RecordCode));
                                    var imgList = itemRemedicals.SelectMany(x => x.ImgList.Where(x => x.ImgType == 0 || x.IsShow == true));
                                    var fileSize = imgList.DistinctBy(x => x.FileToken).Sum(i => i.FileSize);
                                    fileSize += itemRemedicals.DistinctBy(x => x.TerminalImages.OriginImageUrl).Sum(x => x.TerminalImages.ImageSize);
                                    researchProjectDataDTO.FileSize = fileSize.ToString();
                                }
                            }
                            researchProjectDataDTO.CreateTime = item.CreateTime;
                            researchProjectDataDTO.DevicePatientId = item.DevicePatientIDs.FirstOrDefault();
                            researchProjectDataDTO.PatientCode = item.PatientCode;
                            researchProjectDataDTO.PatientName = CryptionHelper.Decrypt(item.Name);
                            var findProjectInfo = projectList.FirstOrDefault(c => c.Code == item.ProjectCode) ?? new ResearchProjectEntity();
                            researchProjectDataDTO.ProjectName = findProjectInfo.Name;
                            researchProjectDataDTO.ProjectCode = findProjectInfo.Code;
                            researchProjectDataDTO.CreatorName = findProjectInfo.CreatorName;
                            dbPage.PageData.Add(researchProjectDataDTO);
                        }
                    }
                }
                return dbPage;
            }
        }

        /// <summary>
        /// 删除指定样本数据
        /// </summary>
        /// <param name="request">病例样本搜索实体</param>
        /// <returns></returns>
        /// <errorCodes></errorCodes>
        public async Task<bool> DeletePatientSampleDataDBAsync(DeletePatientSampleRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} DeletePatientSampleDataDBAsync execute", "DatabaseService"))
            {
                //先找到病人，删除
                var patientFilter = new Dictionary<string, string>();
                patientFilter.Add("Code", request.PatientCode);
                patientFilter.Add("ProjectCode", request.ProjectCode);
                //查询设备基础信息
                bool isNeedDelete = true;
                var patientList = await _patientInfoDBRepository.FindListAsync(patientFilter);
                if (patientList?.Count > 0)
                {
                    foreach (var item in patientList)
                    {
                        isNeedDelete = await _patientInfoDBRepository.DeleteOneAsync("Code", item.Code);
                    }
                }
                if (isNeedDelete)
                {
                    var task = new TaskEntity()
                    {
                        BusinessId = "",
                        TaskName = "",
                        MainTaskCode = "",
                        Description = "",
                        CreatorName = request.UserName,
                        OperatorCode = request.UserCode,
                        OperatorName = request.UserName == "" ? "管理员" : request.UserName,
                        OperateTime = DateTime.UtcNow,
                        Status = VTaskStatus.Completed,
                        TaskType = VTaskType.DeleteSample,
                        CreateTime = DateTime.UtcNow,
                        UpdateTime = DateTime.UtcNow,
                        BusinessCode = request.ProjectCode,
                        JsonData = string.Join(",", patientList.Select(x => x.Name))
                    };
                    await _taskDBRepository.InsertOneAsync(task);
                    //找到记录，删除，
                    var records = await FindRecordInfosByProjectCodeAsync(request.ProjectCode);
                    var itemRecords = records?.Where(x => x.PatientCode == request.PatientCode)?.ToList();
                    var recordCodes = new List<string>();
                    if (itemRecords?.Count > 0)
                    {
                        foreach (var item in itemRecords)
                        {
                            recordCodes.Add(item.RecordCode);
                            await _recordInfoDBRepository.DeleteOneAsync("Code", item.RecordCode);
                        }
                    }
                    //找到remed删除
                    var remedicals = await GetRemedicalsByProjectCodeAsync(request.ProjectCode);
                    var itemRemedicals = remedicals?.Where(x => recordCodes.Contains(x.RecordCode))?.ToList();
                    if (itemRemedicals != null && itemRemedicals.Count > 0)
                    {
                        var secondDirectory = GetSecondDirectory();
                        foreach (var item in itemRemedicals)
                        {
                            try
                            {
                                var originImageFilePath = GetFilePath(item.TerminalImages.OriginImageUrl, secondDirectory);
                                DeleteFile(originImageFilePath);
                                var imageFilePath = GetFilePath(item.TerminalImages.ImageUrl, secondDirectory);
                                DeleteFile(imageFilePath);
                                var coverImageFilePath = GetFilePath(item.TerminalImages.CoverImageUrl, secondDirectory);
                                DeleteFile(coverImageFilePath);
                                var previewFilePath = GetFilePath(item.TerminalImages.PreviewUrl, secondDirectory);
                                DeleteFile(previewFilePath);
                                foreach (var imgDetail in item.ImgList)
                                {
                                    var imgDetailFilePath = GetFilePath(imgDetail.FileToken, secondDirectory);
                                    DeleteFile(imgDetailFilePath);
                                }
                                await _remedicalDBRepository.DeleteOneAsync("Code", item.RemedicalCode);
                            }
                            catch (Exception ex)
                            {
                                Logger.WriteLineError($"patient sample delete file error:{item.RemedicalCode},{ex}");
                            }
                        }
                        //找到测量删除
                        var remedicalMeasuredInfos = await GetRemedicalMeasuredInfosByRemedicalCodesAsync(remedicals.Select(x => x.RemedicalCode).ToList());
                        if (remedicalMeasuredInfos != null && remedicalMeasuredInfos.Count > 0)
                        {
                            foreach (var item in remedicalMeasuredInfos)
                            {
                                try
                                {
                                    var measuredFilePath = GetFilePath(item.MeasuredFileToken, secondDirectory);
                                    DeleteFile(measuredFilePath);
                                    var previewFilePath = GetFilePath(item.PreviewFileToken, secondDirectory);
                                    DeleteFile(previewFilePath);
                                    await _remedicalMeasuredInfoDBRepository.DeleteOneAsync("Code", item.RemedicalMeasuredInfoCode);
                                }
                                catch (Exception ex)
                                {
                                    Logger.WriteLineError($"patient sample delete file remedicalMeasuredInfos error:{item.RemedicalMeasuredInfoCode},{ex}");
                                }
                            }
                        }
                    }
                }
                return true;
            }
        }
    }
}