using System;
using System.Collections.Generic;
using System.Threading.Tasks;
using WingMongoDBModule.DO;
using WingInterfaceLibrary.DB.Request;
using WingInterfaceLibrary.Interface.DBInterface;
using WingInterfaceLibrary.DTO.Role;
using WingServerCommon.Service;
using WingInterfaceLibrary.Enum;
using WingServerCommon.Mapper;
using WingInterfaceLibrary.Request;
using System.Linq;
using WingServerCommon.Utilities;

namespace WingMongoDBModule.Service
{
    public partial class DatabaseService : JsonRpcService, IRoleDBService
    {
        /// <summary>
        /// 通过代码删除角色
        /// </summary>
        /// <param name="code">要删除的角色的代码。</param>
        /// <returns>
        /// 一个布尔值。
        /// </returns>
        public async Task<bool> DeleteRoleByCodeAsync(string code)
        {
            using (new Performance(_performanceThreshold, "{0} DeleteRoleByCodeAsync execute", $"{this.GetType().Name}"))
            {
                return await _roleDBRepository.DeleteOneAsync("Code", code);
            }
        }

        /// <summary>
        /// > 通过代码查找角色
        /// </summary>
        /// <param name="code">要查询的角色代码</param>
        /// <returns>
        /// RoleDTO 对象
        /// </returns>
        public async Task<RoleDTO> FindRoleByCodeAsync(string code)
        {
            using (new Performance(_performanceThreshold, "{0} FindRoleByCodeAsync execute", $"{this.GetType().Name}"))
            {
                var roleDO = await _roleDBRepository.FindOneAsync("Code", code);
                return roleDO.MappingTo<RoleDTO>();
            }
        }

        /// <summary>
        /// > 按名称查找角色
        /// </summary>
        /// <param name="name">要查找的角色的名称。</param>
        /// <returns>
        /// RoleDTO 对象
        /// </returns>
        public async Task<RoleDTO> FindRoleByNameAsync(string name)
        {
            using (new Performance(_performanceThreshold, "{0} FindRoleByNameAsync execute", $"{this.GetType().Name}"))
            {
                var roleDO = await _roleDBRepository.FindOneAsync("RoleName", name);
                return roleDO.MappingTo<RoleDTO>();
            }
        }


        /// <summary>
        /// 查找所有角色，按角色名过滤，返回分页结果
        /// </summary>
        /// <param name="PageFilterRequest">request参数为分页请求参数，包含当前页码、每页记录数、过滤条件。</param>
        /// <returns>
        /// RoleDTO 的 PageCollection
        /// </returns>
        public async Task<PageCollection<RoleDTO>> FindRolePagesAsync(PageFilterRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} FindRolePagesAsync execute", $"{this.GetType().Name}"))
            {
                var roles = await _roleDBRepository.FindAllAsync();
                if (request.Filter.TryGetValue("RoleName", out string roleName) && !string.IsNullOrWhiteSpace(roleName))
                {
                    roles = roles.Where(role =>
                    {
                        if (role.LanguageConfigs == null || !role.LanguageConfigs.Any())
                        {
                            return role.RoleName.Contains(roleName, StringComparison.OrdinalIgnoreCase);
                        }
                        else
                        {
                            return role.LanguageConfigs.Any(x => x.RoleName.Contains(roleName, StringComparison.OrdinalIgnoreCase));
                        }
                    })?.ToList() ?? new();
                }
                var totalCount = roles.Count;
                var pageResult = new PageCollection<UserRoleDO>();
                pageResult.CurrentPage = request.CurrentPage;
                pageResult.PageSize = request.PageSize;
                pageResult.DataCount = totalCount;
                pageResult.PageData = roles.Skip(request.PageSize * (request.CurrentPage - 1)).Take(request.PageSize)?.ToList();
                return PageCollectionConvert<UserRoleDO, RoleDTO>(pageResult);
            }
        }

        /// <summary>
        /// 插入角色
        /// </summary>
        /// <param name="RoleDO">角色信息</param>
        /// <param name="extensionData">扩展信息</param>
        /// <returns></returns>
        public async Task<string> InsertRoleAsync(CreateRoleDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} InsertRoleAsync execute", $"{this.GetType().Name}"))
            {
                return await _roleDBRepository.InsertOneAsync(request.Data.MappingTo<UserRoleDO>(), request.PlatformDatas, request.ExtensionData);
            }
        }

        /// <summary>
        /// 更新角色信息
        /// </summary>
        /// <param name="code">角色编码</param>
        /// <param name="data">角色数据</param>
        /// <param name="extensionData">扩展信息</param>
        /// <returns></returns>
        public async Task<bool> UpdateRoleByCodeAsync(string code, RoleDTO data, string extensionData)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateRoleByCodeAsync execute", $"{this.GetType().Name}"))
            {
                return await _roleDBRepository.UpdateOneAsync("Code", code, data.MappingTo<UserRoleDO>(), extensionData);
            }
        }

        /// <summary>
        /// 根据角色类型查找
        /// </summary>
        /// <param name="type">角色类型</param>
        /// <param name="organizationCode">组织编码</param>
        /// <returns></returns>
        public async Task<List<RoleDTO>> FindRolesByTypeAsync(RoleShowTypeEnum type, string organizationCode)
        {
            using (new Performance(_performanceThreshold, "{0} FindRolesByTypeAsync execute", $"{this.GetType().Name}"))
            {
                var dic = new Dictionary<string, string>();
                dic.Add("RoleShowType", type.ToString());
                if (!string.IsNullOrWhiteSpace(organizationCode))
                {
                    dic.Add("OrganizationCode", organizationCode);
                }
                var roleList = await _roleDBRepository.FindListAsync(dic);
                return roleList.MappingTo<List<RoleDTO>>();
            }
        }


        /// <summary>
        /// 按角色代码查找角色列表
        /// </summary>
        /// <param name="roleCodes">需要查询的角色代码列表</param>
        /// <returns>
        /// RoleDTO 对象的列表。
        /// </returns>
        public async Task<List<RoleDTO>> FindRoleListByRoleCodeAsync(List<string> roleCodes)
        {
            using (new Performance(_performanceThreshold, "{0} FindRoleListByRoleCodeAsync execute", $"{this.GetType().Name}"))
            {
                var dic = new Dictionary<string, List<string>>();
                dic.Add("Code", roleCodes);
                var roleList = await _roleDBRepository.FindInAsync(dic);
                return roleList.MappingTo<List<RoleDTO>>();
            }
        }
    }
}