using System.Collections.Generic;
using System.Threading.Tasks;
using WingServerCommon.Service;
using WingServerCommon.Mapper;
using System.Linq;
using WingMongoDBModule.DO;
using WingInterfaceLibrary.DB.Request;
using WingMongoDBModule.Mongodb.Driver;
using WingMongoDBModule.Entity;
using WingMongoDBModule.Request;
using WingInterfaceLibrary.DTO.Share;
using MongoDB.Driver;
using WingServerCommon.Config;
using System.Text;
using WingInterfaceLibrary.Interface.DBInterface;
using WingServerCommon.Config.Parameters;
using WingServerCommon.Utilities;

namespace WingMongoDBModule.Service
{
    public partial class DatabaseService : JsonRpcService, IShareDBService
    {
        private static string _lastCode = string.Empty;
        private readonly object _addLocker = new object();
        private string _digitNumbers = "0123456789abcdefghijkmnpqrstuvwxy";


        /// <summary>
        /// 新增分享记录
        /// </summary>
        /// <returns></returns>
        public async Task<string> AddShareInfoAsync(AddShareInfoDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} AddShareInfoAsync execute", $"{this.GetType().Name}"))
            {
                var serverPrefix = GetCurrentPrefix();
                await GetLastCode(serverPrefix);

                string shortCode = string.Empty;
                lock (_addLocker)
                {
                    shortCode = $"{serverPrefix}{GenerateShortCode()}";
                    _lastCode = shortCode;
                }
                var dic = new Dictionary<string, string>();
                dic.Add("ShortCode", shortCode);
                var oldEntities = await _shareInfoDBRepository.FindListAsync(dic);
                if (oldEntities != null && oldEntities.Any())
                {
                    var list = oldEntities.Select(f => ("Code", f.Code)).ToList();
                    await _shareInfoDBRepository.RealDeleteAllAsync(list);
                }

                var entity = new ShareDO
                {
                    ServerPrefix = serverPrefix,
                    ShareContent = request.Data.ShareContent,
                    ShortCode = shortCode,
                };
                var shareCode = await _shareInfoDBRepository.InsertOneAsync(entity, request.PlatformDatas, request.ExtensionData);
                if (!string.IsNullOrWhiteSpace(shareCode))
                {
                    return shortCode;
                }
                return string.Empty;
            }
        }

        /// <summary>
        /// 查询分享记录详情
        /// </summary>
        /// <returns></returns>
        public async Task<ShareDTO> GetShareInfoAsync(GetShareInfoDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetShareInfoAsync execute", $"{this.GetType().Name}"))
            {
                var entity = await _shareInfoDBRepository.FindOneAsync("ShortCode", request.ShortCode.ToLower()) ?? new ShareDO();
                return entity.MappingTo<ShareDTO>();
            }
        }

        /// <summary>
        /// 获取最新分享码
        /// </summary>
        /// <param name="prefix"></param>
        /// <returns></returns>
        private async Task<string> GetLastCode(string prefix)
        {
            if (string.IsNullOrWhiteSpace(_lastCode))
            {
                MongoCustomerFilter<MongoTable> mongoCustomerFilter = new();
                List<(string, string)> eqFilters = new();
                //eqFilters.Add(("IsDelete", "False"));
                eqFilters.Add(("ServerPrefix", prefix));
                mongoCustomerFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, eqFilters.ToArray());

                SortDefinitionBuilder<MongoTable> builderSort = Builders<MongoTable>.Sort;
                SortDefinition<MongoTable> sort = builderSort.Descending("BaseData.CreateTime");

                var filter = mongoCustomerFilter.AndCombine();
                var dbPage = new DBPageRequest();
                dbPage.CurrentPage = 1;
                dbPage.PageSize = 1;
                dbPage.Filter = filter;
                dbPage.Sort = sort;
                var pageResult = await _shareInfoDBRepository.GetPages(dbPage);
                if (pageResult.PageData != null && pageResult.PageData.Any())
                {
                    _lastCode = pageResult.PageData.FirstOrDefault().ShortCode;
                }
            }
            return _lastCode;
        }

        /// <summary>
        /// 生成分享码
        /// </summary>
        /// <returns></returns>
        private string GenerateShortCode()
        {
            var codeLength = 3;
            var uniqueId = new StringBuilder();
            if (string.IsNullOrWhiteSpace(_lastCode))
            {
                for (int i = 0; i < codeLength; i++)
                {
                    uniqueId.Append(_digitNumbers[0]);
                }
            }
            else
            {
                bool needCarry = true;
                var lastCode = _lastCode.Substring(1);
                for (int i = codeLength - 1; i >= 0; i--)
                {
                    if (needCarry)
                    {
                        if (_digitNumbers.IndexOf(lastCode[i]) == _digitNumbers.Length - 1)
                        {
                            uniqueId.Insert(0, _digitNumbers[0]);
                            needCarry = true;
                        }
                        else
                        {
                            uniqueId.Insert(0, _digitNumbers[_digitNumbers.IndexOf(lastCode[i]) + 1]);
                            needCarry = false;
                        }
                    }
                    else
                    {
                        uniqueId.Insert(0, lastCode[i]);
                    }
                }
            }
            return uniqueId.ToString();
        }

        private string GetCurrentPrefix()
        {
            string domains = EnvironmentConfigManager.GetParammeter<StringParameter>("Gateway", "Domains").Value;
            string _serverPrefix = EnvironmentConfigManager.GetParammeter<StringParameter>("Report", "ShareCodePrefix").Value;
            if (!string.IsNullOrWhiteSpace(_serverPrefix) && !string.IsNullOrWhiteSpace(domains))
            {
                var serverPrefixArray = _serverPrefix.Split(',');
                if (serverPrefixArray != null && serverPrefixArray.Length > 0)
                {
                    var currentPrefix = "1";
                    var currentPrefixStr = serverPrefixArray.FirstOrDefault(f => f.Contains(domains.Trim('/')));
                    if (!string.IsNullOrEmpty(currentPrefixStr) && currentPrefixStr.Split('|').Length > 1)
                    {
                        currentPrefix = currentPrefixStr.Split('|')[1];
                    }
                    return currentPrefix;
                }
            }
            return "1";
        }
    }
}