﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Threading.Tasks;
using MongoDB.Driver;
using WingInterfaceLibrary.DTO.Storage;
using WingInterfaceLibrary.Interface.DBInterface;
using WingMongoDBModule.DO;
using WingMongoDBModule.Entities;
using WingServerCommon.Mapper;
using WingServerCommon.Service;
using WingServerCommon.Utilities;

namespace WingMongoDBModule.Service
{
    public partial class DatabaseService : JsonRpcService, IStorageDBService
    {
        /// <summary>
        /// 根据Token获取文件详细信息
        /// </summary>
        /// <param name="token">token</param>
        /// <returns>文件信息</returns>
        public async Task<StorageInfoDTO> GetStorageRouteAsync(string token)
        {
            using (new Performance(_performanceThreshold, "{0} GetStorageRouteAsync execute", $"{this.GetType().Name}"))
            {
                var storageInfoDO = await _storageDBRepository.FindStorageRouteAsync(token);
                return storageInfoDO.MappingTo<StorageInfoDTO>();
            }
        }

        /// <summary>
        /// 根据Token删除文件
        /// </summary>
        /// <param name="token">token</param>
        /// <returns>是否成功</returns>
        public async Task<bool> RemoveAsync(string token)
        {
            using (new Performance(_performanceThreshold, "{0} RemoveAsync execute", $"{this.GetType().Name}"))
            {
                await _storageDBRepository.DeleteByTokenAsync(token);
                return await Task.FromResult(true);
            }
        }

        /// <summary>
        /// 存储文件
        /// </summary>
        /// <param name="token">文件Token</param>
        /// <param name="fileName">文件名</param>
        /// <param name="storageRoute">文件地址</param>
        /// <returns>是否成功</returns>
        public async Task<StorageInfoDTO> StorageAsync(string url)
        {
            using (new Performance(_performanceThreshold, "{0} StorageAsync execute", $"{this.GetType().Name}"))
            {
                var token = Guid.NewGuid().ToString("N");
                var storageInfo = new StorageInfoDO()
                {
                    Token = token,
                    FileName = Path.GetFileName(url),
                    StorageRoute = url
                };
                await _storageDBRepository.InsertOneAsync(storageInfo, new Dictionary<string, string>(), string.Empty);
                return storageInfo.MappingTo<StorageInfoDTO>();
            }
        }

        /// <summary>
        /// 根据Code获取资源计费信息数据
        /// </summary>
        /// <param name="code"></param>
        /// <returns></returns>
        public async Task<bool> RealDeleteMultiPartUploadInfoDBAsync(string uploadId)
        {
            using (new Performance(_performanceThreshold, "{0} RealDeleteMultiPartUploadInfoDBAsync execute", "DatabaseService"))
            {
                var result = await _multiPartUploadInfoDBRepository.RealDeleteOneAsync("UploadId", uploadId);
                return result;
            }
        }

        /// <summary>
        /// 根据Code获取资源计费信息数据
        /// </summary>
        /// <param name="code"></param>
        /// <returns></returns>
        public async Task<MultiPartUploadInfoDTO> GetMultiPartUploadInfoDBAsync(string uploadId)
        {
            using (new Performance(_performanceThreshold, "{0} GetMultiPartUploadInfoDBAsync execute", "DatabaseService"))
            {
                var result = new MultiPartUploadInfoDTO();
                var builder = Builders<MultiPartUploadInfoEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false && c.UploadId == uploadId);
                var findEntity = await _multiPartUploadInfoDBRepository.FindOneAsync(filter);
                if (!string.IsNullOrEmpty(findEntity?.Code))
                {
                    result = findEntity.MappingTo<MultiPartUploadInfoDTO>();
                }
                return result;
            }
        }

        /// <summary>
        /// 保存资源计费信息数据
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> SaveMultiPartUploadInfoDBAsync(MultiPartUploadInfoDTO request)
        {
            using (new Performance(_performanceThreshold, "{0} SaveMultiPartUploadInfoDBAsync execute", "DatabaseService"))
            {
                var result = false;
                var builder = Builders<MultiPartUploadInfoEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false && c.UploadId == request.UploadId);
                var findEntity = await _multiPartUploadInfoDBRepository.FindOneAsync(filter);
                if (!string.IsNullOrEmpty(findEntity?.Code))
                {
                    findEntity.PartFileDirectory = request.PartFileDirectory;
                    findEntity.LastFileDirectory = request.LastFileDirectory;
                    findEntity.LastFileName = request.LastFileName;
                    findEntity.LastFileSize = request.LastFileSize;
                    findEntity.LastFileType = request.LastFileType;
                    findEntity.CurrentPartNumber = request.CurrentPartNumber;
                    findEntity.Bucket = request.Bucket;
                    findEntity.DetailList = request.DetailList;
                    findEntity.UpdateTime = DateTime.UtcNow;
                    result = await _multiPartUploadInfoDBRepository.UpdateOneAsync("Code", findEntity.Code, findEntity, true);
                }
                else
                {
                    var entity = request.MappingTo<MultiPartUploadInfoEntity>();
                    var res = await _multiPartUploadInfoDBRepository.InsertOneAsync(entity);
                    if (!string.IsNullOrEmpty(res))
                    {
                        result = true;
                    }
                    else
                    {
                        result = false;
                    }
                }
                return result;
            }
        }
    }
}
