
using System;
using System.Collections.Generic;
using System.Threading.Tasks;
using MongoDB.Driver;
using MongoDB.Bson;
using WingInterfaceLibrary.DB.Request;
using WingInterfaceLibrary.ResearchEdition;
using WingMongoDBModule.Entities;
using WingServerCommon.Mapper;
using WingServerCommon.Service;
using WingServerCommon.Utilities;
using WingInterfaceLibrary.Enum;
using WingMongoDBModule.Request;
using WingInterfaceLibrary.Interface.DBInterface;
using WingServerCommon.Interfaces.Cache;
using System.Linq;
using WingInterfaceLibrary.Request;
using WingInterfaceLibrary.DTO.User;

namespace WingMongoDBModule.Service
{
    public partial class DatabaseService : JsonRpcService, ITaskDBService
    {



        /// <summary>
        /// 查询任务列表
        /// </summary>
        /// <param name="request">查询项目列表请求实体</param>
        /// <returns></returns>
        /// <errorCodes></errorCodes>
        public async Task<IList<TaskDTO>> GetTaskListDBAsync(GetTaskListDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetTaskListDBAsync execute", "DatabaseService"))
            {

                var builder = Builders<TaskEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false);
                if (request.TaskStatus != null)
                {
                    var nameFilter = builder.Eq(x => x.Status, request.TaskStatus);
                    filter = filter & nameFilter;
                }
                if (request.TaskType != null)
                {
                    var nameFilter = builder.Eq(x => x.TaskType, request.TaskType);
                    filter = filter & nameFilter;
                }
                if (request.IsJoinQueue != null)
                {
                    var nameFilter = builder.Eq(x => x.IsJoinQueue, request.IsJoinQueue);
                    filter = filter & nameFilter;
                }
                if (!string.IsNullOrWhiteSpace(request.MainTaskCode))
                {
                    var nameFilter = builder.Eq(x => x.MainTaskCode, request.MainTaskCode);
                    filter = filter & nameFilter;
                }
                var dataList = await _taskDBRepository.FindAllAsync(filter);
                return dataList.MappingTo<List<TaskDTO>>();
            }
        }

        /// <summary>
        /// 查询任务明细
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<TaskDTO> GetTaskDetailDBAsync(GetTaskDBRequest request)
        {
            var builder = Builders<TaskEntity>.Filter;
            var filter = builder.Where(c => c.IsDelete == false && c.Code == request.Code);
            var dataInfo = await _taskDBRepository.FindOneAsync(filter);
            var result = dataInfo.MappingTo<TaskDTO>();
            return result;
        }



        /// <summary>
        /// 创建任务
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<string> CreateTaskDBAsync(CreateTaskDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} CreateTaskDBAsync execute", "DatabaseService"))
            {
                var task = new TaskEntity()
                {
                    BusinessId = request.BusinessId,
                    TaskName = request.TaskName,
                    MainTaskCode = request.MainTaskCode,
                    Description = request.Description,
                    CreatorName = request.UserName,
                    OperatorCode = request.UserCode,
                    OperatorName = request.UserName,
                    OperateTime = DateTime.UtcNow,
                    Status = VTaskStatus.Unhandled,
                    TaskType = request.TaskType,
                    CreateTime = DateTime.UtcNow,
                    UpdateTime = DateTime.UtcNow,
                    BusinessCode = request.BusinessCode,
                    IsJoinQueue = request.IsJoinQueue,
                    IsJumpQueue = request.IsJumpQueue,
                    JsonData = request.JsonData
                };
                var res = await _taskDBRepository.InsertOneAsync(task);
                return res;
            }
        }

        /// <summary>
        /// 修改任务
        /// </summary>
        /// <param name="request">修改任务请求实体</param>
        /// <returns></returns>
        public async Task<bool> UpdateTaskDBAsync(UpdateTaskDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateTaskDBAsync execute", "DatabaseService"))
            {
                var res = false;
                var builder = Builders<TaskEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false && c.Code == request.Code);
                var dataInfo = await _taskDBRepository.FindOneAsync(filter);
                if (!string.IsNullOrEmpty(dataInfo?.Code))
                {
                    dataInfo.OperatorCode = request.UserCode;
                    dataInfo.OperatorName = request.UserName;
                    dataInfo.OperateTime = DateTime.UtcNow;
                    dataInfo.UpdateTime = DateTime.UtcNow;
                    dataInfo.Rate = request.Rate;
                    dataInfo.Result = request.Result;
                    dataInfo.Status = request.Status;
                    if (dataInfo.Status == VTaskStatus.Completed)
                    {
                        dataInfo.EndTime = DateTime.UtcNow;
                    }
                    res = await _taskDBRepository.UpdateOneAsync("Code", dataInfo.Code, dataInfo);
                    //主任务
                    var mainTaskBuilder = Builders<TaskMainEntity>.Filter;
                    var mainTaskFilter = mainTaskBuilder.Where(c => c.IsDelete == false && c.Code == dataInfo.MainTaskCode);
                    var mainTask = await _taskMainDBRepository.FindOneAsync(mainTaskFilter);
                    if (mainTask != null && !string.IsNullOrWhiteSpace(mainTask.Code))
                    {
                        var taskAllBuilder = Builders<TaskEntity>.Filter;
                        var taskAllFilter = taskAllBuilder.Where(c => c.IsDelete == false && c.MainTaskCode == mainTask.Code);
                        var taskList = await _taskDBRepository.FindAllAsync(taskAllFilter);
                        if (!taskList.Any(x => x.Status != VTaskStatus.Completed && x.Status != VTaskStatus.Canceled))
                        {
                            mainTask.Status = VTaskStatus.Completed;
                            mainTask.EndTime = DateTime.UtcNow;
                        }
                        mainTask.OperatorCode = request.UserCode;
                        mainTask.OperatorName = request.UserName;
                        mainTask.OperateTime = DateTime.UtcNow;
                        await _taskMainDBRepository.UpdateOneAsync("Code", mainTask.Code, mainTask);
                    }
                }
                return res;
            }
        }

        /// <summary>
        /// 删除任务
        /// </summary>
        /// <param name="request">删除任务请求实体</param>
        /// <returns></returns>
        public async Task<bool> DeleteTaskDBAsync(DeleteTaskDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} DeleteTaskDBAsync execute", "DatabaseService"))
            {
                var result = await _taskDBRepository.DeleteOneAsync("Code", request.Code);
                return result;
            }
        }

        /// <summary>
        /// 创建主任务
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<string> CreateMainTaskDBAsync(CreateTaskDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} CreateMainTaskDBAsync execute", "DatabaseService"))
            {
                var task = new TaskMainEntity()
                {
                    TaskName = request.TaskName,
                    CreatorName = request.UserName,
                    OperatorCode = request.UserCode,
                    OperatorName = request.UserName,
                    OperateTime = DateTime.UtcNow,
                    Status = VTaskStatus.Unhandled,
                    TaskType = request.TaskType,
                    CreateTime = DateTime.UtcNow,
                    UpdateTime = DateTime.UtcNow,
                    BusinessCode = request.BusinessCode,
                    PatientCode = request.PatientCode,
                };
                var res = await _taskMainDBRepository.InsertOneAsync(task);
                return res;
            }
        }

        /// <summary>
        /// 修改主任务
        /// </summary>
        /// <param name="request">修改任务请求实体</param>
        /// <returns></returns>
        public async Task<bool> UpdateMainTaskDBAsync(UpdateTaskDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateMainTaskDBAsync execute", "DatabaseService"))
            {
                var res = false;
                var builder = Builders<TaskMainEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false && c.Code == request.Code);
                var dataInfo = await _taskMainDBRepository.FindOneAsync(filter);
                if (!string.IsNullOrEmpty(dataInfo?.Code))
                {
                    dataInfo.OperatorCode = request.UserCode;
                    dataInfo.OperatorName = request.UserName;
                    dataInfo.OperateTime = DateTime.UtcNow;
                    dataInfo.UpdateTime = DateTime.UtcNow;
                    dataInfo.Status = request.Status;
                    res = await _taskMainDBRepository.UpdateOneAsync("Code", dataInfo.Code, dataInfo);
                }
                return res;
            }
        }

        /// <summary>
        /// 删除主任务
        /// </summary>
        /// <param name="request">删除任务请求实体</param>
        /// <returns></returns>
        public async Task<bool> DeleteMainTaskDBAsync(DeleteTaskDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} DeleteMainTaskDBAsync execute", "DatabaseService"))
            {
                var result = await _taskMainDBRepository.DeleteOneAsync("Code", request.Code);
                return result;
            }
        }

        /// <summary>
        /// 根据BusinessCode查询任务列表
        /// </summary>
        /// <param name="request">查询项目列表请求实体</param>
        /// <returns></returns>
        /// <errorCodes></errorCodes>
        async Task<IList<TaskDTO>> GetTaskListByBusinessCodeDBAsync(string BusinessCode)
        {
            using (new Performance(_performanceThreshold, "{0} GetTaskListByBusinessCodeDBAsync execute", "DatabaseService"))
            {
                var builder = Builders<TaskEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false);
                var nameFilter = builder.Eq(x => x.BusinessCode, BusinessCode);
                filter = filter & nameFilter;
                var dataList = await _taskDBRepository.FindAllAsync(filter);
                return dataList.MappingTo<List<TaskDTO>>();
            }
        }

        /// <summary>
        /// GetTaskListByMainTaskCodesDBAsync
        /// </summary>
        /// <param name="request">查询项目列表请求实体</param>
        /// <returns></returns>
        /// <errorCodes></errorCodes>
        async Task<IList<TaskDTO>> GetTaskListByMainTaskCodesDBAsync(IList<string> mainTaskCodes)
        {
            using (new Performance(_performanceThreshold, "{0} GetTaskListByMainTaskCodesDBAsync execute", "DatabaseService"))
            {
                if (mainTaskCodes == null || mainTaskCodes.Count == 0)
                {
                    return new List<TaskDTO>();
                }
                var builder = Builders<TaskEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false);
                var nameFilter = builder.In(x => x.MainTaskCode, mainTaskCodes);
                filter = filter & nameFilter;
                var dataList = await _taskDBRepository.FindAllAsync(filter);
                return dataList.MappingTo<List<TaskDTO>>();
            }
        }

    }
}