using System.Threading.Tasks;
using WingServerCommon.Service;
using WingMongoDBModule.DO;
using WingServerCommon.Mapper;
using System.Collections.Generic;
using System.Linq;
using JsonRpcLite.Services;
using System;
using WingMongoDBModule.Mongodb.Driver;
using WingMongoDBModule.Entity;
using MongoDB.Driver;
using WingMongoDBModule.Request;
using WingInterfaceLibrary.Interface;
using WingInterfaceLibrary.Enum;
using WingInterfaceLibrary.OpLog;
using WingInterfaceLibrary.Interface.DBInterface;
using WingInterfaceLibrary.DB.Request;
using WingServerCommon.Utilities;

namespace WingMongoDBModule.Service
{
    public partial class DatabaseService : JsonRpcService, ITokenDBService
    {
        /// <summary>
        /// 添加客户端登录信息
        /// </summary>
        /// <param name="AddTokenInfoRequest">请求对象，即方法的参数。</param>
        /// <returns>
        /// 新创建的TokenDO的id
        /// </returns>
        public async Task<string> AddTokenInfoAsync(AddTokenInfoRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} AddTokenInfoAsync execute", $"{this.GetType().Name}"))
            {
                var tokenDO = request.Data.MappingTo<TokenDO>();
                tokenDO.Code = tokenDO.Code;
                return await _tokensDBRepository.InsertOneAsync(tokenDO, request.PlatformDatas, request.ExtensionData);
            }
        }

        /// <summary>
        /// 通过clientId和登录源查找token
        /// </summary>
        /// <param name="clentId">登录的客户端ID</param>
        /// <param name="LoginSource">登录源，为LoginSource的枚举类型。</param>
        /// <returns>
        /// 令牌对象
        /// </returns>
        public async Task<TokenDTO> FindTokenByClentIdAndLoginSourceAsync(string clentId, LoginSource loginSource)
        {
            using (new Performance(_performanceThreshold, "{0} FindTokenByClentIdAndLoginSourceAsync execute", $"{this.GetType().Name}"))
            {
                MongoCustomerFilter<MongoTable> mongoCustomerFilter = new();
                List<(string, string)> eqFilters = new();
                eqFilters.Add(("LoginSource", loginSource.ToString()));
                eqFilters.Add(("ClientId", clentId.ToString()));
                mongoCustomerFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, eqFilters.ToArray());

                SortDefinitionBuilder<MongoTable> builderSort = Builders<MongoTable>.Sort;
                SortDefinition<MongoTable> sort = builderSort.Descending("BaseData.CreateTime");
                var filter = mongoCustomerFilter.AndCombine();
                var dbPage = new DBPageRequest();
                dbPage.CurrentPage = 1;
                dbPage.PageSize = 1;
                dbPage.Filter = filter;
                dbPage.Sort = sort;
                var pageResult = await _tokensDBRepository.GetPages(dbPage);
                if (pageResult.PageData != null && pageResult.PageData.Count > 0)
                {
                    return pageResult.PageData.FirstOrDefault().MappingTo<TokenDTO>();
                }
                else
                {
                    return null;
                }
            }
        }

        /// <summary>
        /// 根据clentId查询全部token
        /// </summary>
        /// <param name="clentId"></param>
        /// <returns></returns>
        public async Task<List<TokenDTO>> FindTokensByClentIdAsync(string clentId)
        {
            using (new Performance(_performanceThreshold, "{0} FindTokensByClentIdAsync execute", $"{this.GetType().Name}"))
            {
                var allDataFilter = new Dictionary<string, string>();
                allDataFilter.Add("ClientId", clentId);
                var allDatas = await _tokensDBRepository.FindListAsync(allDataFilter);
                return allDatas.MappingTo<List<TokenDTO>>();
            }
        }

        /// <summary>
        /// 按值查找令牌
        /// </summary>
        /// <param name="value">token的值</param>
        /// <returns>
        /// 令牌对象
        /// </returns>
        public async Task<TokenDTO> FindTokenByValueAsync(string value)
        {
            var adminRoleDO = await _tokensDBRepository.FindOneAsync("Code", value);
            return adminRoleDO.MappingTo<TokenDTO>();
        }

        /// <summary>
        /// 查找与 clientId 匹配的所有令牌
        /// </summary>
        /// <param name="ids">Id list</param>
        /// <returns>
        /// 令牌列表。
        /// </returns>
        public async Task<List<TokenDTO>> FindTokensWithIdsAsync(IList<string> ids)
        {
            var dic = new Dictionary<string, List<string>>();
            dic.Add("Code", ids as List<string>);
            var allDatas = await _tokensDBRepository.FindInAsync(dic);
            return allDatas.MappingTo<List<TokenDTO>>();
        }

        /// <summary>
        /// > 查找所有未过期的令牌
        /// </summary>
        /// <returns>
        /// 未过期的令牌列表。
        /// </returns>
        public async Task<List<TokenDTO>> FindNotExpiredTokensAsync()
        {
            MongoCustomerFilter<MongoTable> filter = new();
            filter.AppendGt("BaseData.Expiration", DateTime.UtcNow);
            var entities = await _tokensDBRepository.FindAllAsync(filter.AndCombine());

            if (_isLocalHost)
            {
                var list = new List<TokenDTO>();
                foreach (var token in entities)
                {
                    //arm64mapping速度非常慢 超过20秒 原因暂时未知
                    var dto = new TokenDTO();
                    dto.AccountName = token.AccountName;
                    dto.AccountType = token.AccountType;
                    dto.ClientId = token.ClientId;
                    dto.Code = token.Code;
                    dto.CreateTime =token.CreateTime;
                    dto.Expiration = token.Expiration;
                    dto.InstallVersion = token.InstallVersion;
                    dto.IpValue=token.IpValue;
                    dto.IsOldPlatform = token.IsOldPlatform;
                    dto.IsOnline = false;
                    dto.LoginServer = token.LoginServer;
                    dto.LoginSource = token.LoginSource;
                    dto.Platform = token.Platform;
                    dto.ProxyType = token.ProxyType;
                    dto.Version = token.Version;
                    list.Add(dto);
                }
                return list;
            }
            else
            {
                var result = entities.MappingTo<List<TokenDTO>>();
                return result;
            }

        }

        /// <summary>
        /// 查找所有已过期的令牌
        /// </summary>
        /// <returns></returns>
        public async Task<List<TokenDTO>> FindExpiredTokensAsync()
        {
            MongoCustomerFilter<MongoTable> filter = new();
            filter.AppendLt("BaseData.Expiration", DateTime.UtcNow);
            var list = await _tokensDBRepository.FindAllAsync(filter.AndCombine());
            return list.MappingTo<List<TokenDTO>>();
        }

        /// <summary>
        /// 它从数据库中删除一个令牌。
        /// </summary>
        /// <param name="value">要删除的令牌值。</param>
        /// <returns>
        /// 一个布尔值。
        /// </returns>
        public async Task<bool> RemoveTokenAsync(string value)
        {
            if (string.IsNullOrWhiteSpace(value))
            {
                return false;
            }
            return await _tokensDBRepository.RealDeleteOneAsync("Code", value);
        }

        /// <summary>
        /// > 更新数据库中的令牌
        /// </summary>
        /// <param name="value">要更新的令牌的值。</param>
        /// <param name="TokenDTO">您要更新的令牌对象。</param>
        /// <returns>
        /// 一个布尔值。
        /// </returns>
        public async Task<bool> UpdateTokenAsync(string value, TokenDTO token)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateTokenAsync execute", $"{this.GetType().Name}"))
            {
                return await _tokensDBRepository.UpdateOneAsync("Code", value, token.MappingTo<TokenDO>(), null);
            }
        }
    }
}