using System.Collections.Generic;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Threading.Tasks;
using WingInterfaceLibrary.DB.Request;
using WingInterfaceLibrary.DTO.UserFeature;
using WingInterfaceLibrary.Interface.DBInterface;
using WingInterfaceLibrary.Request;
using WingMongoDBModule.DO;
using WingMongoDBModule.Entity;
using WingMongoDBModule.Mongodb.Driver;
using WingServerCommon.Interfaces.Cache;
using WingServerCommon.Mapper;
using WingServerCommon.Service;
using WingServerCommon.Utilities;

namespace WingMongoDBModule.Service
{
    public partial class DatabaseService : JsonRpcService, IUserFeaturesDBService
    {
        public async Task<UserFeatureInfoDTO> FindUserFeatureByCodeAsync(string code)
        {
            using (new Performance(_performanceThreshold, "{0} FindUserFeatureByCodeAsync execute", $"{this.GetType().Name}"))
            {
                var userFeatureDO = await _userFeatureDBRepository.FindOneAsync("Code", code);
                return userFeatureDO.MappingTo<UserFeatureInfoDTO>();
            }
        }

        public async Task<UserFeatureInfoDTO> FindUserFeatureByNameAsync(string name)
        {
            using (new Performance(_performanceThreshold, "{0} FindUserFeatureByNameAsync execute", $"{this.GetType().Name}"))
            {
                var userFeatureDO = await _userFeatureDBRepository.FindOneAsync("FeatureName", name);
                return userFeatureDO.MappingTo<UserFeatureInfoDTO>();
            }
        }

        public async Task<string> InsertUserFeatureAsync(CreateUserFeatureDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} InsertUserFeatureAsync execute", $"{this.GetType().Name}"))
            {
                var userFeatureDO = request.Data.MappingTo<UserFeatureDO>();
                userFeatureDO.Code = userFeatureDO.UniqueCode;
                return await _userFeatureDBRepository.InsertOneAsync(userFeatureDO, request.PlatformDatas, request.ExtensionData);
            }
        }

        public async Task<bool> UpdateUserFeatureByCodeAsync(string code, UserFeatureInfoDTO data, string extensionData)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateUserFeatureByCodeAsync execute", $"{this.GetType().Name}"))
            {
                var userFeatureDO = data.MappingTo<UserFeatureDO>();
                return await _userFeatureDBRepository.UpdateOneAsync("Code", code, userFeatureDO, extensionData, true);
            }
        }

        public async Task<PageCollection<UserFeatureInfoDTO>> FindUserFeaturePagesAsync(PageFilterRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} FindUserFeaturePagesAsync execute", $"{this.GetType().Name}"))
            {
                var dbRequest = GetDBPageRequest(request);
                var pageInfo = await _userFeatureDBRepository.GetPages(dbRequest);
                return pageInfo.MappingTo<PageCollection<UserFeatureInfoDTO>>();
            }
        }

        public async Task<bool> DeleteUserFeatureByCodeAsync(string code)
        {
            using (new Performance(_performanceThreshold, "{0} DeleteUserFeatureByCodeAsync execute", $"{this.GetType().Name}"))
            {
                return await _userFeatureDBRepository.DeleteOneAsync("Code", code);
            }
        }

        #region db version 1.3 upgrade
        /// <summary>
        /// 修改远程维护管理员角色的用户数据
        /// </summary>
        /// <returns>是否成功</returns>
        public async Task<bool> UpgradeRemoteMaintainUserRoleDBAsync()
        {
            using (new Performance(_performanceThreshold, "{0} UpgradeRemoteMaintainUserRoleDBAsync execute", $"{this.GetType().Name}"))
            {
                var res = false;
                //管理员改普通角色
                MongoCustomerFilter<MongoTable> customerFilter = new();
                customerFilter.AppendArrayEq("BaseData.RoleCodes", "Role_RemoteManager");
                var allUserInfo = await _userDBRepository.FindAllAsync(customerFilter.AndCombine());
                if (allUserInfo?.Count > 0)
                {
                    foreach(var item in allUserInfo)
                    {
                        item.RoleCodes = new List<string>() { "Role_CertifiedPhysician" };
                        item.UserFeatureCodes = new List<string>() { "RemoteManager" };
                        res = await _userDBRepository.UpdateOneForSyncDBAsync("Code", item.Code, item, string.Empty);
                    }
                }
                return res;
            }
        }

        /// <summary>
        /// 添加权限，
        /// 1.Service-远程维护管理员
        /// 2.SalesAgent-远程维护
        /// 删除远程管理员角色
        /// </summary>
        /// <returns>是否成功</returns>
        private async Task<bool> UpdateSystemDefaultAuthDBAsync()
        {
            using (new Performance(_performanceThreshold, "{0} UpdateSystemDefaultAuthDBAsync execute", $"{this.GetType().Name}"))
            {
                var res = false;
                var featureInfo = await _userFeatureDBRepository.FindOneAsync("Code", "RemoteManager");
                if (string.IsNullOrEmpty(featureInfo?.Code))
                {
                    var model = new UserFeatureDO//对应  service
                    {
                        FeatureCode = "RemoteManager",
                        FatherCode = "",
                        FeatureName = "远程维护管理员",
                        UniqueCode = "RemoteManager",//审核申请
                        LanguageConfigs = new List<UserFeatureLanguageConfigDO>() {
                            new UserFeatureLanguageConfigDO() {
                                Language = "zh-CN",
                                FeatureName = "远程维护管理员"
                            },
                            new UserFeatureLanguageConfigDO() {
                                Language = "ru-RU",
                                FeatureName = "Администратор удаленного обслуживания"
                            },
                            new UserFeatureLanguageConfigDO() {
                                Language = "en-US",
                                FeatureName = "Remote Manager"
                            }
                        }
                    };
                    await _userFeatureDBRepository.InsertOneForAsyncDBAsync(model, new Dictionary<string, string>(), string.Empty, false);
                }
                featureInfo = await _userFeatureDBRepository.FindOneAsync("Code", "RemoteMaintain");
                if (string.IsNullOrEmpty(featureInfo?.Code))
                {
                    var model = new UserFeatureDO//对应  service
                    {
                        FeatureCode = "RemoteMaintain",
                        FatherCode = "",
                        FeatureName = "远程维护",
                        UniqueCode = "RemoteMaintain",
                        LanguageConfigs = new List<UserFeatureLanguageConfigDO>() {
                            new UserFeatureLanguageConfigDO() {
                                Language = "zh-CN",
                                FeatureName = "远程维护"
                            },
                            new UserFeatureLanguageConfigDO() {
                                Language = "ru-RU",
                                FeatureName = "Дистанционное обслуживание"
                            },
                            new UserFeatureLanguageConfigDO() {
                                Language = "en-US",
                                FeatureName = "Remote Maintain"
                            }
                        }
                    };
                    await _userFeatureDBRepository.InsertOneForAsyncDBAsync(model, new Dictionary<string, string>(), string.Empty, false);
                }
                //删除权限
                featureInfo = await _userFeatureDBRepository.FindOneAsync("Code", "RemoteManage");
                if (!string.IsNullOrEmpty(featureInfo?.Code))
                {
                    res = await _userFeatureDBRepository.DeleteOneAsync("Code", featureInfo.Code);
                }

                //删除用户角色
                var roleInfo = await _roleDBRepository.FindOneAsync("Code", "Role_RemoteManager");
                if (!string.IsNullOrEmpty(roleInfo?.Code))
                {
                    res = await _roleDBRepository.DeleteOneAsync("Code", roleInfo.Code);
                }
                return res;
            }
        }
        #endregion 

        /// <summary>
        /// 添加质控权限
        /// </summary>
        /// <returns>是否成功</returns>
        private async Task<bool> UpdateQualityControlDBAsync()
        {
            using (new Performance(_performanceThreshold, "{0} UpdateQualityControlDBAsync execute", $"{this.GetType().Name}"))
            {
                var res = false;
                var featureInfo = await _userFeatureDBRepository.FindOneAsync("Code", "QualityControl");
                if (string.IsNullOrEmpty(featureInfo?.Code))
                {
                    var model = new UserFeatureDO//对应  service
                    {
                        FeatureCode = "QualityControl",
                        FatherCode = "",
                        FeatureName = "质控权限",
                        UniqueCode = "QualityControl",//质控
                        LanguageConfigs = new List<UserFeatureLanguageConfigDO>() {
                            new UserFeatureLanguageConfigDO() {
                                Language = "zh-CN",
                                FeatureName = "质控权限"
                            },
                            new UserFeatureLanguageConfigDO() {
                                Language = "ru-RU",
                                FeatureName = "Контроль качества "
                            },
                            new UserFeatureLanguageConfigDO() {
                                Language = "en-US",
                                FeatureName = "Quality Control"
                            },
                            new UserFeatureLanguageConfigDO() {
                                Language = "es-ES",
                                FeatureName = "Control de calidad"
                            }
                        }
                    };
                    await _userFeatureDBRepository.InsertOneForAsyncDBAsync(model, new Dictionary<string, string>(), string.Empty, false);
                }
                
                return res;
            }
        }

        /// <summary>
        /// 添加质控权限
        /// </summary>
        /// <returns>是否成功</returns>
        private async Task<bool> UpdateQualityControlForRODBAsync()
        {
            using (new Performance(_performanceThreshold, "{0} UpdateQualityControlForRODBAsync execute", $"{this.GetType().Name}"))
            {
                var res = false;
                var featureInfo = await _userFeatureDBRepository.FindOneAsync("Code", "QualityControl");
                if (!string.IsNullOrEmpty(featureInfo?.Code))
                {
                    featureInfo.LanguageConfigs.Add(
                            new UserFeatureLanguageConfigDO() {
                                Language = "ro-RO",
                                FeatureName = "Permisiuni de control al calității"
                            }
                    );
                    res = await _userFeatureDBRepository.UpdateOneAsync("Code", featureInfo.Code, featureInfo, string.Empty, false);
                }
                return res;
            }
        }
    }
}