using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using MongoDB.Bson;
using MongoDB.Driver;
using WingInterfaceLibrary.DB.Request;
using WingInterfaceLibrary.DTO.Education;
using WingInterfaceLibrary.DTO.Management;
using WingInterfaceLibrary.DTO.Payment;
using WingInterfaceLibrary.DTO.Record;
using WingInterfaceLibrary.Enum;
using WingInterfaceLibrary.Enum.CourseEnum;
using WingInterfaceLibrary.Interface.DBInterface;
using WingInterfaceLibrary.Request.DBRequest;
using WingInterfaceLibrary.Result;
using WingMongoDBModule.DO;
using WingMongoDBModule.Entities;
using WingMongoDBModule.Entity;
using WingMongoDBModule.Mongodb.Driver;
using WingMongoDBModule.Request;
using WingServerCommon.Log;
using WingServerCommon.Mapper;
using WingServerCommon.Service;
using WingServerCommon.Utilities;

namespace WingMongoDBModule.Service
{
    public partial class DatabaseService : JsonRpcService, IUserGroupDBService
    {
        /// <summary>
        /// 分页查询用户群组
        /// </summary>
        /// <returns></returns>
        public async Task<PageResult<UserGroupDTO>> GetUserGroupByPageDBAsync(GetUserGroupByPageDBRequest request)
        {
            var pageIndex = request.PageIndex;
            var pageSize = request.PageSize;
            var keyword = request.Keyword;
            var userCode = request.UserCode;

            var builder = Builders<UserGroupEntity>.Filter;
            var filter = builder.Where(c => c.IsDelete == false && c.UserGroupType == UserGroupEnum.Group);
            if (!string.IsNullOrEmpty(keyword))
            {
                keyword = EscapeExprSpecialWord(keyword);//字符串转义
                var nameFilter = builder.Regex(c => c.Name, new BsonRegularExpression(keyword, "i"));
                filter = filter & nameFilter;
            }
            if (!string.IsNullOrWhiteSpace(userCode))
            {
                var secondFilter = builder.AnyIn(c => c.UserCodes, new List<string>() { userCode });
                filter = filter & secondFilter;
            }
            var sort = Builders<UserGroupEntity>.Sort.Descending(v => v.CreateTime);
            var pageResult = await _userGroupDBRepository.GetPages(new DBPageRequest<UserGroupEntity>
            {
                CurrentPage = pageIndex,
                PageSize = pageSize,
                Filter = filter,
                Sort = sort,
            });
            return PageResultConvert<UserGroupEntity, UserGroupDTO>(pageResult);
        }

        /// <summary>
        /// 查询用户组详情
        /// </summary>
        /// <returns></returns>
        public async Task<UserGroupDTO> GetUserGroupDBAsync(GetUserGroupDBRequest request)
        {
            var userGroupCode = request.UserGroupCode;
            var builder = Builders<UserGroupEntity>.Filter;
            var filter = builder.Where(c => c.Code == userGroupCode);
            var userGroupEntity = await _userGroupDBRepository.FindOneAsync(filter) ?? new UserGroupEntity();
            return userGroupEntity.MappingTo<UserGroupDTO>();
        }

        /// <summary>
        /// 新增/更新用户组信息
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<string> AddOrUpdateUserGroupDBAsync(UserGroupDTO request)
        {
            try
            {
                var userGroupEntity = request.MappingTo<UserGroupEntity>();
                if (!string.IsNullOrWhiteSpace(userGroupEntity.Code))
                {
                    var userGroupCode = userGroupEntity.Code;
                    var existedEntity = await _userGroupDBRepository.FindOneAsync("Code", userGroupCode, false);
                    if (existedEntity != null && !string.IsNullOrWhiteSpace(existedEntity.Code))
                    {
                        existedEntity.Name = userGroupEntity.Name;
                        existedEntity.MaxPeople = userGroupEntity.MaxPeople;
                        existedEntity.Students = userGroupEntity.Students;
                        existedEntity.IsAgent = userGroupEntity.IsAgent;
                        existedEntity.UserGroupType = userGroupEntity.UserGroupType;
                        existedEntity.UserCodes = userGroupEntity.UserCodes;
                        existedEntity.DeviceCodes = userGroupEntity.DeviceCodes;
                        await _userGroupDBRepository.UpdateOneAsync("Code", userGroupCode, existedEntity);
                        return userGroupCode;
                    }
                }
                else
                {
                    var shortCode = "";
                    for (var i = 0; i < 5; i++)
                    {
                        shortCode = GeneralShortCode();
                        if (!string.IsNullOrWhiteSpace(shortCode))
                        {
                            var userGroup = await GetUserGroupDBByShortCodeAsync(shortCode);
                            if (userGroup == null || string.IsNullOrWhiteSpace(userGroup.Code))
                            {
                                break;
                            }
                        }
                        shortCode = string.Empty;
                    }
                    if (!string.IsNullOrWhiteSpace(shortCode))
                    {
                        userGroupEntity.ShortCode = shortCode;
                        return await _userGroupDBRepository.InsertOneAsync(userGroupEntity);
                    }
                }
            }
            catch(Exception ex)
            {
                Logger.WriteLineWarn($"UserGroupService AddOrUpdateUserGroupDBAsync error, ex:{ex}");
            }
            return string.Empty;
        }

        /// <summary>
        /// 分页查询用户群组检查记录
        /// </summary>
        /// <returns></returns>
        public async Task<PageResult<RecordInfoDTO>> GetUserGroupRecordsByPageDBAsync(GetUserGroupRecordsByPageDBRequest request)
        {
            var pageIndex = request.PageIndex;
            var pageSize = request.PageSize;
            var keyword = request.Keyword;
            var userGroupCode = request.UserGroupCode;
            var startDate = request.StartDate;
            var endDate = request.EndDate;
            var organizationCodes = request.OrganizationCodes;
            var deviceCodes = request.DeviceCodes;

            var entity = await _userGroupDBRepository.FindOneAsync("Code", userGroupCode, false);
            if (entity == null || string.IsNullOrWhiteSpace(entity.Code) || entity.IsDelete || entity.DeviceCodes == null || !entity.DeviceCodes.Any()
                || deviceCodes == null || !deviceCodes.Any())
            {
                return new PageResult<RecordInfoDTO>
                {
                    PageIndex = pageIndex,
                    PageSize = pageSize,
                    TotalCount = 0,
                    PageData = new List<RecordInfoDTO>(),
                };
            }
            deviceCodes = deviceCodes.Intersect(entity.DeviceCodes).ToList();
            if (deviceCodes == null || !deviceCodes.Any())
            {
                return new PageResult<RecordInfoDTO>
                {
                    PageIndex = pageIndex,
                    PageSize = pageSize,
                    TotalCount = 0,
                    PageData = new List<RecordInfoDTO>(),
                };
            }

            MongoCustomerFilter<MongoTable> customerFilter = new();//常规查询
            MongoCustomerFilter<MongoDataItem> queryItemsFilter = new();
            MongoCustomerFilter<MongoTable> customerKeyWordFilter = new();//关键字查询

            if (startDate > new DateTime(2000, 1, 1))
            {
                customerFilter.AppendGt("BaseData.CreateTime", startDate);
            }
            if (endDate > new DateTime(2000, 1, 1))
            {
                customerFilter.AppendLte("BaseData.CreateTime", endDate);
            }
            if (!string.IsNullOrWhiteSpace(keyword))
            {
                //关键字 支持Name、DevicePatientID、手机号、年龄、性别
                var keywordfilter = new Dictionary<string, string>();
                var keywordDictionary = new Dictionary<string, string>();
                var tempKeyWord = CryptionHelper.EncryptOneByOne(keyword);
                keywordDictionary.Add("Name", tempKeyWord);
                keywordDictionary.Add("Phone", tempKeyWord);
                //keywordDictionary.Add("DevicePatientID", keyword);
                keywordDictionary.Add("Age", keyword);
                keywordDictionary.Add("Gender", keyword);
                var pageFilterRequest = new GetPatientsByOrgPageRequest()
                {
                    KeywordOrFilter = keywordDictionary,
                    IsFuzzy = true
                };
                var patientList = GetPatientsDBAsync(pageFilterRequest).Result;
                queryItemsFilter = new();
                queryItemsFilter.AppendEq((item => item.Key, "PatientCode"));
                if (patientList == null || patientList.Count == 0)
                {
                    queryItemsFilter.AppendEq((item => item.Value, "-1"));
                    if (queryItemsFilter.HasFilter)
                    {
                        customerKeyWordFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
                    }
                }
                else
                {
                    queryItemsFilter.AppendIn(item => item.Value, patientList.Select(p => p.PatientCode).ToList());
                    if (queryItemsFilter.HasFilter)
                    {
                        customerKeyWordFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
                    }
                }
                customerKeyWordFilter.AppendRegexForKey("BaseData.DevicePatientID", keyword, true);
            }
            if (organizationCodes != null && organizationCodes.Count > 0)
            {
                queryItemsFilter = new();
                queryItemsFilter.AppendEq((item => item.Key, "RootOrganizationCode"));
                queryItemsFilter.AppendIn(item => item.Value, organizationCodes);
                if (queryItemsFilter.HasFilter)
                {
                    customerFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
                }
            }
            if (deviceCodes != null && deviceCodes.Count > 0)
            {
                queryItemsFilter = new();
                queryItemsFilter.AppendEq((item => item.Key, "DeviceCode"));
                queryItemsFilter.AppendIn(item => item.Value, deviceCodes);
                if (queryItemsFilter.HasFilter)
                {
                    customerFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
                }
            }
            var filter = customerFilter.AndCombine() & customerKeyWordFilter.OrCombine();
            var sort = Builders<MongoTable>.Sort.Descending("BaseData.CreateTime");
            var pageResult = await _recordInfoDBRepository.GetPages(new DBPageRequest
            {
                CurrentPage = pageIndex,
                PageSize = pageSize,
                Filter = filter,
                Sort = sort,
            });
            return PageResultConvert<RecordInfoDO, RecordInfoDTO>(pageResult);
        }

    }
}