using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Threading.Tasks;
using MongoDB.Driver;
using WingInterfaceLibrary.Interface.DBInterface;
using WingInterfaceLibrary.Request.RemedicalAISelected;
using WingInterfaceLibrary.ResearchEdition;
using WingMongoDBModule.Entities;
using WingServerCommon.Log;
using WingServerCommon.Service;
using WingServerCommon.Utilities;

namespace WingMongoDBModule.Service
{
    public partial class DatabaseService : JsonRpcService, IUserUrmConfigDBService
    {
        private readonly string _defaultUrmProcessConfigPath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "Lab", "DefaultUrmProcessConfig.json");

        private List<UrmProcessConfigDTO> _defaultUrmProcessConfigInfos;
        private List<UrmProcessConfigDTO> DefaultUrmProcessConfigInfos
        {
            get
            {
                if (_defaultUrmProcessConfigInfos == null || !_defaultUrmProcessConfigInfos.Any())
                {
                    try
                    {
                        if (File.Exists(_defaultUrmProcessConfigPath))
                        {
                            var baseJson = File.ReadAllText(_defaultUrmProcessConfigPath);
                            if (!string.IsNullOrWhiteSpace(baseJson))
                            {
                                _defaultUrmProcessConfigInfos = Newtonsoft.Json.JsonConvert.DeserializeObject<List<UrmProcessConfigDTO>>(baseJson) ?? new List<UrmProcessConfigDTO>();
                            }
                        }
                    }
                    catch (Exception ex)
                    {
                        Logger.WriteLineWarn($"UserUrmConfigDBService GetDefaultUrmProcessConfigInfos fail, ex:{ex}");
                    }
                }
                return _defaultUrmProcessConfigInfos ?? new List<UrmProcessConfigDTO>();
            }
        }

        /// <summary>
        /// 保存用户URM配置信息
        /// </summary>
        /// <returns></returns>
        public async Task<string> SaveUserUrmConfigDBAsync(UserUrmConfigDTO request)
        {
            using (new Performance(_performanceThreshold, "{0} SaveUserUrmConfigDBAsync execute", "DatabaseService"))
            {
                var code = request.Code;
                UserUrmConfigEntity entity = null;
                if (!string.IsNullOrWhiteSpace(code))
                {
                    entity = await _userUrmConfigDBRepository.FindOneAsync(Builders<UserUrmConfigEntity>.Filter.Where(x => x.Code == code));
                }
                if (entity == null)
                {
                    entity = new UserUrmConfigEntity();
                    entity.UserCode = request.UserCode;
                    entity.ConfigName = request.ConfigName;
                    // entity.IsDefault = request.IsDefault;
                    entity.ConfigType = request.ConfigType;
                    entity.ConfigData = request.ConfigData;
                    entity.CreateTime = DateTime.UtcNow;
                    entity.UpdateTime = DateTime.UtcNow;
                    return await _userUrmConfigDBRepository.InsertOneAsync(entity);
                }
                else if (!entity.IsDelete && entity.UserCode == request.UserCode)
                {
                    entity.UserCode = request.UserCode;
                    entity.ConfigName = request.ConfigName;
                    // entity.IsDefault = request.IsDefault;
                    entity.ConfigType = request.ConfigType;
                    entity.ConfigData = request.ConfigData;
                    entity.UpdateTime = DateTime.UtcNow;
                    var updateResult = await _userUrmConfigDBRepository.UpdateOneAsync("Code", entity.Code, entity);
                    if (updateResult)
                    {
                        return entity.Code;
                    }
                }
                return string.Empty;
            }
        }

        /// <summary>
        /// 删除用户URM配置信息
        /// </summary>
        /// <returns></returns>
        public async Task<bool> DeleteUserUrmConfigDBAsync(string code)
        {
            using (new Performance(_performanceThreshold, "{0} DeleteUserUrmConfigDBAsync execute", "DatabaseService"))
            {
                if (!string.IsNullOrWhiteSpace(code))
                {
                    var entity = await _userUrmConfigDBRepository.FindOneAsync(Builders<UserUrmConfigEntity>.Filter.Where(x => x.Code == code));
                    if (entity != null && !string.IsNullOrWhiteSpace(entity.Code))
                    {
                        if (!entity.IsDelete)
                        {
                            entity.IsDelete = true;
                            entity.UpdateTime = DateTime.UtcNow;
                            return await _userUrmConfigDBRepository.UpdateOneAsync("Code", entity.Code, entity);
                        }
                        return true;
                    }
                }
                return false;
            }
        }

        /// <summary>
        /// 设置默认用户URM配置信息
        /// </summary>
        /// <returns></returns>
        public async Task<bool> SetDefaultUserUrmConfigDBAsync(string userCode, string configCode)
        {
            using (new Performance(_performanceThreshold, "{0} DeleteUserUrmConfigDBAsync execute", "DatabaseService"))
            {
                if (string.IsNullOrWhiteSpace(userCode))
                {
                    var filter = Builders<UserUrmConfigEntity>.Filter.Where(x => x.UserCode == userCode && x.IsDelete == false);
                    var entities = await _userUrmConfigDBRepository.FindAllAsync(filter);
                    if (entities != null)
                    {
                        foreach (var entity in entities)
                        {
                            var isDefault = entity.Code == configCode;
                            if (entity.IsDefault != isDefault)
                            {
                                entity.IsDefault = isDefault;
                                entity.UpdateTime = DateTime.UtcNow;
                                await _userUrmConfigDBRepository.UpdateOneAsync("Code", entity.Code, entity);
                            }
                        }
                        return true;
                    }
                }
                return false;
            }
        }

        /// <summary>
        /// 查看用户URM配置集合
        /// </summary>
        /// <returns></returns>
        public async Task<List<UserUrmConfigDTO>> GetUserUrmConfigsDBAsync(string userCode)
        {
            using (new Performance(_performanceThreshold, "{0} GetUserUrmConfigsDBAsync execute", "DatabaseService"))
            {
                var userUrmConfigList = new List<UserUrmConfigDTO>();
                if (DefaultUrmProcessConfigInfos.Any())
                {
                    userUrmConfigList.Add(new UserUrmConfigDTO
                    {
                        Code = "7AE6EFCB8C864A42BE21C5D8AE03A48C",
                        UserCode = string.Empty,
                        ConfigName = "Default",
                        IsDefault = false,
                        ConfigType = 1,
                        ConfigInfo = DefaultUrmProcessConfigInfos,
                    });
                }
                if (!string.IsNullOrWhiteSpace(userCode))
                {
                    var filter = Builders<UserUrmConfigEntity>.Filter.Where(x => x.UserCode == userCode && x.IsDelete == false);
                    var entities = await _userUrmConfigDBRepository.FindAllAsync(filter);
                    if (entities?.Any() ?? false)
                    {
                        foreach (var entity in entities)
                        {
                            var model = new UserUrmConfigDTO
                            {
                                Code = entity.Code,
                                UserCode = entity.UserCode,
                                ConfigName = entity.ConfigName,
                                IsDefault = entity.IsDefault,
                                ConfigType = entity.ConfigType,
                                ConfigData = entity.ConfigData,
                                CreateTime = entity.CreateTime,
                                UpdateTime = entity.UpdateTime,
                            };
                            if (!string.IsNullOrWhiteSpace(model.ConfigData))
                            {
                                try
                                {
                                    model.ConfigInfo = Newtonsoft.Json.JsonConvert.DeserializeObject<List<UrmProcessConfigDTO>>(entity.ConfigData);
                                }
                                catch { }
                            }
                            userUrmConfigList.Add(model);
                        }
                    }
                }
                if (userUrmConfigList.Any() && userUrmConfigList.All(x => !x.IsDefault))
                {
                    var defaultConfig = userUrmConfigList.FirstOrDefault();
                    defaultConfig.IsDefault = true;
                }
                return userUrmConfigList;
            }
        }
    }
}