using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Threading.Tasks;
using MongoDB.Bson;
using MongoDB.Driver;
using WingInterfaceLibrary.DTO.Vital;
using WingInterfaceLibrary.Enum;
using WingInterfaceLibrary.Interface.DBInterface;
using WingInterfaceLibrary.Request.DBRequest;
using WingInterfaceLibrary.Result;
using WingMongoDBModule.DB.DO;
using WingMongoDBModule.Entities;
using WingMongoDBModule.Request;
using WingServerCommon.Mapper;
using WingServerCommon.Service;

namespace WingMongoDBModule.Service
{
    public partial class DatabaseService : JsonRpcService, IVitalHealthExamBookingDBService
    {
        private readonly DateTime _defaultTime = new DateTime(2010, 1, 1);

        /// <summary>
        /// 分页获取健康检查预约数据
        /// </summary>
        /// <param name="request">分页请求</param>
        /// <returns></returns>
        public async Task<PageResult<HealthExamBookingDTO>> GetHealthExamBookingPageDBAsync(GetHealthExamBookingPageDBRequest request)
        {
            var keyword = request.Keyword;

            var builder = Builders<HealthExamBookingEntity>.Filter;
            var filter = builder.Where(c => c.IsDelete == false);
            if (!string.IsNullOrEmpty(keyword))
            {
                keyword = EscapeExprSpecialWord(keyword);//字符串转义
                var nameFilter = builder.Regex(c => c.Name, new BsonRegularExpression(keyword, "i"));
                filter = filter & nameFilter;
            }
            if (request.OrgCodes?.Any() ?? false)
            {
                var orgCodesFilter = builder.Where(x => request.OrgCodes.Contains(x.OrgCode));
                filter = filter & orgCodesFilter;
            }
            if (request.StartTime >= _defaultTime)
            {
                if (request.OrderByAscending)
                {
                    var startTimeFilter = builder.Where(x => x.UpdateTime > request.StartTime);
                    filter = filter & startTimeFilter;
                }
            }

            var sortDefinition = Builders<HealthExamBookingEntity>.Sort.Descending(x => x.CreateTime);
            if (request.OrderByAscending)
            {
                sortDefinition = Builders<HealthExamBookingEntity>.Sort.Ascending(x => x.UpdateTime);
            }
            var pageFilter = new DBPageRequest<HealthExamBookingEntity>
            {
                CurrentPage = request.PageIndex,
                PageSize = request.PageSize,
                Filter = filter,
                Sort = sortDefinition,
            };
            var result = await _healthExamBookingDBRepository.GetPages(pageFilter);
            return result.MappingTo<PageResult<HealthExamBookingDTO>>();
        }

        /// <summary>
        /// 解除预约数据
        /// </summary>
        /// <param name="request">创建请求</param>
        /// <returns></returns>
        public async Task<bool> CancelHealthExamBookingDBAsync(string code,string identityCard)
        {
            if (!string.IsNullOrWhiteSpace(code))
            {
                var filter = Builders<HealthExamBookingEntity>.Filter.Eq(x => x.Code, code);
                var entity = await _healthExamBookingDBRepository.FindOneAsync(filter);
                if (entity != null)
                {
                    var persons = entity.Persons.Where(x => x.IdentityCard != identityCard).ToList();
                    var update = Builders<HealthExamBookingEntity>.Update.Set(f => f.Persons, persons);
                    return await _healthExamBookingDBRepository.UpdateOneAsync(filter, update) > 0;
                }
            }
            return false;
        }


        /// <summary>
        /// 创建/更新健康检查预约数据
        /// </summary>
        /// <param name="request">创建请求</param>
        /// <returns></returns>
        public async Task<string> CreateOrUpdateHealthExamBookingDBAsync(HealthExamBookingDTO request)
        {
            var code = request.Code;
            var insertFlag = true;
            if (!string.IsNullOrWhiteSpace(code))
            {
                var filter = Builders<HealthExamBookingEntity>.Filter.Eq(x => x.Code, code);
                var entity = await _healthExamBookingDBRepository.FindOneAsync(filter);
                if (entity != null && !string.IsNullOrWhiteSpace(entity.Code))
                {
                    insertFlag = false;
                }
            }
            if (insertFlag)
            {
                var entity = request.MappingTo<HealthExamBookingEntity>();
                if (string.IsNullOrWhiteSpace(entity.Code))
                {
                    entity.Code = Guid.NewGuid().ToString("N");
                }
                return await _healthExamBookingDBRepository.InsertOneAsync(entity);
            }
            else
            {
                var persons = new List<HealthExamPersonEntity>();
                if (request.Persons?.Any() ?? false)
                {
                    foreach (var person in request.Persons)
                    {
                        persons.Add(new HealthExamPersonEntity
                        {
                            Name = person.Name,
                            Gender = person.Gender,
                            IdentityCard = person.IdentityCard,
                            Phone = person.Phone,
                        });
                    }
                }
                var examItems = new List<HealthExamItemEntity>();
                if (request.ExamItems?.Any() ?? false)
                {
                    foreach (var examItem in request.ExamItems)
                    {
                        examItems.Add(new HealthExamItemEntity
                        {
                            Code = examItem.Code,
                            Name = examItem.Name,
                            Content = examItem.Content,
                        });
                    }
                }
                var filter = Builders<HealthExamBookingEntity>.Filter.Eq(f => f.Code, code);
                var update = Builders<HealthExamBookingEntity>.Update
                    // .Set(f => f.BookingType, request.BookingType)
                    .Set(f => f.Name, request.Name ?? string.Empty)
                    .Set(f => f.Location, request.Location ?? string.Empty)
                    .Set(f => f.StartDate, request.StartDate)
                    .Set(f => f.EndDate, request.EndDate)
                    .Set(f => f.Subject, request.Subject ?? string.Empty)
                    .Set(f => f.Description, request.Description ?? string.Empty)
                    // .Set(f => f.InitiatorCode, request.InitiatorCode ?? string.Empty)
                    // .Set(f => f.OrgCode, request.OrgCode ?? string.Empty)
                    .Set(f => f.Status, request.Status)
                    .Set(f => f.ExamItems, examItems)
                    .Set(f => f.UpdateTime, request.UpdateTime);
                if(persons!=null && persons.Count>0)
                {
                    var updatePersons = Builders<HealthExamBookingEntity>.Update.Set(f => f.Persons, persons);
                    update = Builders<HealthExamBookingEntity>.Update.Combine(update, updatePersons);
                }
                var result = await _healthExamBookingDBRepository.UpdateOneAsync(filter, update) > 0;
                if (result)
                {
                    return code;
                }
            }
            return string.Empty;
        }

        /// <summary>
        /// 根据编码获取健康检查预约数据
        /// </summary>
        /// <param name="request">获取请求</param>
        /// <returns></returns>
        public async Task<HealthExamBookingDTO> GetHealthExamBookingDBAsync(GetHealthExamBookingDBRequest request)
        {
            var filter = Builders<HealthExamBookingEntity>.Filter.Eq(x => x.Code, request.Code);
            var entity = await _healthExamBookingDBRepository.FindOneAsync(filter);
            return entity.MappingTo<HealthExamBookingDTO>();
        }

        /// <summary>
        /// 获取最近提交/获取时间
        /// </summary>
        /// <returns></returns>
        public async Task<DateTime> GetLatestSyncTimeDBAsync(GetLatestSyncTimeDBRequest request)
        {
            var code = request.Code;
            var defaultTime = new DateTime(2010, 1, 1);
            if (!string.IsNullOrWhiteSpace(code))
            {
                var entity = await _dictionaryDBRepository.FindOneAsync("Code", code);
                if (entity != null && !string.IsNullOrWhiteSpace(entity.Code))
                {
                    try
                    {
                        var syncTimeValue = Newtonsoft.Json.JsonConvert.DeserializeObject<SyncTimeModel>(entity.Value);
                        return syncTimeValue.SyncTime;
                    }
                    catch
                    {
                        return new DateTime(2010, 1, 1);
                    }
                }
            }
            return defaultTime;
        }

        /// <summary>
        /// 更新最近提交/获取时间
        /// </summary>
        /// <returns></returns>
        public async Task<bool> UpdateLatestSyncTimeDBAsync(UpdateLatestSyncTimeDBRequest request)
        {
            var code = request.Code;
            var latestSyncTime = request.LatestSyncTime;
            var syncTimeValue = new SyncTimeModel { SyncTime = latestSyncTime };
            if (!string.IsNullOrWhiteSpace(code))
            {
                var dictionaryDO = await _dictionaryDBRepository.FindOneAsync("Code", code);
                if (dictionaryDO != null && !string.IsNullOrWhiteSpace(dictionaryDO.Code))
                {
                    dictionaryDO.Value = Newtonsoft.Json.JsonConvert.SerializeObject(syncTimeValue);
                    return await _dictionaryDBRepository.UpdateOneAsync("Code", dictionaryDO.Code, dictionaryDO, "", false);
                }
                else
                {
                    dictionaryDO = new DictionaryDO
                    {
                        Code = code,
                        DictionaryType = (int)DictionaryTypeEnum.VitalSyncType,
                        Value = Newtonsoft.Json.JsonConvert.SerializeObject(syncTimeValue),
                        ParentCode = string.Empty,
                        LanguageConfigs = new List<WingInterfaceLibrary.DTO.Dictionary.DictionaryLanguageConfigDTO>(),
                    };
                    var doCode = await _dictionaryDBRepository.InsertOneAsync(dictionaryDO, new Dictionary<string, string>(), string.Empty, false);
                    return !string.IsNullOrWhiteSpace(doCode);
                }
            }
            return false;
        }

        /// <summary>
        /// 根据录入人查询预约记录
        /// </summary>
        /// <param name="operatorCode"></param>
        /// <returns></returns>
        public async Task<List<HealthExamBookingDTO>> GetHealthExamBookingByInitiatorDBAsync(string operatorCode)
        {
            var filter = Builders<HealthExamBookingEntity>.Filter.Eq(x => x.InitiatorCode, operatorCode);
            var entitys = await _healthExamBookingDBRepository.FindAllAsync(filter);
            var list = entitys.MappingTo<List<HealthExamBookingDTO>>();
            return list;
        }

        /// <summary>
        /// 根据机构查询当日预约记录
        /// </summary>
        /// <param name="orgCode"></param>
        /// <returns></returns>
        public async Task<List<HealthExamBookingDTO>> GetHealthExamBookingByIDCardNoDBAsync(string orgCode)
        {
            var currentTime = DateTime.Now;
            var startTime = new DateTime(currentTime.Year, currentTime.Month, currentTime.Day).ToUniversalTime();
            var endTime = startTime.AddDays(1);
            var filter = Builders<HealthExamBookingEntity>.Filter.Where(x => x.IsDelete == false && x.OrgCode == orgCode && x.StartDate >= startTime && x.StartDate < endTime);
            var entitys = await _healthExamBookingDBRepository.FindAllAsync(filter);
            var list = entitys.MappingTo<List<HealthExamBookingDTO>>();
            return list;
        }

        /// <summary>
        /// 根据编码删除健康检查预约数据
        /// </summary>
        /// <param name="request">获取请求</param>
        /// <returns></returns>
        public async Task<bool> RemoveHealthExamBookingDBAsync(RemoveHealthExamBookingDBRequest request)
        {
            var code = request.Code;
            var filter = Builders<HealthExamBookingEntity>.Filter.Eq(f => f.Code, code);
            var update = Builders<HealthExamBookingEntity>.Update
                .Set(f => f.IsDelete, true)
                .Set(f => f.UpdateTime, request.UpdateTime);
            var result = await _healthExamBookingDBRepository.UpdateOneAsync(filter, update) > 0;
            return result;
        }

        /// <summary>
        /// 批量删除健康检查预约数据
        /// </summary>
        /// <param name="request">获取请求</param>
        /// <returns></returns>
        public async Task<bool> RemoveHealthExamBookingListDBAsync(RemoveHealthExamBookingListDBRequest request)
        {
            var filter = new List<(string key, string val)>();
            foreach (var code in request.Codes)
            {
                filter.Add(("Code", code));
            }
            return await _healthExamBookingDBRepository.DeleteAllAsync(filter) > 0;
        }

        /// <summary>
        /// 团队预约报名
        /// </summary>
        /// <param name="request">创建请求</param>
        /// <returns></returns>
        public async Task<bool> HealthExamBookingSignUpDBASync(HealthExamBookingSignUpDBRequest request)
        {
            var code = request.Code;
            var name = request.Name;
            var phone = request.Phone;
            var gender = request.Gender;
            var identityCard = request.IdentityCard;

            var filter = Builders<HealthExamBookingEntity>.Filter.Eq(x => x.Code, code);
            var entity = await _healthExamBookingDBRepository.FindOneAsync(filter);
            if (entity != null && !string.IsNullOrWhiteSpace(entity.Code))
            {
                if(entity.EndDate < DateTime.UtcNow)
                {
                    return false;
                }
                var persons = entity.Persons;
                var currentPerson = persons.FirstOrDefault(x => x.Phone == phone && x.Name == name || x.IdentityCard == identityCard);
                if (currentPerson != null)
                {
                    ThrowRpcException((int)CustomerRpcCode.SignedUp, "SignedUp");
                    return false;
                }
                else
                {
                    persons.Add(new HealthExamPersonEntity
                    {
                        Name = name,
                        Phone = phone,
                        Gender = gender,
                        IdentityCard = identityCard,
                    });
                }
                var filter2 = Builders<HealthExamBookingEntity>.Filter.Eq(f => f.Code, code);
                var update = Builders<HealthExamBookingEntity>.Update
                    .Set(f => f.Persons, persons)
                    .Set(f => f.UpdateTime, request.UpdateTime);
                await _healthExamBookingDBRepository.UpdateOneAsync(filter2, update);
                return true;
            }
            return false;
        }

        /// <summary>
        /// 团队预约取消报名
        /// </summary>
        /// <param name="request">创建请求</param>
        /// <returns></returns>
        public async Task<bool> HealthExamBookingCancelSignUpDBASync()
        {
            return true;
        }

        /// <summary>
        /// 团队预约修改体检人员信息
        /// </summary>
        /// <param name="request">创建请求</param>
        /// <returns></returns>
        public async Task<bool> HealthExamBookingModifyPersonDBASync()
        {
            return true;
        }

        /// <summary>
        /// 根据机构codes查询所有的预约信息
        /// </summary>
        /// <param name="request">创建请求</param>
        /// <returns></returns>
        public async Task<IList<HealthExamBookingDTO>> GetHealthExamBookingListByOrgCodesDBAsync(List<string> orgCodes)
        {
            var dataList = new List<HealthExamBookingDTO>();
            if (orgCodes?.Count > 0)
            {
                var filter = Builders<HealthExamBookingEntity>.Filter.Where(x => x.IsDelete == false && orgCodes.Contains(x.OrgCode));
                var entitys = await _healthExamBookingDBRepository.FindAllAsync(filter);
                if (entitys?.Count > 0)
                {
                    dataList = entitys.MappingTo<List<HealthExamBookingDTO>>();
                }
            }
            return dataList;
        }
    }

    public class SyncTimeModel
    {
        public DateTime SyncTime { get; set; }
    }
}