﻿using System;
using System.Collections.Generic;
using System.Data;
using System.IO;
using System.Linq;
using System.Threading.Tasks;
using MongoDB.Bson;
using MongoDB.Driver;
using MongoDB.Driver.GeoJsonObjectModel;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using WingInterfaceLibrary.DB.Request;
using WingInterfaceLibrary.DTO.Common;
using WingInterfaceLibrary.DTO.Management;
using WingInterfaceLibrary.DTO.Organization;
using WingInterfaceLibrary.DTO.Report;
using WingInterfaceLibrary.DTO.ReportTemplate;
using WingInterfaceLibrary.DTO.Schedule;
using WingInterfaceLibrary.DTO.User;
using WingInterfaceLibrary.Enum;
using WingInterfaceLibrary.Interface.DBInterface;
using WingInterfaceLibrary.Request;
using WingInterfaceLibrary.Request.DBRequest;
using WingInterfaceLibrary.Request.Management;
using WingInterfaceLibrary.Request.User;
using WingInterfaceLibrary.Result;
using WingMongoDBModule.Common;
using WingMongoDBModule.DO;
using WingMongoDBModule.Entities;
using WingMongoDBModule.Entity;
using WingMongoDBModule.Mongodb.Driver;
using WingMongoDBModule.Request;
using WingServerCommon.Config;
using WingServerCommon.Config.Parameters;
using WingServerCommon.Interfaces.Cache;
using WingServerCommon.Log;
using WingServerCommon.Mapper;
using WingServerCommon.Service;
using WingServerCommon.Utilities;

namespace WingMongoDBModule.Service
{
    public partial class DatabaseService : JsonRpcService, IUserDBService
    {
        /// <summary>
        /// 新增用户
        /// </summary>
        /// <param name="userInfoDO">用户信息</param>
        /// <param name="extensionData">扩展信息</param>
        /// <returns>Id</returns>
        public async Task<string> InsertUserAsync(CreateUserInfoDBRequest request)
        {
            var userDO = request.Data.MappingTo<UserInfoDO>();
            if (!string.IsNullOrEmpty(request.Data.HeadImageUrl))
            {
                userDO.HeadImageToken = request.Data.HeadImageUrl.ToUrlToken();
            }
            if (userDO.SecretPassword.StartsWith("Symmetry_"))
            {
                userDO.SecretPassword = userDO.SecretPassword;
            }
            else
            {
                userDO.SecretPassword = GetMd5SaltPassword(userDO.UserName, userDO.SecretPassword);
            }
            var result = _userDBRepository.InsertOneAsync(userDO, request.PlatformDatas, request.ExtensionData);
            await AddPushUserAsync(userDO);
            return await result;
        }

        /// <summary>
        /// 根据用户名查找用户
        /// </summary>
        /// <param name="name">用户名</param>
        public async Task<UserDTO> FindUserByNameAsync(string name)
        {
            var result = await _userDBRepository.FindOneAsync("UserName", name);
            var userDTO = result.MappingTo<UserDTO>();
            if (userDTO != null)
            {
                SetUserExtInfo(userDTO, false);
            }
            return userDTO;
        }

        /// <summary>
        /// 根据用户名查找用户
        /// </summary>
        /// <param name="name">用户名</param>
        public async Task<UserDTO> FindUserByCommonPlatformUserIdAsync(string commonPlatformUserId)
        {
            UserDTO userDTO = null;
            if (!string.IsNullOrWhiteSpace(commonPlatformUserId))
            {
                MongoCustomerFilter<MongoTable> customerFilter = new();
                customerFilter.AppendEq(("BaseData.CommonPlatformUserId", commonPlatformUserId));
                var entities = await _userDBRepository.FindAllAsync(customerFilter.AndCombine(), new FindOptions<MongoTable> { NoCursorTimeout = true });
                if (entities != null && entities.Any())
                {
                    userDTO = entities.FirstOrDefault().MappingTo<UserDTO>();
                    SetUserExtInfo(userDTO, false);
                }
            }
            return userDTO;
        }

        /// <summary>
        /// 根据OpenId查找用户
        /// </summary>
        /// <param name="name">微信OpenId</param>
        public async Task<UserDTO> FindUserByOpenIdAsync(string openId)
        {
            UserDTO userDTO = null;
            if (!string.IsNullOrWhiteSpace(openId))
            {
                MongoCustomerFilter<MongoTable> customerFilter = new();
                customerFilter.AppendEq(("BaseData.OpenId", openId));
                var entities = await _userDBRepository.FindAllAsync(customerFilter.AndCombine(), new FindOptions<MongoTable> { NoCursorTimeout = true });
                if (entities != null && entities.Any())
                {
                    userDTO = entities.FirstOrDefault().MappingTo<UserDTO>();
                    SetUserExtInfo(userDTO, false);
                }
            }
            return userDTO;
        }

        /// <summary>
        /// 根据LabDongle查找用户
        /// </summary>
        /// <param name="name">LabDongle</param>
        public async Task<UserDTO> FindUserByLabDongleDBAsync(string labDongle)
        {
            UserDTO userDTO = null;
            if (!string.IsNullOrWhiteSpace(labDongle))
            {
                MongoCustomerFilter<MongoTable> customerFilter = new();
                customerFilter.AppendEq(("BaseData.LabDongle", labDongle));
                var entities = await _userDBRepository.FindAllAsync(customerFilter.AndCombine(), new FindOptions<MongoTable> { NoCursorTimeout = true });
                if (entities != null && entities.Any())
                {
                    userDTO = entities.FirstOrDefault().MappingTo<UserDTO>();
                    SetUserExtInfo(userDTO, false);
                }
            }
            return userDTO;
        }

        /// <summary>
        /// 绑定/解绑用户OpenId
        /// </summary>
        /// <param name="name">微信OpenId</param>
        public async Task<bool> SetUserOpenIdAsync(string userCode, string openId)
        {
            var entity = await _userDBRepository.FindOneAsync("Code", userCode);
            if (entity != null && !string.IsNullOrWhiteSpace(entity.Code))
            {
                entity.OpenId = openId ?? string.Empty;
                return await _userDBRepository.UpdateOneAsync("Code", userCode, entity, "", true);
            }
            return false;
        }

        /// <summary>
        /// 根据用户编码查询用户
        /// </summary>
        /// <param name="code">用户编码</param>
        public async Task<UserDTO> FindUserByCodeAsync(string code)
        {
            var userInfo = await _userDBRepository.FindOneAsync("Code", code);
            var userDTO = userInfo.MappingTo<UserDTO>();
            if (userDTO != null)
            {
                SetUserExtInfo(userDTO, false);
            }
            return userDTO;
        }

        /// <summary>
        /// 根据用户编码查询用户
        /// </summary>
        /// <param name="code">用户编码</param>
        public async Task<UserDTO> FindUserDTOByCodeAsync(string code, bool isNeedExtendInfo)
        {
            if (string.IsNullOrEmpty(code))
            {
                return new UserDTO();
            }
            var userInfo = await _userDBRepository.FindOneAsync("Code", code);
            var userDTO = userInfo.MappingTo<UserDTO>();
            if (userDTO != null && isNeedExtendInfo)
            {
                SetUserExtInfo(userDTO, false);
            }
            return userDTO;
        }

        /// <summary>
        /// 根据code查询用户
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<UserDTO> FindUserAsync(FindUserRequest request)
        {
            var userInfo = await _userDBRepository.FindOneAsync("Code", request.Code);
            var userDTO = userInfo.MappingTo<UserDTO>();
            if (userDTO != null)
            {
                SetUserExtInfo(userDTO, request.IsEncryptedShow);
            }
            return userDTO;
        }

        /// <summary>
        /// 根据用户编码查询用户密码
        /// </summary>
        /// <param name="code">用户编码</param>
        public async Task<UserPasswordDTO> FindUserPasswordByCodeAsync(string code)
        {
            var userInfo = await _userDBRepository.FindOneAsync("Code", code);
            var userDTO = userInfo.MappingTo<UserPasswordDTO>();

            return userDTO;
        }

        /// <summary>
        /// 设置用户信息（机构、机构负责人、隐私保护等）
        /// </summary>
        /// <param name="userInfo"></param>
        /// <param name="isEncryptedShow">是否按机构配置做隐私保护</param>
        private void SetUserExtInfo(UserDTO userInfo, bool isEncryptedShow = false)
        {
            if (!string.IsNullOrWhiteSpace(userInfo.OrganizationCode))
            {
                var orgInfo = CacheMaintenance.Instance.Get<IOrganizationsManager>().Get(userInfo.OrganizationCode);
                if (!string.IsNullOrEmpty(orgInfo?.Code))
                {
                    if (orgInfo.OrganizationType == (int)OrganizationTypeEnum.Department)
                    {
                        userInfo.DepartmentName = orgInfo.OrganizationName;
                        userInfo.DepartmentCode = orgInfo.Code;
                        userInfo.DepartmentShortCode = orgInfo.ShortCode;
                        if (!string.IsNullOrWhiteSpace(orgInfo.RootCode))
                        {
                            userInfo.RootOrganizationCode = orgInfo.RootCode;
                            var org = CacheMaintenance.Instance.Get<IOrganizationsManager>().Get(userInfo.RootOrganizationCode);
                            if (!string.IsNullOrEmpty(org?.Code))
                            {
                                userInfo.RootOrganizationName = org.OrganizationName;
                                userInfo.RootOrganizationShortCode = org.ShortCode;
                                if (org.Directors != null && org.Directors.Contains(userInfo.UserCode))
                                {
                                    userInfo.IsDirector = true;
                                }
                                if (isEncryptedShow && org.IsEncryptedShow)
                                {
                                    userInfo.Phone = CryptionHelper.PhoneEncrypt(userInfo.Phone);
                                    userInfo.FullName = CryptionHelper.NameEncrypt(userInfo.FullName);
                                }
                            }
                        }
                    }
                    else if (orgInfo.OrganizationType == (int)OrganizationTypeEnum.Corporation)
                    {
                        userInfo.RootOrganizationCode = orgInfo.Code;
                        userInfo.RootOrganizationName = orgInfo.OrganizationName;
                        userInfo.RootOrganizationShortCode = orgInfo.ShortCode;
                        if (orgInfo.Directors != null && orgInfo.Directors.Contains(userInfo.UserCode))
                        {
                            userInfo.IsDirector = true;
                        }
                        if (isEncryptedShow && orgInfo.IsEncryptedShow)
                        {
                            userInfo.Phone = CryptionHelper.PhoneEncrypt(userInfo.Phone);
                            userInfo.FullName = CryptionHelper.NameEncrypt(userInfo.FullName);
                        }
                    }
                }
                else
                {
                    userInfo.DepartmentName = string.Empty;
                    userInfo.DepartmentCode = string.Empty;
                    userInfo.DepartmentShortCode = string.Empty;
                    userInfo.RootOrganizationCode = string.Empty;
                    userInfo.RootOrganizationName = string.Empty;
                    userInfo.RootOrganizationShortCode = string.Empty;
                }
            }
        }

        /// <summary>
        /// 删除用户
        /// </summary>
        /// <param name="id">用户Id</param>
        public async Task<bool> DeleteUserByCodeAsync(string code)
        {
            using (new Performance(_performanceThreshold, "{0} DeleteUserByCodeAsync execute", $"{this.GetType().Name}"))
            {
                return await _userDBRepository.DeleteOneAsync("Code", code);
            }
        }

        /// <summary>
        /// 更新用户
        /// </summary>
        /// <param name="code"></param>
        /// <param name="data"></param>
        /// <param name="extensionData"></param>
        /// <param name="canNull">是否可更新为空</param>
        /// <returns></returns>
        public async Task<bool> UpdateUserAsync(string code, UserPasswordDTO data, string extensionData, bool canNull = false, bool isChangeHeadImg = true)
        {
            var userDO = data.MappingTo<UserInfoDO>();

            var oldUserDO = await _userDBRepository.FindOneAsync("Code", code);
            var organizationChanged = oldUserDO.OrganizationCode != data.OrganizationCode;

            if (!string.IsNullOrEmpty(data.HeadImageUrl) && isChangeHeadImg)
            {
                userDO.HeadImageToken = data.HeadImageUrl.ToUrlToken();
            }
            if (userDO.PasswordUpdateTime <= Convert.ToDateTime("2000-1-1"))
            {
                var userInfo = await FindUserPasswordByCodeAsync(code);
                if (userInfo != null)
                {
                    userDO.PasswordUpdateTime = userInfo.PasswordUpdateTime;
                    userDO.PasswordRecords = userInfo.PasswordRecords;
                }
            }
            var result = await _userDBRepository.UpdateOneAsync("Code", code, userDO, extensionData, canNull);
            if (organizationChanged)
            {
                await AddPushUserAsync(userDO);
            }
            return result;
        }

        /// <summary>
        /// 更新用户登录信息
        /// </summary>
        /// <param name="code"></param>
        /// <param name="data"></param>
        /// <param name="extensionData"></param>
        /// <param name="canNull">是否可更新为空</param>
        /// <returns></returns>
        public async Task<bool> UpdateUserLoginInfoAsync(UpdateUserLoginInfoDBRequest request)
        {
            var userInfo = await _userDBRepository.FindOneAsync("Code", request.Code);
            if (userInfo != null)
            {
                userInfo.Logintimes = request.LoginTimes;
                userInfo.UserState = request.UserState;
                userInfo.LoginLockInfo = request.LoginLockInfo.MappingTo<LoginLockInfoDO>();
            }
            return await _userDBRepository.UpdateOneAsync("Code", request.Code, userInfo, "", true);
        }
        /// <summary>
        /// 更新密码
        /// </summary>
        /// <param name="code"></param>
        /// <param name="data"></param>
        /// <returns></returns>
        public async Task<bool> UpdateUserPasswordAsync(string code, string password, IList<string> pwdRecords = null, DateTime? updateTime = null)
        {
            var userDO = await _userDBRepository.FindOneAsync("Code", code);
            if (password.StartsWith("Symmetry_"))
            {
                userDO.SecretPassword = password;
            }
            else
            {
                userDO.SecretPassword = GetMd5SaltPassword(userDO.UserName, password);
            }
            if (pwdRecords?.Count > 0)
            {
                userDO.PasswordRecords = pwdRecords.ToList();
            }
            if (updateTime != null && updateTime != DateTime.MinValue && updateTime > Convert.ToDateTime("2000-1-1"))
            {
                userDO.PasswordUpdateTime = updateTime.Value;
            }
            userDO.LoginLockInfo = new LoginLockInfoDO();
            return await _userDBRepository.UpdateOneAsync("Code", code, userDO, "", false);
        }
        /// <summary>
        /// 查找用户列表（分页）
        /// </summary>
        /// <param name="request">分页信息</param>
        /// <returns></returns>
        public async Task<PageCollection<UserDTO>> FindUserPagesAsync(PageFilterRequest request)
        {
            var dbRequest = GetDBPageRequest(request);
            var pageInfo = await _userDBRepository.GetPages(dbRequest);
            return pageInfo.MappingTo<PageCollection<UserDTO>>();
        }

        /// <summary>
        /// 查找用户列表（分页）
        /// </summary>
        /// <param name="request">分页信息</param>
        /// <returns></returns>
        public async Task<PageCollection<UserDTO>> FindUserPagesByOrgsAsync(FindUserPageByOrgsRequest request)
        {
            var filter = GetFilter(request);
            var dbPage = new DBPageRequest();
            dbPage.CurrentPage = request.CurrentPage;
            dbPage.PageSize = request.PageSize;
            dbPage.Filter = filter;
            var result = await _userDBRepository.GetPages(dbPage);
            var resData = PageCollectionConvert<UserInfoDO, UserDTO>(result);
            if (resData?.PageData?.Count > 0)
            {
                var resPackage = await GetChargeResourcePackageByConditionDBAsync(ChargeResourcePackageTypeEnum.Probation) ?? new ChargeResourcePackageDTO();
                foreach (var dataItem in resData.PageData)
                {
                    if (dataItem?.BindChargeResourcePackages?.Count > 0)
                    {
                        foreach (var packageInfo in dataItem?.BindChargeResourcePackages)
                        {
                            if (!string.IsNullOrEmpty(packageInfo?.ResourcePackageCode))
                            {
                                var findInfo = await GetChargeResourcePackageByCodeDBAsync(packageInfo.ResourcePackageCode);
                                if (!string.IsNullOrEmpty(findInfo.Name))
                                {
                                    packageInfo.ResourcePackageName = findInfo.Name;
                                }
                            }
                        }
                    }
                    if (dataItem?.BindChargeResourcePackageRecords?.Count > 0)
                    {
                        dataItem.IsProbationChargeResourcePackage = dataItem.BindChargeResourcePackageRecords.Exists(c => c.ResourcePackageCode == resPackage.Code);
                    }
                }
            }
            return resData;
        }

        private FilterDefinition<MongoTable> GetFilter(FindUserPageByOrgsRequest request)
        {
            List<FilterDefinition<MongoDataItem>> queryItemsFilterList = new();
            MongoCustomerFilter<MongoTable> customerFilter = new();
            MongoCustomerFilter<MongoDataItem> queryItemsFilter = new();
            if (request.OrgList != null && request.OrgList.Count > 0)
            {
                queryItemsFilter.AppendEq((item => item.Key, "OrganizationCode"));
                queryItemsFilter.AppendIn(item => item.Value, request.OrgList);
                if (queryItemsFilter.HasFilter)
                {
                    customerFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
                }
            }
            if (request.ExcludeUserCodes != null && request.ExcludeUserCodes.Count > 0)
            {
                customerFilter.AppendNotIn("BaseData.Code", request.ExcludeUserCodes);
            }
            if (!string.IsNullOrEmpty(request.UserCode))
            {
                customerFilter.AppendEq(("BaseData.Code", request.UserCode));
            }
            if (request.RoleCodes?.Count > 0)
            {
                customerFilter.AppendAnyIn("BaseData.RoleCodes", request.RoleCodes);
            }

            if (request.IsAgent != null)
            {
                if (request.IsAgent == true)
                {
                    customerFilter.AppendEq(("BaseData.IsOldAgent", request.IsAgent.Value));
                }
                else
                {
                    customerFilter.AppendNot(("BaseData.IsOldAgent", true));
                }
            }
            if (!string.IsNullOrEmpty(request.LabDongle))
            {
                customerFilter.AppendEq(("BaseData.LabDongle", request.LabDongle));
            }
            else
            {
                foreach (var kv in request.Filter)
                {
                    if (!string.IsNullOrWhiteSpace(kv.Key) && !string.IsNullOrWhiteSpace(kv.Value))
                    {
                        MongoCustomerFilter<MongoDataItem> tempQueryItemsFilter = new();
                        tempQueryItemsFilter.AppendEq((item => item.Key, kv.Key));
                        tempQueryItemsFilter.AppendRegex(item => item.Value, kv.Value, true, false);
                        queryItemsFilterList.Add(tempQueryItemsFilter.AndCombine());
                    }
                }
            }
            if (request.OnlyDongle)
            {
                customerFilter.AppendAnyNotIn("BaseData.LabDongle", new List<string> { null, "" });
            }
            if (queryItemsFilterList.Count > 0)
            {
                var queryItemAllFilter = queryItemsFilter.OrCombine(queryItemsFilterList.ToArray());
                customerFilter.SubFilter(item => item.QueryData.QueryItems, queryItemAllFilter);
            }
            return customerFilter.AndCombine();
        }

        /// <summary>
        /// 查找用户列表（分页）自定义条件
        /// </summary>
        /// <param name="request">分页信息</param>
        /// <returns></returns>
        public async Task<PageCollection<UserDTO>> FindUserPagesCustomAsync(PageFilterRequest request)
        {
            var dbRequest = GetDBPageCustomRequest(request);
            var pageInfo = await _userDBRepository.GetPages(dbRequest);
            return pageInfo.MappingTo<PageCollection<UserDTO>>();
        }

        /// <summary>
        /// 生成分页查询条件
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        internal DBPageRequest GetDBPageCustomRequest(PageFilterRequest request)
        {
            var customerFilter = new MongoCustomerFilter<MongoTable>();
            var mongoDataItemFilter = new MongoCustomerFilter<MongoDataItem>();
            foreach (var para in request.Filter)
            {
                if (para.Key == "DeviceCode")
                {
                    customerFilter.AppendArrayEq("BaseData.BindDevices", para.Value);
                }
                else
                {
                    if (request.IsFuzzy)
                    {
                        mongoDataItemFilter.AppendEq((item => item.Key, para.Key));
                        mongoDataItemFilter.AppendRegex(item => item.Value, para.Value, request.IsFuzzy);
                    }
                    else
                    {
                        mongoDataItemFilter.AppendEq((item => item.Key, para.Key), (item => item.Value, para.Value));
                    }
                    customerFilter.SubFilter(item => item.QueryData.QueryItems, mongoDataItemFilter.AndCombine());
                    mongoDataItemFilter.ClearFilter();
                }
            }
            SortDefinitionBuilder<MongoTable> builderSort = Builders<MongoTable>.Sort;
            SortDefinition<MongoTable> sort = builderSort.Descending("BaseData.CreateTime");
            var dbPageRequest = new DBPageRequest();
            dbPageRequest.CurrentPage = request.CurrentPage;
            dbPageRequest.PageSize = request.PageSize;
            dbPageRequest.Sort = sort;
            dbPageRequest.Filter = customerFilter.AndCombine();
            return dbPageRequest;
        }

        /// <summary>
        /// 根据手机查找用户
        /// </summary>
        /// <param name="phone">手机号</param>
        /// <returns></returns>
        public async Task<UserDTO> FindUserByPhoneAsync(string phone)
        {
            var result = await _userDBRepository.FindOneAsync("Phone", phone);
            return result.MappingTo<UserDTO>();
        }

        /// <summary>
        /// 根据邮箱地址查找用户
        /// </summary>
        /// <param name="email">邮箱地址</param>
        /// <returns></returns>
        public async Task<UserDTO> FindUserByEmailAsync(string email)
        {
            var result = await _userDBRepository.FindOneAsync("Email", email);
            return result.MappingTo<UserDTO>();
        }

        /// <summary>
        /// 根据角色查询用户列表
        /// </summary>
        /// <param name="roleCode"></param>
        /// <returns></returns>
        public async Task<List<UserDTO>> FindUserListByRoleCodeAsync(string roleCode)
        {
            MongoCustomerFilter<MongoTable> customerFilter = new();
            MongoCustomerFilter<MongoDataItem> queryItemsFilter = new();
            customerFilter.AppendArrayEq("BaseData.RoleCodes", roleCode);
            var userList = await _userDBRepository.FindAllAsync(customerFilter.AndCombine(), new FindOptions<MongoTable> { NoCursorTimeout = true });
            return userList.MappingTo<List<UserDTO>>();
        }

        public async Task<long> UpdateUsersAsync(UpdateUsersDBRequest request)
        {
            if (request.Users?.Count > 0)
            {
                var userCodes = request.Users.FindAll(d => d.Data.PasswordUpdateTime <= Convert.ToDateTime("2000-1-1"))
                                    .Select(d => d.Data.UserCode).Distinct().ToList();
                if (userCodes?.Count > 0)
                {
                    var userList = await FindUsersByUserCodesAsync(userCodes);
                    if (userList?.Count > 0)
                    {
                        foreach (var item in request.Users)
                        {
                            if (item.Data.PasswordUpdateTime <= Convert.ToDateTime("2000-1-1"))
                            {
                                var curUserInfo = userList.Find(c => c.UserCode == item.Data.UserCode) ?? new UserPasswordDTO();
                                item.Data.PasswordUpdateTime = curUserInfo.PasswordUpdateTime;
                                item.Data.PasswordRecords = curUserInfo.PasswordRecords;
                            }
                        }
                    }
                }
            }
            var res = false;
            request.Users.ForEach(info =>
            {
                var userDO = info.Data.MappingTo<UserInfoDO>();
                if (!string.IsNullOrEmpty(info.Data.HeadImageUrl))
                {
                    userDO.HeadImageToken = info.Data.HeadImageUrl.ToUrlToken();
                }
                //return ("Code", info.Data.UserCode, userDO, info.ExtensionData);
                res = _userDBRepository.UpdateOneAsync("Code", info.Data.UserCode, userDO, info.ExtensionData).Result;
            });
            var result = res ? 1 : 0;
            return result;
        }

        public async Task<List<UserDTO>> FindUsersAsync(GetUsersDBRequest request)
        {
            var dic = new Dictionary<string, string>();
            if (!string.IsNullOrWhiteSpace(request.OrganizationCode))
            {
                dic.Add("OrganizationCode", request.OrganizationCode);
            }
            var userList = await _userDBRepository.FindListAsync(dic);
            userList = userList.Where(f => (f.RankCodes.Any(rank => rank == request.RankCode
            ) || string.IsNullOrWhiteSpace(request.RankCode))
            && (f.RoleCodes.Any(role => role == request.RoleCode
            ) || string.IsNullOrWhiteSpace(request.RoleCode))).ToList();
            return userList.MappingTo<List<UserDTO>>();
        }

        /// <summary>
        /// 根据机构codes获取用户列表
        /// </summary>
        /// <param name="OrganizationCodes"></param>
        /// <returns></returns>
        public async Task<List<UserDTO>> FindUserListByOrganizationCodesAsync(List<string> organizationCodes)
        {
            var dic = new Dictionary<string, List<string>>();
            dic.Add("OrganizationCode", organizationCodes);
            var userList = await _userDBRepository.FindInAsync(dic);
            return userList.MappingTo<List<UserDTO>>();
        }

        /// <summary>
        /// 根据机构codes获取用户列表
        /// </summary>
        /// <param name="OrganizationCode"></param>
        /// <returns></returns>
        public async Task<List<UserDTO>> FindUsersByOrganizationCodeAsync(string organizationCode)
        {
            var newOrganizations = new List<string>();
            var organizationInfo = _organizationDBRepository.GetOrganizationByCodeAsync(organizationCode);
            newOrganizations.Add(organizationInfo.Result.RootCode);
            newOrganizations.Add(organizationCode);
            var dic = new Dictionary<string, List<string>>();
            dic.Add("OrganizationCode", newOrganizations);
            var userList = await _userDBRepository.FindInAsync(dic);
            return userList.MappingTo<List<UserDTO>>();
        }

        /// <summary>
        /// 根据codes获取用户列表
        /// </summary>
        /// <param name="codes"></param>
        /// <returns></returns>
        public async Task<List<UserDTO>> FindUserListByUserCodesAsync(List<string> codes)
        {
            if (codes == null || codes.Count == 0)
            {
                return new List<UserDTO>();
            }
            var dic = new Dictionary<string, List<string>>();
            dic.Add("Code", codes);
            var userList = await _userDBRepository.FindInAsync(dic);
            return userList.MappingTo<List<UserDTO>>();
        }


        /// <summary>
        /// 根据codes获取用户列表
        /// </summary>
        /// <param name="codes"></param>
        /// <returns></returns>
        private async Task<List<UserInfoDO>> GetUserListByUserCodesAsync(List<string> codes)
        {
            if (codes == null || codes.Count == 0)
            {
                return new List<UserInfoDO>();
            }
            var dic = new Dictionary<string, List<string>>();
            dic.Add("Code", codes);
            var userList = await _userDBRepository.FindInAsync(dic);
            return userList;
        }

        /// <summary>
        /// 根据codes获取用户DO列表
        /// </summary>
        /// <param name="codes"></param>
        /// <returns></returns>
        public async Task<List<UserInfoDO>> FindUserDOListByUserCodesAsync(List<string> codes)
        {
            if (codes == null || codes.Count == 0)
            {
                return new List<UserInfoDO>();
            }
            var dic = new Dictionary<string, List<string>>();
            dic.Add("Code", codes);
            var userList = await _userDBRepository.FindInAsync(dic);
            return userList;
        }

        /// <summary>
        /// 根据codes获取用户列表
        /// </summary>
        /// <param name="codes"></param>
        /// <returns></returns>
        public async Task<List<UserPasswordDTO>> FindUsersByUserCodesAsync(List<string> codes)
        {
            if (codes == null || codes.Count == 0)
            {
                return new List<UserPasswordDTO>();
            }
            var dic = new Dictionary<string, List<string>>();
            dic.Add("Code", codes);
            var userList = await _userDBRepository.FindInAsync(dic);
            return userList.MappingTo<List<UserPasswordDTO>>();
        }

        /// <summary>
        /// 将设备绑定到用户中
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<long> BingDevicesToUsersAsync(BingDevicesToUsersRequest request)
        {
            MongoCustomerFilter<MongoDataItem> subFilter = new();
            subFilter.AppendEq((item => item.Key, "Code"));
            subFilter.AppendIn(item => item.Value, request.UserCodes);

            MongoCustomerFilter<MongoTable> filter = new();
            filter.SubFilter<MongoDataItem>(item => item.QueryData.QueryItems, subFilter.AndCombine());

            MongoCustomerUpdate<MongoTable> update = new();
            update.AddArrayToSubItem("BaseData.BindDevices", request.DeviceCodes);
            if (request.OrganizationCodes?.Count > 0)
            {
                update.AddArrayToSubItem("BaseData.BindDeviceOrganizations", request.OrganizationCodes.ToList());
            }
            return await _userDBRepository.UpdateAllAsync(filter.AndCombine(), update.AllUpdateDefinition);
        }

        /// <summary>
        /// 验证用户密码是否正确
        /// </summary>
        /// <param name="UserCode"></param>
        /// <param name="password"></param>
        /// <returns></returns>
        public async Task<bool> VerificationPasswordAsync(string UserCode, string password)
        {
            var userDO = await _userDBRepository.FindOneAsync("Code", UserCode);
            if (userDO != null)
            {
                if (userDO.SecretPassword.StartsWith("ServerSalt_"))
                {
                    var saltPassword = GetMd5SaltPassword(userDO.UserName, password);
                    return userDO.SecretPassword == saltPassword;
                }
                if (password.StartsWith("Symmetry_"))
                {
                    if (userDO.SecretPassword.StartsWith("Old_"))
                    {
                        var passwd = CryptionHelper.Decrypt(password.Replace("Symmetry_", ""));
                        var oldPasswd = $"Old_{OldSecurity.Hash(OldSecurity.Salt(passwd))}";
                        if (userDO.SecretPassword == oldPasswd)
                        {
                            var newPasswd = $"Symmetry_{CryptionHelper.Encrypt(passwd)}";
                            return await UpdateUserPasswordAsync(UserCode, newPasswd);
                        }
                    }
                }
                return userDO.SecretPassword == password;
            }
            return false;
        }

        /// <summary>
        /// 获取设备用户列表（分页）
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageResult<ManageDeviceUserDTO>> GetDeviceUserPagesAsync(GetDeviceUserPagesRequest request)
        {
            MongoCustomerFilter<MongoTable> customerFilter = await GetDeviceUserPagesFilter(request);
            PageResult<ManageDeviceUserDTO> result = new PageResult<ManageDeviceUserDTO>();
            result.PageIndex = request.PageIndex;
            result.PageSize = request.PageSize;
            result.TotalCount = (int)await _userDBRepository.CountDocumentsAsync(customerFilter.AndCombine());
            SortDefinitionBuilder<MongoTable> builderSort = Builders<MongoTable>.Sort;
            SortDefinition<MongoTable> sort = builderSort.Ascending("BaseData.OrganizationCode").Descending("BaseData.CreateTime");
            var userInfoList = await _userDBRepository.FindAllAsync(customerFilter.AndCombine(), new FindOptions<MongoTable> { NoCursorTimeout = true, Skip = (request.PageIndex - 1) * request.PageSize, Limit = request.PageSize, Sort = sort });
            if (userInfoList != null && userInfoList.Count > 0)
            {
                var organizationCodes = userInfoList.Select(t => t.OrganizationCode).Where(o => !string.IsNullOrWhiteSpace(o));
                if (organizationCodes.Count() > 0)
                {
                    var orgInfos = await FindOrganizationDOListByCodesAsync(organizationCodes.ToList());
                    var rootOrgInfos = await FindOrganizationDOListByCodesAsync(orgInfos.Select(o => o.RootCode).ToList());
                    foreach (var item in userInfoList)
                    {
                        var manageDeviceUserDTO = item.MappingTo<ManageDeviceUserDTO>();
                        var orgInfo = orgInfos.FirstOrDefault(o => o.OrganizationCode == item.OrganizationCode);
                        if (orgInfo?.OrganizationType == OrganizationTypeEnum.Corporation)
                        {
                            manageDeviceUserDTO.HospitalName = orgInfo?.OrganizationName;
                        }
                        else
                        {
                            var rootOrgInfo = rootOrgInfos.FirstOrDefault(r => r.OrganizationCode == orgInfo?.RootCode);
                            manageDeviceUserDTO.HospitalName = rootOrgInfo?.OrganizationName;
                        }
                        result.PageData.Add(manageDeviceUserDTO);
                    }
                }
            }
            return result;
        }

        /// <summary>
        /// 获取设备用户列表（分页）条件拼接
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        private async Task<MongoCustomerFilter<MongoTable>> GetDeviceUserPagesFilter(GetDeviceUserPagesRequest request)
        {
            MongoCustomerFilter<MongoTable> customerFilter = new();
            MongoCustomerFilter<MongoDataItem> queryItemsFilter = new();

            if (request.Type == 0)
            {
                //任何设备可以分配给任何人
                customerFilter.AppendAnyNotIn("BaseData.BindDevices", request.DeviceCodes);
            }
            else
            {
                //customerFilter.AppendArrayEq("BaseData.BindDevices", request.DeviceCodes);
                customerFilter.AppendAnyIn("BaseData.BindDevices", request.DeviceCodes);
            }
            if (!string.IsNullOrWhiteSpace(request.QueryType) && !string.IsNullOrWhiteSpace(request.Keyword))
            {
                queryItemsFilter = await GetUserFilter(request.QueryType, request.Keyword);
            }
            if (queryItemsFilter.HasFilter)
            {
                customerFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
            }
            queryItemsFilter.ClearFilter();
            queryItemsFilter.AppendEq((item => item.Key, "OrganizationCode"));
            queryItemsFilter.AppendNot((item => item.Value, ""));
            customerFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
            return customerFilter;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        private async Task<MongoCustomerFilter<MongoTable>> GetUserDBPagesFilter(GetDeviceUserPagesRequest request)
        {
            MongoCustomerFilter<MongoTable> customerFilter = new();
            MongoCustomerFilter<MongoDataItem> queryItemsFilter = new();
            //todo 超级管理员和普通管理员查询条件应该不一样（看机构负责人）
            var code = request.DeviceCodes?.FirstOrDefault() ?? string.Empty;
            var deviceModel = await FindDeviceInfoByCodeAsync(code);
            var allOrgList = await SearchNeedOrgByCodesAsync(new List<string> { deviceModel.OrganizationCode });
            queryItemsFilter.AppendEq((item => item.Key, "OrganizationCode"));
            queryItemsFilter.AppendIn(item => item.Value, allOrgList);
            if (queryItemsFilter.HasFilter)
            {
                customerFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
            }
            if (request.Type == 0)
            {
                //customerFilter.AppendNotArrayEq("BaseData.BindDevices", request.DeviceCodes);
                customerFilter.AppendAnyNotIn("BaseData.BindDevices", request.DeviceCodes);
            }
            else
            {
                //customerFilter.AppendArrayEq("BaseData.BindDevices", request.DeviceCodes);
                customerFilter.AppendAnyIn("BaseData.BindDevices", request.DeviceCodes);
            }
            if (!string.IsNullOrWhiteSpace(request.QueryType) && !string.IsNullOrWhiteSpace(request.Keyword))
            {
                queryItemsFilter = await GetUserFilter(request.QueryType, request.Keyword);
            }
            if (queryItemsFilter.HasFilter)
            {
                customerFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
            }
            return customerFilter;
        }

        /// <summary>
        /// 获取用户查询条件
        /// </summary>
        /// <param name="queryType"></param>
        /// <param name="keyword"></param>
        /// <returns></returns>
        private async Task<MongoCustomerFilter<MongoDataItem>> GetUserFilter(string queryType, string keyword)
        {
            MongoCustomerFilter<MongoDataItem> queryItemsFilter = new();
            if (!string.IsNullOrWhiteSpace(queryType) && !string.IsNullOrWhiteSpace(keyword))
            {
                switch (queryType)
                {
                    case "0":
                        queryItemsFilter.AppendEq((item => item.Key, "UserName"));
                        queryItemsFilter.AppendRegex(item => item.Value, keyword, true);
                        break;
                    case "1":
                        queryItemsFilter.AppendEq((item => item.Key, "UserCode"));
                        queryItemsFilter.AppendRegex(item => item.Value, keyword, true);
                        break;
                    case "2":
                        queryItemsFilter.AppendEq((item => item.Key, "NickName"));
                        queryItemsFilter.AppendRegex(item => item.Value, keyword, true);
                        break;
                    case "3":
                        queryItemsFilter.AppendEq((item => item.Key, "Phone"));
                        queryItemsFilter.AppendRegex(item => item.Value, keyword, true);
                        break;
                    case "4":
                        queryItemsFilter.AppendEq((item => item.Key, "Email"));
                        queryItemsFilter.AppendRegex(item => item.Value, keyword, true);
                        break;
                    case "5":
                        queryItemsFilter.AppendEq((item => item.Key, "FullName"));
                        queryItemsFilter.AppendRegex(item => item.Value, keyword, true);
                        break;
                    case "6":
                        var parentOrgs = await SearchOrganizationsAsync(keyword, string.Empty, "", WingInterfaceLibrary.Enum.OrganizationTypeEnum.Corporation);
                        if (parentOrgs != null && parentOrgs.Count > 0)
                        {
                            var parentOrgList = parentOrgs.Select(f => f.OrganizationCode).ToList();
                            var allOrgList = await SearchNeedOrgByCodesAsync(parentOrgList);
                            if (allOrgList != null && allOrgList.Count > 0)
                            {
                                queryItemsFilter.AppendEq((item => item.Key, "OrganizationCode"));
                                queryItemsFilter.AppendIn(item => item.Value, allOrgList);
                            }
                        }
                        break;
                    case "7":
                        var curOrgs = await SearchOrganizationsAsync(string.Empty, keyword, "", WingInterfaceLibrary.Enum.OrganizationTypeEnum.Department);
                        if (curOrgs != null && curOrgs.Count > 0)
                        {
                            var curOrgList = curOrgs.Select(f => f.OrganizationCode).ToList();
                            queryItemsFilter.AppendEq((item => item.Key, "OrganizationCode"));
                            queryItemsFilter.AppendIn(item => item.Value, curOrgList);
                        }
                        break;
                    default:
                        break;
                }
            }
            return queryItemsFilter;
        }

        /// <summary>
        /// 获取用户查询条件
        /// </summary>
        /// <param name="queryType"></param>
        /// <param name="keyword"></param>
        /// <returns></returns>
        private List<FilterDefinition<MongoDataItem>> ManagementUserFilter(string keyword)
        {
            List<FilterDefinition<MongoDataItem>> queryItemsFilterList = new();
            if (!string.IsNullOrWhiteSpace(keyword))
            {
                var items = new List<string> { "UserName", "UserCode", "NickName", "Phone", "Email", "FullName" };
                foreach (var item in items)
                {
                    var queryItemsFilter = new MongoCustomerFilter<MongoDataItem>();
                    queryItemsFilter.AppendEq((item => item.Key, item));
                    queryItemsFilter.AppendRegex(item => item.Value, keyword, true, false);
                    queryItemsFilterList.Add(queryItemsFilter.AndCombine());
                }
            }
            return queryItemsFilterList;
        }

        /// <summary>
        /// 根据设备获取用户列表 
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<List<UserDTO>> GetUsersByDeviceAsync(string deviceCode)
        {
            MongoCustomerFilter<MongoTable> customerFilter = new();
            customerFilter.AppendArrayEq("BaseData.BindDevices", deviceCode);
            var userInfoList = await _userDBRepository.FindAllAsync(customerFilter.AndCombine(), new FindOptions<MongoTable> { NoCursorTimeout = true });
            return userInfoList.MappingTo<List<UserDTO>>();
        }

        /// <summary>
        /// 根据设备获取用户列表 
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<List<UserDTO>> GetUsersByDevicesAsync(List<string> deviceCodes)
        {
            if (deviceCodes == null || deviceCodes.Count == 0) return new List<UserDTO>();
            MongoCustomerFilter<MongoTable> customerFilter = new();
            customerFilter.AppendAnyIn("BaseData.BindDevices", deviceCodes);
            var userInfoList = await _userDBRepository.FindAllAsync(customerFilter.AndCombine(), new FindOptions<MongoTable> { NoCursorTimeout = true });
            return userInfoList.MappingTo<List<UserDTO>>();
        }

        /// <summary>
        /// 根据设备获取用户列表 
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        private async Task<List<UserInfoDO>> GetUserDOsByDevicesAsync(List<string> deviceCodes)
        {
            if (deviceCodes == null || deviceCodes.Count == 0) return new List<UserInfoDO>();
            MongoCustomerFilter<MongoTable> customerFilter = new();
            customerFilter.AppendAnyIn("BaseData.BindDevices", deviceCodes);
            var userInfoList = await _userDBRepository.FindAllAsync(customerFilter.AndCombine(), new FindOptions<MongoTable> { NoCursorTimeout = true });
            return userInfoList;
        }



        /// <summary>
        /// 查询机构用户列表（分页）
        /// </summary>
        /// <param name="request">查询机构用户列表（分页）请求实体</param>
        /// <returns></returns>
        public async Task<PageResult<ManageOrganizationUserDTO>> GetManageOrganizationUserPagesAsync(List<OrganizationDTO> allNeeds, GetOrganizationUserPagesRequest request)
        {

            var codeList = allNeeds.Select(o => o.OrganizationCode).ToList();
            MongoCustomerFilter<MongoTable> customerFilter = new();
            MongoCustomerFilter<MongoDataItem> queryItemsFilter = new();
            if (!string.IsNullOrWhiteSpace(request.Keyword))
            {
                var filters = ManagementUserFilter(request.Keyword);
                MongoCustomerFilter<MongoDataItem> queryAll = new();
                customerFilter.SubFilter(item => item.QueryData.QueryItems, queryAll.OrCombine(filters.ToArray()));
            }
            if (queryItemsFilter.HasFilter)
            {
                customerFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
            }
            queryItemsFilter = new MongoCustomerFilter<MongoDataItem>();
            queryItemsFilter.AppendEq((item => item.Key, "OrganizationCode"));
            queryItemsFilter.AppendIn(item => item.Value, codeList);
            if (queryItemsFilter.HasFilter)
            {
                customerFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
            }
            PageResult<ManageOrganizationUserDTO> result = new PageResult<ManageOrganizationUserDTO>();
            result.PageIndex = request.PageIndex;
            result.PageSize = request.PageSize;
            result.TotalCount = (int)await _userDBRepository.CountDocumentsAsync(customerFilter.AndCombine());
            SortDefinitionBuilder<MongoTable> builderSort = Builders<MongoTable>.Sort;
            SortDefinition<MongoTable> sort = builderSort.Descending("BaseData.CreateTime");
            var userInfoList = await _userDBRepository.FindAllAsync(customerFilter.AndCombine(), new FindOptions<MongoTable> { NoCursorTimeout = true, Skip = (request.PageIndex - 1) * request.PageSize, Limit = request.PageSize, Sort = sort });
            if (userInfoList.Count() > 0)
            {
                foreach (var item in userInfoList)
                {
                    var manageOrganizationUserDTO = item.MappingTo<ManageOrganizationUserDTO>();
                    var orgInfo = allNeeds.FirstOrDefault(o => o.OrganizationCode == item.OrganizationCode);
                    if (orgInfo.OrganizationType == OrganizationTypeEnum.Corporation)
                    {
                        manageOrganizationUserDTO.HospitalName = orgInfo?.OrganizationName;
                    }
                    else
                    {
                        var rootOrgInfo = allNeeds.FirstOrDefault(r => r.OrganizationCode == orgInfo?.RootCode);
                        manageOrganizationUserDTO.HospitalName = rootOrgInfo?.OrganizationName;
                    }
                    result.PageData.Add(manageOrganizationUserDTO);
                }
            }
            return result;
        }

        /// <summary>
        /// 根据组织、岗位身份、职称查询用户列表信息
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<List<UserDTO>> GetUserListAsync(GetUserListDBRequest request)
        {
            FilterDefinition<MongoTable> customerFilter = await GetUserFliter(request);
            SortDefinitionBuilder<MongoTable> builderSort = Builders<MongoTable>.Sort;
            SortDefinition<MongoTable> sort = builderSort.Descending("BaseData.CreateTime");
            var userInfoList = await _userDBRepository.FindAllAsync(customerFilter, new FindOptions<MongoTable> { NoCursorTimeout = true, Sort = sort });
            var userDTOList = userInfoList.MappingTo<List<UserDTO>>();
            var allPositions = await GetPositionsAsync(new GetPositionsDBRequest());
            var allRanks = await GetAllRanksAsync(new GetRanksDBRequest());
            foreach (var item in userDTOList)
            {
                if (item.RankCodes != null && item.RankCodes.Count > 0)
                {
                    item.RankName = allRanks.FirstOrDefault(a => a.RankCode == item.RankCodes[0])?.RankName;
                }
                if (item.PositionCodes != null && item.PositionCodes.Count > 0)
                {
                    item.PositionName = allPositions.FirstOrDefault(a => a.PositionCode == item.PositionCodes[0])?.PositionName;
                }
            }
            return userDTOList;
        }

        /// <summary>
        /// 根据组织、岗位身份、职称查询用户列表信息（分页）
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageResult<UserDTO>> GetUserListPagesDBAsync(GetUserListPagesDBRequest request)
        {

            var filterRequest = request.MappingTo<GetUserListDBRequest>();
            FilterDefinition<MongoTable> customerFilter = await GetUserFliter(filterRequest);
            PageResult<UserDTO> result = new PageResult<UserDTO>();
            result.PageIndex = request.PageIndex;
            result.PageSize = request.PageSize;
            result.TotalCount = (int)await _userDBRepository.CountDocumentsAsync(customerFilter);
            if (result.TotalCount > 0)
            {
                SortDefinitionBuilder<MongoTable> builderSort = Builders<MongoTable>.Sort;
                SortDefinition<MongoTable> sort = builderSort.Descending("BaseData.CreateTime");
                var userInfoList = await _userDBRepository.FindAllAsync(customerFilter, new FindOptions<MongoTable> { NoCursorTimeout = true, Skip = (request.PageIndex - 1) * request.PageSize, Limit = request.PageSize, Sort = sort });
                var userDTOList = userInfoList.MappingTo<List<UserDTO>>();
                var allPositions = await GetPositionsAsync(new GetPositionsDBRequest());
                var allRanks = await GetAllRanksAsync(new GetRanksDBRequest());
                foreach (var item in userDTOList)
                {
                    if (item.RankCodes != null && item.RankCodes.Count > 0)
                    {
                        item.RankName = allRanks.FirstOrDefault(a => a.RankCode == item.RankCodes[0])?.RankName;
                    }
                    if (item.PositionCodes != null && item.PositionCodes.Count > 0)
                    {
                        item.PositionName = allPositions.FirstOrDefault(a => a.PositionCode == item.PositionCodes[0])?.PositionName;
                    }
                }
                result.PageData = userDTOList;
            }
            return result;
        }

        private async Task<FilterDefinition<MongoTable>> GetUserFliter(GetUserListDBRequest request)
        {
            MongoCustomerFilter<MongoTable> customerFilter = new();
            MongoCustomerFilter<MongoDataItem> queryItemsFilter = new();
            MongoCustomerFilter<MongoTable> keywordChildFilter = new MongoCustomerFilter<MongoTable>();

            if (!string.IsNullOrWhiteSpace(request.OrganizationCode))
            {
                queryItemsFilter.AppendEq((item => item.Key, "OrganizationCode"));
                switch (request.OrganizationQueryType)
                {
                    //single 传要查询的组织code,Wait、AllDep传rootCode
                    case OrganizationQueryTypeEnum.Single:
                    case OrganizationQueryTypeEnum.Wait:
                        queryItemsFilter.AppendEq((item => item.Value, request.OrganizationCode));
                        break;
                    case OrganizationQueryTypeEnum.AllDep:
                        var orgsList = await SearchOrgByRootCodesAsync(new List<string> { request.OrganizationCode });
                        //过滤部门
                        var allDepList = orgsList.Where(o => o.OrganizationType != OrganizationTypeEnum.Corporation).ToList();
                        if (allDepList != null && allDepList.Count > 0)
                        {
                            queryItemsFilter.AppendIn(item => item.Value, allDepList.Select(d => d.OrganizationCode).ToList());
                        }
                        else
                        {
                            queryItemsFilter.AppendEq((item => item.Value, "-1"));
                        }
                        break;
                    case OrganizationQueryTypeEnum.All:
                        var orgs = await SearchOrgByRootCodesAsync(new List<string> { request.OrganizationCode });
                        if (orgs != null && orgs.Count > 0)
                        {
                            queryItemsFilter.AppendIn(item => item.Value, orgs.Select(d => d.OrganizationCode).ToList());
                        }
                        else
                        {
                            queryItemsFilter.AppendEq((item => item.Value, "-1"));
                        }
                        break;
                    default:
                        break;
                }
                if (queryItemsFilter.HasFilter)
                {
                    customerFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
                }
            }
            else
            {
                queryItemsFilter.AppendEq((item => item.Key, "OrganizationCode"));
                queryItemsFilter.AppendEq((item => item.Value, "-1"));
                customerFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
            }
            if (!string.IsNullOrWhiteSpace(request.PositionCode))
            {
                customerFilter.AppendArrayEq("BaseData.PositionCodes", request.PositionCode);
            }
            // if (!string.IsNullOrWhiteSpace(request.IsResearchProjectUser))
            // {
            //     customerFilter.AppendEq(("BaseData.IsResearchProjectUser", request.IsResearchProjectUser));
            // }
            if (request.RoleCodes != null && request.RoleCodes.Count > 0)
            {
                customerFilter.AppendAnyIn("BaseData.RoleCodes", request.RoleCodes.ToList());
            }
            if (!string.IsNullOrWhiteSpace(request.RankCode))
            {
                customerFilter.AppendArrayEq("BaseData.RankCodes", request.RankCode);
            }
            if (request.BindAssistants != null && request.BindAssistants.Count > 0)
            {
                customerFilter.AppendQueryItemInFilter(MongoCombineTypeEnum.And, "BindAssistantUserCode", request.BindAssistants);
            }
            if (!string.IsNullOrWhiteSpace(request.Keyword))
            {

                queryItemsFilter.AppendEq((item => item.Key, "FullName"));
                queryItemsFilter.AppendRegex(item => item.Value, request.Keyword, true, false);
                keywordChildFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());

                queryItemsFilter.AppendEq((item => item.Key, "UserName"));
                queryItemsFilter.AppendRegex(item => item.Value, request.Keyword, true, false);
                keywordChildFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());

                queryItemsFilter.AppendEq((item => item.Key, "Phone"));
                queryItemsFilter.AppendRegex(item => item.Value, request.Keyword, true, false);
                keywordChildFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());

                queryItemsFilter.AppendEq((item => item.Key, "Email"));
                queryItemsFilter.AppendRegex(item => item.Value, request.Keyword, true, false);
                keywordChildFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());


            }
            if (request.ExceptUsers != null && request.ExceptUsers.Count > 0)
            {
                queryItemsFilter.AppendEq((item => item.Key, "Code"));
                queryItemsFilter.AppendNotIn(item => item.Value, request.ExceptUsers);
                customerFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
            }
            return customerFilter.AndCombine() & keywordChildFilter.OrCombine();
        }

        /// <summary>
        /// 删除病人
        /// </summary>
        /// <returns></returns>
        public async Task<bool> DeletePatientsByUserCodeAsync(DeletePatientsDBRequest request)
        {
            if (string.IsNullOrWhiteSpace(request.UserCode) || request.PatientCodes == null || !request.PatientCodes.Any())
            {
                return false;
            }
            MongoCustomerFilter<MongoTable> mongoCustomerFilter = new();
            mongoCustomerFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("Code", request.UserCode));
            var filter = mongoCustomerFilter.AndCombine();
            MongoCustomerUpdate<MongoTable> update = new();
            update.AddArrayToSubItem("BaseData.DeletePatientCodes", request.PatientCodes);
            return await _userDBRepository.UpdateAllAsync(filter, update.AllUpdateDefinition) > 0;
        }

        /// <summary>
        /// 设置用户默认词条库编码
        /// </summary>
        /// <param name="userCode">用户编码</param>
        /// <param name="thesaurusCode">默认词条库编码</param>
        /// <param name="languageCode">语言</param>
        /// <returns></returns>
        public async Task<bool> SetUserDefaultThesaurusCodeAsync(string userCode, string thesaurusCode, string languageCode = "en-US")
        {
            MongoCustomerFilter<MongoTable> mongoCustomerFilter = new();
            mongoCustomerFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("UserCode", userCode));
            mongoCustomerFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("LanguageCode", languageCode));
            var filter = mongoCustomerFilter.AndCombine();
            var findTable = await _userExtendsDBRepository.FindTableOneAsync(filter);
            if (findTable != null)
            {
                MongoCustomerUpdate<MongoTable> update = new();
                update.SetField("BaseData.DefaultThesaurusCode", thesaurusCode);
                var queryItems = findTable.QueryData.QueryItems;
                queryItems.RemoveWhere(r => r.Key == "DefaultThesaurusCode");
                queryItems.Add(new MongoDataItem { Key = "DefaultThesaurusCode", Value = thesaurusCode });
                update.SetField("QueryData.QueryItems", queryItems);
                return await _userExtendsDBRepository.UpdateAllAsync(filter, update.AllUpdateDefinition) > 0;
            }
            else
            {
                var extends = new UserExtendsDO();
                extends.UserCode = userCode;
                extends.DefaultThesaurusCode = thesaurusCode;
                extends.LanguageCode = languageCode;
                return !string.IsNullOrWhiteSpace(await _userExtendsDBRepository.InsertOneAsync(extends, new Dictionary<string, string>(), string.Empty));
            }
        }

        /// <summary>
        /// 获取用户默认词条库编码
        /// </summary>
        /// <param name="userCode">用户编码</param>
        /// <returns>词条库编码</returns>
        public async Task<string> GetUserDefaultThesaurusCodeAsync(string userCode, string language = "en-US")
        {
            MongoCustomerFilter<MongoTable> mongoCustomerFilter = new();
            mongoCustomerFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("UserCode", userCode));
            mongoCustomerFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("LanguageCode", language));
            var filter = mongoCustomerFilter.AndCombine();
            var findInfo = await _userExtendsDBRepository.FindOneAsync(filter);
            if (!string.IsNullOrEmpty(findInfo?.DefaultThesaurusCode))
            {
                return findInfo.DefaultThesaurusCode;
            }
            //用户是否有
            var userInfo = await _userDBRepository.FindOneAsync("Code", userCode);

            SortDefinitionBuilder<MongoTable> builderSort = Builders<MongoTable>.Sort;
            SortDefinition<MongoTable> sort = builderSort.Descending("BaseData.ThesaurusType").Ascending("BaseData.CreateTime").Ascending("BaseData.ThesaurusUsageType").Ascending("BaseData.Code");
            FindOptions<MongoTable> options = new FindOptions<MongoTable> { NoCursorTimeout = true, Sort = sort };

            MongoCustomerFilter<MongoTable> customFilter = new();
            if (!string.IsNullOrEmpty(userInfo?.OrganizationCode))
            {
                var cacheOrgInfo = CacheMaintenance.Instance.Get<IOrganizationsManager>().Where(c => c.Code == userInfo.OrganizationCode)?.FirstOrDefault() ?? new CacheOrganizationDTO();
                customFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("PersonalCode", userCode), ("IsDefault", "True"), ("ThesaurusStateType", Enum.GetName(ThesaurusStatusTypeEnum.Published)), ("ThesaurusType", Enum.GetName(ThesaurusTypeEnum.Personal)), ("ThesaurusUsageType", ((ThesaurusUsageTypeEnum)cacheOrgInfo.PatientType).ToString()), ("LanguageCode", language));
                var dataListTemp = await _thesaurusDBRepository.FindAllAsync(customFilter.AndCombine(), options);
                if (dataListTemp?.Count > 0)
                {
                    var result = dataListTemp.FirstOrDefault();
                    return result.Code;
                }
                customFilter = new();
            }
            customFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("PersonalCode", userCode), ("IsDefault", "True"), ("ThesaurusStateType", Enum.GetName(ThesaurusStatusTypeEnum.Published)), ("ThesaurusType", Enum.GetName(ThesaurusTypeEnum.Personal)), ("LanguageCode", language));
            var dataList = await _thesaurusDBRepository.FindAllAsync(customFilter.AndCombine(), options);
            if (dataList?.Count > 0)
            {
                var result = dataList.FirstOrDefault();
                return result.Code;
            }

            if (!string.IsNullOrEmpty(userInfo?.OrganizationCode))
            {
                var orgInfo = await _organizationDBRepository.GetOrganizationByCodeAsync(userInfo.OrganizationCode);
                if (!string.IsNullOrEmpty(orgInfo?.DefaultThesaurusCode))
                {
                    var thesaurusInfo = await _thesaurusDBRepository.FindOneAsync("Code", orgInfo.DefaultThesaurusCode);
                    if (thesaurusInfo != null && thesaurusInfo.LanguageCode == language)
                    {
                        return orgInfo.DefaultThesaurusCode;
                    }
                }
            }
            MongoCustomerFilter<MongoTable> reportTemplateFilter = new();
            if (!string.IsNullOrEmpty(userInfo?.OrganizationCode))
            {
                var cacheOrgInfo = CacheMaintenance.Instance.Get<IOrganizationsManager>().Where(c => c.Code == userInfo.OrganizationCode)?.FirstOrDefault() ?? new CacheOrganizationDTO();
                if (!string.IsNullOrWhiteSpace(cacheOrgInfo?.Code) && (cacheOrgInfo.PatientType == 0 || cacheOrgInfo.PatientType == 1))
                {
                    reportTemplateFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("OrganizationCode", userInfo.OrganizationCode), ("IsDefault", "True"), ("ThesaurusStateType", Enum.GetName(ThesaurusStatusTypeEnum.Published)), ("ThesaurusType", Enum.GetName(ThesaurusTypeEnum.Organization)), ("ThesaurusUsageType", ((ThesaurusUsageTypeEnum)cacheOrgInfo.PatientType).ToString()), ("LanguageCode", language));
                    dataList = await _thesaurusDBRepository.FindAllAsync(reportTemplateFilter.AndCombine(), options);
                    if (dataList?.Count > 0)
                    {
                        var result = dataList.FirstOrDefault();
                        return result.Code;
                    }
                    reportTemplateFilter = new();
                }
                reportTemplateFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("OrganizationCode", userInfo.OrganizationCode), ("IsDefault", "True"), ("ThesaurusStateType", Enum.GetName(ThesaurusStatusTypeEnum.Published)), ("ThesaurusType", Enum.GetName(ThesaurusTypeEnum.Organization)), ("LanguageCode", language));
                dataList = await _thesaurusDBRepository.FindAllAsync(reportTemplateFilter.AndCombine(), options);
                if (dataList?.Count > 0)
                {
                    var result = dataList.FirstOrDefault();
                    return result.Code;
                }
            }
            List<(string, string)> eqFilters = new();
            eqFilters.Add(("IsDefault", "True"));
            eqFilters.Add(("ThesaurusStateType", Enum.GetName(ThesaurusStatusTypeEnum.Published)));
            eqFilters.Add(("LanguageCode", language));
            eqFilters.Add(("ThesaurusType", Enum.GetName(ThesaurusTypeEnum.Default)));
            if (!string.IsNullOrEmpty(userInfo?.OrganizationCode))
            {
                var cacheOrgInfo = CacheMaintenance.Instance.Get<IOrganizationsManager>().Where(c => c.Code == userInfo.OrganizationCode)?.FirstOrDefault() ?? new CacheOrganizationDTO();
                eqFilters.Add(("ThesaurusUsageType", ((ThesaurusUsageTypeEnum)cacheOrgInfo.PatientType).ToString()));
                reportTemplateFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, eqFilters.ToArray());
                var resultListTemp = await _thesaurusDBRepository.FindAllAsync(reportTemplateFilter.AndCombine(), options);
                if (resultListTemp?.Count > 0)
                {
                    var res = resultListTemp.FirstOrDefault();
                    return res.Code;
                }
                eqFilters = new();
                reportTemplateFilter = new();
            }
            eqFilters.Add(("IsDefault", "True"));
            eqFilters.Add(("ThesaurusStateType", Enum.GetName(ThesaurusStatusTypeEnum.Published)));
            eqFilters.Add(("LanguageCode", language));
            eqFilters.Add(("ThesaurusType", Enum.GetName(ThesaurusTypeEnum.Default)));
            reportTemplateFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, eqFilters.ToArray());
            var resultList = await _thesaurusDBRepository.FindAllAsync(reportTemplateFilter.AndCombine(), options);
            if (resultList?.Count > 0)
            {
                var res = resultList.FirstOrDefault();
                return res.Code;
            }
            return null;
        }

        /// <summary>
        /// 获取用户默认词条库内容
        /// </summary>
        /// <param name="userCode">用户编码</param>
        /// <returns></returns>
        public async Task<ThesaurusAllDTO> GetUserDefaultThesaurusContentAsync(string userCode, string language = "en-US")
        {
            var code = await GetUserDefaultThesaurusCodeAsync(userCode, language);
            if (string.IsNullOrEmpty(code))
            {
                code = await GetUserDefaultThesaurusCodeAsync(userCode, "en-US");
            }
            var data = await _thesaurusDBRepository.FindOneAsync("Code", code);
            if (data != null)
            {
                var allDTO = new ThesaurusAllDTO();
                allDTO.BaseInfo = data.MappingTo<ThesaurusDTO>();
                allDTO.Items = data.ThesaurusItems.MappingTo<List<ThesaurusItemDTO>>();
                return allDTO;
            }
            return null;
        }


        /// <summary>
        /// 设置用户默认报告模板编码
        /// </summary>
        /// <param name="userCode">用户编码</param>
        /// <param name="reportTemplateCode">默认报告模板编码</param>
        /// <returns></returns>
        public async Task<bool> SetUserDefaultReportTemplateCodeAsync(string userCode, string reportTemplateCode, string languageCode = "en-US")
        {
            MongoCustomerFilter<MongoTable> mongoCustomerFilter = new();
            mongoCustomerFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("UserCode", userCode));
            mongoCustomerFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("LanguageCode", languageCode));
            var filter = mongoCustomerFilter.AndCombine();
            var findTable = await _userExtendsDBRepository.FindTableOneAsync(filter);
            if (findTable != null)
            {
                MongoCustomerUpdate<MongoTable> update = new();
                update.SetField("BaseData.DefaultReportTemplateCode", reportTemplateCode);
                var queryItems = findTable.QueryData.QueryItems;
                queryItems.RemoveWhere(r => r.Key == "DefaultReportTemplateCode");
                queryItems.Add(new MongoDataItem { Key = "DefaultReportTemplateCode", Value = reportTemplateCode });
                update.SetField("QueryData.QueryItems", queryItems);
                return await _userExtendsDBRepository.UpdateAllAsync(filter, update.AllUpdateDefinition) > 0;
            }
            else
            {
                var extends = new UserExtendsDO();
                extends.UserCode = userCode;
                extends.DefaultReportTemplateCode = reportTemplateCode;
                extends.LanguageCode = languageCode;
                return !string.IsNullOrWhiteSpace(await _userExtendsDBRepository.InsertOneAsync(extends, new Dictionary<string, string>(), string.Empty));
            }
        }

        /// <summary>
        /// 获取用户默认报告模板编码
        /// </summary>
        /// <param name="userCode">用户编码</param>
        /// <returns>报告模板编码</returns>
        public async Task<string> GetUserDefaultReportTemplateCodeAsync(string userCode, string language = "en-US")
        {
            MongoCustomerFilter<MongoTable> mongoCustomerFilter = new();
            mongoCustomerFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("UserCode", userCode));
            var filter = mongoCustomerFilter.AndCombine();
            var findInfo = await _userExtendsDBRepository.FindOneAsync(filter);
            if (findInfo != null)
            {
                return findInfo.DefaultReportTemplateCode;
            }
            else
            {
                var reportTemplate = await GetUserDefaultReportTemplateContentAsync(userCode, language);
                if (reportTemplate != null)
                {
                    return reportTemplate.ReportTemplateCode;
                }
                else
                {
                    return "";
                }
            }
        }

        /// <summary>
        /// 获取用户默认报告模板信息
        /// </summary>
        /// <param name="userCode">用户编码</param>
        /// <returns></returns>
        public async Task<ReportTemplateDTO> GetUserDefaultReportTemplateContentAsync(string userCode, string language = "en-US")
        {
            MongoCustomerFilter<MongoTable> mongoCustomerFilter = new();
            mongoCustomerFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("UserCode", userCode), ("LanguageCode", language));
            var filter = mongoCustomerFilter.AndCombine();
            var findInfo = await _userExtendsDBRepository.FindOneAsync(filter);
            if (findInfo != null)
            {
                var defaultReportTemplateCode = findInfo.DefaultReportTemplateCode;
                if (!string.IsNullOrWhiteSpace(defaultReportTemplateCode))
                {
                    var data = await _reportTemplateDBRepository.FindOneAsync("Code", defaultReportTemplateCode);
                    if (data != null)
                    {
                        var reportTemplateInfo = data.MappingTo<ReportTemplateDTO>();
                        if (reportTemplateInfo != null)
                        {
                            reportTemplateInfo.IsUserDefault = true;
                            return reportTemplateInfo;
                        }
                    }
                }
            }

            var userInfo = await _userDBRepository.FindOneAsync("Code", userCode);
            SortDefinitionBuilder<MongoTable> builderSort = Builders<MongoTable>.Sort;
            SortDefinition<MongoTable> sort = builderSort.Descending("BaseData.ReportTemplateType").Ascending("BaseData.CreateTime").Ascending("BaseData.ReportTemplateUsageType").Ascending("BaseData.Code");
            FindOptions<MongoTable> options = new FindOptions<MongoTable> { NoCursorTimeout = true, Sort = sort };
            var statusType = ReportTemplateStatusTypeEnum.Published.ToString();
            MongoCustomerFilter<MongoTable> customFilter = new();
            if (!string.IsNullOrEmpty(userInfo?.OrganizationCode))
            {
                var cacheOrgInfo = CacheMaintenance.Instance.Get<IOrganizationsManager>().Where(c => c.Code == userInfo.OrganizationCode)?.FirstOrDefault() ?? new CacheOrganizationDTO();
                customFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("IsDefault", "True"), ("PersonalCode", userCode), ("ReportTemplateStateType", statusType), ("ReportTemplateType", Enum.GetName(ReportTemplateTypeEnum.Personal)), ("ReportTemplateUsageType", ((ReportTemplateUsageTypeEnum)cacheOrgInfo.PatientType).ToString()), ("LanguageCode", language));

                var dataListTemp = await _reportTemplateDBRepository.FindAllAsync(customFilter.AndCombine(), options);
                if (dataListTemp?.Count > 0)
                {
                    var result = dataListTemp.FirstOrDefault();
                    var baseInfo = new ReportTemplateDTO();
                    baseInfo = result.MappingTo<ReportTemplateDTO>();
                    baseInfo.IsUserDefault = true;
                    return baseInfo;
                }
                customFilter = new();
            }
            customFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("IsDefault", "True"), ("PersonalCode", userCode), ("ReportTemplateStateType", statusType), ("ReportTemplateType", Enum.GetName(ReportTemplateTypeEnum.Personal)), ("LanguageCode", language));

            var dataList = await _reportTemplateDBRepository.FindAllAsync(customFilter.AndCombine(), options);
            if (dataList?.Count > 0)
            {
                var result = dataList.FirstOrDefault();
                var baseInfo = new ReportTemplateDTO();
                baseInfo = result.MappingTo<ReportTemplateDTO>();
                baseInfo.IsUserDefault = true;
                return baseInfo;
            }

            if (!string.IsNullOrEmpty(userInfo?.OrganizationCode))
            {
                var orgInfo = await _organizationDBRepository.GetOrganizationByCodeAsync(userInfo.OrganizationCode);
                if (!string.IsNullOrEmpty(orgInfo?.DefaultReportTemplateCode))
                {
                    var data = await _reportTemplateDBRepository.FindOneAsync("Code", orgInfo.DefaultReportTemplateCode);
                    if (data != null && data.LanguageCode == language)
                    {
                        var reportTemplateInfo = data.MappingTo<ReportTemplateDTO>();
                        if (reportTemplateInfo != null)
                        {
                            reportTemplateInfo.IsUserDefault = true;
                            return reportTemplateInfo;
                        }
                    }
                }
            }

            MongoCustomerFilter<MongoTable> reportTemplateFilter = new();
            if (!string.IsNullOrEmpty(userInfo?.OrganizationCode))
            {
                var cacheOrgInfo = CacheMaintenance.Instance.Get<IOrganizationsManager>().Where(c => c.Code == userInfo.OrganizationCode)?.FirstOrDefault() ?? new CacheOrganizationDTO();
                if (!string.IsNullOrWhiteSpace(cacheOrgInfo?.Code) && (cacheOrgInfo.PatientType == 0 || cacheOrgInfo.PatientType == 1))
                {
                    reportTemplateFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("IsDefault", "True"), ("OrganizationCode", userInfo.OrganizationCode), ("ReportTemplateStateType", statusType), ("ReportTemplateType", Enum.GetName(ReportTemplateTypeEnum.Organization)), ("ReportTemplateUsageType", ((ReportTemplateUsageTypeEnum)cacheOrgInfo.PatientType).ToString()), ("LanguageCode", language));
                    dataList = await _reportTemplateDBRepository.FindAllAsync(reportTemplateFilter.AndCombine(), options);
                    if (dataList?.Count > 0)
                    {
                        var result = dataList.FirstOrDefault();
                        var baseInfo = new ReportTemplateDTO();
                        baseInfo = result.MappingTo<ReportTemplateDTO>();
                        baseInfo.IsUserDefault = true;
                        return baseInfo;
                    }
                    reportTemplateFilter = new();
                }
                reportTemplateFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("IsDefault", "True"), ("OrganizationCode", userInfo.OrganizationCode), ("ReportTemplateStateType", statusType), ("ReportTemplateType", Enum.GetName(ReportTemplateTypeEnum.Organization)), ("LanguageCode", language));
                dataList = await _reportTemplateDBRepository.FindAllAsync(reportTemplateFilter.AndCombine(), options);
            }
            if (dataList == null || dataList.Count <= 0)
            {
                List<(string, string)> eqFilters = new();
                eqFilters.Add(("IsDefault", "True"));
                eqFilters.Add(("ReportTemplateStateType", statusType));
                eqFilters.Add(("LanguageCode", language));
                eqFilters.Add(("ReportTemplateType", Enum.GetName(ReportTemplateTypeEnum.Default)));
                if (!string.IsNullOrEmpty(userInfo?.OrganizationCode))
                {
                    var cacheOrgInfo = CacheMaintenance.Instance.Get<IOrganizationsManager>().Where(c => c.Code == userInfo.OrganizationCode)?.FirstOrDefault() ?? new CacheOrganizationDTO();
                    eqFilters.Add(("ReportTemplateUsageType", ((ReportTemplateUsageTypeEnum)cacheOrgInfo.PatientType).ToString()));
                    reportTemplateFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, eqFilters.ToArray());
                    dataList = await _reportTemplateDBRepository.FindAllAsync(reportTemplateFilter.AndCombine(), options);
                    if (dataList?.Count > 0)
                    {
                        var result = dataList.FirstOrDefault();
                        var baseInfo = new ReportTemplateDTO();
                        baseInfo = result.MappingTo<ReportTemplateDTO>();
                        baseInfo.IsUserDefault = true;
                        return baseInfo;
                    }
                }
                eqFilters = new();
                reportTemplateFilter = new();
                eqFilters.Add(("IsDefault", "True"));
                eqFilters.Add(("ReportTemplateStateType", statusType));
                eqFilters.Add(("LanguageCode", language));
                eqFilters.Add(("ReportTemplateType", Enum.GetName(ReportTemplateTypeEnum.Default)));
                reportTemplateFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, eqFilters.ToArray());
                dataList = await _reportTemplateDBRepository.FindAllAsync(reportTemplateFilter.AndCombine(), options);
            }
            if (dataList?.Count > 0)
            {
                var result = dataList.FirstOrDefault();
                var baseInfo = new ReportTemplateDTO();
                baseInfo = result.MappingTo<ReportTemplateDTO>();
                baseInfo.IsUserDefault = true;
                return baseInfo;
            }
            return null;
        }

        /// <summary>查询专家助理绑定的所有专家</summary>
        /// <returns></returns>
        public async Task<IList<UserDTO>> GetBindExpertsAsync(string assistentCode)
        {
            MongoCustomerFilter<MongoTable> customerFilter = new();
            customerFilter.AppendEq(("BaseData.BindAssistantUserCode", assistentCode));
            var entities = await _userDBRepository.FindAllAsync(customerFilter.AndCombine());
            return entities.MappingTo<List<UserDTO>>();
        }

        /// <summary>查询助理医师绑定的所有专家</summary>
        /// <returns></returns>
        private async Task<IList<UserDTO>> GetBindExpertsByAssistantDoctorAsync(string assistentCode)
        {
            MongoCustomerFilter<MongoTable> customerFilter = new();
            customerFilter.AppendEq(("BaseData.BindAssistantDoctorUserCode", assistentCode));
            var entities = await _userDBRepository.FindAllAsync(customerFilter.AndCombine());
            return entities.MappingTo<List<UserDTO>>();
        }

        /// <summary>
        /// 更新用户锁定信息
        /// </summary>
        /// <param name="code">用户编码</param>
        /// <param name="LoginLockInfoDTO">锁定信息</param>
        /// <returns></returns>
        public async Task<bool> UpdateUserLoginLockInfoAsync(string code, LoginLockInfoDTO loginLockInfoDTO)
        {
            var userDO = await _userDBRepository.FindOneAsync("Code", code);
            if (loginLockInfoDTO == null)
            {
                userDO.LoginLockInfo = new LoginLockInfoDO();
            }
            else
            {
                userDO.LoginLockInfo = loginLockInfoDTO.MappingTo<LoginLockInfoDO>();
            }
            return await _userDBRepository.UpdateOneAsync("Code", code, userDO, "", true);
        }

        /// <summary>
        /// 获取最后新增的用户的创建时间
        /// </summary>
        /// <returns></returns>
        public async Task<DateTime> GetLastUserCreateTime()
        {
            MongoCustomerFilter<MongoTable> filter = new MongoCustomerFilter<MongoTable>();
            filter.AppendEq(("BaseData.IsDelete", false));
            var users = await _userDBRepository.FindAllAsync(filter.AndCombine());
            var lastCreateTime = users.OrderByDescending(o => o.CreateTime)?.FirstOrDefault()?.CreateTime;
            if (lastCreateTime != null)
            {
                return lastCreateTime.Value;
            }
            return DateTime.UtcNow;
        }

        private string GetMd5SaltPassword(string userName, string password)
        {
            if (!password.StartsWith("ServerSalt_"))
            {
                var saltPassword = MD5.GetHashString(userName + password + "Server");
                return $"ServerSalt_{saltPassword}";
            }
            else
            {
                return password;
            }
        }

        public async Task<string> AddScheduleDBAsync(AddScheduleDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} AddScheduleDBAsync execute", $"{this.GetType().Name}"))
            {
                var scheduleDO = request.Data.MappingTo<ScheduleEntity>();
                var scheduleCode = await _scheduleDBRepository.InsertOneAsync(
                  scheduleDO);
                return scheduleCode;
            }
        }

        public async Task<bool> UpdateScheduleDBAsync(UpdateScheduleDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateScheduleDBAsync execute", $"{this.GetType().Name}"))
            {
                var code = request.Data.Code;
                var scheduleDO = request.Data.MappingTo<ScheduleEntity>();
                return await _scheduleDBRepository.UpdateOneAsync("Code", code, scheduleDO);
            }
        }

        public async Task<bool> AddScheduleUsersDBAsync(AddScheduleUsersDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} AddScheduleUsersDBAsync execute", $"{this.GetType().Name}"))
            {
                var relevanceCode = request.RelevanceCode;
                var addUserCodes = request.UserCodes;
                if (!string.IsNullOrWhiteSpace(relevanceCode) && addUserCodes?.Any() == true)
                {
                    var entity = await _scheduleDBRepository.FindOneAsync("RelevanceCode", relevanceCode);
                    if (entity != null && !string.IsNullOrWhiteSpace(entity.Code))
                    {
                        var userCodes = entity.UserCodes.ToList();
                        foreach (var userCode in addUserCodes)
                        {
                            if (!string.IsNullOrWhiteSpace(userCode) && !userCodes.Contains(userCode))
                            {
                                userCodes.Add(userCode);
                            }
                        }
                        entity.UserCodes = userCodes;
                        return await _scheduleDBRepository.UpdateOneAsync("Code", entity.Code, entity);
                    }
                }
                return false;
            }
        }

        public async Task<bool> UpdateScheduleStateDBAsync(UpdateScheduleStateDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateScheduleStateDBAsync execute", $"{this.GetType().Name}"))
            {
                var relevanceCode = request.RelevanceCode;
                var status = request.Status;
                if (!string.IsNullOrWhiteSpace(relevanceCode))
                {
                    var entity = await _scheduleDBRepository.FindOneAsync("RelevanceCode", relevanceCode);
                    if (entity != null && !string.IsNullOrWhiteSpace(entity.Code))
                    {
                        entity.Status = status;
                        return await _scheduleDBRepository.UpdateOneAsync("Code", entity.Code, entity);
                    }
                }
                return false;
            }
        }

        public async Task<bool> DeleteScheduleDBAsync(string code)
        {
            using (new Performance(_performanceThreshold, "{0} DeleteScheduleDBAsync execute", $"{this.GetType().Name}"))
            {
                return await _scheduleDBRepository.DeleteOneAsync("Code", code);
            }
        }

        public async Task<ScheduleDTO> GetScheduleDBByCodeAsync(string code)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateScheduleDBAsync execute", $"{this.GetType().Name}"))
            {
                var scheduleDO = await _scheduleDBRepository.FindOneAsync("Code", code);
                return scheduleDO?.MappingTo<ScheduleDTO>();
            }
        }

        public async Task<ScheduleDTO> GetScheduleDBByRelevanceCodeAsync(string code)
        {
            using (new Performance(_performanceThreshold, "{0} GetScheduleDBByRelevanceCodeAsync execute", $"{this.GetType().Name}"))
            {
                var scheduleDO = await _scheduleDBRepository.FindOneAsync("RelevanceCode", code);
                return scheduleDO?.MappingTo<ScheduleDTO>();
            }
        }
        public async Task<IList<ScheduleDTO>> GetScheduleDBAsync(GetScheduleDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateScheduleDBAsync execute", $"{this.GetType().Name}"))
            {
                var userCode = request.UserCode;
                var startTime = request.StartTime.Date;
                var endTime = request.EndTime.AddDays(1).Date;
                FilterDefinition<ScheduleEntity> filter =
                Builders<ScheduleEntity>.Filter.Where(x => x.StartTime >= startTime && x.StartTime < endTime
                && x.UserCodes.Contains(userCode));
                var list = await _scheduleDBRepository.FindAllAsync(filter);
                return list?.MappingTo<List<ScheduleDTO>>();
            }
        }

        private async Task<List<UserInfoDO>> FindUsersByName(string userName)
        {
            MongoCustomerFilter<MongoTable> customerFilter = new();
            List<FilterDefinition<MongoDataItem>> queryItemsFilterList = new();
            var items = new List<string> { "UserName", "FullName" };
            foreach (var item in items)
            {
                var queryItemsFilter = new MongoCustomerFilter<MongoDataItem>();
                queryItemsFilter.AppendEq((item => item.Key, item));
                queryItemsFilter.AppendRegex(item => item.Value, userName, true, false);
                queryItemsFilterList.Add(queryItemsFilter.AndCombine());
            }
            MongoCustomerFilter<MongoDataItem> queryAll = new();
            customerFilter.SubFilter(item => item.QueryData.QueryItems, queryAll.OrCombine(queryItemsFilterList.ToArray()));
            var result = await _userDBRepository.FindAllAsync(customerFilter.AndCombine());
            return result;
        }

        public async Task<long> GetActiveUsersCount()
        {
            MongoCustomerFilter<MongoTable> customerFilter = new();
            customerFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("IsDelete", "False"));
            var count = await _userDBRepository.CountDocumentsAsync(customerFilter.AndCombine());
            return count;
        }
        /// <summary>
        /// 通过多个角色获取用户信息
        /// </summary>
        /// <param name="roleCodes"></param>
        /// <returns></returns>
        public async Task<List<UserDTO>> FindUserListByRoleCodesAsync(IList<string> roleCodes)
        {
            MongoCustomerFilter<MongoTable> customerFilter = new();
            customerFilter.AppendAnyIn("BaseData.RoleCodes", roleCodes.ToList());
            var userInfoList = await _userDBRepository.FindAllAsync(customerFilter.AndCombine());
            return userInfoList.MappingTo<List<UserDTO>>();
        }

        /// <summary>
        /// 清除第三方关联信息
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> ClearAssociatedInfosAsync(ClearAssociatedInfosDBRequest request)
        {
            if (string.IsNullOrWhiteSpace(request.CommonPlatformUserId))
            {
                ThrowCustomerException(CustomerRpcCode.UserCodeEmptyError, "UserCodeEmptyError");
            }
            var commonUserFindFilter = new MongoCustomerFilter<MongoTable>();
            commonUserFindFilter.AppendEq(("BaseData.CommonPlatformUserId", request.CommonPlatformUserId));
            var userInfo = await _userDBRepository.FindOneAsync(commonUserFindFilter.AndCombine());
            if (userInfo != null)
            {
                var commonUserUpdateFilter = new MongoCustomerFilter<MongoTable>();
                commonUserUpdateFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("Code", userInfo.Code));
                MongoCustomerUpdate<MongoTable> commonUserUpdate = new MongoCustomerUpdate<MongoTable>();
                commonUserUpdate.SetField<List<AssociatedInfo>>("BaseData.AssociatedInfos", null);
                commonUserUpdate.SetField<List<AssociatedInfo>>("BaseData.CommonPlatformUserId", null);
                commonUserUpdate.SetField("BaseData.IsOldAgent", false);
                var result = await _userDBRepository.UpdateOneAsync(commonUserUpdateFilter.AndCombine(), commonUserUpdate.AllUpdateDefinition);
                return result > 0;
            }
            return true;
        }

        /// <summary>
        /// 更新第三方关联信息
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> UpdateAssociatedInfosAsync(UpdateAssociatedInfosDBRequest request)
        {
            if (string.IsNullOrWhiteSpace(request.Code))
            {
                ThrowCustomerException(CustomerRpcCode.UserCodeEmptyError, "User code empty error");
            }

            MongoCustomerFilter<MongoTable> filter = new MongoCustomerFilter<MongoTable>();
            filter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("Code", request.Code));
            var filterUser = filter.AndCombine();
            var userInfo = await _userDBRepository.FindOneAsync(filterUser);
            if (userInfo == null)
            {
                ThrowCustomerException(CustomerRpcCode.UserNotFind, "User not find");
            }
            List<AssociatedInfo> associatedInfos = new();
            bool isExistAgentAssessment = false;
            foreach (var associatedInfo in request.AssociatedInfos)
            {
                var info = new AssociatedInfo();
                info.Id = associatedInfo.Id;
                info.RelationName = associatedInfo.RelationName;
                info.Title = associatedInfo.Title;
                info.CTitle = associatedInfo.CTitle;
                info.ETitle = associatedInfo.ETitle;
                info.Icon = associatedInfo.Icon;
                info.Description = associatedInfo.Description;
                info.Url = associatedInfo.Url;
                info.Index = associatedInfo.Index;
                if (associatedInfo.ETitle.Contains("AgentAssessment"))
                {
                    isExistAgentAssessment = true;
                }
                associatedInfos.Add(info);
            }
            if (!string.IsNullOrEmpty(request.CommonPlatformUserId))
            {
                MongoCustomerFilter<MongoTable> filterCommonUser = new MongoCustomerFilter<MongoTable>();
                filterCommonUser.AppendEq(("BaseData.CommonPlatformUserId", request.CommonPlatformUserId));
                var commonFilter = filterCommonUser.AndCombine();
                var commonUsers = await _userDBRepository.FindAllAsync(commonFilter);
                if (commonUsers != null)
                {
                    foreach (var commonUser in commonUsers)
                    {
                        var commonUserUpdateFilter = new MongoCustomerFilter<MongoTable>();
                        commonUserUpdateFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("Code", commonUser.Code));
                        MongoCustomerUpdate<MongoTable> commonUserUpdate = new MongoCustomerUpdate<MongoTable>();
                        commonUserUpdate.SetField<List<AssociatedInfo>>("BaseData.AssociatedInfos", null);
                        commonUserUpdate.SetField<List<AssociatedInfo>>("BaseData.CommonPlatformUserId", null);
                        await _userDBRepository.UpdateOneAsync(commonUserUpdateFilter.AndCombine(), commonUserUpdate.AllUpdateDefinition);
                    }
                }
            }
            MongoCustomerUpdate<MongoTable> update = new MongoCustomerUpdate<MongoTable>();
            update.SetField("BaseData.AssociatedInfos", associatedInfos);
            if (!string.IsNullOrEmpty(request.CommonPlatformUserId))
            {
                update.SetField("BaseData.CommonPlatformUserId", request.CommonPlatformUserId);
            }
            update.SetField("BaseData.IsOldAgent", isExistAgentAssessment);
            if (isExistAgentAssessment)
            {
                //新增权限
                if (userInfo.UserFeatureCodes?.Count > 0)
                {
                    userInfo.UserFeatureCodes.Add("AgentAuthority");
                    userInfo.UserFeatureCodes = userInfo.UserFeatureCodes.Distinct().ToList();
                }
                else
                {
                    userInfo.UserFeatureCodes = new List<string>() { "AgentAuthority" };
                }
            }
            else
            {
                //移除权限
                if (userInfo.UserFeatureCodes?.Count > 0 && userInfo.UserFeatureCodes.Contains("AgentAuthority"))
                {
                    userInfo.UserFeatureCodes.Remove("AgentAuthority");
                }
            }
            update.SetField("BaseData.UserFeatureCodes", userInfo.UserFeatureCodes);
            // item.UserFeatureCodes.Add("AgentAuthority");
            var userUpdate = update.AllUpdateDefinition;
            var result = await _userDBRepository.UpdateOneAsync(filterUser, userUpdate);
            return true;
        }

        /// <summary>
        /// 修改用户急诊设备
        /// </summary>
        /// <returns></returns>
        public async Task<bool> UpdateEmergencyDeviceCodeDBAsync(UpdateEmergencyDeviceCodeDBRequest request)
        {
            var userCode = request.UserCode;
            var emergencyDeviceCode = request.EmergencyDeviceCode;
            using (new Performance(_performanceThreshold, "{0} UpdateEmergencyDeviceCodeDBAsync execute", $"{this.GetType().Name}"))
            {
                var entity = await _userDBRepository.FindOneAsync("Code", userCode);
                if (entity != null && !string.IsNullOrWhiteSpace(entity.Code))
                {
                    entity.BindEmergencyDeviceCode = emergencyDeviceCode;
                    return await _userDBRepository.UpdateOneAsync("Code", entity.Code, entity, "", true);
                }
                return false;
            }
        }

        /// <summary>
        /// 修改用户急诊配置
        /// </summary>
        /// <returns></returns>
        public async Task<bool> UpdateEmergencyInfoDBAsync(UpdateEmergencyInfoDBRequest request)
        {
            var userCode = request.UserCode;
            var emergencyDeviceCode = request.EmergencyDeviceCode;
            var emergencyExpertCode = request.EmergencyExpertCode;
            using (new Performance(_performanceThreshold, "{0} UpdateEmergencyInfoDBAsync execute", $"{this.GetType().Name}"))
            {
                var entity = await _userDBRepository.FindOneAsync("Code", userCode);
                if (entity != null && !string.IsNullOrWhiteSpace(entity.Code))
                {
                    if (entity.BindEmergencyDeviceCode != emergencyDeviceCode || entity.BindEmergencyExpertCode != emergencyExpertCode)
                    {
                        entity.BindEmergencyDeviceCode = emergencyDeviceCode;
                        entity.BindEmergencyExpertCode = emergencyExpertCode;
                        return await _userDBRepository.UpdateOneAsync("Code", entity.Code, entity, "", true);
                    }
                    return true;
                }
                return false;
            }
        }

        /// <summary>
        /// 根据机构codes和角色获取用户列表
        /// </summary>
        /// <param name="OrganizationCodes"></param>
        /// <returns></returns>
        public async Task<List<UserDTO>> FindUserListByOrganizationCodesAndRolesAsync(FindUserListByOrganizationCodesAndRolesRequest request)
        {
            List<string> organizationCodes = request.OrganizationCodes;
            List<string> roleCodes = request.RoleCodes;
            MongoCustomerFilter<MongoTable> customerFilter = new();
            MongoCustomerFilter<MongoDataItem> queryItemsFilter = new();
            if (organizationCodes != null && organizationCodes.Count > 0)
            {
                queryItemsFilter.AppendIn(item => item.Value, organizationCodes);
            }
            else
            {
                queryItemsFilter.AppendEq((item => item.Value, "-1"));
            }
            if (queryItemsFilter.HasFilter)
            {
                customerFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
            }
            if (roleCodes != null && roleCodes.Count > 0)
            {
                customerFilter.AppendAnyIn("BaseData.RoleCodes", roleCodes);
            }
            else
            {
                customerFilter.AppendAnyIn("BaseData.RoleCodes", new List<string> { "-1" });
            }
            SortDefinitionBuilder<MongoTable> builderSort = Builders<MongoTable>.Sort;
            SortDefinition<MongoTable> sort = builderSort.Descending("BaseData.CreateTime");
            var userInfoList = await _userDBRepository.FindAllAsync(customerFilter.AndCombine(), new FindOptions<MongoTable> { NoCursorTimeout = true, Sort = sort });
            var userDTOList = userInfoList.MappingTo<List<UserDTO>>();
            return userDTOList;
        }
        /// <summary>
        /// 批量新增用户数据
        /// </summary>
        /// <param name="request">登录token信息</param>
        /// <returns>是否成功</returns>
        public async Task<bool> BatchInsertUserDataDBAsync(BatchInsertUserRequest request)
        {
            bool res = false;
            try
            {
                if (request?.UserMigratorys?.Count > 0)
                {
                    //找到相同的
                    var ids = request.UserMigratorys.Select(c => c.UserCode).Distinct().ToList();
                    var dic = new Dictionary<string, List<string>>();
                    dic.Add("Code", ids);
                    var userList = await _userDBRepository.FindInAsync(dic);
                    var reportTemplateList = new List<ReportTemplateDO>();
                    //求同存异
                    if (userList?.Count > 0)
                    {
                        var existCodes = userList.Select(c => c.Code.Trim()).Distinct().ToList();
                        var updateData = new List<UserMigratoryInfo>();//request.UserMigratorys.Where(c => existCodes.Contains(c.UserCode));
                        var insertData = new List<UserMigratoryInfo>();//request.UserMigratorys.Where(c => !existCodes.Contains(c.UserCode));
                        foreach (var userInfo in request.UserMigratorys)
                        {
                            if (existCodes.Contains(userInfo.UserCode))
                            {
                                updateData.Add(userInfo);
                            }
                            else
                            {
                                insertData.Add(userInfo);
                            }
                        }
                        if (insertData?.Count() > 0)
                        {
                            //新增
                            var names = insertData.Where(c => !string.IsNullOrEmpty(c.UserName)).Select(c => c.UserName).Distinct().ToList();
                            dic = new Dictionary<string, List<string>>();
                            dic.Add("UserName", names);
                            var nameList = _userDBRepository.FindInAsync(dic).Result;
                            if (nameList?.Count > 0)
                            {
                                // var existNames = nameList.Select(c => c.UserName).Distinct().ToList();
                                var updateNameData = new List<UserMigratoryInfo>();//insertData.Where(c => existNames.Contains(c.UserName));
                                var insertNameData = new List<UserMigratoryInfo>();//insertData.Where(c => !existNames.Contains(c.UserName));
                                foreach (var insertInfo in insertData)
                                {
                                    if (nameList.Exists(c => c.UserName.Trim() == insertInfo.UserName.Trim()))
                                    {
                                        updateNameData.Add(insertInfo);
                                    }
                                    else
                                    {
                                        insertNameData.Add(insertInfo);
                                    }
                                }
                                if (insertNameData?.Count() > 0)
                                {
                                    res = BatchAddUserData(insertNameData, reportTemplateList, request.SyncType).Result;
                                }
                                if (updateNameData?.Count() > 0)
                                {
                                    Logger.WriteLineWarn($"UserDBService BatchInsertUserDataDBAsync Repeat Name, insertData: {JsonConvert.SerializeObject(updateNameData)}");
                                }
                            }
                            else
                            {
                                res = BatchAddUserData(insertData, reportTemplateList, request.SyncType).Result;
                            }
                        }
                        if (updateData?.Count() > 0)
                        {
                            var req = updateData.Select(f =>
                            {
                                var findUserInfo = userList.FirstOrDefault(c => c.Code == f.UserCode);
                                if (!string.IsNullOrWhiteSpace(findUserInfo?.Code))
                                {
                                    findUserInfo.IsDelete = f.IsDelete;
                                    findUserInfo.CreateTime = f.CreateTime;
                                    findUserInfo.UpdateTime = f.UpdateTime;
                                    findUserInfo.UserName = f.UserName;
                                    findUserInfo.SecretPassword = f.SecretPassword;
                                    findUserInfo.Phone = f.Phone;
                                    findUserInfo.Email = f.Email;
                                    findUserInfo.NickName = f.NickName;
                                    findUserInfo.FullName = f.FullName;
                                    findUserInfo.OrganizationCode = f.OrganizationCode;
                                    if (findUserInfo.BindDevices?.Count > 0)
                                    {
                                        if (f.BindDevices?.Count > 0)
                                        {
                                            findUserInfo.BindDevices.AddRange(f.BindDevices);
                                            findUserInfo.BindDevices = findUserInfo.BindDevices.Distinct().ToList();
                                        }
                                    }
                                    else
                                    {
                                        findUserInfo.BindDevices = f.BindDevices;
                                    }
                                    findUserInfo.IsOldAgent = f.IsOldAgent;
                                    findUserInfo.UserState = f.UserState;
                                    findUserInfo.PasswordUpdateTime = f.PasswordUpdateTime;
                                    findUserInfo.PasswordRecords = f.PasswordRecords?.ToList() ?? new List<string>();
                                    findUserInfo.Signature = f.Signature;
                                    //findUserInfo.BindDeviceOrganizations = f.BindDeviceOrganizations?.ToList() ?? new List<string>();

                                    if (findUserInfo.BindDeviceOrganizations?.Count > 0)
                                    {
                                        if (f.BindDeviceOrganizations?.Count > 0)
                                        {
                                            findUserInfo.BindDeviceOrganizations.AddRange(f.BindDeviceOrganizations);
                                            findUserInfo.BindDeviceOrganizations = findUserInfo.BindDeviceOrganizations.Distinct().ToList();
                                        }
                                    }
                                    else
                                    {
                                        findUserInfo.BindDeviceOrganizations = f.BindDeviceOrganizations?.ToList() ?? new List<string>();
                                    }
                                    var oriRoleCode = findUserInfo.RoleCodes[0];
                                    if (f.UserRole != UserMigratoryRoleEnum.Role_UnSet)
                                    {
                                        findUserInfo.RoleCodes = new List<string>() {
                                            f.UserRole.ToString()
                                        };
                                    }
                                    else
                                    {
                                        findUserInfo.RoleCodes = new List<string>() { "Role_GeneralUser" };
                                    }
                                    findUserInfo.UserFeatureCodes = findUserInfo.UserFeatureCodes ?? new List<string>();
                                    if (findUserInfo.UserFeatureCodes?.Count > 0 && oriRoleCode != findUserInfo.RoleCodes[0])
                                    {
                                        //找到旧的，移除
                                        var oldUserFeatureCodes = new List<string>();
                                        var oldUserRoleInfo = FindRoleByCodeAsync(oriRoleCode).Result;
                                        if (!string.IsNullOrEmpty(oldUserRoleInfo?.UserGroupCode))
                                        {
                                            var userAuthInfo = FindUserAuthorityGroupByCodeAsync(oldUserRoleInfo.UserGroupCode).Result;
                                            if (userAuthInfo?.Features?.Count > 0)
                                            {
                                                oldUserFeatureCodes = userAuthInfo.Features;
                                            }
                                        }
                                        //由于二者选其一，只能先全部排除，
                                        if (f.RemoteMainFeature == "SalesAgent" || f.RemoteMainFeature == "Service")
                                        {
                                            oldUserFeatureCodes.Add("RemoteMaintain");
                                            oldUserFeatureCodes.Add("RemoteManager");
                                        }
                                        //先移除老的权限，然后在绑定新的权限
                                        if (oldUserFeatureCodes?.Count > 0)
                                        {
                                            foreach (var item in oldUserFeatureCodes)
                                            {
                                                if (findUserInfo.UserFeatureCodes.Contains(item))
                                                {
                                                    findUserInfo.UserFeatureCodes.Remove(item);
                                                }
                                            }
                                        }
                                    }
                                    //添加新角色数据
                                    var newUserRoleInfo = FindRoleByCodeAsync(findUserInfo.RoleCodes[0]).Result;
                                    if (!string.IsNullOrEmpty(newUserRoleInfo?.UserGroupCode))
                                    {
                                        var userAuthInfo = FindUserAuthorityGroupByCodeAsync(newUserRoleInfo.UserGroupCode).Result;
                                        if (userAuthInfo?.Features?.Count > 0)
                                        {
                                            findUserInfo.UserFeatureCodes.AddRange(userAuthInfo.Features);
                                        }
                                    }
                                    //添加额外远程管理权限
                                    if (f.RemoteMainFeature == "SalesAgent")
                                    {
                                        findUserInfo.UserFeatureCodes.Add("RemoteMaintain");
                                        if (findUserInfo.UserFeatureCodes.Contains("RemoteManager"))
                                        {
                                            findUserInfo.UserFeatureCodes.Remove("RemoteManager");
                                        }
                                    }
                                    else if (f.RemoteMainFeature == "Service")
                                    {
                                        findUserInfo.UserFeatureCodes.Add("RemoteManager");
                                        if (findUserInfo.UserFeatureCodes.Contains("RemoteMaintain"))
                                        {
                                            findUserInfo.UserFeatureCodes.Remove("RemoteMaintain");
                                        }
                                    }
                                    if (f.IsCertifiedInstructor)
                                    {
                                        findUserInfo.UserFeatureCodes.Add("CertifiedInstructor");
                                    }
                                    findUserInfo.UserFeatureCodes = findUserInfo.UserFeatureCodes.Distinct().ToList();
                                    if (f.IsDirector)
                                    {
                                        var updateResult = UpdateOrganizationDirectData(f.OrganizationCode, f.UserCode).Result;
                                    }
                                    if (reportTemplateList?.Count > 0 && !string.IsNullOrEmpty(f.OrganizationCode))
                                    {
                                        var reportTemplateInfo = reportTemplateList.FirstOrDefault(c => c.OrganizationCode == f.OrganizationCode) ?? new ReportTemplateDO();
                                        if (!string.IsNullOrEmpty(reportTemplateInfo.Code))
                                        {
                                            var setResult = SetUserDefaultReportTemplateCodeAsync(findUserInfo.UserCode, reportTemplateInfo.Code).Result;
                                        }
                                    }
                                    findUserInfo.HeadImageToken = f.HeadImageUrl;
                                    if (request.SyncType == SyncDBEnum.Synchronize)
                                    {
                                        var r = _userDBRepository.UpdateOneForSyncDBAsync("Code", findUserInfo.Code, findUserInfo, null).Result;
                                    }
                                    return findUserInfo;
                                }
                                else
                                {
                                    var updateEntity = f.MappingTo<UserInfoDO>();
                                    updateEntity.Code = updateEntity.UserCode.Trim();
                                    updateEntity.IsOldAgent = f.IsOldAgent;
                                    updateEntity.IsDelete = f.IsDelete;
                                    if (f.UserRole != UserMigratoryRoleEnum.Role_UnSet)
                                    {
                                        updateEntity.RoleCodes = new List<string>() {
                                        f.UserRole.ToString()
                                        };
                                    }
                                    else
                                    {
                                        updateEntity.RoleCodes = new List<string>() { "Role_GeneralUser" };
                                    }
                                    updateEntity.UserFeatureCodes = updateEntity.UserFeatureCodes ?? new List<string>();
                                    var userRoleInfo = FindRoleByCodeAsync(updateEntity.RoleCodes[0]).Result;
                                    if (!string.IsNullOrEmpty(userRoleInfo?.UserGroupCode))
                                    {
                                        var userAuthInfo = FindUserAuthorityGroupByCodeAsync(userRoleInfo.UserGroupCode).Result;
                                        if (userAuthInfo?.Features?.Count > 0)
                                        {
                                            updateEntity.UserFeatureCodes = userAuthInfo.Features;
                                        }
                                    }
                                    if (f.RemoteMainFeature == "SalesAgent")
                                    {
                                        updateEntity.UserFeatureCodes.Add("RemoteMaintain");
                                        if (findUserInfo.UserFeatureCodes.Contains("RemoteManager"))
                                        {
                                            findUserInfo.UserFeatureCodes.Remove("RemoteManager");
                                        }
                                    }
                                    else if (f.RemoteMainFeature == "Service")
                                    {
                                        updateEntity.UserFeatureCodes.Add("RemoteManager");
                                        if (findUserInfo.UserFeatureCodes.Contains("RemoteMaintain"))
                                        {
                                            findUserInfo.UserFeatureCodes.Remove("RemoteMaintain");
                                        }
                                    }
                                    if (f.IsCertifiedInstructor)
                                    {
                                        updateEntity.UserFeatureCodes.Add("CertifiedInstructor");
                                    }
                                    updateEntity.UserFeatureCodes = updateEntity.UserFeatureCodes.Distinct().ToList();
                                    if (!string.IsNullOrWhiteSpace(findUserInfo?.Code) && findUserInfo?.AssociatedInfos?.Count > 0)
                                    {
                                        updateEntity.AssociatedInfos = findUserInfo.AssociatedInfos;
                                    }
                                    if (f.IsDirector)
                                    {
                                        var updateResult = UpdateOrganizationDirectData(f.OrganizationCode, f.UserCode).Result;
                                    }
                                    if (reportTemplateList?.Count > 0 && !string.IsNullOrEmpty(f.OrganizationCode))
                                    {
                                        var reportTemplateInfo = reportTemplateList.FirstOrDefault(c => c.OrganizationCode == f.OrganizationCode) ?? new ReportTemplateDO();
                                        if (!string.IsNullOrEmpty(reportTemplateInfo.Code))
                                        {
                                            var setResult = SetUserDefaultReportTemplateCodeAsync(updateEntity.Code, reportTemplateInfo.Code).Result;
                                        }
                                    }
                                    updateEntity.HeadImageToken = f.HeadImageUrl;
                                    if (request.SyncType == SyncDBEnum.Synchronize)
                                    {
                                        var r = _userDBRepository.UpdateOneForSyncDBAsync("Code", updateEntity.Code, updateEntity, null).Result;
                                    }
                                    return updateEntity;
                                }
                            }).ToList();
                            if (request.SyncType == SyncDBEnum.Migrate)
                            {
                                res = _userDBRepository.UpdateAllWithoutOplogLogAsync(req).Result;
                            }
                        }
                    }
                    else
                    {
                        var names = request.UserMigratorys.Where(c => !string.IsNullOrEmpty(c.UserName)).Select(c => c.UserName).Distinct().ToList();
                        dic = new Dictionary<string, List<string>>();
                        dic.Add("UserName", names);
                        var nameList = _userDBRepository.FindInAsync(dic).Result;
                        if (nameList?.Count > 0)
                        {
                            var updateNameData = new List<UserMigratoryInfo>();//insertData.Where(c => existNames.Contains(c.UserName));
                            var insertNameData = new List<UserMigratoryInfo>();//insertData.Where(c => !existNames.Contains(c.UserName));
                            foreach (var insertInfo in request.UserMigratorys)
                            {
                                if (nameList.Exists(c => c.UserName.Trim() == insertInfo.UserName.Trim()))
                                {
                                    updateNameData.Add(insertInfo);
                                }
                                else
                                {
                                    insertNameData.Add(insertInfo);
                                }
                            }
                            if (insertNameData?.Count() > 0)
                            {
                                res = BatchAddUserData(insertNameData, reportTemplateList, request.SyncType).Result;
                            }
                            if (updateNameData?.Count() > 0)
                            {
                                Logger.WriteLineWarn($"UserDBService BatchInsertUserDataDBAsync Repeat Name, UserMigratorys: {JsonConvert.SerializeObject(updateNameData)}");
                            }
                        }
                        else
                        {
                            res = BatchAddUserData(request.UserMigratorys, reportTemplateList, request.SyncType).Result;
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"UserDBService BatchInsertUserDataDBAsync Error: {ex}");
            }
            return res;
        }

        /// <summary>
        /// 更新为机构负责人
        /// </summary>
        private async Task<bool> UpdateOrganizationDirectData(string organizationCode, string userCode)
        {
            if (string.IsNullOrWhiteSpace(organizationCode))
            {
                return true;
            }
            var res = await _organizationDBRepository.GetOrganizationByCodeAsync(organizationCode);
            if (res != null && !string.IsNullOrEmpty(res.OrganizationCode))
            {
                res.Directors = new List<string>() { userCode };
                await _organizationDBRepository.UpdateOneAsync("Code", organizationCode, res, "", false);
            }
            return true;
        }

        /// <summary>
        /// 批量新增数据
        /// </summary>
        private async Task<bool> BatchAddUserData(IEnumerable<UserMigratoryInfo> insertData, List<ReportTemplateDO> reportTemplateList, SyncDBEnum syncType)
        {
            //新增
            var req = insertData.Select(
                f =>
                {
                    var data = f.MappingTo<UserInfoDO>();
                    data.Code = data.UserCode.Trim();
                    data.IsDelete = f.IsDelete;
                    if (f.IsDirector)
                    {
                        var updateResult = UpdateOrganizationDirectData(f.OrganizationCode, f.UserCode).Result;
                    }
                    if (reportTemplateList?.Count > 0 && !string.IsNullOrEmpty(f.OrganizationCode))
                    {
                        var reportTemplateInfo = reportTemplateList.FirstOrDefault(c => c.OrganizationCode == f.OrganizationCode) ?? new ReportTemplateDO();
                        if (!string.IsNullOrEmpty(reportTemplateInfo.Code))
                        {
                            var setResult = SetUserDefaultReportTemplateCodeAsync(data.Code, reportTemplateInfo.Code).Result;
                        }
                    }
                    if (f.UserRole != UserMigratoryRoleEnum.Role_UnSet)
                    {
                        data.RoleCodes = new List<string>() {
                                f.UserRole.ToString()
                        };
                    }
                    else
                    {
                        data.RoleCodes = new List<string>() { "Role_GeneralUser" };
                    }
                    //新增直接绑定角色和权限即可
                    data.UserFeatureCodes = data.UserFeatureCodes ?? new List<string>();
                    var userRoleInfo = FindRoleByCodeAsync(data.RoleCodes[0]).Result;
                    if (!string.IsNullOrEmpty(userRoleInfo?.UserGroupCode))
                    {
                        var userAuthInfo = FindUserAuthorityGroupByCodeAsync(userRoleInfo.UserGroupCode).Result;
                        if (userAuthInfo?.Features?.Count > 0)
                        {
                            data.UserFeatureCodes = userAuthInfo.Features;
                        }
                    }
                    if (f.RemoteMainFeature == "SalesAgent")
                    {
                        data.UserFeatureCodes.Add("RemoteMaintain");
                    }
                    else if (f.RemoteMainFeature == "Service")
                    {
                        data.UserFeatureCodes.Add("RemoteManager");
                    }
                    if (f.IsCertifiedInstructor)
                    {
                        data.UserFeatureCodes.Add("CertifiedInstructor");
                    }
                    data.UserFeatureCodes = data.UserFeatureCodes.Distinct().ToList();
                    data.HeadImageToken = f.HeadImageUrl;
                    if (syncType == SyncDBEnum.Synchronize)
                    {
                        var r = _userDBRepository.InsertOneForAsyncDBAsync(data, new Dictionary<string, string>(), "").Result;
                        var createUserResult = AddPushUserAsync(data).Result;
                    }
                    return data;
                }
            ).ToList();
            var res = true;
            if (syncType == SyncDBEnum.Migrate)
            {
                res = await _userDBRepository.InsertAllWithoutOplogLogAsync(req);
            }

            return res;
        }

        /// <summary>
        /// 获取待迁移时间
        /// </summary>
        /// <returns>是否成功</returns>
        public async Task<MigrateRecordDTO> QueryMigrateTimeDBAsync(bool isInitMigreate, MigrateRecordType type = MigrateRecordType.Default)
        {
            using (new Performance(_performanceThreshold, "{0} QueryMigrateTimeAsync execute", "DatabaseService"))
            {
                var result = new MigrateRecordDTO();
                var endTimeStr = EnvironmentConfigManager.GetParammeter<StringParameter>("Database", "InitMigrateEndTime").Value;
                //迁移，则一直走配置
                if (string.IsNullOrWhiteSpace(endTimeStr))
                {
                    endTimeStr = DateTime.UtcNow.ToString("yyyy-MM-dd HH:mm:ss");
                }
                var curTime = DateTime.UtcNow;
                if (!isInitMigreate)
                {
                    var filter = Builders<MigrateRecordEntity>.Filter.Where(c => c.IsDelete == false && c.Type == type);
                    var sort = Builders<MigrateRecordEntity>.Sort.Descending("MigrateTime");
                    var options = new FindOptions<MigrateRecordEntity>();
                    options.Sort = sort;
                    var findOne = await _migrateRecordDBRepository.FindOneAsync(filter, options);
                    if (!string.IsNullOrEmpty(findOne?.Code))
                    {
                        //找到
                        result.StartTime = findOne.MigrateTime;
                    }
                    else
                    {
                        if (type == MigrateRecordType.MigrateDataCenter)
                        {
                            //没找到
                            result.StartTime = DateTime.MinValue;
                        }
                        else
                        {
                            //没找到
                            result.StartTime = Convert.ToDateTime(endTimeStr);
                        }
                    }
                    result.EndTime = curTime;
                    //新增数据
                    var entity = new MigrateRecordEntity()
                    {
                        MigrateTime = curTime,
                        Type = type
                    };
                    await _migrateRecordDBRepository.InsertOneAsync(entity);
                }
                else
                {
                    result.EndTime = Convert.ToDateTime(endTimeStr);
                    var beginTimeStr = EnvironmentConfigManager.GetParammeter<StringParameter>("Database", "InitMigrateBeginTime").Value;
                    //迁移，则一直走配置
                    if (string.IsNullOrWhiteSpace(beginTimeStr))
                    {
                        result.StartTime = DateTime.MinValue;
                    }
                    else
                    {
                        result.StartTime = Convert.ToDateTime(beginTimeStr);
                    }
                }
                return result;
            }
        }

        /// <summary>
        /// 复位上次迁移时间
        /// </summary>
        /// <param name="curTime"></param>
        /// <param name="type"></param>
        /// <returns></returns>
        public async Task<bool> ResetLastMigrateTimeDBAsync(MigrateRecordType type = MigrateRecordType.Default)
        {
            using (new Performance(_performanceThreshold, "{0} ResetLastMigrateTimeDBAsync execute", "DatabaseService"))
            {
                var filter = Builders<MigrateRecordEntity>.Filter.Where(c => c.IsDelete == false && c.Type == type);
                var findList = await _migrateRecordDBRepository.FindAllAsync(filter);
                if (findList?.Count > 0)
                {
                    var first = findList.OrderByDescending(c => c.MigrateTime).ToList().FirstOrDefault();
                    if (first != null)
                    {
                        await _migrateRecordDBRepository.DeleteOneAsync("Code", first.Code);
                    }
                }
            }
            return true;
        }

        /// <summary>
        /// 根据绑定处理和急诊医生查询用户列表信息
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<List<UserDTO>> GetUserListByBindUserCodeAsync(string userCode)
        {
            MongoCustomerFilter<MongoTable> customerFilter = new();
            MongoCustomerFilter<MongoDataItem> queryItemsFilter = new();

            if (!string.IsNullOrWhiteSpace(userCode))
            {
                customerFilter.AppendEq(("BaseData.BindEmergencyExpertCode", userCode));
            }
            if (!string.IsNullOrWhiteSpace(userCode))
            {
                MongoCustomerFilter<MongoTable> childFilter = new MongoCustomerFilter<MongoTable>();
                queryItemsFilter.AppendEq((item => item.Key, "BindAssistantUserCode"));
                queryItemsFilter.AppendEq((item => item.Value, userCode));
                childFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());

                queryItemsFilter.AppendEq((item => item.Key, "BindAssistantDoctorUserCode"));
                queryItemsFilter.AppendEq((item => item.Value, userCode));
                childFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());

                customerFilter.OrCombine(childFilter.OrCombine());
            }
            var userInfoList = await _userDBRepository.FindAllAsync(customerFilter.OrCombine(), new FindOptions<MongoTable> { NoCursorTimeout = true });
            var userDTOList = userInfoList.MappingTo<List<UserDTO>>();
            return userDTOList;
        }

        /// <summary>
        /// 清洗数据-修改用户绑定设备机构，以及绑定机构的机构负责人对应的设备和机构数据；
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns></returns>
        public async Task<long> ModifyUserBindDeviceOrganizationsDBAsync(ModifyUserBindDeviceOrganizationRequest request)
        {
            long res = 0;
            MongoCustomerFilter<MongoTable> customerFilter = new();
            List<FilterDefinition<MongoDataItem>> queryItemsFilterList = new();
            //查询所有设备
            var queryItemsFilter = new MongoCustomerFilter<MongoDataItem>();
            queryItemsFilter.AppendEq((item => item.Key, "IsDelete"));
            queryItemsFilter.AppendEq((item => item.Value, "False"));
            customerFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
            var deviceInfoList = await _deviceInfoDBRepository.FindAllAsync(customerFilter.AndCombine());
            if (deviceInfoList == null || deviceInfoList.Count <= 0)
            {
                return res;
            }
            //先查询用户列表
            customerFilter = new();
            queryItemsFilterList = new();
            queryItemsFilter = new MongoCustomerFilter<MongoDataItem>();
            queryItemsFilter.AppendEq((item => item.Key, "IsDelete"));
            queryItemsFilter.AppendEq((item => item.Value, "False"));
            customerFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
            if (request.UserCodes?.Count > 0)
            {
                queryItemsFilter = new MongoCustomerFilter<MongoDataItem>();
                queryItemsFilter.AppendEq((item => item.Key, "Code"));
                queryItemsFilter.AppendIn(item => item.Value, request.UserCodes.ToList());
                customerFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
            }
            customerFilter.AppendSizeGte("BaseData.BindDevices", 1);
            var userList = await _userDBRepository.FindAllAsync(customerFilter.AndCombine());
            if (userList?.Count > 0)
            {
                userList.ForEach(userItem =>
                {
                    var bindDeviceInfoList = deviceInfoList.FindAll(c => userItem.BindDevices != null && userItem.BindDevices.Contains(c.Code)) ?? new List<DeviceInfoDO>();
                    if (bindDeviceInfoList.Count > 0)
                    {
                        var orgList = bindDeviceInfoList.Where(c => !string.IsNullOrWhiteSpace(c.OrganizationCode))?.Select(c => c.OrganizationCode)?.Distinct()?.ToList();
                        if (orgList?.Count > 0)
                        {
                            userItem.BindDeviceOrganizations = userItem.BindDeviceOrganizations.Concat(orgList).Distinct().ToList();
                            var result = _userDBRepository.UpdateOneForSyncDBAsync("Code", userItem.UserCode, userItem, string.Empty).Result;
                        }
                    }
                });
            }
            //查询机构负责人
            customerFilter = new();
            queryItemsFilterList = new();
            queryItemsFilter = new MongoCustomerFilter<MongoDataItem>();
            queryItemsFilter.AppendEq((item => item.Key, "IsDelete"));
            queryItemsFilter.AppendEq((item => item.Value, "False"));
            customerFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
            if (request.OrganizationCodes?.Count > 0)
            {
                queryItemsFilter = new MongoCustomerFilter<MongoDataItem>();
                queryItemsFilter.AppendEq((item => item.Key, "Code"));
                queryItemsFilter.AppendIn(item => item.Value, request.OrganizationCodes.ToList());
                customerFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
            }
            customerFilter.AppendSizeGte("BaseData.Directors", 1);
            var orgList = await _organizationDBRepository.FindAllAsync(customerFilter.AndCombine());
            if (orgList?.Count > 0)
            {
                var directors = new List<string>();
                orgList.ForEach(c =>
                {
                    if (c.Directors?.Count > 0)
                    {
                        directors.AddRange(c.Directors);
                    }
                });
                directors = directors.Distinct().ToList();
                if (directors?.Count > 0)
                {
                    //先查询用户列表
                    customerFilter = new();
                    queryItemsFilterList = new();
                    queryItemsFilter = new MongoCustomerFilter<MongoDataItem>();
                    queryItemsFilter.AppendEq((item => item.Key, "IsDelete"));
                    queryItemsFilter.AppendEq((item => item.Value, "False"));
                    customerFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
                    queryItemsFilter = new MongoCustomerFilter<MongoDataItem>();
                    queryItemsFilter.AppendEq((item => item.Key, "Code"));
                    queryItemsFilter.AppendIn(item => item.Value, directors);
                    customerFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
                    var directUserList = await _userDBRepository.FindAllAsync(customerFilter.AndCombine());
                    if (directUserList?.Count > 0)
                    {
                        foreach (var orgItem in orgList)
                        {
                            var findUserList = directUserList.FindAll(c => orgItem.Directors != null && orgItem.Directors.Contains(c.UserCode));
                            if (findUserList?.Count > 0)
                            {
                                findUserList.ForEach(userItem =>
                                {
                                    userItem.BindDeviceOrganizations = userItem.BindDeviceOrganizations ?? new List<string>();
                                    userItem.BindDeviceOrganizations.Add(orgItem.OrganizationCode);
                                    userItem.BindDeviceOrganizations = userItem.BindDeviceOrganizations.Distinct().ToList();
                                    var orgDevices = deviceInfoList.FindAll(c => c.OrganizationCode == orgItem.OrganizationCode) ?? new List<DeviceInfoDO>();
                                    if (orgDevices?.Count > 0)
                                    {
                                        var orgDeviceCodes = orgDevices.Select(c => c.Code);
                                        userItem.BindDevices = userItem.BindDevices.Concat(orgDeviceCodes).Distinct().ToList();
                                    }
                                    var result = _userDBRepository.UpdateOneForSyncDBAsync("Code", userItem.UserCode, userItem, string.Empty).Result;
                                });
                            }
                        }
                    }
                }
            }
            Logger.WriteLineInfo($"UserDBService ModifyUserBindDeviceOrganizationsDBAsync Completed, Request Data:{JsonConvert.SerializeObject(request)}");
            return res;
        }


        /// <summary>
        /// 清洗有医院的用户数据
        /// </summary>
        /// <param name="request">登录token信息</param>
        /// <returns>是否成功</returns>
        public async Task<bool> BatchUpdateUserOrganizationDBAsync(BatchInsertUserRequest request)
        {
            try
            {
                if (request.UserMigratorys != null && request.UserMigratorys.Count > 0)
                {
                    var ids = request.UserMigratorys.Select(c => c.UserCode).Distinct().ToList();
                    var dic = new Dictionary<string, List<string>>();
                    dic.Add("Code", ids);
                    var userList = await _userDBRepository.FindInAsync(dic);
                    foreach (var item in userList)
                    {
                        try
                        {
                            var user = request.UserMigratorys.FirstOrDefault(x => x.UserCode == item.UserCode);
                            if (user != null && !string.IsNullOrWhiteSpace(user.OrganizationCode))
                            {
                                if (item.OrganizationCode != user?.OrganizationCode)
                                {
                                    item.OrganizationCode = user?.OrganizationCode;
                                    await _userDBRepository.UpdateOneForSyncDBAsync("Code", item.Code, item, null);
                                }
                            }
                        }
                        catch (Exception ex)
                        {
                            Logger.WriteLineWarn($"BatchUpdateUserOrganizationDBAsync foreach ex:{ex.ToString()}");
                        }
                    }

                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"BatchUpdateUserOrganizationDBAsync ex:{ex.ToString()}");
            }
            return true;
        }

        /// <summary>
        /// 清洗数据-将机构数据为空，且用户角色不为空的用户数据清洗到vinno_fis平台下面
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns></returns>
        public async Task<bool> ModifyUserSpecialOrganizationInfoDBAsycn(ModifyUserSpecialOrganizationRequest request)
        {
            var res = false;
            MongoCustomerFilter<MongoTable> customerFilter = new();
            List<FilterDefinition<MongoDataItem>> queryItemsFilterList = new();
            var queryItemsFilter = new MongoCustomerFilter<MongoDataItem>();
            queryItemsFilter.AppendEq((item => item.Key, "IsDelete"));
            queryItemsFilter.AppendEq((item => item.Value, "False"));
            customerFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
            if (request.UserCodes?.Count > 0)
            {
                queryItemsFilter = new MongoCustomerFilter<MongoDataItem>();
                queryItemsFilter.AppendEq((item => item.Key, "Code"));
                queryItemsFilter.AppendIn(item => item.Value, request.UserCodes.ToList());
                customerFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
            }
            if (string.IsNullOrWhiteSpace(request.OriginOrganizationCode))
            {
                request.OriginOrganizationCode = "B00F159EB49B445C916ACF023F83259B";
            }
            queryItemsFilter = new MongoCustomerFilter<MongoDataItem>();
            queryItemsFilter.AppendEq((item => item.Key, "OrganizationCode"));
            queryItemsFilter.AppendEq((item => item.Value, request.OriginOrganizationCode));
            customerFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
            customerFilter.AppendSizeGte("BaseData.RoleCodes", 1);
            var userList = await _userDBRepository.FindAllAsync(customerFilter.AndCombine());
            if (userList?.Count > 0)
            {
                userList.ForEach(userItem =>
                {
                    if (!string.IsNullOrEmpty(request.TargetOrganizationCode))
                    {
                        userItem.OrganizationCode = request.TargetOrganizationCode;
                    }
                    else
                    {
                        userItem.OrganizationCode = "";
                        userItem.RoleCodes = new List<string>();
                    }
                    res = _userDBRepository.UpdateOneForSyncDBAsync("Code", userItem.UserCode, userItem, string.Empty).Result;
                });
            }
            Logger.WriteLineInfo($"UserDBService ModifyUserSpecialOrganizationInfoDBAsycn Completed, Request Data:{JsonConvert.SerializeObject(request)}");
            return res;
        }

        /// <summary>
        /// 通用用户注销
        /// </summary>
        /// <param name="request">注销请求实体</param>
        /// <returns>是否成功</returns>
        public async Task<bool> CommonUserLogOffDBAsync(UserPasswordDTO request)
        {
            using (new Performance(_performanceThreshold, "{0} CommonUserLogOffDBAsync execute", $"{this.GetType().Name}"))
            {
                //查询绑定绑定助理和绑定医生以及绑定急诊医生 的其他用户
                var res = false;
                MongoCustomerFilter<MongoTable> customerFilter = new();
                List<FilterDefinition<MongoDataItem>> queryItemsFilterList = new();
                var queryItemsFilter = new MongoCustomerFilter<MongoDataItem>();
                queryItemsFilter.AppendEq((item => item.Key, "BindAssistantUserCode"));
                queryItemsFilter.AppendEq((item => item.Value, request.UserCode));
                queryItemsFilterList.Add(queryItemsFilter.AndCombine());

                queryItemsFilter = new MongoCustomerFilter<MongoDataItem>();
                queryItemsFilter.AppendEq((item => item.Key, "BindAssistantDoctorUserCode"));
                queryItemsFilter.AppendEq((item => item.Value, request.UserCode));
                queryItemsFilterList.Add(queryItemsFilter.AndCombine());

                if (queryItemsFilterList.Count > 0)
                {
                    var queryItemAllFilter = queryItemsFilter.OrCombine(queryItemsFilterList.ToArray());
                    customerFilter.SubFilter(item => item.QueryData.QueryItems, queryItemAllFilter);
                }
                customerFilter.AppendEq(("BaseData.BindEmergencyExpertCode", request.UserCode));
                var userList = await _userDBRepository.FindAllAsync(customerFilter.OrCombine());
                if (userList?.Count > 0)
                {
                    userList.ForEach(userItem =>
                    {
                        if (userItem.BindEmergencyExpertCode == request.UserCode)
                        {
                            userItem.BindEmergencyExpertCode = "";
                        }
                        if (userItem.BindAssistantUserCode == request.UserCode)
                        {
                            userItem.BindAssistantUserCode = "";
                        }
                        if (userItem.BindAssistantDoctorUserCode == request.UserCode)
                        {
                            userItem.BindAssistantDoctorUserCode = "";
                        }
                        res = _userDBRepository.UpdateOneAsync("Code", userItem.UserCode, userItem, string.Empty, true).Result;
                    });
                }
                //解绑机构负责人和急诊医生
                customerFilter = new();
                customerFilter.AppendArrayEq("BaseData.Directors", request.UserCode);
                customerFilter.AppendEq(("BaseData.EmergencyDoctorCode", request.UserCode));
                var orgList = await _organizationDBRepository.FindAllAsync(customerFilter.OrCombine());
                if (orgList?.Count > 0)
                {
                    orgList.ForEach(orgItem =>
                    {
                        if (orgItem.Directors != null && orgItem.Directors.Contains(request.UserCode))
                        {
                            orgItem.Directors.Remove(request.UserCode);
                        }
                        if (orgItem.EmergencyDoctorCode == request.UserCode)
                        {
                            orgItem.EmergencyDoctorCode = "";
                        }
                        res = _organizationDBRepository.UpdateOneForSyncDBAsync("Code", orgItem.OrganizationCode, orgItem, string.Empty, true).Result;
                    });
                }
                //会诊和远程诊断保留数据，保证查询不报错就行
                //转诊保留数据，保证查询不报错就行
                //将当前用户置为无效
                var curUserInfo = request.MappingTo<UserInfoDO>();
                curUserInfo.IsDelete = true;
                var result = await _userDBRepository.UpdateOneAsync("Code", curUserInfo.Code, curUserInfo, string.Empty);
                //根据用户code，清理用户管理员
                MongoCustomerFilter<MongoTable> adminFilter = new();
                customerFilter.AppendEq(("BaseData.BindUserCode", request.UserCode));
                var adminList = await _adminDBRepository.FindAllAsync(customerFilter.AndCombine());
                if (adminList?.Count > 0)
                {
                    foreach (var item in adminList)
                    {
                        item.IsDelete = true;
                        res = await _adminDBRepository.UpdateOneAsync("Code", item.Code, item, string.Empty, true);
                    }
                }
                return result;
            }
        }

        /// <summary>
        /// 关键词查询用户信息
        /// </summary>
        /// <returns>是否成功</returns>
        private async Task<List<UserInfoDO>> GetUserDOListByKeyword(string keyword)
        {
            var list = new List<UserInfoDO>();
            if (string.IsNullOrWhiteSpace(keyword))
            {
                return list;
            }
            MongoCustomerFilter<MongoTable> filter = new();
            List<FilterDefinition<MongoDataItem>> queryItemsFilterList = new();
            MongoCustomerFilter<MongoDataItem> queryItemsFilter = new();
            if (!string.IsNullOrWhiteSpace(keyword))
            {
                queryItemsFilter = new();
                queryItemsFilter.AppendEq((item => item.Key, "UserName"));
                queryItemsFilter.AppendRegex(item => item.Value, keyword, true, false);
                queryItemsFilterList.Add(queryItemsFilter.AndCombine());

                queryItemsFilter = new();
                queryItemsFilter.AppendEq((item => item.Key, "FullName"));
                queryItemsFilter.AppendRegex(item => item.Value, keyword, true, false);
                queryItemsFilterList.Add(queryItemsFilter.AndCombine());
            }
            if (queryItemsFilterList.Count > 0)
            {
                var queryItemAllFilter = queryItemsFilter.OrCombine(queryItemsFilterList.ToArray());
                filter.SubFilter(item => item.QueryData.QueryItems, queryItemAllFilter);
            }
            var datas = await _userDBRepository.FindAllAsync(filter.AndCombine());
            return datas;
        }

        /// <summary>
        /// 修改/删除个人机构，清空个人绑定数据
        /// </summary>
        /// <returns>是否成功</returns>
        public async Task<bool> ClearPersonDefaultTemplateDBAsync(string userCode, string language)
        {
            using (new Performance(_performanceThreshold, "{0} CommonUserLogOffDBAsync execute", $"{this.GetType().Name}"))
            {
                var res = true;
                MongoCustomerFilter<MongoTable> mongoCustomerFilter = new();
                mongoCustomerFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("UserCode", userCode));
                var filter = mongoCustomerFilter.AndCombine();
                var findList = await _userExtendsDBRepository.FindAllAsync(filter);
                if (findList?.Count > 0)
                {
                    foreach (var findInfo in findList)
                    {
                        if (!string.IsNullOrEmpty(findInfo?.Code))
                        {
                            findInfo.DefaultReportTemplateCode = "";
                            findInfo.DefaultThesaurusCode = "";
                            res = await _userExtendsDBRepository.UpdateOneAsync("Code", findInfo.Code, findInfo, "", true);
                        }
                    }
                }
                return res;
            }
        }

        #region db version 1.1 upgrade
        /// <summary>
        /// 删除数据导出权限，1.1
        /// </summary>
        /// <returns>是否成功</returns>
        private async Task<bool> UpdateUserFeatureDBAsync()
        {
            using (new Performance(_performanceThreshold, "{0} UpdateUserFeatureDBAsync execute", $"{this.GetType().Name}"))
            {
                return await _userFeatureDBRepository.DeleteOneAsync("Code", "DataExport");
            }
        }

        /// <summary>
        /// 更新认证医师  为 医师
        /// </summary>
        /// <returns>是否成功</returns>
        private async Task<bool> UpdateUserRoleDBAsync()
        {
            using (new Performance(_performanceThreshold, "{0} UpdateUserRoleDBAsync execute", $"{this.GetType().Name}"))
            {
                var res = true;
                var userRoleDO = await _roleDBRepository.FindOneAsync("Code", "Role_CertifiedPhysician");
                userRoleDO.RoleName = "医师";
                userRoleDO.Description = "开展实时会诊\n可在会诊中安排医师辅助出报告\n可由助理辅助分诊、安排会诊日程";
                userRoleDO.LanguageConfigs = new List<UserRoleLanguageConfigDO>
                {
                    new UserRoleLanguageConfigDO { Language = "zh-CN", RoleName = "医师", Description = "开展实时会诊\n可在会诊中安排医师辅助出报告\n可由助理辅助分诊、安排会诊日程" },
                    new UserRoleLanguageConfigDO { Language = "en-US", RoleName = "Physician", Description = "Conduct real-time consultations\nPhysicians can be arranged to assist in producing reports during consultations\nAssistants can assist in triage and schedule consultations" },
                    new UserRoleLanguageConfigDO { Language = "ru-RU", RoleName = "врач", Description = "Проведение консультаций в режиме реального времени\nПомощник может оказывать помощь в распределении и составлении графика\nконсультаций" }
                };
                res = await _roleDBRepository.UpdateOneForSyncDBAsync("Code", userRoleDO.Code, userRoleDO, "", true, false);

                userRoleDO = await _roleDBRepository.FindOneAsync("Code", "Role_CertifiedExpert");
                userRoleDO.RoleName = "专家";
                userRoleDO.Description = "开展实时会诊\n可在会诊中安排医师辅助出报告\n可由助理辅助分诊、安排会诊日程";
                userRoleDO.RoleShowType = RoleShowTypeEnum.NotShow;
                userRoleDO.LanguageConfigs = new List<UserRoleLanguageConfigDO>
                {
                    new UserRoleLanguageConfigDO { Language = "zh-CN", RoleName = "专家", Description = "开展实时会诊\n可在会诊中安排医师辅助出报告\n可由助理辅助分诊、安排会诊日程" },
                    new UserRoleLanguageConfigDO { Language = "en-US", RoleName = "Expert", Description = "Conduct real-time consultations\nPhysicians can be arranged to assist in producing reports during consultations\nAssistants can assist in triage and schedule consultations" },
                    new UserRoleLanguageConfigDO { Language = "ru-RU", RoleName = "cпециалисты", Description = "Проведение консультаций в режиме реального времени\nПомощник может оказывать помощь в распределении и составлении графика\nконсультаций" }
                };
                res = await _roleDBRepository.UpdateOneForSyncDBAsync("Code", userRoleDO.Code, userRoleDO, "", true, false);

                userRoleDO = await _roleDBRepository.FindOneAsync("Code", "Role_ExpertAssistant");
                userRoleDO.RoleName = "专家助理";
                userRoleDO.Description = "辅助专家开展实时会诊日程安排\n辅助专家管理会诊数据";
                userRoleDO.LanguageConfigs = new List<UserRoleLanguageConfigDO>
                {
                    new UserRoleLanguageConfigDO { Language = "zh-CN", RoleName = "专家助理", Description = "辅助专家开展实时会诊日程安排\n辅助专家管理会诊数据" },
                    new UserRoleLanguageConfigDO { Language = "en-US", RoleName = "Expert assistant", Description = "Assist experts in scheduling real-time consultations\nAssist experts in managing consultation data" },
                    new UserRoleLanguageConfigDO { Language = "ru-RU", RoleName = "ассистент", Description = "Планирование врачебных консультаций в режиме реального времени\nВспомогательные специалисты для управления данными консультаций" }
                };
                res = await _roleDBRepository.UpdateOneForSyncDBAsync("Code", userRoleDO.Code, userRoleDO, "", true, false);

                userRoleDO = await _roleDBRepository.FindOneAsync("Code", "Role_InternshipDoctor");
                userRoleDO.RoleShowType = RoleShowTypeEnum.NotShow;
                res = await _roleDBRepository.UpdateOneForSyncDBAsync("Code", userRoleDO.Code, userRoleDO, "", true, false);
                return res;
            }
        }

        /// <summary>
        /// 医师:新增开始会诊
        /// 所有角色:删除数据导出        
        /// 普通用户:添加我的设备、我的词条、我的模板、分时诊断
        /// 普通用户 实习医生 会诊助理 远程维护管理员屏蔽个性化设置
        /// </summary>
        /// <returns>是否成功</returns>
        private async Task<bool> UpdateUserAuthorityGroupsDBAsync()
        {
            using (new Performance(_performanceThreshold, "{0} UpdateUserAuthorityGroupsDBAsync execute", $"{this.GetType().Name}"))
            {
                var res = false;
                //医师，新增开始会诊权限，个性化设置
                var userRoleDO = await _roleDBRepository.FindOneAsync("Code", "Role_CertifiedPhysician");
                if (!string.IsNullOrEmpty(userRoleDO?.UserGroupCode))
                {
                    var userAuthorityGroupDO = await _userAuthorityGroupDBRepository.FindOneAsync("Code", userRoleDO.UserGroupCode);
                    if (userAuthorityGroupDO.Features?.Count > 0)
                    {
                        userAuthorityGroupDO.Features.Add("StartConsultation");
                        userAuthorityGroupDO.Features.Add("Setting");
                        userAuthorityGroupDO.Features.Add("PersonalSetting");
                        userAuthorityGroupDO.Features = userAuthorityGroupDO.Features.Distinct().ToList();
                    }
                    else
                    {
                        userAuthorityGroupDO.Features = new List<string>() { "StartConsultation", "Setting", "PersonalSetting" };
                    }
                    res = await _userAuthorityGroupDBRepository.UpdateOneForSyncDBAsync("Code", userAuthorityGroupDO.Code, userAuthorityGroupDO, "", true, false);
                }

                //专家，个性化设置
                userRoleDO = await _roleDBRepository.FindOneAsync("Code", "Role_CertifiedExpert");
                if (!string.IsNullOrEmpty(userRoleDO?.UserGroupCode))
                {
                    var userAuthorityGroupDO = await _userAuthorityGroupDBRepository.FindOneAsync("Code", userRoleDO.UserGroupCode);
                    if (userAuthorityGroupDO.Features?.Count > 0)
                    {
                        userAuthorityGroupDO.Features.Add("Setting");
                        userAuthorityGroupDO.Features.Add("PersonalSetting");
                        userAuthorityGroupDO.Features = userAuthorityGroupDO.Features.Distinct().ToList();
                    }
                    else
                    {
                        userAuthorityGroupDO.Features = new List<string>() { "Setting", "PersonalSetting" };
                    }
                    res = await _userAuthorityGroupDBRepository.UpdateOneForSyncDBAsync("Code", userAuthorityGroupDO.Code, userAuthorityGroupDO, "", true, false);
                }

                //所有角色:删除数据导出     
                var userAuthorityGroupList = await _userAuthorityGroupDBRepository.FindAllAsync();
                if (userAuthorityGroupList?.Count > 0)
                {
                    foreach (var item in userAuthorityGroupList)
                    {
                        if (item.Features?.Count > 0 && item.Features.Contains("DataExport"))
                        {
                            item.Features.Remove("DataExport");
                            res = await _userAuthorityGroupDBRepository.UpdateOneForSyncDBAsync("Code", item.Code, item, "", true, false);
                        }
                    }
                }

                //普通用户:添加我的设备、我的词条、我的模板、分时诊断
                userRoleDO = await _roleDBRepository.FindOneAsync("Code", "Role_GeneralUser");
                if (!string.IsNullOrEmpty(userRoleDO?.UserGroupCode))
                {
                    var userAuthorityGroupDO = await _userAuthorityGroupDBRepository.FindOneAsync("Code", userRoleDO.UserGroupCode);
                    if (userAuthorityGroupDO.Features?.Count > 0)
                    {
                        userAuthorityGroupDO.Features.Add("UserEntry");
                        userAuthorityGroupDO.Features.Add("UserDevices");
                        userAuthorityGroupDO.Features.Add("UserTemplate");
                        userAuthorityGroupDO.Features.Add("Diagnosis");
                        userAuthorityGroupDO.Features.Add("ImagePool");
                        userAuthorityGroupDO.Features.Add("PatientLists");
                        userAuthorityGroupDO.Features.Add("DiagnosisEditReport");
                        userAuthorityGroupDO.Features.Add("EditReportTag");
                        userAuthorityGroupDO.Features = userAuthorityGroupDO.Features.Distinct().ToList();
                    }
                    else
                    {
                        userAuthorityGroupDO.Features = new List<string>() { "UserEntry", "UserDevices", "UserTemplate", "Diagnosis", "ImagePool", "PatientLists", "DiagnosisEditReport", "EditReportTag" };
                    }
                    res = await _userAuthorityGroupDBRepository.UpdateOneForSyncDBAsync("Code", userAuthorityGroupDO.Code, userAuthorityGroupDO, "", true, false);
                }

                //普通用户 实习医生 会诊助理 远程维护管理员屏蔽个性化设置
                var dic = new Dictionary<string, List<string>>();
                var tempList = new List<string>() {
                    "Role_GeneralUser", "Role_ExpertAssistant", "Role_InternshipDoctor"
                };
                dic.Add("Code", tempList);
                var userRoleList = await _roleDBRepository.FindInAsync(dic);
                if (userRoleList?.Count > 0)
                {
                    var roleGroupCodes = userRoleList.Select(c => c.UserGroupCode).Distinct().ToList();
                    dic = new Dictionary<string, List<string>>();
                    dic.Add("Code", roleGroupCodes);
                    userAuthorityGroupList = await _userAuthorityGroupDBRepository.FindInAsync(dic);
                    if (userAuthorityGroupList?.Count > 0)
                    {
                        foreach (var item in userAuthorityGroupList)
                        {
                            if (item.Features?.Count > 0 && item.Features.Contains("PersonalSetting"))
                            {
                                item.Features.Remove("PersonalSetting");
                                if (item.Features.Contains("Setting"))
                                {
                                    item.Features.Remove("Setting");
                                }
                                res = await _userAuthorityGroupDBRepository.UpdateOneForSyncDBAsync("Code", item.Code, item, "", true, false);
                            }
                            if (item.Features?.Count > 0 && item.Features.Contains("Setting"))
                            {
                                item.Features.Remove("Setting");
                                res = await _userAuthorityGroupDBRepository.UpdateOneForSyncDBAsync("Code", item.Code, item, "", true, false);
                            }
                        }
                    }
                }
                return res;
            }
        }

        /// <summary>
        /// 更新用户角色为空的，调整为默认普通用户角色
        /// </summary>
        /// <returns>是否成功</returns>
        public async Task<bool> UpdateUserBindRoleDBAsync(ModifyUserBindDeviceOrganizationRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateUserBindRoleDBAsync execute", $"{this.GetType().Name}"))
            {
                var res = false;
                if (request.OrganizationCodes?.Count > 0)
                {
                    var dic = new Dictionary<string, List<string>>();
                    dic.Add("Code", request.OrganizationCodes.ToList());
                    var organizationDOList = await _userDBRepository.FindInAsync(dic);
                    foreach (var item in organizationDOList)
                    {
                        if (item.RoleCodes == null || item.RoleCodes.Count <= 0)
                        {
                            item.RoleCodes = new List<string>() { "Role_GeneralUser" };
                            res = await _userDBRepository.UpdateOneAsync("Code", item.Code, item, "", true);
                            CacheMaintenance.Instance.Get<IUserInfoManager>().Remove(item.Code);
                        }
                    }
                }
                return res;
            }
        }

        /// <summary>
        /// 保存用户权限列表codes
        /// </summary>
        /// <returns>数据列表</returns>
        public async Task<bool> SaveUserFeatureCodesDBAsync(SaveUserFeatureInfoRequest request)
        {
            var result = false;
            var userInfo = await _userDBRepository.FindOneAsync("Code", request.UserCode);
            if (!string.IsNullOrEmpty(userInfo?.UserCode))
            {
                userInfo.UserFeatureCodes = request.FeaturesCodeList;
                result = await _userDBRepository.UpdateOneAsync("Code", userInfo.UserCode, userInfo, "");
            }
            return result;
        }

        /// <summary>
        /// 统一反馈系统小图标地址
        /// </summary>
        /// <returns>是否成功</returns>
        private async Task<bool> UpdateUserAssociatedIconDBAsync()
        {
            using (new Performance(_performanceThreshold, "{0} UpdateUserAssociatedIconDBAsync execute", $"{this.GetType().Name}"))
            {
                var res = false;
                MongoCustomerFilter<MongoTable> customerFilter = new();
                customerFilter.AppendSizeGte("BaseData.AssociatedInfos", 1);
                var userList = await _userDBRepository.FindAllAsync(customerFilter.AndCombine());
                var generlIconUrl = EnvironmentConfigManager.GetParammeter<StringParameter>("General", "AssociatedSystemIcon").Value;
                var fileName = "EA34BFF115404DE1BDA789001ECC1249.jpg";
                if (!string.IsNullOrEmpty(generlIconUrl))
                {
                    fileName = Path.GetFileName(generlIconUrl);
                }
                var newUrl = _curCDNHost + "/" + fileName;
                if (userList?.Count > 0)
                {
                    foreach (var item in userList)
                    {
                        bool isNeedUpdate = false;
                        if (item?.AssociatedInfos?.Count > 0)
                        {
                            foreach (var associatedInfoItem in item?.AssociatedInfos)
                            {
                                if (associatedInfoItem.Icon != newUrl && associatedInfoItem.ETitle == "Feedback System")
                                {
                                    associatedInfoItem.Icon = newUrl;
                                    isNeedUpdate = true;
                                }
                            }
                        }
                        if (isNeedUpdate)
                        {
                            res = await _userDBRepository.UpdateOneForSyncDBAsync("Code", item.Code, item, "", true, false);
                        }
                    }
                }
                return res;
            }

        }
        #endregion

        #region db version 1.2 upgrade
        /// <summary>
        /// 修复数据导出权限，1.2
        /// </summary>
        /// <returns>是否成功</returns>
        private async Task<bool> UpdateUserAuthDBAsync()
        {
            using (new Performance(_performanceThreshold, "{0} UpdateUserAuthDBAsync execute", $"{this.GetType().Name}"))
            {
                var res = false;
                var userFeatureList = await _userFeatureDBRepository.FindAllAsync();
                if (userFeatureList?.Count > 0)
                {
                    var languages = new List<string>();
                    DirectoryInfo theFolder = new DirectoryInfo(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "Languge"));
                    DirectoryInfo[] dirInfo = theFolder.GetDirectories();//获取所在目录的文件夹
                    foreach (var item in dirInfo)
                    {
                        languages.Add(item.Name);
                    }
                    var dicLanguage = new Dictionary<string, JObject>();
                    foreach (var langaugeItem in languages)
                    {
                        var jsonContent = WingServerCommon.ResourceManage.ResourceManager.GetSetting(langaugeItem + "authority");
                        if (string.IsNullOrWhiteSpace(jsonContent))
                        {
                            ThrowCustomerException(CustomerRpcCode.ExceptionError, "Not find Authority Config");
                        }
                        JObject json = JObject.Parse(jsonContent);
                        dicLanguage.Add(langaugeItem, json);
                    }
                    var updateDataList = userFeatureList.Select(item =>
                    {
                        if (item != null && !string.IsNullOrWhiteSpace(item.FeatureCode) && (item.LanguageConfigs == null || item.LanguageConfigs.Count <= 0))
                        {
                            var tempList = new List<UserFeatureLanguageConfigDO>();
                            foreach (var langaugeItem in languages)
                            {
                                var jsonData = dicLanguage[langaugeItem];
                                var entity = new UserFeatureLanguageConfigDO()
                                {
                                    Language = langaugeItem,
                                    FeatureName = jsonData["User"][item.UniqueCode].ToString()
                                };
                                tempList.Add(entity);
                            }
                            item.LanguageConfigs = tempList;
                        }
                        return ("Code", item.Code, item, string.Empty);
                    }).ToList();
                    var result = await _userFeatureDBRepository.UpdateAllAsync(updateDataList);
                    if (result > 0)
                    {
                        res = true;
                    }
                }
                return res;
            }
        }

        /// <summary>
        /// 更新语言
        /// </summary>
        /// <returns>是否成功</returns>
        private async Task<bool> UpdateUserFeatureForESDBAsync()
        {
            using (new Performance(_performanceThreshold, "{0} UpdateUserFeatureForESDBAsync execute", $"{this.GetType().Name}"))
            {
                try
                {
                    Logger.WriteLineInfo($"UserDBService UpdateUserFeatureForESDBAsync Running ");

                    var res = true;
                    var userRoleDOList = await _roleDBRepository.FindAllAsync();
                    if (userRoleDOList?.Count > 0)
                    {
                        foreach (var item in userRoleDOList)
                        {
                            if (item.LanguageConfigs?.Count > 0)
                            {
                                if (!item.LanguageConfigs.Exists(c => c.Language == "es-ES"))
                                {
                                    if (item.Code == "Role_GeneralUser")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserRoleLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                RoleName = "usuario habitual",
                                                Description = "Función predeterminada del sistema"
                                            }
                                        );
                                    }
                                    else if (item.Code == "Role_CertifiedPhysician") //医师
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserRoleLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                RoleName = "Médico",
                                                Description = "Realización de consultas en tiempo real\ncon posibilidad de organizar informes asistidos por médicos durante las consultas\ny asistencia en el triaje y la programación de consultas por parte de asistentes."
                                            }
                                        );
                                    }
                                    else if (item.Code == "Role_ExpertAssistant") //专家助理
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserRoleLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                RoleName = "Asistente especializado",
                                                Description = "Ayudar a los especialistas a programar consultas en tiempo real\nayudar a los especialistas a gestionar los datos de las consultas."
                                            }
                                        );
                                    }
                                    else if (item.Code == "Role_InternshipDoctor") //实习医生
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserRoleLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                RoleName = "Médico en prácticas",
                                                Description = "Asistir a clases con el rol de alumno\nparticipar en consultas en tiempo real como alumno\nparticipar en consultas a distancia como alumno"
                                            }
                                        );
                                    }
                                    else if (item.Code == "Role_CertifiedExpert") //专家
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserRoleLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                RoleName = "Especialista",
                                                Description = "Realización de consultas en tiempo real\ncon posibilidad de organizar informes asistidos por médicos durante las consultas\ny asistencia en el triaje y la programación de consultas por parte de asistentes."
                                            }
                                        );
                                    }

                                    var result = await _roleDBRepository.UpdateOneForSyncDBAsync("Code", item.Code, item, string.Empty, true, false);
                                }
                            }
                        }
                    }
                    return res;
                }
                catch (Exception ex)
                {
                    Logger.WriteLineInfo($"UserDBService UpdateUserFeatureForESDBAsync error, ex: {ex}");
                    return false;
                }
            }
        }

        /// <summary>
        /// 更新用户角色西语数据
        /// </summary>
        /// <returns>是否成功</returns>
        private async Task<bool> UpdateUserRoleForESDBAsync()
        {
            using (new Performance(_performanceThreshold, "{0} UpdateUserRoleForESDBAsync execute", $"{this.GetType().Name}"))
            {
                var res = true;
                try
                {
                    var userFeatureInfoList = await _userFeatureDBRepository.FindAllAsync();
                    if (userFeatureInfoList?.Count > 0)
                    {
                        foreach (var item in userFeatureInfoList)
                        {
                            if (item.LanguageConfigs?.Count > 0)
                            {
                                if (!item.LanguageConfigs.Any(c => c.Language == "es-ES"))
                                {
                                    if (item.Code == "RemoteMaintain")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Mantenimiento a distancia",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "RemoteManager")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Administrador de mantenimiento remoto",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "UserCenter")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Centro Personal",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "UserGroups")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Grupos de usuarios",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "OrgPersonnel")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Estructura organizativa",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "UserTemplate")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Plantillas personales",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "Password")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Contraseñas",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "Setting")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Configuración",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "PersonalSetting")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Personalización",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "UserProfile")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Cuentas",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "UserEntry")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Palabras personales",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "UserDevices")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Dispositivos personales",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "LogOut")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Cierre de sesión",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "OrgEntries")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Organizaciones",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "OrgDevices")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Equipos de organización",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "OrgReportTemplate")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Plantillas de organización",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "OrgInfo")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Información sobre la organización",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "Messages")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Noticias",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "Consultation")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Consulta Cita previa",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "EmergencyConsultation")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Consulta de urgencia",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "ApplyConsultationList")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Lista de solicitudes",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "ApplyConsultation")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Crear una solicitud",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "UpdateApplyConsultation")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Modificar o retirar una solicitud",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "FindApplyConsultation")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Ver solicitud",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "ApplyUserField")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Campos del solicitante",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "AuditApplication")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Aceptar o rechazar solicitud",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "AuditInvitation")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Aceptar o rechazar invitación",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "SelectAssistant")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Asistente de configuración",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "StartConsultation")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Iniciar consulta",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "EditReport")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Redactar o editar un informe (hoja de consulta)",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "FindConsultationReport")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Visualizar el informe de consulta o la ficha de opinión",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "CreateFollowups")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Crear una visita de seguimiento",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "Appraise")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Evaluación",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "ConsultationPage")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Página de consulta en tiempo real",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "Line")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Segmentos de línea",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "Pen")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Pincel",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "Clear")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Borrar",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "SaveImage")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Guardar",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "CloseConsultation")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Finalizar sesión",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "StartRecording")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Grabación completa (inicio/pausa)",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "CloseRecording")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Fin de la grabación",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "RemoteParameter")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Ajuste remoto",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "Diagnosis")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Diagnóstico en tiempo compartido",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "DeleteRecord")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Borrar caso",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "ImagePool")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Pool de imágenes",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "PatientLists")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Lista de pacientes",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "DiagnosisEditReport")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Escribir/modificar informe",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "EditReportTag")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Añadir etiquetas",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "FlyinsonoCollege")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "es-ES",
                                                FeatureName = "Apricot Learning Institute",
                                                Description = ""
                                            }
                                        );
                                    }
                                    var result = await _userFeatureDBRepository.UpdateOneForSyncDBAsync("Code", item.Code, item, string.Empty, true, false);
                                }
                            }
                        }
                    }

                    return res;
                }
                catch (Exception ex)
                {
                    WingServerCommon.Log.Logger.WriteLineError($"UserDBService UpdateAdminRoleForESDBAsync error, ex: {ex}");
                    return res;
                }
            }
        }

        /// <summary>
        /// 更新语言
        /// </summary>
        /// <returns>是否成功</returns>
        private async Task<bool> UpdateUserFeatureForRODBAsync()
        {
            using (new Performance(_performanceThreshold, "{0} UpdateUserFeatureForRODBAsync execute", $"{this.GetType().Name}"))
            {
                try
                {
                    Logger.WriteLineInfo($"UserDBService UpdateUserFeatureForRODBAsync Running ");

                    var res = true;
                    var userRoleDOList = await _roleDBRepository.FindAllAsync();
                    if (userRoleDOList?.Count > 0)
                    {
                        foreach (var item in userRoleDOList)
                        {
                            if (item.LanguageConfigs?.Count > 0)
                            {
                                if (!item.LanguageConfigs.Exists(c => c.Language == "ro-RO"))
                                {
                                    if (item.Code == "Role_GeneralUser")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserRoleLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                RoleName = "Utilizatori disponibili",
                                                Description = "Funcții implicite ale sistemului"
                                            }
                                        );
                                    }
                                    else if (item.Code == "Role_CertifiedPhysician") //医师
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserRoleLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                RoleName = "tratament medical",
                                                Description = "Implementarea interogărilor în timp real\nEste posibil să se organizeze rapoarte cu ajutorul medicilor în perioada de consultare\nAsistarea asistenților în clasificarea și organizarea consultărilor."
                                            }
                                        );
                                    }
                                    else if (item.Code == "Role_ExpertAssistant") //专家助理
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserRoleLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                RoleName = "Gestionarea familiei",
                                                Description = "Asistarea experților în aranjarea interogărilor în timp real\nAsitarea Expert Oiler n Aranjaria Interog Rilor n TIMP Real."
                                            }
                                        );
                                    }
                                    else if (item.Code == "Role_InternshipDoctor") //实习医生
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserRoleLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                RoleName = "tratament medical",
                                                Description = "Clasa ca student\nCa student care participă la interogări în timp real\nParticipați la consultații de la distanță ca student"
                                            }
                                        );
                                    }
                                    else if (item.Code == "Role_CertifiedExpert") //专家
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserRoleLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                RoleName = "familie",
                                                Description = "Implementarea interogărilor în timp real\nEste posibil să se organizeze rapoarte cu ajutorul medicilor în perioada de consultare\nAsistarea asistenților în clasificarea și organizarea consultărilor."
                                            }
                                        );
                                    }

                                    var result = await _roleDBRepository.UpdateOneForSyncDBAsync("Code", item.Code, item, string.Empty, true, false);
                                }
                            }
                        }
                    }
                    return res;
                }
                catch (Exception ex)
                {
                    Logger.WriteLineInfo($"UserDBService UpdateUserFeatureForRODBAsync error, ex: {ex}");
                    return false;
                }
            }
        }

        /// <summary>
        /// 更新用户角色西语数据
        /// </summary>
        /// <returns>是否成功</returns>
        private async Task<bool> UpdateUserRoleForRODBAsync()
        {
            using (new Performance(_performanceThreshold, "{0} UpdateUserRoleForRODBAsync execute", $"{this.GetType().Name}"))
            {
                var res = true;
                try
                {
                    var userFeatureInfoList = await _userFeatureDBRepository.FindAllAsync();
                    if (userFeatureInfoList?.Count > 0)
                    {
                        foreach (var item in userFeatureInfoList)
                        {
                            if (item.LanguageConfigs?.Count > 0)
                            {
                                if (!item.LanguageConfigs.Any(c => c.Language == "ro-RO"))
                                {
                                    if (item.Code == "RemoteMaintain")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Întreţinere la distanţă",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "RemoteManager")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Administrator de întreținere la distanță",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "UserCenter")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Centru personal",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "UserGroups")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Grupuri de utilizatori",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "OrgPersonnel")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "structura organizației",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "UserTemplate")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Șablon personal",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "Password")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "parolă",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "Setting")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "alocarea",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "PersonalSetting")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Setări personale",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "UserProfile")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Profilul utilizatorului",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "UserEntry")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Introducerea utilizatorului",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "UserDevices")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Dispozitive utilizator",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "LogOut")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "deconectare",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "OrgEntries")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "organizație",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "OrgDevices")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Organizarea echipei",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "OrgReportTemplate")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Șablon organizațional",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "OrgInfo")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Informații organizaționale",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "Messages")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "informații",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "Consultation")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "servicii de consultanță",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "EmergencyConsultation")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Consultarea de urgență",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "ApplyConsultationList")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Lista de verificare a consultării cererilor",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "ApplyConsultation")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Consultanță privind aplicațiile",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "UpdateApplyConsultation")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Actualizarea consultării cererilor",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "FindApplyConsultation")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Consultarea cererilor",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "ApplyUserField")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Domeniu utilizator aplicație",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "AuditApplication")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Cerere de audit",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "AuditInvitation")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Invitație de revizuire",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "SelectAssistant")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Asistent selectat",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "StartConsultation")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Începe consultanţa",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "EditReport")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Editează raportul",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "FindConsultationReport")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Căutarea rapoartelor de consultare",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "CreateFollowups")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Crearea de vizite ulterioare",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "Appraise")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "evaluează",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "ConsultationPage")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Pagina de consultare",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "Line")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "linie",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "Pen")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "stilou injector (pen)",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "Clear")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "clar",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "SaveImage")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Salvează imaginea",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "CloseConsultation")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Consultare strânsă",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "StartRecording")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "începe înregistrarea",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "CloseRecording")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Opriți înregistrarea",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "RemoteParameter")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Parametri de la distanță",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "Diagnosis")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "diagnostic",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "DeleteRecord")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Șterge înregistrarea",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "ImagePool")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Pool de imagini",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "PatientLists")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Lista pacienţilor",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "DiagnosisEditReport")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Raport de editare a diagnosticului",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "EditReportTag")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Adaugă etichete",
                                                Description = ""
                                            }
                                        );
                                    }
                                    else if (item.Code == "FlyinsonoCollege")
                                    {
                                        //不存在，添加
                                        item.LanguageConfigs.Add(
                                            new UserFeatureLanguageConfigDO()
                                            {
                                                Language = "ro-RO",
                                                FeatureName = "Colegiul Flinson",
                                                Description = ""
                                            }
                                        );
                                    }
                                    var result = await _userFeatureDBRepository.UpdateOneForSyncDBAsync("Code", item.Code, item, string.Empty, true, false);
                                }
                            }
                        }
                    }

                    return res;
                }
                catch (Exception ex)
                {
                    WingServerCommon.Log.Logger.WriteLineError($"UserDBService UpdateUserRoleForRODBAsync error, ex: {ex}");
                    return res;
                }
            }
        }

        /// <summary>
        /// 添加认证讲师权限
        /// </summary>
        /// <returns>是否成功</returns>
        private async Task<bool> UpdateUserFeatureForCertifiedInstructorDBAsync()
        {
            using (new Performance(_performanceThreshold, "{0} UpdateUserFeatureForCertifiedInstructorDBAsync execute", $"{this.GetType().Name}"))
            {
                var res = true;
                try
                {
                    var certifiedInstructorInfo = await _userFeatureDBRepository.FindOneAsync("Code", "CertifiedInstructor");
                    if (string.IsNullOrEmpty(certifiedInstructorInfo?.Code))
                    {
                        //为空，或者不存在，在添加
                        var model = new UserFeatureDO()
                        {
                            Code = "CertifiedInstructor",
                            FeatureName = "",
                            LanguageConfigs = new List<UserFeatureLanguageConfigDO>() {
                                new UserFeatureLanguageConfigDO()
                                {
                                    Language = "es-ES",
                                    FeatureName = "Entrenador",
                                    Description = ""
                                },
                                new UserFeatureLanguageConfigDO()
                                {
                                    Language = "zh-CN",
                                    FeatureName = "认证讲师",
                                    Description = ""
                                },
                                new UserFeatureLanguageConfigDO()
                                {
                                    Language = "en-US",
                                    FeatureName = "Certified Instructor",
                                    Description = ""
                                },
                                new UserFeatureLanguageConfigDO()
                                {
                                    Language = "ru-RU",
                                    FeatureName = "Тренер",
                                    Description = ""
                                }
                            },
                            FatherCode = "",
                            UniqueCode = "CertifiedInstructor"
                        };
                        var result = await _userFeatureDBRepository.InsertOneForAsyncDBAsync(model, new Dictionary<string, string>(), string.Empty, false);
                        if (!string.IsNullOrEmpty(result))
                        {
                            res = true;
                        }
                    }
                }
                catch (Exception ex)
                {
                    Logger.WriteLineInfo($"UserDBService UpdateUserFeatureForCertifiedInstructorDBAsync error, ex: {ex}");
                    return false;
                }
                return res;
            }
        }

        /// <summary>
        /// 添加代理商权限
        /// </summary>
        /// <returns>是否成功</returns>
        private async Task<bool> UpdateUserFeatureForAgentDBAsync()
        {
            using (new Performance(_performanceThreshold, "{0} UpdateUserFeatureForAgentDBAsync execute", $"{this.GetType().Name}"))
            {
                var res = true;
                try
                {
                    var certifiedInstructorInfo = await _userFeatureDBRepository.FindOneAsync("Code", "AgentAuthority");
                    if (string.IsNullOrEmpty(certifiedInstructorInfo?.Code))
                    {
                        //为空，或者不存在，在添加
                        var model = new UserFeatureDO()
                        {
                            Code = "AgentAuthority",
                            FeatureName = "",
                            LanguageConfigs = new List<UserFeatureLanguageConfigDO>() {
                                new UserFeatureLanguageConfigDO()
                                {
                                    Language = "es-ES",
                                    FeatureName = "Autorización del agente",
                                    Description = ""
                                },
                                new UserFeatureLanguageConfigDO()
                                {
                                    Language = "zh-CN",
                                    FeatureName = "代理商权限",
                                    Description = ""
                                },
                                new UserFeatureLanguageConfigDO()
                                {
                                    Language = "en-US",
                                    FeatureName = "Agent Authority",
                                    Description = ""
                                },
                                new UserFeatureLanguageConfigDO()
                                {
                                    Language = "ru-RU",
                                    FeatureName = "Делегирование полномочий агентом",
                                    Description = ""
                                }
                            },
                            FatherCode = "",
                            UniqueCode = "AgentAuthority"
                        };
                        var result = await _userFeatureDBRepository.InsertOneForAsyncDBAsync(model, new Dictionary<string, string>(), string.Empty, false);
                        if (!string.IsNullOrEmpty(result))
                        {
                            res = true;
                            //针对用户表，已绑定的代理商权限，进行迁移；
                            MongoCustomerFilter<MongoTable> filterCommonUser = new MongoCustomerFilter<MongoTable>();
                            filterCommonUser.AppendEq(("BaseData.IsOldAgent", true));
                            var filter = filterCommonUser.AndCombine();
                            var findUsers = await _userDBRepository.FindAllAsync(filter);
                            if (findUsers?.Count > 0)
                            {
                                foreach (var item in findUsers)
                                {
                                    if (item.UserFeatureCodes?.Count > 0)
                                    {
                                        item.UserFeatureCodes.Add("AgentAuthority");
                                        item.UserFeatureCodes = item.UserFeatureCodes.Distinct().ToList();
                                    }
                                    else
                                    {
                                        item.UserFeatureCodes = new List<string>() { "AgentAuthority" };
                                    }
                                    res = await _userDBRepository.UpdateOneAsync("Code", item.Code, item, string.Empty, true, false);
                                }
                            }
                        }
                    }
                    else
                    {
                        //针对用户表，已绑定的代理商权限，进行迁移；
                        MongoCustomerFilter<MongoTable> filterCommonUser = new MongoCustomerFilter<MongoTable>();
                        filterCommonUser.AppendEq(("BaseData.IsOldAgent", true));
                        var filter = filterCommonUser.AndCombine();
                        var findUsers = await _userDBRepository.FindAllAsync(filter);
                        if (findUsers?.Count > 0)
                        {
                            foreach (var item in findUsers)
                            {
                                if (item.UserFeatureCodes?.Count > 0)
                                {
                                    item.UserFeatureCodes.Add("AgentAuthority");
                                    item.UserFeatureCodes = item.UserFeatureCodes.Distinct().ToList();
                                }
                                else
                                {
                                    item.UserFeatureCodes = new List<string>() { "AgentAuthority" };
                                }
                                res = await _userDBRepository.UpdateOneAsync("Code", item.Code, item, string.Empty, true, false);
                            }
                        }
                    }
                }
                catch (Exception ex)
                {
                    Logger.WriteLineInfo($"UserDBService UpdateUserFeatureForAgentDBAsync error, ex: {ex}");
                    return false;
                }
                return res;
            }
        }

        /// <summary>
        /// 更新科研版权限及管理配置
        /// </summary>
        /// <returns>是否成功</returns>
        private async Task<bool> UpdateUserFeatureForResearchProjectDBAsync()
        {
            using (new Performance(_performanceThreshold, "{0} UpdateUserFeatureForResearchProjectDBAsync execute", $"{this.GetType().Name}"))
            {
                var res = true;
                try
                {
                    var certifiedInstructorInfo = await _userFeatureDBRepository.FindOneAsync("Code", "URMAuthority");
                    if (string.IsNullOrEmpty(certifiedInstructorInfo?.Code))
                    {
                        //为空，或者不存在，在添加
                        var model = new UserFeatureDO()
                        {
                            Code = "URMAuthority",
                            FeatureName = "URM",
                            LanguageConfigs = new List<UserFeatureLanguageConfigDO>() {
                                new UserFeatureLanguageConfigDO()
                                {
                                    Language = "es-ES",
                                    FeatureName = "URM",
                                    Description = ""
                                },
                                new UserFeatureLanguageConfigDO()
                                {
                                    Language = "zh-CN",
                                    FeatureName = "URM权限",
                                    Description = ""
                                },
                                new UserFeatureLanguageConfigDO()
                                {
                                    Language = "en-US",
                                    FeatureName = "URM",
                                    Description = ""
                                },
                                new UserFeatureLanguageConfigDO()
                                {
                                    Language = "ru-RU",
                                    FeatureName = "URM",
                                    Description = ""
                                },
                                new UserFeatureLanguageConfigDO()
                                {
                                    Language = "ro-RO",
                                    FeatureName = "URM",
                                    Description = ""
                                }
                            },
                            FatherCode = "",
                            UniqueCode = "URMAuthority",
                            IsResearchProjectUser = true
                        };
                        var result = await _userFeatureDBRepository.InsertOneForAsyncDBAsync(model, new Dictionary<string, string>(), string.Empty, false);

                        model = new UserFeatureDO()
                        {
                            Code = "CreateProjectAuthority",
                            FeatureName = "Create Project",
                            LanguageConfigs = new List<UserFeatureLanguageConfigDO>() {
                                new UserFeatureLanguageConfigDO()
                                {
                                    Language = "es-ES",
                                    FeatureName = "Create Project",
                                    Description = ""
                                },
                                new UserFeatureLanguageConfigDO()
                                {
                                    Language = "zh-CN",
                                    FeatureName = "创建项目权限",
                                    Description = ""
                                },
                                new UserFeatureLanguageConfigDO()
                                {
                                    Language = "en-US",
                                    FeatureName = "Create Project",
                                    Description = ""
                                },
                                new UserFeatureLanguageConfigDO()
                                {
                                    Language = "ru-RU",
                                    FeatureName = "Create Project",
                                    Description = ""
                                },
                                new UserFeatureLanguageConfigDO()
                                {
                                    Language = "ro-RO",
                                    FeatureName = "Create Project",
                                    Description = ""
                                }
                            },
                            FatherCode = "",
                            UniqueCode = "CreateProjectAuthority",
                            IsResearchProjectUser = true
                        };
                        result = await _userFeatureDBRepository.InsertOneForAsyncDBAsync(model, new Dictionary<string, string>(), string.Empty, false);

                        model = new UserFeatureDO()
                        {
                            Code = "AutoMeasureAuthority",
                            FeatureName = "Measure",
                            LanguageConfigs = new List<UserFeatureLanguageConfigDO>() {
                                new UserFeatureLanguageConfigDO()
                                {
                                    Language = "es-ES",
                                    FeatureName = "Measure",
                                    Description = ""
                                },
                                new UserFeatureLanguageConfigDO()
                                {
                                    Language = "zh-CN",
                                    FeatureName = "测量权限",
                                    Description = ""
                                },
                                new UserFeatureLanguageConfigDO()
                                {
                                    Language = "en-US",
                                    FeatureName = "Measure",
                                    Description = ""
                                },
                                new UserFeatureLanguageConfigDO()
                                {
                                    Language = "ru-RU",
                                    FeatureName = "Measure",
                                    Description = ""
                                },
                                new UserFeatureLanguageConfigDO()
                                {
                                    Language = "ro-RO",
                                    FeatureName = "Create Project",
                                    Description = ""
                                }
                            },
                            FatherCode = "",
                            UniqueCode = "AutoMeasureAuthority",
                            IsResearchProjectUser = true
                        };
                        result = await _userFeatureDBRepository.InsertOneForAsyncDBAsync(model, new Dictionary<string, string>(), string.Empty, false);

                        model = new UserFeatureDO()
                        {
                            Code = "SystemManagementAuthority",
                            FeatureName = "System Manage",
                            LanguageConfigs = new List<UserFeatureLanguageConfigDO>() {
                                new UserFeatureLanguageConfigDO()
                                {
                                    Language = "es-ES",
                                    FeatureName = "System Manage",
                                    Description = ""
                                },
                                new UserFeatureLanguageConfigDO()
                                {
                                    Language = "zh-CN",
                                    FeatureName = "系统管理权限",
                                    Description = ""
                                },
                                new UserFeatureLanguageConfigDO()
                                {
                                    Language = "en-US",
                                    FeatureName = "System Manage",
                                    Description = ""
                                },
                                new UserFeatureLanguageConfigDO()
                                {
                                    Language = "ru-RU",
                                    FeatureName = "System Manage",
                                    Description = ""
                                },
                                new UserFeatureLanguageConfigDO()
                                {
                                    Language = "ro-RO",
                                    FeatureName = "System Manage",
                                    Description = ""
                                }
                            },
                            FatherCode = "",
                            UniqueCode = "SystemManagementAuthority",
                            IsResearchProjectUser = true
                        };
                        result = await _userFeatureDBRepository.InsertOneForAsyncDBAsync(model, new Dictionary<string, string>(), string.Empty, false);
                    }

                    certifiedInstructorInfo = await _userFeatureDBRepository.FindOneAsync("Code", "RRTimeAxis");
                    if (string.IsNullOrEmpty(certifiedInstructorInfo?.Code))
                    {
                        //为空，或者不存在，在添加
                        var model = new UserFeatureDO()
                        {
                            Code = "RRTimeAxis",
                            FeatureName = "时间轴",
                            LanguageConfigs = new List<UserFeatureLanguageConfigDO>() {
                                new UserFeatureLanguageConfigDO()
                                {
                                    Language = "es-ES",
                                    FeatureName = "Time Axis",
                                    Description = ""
                                },
                                new UserFeatureLanguageConfigDO()
                                {
                                    Language = "zh-CN",
                                    FeatureName = "时间轴",
                                    Description = ""
                                },
                                new UserFeatureLanguageConfigDO()
                                {
                                    Language = "en-US",
                                    FeatureName = "Time Axis",
                                    Description = ""
                                },
                                new UserFeatureLanguageConfigDO()
                                {
                                    Language = "ru-RU",
                                    FeatureName = "Time Axis",
                                    Description = ""
                                },
                                new UserFeatureLanguageConfigDO()
                                {
                                    Language = "ro-RO",
                                    FeatureName = "Time Axis",
                                    Description = ""
                                }
                            },
                            FatherCode = "",
                            UniqueCode = "RRTimeAxis",
                            IsResearchProjectUser = true
                        };
                        var result = await _userFeatureDBRepository.InsertOneForAsyncDBAsync(model, new Dictionary<string, string>(), string.Empty, false);
                    }

                    var dic = new Dictionary<string, string>();
                    dic.Add("ConfigKey", "CMS");
                    var entities = await _SystemConfigDBRepository.FindListAsync(dic);
                    var systemConfigInfo = entities?.FirstOrDefault();
                    if (!string.IsNullOrEmpty(systemConfigInfo?.ConfigContent))
                    {
                        var config = EnvironmentConfigManager.GetParammeter<StringParameter>("Gateway", "Host").Value;
                        var host = "";
                        if (!string.IsNullOrEmpty(config) && !config.Contains("*") && config.Split(':').Length > 0)
                        {
                            host = config.Split(':')[0] + ":" + config.Split(':')[1];
                        }
                        else
                        {
                            host = "http://127.0.0.1";
                        }
                        var value = host + ":9302/researchProjectUserList?lang={0}&dongleId={1}";
                        var obj = JObject.Parse(systemConfigInfo.ConfigContent);
                        if (obj.ContainsKey("ManageUrl"))
                        {
                            obj["ManageUrl"] = value;
                        }
                        else
                        {
                            obj.Add("ManageUrl", value);
                        }
                        systemConfigInfo.ConfigContent = JsonConvert.SerializeObject(obj);
                        systemConfigInfo.UpdateTime = DateTime.UtcNow;
                        var result1 = await _SystemConfigDBRepository.UpdateOneAsync("Code", systemConfigInfo.Code, systemConfigInfo, true, false);
                    }
                }
                catch (Exception ex)
                {
                    Logger.WriteLineInfo($"UserDBService UpdateUserFeatureForResearchProjectDBAsync error, ex: {ex}");
                    return false;
                }
                return res;
            }
        }
        #endregion 

        #region 计费资源包
        /// <summary>
        /// 分页获取资源计费信息列表
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageResult<ChargeResourcePackageDTO>> GetChargeResourcePackagePagesDBAsync(ChargeResourcePackagePageRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetChargeResourcePackageByCodeDBAsync execute", "DatabaseService"))
            {
                var keyword = request.Keyword;
                var languageCode = "zh-CN";
                if (!string.IsNullOrEmpty(request.LanguageCode))
                {
                    languageCode = request.LanguageCode;
                }
                var tempDataItemList = new List<DataItemDTO>();
                var chargeBusinessJson = File.ReadAllText(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "Languge", languageCode, "chargeBusiness.json"));
                var chargeBusinessObject = JObject.Parse(chargeBusinessJson);
                if (chargeBusinessObject != null)
                {
                    JArray array = (JArray)chargeBusinessObject["BusinessList"];
                    foreach (JToken jObject in array)
                    {
                        //赋值属性
                        var dataItem = new DataItemDTO()
                        {
                            Key = jObject["Code"].ToString(),
                            Value = jObject["Name"].ToString(),
                        };
                        tempDataItemList.Add(dataItem);
                    }
                }

                var builder = Builders<ChargeResourcePackageEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false);
                if (!string.IsNullOrEmpty(keyword))
                {
                    keyword = EscapeExprSpecialWord(keyword);//字符串转义
                    var nameFilter = builder.Regex(c => c.Name, new BsonRegularExpression(keyword, "i"));
                    var businessCodes = tempDataItemList?.Where(c => c.Value.Contains(keyword))?.Select(c => c.Key)?.Distinct()?.ToList() ?? new List<string>();
                    var businessFilter = builder.Where(c => businessCodes.Contains(c.BusinessCode));
                    filter = filter & (nameFilter | businessFilter);
                }

                if (((int)request.ResourcePackageType) > 0)
                {
                    filter = filter & builder.Where(x => x.ResourcePackageType == request.ResourcePackageType);
                }
                var dbPage = new DBPageRequest<ChargeResourcePackageEntity>();
                dbPage.CurrentPage = request.PageIndex;
                dbPage.PageSize = request.PageSize;
                dbPage.Filter = filter;
                var pageResult = await _chargeResourcePackageDBRepository.GetPages(dbPage);
                var dataResult = PageResultConvert<ChargeResourcePackageEntity, ChargeResourcePackageDTO>(pageResult);
                if (dataResult?.PageData?.Count > 0)
                {
                    foreach (var item in dataResult.PageData)
                    {
                        item.BusinessName = tempDataItemList?.FirstOrDefault(c => c.Key == item.BusinessCode)?.Value ?? string.Empty;
                    }
                }
                return dataResult;
            }
        }

        /// <summary>
        /// 根据Code获取资源计费信息数据
        /// </summary>
        /// <param name="code"></param>
        /// <returns></returns>
        public async Task<ChargeResourcePackageDTO> GetChargeResourcePackageByCodeDBAsync(string code)
        {
            using (new Performance(_performanceThreshold, "{0} GetChargeResourcePackageByCodeDBAsync execute", "DatabaseService"))
            {
                var result = new ChargeResourcePackageDTO();
                var builder = Builders<ChargeResourcePackageEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false && c.Code == code);
                var findEntity = await _chargeResourcePackageDBRepository.FindOneAsync(filter);
                if (!string.IsNullOrEmpty(findEntity?.Code))
                {
                    result = findEntity.MappingTo<ChargeResourcePackageDTO>();
                }
                return result;
            }
        }

        /// <summary>
        /// 根据Code获取资源计费信息数据
        /// </summary>
        /// <param name="code"></param>
        /// <returns></returns>
        public async Task<ChargeResourcePackageDTO> GetChargeResourcePackageByConditionDBAsync(ChargeResourcePackageTypeEnum pageType, string businessCode = "VET")
        {
            using (new Performance(_performanceThreshold, "{0} GetChargeResourcePackageByConditionDBAsync execute", "DatabaseService"))
            {
                var result = new ChargeResourcePackageDTO();
                var builder = Builders<ChargeResourcePackageEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false && c.ResourcePackageType == pageType && c.BusinessCode == businessCode);
                var findEntity = await _chargeResourcePackageDBRepository.FindOneAsync(filter);
                if (!string.IsNullOrEmpty(findEntity?.Code))
                {
                    result = findEntity.MappingTo<ChargeResourcePackageDTO>();
                }
                return result;
            }
        }

        /// <summary>
        /// 保存资源计费信息数据
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> SaveChargeResourcePackageDBAsync(SaveChargeResourcePackageRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} SaveChargeResourcePackageDBAsync execute", "DatabaseService"))
            {
                var result = false;
                var entity = request.MappingTo<ChargeResourcePackageEntity>();
                if (string.IsNullOrEmpty(request.Code))
                {
                    var res = await _chargeResourcePackageDBRepository.InsertOneAsync(entity);
                    if (!string.IsNullOrEmpty(res))
                    {
                        result = true;
                    }
                    else
                    {
                        result = false;
                    }
                }
                else
                {
                    var builder = Builders<ChargeResourcePackageEntity>.Filter;
                    var filter = builder.Where(c => c.IsDelete == false && c.Code == request.Code);
                    var findEntity = await _chargeResourcePackageDBRepository.FindOneAsync(filter);
                    if (!string.IsNullOrEmpty(findEntity.Code))
                    {
                        findEntity.BusinessCode = request.BusinessCode;
                        findEntity.Name = request.Name;
                        findEntity.Price = request.Price;
                        findEntity.ResourcePackageType = request.ResourcePackageType;
                        findEntity.UpdateTime = DateTime.UtcNow;
                        result = await _chargeResourcePackageDBRepository.UpdateOneAsync("Code", findEntity.Code, findEntity, true);
                    }
                }
                return result;
            }
        }

        /// <summary>
        /// 删除资源计费信息数据
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> DeleteChargeResourcePackageDBAsync(DeleteChargeResourcePackageRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} DeleteChargeResourcePackageDBAsync execute", "DatabaseService"))
            {
                var result = false;
                var builder = Builders<ChargeResourcePackageEntity>.Filter;
                if (request?.Codes?.Count > 0)
                {
                    var filter = builder.Where(c => c.IsDelete == false && request.Codes.Contains(c.Code));
                    var findList = await _chargeResourcePackageDBRepository.FindAllAsync(filter);
                    if (findList?.Count > 0)
                    {
                        foreach (var item in findList)
                        {
                            result = await _chargeResourcePackageDBRepository.DeleteOneAsync("Code", item.Code);
                        }
                    }
                }
                return result;
            }
        }
        #endregion 

        /// <summary>
        /// 删除科研版权限
        /// </summary>
        /// <returns>是否成功</returns>
        private async Task<bool> DeleteUserFeatureForResearchProjectDBAsync()
        {
            using (new Performance(_performanceThreshold, "{0} DeleteUserFeatureForResearchProjectDBAsync execute", $"{this.GetType().Name}"))
            {
                var res = await _userFeatureDBRepository.DeleteOneAsync("Code", "URMAuthority");
                await _userFeatureDBRepository.DeleteOneAsync("Code", "AutoMeasureAuthority");
                
                //科研版,清理默认角色的所有权限
                if (_isResearchEditionService)
                {
                    var userRoleDO = await _roleDBRepository.FindOneAsync("Code", "Role_GeneralUser");
                    if (!string.IsNullOrEmpty(userRoleDO?.Code) && !string.IsNullOrEmpty(userRoleDO?.UserGroupCode))
                    {
                        var groupInfo = await _userAuthorityGroupDBRepository.FindOneAsync("Code", userRoleDO.UserGroupCode);
                        if (!string.IsNullOrEmpty(groupInfo?.Code))
                        {
                            groupInfo.Features = new List<string>();
                            res = await _userAuthorityGroupDBRepository.UpdateOneAsync("Code", groupInfo.Code, groupInfo, "", true);
                        }
                    }
                }
                return res;
            }
        }

        /// <summary>
        /// 删除科研版权限-时间轴/系统管理权限
        /// </summary>
        /// <returns>是否成功</returns>
        private async Task<bool> DeleteTwoUserFeatureForResearchProjectDBAsync()
        {
            using (new Performance(_performanceThreshold, "{0} DeleteTwoUserFeatureForResearchProjectDBAsync execute", $"{this.GetType().Name}"))
            {
                var res = await _userFeatureDBRepository.DeleteOneAsync("Code", "RRTimeAxis");
                await _userFeatureDBRepository.DeleteOneAsync("Code", "SystemManagementAuthority");
                return res;
            }
        }
    }
}
